var express = require('express');
const compression = require('compression');
var expressJWT = require('express-jwt');
var path = require('path');
var favicon = require('serve-favicon');
var logger = require('morgan');
var cookieParser = require('cookie-parser');
var bodyParser = require('body-parser');
require('dotenv').config({
  path: '/var/local/env/sdp'
});
var cors = require('cors');
var models = require('./models');
var moment = require('moment');
var blacklist = require('express-jwt-blacklist');
const redisClient = require('./config/cache');
const {
  spawn
} = require('child_process');
const util = require('util');
const exec = util.promisify(require('child_process').exec);

const {
  isCurrentRoleAuthorized
} = require('./rbac/roleBasedAccess');
blacklist.configure({
  tokenId: 'jti',
  store: {
    type: 'redis',
    client: redisClient,
    keyPrefix: 'jwtBlackList:'
  }
});

// global.MFA_URL = (process.env.MFA_BASE_URL || 'https://localhost') + ':' + process.env.MFA_PORT || 9091;

var apiFolder = 'api/';
var apiBasePath = '/api';

var models = require('./models');
var assign = require('./routes/' + apiFolder + 'assign');
var roles = require('./routes/' + apiFolder + 'roles');
var securityQuestions = require('./routes/' + apiFolder + 'securityQuestions');
var users = require('./routes/' + apiFolder + 'usersFile');
var ldapconfig = require('./routes/' + apiFolder + 'ldaps');
var gateway = require('./routes/' + apiFolder + 'gateways');
var controller = require('./routes/' + apiFolder + 'controllers');
var client = require('./routes/' + apiFolder + 'clients');
var clientservice = require('./routes/' + apiFolder + 'clientservice');
// var controllersgateway = require('./routes/' +
//   apiFolder +
//   'controllersgateway');
var authentication = require('./routes/' + apiFolder + 'authentication');
var dashboard = require('./routes/' + apiFolder + 'dashboard');
var ldapmanagement = require('./routes/' + apiFolder + 'ldapmanagement');
var update = require('./routes/' + apiFolder + 'updates');
var web = require('./routes/web/index');
var service = require('./routes/' + apiFolder + 'services');
let clientsAccessLog = require('./routes/' + apiFolder + 'clientsAccessLog');
let settings = require('./routes/' + apiFolder + 'settings');
let deviceid = require('./routes/' + apiFolder + 'deviceid');
let onboard = require('./routes/' + apiFolder + 'onboard');
let verify = require('./routes/' + apiFolder + 'verify');
// let checkUpdate = require('./routes/' + apiFolder + 'checkUpdate');
let download = require('./routes/' + apiFolder + 'download');
let license = require('./routes/' + apiFolder + 'license');
var trustedHosts = require('./routes/' + apiFolder + 'trusted-hosts');
var whiteListedHosts = require('./routes/' + apiFolder + 'whiteListedHosts');
var groups = require('./routes/' + apiFolder + 'groups');
var backup = require('./routes/' + apiFolder + 'backup');
var recover = require('./routes/' + apiFolder + 'recover');
var adminAccessLog = require('./routes/' + apiFolder + 'adminAccessLog');
var deviceInventory = require('./routes/' + apiFolder + 'deviceInventory');
var localUsers = require('./routes/' + apiFolder + 'localUsers');
var organizations = require('./routes/' + apiFolder + 'organizations');
var devices = require('./routes/' + apiFolder + 'devices');
var softwares = require('./routes/' + apiFolder + 'softwares');
var osVersions = require('./routes/' + apiFolder + 'osVersions');
var validDomains = require('./routes/' + apiFolder + 'validDomains');
var reports = require('./routes/' + apiFolder + 'reports');
var serviceGroups = require('./routes/' + apiFolder + 'servicegroup');
var gatewayGroups = require('./routes/' + apiFolder + 'gatewaygroup');
var policies = require('./routes/' + apiFolder + 'policies');
var alerts = require('./routes/' + apiFolder + 'alerts');
var idps = require('./routes/' + apiFolder + 'idps');
var workloads = require('./routes/' + apiFolder + 'workloads');
var microsegments = require('./routes/' + apiFolder + 'microsegments');
const serverBridge = require('./routes/' + apiFolder + 'serverbridge');
const publicServices = require('./routes/' + apiFolder + 'publicServices');
const saasBridge = require('./routes/' + apiFolder + 'saasBridge');
const templates = require('./routes/' + apiFolder + 'templates');
const apiUserRoutes = require('./routes/' + apiFolder + 'apiUser');
const swaggerUi = require('swagger-ui-express');
const apiDoc = require('./config/api-doc.json');
const expressSanitizer = require('express-sanitizer');
const fs = require('fs');
const queues = require('./queues');
const {
  createAdmin
} = require('./dao/license');
const subscribeToLicense = require('./routes/common/event-subscriber');

var app = express();
app.use(cors());

app.options('*', cors()); // Allow preflight requests for all routes
app.use(cookieParser());
app.use((req, res, next) => {
  if (req.cookies.token) {
    console.log(req.cookies.token);
    req.headers.authorization = `Bearer ${req.cookies.token}`;
  }

  next();
});
app.use(compression());

app.disable('x-powered-by');

queues();

app.use('/api-docs', swaggerUi.serve, swaggerUi.setup(apiDoc));

var isExemptedFromLicense = function (controller) {
  var allowedControllers = [
    /login/,
    /authentication/,
    /license/,
    /whitelisted-hosts*/,
    /controllers/,
    /settings/,
    /validDomains/,
    /graph/,
    /updates/,
    /templates/,
    /services\/all*/,
    /securityQuestions/,
    /gatewaysGroups\/dropdown*/,
    /users\/email*/,
    /whitelisted-hosts*/,
    /dashboard\/count*/,
    /dashboard\/os*/,
    /clients-access-logs\/20\/1*/
  ];

  for (let i = 0; i < allowedControllers.length; i++) {
    if (allowedControllers[i].test(controller)) {
      return true;
    }
  }

  return false;
};

app.use((req, res, next) => {
  res.setHeader('Cache-Control', 'no-cache, no-store, must-revalidate'); // HTTP 1.1.
  res.setHeader('Pragma', 'no-cache'); // HTTP 1.0.
  next();
});

var validateLicense = async function (req, res, next) {
  var LICENSE_NOT_VALID = 'Invalid License';
  var request_parts = req.originalUrl.split('/');
  var first_controller = request_parts[1];
  var controller = request_parts.slice(2, request_parts.length).join('/');
  var cont = typeof controller;
  if (first_controller === 'api') {
    const controllerCount = await models.Controller.count({
      where: {}
    });
    res.set('X-Controller-Count', controllerCount);
    models.License.findOne({
      order: [
        ['id', 'DESC']
      ],
      limit: 1
    }).then(function (license) {
      if (!license && !isExemptedFromLicense(controller)) {
        const NO_LICENSE = 'No License';
        res.status(403).send(NO_LICENSE);
      } else if (license && license.violated) {
        console.log('violated');
        res.locals.license_days = -1;
        res.setHeader('X-License-Days', 'Invalid');
        res.setHeader('X-License-Violation-Reason', license.violated);
        if (!isExemptedFromLicense(controller)) {
          res.status(403).send(`Violation-Reason::${license.violated}`);
        } else {
          next();
        }
      } else if (
        license &&
        (license.number_of_days || license.number_of_days === 0)
      ) {
        res.locals.isMfaDisabled =
          license.mfa_feature === null ? false : license.mfa_feature;
        let validUntil = moment(license.license_submission_date).add(
          license.number_of_days,
          'days'
        );
        let difference = moment(validUntil).diff(moment(), 'seconds');

        res.setHeader(
          'X-License-Days',
          moment(validUntil).diff(moment(), 'days', true)
        );

        res.setHeader(
          'X-License-Expiring-Today',
          validUntil.isSame(moment(), 'day')
        );

        res.setHeader(
          'X-License-Expiring-Tomorrow',
          validUntil.isSame(
            moment()
            .startOf('day')
            .add(1, 'd'),
            'day'
          )
        );

        if (difference <= 0) {
          if (!isExemptedFromLicense(controller)) {
            res.status(403).send(LICENSE_NOT_VALID);
          } else {
            next();
          }
        } else {
          next();
        }
      } else if (license && !license.number_of_days) {
        res.locals.license_days = -1;
        if (!isExemptedFromLicense(controller)) {
          res.setHeader('X-License-Days', 'ProcessingRestricted');
          res.status(403).send('ProcessingRestricted');
        } else {
          res.setHeader('X-License-Days', 'Processing');
          next();
        }
      } else {
        res.locals.license_days = -1;
        res.setHeader('X-License-Days', 'NoLicense');
        if (!isExemptedFromLicense(controller)) {
          res.status(403).send(LICENSE_NOT_VALID);
        } else {
          next();
        }
      }
    });
  } else {
    next();
  }
};

var validateUser = async function (req, res, next) {
  var OPERATION_NOT_PERMITTED = 'Operation not permitted';
  var user = req.user;
  var controller = req.originalUrl.split('/')[2] ?
    req.originalUrl.split('/')[2].split('?')[0] :
    req.originalUrl.split('/')[2];
  var method = req.originalUrl.split('/')[3];
  var route = req.originalUrl.split('/')[4];
  if (req.originalUrl.split('/').length > 5) {
    // create a route of the API to access
    route = req.originalUrl.split('/');
    route.splice(0, 4);
    route = route.join('/');
  }
  if (route === undefined) {
    route = '';
  } else {
    route = route.split('?')[0]; // discard the parameter from the API route
  }
  var type = req.method;
  if (user) {
    const userExist = await models.User.findOne({
      include: [{
        model: models.Role
      }],
      where: {
        id: req.user.id
      }
    });

    if (!userExist) {
      return res.status(401).json({
        message: 'User does not exist'
      });
    }
    if (!!userExist.is_locked || !!userExist.is_locked_auto) {
      return res.status(401).json({
        message: 'You are not authorized to perform this operation.'
      });
    }
    // Check Either Given user with current role allowed to access the API or not.
    if (
      !isCurrentRoleAuthorized(
        userExist.Roles[0].title,
        controller,
        type,
        route,
        userExist
      )
    ) {
      return res.status(403).send(OPERATION_NOT_PERMITTED);
    }
    next();
  } else {
    next();
  }
};

async function doubleCheckMigrations() {
  try {
    let skippedMigrations = [];
    let allFiles = [];
    let allMigrations = [];
    const query = 'SELECT * from SequelizeMeta;';
    const allMigrationsResult = await models.sequelize.query(query, {
      raw: true
    });

    allMigrationsResult[0].forEach(file => {
      allMigrations.push(file.name);
    });

    const directoryPath = './migrations';
    const promise = new Promise((resolve, reject) => {
      console;
      fs.readdir(directoryPath, (err, files) => {
        if (err) {
          console.error('Error reading directory:', err);
          reject(err);
        }
        resolve(files);
      });
    });
    allFiles = await promise;
    skippedMigrations = allFiles.filter(val => !allMigrations.includes(val));

    if (skippedMigrations.includes('common')) {
      skippedMigrations.splice(skippedMigrations.indexOf('common'), 1);
    }

    const scriptCommand = 'npx sequelize-cli db:migrate --name ';

    const currentDirectory = process.cwd();

    const runScript = async filename => {
      return ({
        stdout,
        stderr
      } = await exec(`${scriptCommand} ${filename}`));
    };

    skippedMigrations.length > 0 ?
      console.log('Re-running skipped migrations') :
      console.log('No migrations were skipped');

    console.log(skippedMigrations);
    for (let i = 0; i < skippedMigrations.length; i++) {
      await runScript(skippedMigrations[i]);
    }
  } catch (error) {
    console.log(error.message);
  }
}

async function rolesAndPermissionsMigrateExistingUsers() {
  try {
    const license = await models.License.findAll();

    if (license && license.length > 0) {
      let query1 = `
        SELECT u.id, u.firstName, u.lastName, ur.roleId
        FROM Users u
        LEFT JOIN UserTemplates ut ON ut.user_id = u.id
        INNER JOIN UserRoles ur ON ur.userId = u.id
        INNER JOIN Roles r ON r.id = ur.roleId
        WHERE ut.user_id IS NULL
        ORDER BY u.id
      `;

      let users = (await models.sequelize.query(query1))[0];

      let query2 = `
        SELECT id, name
        FROM Templates
        WHERE id BETWEEN 2 AND 6
        ORDER BY id
      `;

      let templates = (await models.sequelize.query(query2))[0];

      // Normal flow:
      // Assign templates to existing users (which aren't present in user templates, i.e. don't have any permissions assigned)
      if (users && users.length > 0) {
        console.log('NON MIGRATED USERS: ', users.length);
        for (let i = 0; i < users.length; i++) {
          if (templates.length !== 0) {
            // roleId : temp id [1:3, 2:2, 3:4, 4:5, 5:6]
            let tempId;

            if (users[i].roleId === 1) {
              tempId = templates[1].id;
            }

            if (users[i].roleId === 2) {
              tempId = templates[0].id;
            }

            if (users[i].roleId === 3) {
              tempId = templates[2].id;
            }

            if (users[i].roleId === 4) {
              tempId = templates[3].id;
            }

            if (users[i].roleId === 5) {
              tempId = templates[4].id;
            }

            if (tempId === 2) {
              // creating link between all templates and user (except infra template) so they can be assigned to new admin users
              for (let k = 0; k < templates.length; k++) {
                await models.UserTemplatePivot.create({
                  temp_id: templates[k].id,
                  user_id: users[i].id
                });
              }
            } else {
              // creating link of template with respect to current user role (for non admin and non infra users)
              await models.UserTemplatePivot.create({
                temp_id: tempId,
                user_id: users[i].id
              });
            }

            // assign admin template to user
            let query3 = `
              SELECT DISTINCT t.id as temp_id, t.name, t.description, tp.id as temp_permission_id
              FROM Templates t
              INNER JOIN TemplatePermissions tp ON tp.temp_id = t.id
              INNER JOIN Permissions p ON p.id = tp.permission_id
              INNER JOIN Privilege pv ON pv.id = p.privilege_id
              WHERE t.id = ${tempId}
            `;

            let templatePermissions = (await models.sequelize.query(query3))[0];

            if (templatePermissions.length !== 0) {
              for (let j = 0; j < templatePermissions.length; j++) {
                await models.UserTemplates.create({
                  temp_permission_id: templatePermissions[j].temp_permission_id,
                  user_id: users[i].id
                });
              }
            }
          }
        }
      }

      // Edge case:
      // If only one user present in table (which would be infra admin) and license exists, create another admin and assign admin template to it.
      else {
        const user = await models.User.findAll();

        if (user && user.length === 1) {
          let err = await createAdmin(license[0].licensee_email);

          if (err !== null) {
            console.log('Error while creating admin: ' + err.message);
          }
        }
      }
    }
  } catch (error) {
    console.log(error.message);
  }
}

(async () => {
  await doubleCheckMigrations();
  await rolesAndPermissionsMigrateExistingUsers();
  await subscribeToLicense();
})();

// view engine setup
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'ejs');

// uncomment after placing your favicon in /public
app.use(favicon(path.join(__dirname, 'public', 'favicon.ico')));
app.use(logger('dev'));
app.use(bodyParser.json({
  limit: '400mb'
}));
app.use(bodyParser.urlencoded({
  extended: false
}));
app.use(expressSanitizer());
app.use(express.static(path.join(__dirname, 'public')));

app.use(
  expressJWT({
    secret: process.env.JWT_SECRET_KEY,
    isRevoked: blacklist.isRevoked
  }).unless({
    path: [
      '/api/authentication/login',
      '/api/authentication/createLinToken',
      '/api/authentication/validateAnswers',
      '/api/authentication/validateLinToken',
      '/api/authentication/validateOTPToken',
      '/api/authentication/passRecaptchaStatus',
      '/api/authentication/getRecaptchaSiteKey',
      '/api/authentication/logout',
      '/api/authentication/forgotPassword',
      '/api/localUsers/sendResetLink',
      '/api/authentication/confirmPassword',
      '/api/authentication/confirmToken',
      '/api/apiUser/sendPasswordResetEmail',
      '/api/authentication',
      '/api/alerts/notifications',
      /\/api\/localUsers\/checkValidToken*/,
      /\/api\/services\/public*/,
      /\/api\/localUsers\/token*/,
      /\/api\/serverBridge\/bridge\/download*/,
      /\/api\/saasBridge\/download*/,
      /\/api\/localUsers\/updatePassword*/,
      /\/api\/check-update*/,
      /\/api\/settings\/mfaSettings*/,
      /\/api\/clients\/download\/service*/,
      // /\/api\/controllers\/download*/,
      // /\/api\/gateways\/download*/,
      /\/api\/download*/,
      /\/api\/clients\/download*/,
      /\/api\/clients\/verify\/otp/,
      /\/api\/localUsers\/resetPassword*/,
      /\/api\/deviceid*/,
      /\/api\/onboard*/,
      /\/api\/verify*/,
      /\/web*/,
      /^\/.*socket.io.*$/,
      /\/api\/clients\/riskScore*/,
      '/',
      '/favicon.ico',
      '/686.js'
    ]
  })
);

app.use(validateLicense);
app.use(validateUser);

app.use('/', web);
app.use(apiBasePath + '/users', users);
app.use(apiBasePath + '/ldaps', ldapconfig);
app.use(apiBasePath + '/workloads', workloads);
app.use(apiBasePath + '/microsegments', microsegments);
app.use(apiBasePath + '/services', service);
app.use(apiBasePath + '/assign', assign);
app.use(apiBasePath + '/roles', roles);
app.use(apiBasePath + '/securityQuestions', securityQuestions);
app.use(apiBasePath + '/authentication', authentication);
app.use(apiBasePath + '/dashboard', dashboard);
app.use(apiBasePath + '/ldapmanagement', ldapmanagement);
app.use(apiBasePath + '/gateways', gateway);
app.use(apiBasePath + '/controllers', controller);
app.use(apiBasePath + '/clients', client);
app.use(apiBasePath + '/clientservice', clientservice);
// app.use(apiBasePath + '/controllersgateway', controllersgateway);
app.use(apiBasePath + '/updates', update);
// app.use(apiBasePath + '/deviceid', deviceid);
app.use(apiBasePath + '/settings', settings);
app.use(apiBasePath + '/license', license);
app.use(apiBasePath + '/clients-access-logs', clientsAccessLog);
//app.use(apiBasePath + '/onboard', onboard);
app.use(apiBasePath + '/verify', verify);
// app.use(apiBasePath + '/check-update', checkUpdate);
app.use(apiBasePath + '/download', download);
app.use(apiBasePath + '/trusted-hosts', trustedHosts);
app.use(apiBasePath + '/whitelisted-hosts', whiteListedHosts);
app.use(apiBasePath + '/groups', groups);
app.use(apiBasePath + '/backup', backup);
app.use(apiBasePath + '/restore', recover);
app.use(apiBasePath + '/adminAccessLog', adminAccessLog);
app.use(apiBasePath + '/deviceInventory', deviceInventory);
app.use(apiBasePath + '/idps', idps);
//app.use(apiBasePath + '/enhanced-onboard', enhancedOnboard);
app.use(apiBasePath + '/localUsers', localUsers);
app.use(apiBasePath + '/organizations', organizations);
app.use(apiBasePath + '/devices', devices);
app.use(apiBasePath + '/softwares', softwares);
app.use(apiBasePath + '/osVersions', osVersions);
app.use(apiBasePath + '/validDomains', validDomains);
app.use(apiBasePath + '/reports', reports);
app.use(apiBasePath + '/serviceGroups', serviceGroups);
app.use(apiBasePath + '/gatewaysGroups', gatewayGroups);
app.use(apiBasePath + '/policies', policies);
app.use(apiBasePath + '/alerts', alerts);
app.use(apiBasePath + '/serverBridge', serverBridge);
app.use(apiBasePath + '/saasBridge', saasBridge);
app.use(apiBasePath + '/services/public', publicServices);
app.use(apiBasePath + '/templates', templates);
app.use(apiBasePath + '/apiUser', apiUserRoutes);

app.use('/web*', web);

// catch 404 and forward to error handler
app.use(function (req, res, next) {
  var err = new Error('Not Found');
  err.statusCode = 404;
  next(err);
});

// error handler
app.use(function (err, req, res, next) {
  let code = err.statusCode || 500;
  if (err.name && err.name === 'UnauthorizedError') {
    code = 403;
  }
  if (code === 500) {
    console.error(err.stack);
  }

  res.status(code).send({
    message: err.message
  });
  // set locals, only providing error in development
  // res.locals.message = err.message;
  // res.locals.error = req.app.get('env') === 'development' ? err : {};

  // render the error page
  // res.status(err.status || 500);
  // res.render('error');
});
module.exports = app;