const md5File = require('md5-file');
const cron = require('node-cron');
const models = require('../models');
const { job } = require('../cronjobs/device_validation');
const { dashboardJob } = require('../cronjobs/dashboard');

const tasks = {
  DELETE_OUTPUT_PACKAGES: require('../cronjobs/delete-output-packages'),
  PURGE_ACCESS_LOGS: require('../cronjobs/purge-access-logs'),
  SEND_LICENSE_EXPIRY_REMINDER: require('../cronjobs/send-license-expiry-reminder'),
  LDAP_SYNC: require('../cronjobs/sync-ldap'),
  PASSWORD_RESET: require('../cronjobs/reset-password'),
  COUNT_HISTORY: require('../cronjobs/countHistory'),
  DELETE_DEVICES_GATEWAY_AND_CONTROLLER_LOGS: require('../cronjobs/deleteGatewayAndControllerLogs')
};

const scheduledTasks = {
  // Cron job to delete 30 day old Devices, Gateway and Controller health logs
  // Cronjob is configured to run 1:30 AM

  DELETE_DEVICES_GATEWAY_AND_CONTROLLER_LOGS: cron.schedule(
    '30 1 * * *',
    tasks['DELETE_DEVICES_GATEWAY_AND_CONTROLLER_LOGS'],
    { scheduled: true }
  ),

  DASHBOARD_JOBS: cron.schedule('0 * * * *', dashboardJob, { scheduled: true }),

  DELETE_OUTPUT_PACKAGES: cron.schedule(
    '0 0 */1 * * *',
    tasks['DELETE_OUTPUT_PACKAGES'],
    { scheduled: true }
  ),
  PURGE_ACCESS_LOGS: cron.schedule('0 * * * *', tasks['PURGE_ACCESS_LOGS'], {
    scheduled: true
  }),
  SEND_LICENSE_EXPIRY_REMINDER: cron.schedule(
    '0 0 11 * * *',
    tasks['SEND_LICENSE_EXPIRY_REMINDER'],
    { scheduled: true }
  ),
  COUNT_HISTORY: cron.schedule('0 0 0 * * *', tasks.COUNT_HISTORY)
};

(async () => {
  // running dashboard cron on server restart
  await dashboardJob(false);

  const callFunc = async () => {
    try {
      let manualTrigger = false;
      await job(manualTrigger);
    } catch (error) {
      console.log(error);
    }
  };
  let interval;
  let data;
  let task;
  interval = await models.Settings.findOne({
    limit: 1,
    order: [['id', 'DESC']],
    attributes: ['device_validation_freq'],
    raw: true
  }).then(d => {
    if (process.env.NODE_APP_INSTANCE === '0') {
      clearInterval(task);
      task = setInterval(async () => {
        await callFunc();
      }, d.device_validation_freq * 1000);
    }
    return d.device_validation_freq;
  });
  setInterval(async () => {
    data = await models.Settings.findOne({
      limit: 1,
      order: [['id', 'DESC']],
      attributes: ['device_validation_freq'],
      raw: true
    }).then(d => {
      return d.device_validation_freq;
    });
    if (data != interval) {
      interval = data;
      if (process.env.NODE_APP_INSTANCE === '0') {
        clearInterval(task);
        task = setInterval(async () => {
          await callFunc();
        }, data * 1000);
      }
    }
  }, 1000);
})();

models.Settings.findOne({
  limit: 1,
  order: [['id', 'DESC']],
  attributes: [
    'dirsync_enabled',
    'dirsync_time',
    'dirclean_time',
    'device_validation_freq'
  ]
})
  .then(function({
    dirsync_enabled,
    dirsync_time,
    dirclean_time,
    ddd,
    device_validation_freq
  }) {
    if (dirsync_enabled) {
      return tasks['LDAP_SYNC']().then(() => {
        scheduledTasks['LDAP_SYNC'] = cron.schedule(
          `*/${dirsync_time} * * * *`,
          tasks['LDAP_SYNC'],
          {
            scheduled: true
          }
        );
      });
    }

    return Promise.reject(new Error('Directory Sync Not Enabled'));
  })
  .then(() => console.log('Done'))
  .catch(e => console.log(e && (e.message || e)));

process.on('message', m => {
  console.log('CRON: Received Request', m);
  try {
    switch (m.type) {
      case 'SCHEDULE': {
        const { expression, taskName, options } = m;
        const currentlyScheduledTask = scheduledTasks[taskName];

        if (currentlyScheduledTask) {
          currentlyScheduledTask.destroy();
        }

        scheduledTasks[taskName] = cron.schedule(
          expression,
          tasks[taskName],
          options
        );
        break;
      }
      case 'START': {
        const { taskName } = m;
        const currentlyScheduledTask = scheduledTasks[taskName];

        if (currentlyScheduledTask) {
          currentlyScheduledTask.start();
        }
        break;
      }
      case 'STOP': {
        const { taskName } = m;
        const currentlyScheduledTask = scheduledTasks[taskName];

        if (currentlyScheduledTask) {
          currentlyScheduledTask.stop();
        }
        break;
      }
      case 'DESTROY': {
        const { taskName } = m;
        const currentlyScheduledTask = scheduledTasks[taskName];

        if (currentlyScheduledTask) {
          currentlyScheduledTask.destroy();
        }
        break;
      }
    }
  } catch (e) {
    console.log(e && (e.message || e));
  }
});
