const logsPriority = {
  PRIORITY_HIGH: 'high',
  PRIORITY_MEDIUM: 'medium',
  PRIORITY_LOW: 'low'
};

const policyTypes = {
  ACCESS_POLICY: 'Entitlement',
  DEVICE_POLICY: 'device',
  SOFTWARE_POLICY: 'software policy',
  DATA_VOLUME_POLICY: 'data-volume policy',
  PROCESS_POLICY: 'process policy',
  OS_POLICY: 'OS policy',
  ROUTING_POLICY: 'routing policy',
  SECURITY_POLICY: 'Security policy'
};

const serverTypes = {
  SERVER_BRIDGE: 'Server bridge',
  DEVICE_BRIDGE_x86: 'Device bridge (x86)',
  DEVICE_BRIDGE__ARM: 'Device bridge (ARM)',
  B2B_BRIDGE: 'B2B bridge',
  RBI_BRIDGE: 'RBI bridge',
  SAAS_BRIDGE: 'Saas bridge'
}

module.exports = {
  logsPriority,
  policyTypes,
  serverTypes,

  getUserDeletMessage: adminName => ({
    action: `Deleted admin ${adminName}`,
    priority: logsPriority.PRIORITY_HIGH
  }),
  getUserCreateMessage: adminName => ({
    action: `Created admin ${adminName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),
  getUpdateUserMessage: adminName => ({
    action: `Edited portal admin ${adminName} profile`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getLdapCreateMessage: () => ({
    action: `New ldap created`,
    priority: logsPriority.PRIORITY_LOW
  }),
  getLdapEditMessage: () => ({
    action: `Edited ldap`,
    priority: logsPriority.PRIORITY_LOW
  }),
  getLdapDeleteMessage: () => ({
    action: `Deleted ldap`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getServiceCreateMessage: serviceName => ({
    action: `Created service ${serviceName}`,
    priority: logsPriority.PRIORITY_LOW
  }),
  getServiceEditMessage: serviceName => ({
    action: `Edited service ${serviceName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),
  getServiceDeleteMessage: serviceName => ({
    action: `Deleted service ${serviceName}`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getControllerCreateMessage: controllerName => ({
    action: `Created controller ${controllerName}`,
    priority: logsPriority.PRIORITY_LOW
  }),
  getControllerEditMessage: controllerName => ({
    action: `Edited controller ${controllerName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),
  getControllerDeleteMessage: controllerName => ({
    action: `Deleted controller ${controllerName}`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getGatewayCreateMessage: gatewayName => ({
    action: `Created gateway ${gatewayName}`,
    priority: logsPriority.PRIORITY_LOW
  }),
  getGatewayEditMessage: gatewayName => ({
    action: `Edited gateway ${gatewayName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),
  getGatewayDeleteMessage: gatewayName => ({
    action: `Deleted gateway ${gatewayName}`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getHostCreateMessage: () => ({
    action: `New trusted host created`,
    priority: logsPriority.PRIORITY_LOW
  }),
  getHostEditMessage: () => ({
    action: `Edited trusted host`,
    priority: logsPriority.PRIORITY_LOW
  }),
  getHostDeleteMessage: () => ({
    action: `Deleted trusted host`,
    priority: logsPriority.PRIORITY_LOW
  }),

  getBackupCreateMessage: time => ({
    action: `Database backed up at ${time}`,
    priority: logsPriority.PRIORITY_LOW
  }),
  getBackupDeleteMessage: fileName => ({
    action: `Deleted backup with time ID ${fileName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getGroupCreateMessage: groupName => ({
    action: `Created group ${groupName}`,
    priority: logsPriority.PRIORITY_LOW
  }),
  getGroupEditMessage: groupName => ({
    action: `Edited group ${groupName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),
  getGroupDeleteMessage: groupName => ({
    action: `Deleted group ${groupName}`,
    priority: logsPriority.PRIORITY_HIGH
  }),
  getClientAddToGroupMessage: (clientName, groupName) => ({
    action: `Added client ${clientName} to group ${groupName}`,
    priority: logsPriority.PRIORITY_LOW
  }),
  getClientDeleteFromGroupMessage: (clientName, groupName) => ({
    action: `Deleted client ${clientName} from group ${groupName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getPolicyCreateMessage: (policyName, policyType) => ({
    action: `Created ${policyType} ${policyName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),
  getPolicyEditMessage: (policyName, policyType) => ({
    action: `Updated ${policyType} ${policyName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),
  getPolicyDeleteMessage: (policyName, policyType) => ({
    action: `Deleted ${policyType} ${policyName}`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getLoginMessage: (username, roleName) => ({
    action: `User logged in`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),
  getLogoutMessage: (username, roleName) => ({
    action: `User logged out`,
    priority: logsPriority.PRIORITY_LOW
  }),

  getPolicyEnableMessage: (policyName, policyType) => ({
    action: `Enabled ${policyType} ${policyName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getPolicyDisableMessage: (policyName, policyType) => ({
    action: `Disabled ${policyType} ${policyName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getServiceGroupCreateMessage: serviceGroupName => ({
    action: `Created service group ${serviceGroupName}`,
    priority: logsPriority.PRIORITY_LOW
  }),

  getServiceGroupEditMessage: serviceGroupName => ({
    action: `Edited service group ${serviceGroupName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getServiceGroupDeleteMessage: serviceGroupName => ({
    action: `Deleted service group ${serviceGroupName}`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getControllerEmailMessage: controllerName => ({
    action: `Generated email for controller ${controllerName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getControllerDownloadlMessage: controllerName => ({
    action: `Downloaded controller ${controllerName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getGatewayEmailMessage: gatewaytName => ({
    action: `Generated email for gateway ${gatewaytName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getGatewayDownloadlMessage: gatewaytName => ({
    action: `Downloaded gateway ${gatewaytName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getGatewayEnableMessage: gatewayName => ({
    action: `Enabled Gateway ${gatewayName}`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getGatewayDisableMessage: gatewayName => ({
    action: `Disabled Gateway ${gatewayName}`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getCloneMessage: (gatewayOldName, gatewayNewName) => ({
    action: `Cloned gatway ${gatewayOldName} and created ${gatewayNewName}`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getGatewayGroupCreateMessage: gatewayName => ({
    action: `Created gateway group ${gatewayName}`,
    priority: logsPriority.PRIORITY_LOW
  }),

  getGatewayGroupEditMessage: gatewayName => ({
    action: `Edited gateway group ${gatewayName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getGatewayGroupDeleteMessage: gatewayName => ({
    action: `Deleted gateway group ${gatewayName}`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getServerCreateMessage: (serverName, serverType) => ({
    action: `Created ${serverType} ${serverName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),
  getServerEditMessage: (serverName, serverType) => ({
    action: `Updated ${serverType} ${serverName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),
  getServerDeleteMessage: (serverName, serverType) => ({
    action: `Deleted ${serverType} ${serverName}`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getServerEnableMessage: (serverName, serverType) => ({
    action: `Enabled ${serverType} ${serverName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getServerDisableMessage: (serverName, serverType) => ({
    action: `Disabled ${serverType} ${serverName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getServerDownloadlMessage: (serverName, serverType) => ({
    action: `Downloaded ${serverType} ${serverName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getIdpCreateMessage: idpName => ({
    action: `Created IDP ${idpName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getIdpEditMessage: idpName => ({
    action: `Updated IDP ${idpName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getIdpDeleteMessage: idpName => ({
    action: `Deleted IDP ${idpName}`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getLocalUserCreateMessage: Name => ({
    action: `Created Local user ${Name}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getLocalUserEditMessage: Name => ({
    action: `Updated Local user ${Name}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getLocalUserDeleteMessage: Name => ({
    action: `Deleted Local user ${Name}`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getDevicetoLocalUserAdded:(userName, deviceName) => ({
    action: `Added device ${deviceName} to local user ${userName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getDevicetoLocalUserDeleted:(userName, deviceName) => ({
    action: `Deleted device ${deviceName} from local user ${userName}`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getLocalUserSendOtp: name => ({
    action: `Send OTP QR Code to Local user ${name}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getLocalUserPassChangeMessage: name => ({
    action: `Local user ${name} password changed`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getLocalUserMfaMessage: name => ({
    action: `Local user ${name} updated `,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getDeviceUpdateMessage: name => ({
    action: `Device ${name} Updated`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getDeviceCreateMessage: name => ({
    action: `Device ${name} created`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getDeviceDeleteMessage: name => ({
    action: `Device ${name} deleted`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getDeviceValidationMessage: data => ({
    action: `Requested Validation of ${data}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getWorkloadUpdateMessage: name => ({
    action: `Workload ${name} Updated`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getWorkloadCreateMessage: name => ({
    action: `Workload ${name} created`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getWorkloadDeleteMessage: name => ({
    action: `Workload ${name} deleted`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getPublishMicrosegmentMessage: name => ({
    action: `Workload ${name} published`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getSaveDiscoveryMessage: name => ({
    action: `Discovery for ${name} Started`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getStopDiscoveryMessage: name => ({
    action: `Discovery for ${name} Stopped`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getClientUpdateMessage: name => ({
    action: `Client ${name} Updated`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getClientAllowMessage: name => ({
    action: `Client ${name} enabled`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getClientBlockMessage: name => ({
    action: `Client ${name} disabled`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getClientForceOnBoardMessage: name => ({
    action: `Client ${name} force re onboarded`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getClientEmailMessage: cleintName => ({
    action: `Generated email for Client ${cleintName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getOrganizationUpdateMessage: name => ({
    action: `Organization ${name} Updated`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getOrganizationCreateMessage: name => ({
    action: `Organization ${name} created`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getOrganizationDeleteMessage: name => ({
    action: `Organization ${name} deleted`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getDepartmentAddMessage: (departmentName, organizationName) => ({
    action: `Added department ${departmentName} to organization ${organizationName}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getDepartmentDeleteMessage: (departmentName, organizationName) => ({
    action: `Deleted department ${departmentName} from organization ${organizationName}`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getSoftwareUpdateMessage: name => ({
    action: `Software ${name} Updated in Software Master List`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getSoftwareCreateMessage: name => ({
    action: `Software ${name} created in Software Master List`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getSoftwareDeleteMessage: name => ({
    action: `Software ${name} deleted in Software Master List`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getProcessBindingUpdateMessage: name => ({
    action: `Software process ${name} updated`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getProcessBindingCreateMessage: name => ({
    action: `${name} Software process binded`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getProcessBindingDeleteMessage: name => ({
    action: `Software process ${name} deleted`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getSettingsUpdateMessage: () => ({
    action: `Portal settings updated`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getClientSettingUpdateMessage: () => ({
    action: `Client settings updated`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getDomainCreateMessage: name => ({
    action: `Domain ${name} created`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getDomainUpdateMessage: (previousDomain, newDomain) => ({
    action: `Domain ${previousDomain} updated to ${newDomain}`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getDomainDeleteMessage: name => ({
    action: `Domain ${name} deleted`,
    priority: logsPriority.PRIORITY_HIGH
  }),

  getCoreSettingUpdateMessage: () => ({
    action: `Core settings updated`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getPackageUpdateMessage: (os, component, version) => ({
    action: `Component ${component} with os ${os} and version ${version} updated in software updates`,
    priority: logsPriority.PRIORITY_MEDIUM
  }),

  getPackageCreateMessage: (os, component, version) => ({
    action: `Component ${component} with os ${os} and version ${version} added in software updates`,
    priority: logsPriority.PRIORITY_MEDIUM
  })
};
