const redis = require('redis');
const util = require('util');

const subscriber = redis.createClient({
    host: process.env.REDIS_HOST || '127.0.0.1',
    port: process.env.REDIS_PORT || '6379',
    password: process.env.REDIS_PASSWORD || undefined,
    prefix: "invisily:",
    enable_offline_queue: true,
    retry_strategy: function(options) {
    console.error('Redis connection error: ', (options.error || {}).message);
    console.log(`Retry connection to redis in ${Math.min(options.attempt * 5, 60)}s`);
    return Math.min(options.attempt * 5000, 60000);
    }
});


subscriber.on("connect", function(error) {
    console.log('Subscriber Connected.');
});

subscriber.on("end", function(error) {
    console.error('Subscriber connection lost.');
});

subscriber.on("reconnecting", function(error) {
    console.log('Subscriber connection reconnecting.');
});

subscriber.on("ready", function(error) {
    console.log('Subscriber connection ready.');
});

subscriber.on("error", function(error) {
    console.error('Subscriber error:', error || '');
});

module.exports = subscriber;
