var models = require('../models');

async function getAlertLogs(id) {
  if (id === 2) { 
    const transaction = await models.sequelize.transaction();
  
    let alerts = (
      await models.sequelize.query(
        `
          SELECT id, message, createdAt
          FROM AlertLogs al
          WHERE type != 'Admin Request' AND is_viewed = 0
          ORDER BY createdAt ASC;
        `
      )
    )[0];
  
    for (let alert of alerts) {
      await models.AlertLogs.update(
        {
          is_viewed: 1
        },
        {
          where: {
            id: alert.id
          },
          transaction: transaction
        }
      );
    }
  
    await transaction.commit();
    return alerts;
  }
}

async function getInfraPermissionsNotification(id) {
  if (id === 1) {
    const transaction = await models.sequelize.transaction();

    const query = `
      SELECT id, message, createdAt
      FROM AlertLogs
      WHERE type = :type 
      AND message LIKE :messagePattern
      AND is_viewed = 0
      ORDER BY createdAt ASC;
    `;
    let alerts = await models.sequelize.query(query, {
      replacements: {
        type: 'Admin Request',
        messagePattern: '%An additional permissions request is created by%'
      },
      type: models.sequelize.QueryTypes.SELECT
    });

    for (let alert of alerts) {
      await models.AlertLogs.update(
        {
          is_viewed: 1
        },
        {
          where: {
            id: alert.id
          },
          transaction: transaction
        }
      );
    }

    await transaction.commit();
    return alerts;
  }
}

async function getAdminPermissionsNotification(id) {
  if (id === 2) {
    const transaction = await models.sequelize.transaction();

    const query = `
      SELECT id, message, createdAt
      FROM AlertLogs
      WHERE type = :type 
      AND (
        message LIKE :messagePattern1
        OR message LIKE :messagePattern2
      )
      AND is_viewed = 0
      ORDER BY createdAt ASC;
    `;

    let alerts = await models.sequelize.query(query, {
      replacements: {
        type: 'Admin Request',
        messagePattern1: '%An additional permissions request is rejected by%',
        messagePattern2: '%An additional permissions request is approved by%'
      },
      type: models.sequelize.QueryTypes.SELECT
    });

    for (let alert of alerts) {
      await models.AlertLogs.update(
        {
          is_viewed: 1
        },
        {
          where: {
            id: alert.id
          },
          transaction: transaction
        }
      );
    }

    await transaction.commit();
    return alerts;
  }
}

module.exports = {
  getAlertLogs,
  getInfraPermissionsNotification,
  getAdminPermissionsNotification,
};
