const spawn = require('child_process').spawn;
const models = require('../models');

const dbConfig = require('../config/config')[process.env.NODE_ENV];
const dirSyncManager = require('../routes/common/dirsync-manager');

module.exports = function() {
  return models.Ldap.findOne({
    order: [['createdAt', 'DESC']],
    limit: 1,
    where: { }
  })
    .then(ldapConfig => {
      if (ldapConfig) {
        return dirSyncManager.prepareLSC(ldapConfig, dbConfig);
      }

      return Promise.reject(new Error('LDAP Settings not found'));
    })
    .then(() => {
      return new Promise((resolve, reject) => {
        const child = spawn(`lsc -c all -f ./lsc`, {
          shell: true,
          stdio: 'inherit'
        });

        child.on('close', resolve);
        child.on('error', reject);
      });
    })
    .then(() => {
      console.log('Sync Completed');
    })
    .catch(e => {
      console.log(e && (e.message || e));
      console.log('Sync Error');
    });
};
