const models = require('../models');
const moment = require('moment');

module.exports = function() {
  let CONNECTION_ATTEMPT_TIME = {};
  let whereClause = {};
  return models.Settings.findOne({
    limit: 1,
    order: [['id', 'DESC']]
  })
    .then(function(setting) {
      if (setting && setting.purge_allowed) {
        if (setting.auto_purge_days) {
          console.log(
            `======PURGING ACCESS LOGS OLDER THAN ${setting.auto_purge_days} DAYS======`
          );
          const purge_time = moment().subtract(
            setting.auto_purge_days * 24,
            'hours'
          );
          CONNECTION_ATTEMPT_TIME.lte = purge_time;
          whereClause.CONNECTION_ATTEMPT_TIME = CONNECTION_ATTEMPT_TIME;

          return Promise.all([
            models.Connection.destroy({
              where: whereClause
            }),
            setting.update({
              last_purge_date: moment()
            })
          ]);
        } else {
          return Promise.reject(new Error('Auto Purge Days Not Provided'));
        }
      } else {
        return Promise.reject(new Error('Auto Purge Days Not Turned ON'));
      }
    })
    .then(function() {
      console.log('Purged Old Access Logs');
    })
    .catch(error => {
      console.log(error && (error.message || error));
    });
};
