const moment = require('moment');
const models = require('../models');
const helper = require('../routes/common/helper');
const { actionType, publishLogOut} = require('../routes/common/event-publisher');

module.exports = async function(){
    try {
        
        const { user_pass_validity_period } = await models.ClientSetting.findOne({
            attributes: ['user_pass_validity_period'],
            raw: true
        });

        console.log("################################### LocalUsers Update Cron Started ###################################");


        const usersToUpdate = await models.LocalUsers.findAll({
            attributes: [
                'userName',
                'email',
                'name',
                'identity_provider'
            ],
            where: {
                [Op.and]: [
                    models.Sequelize.literal(`DATE_ADD(password_updated_at, INTERVAL ${user_pass_validity_period} DAY)<=NOW()`)
                ],
                userName: {
                    [Op.notLike]: '%gwadmin%'
                },
                identity_provider: {
                    [Op.eq]: 'Local User'
                }
            },
            raw: true
        });

        const clients = await models.Client.findAll({
            attributes: ['id'],
            where: {
                user: {
                    [Op.in]: usersToUpdate.map(u => u.userName)
                }
            },
            raw: true
        });

        let clientsIds = clients.map(c => c.id);
        clientsIds = [...new Set(clientsIds)];

        await Promise.all([
            models.LocalUsers.update({
                change_pass: true,
                password_updated_at: moment()
            }, {
                where: {
                    [Op.and]: [
                        models.Sequelize.literal(`DATE_ADD(password_updated_at, INTERVAL ${user_pass_validity_period} DAY)<=NOW()`)
                    ],
                    userName: {
                        [Op.notLike]: '%gwadmin%'
                    }
                }
            }),
            clientsIds.length ? publishLogOut(actionType.UPDATED, {
                code: 63,
                ids: clientsIds
            }) : []
        ]);

        usersToUpdate.forEach(usr => {
            helper.sendPasswordExpiryMail(usr.email, usr.name)
        });
  
    } catch (error) {
        console.log(error);
    }
}