var axios = require('axios');
var https = require('https');
var models = require('../models');
var path = require('path');
var fs = require('fs');
var archiver = require('archiver');
var projectConfig = require('../config/config')['project'][
  process.env.NODE_ENV
];
require('dotenv').config('/var/local/env');
var appDir = path.dirname(require.main.filename);
var Openssl = require('../openssl/openssl.js');
var OpensslObj = new Openssl();
var { exec } = require('child_process');
appDir = appDir.substring(0, appDir.lastIndexOf('/'));
let helper = require('../routes/common/helper');
let settings = require('../routes/common/settings');
let language = require('../routes/common/language');
const moment = require('moment');
const httpErrors = require('http-errors');
const crypto = require('crypto');

const { where, fn, col, literal } = require('sequelize');
const {
  actionType,
  publishClients,
  publishMsgFile,
  publishForceOnboard,

  clientColumnsToPublish
} = require('../routes/common/event-publisher');
const { Op } = require('sequelize');

var { generateLink } = require('../routes/common/content-upload');

const contentQueue = require('../queues/uploadToContent');

var log = Array();
let fileTypeMeta = {
  ubuntu: 'deb',
  windows: 'exe',
  macintosh: 'pkg',
  ubuntuserver: 'deb',
  ubuntuserver18: 'deb',
  ubuntu18: 'deb',
  centosserver: 'rpm'
};
let installerTypeMeta = {
  ubuntu: 'linux64-16',
  windows: 'win64',
  macintosh: 'mac',
  ios: 'ios',
  android: 'android',
  ubuntuserver: 'linux64-d-16',
  ubuntu18: 'linux64-18',
  ubuntuserver18: 'linux64-d-18',
  centosserver: 'linux64-d-16'
};
let installerDescriptiveName = {
  ubuntu: 'Ubuntu 16 Desktop',
  windows: 'Windows',
  macintosh: 'Macintosh',
  ios: 'iOS',
  android: 'Android',
  ubuntuserver: 'Ubuntu 16 Server',
  ubuntu18: 'Ubuntu Desktop',
  ubuntuserver18: 'Ubuntu Server',
  windowsserver: 'Windows Server',
  ubuntuserver20armv8: 'Ubuntu Server (ARMv8)',
  centosserver: 'CentOS Server'
};
var emailLinkExpiryInMinutes = 60;
let emailTemplateName = 'mobile_client_onboarding_email.html';

async function setExpiryTime() {
  emailLinkExpiryInMinutes = await settings.getExpirySettings();
}

const { token } = require('morgan');
const { setTimeout } = require('timers');
const { trim } = require('lodash');

const {
  getClientUpdateMessage,
  getClientAllowMessage,
  getClientBlockMessage,
  getClientForceOnBoardMessage,
  getClientEmailMessage
} = require('../config/adminLogs.config');

const { insertIntoAdminLogs } = require('../routes/common/adminLogs');

setExpiryTime();

const httpsAgent = new https.Agent({
  rejectUnauthorized: false // (NOTE: this will disable client ssl verification)
});

class LinkNotValidError extends Error {
  constructor(status, allowedStatuses) {
    super('Link may have been already expired or you have taken a wrong turn.');
    this.name = 'LinkNotValidError';
  }
}

class LinkAlreadyUsed extends Error {
  constructor() {
    super('Link is already used, please request for a new link');
    this.name = 'LinkAlreadyUsed';
  }
}

class PackageCancelError extends Error {
  constructor() {
    super();
  }
}

const downloadService = (req, res) => {
  let key = new Buffer(req.params.key, 'base64').toString('ascii');
  let os = key.split(':')[1];
  let folderPath = `${projectConfig.serviceInstallerPath}/client/output/` + os;
  var filePath = '';
  if (
    installerTypeMeta[os] === installerTypeMeta['ubuntu'] ||
    installerTypeMeta[os] === installerTypeMeta['ubuntu18']
  ) {
    filePath = `${folderPath}/invisily-client-` + installerTypeMeta[os];
  } else if (
    installerTypeMeta[os] === installerTypeMeta['ubuntuserver'] ||
    installerTypeMeta[os] === installerTypeMeta['ubuntuserver18'] ||
    installerTypeMeta[os] === installerTypeMeta['centosserver']
  ) {
    filePath = `${folderPath}/invisily-clientd-` + installerTypeMeta[os];
  } else {
    filePath = `${folderPath}/invisily-client-` + installerTypeMeta[os];
  }
  let downloadFile = `invisily-client-` + installerTypeMeta[os];
  let message = '';
  models.Client.findOne({
    where: {
      link_key: key
    }
  }).then(function(client) {
    if (client) {
      downloadFile += `-` + client.id + `.zip`;
      filePath += `-` + client.id + `.zip`;
      console.log(filePath);
      if (fs.existsSync(filePath)) {
        const linkExpiry = moment(client.link_key_expiry);
        console.log(
          'Client [' + client.Name + '] installation package expiry is: ',
          linkExpiry.format()
        );
        if (moment().isBefore(linkExpiry)) {
          console.log(
            'Client [' + client.Name + '] installation package is not expired'
          );
          res.download(filePath, downloadFile, function(err) {
            // file download is successful, expire link now
            if (!err) {
              client
                .update({
                  link_key_expiry: moment(),
                  package_used: true,
                  packageDownloaded: true
                })
                .then(function() {
                  console.log(
                    'Updated expiry of client [' + client.Name + '] to now'
                  );
                });
            } else if (err) {
              console.log(err && (err.message || err));
            }
          });
        } else {
          message =
            'Client [' +
            client.Name +
            '] installation package is expired, please request for a new link';
          helper.sendEmailLinkErrorHtmlResponse(message, 400, res);
        }
      } else {
        message =
          'Unable to find installation package for client component, please request for installer';
        helper.sendEmailLinkErrorHtmlResponse(message, 404, res);
      }
    } else {
      message =
        'Unable to find installation package for client component, please request for installer';
      helper.sendEmailLinkErrorHtmlResponse(message, 404, res);
    }
  });
};

//PUT /status
const status = async (req, res, next) => {
  try {
    const clientObj = req.body.client;
    let associatedClientIds = [];
    const client = await models.Client.findOne({
      where: {
        id: clientObj.id
      }
    });
    if (!client) {
      throw new httpErrors.NotFound('No client found.');
    }

    const bridgeDevice = await models.ServerBridge.findOne({
      where: {
        clientId: clientObj.id
      }
    });

    if (bridgeDevice) {
      associatedClientIds = await models.ServerBridgeDeviceId.findAll({
        attributes: ['client_id'],
        where: {
          serverBridgeId: bridgeDevice.id
        }
      });

      associatedClientIds = associatedClientIds.map(client => client.client_id);

      const associatedClients = await models.Client.findAll({
        where: {
          id: {
            [Op.in]: associatedClientIds
          }
        }
      });

      for (let j = 0; j < associatedClients.length; j++) {
        associatedClients[j].is_active = !clientObj.is_active;
        if (!client.is_active) {
          associatedClients[j].wrong_attempts = 0;
        }
        await associatedClients[j].save();
      }
    }

    clientObj.is_active = !clientObj.is_active;
    if (!client.is_active) {
      clientObj.wrong_attempts = 0;
    }

    await client.update(clientObj);

    let changedColumns = Object.keys(client._changed);
    changedColumns = changedColumns.filter(c =>
      clientColumnsToPublish.includes(c)
    );

    if (changedColumns.length) {
      const columns = {};
      changedColumns.forEach(column => {
        columns[column] = client[column];
      });

      await publishClients(actionType.UPDATED, {
        ids: [...associatedClientIds, client.id],
        data: columns,
        type: client.client_type
      });
    }

    let logMessage;
    if (clientObj.is_active) {
      logMessage = getClientAllowMessage(clientObj.Name);
    } else {
      logMessage = getClientBlockMessage(clientObj.Name);
    }
    insertIntoAdminLogs(logMessage, req);

    res.json({
      success: true,
      message: language.get['UPDATE_SUCCESS']
    });
  } catch (error) {
    next(error);
  }
};

//PUT /approve
const approve = async (req, res, next) => {
  try {
    const clientObj = req.body.client;
    const client = await models.Client.findOne({
      where: {
        id: clientObj.id
      }
    });

    if (!client) {
      throw new httpErrors.NotFound('No client found.');
    }

    clientObj.client_status = 3;

    await client.update(clientObj);

    let changedColumns = Object.keys(client._changed);
    changedColumns = changedColumns.filter(c =>
      clientColumnsToPublish.includes(c)
    );

    if (changedColumns.length) {
      const columns = {};
      changedColumns.forEach(column => {
        columns[column] = client[column];
      });

      await publishClients(actionType.UPDATED, {
        ids: [client.id],
        data: columns,
        type: client.client_type
      });
    }

    res.json({
      success: true
    });
  } catch (error) {
    next(error);
  }
};

//PUT /cancelPackage
const cancelPackage = async (req, res, next) => {
  try {
    const clientObj = req.body.client;
    const client = await models.Client.findOne({
      where: {
        id: clientObj.id
      }
    });

    if (!client) {
      throw new httpErrors.NotFound('No Client found.');
    }

    clientObj.isCancelled = true;
    clientObj.installer_status = 'canceled';

    await client.update(clientObj);

    let changedColumns = Object.keys(client._changed);
    changedColumns = changedColumns.filter(c =>
      clientColumnsToPublish.includes(c)
    );

    if (changedColumns.length) {
      const columns = {};
      changedColumns.forEach(column => {
        columns[column] = client[column];
      });

      await publishClients(actionType.UPDATED, {
        ids: [client.id],
        data: columns,
        type: client.client_type
      });
    }

    res.json({
      success: true,
      message: 'Cancelled'
    });
  } catch (error) {
    next(error);
  }
};

//POST /sendOTPQrCode
const sendOTPQrCode = async (req, res, next) => {
  try {
    const { user, name, email } = req.body;
    const username = trim(user || '');

    //TODO : Need to move to validations
    if (!username) {
      throw new httpErrors.UnprocessableEntity('username is required');
    }

    const client = await models.Client.findOne({
      where: {
        user: username
      }
    });

    if (!client) {
      throw new httpErrors.NotFound('Client not found');
    }

    let localUser = await client.getLocalUser();

    if (!client.OTP_image_data) {
      await client.generateOtp();

      let changedColumns = Object.keys(client._changed);
      changedColumns = changedColumns.filter(c =>
        clientColumnsToPublish.includes(c)
      );

      if (changedColumns.length) {
        const columns = {};
        changedColumns.forEach(column => {
          columns[column] = client[column];
        });

        await publishClients(actionType.UPDATED, {
          ids: [client.id],
          data: columns,
          type: client.client_type
        });
      }
    }

    helper.sendQRCode({
      OTP_image_data: client.OTP_image_data,
      recipient: email,
      recipientName: localUser ? localUser.name : client.Name,
      emailSubject: 'Your Invisily QR Code',
      templateFileName: 'client_linotp_email.html',
      onPremApiUrl: projectConfig.onPremApiUrl
    });

    res.json({
      success: true
    });
  } catch (error) {
    next(error);
  }
};

const sendOTPQr = async (req, res, next) => {
  try {
    const { device_id, user_id } = req.body;

    const [user, client] = await Promise.all([
      models.LocalUsers.findOne({
        where: {
          id: user_id
        }
      }),
      models.Client.findOne({
        where: {
          device_id
        }
      })
    ]);

    if (!user) {
      throw new httpErrors.NotFound('User not found');
    }

    if (!client) {
      throw new httpErrors.NotFound('Client not found');
    }

    if (!client.OTP_image_data) {
      await client.generateOtp();

      let changedColumns = Object.keys(client._changed);
      changedColumns = changedColumns.filter(c =>
        clientColumnsToPublish.includes(c)
      );

      if (changedColumns.length) {
        const columns = {};
        changedColumns.forEach(column => {
          columns[column] = client[column];
        });

        await publishClients(actionType.UPDATED, {
          ids: [client.id],
          data: columns,
          type: client.client_type
        });
      }
    }

    helper.sendQRCode({
      OTP_image_data: client.OTP_image_data,
      recipient: user.email,
      recipientName: user.name,
      emailSubject: 'Your Invisily QR Code',
      templateFileName: 'client_linotp_email.html',
      onPremApiUrl: projectConfig.onPremApiUrl
    });

    res.json({
      success: true
    });
  } catch (error) {
    next(error);
  }
};

//GET /verify/otp
const verifyOtp = async (req, res, next) => {
  try {
    const { user, pass } = req.query;
    const username = trim(user || '');
    const token = trim(pass || '');

    //TODO : Need to move to validations
    if (!username)
      throw new httpErrors.UnprocessableEntity('username is required');

    if (!token) throw new httpErrors.UnprocessableEntity('token is required');

    const client = await models.Client.findOne({
      attributes: ['id', 'user', 'name', 'linOTP_token'],
      where: {
        user: username,
        linOTP_token: {
          [Op.ne]: null
        }
      }
    });

    res.json({
      success: !client ? false : client.verifyOtp(token)
    });
  } catch (error) {
    next(error);
  }
};

//POST deregMFA
const deregMFA = async (req, res, next) => {
  try {
    const { id } = req.body;

    const client = await models.Client.findOne({
      where: {
        id
      }
    });

    if (!client) {
      throw new httpErrors.NotFound('Client not found.');
    }

    await client.update({
      mfa_status: 'deleted'
    });

    let changedColumns = Object.keys(client._changed);
    changedColumns = changedColumns.filter(c =>
      clientColumnsToPublish.includes(c)
    );

    if (changedColumns.length) {
      const columns = {};
      changedColumns.forEach(column => {
        columns[column] = client[column];
      });

      await publishClients(actionType.UPDATED, {
        ids: [client.id],
        data: columns,
        type: client.client_type
      });
    }

    res.json({ statusCode: 1200 });
  } catch (error) {
    next(error);
  }
};

//POST /getInstallerStatus
const getInstallerStatus = async (req, res, next) => {
  try {
    const { id } = req.body;
    const data = await models.Client.findAll({
      where: {
        id
      }
    });
    res.json(data);
  } catch (error) {
    next(error);
  }
};

//POST /unlockClient
const unlockClient = async (req, res, next) => {
  try {
    const { id } = req.body.id;
    const client = await models.Client.findOne({
      where: {
        id
      }
    });

    if (!client) {
      throw new httpErrors.NotFound('No client found.');
    }

    client.installer_status = 'new';
    await client.save();

    let changedColumns = Object.keys(client._changed);
    changedColumns = changedColumns.filter(c =>
      clientColumnsToPublish.includes(c)
    );

    if (changedColumns.length) {
      const columns = {};
      changedColumns.forEach(column => {
        columns[column] = client[column];
      });

      await publishClients(actionType.UPDATED, {
        ids: [client.id],
        data: columns,
        type: client.client_type
      });
    }

    res.json({
      success: true,
      message: language.get['CLIENT_UNLOCK_SUCCESS']
    });
  } catch (error) {
    next(error);
  }
};

const getQuickClients = async (req, res, next) => {
  try {
    console.log('req body is ', req.body);
    const {
      ids,
      allSelected,
      except = [],
      clientType,
      os = [],
      search = '',
      status = [],
      quickFilters
    } = req.body;

    const is_online = quickFilters && quickFilters.is_online;
    const is_active = quickFilters && quickFilters.is_active;

    const clients = await getAllClientwithfilters(
      ids,
      allSelected,
      except,
      clientType,
      os,
      search,
      status,
      is_online,
      is_active
    );

    res.json({
      success: true,
      data: clients || []
    });
  } catch (error) {
    next(error);
  }
};

const sendBulkInstallerLink = async clients => {
  clients.forEach(async client => {
    Promise.resolve()
      .then(async () => {
        let localUser = await client.getLocalUser();
        localUser = localUser ? localUser : { name: client.Name };
        const action =
          client.device_os === 'android' || client.device_os === 'ios'
            ? generateZip(client, client.client_status)
            : generateLink({
                os: client.device_os,
                component: 'client',
                componentId: client.id,
                expiry_time: emailLinkExpiryInMinutes,
                filename: 'installer'
              });
        return Promise.all([action, client, localUser]);
      })
      .then(function([link, client, localUser]) {
        let onuri = process.env.CONTENT_URL + '/api/v1/links';
        helper.sendInstallerEmail({
          recipient: client.Email,
          emailSubject: 'Invisily Client Software Installation',
          recipientName: localUser.name,
          onPremiseDownloadLink: `${onuri}/download/${link.key}`,
          offPremiseDownloadLink: `${onuri}/download/${link.key}`,
          templateFileName:
            projectConfig.onPremApiUrl !== projectConfig.offPremApiUrl
              ? 'client_onboarding_email.html'
              : 'client_onboarding_email_single.html',
          device_os: installerDescriptiveName[client.device_os]
            ? installerDescriptiveName[client.device_os]
            : client.device_os,
          package_expiry_time: moment(link.link_expired_at).fromNow(true)
        });
      });
  });
};

const getClientServicesWithGatewayGraph = async (req, res, next) => {
  try {
    let graphArray = [];
    let allServicesAndGatewaysRoutingPolicy = [];
    let allServicesOrGatewaysRoutingPolicy = [];
    let noAllServicesOrGatewaysRoutingPolicy = [];
    let notExistServices = [];
    let simplifiedAllServicesWithGateways = [];
    let notExistinGraphArray = [];

    // Get All Services
    const allServices = `SELECT services.id, services.Name, services.isGateWayAssigned AS isGateway, services.is_active AS isActive, services.is_gateway_less
        FROM Services services`;
    const allServicesResult = (await models.sequelize.query(allServices))[0];

    // Get All Gateways
    const allGateways = `SELECT gateways.id, gateways.Name FROM Gateways gateways`;
    const allGatewaysResult = (await models.sequelize.query(allGateways))[0];

    // Get all routing policies to check which are services will went through the gateways
    const routingPolicyCheck = `SELECT routingPolicies.id AS routingPolicyId, routingPolicies.name AS routingPolicyName, routingPolicies.all_gateways AS allGateways, routingPolicies.all_services AS allServices
        FROM RoutingPolicies routingPolicies`;

    const routingPolicyResult = (
      await models.sequelize.query(routingPolicyCheck)
    )[0];

    // All Services and All Gateways ( SuperSet )
    allServicesAndGatewaysRoutingPolicy = routingPolicyResult.filter(
      obj => obj.allServices == 1 && obj.allGateways == 1
    );

    // Service Groups / Gateways are assigned
    noAllServicesOrGatewaysRoutingPolicy = routingPolicyResult.filter(
      obj => obj.allServices == 0 && obj.allGateways == 0
    );

    // Either all services or All Gateways are assigned
    allServicesOrGatewaysRoutingPolicy = routingPolicyResult.filter(
      obj => (obj.allServices ^ obj.allGateways) === 1
    );

    if (allServicesAndGatewaysRoutingPolicy.length > 0) {
      allServicesResult.map((service, index) => {
        allGatewaysResult.forEach((gateway, index) => {
          graphArray.push({
            serviceId: service.id,
            serviceName: service.Name,
            isGateway: service.isGateway,
            is_active: service.is_active,
            routingPolicyId:
              allServicesAndGatewaysRoutingPolicy[0].routingPolicyId,
            routingPolicyName:
              allServicesAndGatewaysRoutingPolicy[0].routingPolicyName,
            gatewayId: gateway.id,
            gatewayName: gateway.Name
          });
        });
      });
    }

    if (allServicesOrGatewaysRoutingPolicy.length > 0) {
      await Promise.all(
        allServicesOrGatewaysRoutingPolicy.map(async (policy, count) => {
          if (policy.allServices == 1) {
            let gatewaysToAttached = `
                    SELECT DISTINCT gateways.id, gateways.Name
                    FROM RoutingPolicies routingPolicies
                    LEFT JOIN GatewayGroupPolicies AS gatewayGroupPolicies on gatewayGroupPolicies.policy_id  =  routingPolicies.id
                    LEFT JOIN GatewayGroups AS gatewayGroups on gatewayGroups.id = gatewayGroupPolicies.gateway_group_id
                    LEFT JOIN GatewayGroupPivots gatewayGroupPivots on gatewayGroupPivots.gateway_group_id = gatewayGroups.id
                    INNER JOIN Gateways AS gateways on gateways.id = gatewayGroupPivots.gateway_id
                    WHERE routingPolicies.id =  ${policy.routingPolicyId}

                    UNION

                    SELECT DISTINCT gateways.id, gateways.Name
                    FROM RoutingPolicies routingPolicies
                    LEFT JOIN GatewayPolicies AS gatewayPolicies on gatewayPolicies.policy_id =  routingPolicies.id
                    INNER JOIN Gateways AS gateways on gateways.id = gatewayPolicies.gateway_id
                    WHERE routingPolicies.id =  ${policy.routingPolicyId}`;

            const gatewaysToAttachedResult = (
              await models.sequelize.query(gatewaysToAttached)
            )[0];

            await Promise.all(
              allServicesResult.map(async (service, index) => {
                await Promise.all(
                  gatewaysToAttachedResult.map(async (gateway, index) => {
                    graphArray.push({
                      serviceId: service.id,
                      serviceName: service.Name,
                      isGateway: service.isGateway,
                      is_active: service.is_active,
                      routingPolicyId:
                        allServicesOrGatewaysRoutingPolicy[count]
                          .routingPolicyId,
                      routingPolicyName:
                        allServicesOrGatewaysRoutingPolicy[count]
                          .routingPolicyName,
                      gatewayId: gateway.id,
                      gatewayName: gateway.Name
                    });
                  })
                );
              })
            );
          } else if (policy.allGateways == 1) {
            let servicesToAttached = `
                    SELECT DISTINCT services.id, services.Name, services.isGateWayAssigned AS isGateway, services.is_active AS isActive,
                        routingPolicies.id AS RoutingPolicyGroupId, routingPolicies.name AS RoutingPolicyGroupName,
                        routingPolicies.id as RoutingPoliciesId
                    FROM Services services
                    LEFT JOIN GroupServices AS groupServices ON groupServices.serviceId = services.id
                    LEFT JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
                    LEFT JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
                    INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = serviceGroupPolicies.policy_id
                    WHERE routingPolicies.id =  ${policy.routingPolicyId}

                    UNION

                    SELECT DISTINCT services.id, services.Name, services.isGateWayAssigned AS isGateway, services.is_active AS isActive,
                        routingPolicies.id AS RoutingPolicyGroupId, routingPolicies.name AS RoutingPolicyGroupName,
                        routingPolicies.id as RoutingPoliciesId
                    FROM Services services
                    INNER JOIN ServicePolicies servicePolicies on servicePolicies.service_id = services.id
                    INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = servicePolicies.policy_id
                    WHERE routingPolicies.id =  ${policy.routingPolicyId}`;

            const servicesToAttachedResult = (
              await models.sequelize.query(servicesToAttached)
            )[0];
            await Promise.all(
              servicesToAttachedResult.map(async (service, index) => {
                await Promise.all(
                  allGatewaysResult.map(async (gateway, index) => {
                    graphArray.push({
                      serviceId: service.id,
                      serviceName: service.Name,
                      isGateway: service.isGateway,
                      is_active: service.is_active,
                      routingPolicyId:
                        allServicesOrGatewaysRoutingPolicy[count]
                          .routingPolicyId,
                      routingPolicyName:
                        allServicesOrGatewaysRoutingPolicy[count]
                          .routingPolicyName,
                      gatewayId: gateway.id,
                      gatewayName: gateway.Name
                    });
                  })
                );
              })
            );
          }
        })
      );
    }

    if (noAllServicesOrGatewaysRoutingPolicy.length > 0) {
      let serviceWithGateways = `
                -- Services Groups with Gateways Groups -> Routing Policy
                SELECT services.id as serviceId, services.Name as serviceName, services.isGateWayAssigned AS isGateway, services.is_active AS isActive,
                   routingPolicies.id AS routingPolicyId, routingPolicies.name AS routingPolicyName,
                   routingPolicies.id as RoutingPoliciesId, gateways.id as gatewayId, gateways.Name as gatewayName
                FROM Services services
                LEFT JOIN GroupServices AS groupServices ON groupServices.serviceId = services.id
                LEFT JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
                LEFT JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
                INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = serviceGroupPolicies.policy_id
                LEFT JOIN GatewayGroupPolicies AS gatewayGroupPolicies on gatewayGroupPolicies.policy_id  =  routingPolicies.id
                LEFT JOIN GatewayGroups AS gatewayGroups on gatewayGroups.id = gatewayGroupPolicies.gateway_group_id
                LEFT JOIN GatewayGroupPivots gatewayGroupPivots on gatewayGroupPivots.gateway_group_id = gatewayGroups.id
                INNER JOIN Gateways AS gateways on gateways.id = gatewayGroupPivots.gateway_id

                UNION

                -- Services with Gateways Groups -> Routing Policy
                SELECT services.id as serviceId, services.Name as serviceName, services.isGateWayAssigned AS isGateway, services.is_active AS isActive,
                   routingPolicies.id AS routingPolicyId, routingPolicies.name AS routingPolicyName,
                   routingPolicies.id as RoutingPoliciesId, gateways.id as gatewayId, gateways.Name as gatewayName
                FROM Services services
                INNER JOIN ServicePolicies servicePolicies on servicePolicies.service_id = services.id
                INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = servicePolicies.policy_id
                LEFT JOIN GatewayGroupPolicies AS gatewayGroupPolicies on gatewayGroupPolicies.policy_id  =  routingPolicies.id
                LEFT JOIN GatewayGroups AS gatewayGroups on gatewayGroups.id = gatewayGroupPolicies.gateway_group_id
                LEFT JOIN GatewayGroupPivots gatewayGroupPivots on gatewayGroupPivots.gateway_group_id = gatewayGroups.id
                INNER JOIN Gateways AS gateways on gateways.id = gatewayGroupPivots.gateway_id

                UNION

                 -- Services with Gateways -> Routing Policy
                SELECT services.id as serviceId, services.Name as serviceName, services.isGateWayAssigned AS isGateway, services.is_active AS isActive,
                   routingPolicies.id AS routingPolicyId, routingPolicies.name AS routingPolicyName,
                   routingPolicies.id as RoutingPoliciesId, gateways.id as gatewayId, gateways.Name as gatewayName
                FROM Services services
                INNER JOIN ServicePolicies servicePolicies on servicePolicies.service_id = services.id
                INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = servicePolicies.policy_id
                LEFT JOIN GatewayPolicies AS gatewayPolicies on gatewayPolicies.policy_id =  routingPolicies.id
                INNER JOIN Gateways AS gateways on gateways.id = gatewayPolicies.gateway_id

                UNION

                -- Services with Gateways Groups -> Routing Policy
                SELECT services.id as serviceId, services.Name as serviceName, services.isGateWayAssigned AS isGateway, services.is_active AS isActive,
                   routingPolicies.id AS routingPolicyId, routingPolicies.name AS routingPolicyName,
                   routingPolicies.id as RoutingPoliciesId, gateways.id as gatewayId, gateways.Name as gatewayName
                FROM Services services
                LEFT JOIN GroupServices AS groupServices ON groupServices.serviceId = services.id
                LEFT JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
                LEFT JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
                INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = serviceGroupPolicies.policy_id
                LEFT JOIN GatewayGroupPolicies AS gatewayGroupPolicies on gatewayGroupPolicies.policy_id  =  routingPolicies.id
                LEFT JOIN GatewayPolicies AS gatewayPolicies on gatewayPolicies.policy_id =  routingPolicies.id
                INNER JOIN Gateways AS gateways on gateways.id = gatewayPolicies.gateway_id `;

      const serviceWithGatewaysResult = (
        await models.sequelize.query(serviceWithGateways)
      )[0];

      await Promise.all(
        serviceWithGatewaysResult.map(async (object, index) => {
          graphArray.push({
            serviceId: object.serviceId,
            serviceName: object.serviceName,
            isGateway: object.isGateway,
            is_active: object.is_active,
            routingPolicyId: object.routingPolicyId,
            routingPolicyName: object.routingPolicyName,
            gatewayId: object.gatewayId,
            gatewayName: object.gatewayName
          });
        })
      );
    }

    // Remove the Duplicate rows
    simplifiedAllServicesWithGateways = graphArray.filter(
      (data, index, arr) => {
        const duplicateIndex = arr.findIndex(
          (d, i) =>
            (i !== index &&
              d.serviceId === data.serviceId &&
              d.routingPolicyId !== null &&
              data.routingPolicyId === null) ||
            (d.gatewayId !== null && data.gatewayId === null)
        );
        return duplicateIndex === -1;
      }
    );
    graphArray = [];

    // Check If all service all assigned to the Client
    const isAllServicesAssigned = `
        SELECT entitlements.all_services
        FROM ClientPolicies AS clientPolicies
        INNER JOIN Clients AS client ON clientPolicies.client_id = client.id
        LEFT JOIN Entitlements entitlements ON entitlements.id = clientPolicies.policy_id
        WHERE clientPolicies.policy_type_id = 1 AND client.id = ${req.body.clientId} AND entitlements.all_services IS NOT NULL

        UNION

        SELECT entitlements.all_services
        FROM ClientGroupPolicies AS clientGroupPolicies
        INNER JOIN GroupClients AS groupClients ON groupClients.groupId = clientGroupPolicies.client_group_id
        INNER JOIN Clients AS client ON groupClients.clientId  = client.id
        LEFT JOIN Entitlements entitlements ON entitlements.id = clientGroupPolicies.policy_id
        WHERE clientGroupPolicies.policy_type_id = 1 AND client.id = ${req.body.clientId} and entitlements.all_services IS NOT NULL
        
        UNION
        
        SELECT entitlements.all_services
        FROM UserPolicies  AS userPolicies
        INNER JOIN Users AS users ON userPolicies.user_id  = users.id
        LEFT JOIN Entitlements entitlements ON entitlements.id = userPolicies.policy_id
        INNER JOIN Clients AS clients ON clients.user_id = users.id 
        WHERE userPolicies.policy_type_id = 1 AND clients.id = ${req.body.clientId} AND entitlements.all_services IS NOT NULL

        UNION

        SELECT entitlements.all_services
        FROM UserGroupPolicies AS userGroupPolicies
        INNER JOIN UserPolicies AS userPoliucies ON userPoliucies.policy_id  = userGroupPolicies.policy_id 
        INNER JOIN Users AS users ON userPoliucies.user_id  = users.id
        LEFT JOIN Entitlements entitlements ON entitlements.id = userGroupPolicies.policy_id
        INNER JOIN Clients AS clients ON clients.user_id = users.id 
        WHERE userGroupPolicies.policy_type_id = 1 AND clients.id = ${req.body.clientId} and entitlements.all_services IS NOT NULL

`;

    let allServicesEntitlements = await models.sequelize.query(
      isAllServicesAssigned,
      { plain: true }
    );

    if (!allServicesEntitlements) allServicesEntitlements = 0;

    if (
      allServicesEntitlements?.all_services == 1 &&
      allServicesAndGatewaysRoutingPolicy.length < 1
    ) {
      notExistServices = allServicesResult.filter(
        serviceObj =>
          !simplifiedAllServicesWithGateways.some(
            graphObj => serviceObj.id === graphObj.serviceId
          )
      );
      notExistServices.map(service => {
        simplifiedAllServicesWithGateways.push({
          serviceId: service.id,
          serviceName: service.Name,
          isGateway: null,
          is_active: null,
          routingPolicyId: null,
          routingPolicyName: null,
          gatewayId: null,
          gatewayName: null
        });
      });
      graphArray = simplifiedAllServicesWithGateways;
    }
    if (
      (allServicesEntitlements?.all_services == 0 &&
        allServicesAndGatewaysRoutingPolicy.length < 1) ||
      !allServicesEntitlements
    ) {
      const notAllServicesAssigned = `
            SELECT s.id, s.Name
            FROM Services s
            JOIN ServicePolicies sp ON s.id = sp.service_id
            JOIN ClientPolicies cp ON cp.policy_id = sp.policy_id
            JOIN Entitlements e ON e.id = cp.policy_id
            WHERE cp.policy_type_id = (
                SELECT id
                FROM PolicyTypeLookup
                WHERE policy_type = 'Entitlement'
            )
            AND cp.client_id = ${req.body.clientId}

            UNION

            SELECT s.id, s.Name
            FROM Services s
            JOIN ServicePolicies sp ON s.id = sp.service_id
            JOIN ClientGroupPolicies AS cgp ON cgp.policy_id = sp.policy_id
            INNER JOIN GroupClients AS groupClients ON groupClients.groupId = cgp.client_group_id
            JOIN Entitlements e ON e.id = cgp.policy_id
            WHERE cgp.policy_type_id = (
                SELECT id
                FROM PolicyTypeLookup
                WHERE policy_type = 'Entitlement'
            )
            AND groupClients.clientId = ${req.body.clientId}

            UNION

            SELECT s.id, s.Name
            FROM Services s
            JOIN GroupServices gs ON gs.serviceId = s.id
            JOIN ServiceGroupPolicies sgp ON sgp.service_group_id = gs.groupId
            JOIN ClientPolicies cp ON cp.policy_id = sgp.policy_id
            JOIN Entitlements e ON e.id = cp.policy_id
            WHERE cp.policy_type_id = (
                SELECT id
                FROM PolicyTypeLookup
                WHERE policy_type = 'Entitlement'
            )
            AND cp.client_id = ${req.body.clientId}

            UNION

            SELECT s.id, s.Name
            FROM Services s
            JOIN GroupServices gs ON gs.serviceId = s.id
            JOIN ServiceGroupPolicies sgp ON sgp.service_group_id = gs.groupId
            JOIN ClientGroupPolicies AS cgp ON cgp.policy_id = sgp.policy_id
            INNER JOIN GroupClients AS groupClients ON groupClients.groupId = cgp.client_group_id
            JOIN Entitlements e ON e.id = cgp.policy_id
            WHERE cgp.policy_type_id = (
                SELECT id
                FROM PolicyTypeLookup
                WHERE policy_type = 'Entitlement'
                )
                AND groupClients.clientId = ${req.body.clientId}
                
            UNION
            
            SELECT s.id, s.Name
            FROM Services s
            JOIN ServicePolicies sp ON s.id = sp.service_id
            INNER JOIN UserPolicies up on up.policy_id = sp.policy_id
            INNER JOIN Users AS users ON up.user_id  = users.id
            JOIN Entitlements e ON e.id = up.policy_id
            INNER JOIN Clients c on c.user_id = users.id 
            WHERE up.policy_type_id = (
                SELECT id
                FROM PolicyTypeLookup
                WHERE policy_type = 'Entitlement'
                )
                AND c.id = ${req.body.clientId}

            UNION

            SELECT s.id, s.Name
            FROM Services s
            JOIN ServicePolicies sp ON s.id = sp.service_id
            JOIN UserGroupPolicies ugp on ugp.policy_id = sp.policy_id
            JOIN UserGroupsPivot ugp2 on ugp2.group_id = ugp.user_group_id 
            JOIN Entitlements e ON e.id = ugp.policy_id
            JOIN Clients c on c.user_id = ugp2.user_id 
            WHERE ugp.policy_type_id = (
                SELECT id
                FROM PolicyTypeLookup
                WHERE policy_type = 'Entitlement'
                )
                AND c.id = ${req.body.clientId}

            UNION

            SELECT s.id, s.Name
            FROM Services s
            JOIN GroupServices gs ON gs.serviceId = s.id
            JOIN ServiceGroupPolicies sgp ON sgp.service_group_id = gs.groupId
            JOIN UserPolicies up on up.policy_id = sgp.policy_id
            INNER JOIN Users AS users ON up.user_id  = users.id
            JOIN Entitlements e ON e.id = up.policy_id
            INNER JOIN Clients c on c.user_id = users.id 
            WHERE up.policy_type_id = (
                SELECT id
                FROM PolicyTypeLookup
                    WHERE policy_type = 'Entitlement'
                )
                AND c.id  = ${req.body.clientId}

            UNION

            SELECT s.id, s.Name
            FROM Services s
            JOIN GroupServices gs ON gs.serviceId = s.id
            JOIN ServiceGroupPolicies sgp ON sgp.service_group_id = gs.groupId
            JOIN UserGroupPolicies ugp on ugp.policy_id = sgp.policy_id
            JOIN UserGroupsPivot ugp2 on ugp2.group_id = ugp.user_group_id 
            JOIN Entitlements e ON e.id = ugp.policy_id
            JOIN Clients c on c.user_id = ugp2.user_id 
            WHERE sgp.policy_type_id = (
                SELECT id
                FROM PolicyTypeLookup
                WHERE policy_type = 'Entitlement'
                )
                AND c.id  = ${req.body.clientId}`;

      const notAllServicesEntitlements = (
        await models.sequelize.query(notAllServicesAssigned)
      )[0];

      if (notAllServicesEntitlements.length > 0) {
        graphArray = [];
        const existingServicesWithGateways = simplifiedAllServicesWithGateways.filter(
          serviceObj =>
            notAllServicesEntitlements.some(
              graphObj => serviceObj.serviceId === graphObj.id
            )
        );

        notExistServices = notAllServicesEntitlements.filter(
          serviceObj =>
            !simplifiedAllServicesWithGateways.some(
              graphObj => serviceObj.id === graphObj.serviceId
            )
        );

        notExistServices.map(service => {
          graphArray.push({
            serviceId: service.id,
            serviceName: service.Name,
            isGateway: null,
            is_active: null,
            routingPolicyId: null,
            routingPolicyName: null,
            gatewayId: null,
            gatewayName: null
          });
        });
        graphArray = graphArray.concat(existingServicesWithGateways);
      }
    }

    // Check if entitlements exists where all clients is 1 with services /groups
    const allCientsAssigned = `
        SELECT id AS serviceId, Name AS serviceName FROM Services
        WHERE id IN (
        SELECT service_id FROM ServicePolicies
        WHERE policy_id IN (
            SELECT entitlements.id FROM Entitlements entitlements
            WHERE entitlements.all_clients = 1
            ) AND policy_type_id = 1
        )

        UNION

        SELECT services.id AS serviceId, services.Name AS serviceName FROM Services services
        LEFT JOIN GroupServices AS groupServices ON groupServices.serviceId = services.id
        LEFT JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
        INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
        WHERE serviceGroupPolicies.policy_id IN (
        SELECT entitlements.id FROM Entitlements entitlements
        WHERE entitlements.all_clients = 1
        ) AND serviceGroupPolicies.policy_type_id = 1
        
        UNION
        
        SELECT id AS serviceId, Name AS serviceName FROM Services
        WHERE id IN (
        SELECT service_id FROM ServicePolicies
        WHERE policy_id IN (
            SELECT entitlements.id FROM Entitlements entitlements
            WHERE entitlements.all_users = 1
            ) AND policy_type_id = 1
        )

        UNION

        SELECT services.id AS serviceId, services.Name AS serviceName FROM Services services
        LEFT JOIN GroupServices AS groupServices ON groupServices.serviceId = services.id
        LEFT JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
        INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
        WHERE serviceGroupPolicies.policy_id IN (
        SELECT entitlements.id FROM Entitlements entitlements
        WHERE entitlements.all_users = 1
        ) AND serviceGroupPolicies.policy_type_id = 1`;
    const allCientsAssignedResult = (
      await models.sequelize.query(allCientsAssigned)
    )[0];

    if (allCientsAssignedResult.length > 0) {
      notExistinGraphArray = allCientsAssignedResult.filter(
        services =>
          !graphArray.some(
            serviceObj => serviceObj.serviceId === services.serviceId
          )
      );

      const alsoNotGatewayServices = notExistinGraphArray.filter(
        services =>
          !simplifiedAllServicesWithGateways.some(
            serviceObj => serviceObj.serviceId === services.serviceId
          )
      );

      const alsoGatewayServices = simplifiedAllServicesWithGateways.filter(
        serviceObj =>
          notExistinGraphArray.some(
            services => services.serviceId === serviceObj.serviceId
          )
      );
      if (alsoGatewayServices.length > 0) {
        graphArray = graphArray.concat(alsoGatewayServices);
      }
      if (alsoNotGatewayServices.length > 0) {
        alsoNotGatewayServices.map((service, index) => {
          graphArray.push({
            serviceId: service.serviceId,
            serviceName: service.serviceName,
            isGateway: null,
            is_active: null,
            routingPolicyId: null,
            routingPolicyName: null,
            gatewayId: null,
            gatewayName: null
          });
        });
      }
    }

    const everyThing = `SELECT entitlements.id FROM Entitlements entitlements
        WHERE entitlements.all_clients = 1 AND entitlements .all_services = 1`;

    const everyThingResult = (await models.sequelize.query(everyThing))[0];

    //Check if all clients and services are assigned
    if (everyThingResult.length > 0) {
      notExistServices = allServicesResult.filter(
        serviceObj =>
          !simplifiedAllServicesWithGateways.some(
            graphObj => serviceObj.id === graphObj.serviceId
          )
      );
      notExistServices.map(service => {
        simplifiedAllServicesWithGateways.push({
          serviceId: service.id,
          serviceName: service.Name,
          isGateway: null,
          is_active: null,
          routingPolicyId: null,
          routingPolicyName: null,
          gatewayId: null,
          gatewayName: null
        });
      });
      graphArray = simplifiedAllServicesWithGateways;
    }

    if (
      allServicesEntitlements ||
      allServicesEntitlements == 0 ||
      allCientsAssignedResult.length > 0 ||
      everyThingResult.length > 0
    ) {
      res.json(graphArray);
    } else {
      res.json([]);
    }
  } catch (error) {
    next(error);
  }
};

const getAllClientwithfilters = async (
  ids,
  allSelected,
  except = [],
  clientType,
  os = [],
  search = '',
  status = [],
  isOnline,
  activeFilter
) => {
  const whereOptions = {
    client_type: clientType,
    [Op.and]: []
  };

  let findQuery = models.Client;

  if (!allSelected && ids) {
    whereOptions.id = ids;
  } else {
    if (allSelected && except.length > 0) {
      whereOptions[Op.and].push({
        id: {
          [Op.notIn]: except
        }
      });
    }

    if (search && search.trim().length > 0) {
      let name = `%${search}%`;
      whereOptions[Op.and].push({
        [Op.or]: [
          {
            Name: {
              [Op.like]: name
            }
          },
          {
            user: {
              [Op.like]: name
            }
          },
          {
            agent_id: {
              [Op.like]: name
            }
          },
          {
            client_version: {
              [Op.like]: name
            }
          }
        ]
      });
    }

    if (os && os.length > 0) {
      whereOptions[Op.and].push({
        device_os: {
          [Op.in]: os
        }
      });
    }

    if (status && status.length > 0) {
      if (status.includes(4)) {
        //provisioned
        whereOptions[Op.and].push({
          [Op.or]: [
            {
              client_status: {
                [Op.in]: status
              }
            },
            {
              device_os: {
                [Op.in]: ['android', 'ios']
              }
            }
          ]
        });
      } else {
        //new,waiting,registered
        whereOptions[Op.and].push({
          client_status: {
            [Op.in]: status
          },
          device_os: {
            [Op.notIn]: ['android', 'ios']
          }
        });
      }
    }

    // const asf = activeFilter.trim()
    // const len = asf.length

    if (activeFilter && activeFilter.trim().length) {
      let activeArray = activeFilter.split(',');
      activeArray = activeArray.filter(active => active != -1);
      activeArray.length > 0 &&
        whereOptions[Op.and].push({
          is_active: {
            [Op.in]: activeArray
          }
        });
    }

    let scope = '';
    let scopeValue = false;
    let totalScopes = [];
    if (isOnline && isOnline.trim().length) {
      const isOnlineArray = isOnline.split(',');
      if (isOnlineArray.length !== 2) {
        scope = 'online';
        scopeValue = isOnlineArray[0] === '0' ? false : true;
      }
    }

    if (scope) {
      totalScopes.push({ method: [scope, scopeValue] });
    }

    if (clientType === 'Server') {
      totalScopes = [...totalScopes, 'boundServerBridge'];
    }

    if (totalScopes.length) {
      findQuery = models.Client.scope(totalScopes);
    }
  }

  const clients = await findQuery.findAll({
    where: whereOptions
  });

  return clients;
};

//TODO : Need to be refactor
const sendLink = async (req, res, next) => {
  const {
    ids,
    allSelected,
    except = [],
    clientType,
    os = [],
    search = '',
    status = [],
    quickFilters: { is_online, is_active }
  } = req.body;

  const options = {
    where: {
      [Op.and]: [{ client_type: clientType }]
    }
  };

  if (!allSelected && ids && ids.length > 0) {
    options.where[Op.and].push({
      id: { [Op.in]: ids }
    });
  } else {
    if (allSelected) {
      options.where[Op.and].push({
        id: { [Op.notIn]: ids }
      });
    }
  }

  const userOptions = {
    ...options,
    attributes: ['Name']
  };

  const clients = await models.Client.findAll(userOptions);

  for (const client of clients) {
    insertIntoAdminLogs(getClientEmailMessage(client.Name), req);
  }
  // Adding Device ID link to DB
  setExpiryTime()
    .then(async () => {
      const clients = await getAllClientwithfilters(
        ids,
        allSelected,
        except,
        clientType,
        os,
        search,
        status,
        is_online,
        is_active
      );
      if (!clients || clients.length < 0) {
        throw new httpErrors.NotFound('Client not found.');
      }

      sendBulkInstallerLink(clients);

      res.json({
        status: true
      });
    })
    .catch(err => {
      console.log(err);
      console.log(err && (err.message || err));
      res.json({
        status: false,
        message: (err || {}).message
      });
    });
};

const sendPackage = client => {
  return new Promise(async (resolve, reject) => {
    let localUser = await client.getLocalUser();
    localUser = localUser ? localUser : { name: client.Name };

    const link = await generateLink({
      os: client.device_os,
      component: 'client',
      componentId: client.id,
      expiry_time: emailLinkExpiryInMinutes,
      filename: 'installer'
    });

    let onuri = process.env.CONTENT_URL + '/api/v1/links';
    helper.sendInstallerEmail({
      recipient: client.Email,
      emailSubject: 'Invisily Client Software Installation',
      recipientName: localUser.name,
      onPremiseDownloadLink: `${onuri}/download/${link.key}`,
      offPremiseDownloadLink: `${onuri}/download/${link.key}`,
      templateFileName:
        projectConfig.onPremApiUrl !== projectConfig.offPremApiUrl
          ? 'client_onboarding_email.html'
          : 'client_onboarding_email_single.html',
      device_os: installerDescriptiveName[client.device_os]
        ? installerDescriptiveName[client.device_os]
        : client.device_os,
      package_expiry_time: moment(link.link_expired_at).fromNow(true)
    });

    resolve(true);
  });
};

//TODO : Need to be refactor
const download = async (req, res, next) => {
  const { key } = req.params;
  let dp = null;
  models.DownloadPackage.findOne({
    where: {
      link_key: key
    }
  })
    .then(function(DP) {
      if (DP) {
        if (moment().isBefore(moment(DP.link_key_expiry))) {
          if (DP.used == 0) {
            dp = DP;
            return models.Client.findOne({
              where: {
                id: DP.client_id
              }
            });
          } else {
            throw new LinkAlreadyUsed();
          }
        } else {
          //Link Expiry.
          throw new LinkNotValidError();
        }
      }
    })
    .then(function(client) {
      if (client) {
        return client.device_os;
      }
      throw new Error();
    })
    .then(deviceOS => {
      var filePath =
        `${projectConfig.serviceInstallerPath}/` +
        'client' +
        '/input/' +
        deviceOS +
        '/';
      if (
        deviceOS.includes('ubuntu') ||
        deviceOS.includes('macintosh') ||
        deviceOS.includes('windows') ||
        deviceOS.includes('centosserver')
      ) {
        filePath += 'installer.zip';
      }
      var fileName = 'invisily-client.zip';
      if (deviceOS.includes('ubuntu')) {
        if (deviceOS === 'ubuntu' || deviceOS === 'ubuntuserver') {
          if (deviceOS.includes('server')) {
            fileName = 'invisily-client-linux64-server-16.zip';
          } else {
            fileName = 'invisily-client-linux64-desktop-16.zip';
          }
        } else {
          if (deviceOS.includes('server')) {
            fileName = 'invisily-client-linux64-server-18.zip';
          } else {
            fileName = 'invisily-client-linux64-desktop-18.zip';
          }
        }
      } else if (deviceOS.includes('centosserver')) {
        fileName = 'invisily-client-linux64-centosserver.zip';
      } else if (deviceOS == 'macintosh') {
        fileName = 'invisily-client-mac.zip';
      } else if (deviceOS == 'windows') {
        fileName = 'invisily-client-window.zip';
      } else if (deviceOS == 'windowsserver') {
        fileName = 'invisily-client-window-server.zip';
      }
      res.download(filePath, fileName);
    })
    .then(() => {
      if (dp) {
        //update used key to 1
        return dp.update({
          used: 1
        });
      }
    })
    .catch(err => {
      console.log(err && (err.message || err));
      if (err instanceof LinkNotValidError) {
        let message =
          'Installation package link is expired, please request for a new link.';
        helper.sendEmailLinkErrorHtmlResponse(message, 400, res);
      } else if (err instanceof LinkAlreadyUsed) {
        //let message = 'Installation package link is already used, please request for a new link.';
        let message =
          'Installation package link is expired, please request for a new link.';
        helper.sendEmailLinkErrorHtmlResponse(message, 400, res);
      } else {
        res.status(500).json(err);
      }
    });
};

// generate installer package for client
const generateService = async (req, res, next) => {
  try {
    const { id, state } = req.params;
    const client = await models.Client.findOne({
      where: {
        id
      }
    });

    if (!client) {
      throw new httpErrors.NotFound('No client found.');
    }

    client.installer_logs = '';
    client.package_used = false;
    client.isCancelled = false;

    await client.save();

    let changedColumns = Object.keys(client._changed);
    changedColumns = changedColumns.filter(c =>
      clientColumnsToPublish.includes(c)
    );

    if (changedColumns.length) {
      const columns = {};
      changedColumns.forEach(column => {
        columns[column] = client[column];
      });

      await publishClients(actionType.UPDATED, {
        ids: [client.id],
        data: columns,
        type: client.client_type
      });
    }

    generateZip(client, state);

    res.json({
      status: true,
      message: language.get['CHECK_EMAIL_NEW_INSTALLER']
    });
  } catch (error) {
    next(error);
  }
};

/* GET clients listing. */
const getClients = async (req, res, next) => {
  try {
    const {
      limit,
      offset,
      sort = 'id',
      dir = 'desc',
      type = '',
      adOrphand,
      localOrphand,
      isMfa,
      isDarkened,
      isOnline,
      name: nameFilter,
      package_status: packageFilter,
      is_active: activeFilter,
      device_os: deviceOSFilter,
      all = false,
      except = []
    } = req.query;

    const options = {
      attributes: [
        'agent_id',
        'Name',
        'Email',
        'device_hardware_id',
        'user',
        'device_os',
        'client_type',
        'client_status',
        'is_local_deleted',
        'is_ldap_deleted',
        'waiting_reason',
        'device_os_full_name',
        'recieved_os_version',
        'recieved_antivirus',
        'recieved_epp',
        'serial_number',
        'is_active',
        'id',
        'qr_json',
        'client_mfa',
        'heart_beat',
        'heart_beat_sent_at',
        'link_key_expiry',
        'package_used',
        'installer_logs',
        'client_version',
        'mfa_status',
        'risk_score',
        'auto_updates'
      ],
      where: {
        $and: []
      },
      limit: +limit,
      offset: +offset * +limit,
      order: []
    };

    // const clientType = type === '*' ? false : true;
    // console.log('client type', clientType);
    if (type) {
      options.where.client_type = type;
    }

    if (nameFilter && nameFilter.trim().length) {
      let name = `%${nameFilter}%`;
      options.where.$and.push({
        [Op.or]: [
          {
            Name: {
              [Op.like]: name
            }
          },
          {
            user: {
              [Op.like]: name
            }
          },
          {
            agent_id: {
              [Op.like]: name
            }
          },
          {
            client_version: {
              [Op.like]: name
            }
          }
        ]
      });
    }

    if (deviceOSFilter && deviceOSFilter.trim().length) {
      const osArray = deviceOSFilter.split(',');
      options.where.$and.push({
        device_os: {
          [Op.in]: osArray
        }
      });
    }

    if (packageFilter && packageFilter.trim().length) {
      const packageStatusArray = packageFilter.split(',');
      options.where.$and.push({
        [Op.or]: [
          { client_status: { [Op.in]: packageStatusArray } },
          {
            [Op.or]: [{ device_os: 'Android' }, { device_os: 'iOS' }]
          }
        ]
      });
    }

    if (packageFilter && !packageFilter.includes(4)) {
      options.where.$and.push({
        [Op.or]: [
          { device_hardware_id: '00000000000000000000000000000000' },
          {
            [Op.not]: {
              [Op.or]: [{ device_os: 'android' }, { device_os: 'ios' }]
            }
          }
        ]
      });
    }

    if (packageFilter && !packageFilter.includes(1)) {
      options.where.$and.push({
        [Op.or]: [
          {
            [Op.not]: {
              device_hardware_id: '00000000000000000000000000000000'
            }
          },
          {
            [Op.not]: {
              [Op.or]: [{ device_os: 'android' }, { device_os: 'ios' }]
            }
          }
        ]
      });
    }

    if (activeFilter && activeFilter.trim().length) {
      const activeArray = activeFilter.split(',');
      options.where.$and.push({
        is_active: {
          [Op.in]: activeArray
        }
      });
    }

    if (isDarkened && isDarkened.trim().length) {
      const darkenedArray = isDarkened.split(',');
      options.where.$and.push({
        client_darkening: {
          [Op.in]: darkenedArray
        }
      });
    }

    if (isMfa && isMfa.trim().length) {
      const isMfaArray = isMfa.split(',');
      options.where.$and.push({
        client_mfa: {
          [Op.in]: isMfaArray
        }
      });
    }

    let scope = '';
    let scopeValue = false;
    let totalScopes = [];
    if (isOnline && isOnline.trim().length) {
      const isOnlineArray = isOnline.split(',');
      if (isOnlineArray.length !== 2) {
        scope = 'online';
        scopeValue = isOnlineArray[0] === '0' ? false : true;
      }
    }

    // if (options.where.$and.length > 0) {
    //   options.where.options.where.$and = options.where$and;
    // }

    if (adOrphand === 'true') {
      options.where['is_ldap_deleted'] = true;
    }

    if (localOrphand === 'true') {
      options.where['is_local_deleted'] = true;
    }

    let findQuery = models.Client;

    if (scope) {
      totalScopes.push({ method: [scope, scopeValue] });
    }

    if (type === 'Server') {
      totalScopes = [...totalScopes, 'boundServerBridge'];
    }

    if (totalScopes.length) {
      findQuery = models.Client.scope(totalScopes);
    }

    const allowedSortValues = ['Name', 'id', 'createdAt'];

    if (allowedSortValues.includes(sort)) {
      options.order.push([models.sequelize.literal(sort), dir]);
    }

    const settingOptions = {
      attributes: ['is_ldap_syncing', 'biometrics_state'],
      order: [['id', 'DESC']],
      limit: 1
    };

    const clientSettingOptions = {
      attributes: ['otp_state'],
      order: [['id', 'DESC']],
      limit: 1
    };

    if (all == 'true') {
      delete options['offset'];
      delete options['limit'];
    }

    let skip = [];
    try {
      skip = JSON.parse(except);
    } catch (e) {
      console.log('error');
    }

    if (Array.isArray(skip) && skip.length > 0) {
      options.where['id'] = {
        [Op.not]: skip
      };
    }

    let [
      setting,
      clientSetting,
      { rows: data, count: recordsTotal }
    ] = await Promise.all([
      models.Settings.findOne(settingOptions),
      models.ClientSetting.findOne(clientSettingOptions),
      findQuery.findAndCountAll(options)
    ]);

    // console.log();

    const response = {
      data: data,
      isLdapSyncing: setting.is_ldap_syncing,
      biometric: setting.biometrics_state,
      otp: clientSetting.otp_state
    };

    if (type) {
      //   const promises = (data || []).map(client => {
      //     return appendMFAStatus(client.id);
      //   });

      // let boundedValues;
      // if(type === 'Server'){
      //   const boundedPromises = (data || []).map(client => {
      //     return client.isBound();
      //   });
      //   boundedValues = await Promise.all(boundedPromises);
      // }

      //const promiseValues = await Promise.all(promises);
      response.data = (response.data || []).map((client, index) => {
        return {
          ...client.toJSON(),
          isMfaRegistered: 0,
          isServerBridgeBound:
            client.ServerBridges && client.ServerBridges.length ? true : false
        };
      });
    }
    response.recordsTotal = recordsTotal;
    res.json(response);
  } catch (error) {
    next(error);
  }
};

const updateClient = async (req, res, next) => {
  try {
    const { id } = req.params;
    const { client: updateData } = req.body;

    if (
      (!updateData.client_mfa && updateData.biometric_state) ||
      (!updateData.biometric_state && updateData.is_strong_mfa)
    ) {
      throw new httpErrors.UnprocessableEntity(
        'You cannot enable (Biometric MFA without MFA)/(Strong MFA without Biometric MFA).'
      );
    }

    const client = await models.Client.findOne({
      where: {
        id,
        client_type: {
          $like: `%${req.params.type}`
        }
      }
    });

    if (!client) {
      throw new httpErrors.NotFound('No client found.');
    }

    await client.update(updateData);

    let changedColumns = Object.keys(client._changed);
    changedColumns = changedColumns.filter(c =>
      clientColumnsToPublish.includes(c)
    );

    if (changedColumns.length) {
      const columns = {};
      changedColumns.forEach(column => {
        columns[column] = client[column];
      });

      await publishClients(actionType.UPDATED, {
        ids: [client.id],
        data: columns,
        type: client.client_type
      });
    }

    return res.json({
      success: true,
      message: language.get['EDIT_SUCCESS']
    });
  } catch (error) {
    next(error);
  }
};

/* Get a single client detail */
const getClientDetail = async (req, res, next) => {
  try {
    const options = {
      where: {
        id: req.params.id,
        client_type: {
          $like: `%${req.params.type}`
        }
      },
      include: [
        {
          attributes: ['device_name'],
          model: models.DeviceInventories
        },
        {
          model: models.ServerBridgeDeviceId,
          attributes: ['id']
        }
      ]
    };

    const data = await models.Client.findOne(options);

    if (!data) {
      throw new httpErrors.NotFound('Client not found.');
    }

    data.dataValues.device_name = data.DeviceInventory
      ? data.DeviceInventory.device_name
      : '';
    data.dataValues.isBridgeClient =
      data.dataValues.ServerBridgeDeviceIds &&
      data.dataValues.ServerBridgeDeviceIds.length &&
      data.dataValues.ServerBridgeDeviceIds.length > 0;
    data.isMfaDisabled = res.locals.isMfaDisabled;
    res.json(data);
  } catch (error) {
    next(error);
  }
};

const generateLinOTPToken = async (req, res, next) => {
  const clientObj = req.body.client;
  try {
    const client = await models.Client.findOne({
      attributes: [
        'id',
        'user',
        'agent_id',
        'name',
        'linOTP_token',
        'OTP_image_data',
        'client_type'
      ],
      where: {
        agent_id: clientObj.agent_id
      }
    });

    if (!client) {
      throw new httpErrors.NotFound('Client not found');
    }

    if (!client.OTP_image_data) {
      await client.generateOtp();

      let changedColumns = Object.keys(client._changed);
      changedColumns = changedColumns.filter(c =>
        clientColumnsToPublish.includes(c)
      );

      if (changedColumns.length) {
        const columns = {};
        changedColumns.forEach(column => {
          columns[column] = client[column];
        });

        await publishClients(actionType.UPDATED, {
          ids: [client.id],
          data: columns,
          type: client.client_type
        });
      }
    }

    res.json({
      OTP_image_data: client.OTP_image_data
    });
  } catch (error) {
    next(error);
  }
};

const generateOTPToken = async (req, res, next) => {
  const { device_id } = req.params;
  try {
    const client = await models.Client.findOne({
      attributes: [
        'id',
        'user',
        'agent_id',
        'name',
        'linOTP_token',
        'OTP_image_data',
        'client_type'
      ],
      where: {
        device_id
      }
    });

    if (!client) {
      throw new httpErrors.NotFound('Client not found');
    }

    if (!client.OTP_image_data) {
      await client.generateOtp();

      let changedColumns = Object.keys(client._changed);
      changedColumns = changedColumns.filter(c =>
        clientColumnsToPublish.includes(c)
      );

      if (changedColumns.length) {
        const columns = {};
        changedColumns.forEach(column => {
          columns[column] = client[column];
        });

        await publishClients(actionType.UPDATED, {
          ids: [client.id],
          data: columns,
          type: client.client_type
        });
      }
    }

    res.json({
      OTP_image_data: client.OTP_image_data
    });
  } catch (error) {
    next(error);
  }
};

//TODO : Code need to refactor
// add new client
const addClient = async (req, res, next) => {
  var client = req.body.client;
  client.access_restrictions = 0;
  var services = req.body.client.services;
  var clientServices = [];

  let _gips = req.body.gips;

  if (req.body.isClone) {
    _gips = await models.GlobalIps.findAll({
      attributes: [
        'parentType',
        'address',
        'port',
        'protocol',
        'label',
        'direction'
      ],
      where: {
        parentId: req.body.parentId
      },
      raw: true
    });
  }

  if (
    (!client.client_mfa && client.biometric_state) ||
    (!client.biometric_state && client.is_strong_mfa)
  ) {
    throw new httpErrors.UnprocessableEntity(
      'You cannot enable Biometric MFA without MFA or Strong MFA without Biometric MFA.'
    );
  }

  new Promise((resolve, reject) => {
    if (req.params.type === 'Server') {
      console.log('server');
      models.Client.findOne({
        where: {
          client_type: 'Client',
          user: client.user
        },
        attributes: ['id', 'user']
      }).then(client => {
        if (!client) {
          resolve();
        } else {
          const myError = new Error(
            'Server side client cannot be created as end user client with same name already exist.'
          );
          myError.name = 'ClientAlreadyExist';
          reject(myError);
        }
      });
    } else {
      resolve();
    }
  })
    .then(async () => {
      console.log('client');
      models.Client.count({
        where: {}
      }).then(async function(totalClients) {
        models.License.findOne({
          order: [['id', 'DESC']],
          limit: 1
        }).then(async function(license) {
          if (license.max_number_of_clients > totalClients) {
            const [encKey, hmacKey] = await Promise.all([
              OpensslObj.generateEncKey(),
              OpensslObj.generateHMacKey()
            ]);

            if (encKey.error) {
              return next(new httpErrors.BadRequest('Something went wrong.'));
              // return res.status(500).json({
              //     success: false,
              //     message: 'Something went wrong'
              // });
            }
            client.enc_key = encKey.key;

            if (hmacKey.error) {
              return next(new httpErrors.BadRequest('Something went wrong.'));
              // return res.status(500).json({
              //     success: false,
              //     message: 'Something went wrong'
              // });
            }
            client.HMacKey = hmacKey.key;
            client.agent_id = 0;
            client.counter = '1';
            return models.Client.create({
              ...client,
              client_type: req.params.type,
              package_used: false
            })
              .then(_client => {
                return Promise.all([
                  _client,
                  publishMsgFile(actionType.ADDED, {
                    id: _client.id,
                    data: {
                      enc_key: _client.enc_key,
                      ENC_KEY_LEN: _client.ENC_KEY_LEN,
                      HMacKey: _client.HMacKey,
                      HMAC_KEY_LEN: _client.HMAC_KEY_LEN,
                      device_hardware_id: _client.device_hardware_id
                    },
                    type: _client.client_type
                  })
                ]);
              })
              .then(function([data]) {
                console.log('client created in backend');
                let gips = _gips.map(gip => ({
                  ...gip,
                  clientId: data.id,
                  parentId: data.id,
                  parentType: 'Client'
                }));
                data.agent_id = data.id;
                data.save().then(function() {
                  models.GlobalIps.bulkCreate(gips).then(function(data) {
                    res.json({
                      success: true,
                      message: language.get['ADD_SUCCESS']
                    });
                  });
                });
              });
          } else {
            res.json({
              success: false,
              message:
                'Limit has reached, You are not allowed to add more than ' +
                license.max_number_of_clients +
                ' clients.'
            });
          }
        });
      });
    })
    .catch(error => {
      if (error.name === 'ClientAlreadyExist') {
        return next(new httpErrors.Conflict(error.message));
        // return res.status(500).send(error.message);
      }
      return next(
        new httpErrors.BadRequest('Something went wrong in creating client.')
      );
      // return res.status(500).send('Something went wrong in creating client');
    });
};

// delete a single user
const deleteClient = async (req, res, next) => {
  try {
    const { id, type, user } = req.params;

    await deleteSingleClient(id, user, type);

    await publishClients(actionType.DELETED, {
      ids: [id],
      type: type,
      data: {}
    });

    res.json({
      success: true,
      message: 'Delete Successfully'
    });
  } catch (error) {
    next(error);
  }
};

/**
 * Perform actions based on type
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
const bulkActions = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const {
      type,
      selectAll,
      selectedIds,
      client_type,
      filters,
      status,
      search = ''
    } = req.body;

    const parsedFilters = JSON.parse(filters);

    const options = {
      where: {
        [Op.and]: [
          {
            client_type: client_type
          }
        ]
      }
    };

    if (!selectAll && selectedIds) {
      options.where[Op.and].push({
        id: {
          [Op.in]: selectedIds
        }
      });
    } else {
      if (selectAll) {
        options.where[Op.and].push({
          id: {
            [Op.notIn]: selectedIds
          }
        });
      }
      if (search.trim().length) {
        let name = `%${search.trim()}%`;
        options.where[Op.and].push({
          [Op.or]: [
            {
              Name: {
                [Op.like]: name
              }
            },
            {
              user: {
                [Op.like]: name
              }
            },
            {
              agent_id: {
                [Op.like]: name
              }
            }
          ]
        });
      }

      if (parsedFilters.status.length) {
        options.where[Op.and].push({
          client_status: {
            [Op.in]: parsedFilters.status
          }
        });
      }

      if (parsedFilters.os.length) {
        options.where[Op.and].push({
          device_os: {
            [Op.in]: parsedFilters.os
          }
        });
      }

      if (parsedFilters.quickFilters.localOrphand) {
        options.where[Op.and].push({
          is_ldap_deleted: true
        });
      }

      if (parsedFilters.quickFilters.adOrphand) {
        options.where[Op.and].push({
          is_local_deleted: true
        });
      }

      if (
        parsedFilters.quickFilters.isMfa &&
        parsedFilters.quickFilters.isMfa.trim().length
      ) {
        const isMfaArray = parsedFilters.quickFilters.isMfa.split(',');
        options.where[Op.and].push({
          client_mfa: {
            [Op.in]: isMfaArray
          }
        });
      }

      if (
        parsedFilters.quickFilters.isDarkened &&
        parsedFilters.quickFilters.isDarkened.trim().length
      ) {
        const darkenedArray = parsedFilters.quickFilters.isDarkened.split(',');
        options.where[Op.and].push({
          client_darkening: {
            [Op.in]: darkenedArray
          }
        });
      }

      if (
        parsedFilters.quickFilters.is_active &&
        parsedFilters.quickFilters.is_active.trim().length &&
        parsedFilters.quickFilters.is_active != -1
      ) {
        const activeArray = parsedFilters.quickFilters.is_active.split(',');
        options.where[Op.and].push({
          is_active: {
            [Op.in]: activeArray
          }
        });
      }

      let scope = '';
      let scopeValue = false;
      if (
        parsedFilters.quickFilters.isOnline &&
        parsedFilters.quickFilters.isOnline.trim().length
      ) {
        const isOnlineArray = parsedFilters.quickFilters.isOnline.split(',');
        if (isOnlineArray.length !== 2) {
          scope = 'online';
          scopeValue = isOnlineArray[0] === '0' ? false : true;
        }
      }

      if (scope) {
        const isOnline = scopeValue;
        options.where[Op.and].push({
          [isOnline ? Op.and : Op.or]: [
            {
              heart_beat: isOnline
                ? 1
                : {
                    [Op.in]: [0]
                  }
            },
            where(
              fn(
                'TIMESTAMPDIFF',
                literal('SECOND'),
                fn('DATE_SUB', fn('NOW'), literal('INTERVAL 5 MINUTE')),
                col('heart_beat_sent_at')
              ),
              {
                [isOnline ? Op.gte : Op.lte]: 2
              }
            )
          ]
        });
      }
    }

    if (type == 'sendOtpQrBulkClients') {
      const userOptions = {
        ...options,
        attributes: [
          'Email',
          'Name',
          'user',
          [
            models.sequelize.fn('max', models.sequelize.col('linOTP_token')),
            'linOTP_token'
          ],
          [
            models.sequelize.fn('max', models.sequelize.col('OTP_image_data')),
            'OTP_image_data'
          ],
          [models.sequelize.fn('max', models.sequelize.col('id')), 'id']
        ],
        group: ['Email', 'Name', 'user']
      };

      let users = await models.Client.findAll(userOptions);

      users = await Promise.all(
        users.map(user => {
          if (user.OTP_image_data) return user;
          user.skip_otp_check = true;
          return user.generateOtp();
        })
      );

      sendOtpQrBulkUsers(users);

      return res
        .status(200)
        .send({ message: 'Client(s) will receive OTP Code within 2 minutes.' });
    } else if (type === 'deleteClients') {
      let ids = [];
      parsedFilters.quickFilters.is_active == -1
        ? delete parsedFilters.quickFilters.is_active
        : '';
      if (
        parsedFilters.os.length ||
        parsedFilters.status.length ||
        Object.keys(parsedFilters.quickFilters).length
      ) {
        const _clientsToDelete = await models.Client.findAll({
          ...options,
          attributes: ['id'],
          raw: true
        });

        ids = _clientsToDelete.map(c => c.id);
        if (ids && ids.length > 0) {
          await publishClients(actionType.DELETED, {
            ids,
            mode: 0,
            type: client_type,
            data: {}
          });
        }
      } else {
        ids = selectedIds;
        if (ids && ids.length > 0) {
          await publishClients(actionType.DELETED, {
            ids,
            mode: selectAll ? 1 : 0,
            type: client_type,
            data: {}
          });
        }
      }

      await Promise.all([
        models.sequelize.query(
          `DELETE FROM DevicePolicies WHERE DevicePolicies.id IN (SELECT ClientPolicies.policy_id AS id
                    FROM
                        ClientPolicies
                    WHERE
                        ClientPolicies.policy_type_id = 2 AND
                        ClientPolicies.policy_id IN (SELECT ClientPolicies.policy_id FROM ClientPolicies WHERE ClientPolicies.policy_type_id = 2 AND ClientPolicies.client_id IN (${selectedIds.toString() ||
                          null}))
                    Group by
                        ClientPolicies.policy_id
                    HAVING
                        count(ClientPolicies.id) < 2)`,
          { transaction }
        ),
        models.sequelize.query(
          `DELETE FROM Entitlements WHERE Entitlements.id IN (SELECT ClientPolicies.policy_id AS id
                    FROM
                        ClientPolicies
                    WHERE
                        ClientPolicies.policy_type_id = 1 AND
                        ClientPolicies.policy_id IN (SELECT ClientPolicies.policy_id FROM ClientPolicies WHERE ClientPolicies.policy_type_id = 1 AND ClientPolicies.client_id IN (${selectedIds.toString() ||
                          null}))
                    Group by
                        ClientPolicies.policy_id
                    HAVING
                        count(ClientPolicies.id) < 2)`,
          { transaction }
        ),
        models.Client.destroy({ ...options, transaction })
      ]);
      await transaction.commit();

      return res
        .status(200)
        .send({ message: 'Client(s) deleted successfully.' });
    }

    let values = {
      ...(type == 'changeStatusBulkClients'
        ? {
            is_active: status
          }
        : type == 'changeDarkeningStatusBulkClients'
        ? {
            client_darkening: status
          }
        : type == 'changeMfaStatusBulkClients'
        ? {
            client_mfa: status,
            ...(!status
              ? {
                  biometric_state: false,
                  is_strong_mfa: false
                }
              : {})
          }
        : type == 'changeBiometricMfaStatusBulkClients'
        ? {
            biometric_state: status,
            ...(status
              ? {
                  client_mfa: status
                }
              : {}),
            ...(!status
              ? {
                  is_strong_mfa: false
                }
              : {})
          }
        : type == 'changeStrongMfaStatusBulkClients'
        ? {
            is_strong_mfa: status,
            ...(status
              ? {
                  client_mfa: status,
                  biometric_state: status
                }
              : {})
          }
        : type == 'forceOnboard'
        ? {
            onboarding_required: status
          }
        : {})
    };

    const check = await models.Client.update(values, options);

    // console.log(check);

    parsedFilters.quickFilters.is_active == -1
      ? delete parsedFilters.quickFilters.is_active
      : '';

    if (
      parsedFilters.os.length ||
      parsedFilters.status.length ||
      Object.keys(parsedFilters.quickFilters).length
    ) {
      const _clientsToUpdate = await models.Client.findAll({
        ...options,
        attributes: ['id'],
        raw: true
      });

      const ids = _clientsToUpdate.map(c => c.id);

      if (type == 'forceOnboard') {
        console.log('pubsub workinggggg');
        await publishForceOnboard(actionType.UPDATED, {
          ids
        });
      }

      // console.log(ids);

      let changedColumns = Object.keys(values);
      changedColumns = changedColumns.filter(c =>
        clientColumnsToPublish.includes(c)
      );

      if (changedColumns.length) {
        const columns = {};
        changedColumns.forEach(column => {
          columns[column] = status;
        });

        await publishClients(actionType.UPDATED, {
          ids,
          mode: 0,
          type: client_type,
          data: {
            ...columns,
            updatedAt: new Date().toISOString().slice(0, 19)
          }
        });
      }
    } else {
      const ids = selectedIds;

      if (type == 'forceOnboard') {
        console.log('pubsub else workinggg');
        await publishForceOnboard(actionType.UPDATED, {
          ids
        });
      }

      let changedColumns = Object.keys(values);
      changedColumns = changedColumns.filter(c =>
        clientColumnsToPublish.includes(c)
      );

      if (changedColumns.length) {
        const columns = {};
        changedColumns.forEach(column => {
          columns[column] = status;
        });

        await publishClients(actionType.UPDATED, {
          ids,
          mode: selectAll ? 1 : 0,
          type: client_type,
          data: {
            ...columns,
            updatedAt: new Date().toISOString().slice(0, 19)
          }
        });
      }
    }

    const userOptions = {
      ...options,
      attributes: ['Name']
    };

    let clients = await models.Client.findAll(userOptions);
    let logMessage;

    for (const client of clients) {
      if (type == 'changeStatusBulkClients' && status == 1) {
        logMessage = getClientAllowMessage(client.Name);
      } else if (type == 'changeStatusBulkClients' && status == 0) {
        logMessage = getClientBlockMessage(client.Name);
      } else if (type == 'forceOnboard') {
        logMessage = getClientForceOnBoardMessage(client.Name);
      } else {
        logMessage = getClientUpdateMessage(client.Name);
      }
      insertIntoAdminLogs(logMessage, req);
    }
    return res.status(200).send({ message: 'Client(s) updated successfully.' });
  } catch (error) {
    await transaction.rollback();
    return res.status(500).send('Something went wrong.');
  }
};

const appendMFAStatus = clientId => {
  const url = `${global.MFA_URL}/v1/public/userExists/${clientId}`;
  return axios
    .get(url, {
      httpsAgent: httpsAgent
    })
    .then(response => {
      const status = response.data.status;
      return Promise.resolve(status ? true : false);
    })
    .catch(err => {
      // console.log(err && (err.message || err));
      return Promise.resolve(false);
    });
};

let prepareEmailOptions = (
  recipient,
  emailSubject,
  templateFileName,
  onPremiseDownloadLink,
  offPremiseDownloadLink,
  recipientName,
  linkExpiryInMinutes,
  component,
  onPremDeviceIdLink,
  offPremDeviceIdLink,
  onPremWebMobileLink = '',
  offPremWebMobileLink = ''
) => {
  let log = {
    recipient: recipient,
    emailSubject: emailSubject,
    templateFileName: templateFileName,
    onPremiseDownloadLink: onPremiseDownloadLink,
    offPremiseDownloadLink: offPremiseDownloadLink,
    onPremDeviceIdLink: onPremDeviceIdLink,
    offPremDeviceIdLink: offPremDeviceIdLink,
    recipientName: recipientName,
    linkExpiryInMinutes: linkExpiryInMinutes,
    component: component,
    onPremApiUrl: `${projectConfig.onPremApiUrl}`,
    onPremWebMobileUrl: onPremWebMobileLink,
    offPremWebMobileUrl: offPremWebMobileLink
  };
  console.log(log);
  return log;
};

function getAPIUrlForDeviceOnPrem(dkey, controller) {
  return (
    `${controller.onboarding_url}:${controller.onboarding_port}/api/deviceid/add/serve/` +
    encodeURI(dkey)
  );
}

function getAPIUrlForDeviceOffPrem(dkey) {
  return `${projectConfig.offPremApiUrl}/deviceid/add/serve/` + encodeURI(dkey);
}

function prepareOnPremiseDownloadLinkForJSON(
  controller,
  clientOs,
  key,
  dkey = null,
  deviceId = null,
  spa_port,
  deviceSerialNumber
) {
  let emailLink;
  const md5deviceId = crypto
    .createHash('md5')
    .update(deviceId)
    .digest('hex');
  if (clientOs === installerTypeMeta['android']) {
    emailLink = `http://www.invisily.com?&api=${controller.onboarding_url}:${
      controller.onboarding_port
    }&deviceId=${encodeURI(md5deviceId)}&spa_port=${encodeURI(
      spa_port
    )}&deviceSerialNumber=${encodeURI(deviceSerialNumber)}`;
  } else if (clientOs === installerTypeMeta['ios']) {
    emailLink =
      `${process.env.CONTENT_URL}/api/v1/links/redirect?url=` +
      encodeURIComponent(
        `sdpclient://?api=${controller.onboarding_url}:${
          controller.onboarding_port
        }&deviceId=${encodeURI(md5deviceId)}&spa_port=${encodeURI(
          spa_port
        )}&deviceSerialNumber=${encodeURI(deviceSerialNumber)}`
      );
  }

  return emailLink;
}

function prepareOnPremiseDownloadLink(
  controller,
  clientOs,
  key,
  dkey = null,
  deviceId = null,
  spa_port = null,
  deviceSerialNumber = null
) {
  let emailLink;
  if (clientOs === installerTypeMeta['android']) {
    const md5deviceId = crypto
      .createHash('md5')
      .update(deviceId)
      .digest('hex');
    emailLink = `http://www.invisily.com?&api=${controller.onboarding_url}:${
      controller.onboarding_port
    }&deviceId=${encodeURI(md5deviceId)}&spa_port=${encodeURI(
      spa_port
    )}&deviceSerialNumber=${encodeURI(deviceSerialNumber)}`;
  } else if (clientOs === installerTypeMeta['ios']) {
    const md5deviceId = crypto
      .createHash('md5')
      .update(deviceId)
      .digest('hex');
    emailLink =
      `${process.env.CONTENT_URL}/api/v1/links/redirect?url=` +
      encodeURIComponent(
        `sdpclient://?api=${controller.onboarding_url}:${
          controller.onboarding_port
        }&deviceId=${encodeURI(md5deviceId)}&spa_port=${encodeURI(
          spa_port
        )}&deviceSerialNumber=${encodeURI(deviceSerialNumber)}`
      );
  } else {
    emailLink = `${process.env.CONTENT_URL}/api/v1/links/download/` + key;
    console.log('------------------emailLink---------------------', emailLink);
  }
  return emailLink;
}

function prepareOnPremiseDeviceIdLink(clientOs, key) {
  return `${projectConfig.onPremApiUrl}/deviceid/add/` + key;
}

function prepareOffPremiseDeviceIdLink(clientOs, key) {
  return `${projectConfig.offPremApiUrl}/deviceid/add/` + key;
}

let generateCfgFile = (client, controller, gateway, settings) => {
  let config = {
    agent_ID: client.agent_id,
    hmac_key_len: 24,
    henc_key_len: 32,
    ctrlspaPort: controller.spa_port,
    gatespaPort: gateway.spa_port,
    ctrlPort: controller.port,
    ctrlPubPort: controller.public_ipv4_port,
    clientDeviceId: client.serial_number
    /*,
          mfa: mfa,
          mfa_mode_otp: otp,
          mfa_mode_biometrics: biometrics*/
  };
  if (client.counter === null) {
    config.counter = 1;
  } else {
    config.counter = client.counter;
  }

  if (isMobileOS(client)) {
    config.ctrlClientCertFilePath = 'certs/client.cer';
    config.ctrlClientKeyPemFilePath = 'certs/client.p12';
    config.ctrlClientRootCertFilePath = 'certs/cacert.cer';
    config.henc_key = client.enc_key.split('=')[0];
    config.henc_key_iv = client.enc_key.split('=')[1];
    config.hmac_key = client.HMacKey.replace('\n', '').trim();
    config.ctrlIPAddress = controller.ipv4_address;
    config.ctrlPubIPAddress = controller.public_ipv4_address;
    config.ctrlPemPassword = projectConfig.pemPassword;
    config.ctrlClientCAlocation = 'certs';
  } else if (isMac(client)) {
    config.ctrlClientCertFilePath =
      '"' +
      '/Applications/Invisily.app/Contents/Resources/certs/client.crt' +
      '"';
    config.ctrlClientKeyPemFilePath =
      '"' +
      '/Applications/Invisily.app/Contents/Resources/certs/client.pem' +
      '"';
    config.ctrlClientRootCertFilePath =
      '"' +
      '/Applications/Invisily.app/Contents/Resources/certs/cacert.pem' +
      '"';
    config.henc_key = '"' + client.enc_key.split('=')[0] + '"';
    config.henc_key_iv = '"' + client.enc_key.split('=')[1] + '"';
    config.hmac_key = '"' + client.HMacKey.replace('\n', '').trim() + '"';
    config.ctrlIPAddress = '"' + controller.ipv4_address + '"';
    config.ctrlPubIPAddress = '"' + controller.public_ipv4_address + '"';
    config.ctrlPemPassword = '"' + projectConfig.pemPassword + '"';
    config.ctrlClientCAlocation =
      '"' + '/Applications/Invisily.app/Contents/Resources/certs' + '"';
    config.auth_password = '"' + 'dummy' + '"';
    config.auth_userName = '"' + 'dummy' + '"';
    config.gwVTI_devName = '"' + 'invi' + '"';
    config.application_mode = '"' + 'gui' + '"';
    config.on_premises = 1;
    config.clientlocaldomain = '"' + projectConfig.clientLocalDomain + '"';
  } else {
    config.ctrlClientCertFilePath = '"' + 'certs/client.crt' + '"';
    config.ctrlClientKeyPemFilePath = '"' + 'certs/client.pem' + '"';
    config.ctrlClientRootCertFilePath = '"' + 'certs/cacert.pem' + '"';
    config.henc_key = '"' + client.enc_key.split('=')[0] + '"';
    config.henc_key_iv = '"' + client.enc_key.split('=')[1] + '"';
    config.hmac_key = '"' + client.HMacKey.replace('\n', '').trim() + '"';
    config.ctrlIPAddress = '"' + controller.ipv4_address + '"';
    config.ctrlPubIPAddress = '"' + controller.public_ipv4_address + '"';
    config.ctrlPemPassword = '"' + projectConfig.pemPassword + '"';
    config.ctrlClientCAlocation = '"' + 'certs' + '"';
    config.auth_password = '"' + 'dummy' + '"';
    config.auth_userName = '"' + 'dummy' + '"';
    config.gwVTI_devName = '"' + 'invtun' + '"';
    config.application_mode = '"' + 'gui' + '"';
    config.clientlocaldomain = '"' + projectConfig.clientLocalDomain + '"';
    config.on_premises = 1;
  }

  if (client.client_darkening === false) {
    config.client_darkening = 0;
  } else {
    config.client_darkening = 1;
  }
  config.otpPort = controller.onboarding_port;
  config.mfaPort = controller.mfa_server_port;

  if (isMobileOS(client)) {
    config.invisily_url = `${controller.onboarding_url}:${controller.onboarding_port}/api`;
  } else {
    config.invisily_url =
      '"' +
      `${controller.onboarding_url}:${controller.onboarding_port}/api` +
      '"';
  }

  if (isMobileOS(client)) {
    let JsonStr = JSON.stringify(config, null, 2);
    return JsonStr.replace(/"counter": "(\d+)"/g, '"counter": $1');
  } else {
    return Object.keys(config).reduce(
      (acc, key) => `${acc}${key}=${config[key]}\n`,
      ''
    );
  }
};

let writeCfgFile = (client, installerDir, resolve, reject) => {
  models.Settings.findOne().then(function(settings) {
    models.Gateway.findOne({
      where: {}
    }).then(function(gateway) {
      if (gateway) {
        console.log('Found gateway record in DB: ' + gateway.Name);

        models.Controller.findOne({
          where: {
            roleId: 1
          }
        }).then(function(controller) {
          if (controller) {
            console.log('Found controller record in DB: ' + controller.Name);
            try {
              fs.writeFile(
                `${installerDir}/settings.cfg`,
                generateCfgFile(client, controller, gateway, settings),
                function(err) {
                  if (err) {
                    reject(err);
                  } else {
                    resolve();
                  }
                }
              );
            } catch (err) {
              console.log(language.get['LOG_DIVIDER']);
              console.log(err && (err.message || err));
              reject(err);
              console.log(language.get['LOG_DIVIDER']);
            }
          } else {
            console.log(language.get['NO_CONTROLLER_FOUND']);
            reject();
          }
        });
      } else {
        console.log(language.get['NO_GATEWAY_FOUND']);
        reject();
      }
    });
  });
};

let zipInstaller = (
  client,
  installerDir,
  packageDir,
  type,
  installerType,
  resolve,
  reject
) => {
  let output = '';
  if (
    installerType === installerTypeMeta['ubuntu'] ||
    installerType === installerTypeMeta['ubuntu18']
  ) {
    output = fs.createWriteStream(
      `${packageDir}/invisily-client-` +
        installerType +
        `-` +
        client.id +
        `.zip`
    );
  } else if (
    installerType === installerTypeMeta['ubuntuserver'] ||
    installerType === installerTypeMeta['ubuntuserver18'] ||
    installerType === installerTypeMeta['centosserver']
  ) {
    output = fs.createWriteStream(
      `${packageDir}/invisily-clientd-` +
        installerType +
        `-` +
        client.id +
        `.zip`
    );
  } else {
    output = fs.createWriteStream(
      `${packageDir}/invisily-client-` +
        installerType +
        `-` +
        client.id +
        `.zip`
    );
  }
  let archive = archiver('zip', {
    zlib: {
      level: 9
    } // Sets the compression level.
  });
  output.on('close', function() {
    console.log(language.get['ZIP_GENERATED']);
    log[client.id] += language.get['LOG_ZIP_FILE_GENERATED'];
    client.installer_logs = log[client.id];
    client.save();
    resolve();
  });
  output.on('error', function(err) {
    console.log(language.get['ZIP_NOT_GENERATED']);
    console.log(language.get['LOG_DIVIDER']);
    console.log(err && (err.message || err));
    console.log(language.get['LOG_DIVIDER']);
    log[client.id] += language.get['LOG_ZIP_FILE_NOT_GENERATED'];
    client.installer_logs = log[client.id];
    client.save();
    resolve();
  });
  archive.on('error', function(err) {
    log[client.id] += language.get['LOG_ZIP_ERROR'];
    client.installer_logs = log[client.id];
    client.save();
    reject(err);
  });
  // pipe archive data to the file
  archive.pipe(output);

  if (installerType === installerTypeMeta['ubuntu']) {
    if (isPackageWithInstallerPackage(installerType)) {
      log[client.id] += language.get['ADDING_NEW_INSTALLER_ZIP'];
      client.installer_logs = log[client.id];
      client.save();
      archive.append(
        fs.createReadStream(`${installerDir}/invisily_client.${type}`),
        {
          name: `invisily_client.${type}`
        }
      );
    }
  } else if (installerType === installerTypeMeta['ubuntuserver']) {
    if (isPackageWithInstallerPackage(installerType)) {
      log[client.id] += language.get['ADDING_NEW_INSTALLER_ZIP'];
      client.installer_logs = log[client.id];
      client.save();
      archive.append(
        fs.createReadStream(`${installerDir}/invisily_client.${type}`),
        {
          name: `invisily_client.${type}`
        }
      );
    }
  } else if (installerType === installerTypeMeta['ubuntu18']) {
    if (isPackageWithInstallerPackage(installerType)) {
      log[client.id] += language.get['ADDING_NEW_INSTALLER_ZIP'];
      client.installer_logs = log[client.id];
      client.save();
      archive.append(
        fs.createReadStream(`${installerDir}/invisily_client.${type}`),
        {
          name: `invisily_client.${type}`
        }
      );
    }
  } else if (
    installerType === installerTypeMeta['ubuntuserver18'] ||
    installerType === installerTypeMeta['centosserver']
  ) {
    if (isPackageWithInstallerPackage(installerType)) {
      log[client.id] += language.get['ADDING_NEW_INSTALLER_ZIP'];
      client.installer_logs = log[client.id];
      client.save();
      archive.append(
        fs.createReadStream(`${installerDir}/invisily_client.${type}`),
        {
          name: `invisily_client.${type}`
        }
      );
    }
  } else if (installerType === installerTypeMeta['macintosh']) {
    if (isPackageWithInstallerPackage(installerType)) {
      log[client.id] += language.get['ADDING_NEW_INSTALLER_ZIP'];
      client.installer_logs = log[client.id];
      client.save();
      archive.append(
        fs.createReadStream(`${installerDir}/Invisily-client.${type}`),
        {
          name: `Invisily-client.${type}`
        }
      );
    }
  } else {
    log[client.id] += language.get['LOG_ADD_SETTING_TO_ZIP'];
    client.installer_logs = log[client.id];
    client.save();
    if (fs.existsSync(`${installerDir}/settings.cfg`)) {
      try {
        archive.append(fs.createReadStream(`${installerDir}/settings.cfg`), {
          name: 'settings.cfg'
        });
      } catch (err) {
        console.log(language.get['LOG_DIVIDER']);
        console.log(err && (err.message || err));
        reject(err);
        console.log(language.get['LOG_DIVIDER']);
      }
      if (isPackageWithInstallerPackage(installerType)) {
        log[client.id] += ',Adding installer in zip.';
        client.installer_logs = log[client.id];
        client.save();
        archive.append(
          fs.createReadStream(`${installerDir}/installer.${type}`),
          {
            name: `installer.${type}`
          }
        );
      }
    } else {
      console.log(language.get['SETTING_PATH_NOT_AVALIBLE']);
      log[client.id] += language.get['LOG_SETTING_PATH_NOT_AVALIBLE'];
      client.installer_logs = log[client.id];
      client.save();
      reject();
    }
  }

  if (installerType === installerTypeMeta['ubuntu']) {
    log[client.id] += language.get['UBUNTU_16_INTO_ZIP'];
    client.installer_logs = log[client.id];
    client.save();
    archive.append(
      fs.createReadStream(`${installerDir}/Installation_Guide_Client.txt`),
      {
        name: `Installation_Guide_Client.txt`
      }
    );
  } else if (installerType === installerTypeMeta['ubuntuserver']) {
    log[client.id] += language.get['LOG_UBUNTU_16_DEAMON_INTO_ZIP'];
    client.installer_logs = log[client.id];
    client.save();
    archive.append(
      fs.createReadStream(
        `${installerDir}/Installation_Guide_Client_Deamon.txt`
      ),
      {
        name: `Installation_Guide_Client_Deamon.txt`
      }
    );
  } else if (installerType === installerTypeMeta['ubuntu18']) {
    log[client.id] += ',Adding Guide for Ubuntu 18 into zip.';
    client.installer_logs = log[client.id];
    client.save();
    archive.append(
      fs.createReadStream(`${installerDir}/Installation_Guide_Client.txt`),
      {
        name: `Installation_Guide_Client.txt`
      }
    );
  } else if (installerType === installerTypeMeta['ubuntuserver18']) {
    log[client.id] += ',Adding Guide for Ubuntu 18 Deamon into zip.';
    client.installer_logs = log[client.id];
    client.save();
    archive.append(
      fs.createReadStream(
        `${installerDir}/Installation_Guide_Client_Deamon.txt`
      ),
      {
        name: `Installation_Guide_Client_Deamon.txt`
      }
    );
  } else if (installerType === installerTypeMeta['centosserver']) {
    log[client.id] += ',Adding Guide for CentOS Server into zip.';
    client.installer_logs = log[client.id];
    client.save();
    archive.append(
      fs.createReadStream(
        `${installerDir}/Installation_Guide_Client_CentOS_Deamon.txt`
      ),
      {
        name: `Installation_Guide_Client_CentOS_Deamon.txt`
      }
    );
  } else if (installerType === installerTypeMeta['macintosh']) {
    log[client.id] += ',Adding Guide for Macintosh into zip.';
    client.installer_logs = log[client.id];
    client.save();
    archive.append(
      fs.createReadStream(`${installerDir}/Installation_Guide_Client_Mac.txt`),
      {
        name: `Installation_Guide_Client_Mac.txt`
      }
    );
  } else if (installerType === installerTypeMeta['windows']) {
    log[client.id] += ',Adding Guide for Windows into zip.';
    client.installer_logs = log[client.id];
    client.save();
    archive.append(
      fs.createReadStream(`${installerDir}/Installation_Guide_Client.txt`),
      {
        name: `Installation_Guide_Client.txt`
      }
    );
  }

  if (
    installerType !== installerTypeMeta['ubuntu'] &&
    installerType !== installerTypeMeta['macintosh'] &&
    installerType !== installerTypeMeta['ubuntuserver'] &&
    installerType !== installerTypeMeta['ubuntu18'] &&
    installerType !== installerTypeMeta['ubuntuserver18'] &&
    installerType !== installerTypeMeta['centosserver']
  ) {
    log[client.id] += ',Adding Certificates into zip.';
    client.installer_logs = log[client.id];
    client.save();
    archive.directory(`${installerDir}/certs`, 'certs');
  }
  archive.finalize();
};

function getClientCancelStatus(client) {
  return new Promise((resolve, reject) => {
    models.Client.findOne({
      where: {
        id: client.id
      }
    })
      .then(client => {
        console.log('sa ' + client.isCancelled);
        if (!client.isCancelled) {
          return resolve(client);
        }
        reject(new PackageCancelError());
      })
      .catch(() => {
        reject(new PackageCancelError());
      });
  });
}

const withClientLogs = client => {
  client.installer_logs = '';

  return (task, log) => {
    return task.then(() => {
      client.installer_logs += `${log}\n\n`;
      return client.save();
    });
  };
};

const generateZipFile = (parentPath, dirName) => {
  return new Promise((resolve, reject) => {
    const archive = archiver('zip', {
      zlib: {
        level: 9
      }
    });
    const zipPath = `${parentPath}/${dirName}.zip`;

    const zipStream = fs.createWriteStream(zipPath);

    zipStream.on('close', function() {
      resolve(zipPath);
    });
    zipStream.on('error', function(err) {
      reject(err);
    });

    archive.pipe(zipStream);
    archive.glob(`**/*`, {
      cwd: `${parentPath}/${dirName}`
    });
    archive.finalize();
  });
};

/**
 * @description function to generate a zip package and shoot an email
 * */
async function generateZip(client, packageState) {
  setExpiryTime()
    .then(async function() {
      return new Promise(async (resolve, reject) => {
        if (!client.enc_key || !client.HMacKey) {
          const [encKey, hmacKey] = await Promise.all([
            OpensslObj.generateEncKey(),
            OpensslObj.generateHMacKey()
          ]);

          if (encKey.error) {
            return reject();
          }

          client.enc_key = encKey.key;

          if (hmacKey.error) {
            return reject();
          }

          client.HMacKey = hmacKey.key;

          await client.save();

          await publishClients(actionType.UPDATED, {
            id: client.id,
            data: {
              enc_key: encKey,
              ENC_KEY_LEN: client.ENC_KEY_LEN,
              HMacKey: hmacKey,
              HMAC_KEY_LEN: client.HMAC_KEY_LEN,
              device_hardware_id: client.device_hardware_id
            },
            type: client.client_type
          });

          resolve();
        } else {
          resolve();
        }
      });
    })
    .then(function() {
      client.packageDownloaded = false;
      if (packageState === '1') {
        client.isDownloadPackage = true;
        client.save();
      } else {
        client.isDownloadPackage = false;
        client.save();
      }
      client.link_key_expiry = moment();
      client.save();

      log[client.id] = '';
      if (
        client.cacert != null &&
        client.clientcrt != null &&
        client.clientpem != null &&
        client.cacer != null &&
        client.clientcer != null &&
        client.clientp12 != null
      ) {
        if (
          client.cacert.length > 20 &&
          client.clientcrt.length > 20 &&
          client.clientpem.length > 20 &&
          client.cacer.length > 20 &&
          client.clientcer.length > 20 &&
          client.clientp12.length > 20
        ) {
          var time = moment()
            .utc()
            .local()
            .format('YYYY-MM-DD HH:mm:ss');
          client.updatedAt = time;
        }
      }
      log[client.id] +=
        'Logs Starting at ' +
        moment()
          .utc()
          .local()
          .format('YYYY-MM-DD HH:mm:ss');
      let dirName =
        '/temp' +
        client.agent_id +
        '-' +
        moment()
          .utc()
          .local()
          .format('YYYY-MM-DD-HH-mm-ss') +
        '-' +
        Math.random()
          .toString(36)
          .substring(2);
      let packageDir = `${projectConfig.serviceInstallerPath}/client/output/${client.device_os}`;
      let key = `${client.updatedAt.getTime()}:${client.device_os}`;
      let installerDir =
        `${projectConfig.serviceInstallerPath}/client/input/${client.device_os}` +
        dirName;
      let certDirectory = `${installerDir}/certs`;
      let installerFile = `${projectConfig.serviceInstallerPath}/client/input/${
        client.device_os
      }/installer.${fileTypeMeta[client.device_os]}`;
      let guidePath = `${projectConfig.serviceInstallerPath}/client/input/${client.device_os}/*.txt`;
      let fkey = '';

      log[client.id] += ',Directory name is ' + dirName;
      log[client.id] += ',Key is ' + key;
      log[client.id] += ',Certificate Directory is ' + certDirectory;
      log[client.id] += ',Installer File is ' + installerFile;
      client.installer_logs = log[client.id];
      client.save();

      console.log('dirName ...' + dirName);
      console.log('installerDir ...' + installerDir);
      console.log('packageDir ...' + packageDir);
      console.log('key ...' + key);
      console.log('certDirectory ...' + certDirectory);

      // if something is wrong with the directory throw exception
      Promise.all([
        helper.mkDir(certDirectory),
        helper.mkDir(packageDir),
        helper.mkDir(installerDir)
      ])
        .then(() => {
          return new Promise((resolve, reject) => {
            log[client.id] += ',Process Started!';
            client.installer_status = 'processing';
            client.package_sent_at = moment().add(
              emailLinkExpiryInMinutes,
              'minutes'
            );
            client.isDeviceIdSet = false;
            client.installer_logs = log[client.id];
            client.save();
            resolve();
          });
        })
        .then(
          () =>
            new Promise((resolve, reject) => {
              log[client.id] += ',Certificate Directory Created!';
              log[client.id] += ',Package Directory Created!';
              log[client.id] += ',Installer Directory Created!';
              client.installer_logs = log[client.id];
              client.save();
              if (
                client.device_os !== 'android' &&
                client.device_os !== 'ios'
              ) {
                exec(
                  `${projectConfig.serviceInstallerPath}/give-write-permission.sh ${installerDir} ${installerFile}`,
                  (error, stdout, stderr) => {
                    console.log('Executing give-write-permission.sh');
                    console.log(language.get['STD_OUT'] + stdout);
                    if (error) {
                      log[client.id] +=
                        ',Process Stoped at give-write-permission script.';
                      client.installer_logs = log[client.id];
                      client.save();
                      reject(error);
                    } else {
                      log[client.id] +=
                        ',give-write-permission script executed Successfully';
                      client.installer_logs = log[client.id];
                      client.save();
                      resolve();
                    }
                  }
                );
              } else {
                resolve();
              }
            })
        )
        .then(
          () =>
            new Promise((resolve, reject) => {
              console.log('Writing Guide');
              log[client.id] += ',Copying guide to the package directory.';
              client.installer_logs = log[client.id];
              client.save();
              if (
                client.device_os !== 'android' &&
                client.device_os !== 'ios'
              ) {
                exec(
                  `${projectConfig.serviceInstallerPath}/copy-file-to-path.sh ${installerDir} ${guidePath}`,
                  (error, stdout, stderr) => {
                    console.log(language.get['STD_OUT'] + stdout);
                    if (error) {
                      log[client.id] +=
                        ',Process Stoped while copying guide to installer directory.';
                      client.installer_logs = log[client.id];
                      client.save();
                      reject(error);
                    } else {
                      log[client.id] += ',Guide placed at installer directory.';
                      client.installer_logs = log[client.id];
                      client.save();
                      resolve();
                    }
                  }
                );
              } else {
                resolve();
              }
            })
        )
        .then(
          () =>
            new Promise((resolve, reject) => {
              log[client.id] += ',Generating Certificates.';
              client.installer_logs = log[client.id];
              client.save();
              // generate certificates
              // this will also generate the certificates in the given folder
              if (
                client.cacert != null &&
                client.clientcrt != null &&
                client.clientpem != null &&
                client.cacer != null &&
                client.clientcer != null &&
                client.clientp12 != null
              ) {
                if (
                  client.cacert.length > 20 &&
                  client.clientcrt.length > 20 &&
                  client.clientpem.length > 20 &&
                  client.cacer.length > 20 &&
                  client.clientcer.length > 20 &&
                  client.clientp12.length > 20
                ) {
                  console.log('Using Old Certs');
                  log[client.id] += ',Using certificates from DB.';
                  client.installer_logs = log[client.id];
                  client.save();
                  exec(
                    `${projectConfig.serviceInstallerPath}/change-certs-permission.sh 1 ${certDirectory}`,
                    (error, stdout, stderr) => {
                      if (error) {
                        log[client.id] +=
                          ',Process Stoped while generating changing certificates permission.';
                        client.installer_logs = log[client.id];
                        client.save();
                        reject(error);
                      } else {
                        new Promise((resolv, reject) => {
                          log[client.id] +=
                            ',Writing certificates from DB to file.';
                          client.installer_logs = log[client.id];
                          client.save();
                          fs.writeFile(
                            `${certDirectory}/cacert.pem`,
                            client.cacert,
                            () => {}
                          );
                          fs.writeFile(
                            `${certDirectory}/client.crt`,
                            client.clientcrt,
                            () => {}
                          );
                          fs.writeFile(
                            `${certDirectory}/client.pem`,
                            client.clientpem,
                            () => {}
                          );
                          fs.writeFile(
                            `${certDirectory}/cacert.cer`,
                            client.cacer,
                            () => {}
                          );
                          fs.writeFile(
                            `${certDirectory}/client.cer`,
                            client.clientcer,
                            () => {}
                          );
                          fs.writeFile(
                            `${certDirectory}/client.p12`,
                            client.clientp12,
                            () => {}
                          );
                          resolv();
                        })
                          .then(
                            () =>
                              new Promise((resolv, rejct) => {
                                exec(
                                  `${projectConfig.serviceInstallerPath}/change-certs-permission.sh 0 ${certDirectory}`,
                                  (error, stdout, stderr) => {
                                    if (error) {
                                      log[client.id] +=
                                        ',Process Stoped while generating changing certificates permission.';
                                      client.installer_logs = log[client.id];
                                      client.save();
                                      rejct(error);
                                    } else {
                                      log[client.id] +=
                                        ',Certificates permission successfully set.';
                                      client.installer_logs = log[client.id];
                                      client.save();
                                      resolv();
                                    }
                                  }
                                );
                              })
                          )
                          .then(() => {
                            resolve();
                          })
                          .catch(err => {
                            console.log(err && (err.message || err));
                            client.installer_status = 'error';
                            client.save();
                            reject();
                          });
                      }
                    }
                  );
                } else {
                  console.log('Using New Certs!');
                  log[client.id] += ',Certificates are not valid.';
                  log[client.id] += ',Creating New Certificates.';
                  client.installer_logs = log[client.id];
                  client.save();
                  exec(
                    `sudo ${projectConfig.serviceInstallerPath}/generate-ssl-certs.sh ${key} ${certDirectory}`,
                    (error, stdout, stderr) => {
                      console.log(language.get['LOG_DIVIDER']);
                      console.log(language.get['LOG_DIVIDER']);
                      console.log(language.get['STD_OUT']);
                      console.log(stdout);
                      console.log('stderr ... ');
                      console.log(stderr);
                      console.log('error ... ');
                      console.log(error);
                      console.log(language.get['LOG_DIVIDER']);
                      console.log(language.get['LOG_DIVIDER']);
                      if (error) {
                        log[client.id] +=
                          ',Process Stoped while generating new certificates.';
                        client.installer_logs = log[client.id];
                        client.save();
                        reject(error);
                      } else {
                        log[client.id] +=
                          ',New certificates generated successfully.';
                        client.installer_logs = log[client.id];
                        client.save();
                        resolve();
                      }
                    }
                  );
                }
              } else {
                console.log('Using New Certs');
                log[client.id] += ',Creating New Certificates.';
                client.installer_logs = log[client.id];
                client.save();
                exec(
                  `sudo ${projectConfig.serviceInstallerPath}/generate-ssl-certs.sh ${key} ${certDirectory}`,
                  (error, stdout, stderr) => {
                    console.log(language.get['LOG_DIVIDER']);
                    console.log(language.get['LOG_DIVIDER']);
                    console.log(language.get['STD_OUT']);
                    console.log(stdout);
                    console.log('stderr ... ');
                    console.log(stderr);
                    console.log('error ... ');
                    console.log(error);
                    console.log(language.get['LOG_DIVIDER']);
                    console.log(language.get['LOG_DIVIDER']);
                    if (error) {
                      log[client.id] +=
                        ',Process Stoped while generating new certificates.';
                      client.installer_logs = log[client.id];
                      client.save();
                      reject(error);
                    } else {
                      log[client.id] +=
                        ',New certificates generated successfully.';
                      client.installer_logs = log[client.id];
                      client.save();
                      resolve();
                    }
                  }
                );
              }
            })
        )
        .then(
          () =>
            new Promise((resolve, reject) => {
              console.log('Generating configuration file');
              writeCfgFile(client, installerDir, resolve, reject);
            })
        )
        .then(
          () =>
            new Promise((resolve, reject) => {
              log[client.id] += ',Checking Package Prereqs.';
              client.installer_logs = log[client.id];
              client.save();
              if (
                fs.existsSync(certDirectory + '/cacert.pem') &&
                fs.existsSync(certDirectory + '/client.crt') &&
                fs.existsSync(certDirectory + '/client.pem') &&
                fs.existsSync(certDirectory + '/cacert.cer') &&
                fs.existsSync(certDirectory + '/client.cer') &&
                fs.existsSync(certDirectory + '/client.p12')
              ) {
                log[client.id] += ',Certificates exists.';
                client.installer_logs = log[client.id];
                client.save();
                console.log(language.get['LOG_DIVIDER']);
                console.log(language.get['LOG_DIVIDER']);
                console.log(fs.existsSync(installerFile));
                console.log(installerFile);
                console.log(language.get['LOG_DIVIDER']);
                console.log(language.get['LOG_DIVIDER']);
                if (fs.existsSync(installerFile)) {
                  log[client.id] += ',Installer exists.';
                  client.installer_logs = log[client.id];
                  client.save();
                  if (client.device_os === 'ubuntu') {
                    log[client.id] += ',Packaging Ubuntu Installer!.';
                    client.installer_logs = log[client.id];
                    client.save();
                    exec(
                      `${projectConfig.serviceInstallerPath}/client/input/ubuntu/invisily_package_client.sh ${installerDir}`,
                      (error, stdout, stderr) => {
                        if (error) {
                          log[client.id] +=
                            ',Process Stoped at packaging ubuntu 16 installer.';
                          client.installer_logs = log[client.id];
                          client.save();
                          reject(error);
                        } else {
                          log[client.id] += ',Ubuntu 16 Installer Packaged.';
                          client.installer_logs = log[client.id];
                          client.save();
                          resolve();
                        }
                      }
                    );
                  } else if (client.device_os === 'ubuntuserver') {
                    log[client.id] += ',Packaging Ubuntu Server.';
                    client.installer_logs = log[client.id];
                    client.save();
                    exec(
                      `${projectConfig.serviceInstallerPath}/client/input/ubuntuserver/invisily_package_client.sh ${installerDir}`,
                      (error, stdout, stderr) => {
                        if (error) {
                          log[client.id] +=
                            ',Process Stoped at packaging ubuntu 16 server..';
                          client.installer_logs = log[client.id];
                          client.save();
                          reject(error);
                        } else {
                          log[client.id] += ',Ubuntu 16 Server packaged.';
                          client.installer_logs = log[client.id];
                          client.save();
                          resolve();
                        }
                      }
                    );
                  } else if (client.device_os === 'ubuntu18') {
                    log[client.id] += ',Packaging Ubuntu 18.0.';
                    client.installer_logs = log[client.id];
                    client.save();
                    exec(
                      `${projectConfig.serviceInstallerPath}/client/input/ubuntu18/invisily_package_client.sh ${installerDir}`,
                      (error, stdout, stderr) => {
                        if (error) {
                          log[client.id] +=
                            ',Process Stoped at packaging Ubuntu 18.';
                          client.installer_logs = log[client.id];
                          client.save();
                          reject(error);
                        } else {
                          log[client.id] += ',Ubuntu 18 installer packaged.';
                          client.installer_logs = log[client.id];
                          client.save();
                          resolve();
                        }
                      }
                    );
                  } else if (client.device_os === 'ubuntuserver18') {
                    log[client.id] += ',Packaging Ubuntu 18 Server.';
                    client.installer_logs = log[client.id];
                    client.save();
                    exec(
                      `${projectConfig.serviceInstallerPath}/client/input/ubuntuserver18/invisily_package_client.sh ${installerDir}`,
                      (error, stdout, stderr) => {
                        if (error) {
                          log[client.id] +=
                            ',Process Stoped at Ubuntu 18 Server.';
                          client.installer_logs = log[client.id];
                          client.save();
                          reject(error);
                        } else {
                          log[client.id] += ',Ubuntu 18 server packaged.';
                          client.installer_logs = log[client.id];
                          client.save();
                          resolve();
                        }
                      }
                    );
                  } else if (client.device_os === 'macintosh') {
                    log[client.id] += ',Packaging macintosh.';
                    client.installer_logs = log[client.id];
                    client.save();
                    exec(
                      `${projectConfig.serviceInstallerPath}/client/input/macintosh/invisily_package_mac_client.sh ${installerDir}`,
                      (error, stdout, stderr) => {
                        if (error) {
                          log[client.id] += ',Process Stoped at macintosh.';
                          client.installer_logs = log[client.id];
                          client.save();
                          reject(error);
                        } else {
                          log[client.id] += ',Macintosh processing complete.';
                          client.installer_logs = log[client.id];
                          client.save();
                          resolve();
                        }
                      }
                    );
                  } else {
                    resolve();
                  }
                } else {
                  if (
                    client.device_os === 'android' ||
                    client.device_os === 'ios'
                  ) {
                    resolve();
                  } else {
                    log[client.id] += ',Installer doesnot exist.';
                    client.installer_logs = log[client.id];
                    client.save();
                    reject();
                  }
                }
              } else {
                log[client.id] += ',Certificates doesnot exist.';
                client.installer_logs = log[client.id];
                client.save();
                reject();
              }
            })
        )
        .then(
          () =>
            new Promise((resolve, reject) => {
              log[client.id] += ',Going For Zip generation.';
              client.installer_logs = log[client.id];
              client.save();
              zipInstaller(
                client,
                installerDir,
                packageDir,
                fileTypeMeta[client.device_os],
                installerTypeMeta[client.device_os],
                resolve,
                reject
              );
            })
        )
        .then(
          () =>
            new Promise((resolve, reject) => {
              // save file in database
              console.log('reading certificate files');
              log[client.id] += ',Zip generated successfully.';
              client.installer_logs = log[client.id];
              client.save();
              client.cacert = fs.readFileSync(`${certDirectory}/cacert.pem`);
              client.clientcrt = fs.readFileSync(`${certDirectory}/client.crt`);
              client.clientpem = fs.readFileSync(`${certDirectory}/client.pem`);
              client.cacer = fs.readFileSync(`${certDirectory}/cacert.cer`);
              client.clientcer = fs.readFileSync(`${certDirectory}/client.cer`);
              client.clientp12 = fs.readFileSync(`${certDirectory}/client.p12`);
              client.link_key = key;
              client.link_key_expiry = moment().add(
                emailLinkExpiryInMinutes + 2,
                'minutes'
              );
              client.save();
              resolve();
            })
        )
        .then(
          () =>
            new Promise((resolve, reject) => {
              fkey =
                Math.random()
                  .toString(36)
                  .substring(2) +
                `${client.updatedAt.getTime()}` +
                Math.random()
                  .toString(36)
                  .substring(2);
              // Adding Device ID link to DB
              models.DownloadPackage.create({
                link_key: fkey,
                used: 0,
                client_id: client.id,
                name: 'DownloadLink',
                link_key_expiry: client.link_key_expiry
              }).then(function(DP) {
                if (DP) {
                  resolve();
                } else {
                  reject();
                }
              });
            })
        )
        .then(
          () =>
            new Promise((resolve, reject) => {
              log[client.id] += ',Deleting Temporary Folder!';
              client.installer_logs = log[client.id];
              client.save();
              exec(
                `${projectConfig.serviceInstallerPath}/delete-temp-folder.sh ${installerDir}`,
                (error, stdout, stderr) => {
                  console.log('Executing delete-temp-folder.sh');
                  console.log(language.get['STD_OUT'] + stdout);
                  if (error) {
                    log[client.id] +=
                      ',Process Stoped while deleting temp folder.';
                    client.installer_logs = log[client.id];
                    client.save();
                    reject(error);
                  } else {
                    log[client.id] += ',Temp folder deleted Successfully';
                    client.installer_logs = log[client.id];
                    client.save();
                    resolve();
                  }
                }
              );
            })
        )
        .then(() => {
          return getClientCancelStatus(client);
        })
        .then(client => {
          if (!client) {
            throw new httpErrors.NotFound('Client not found.');
          }
          return Promise.all([
            generateLink({
              os: client.device_os,
              component: 'client',
              componentId: client.id,
              expiry_time: emailLinkExpiryInMinutes,
              filename: `invisily-client-${client.device_os}-${client.id}`
            }),
            client
          ]);
        })
        .then(async ([link, client]) => {
          log[client.id] += ',Sending Email.';
          client.installer_logs = log[client.id];
          client.link_key = link.key;
          client.save();
          let localUser = await client.getLocalUser();
          localUser = localUser ? localUser : { name: client.Name };
          if (packageState !== '1') {
            if (client.device_os === 'ios') {
              console.log('Calling to add ios content to content');
              contentQueue.addtoQueue({
                os: client.device_os,
                component: 'client',
                folderPath: packageDir + '/',
                filename: `invisily-client-ios-${client.id}`
              });
              setTimeout(async function() {
                const device = await models.DeviceInventories.findOne({
                  where: {
                    user_id: client.user_id,
                    id: client.device_id
                  }
                });
                if (!device) {
                  throw new Error('device not found');
                }
                const controller = await models.Controller.findOne({
                  where: {
                    roleId: 1
                  }
                });
                const clientSetting = await models.ClientSetting.findOne();

                helper.sendEmail(
                  prepareEmailOptions(
                    client.Email,
                    language.get['INV_CLIENT_SOFTWARE_INS'],
                    emailTemplateName,
                    prepareOnPremiseDownloadLink(
                      controller,
                      client.device_os,
                      link.key,
                      fkey,
                      device.device_id,
                      controller.spa_port,
                      device.deviceSerialNumber
                    ),
                    prepareOnPremiseDownloadLink(
                      controller,
                      client.device_os,
                      link.key,
                      fkey,
                      device.device_id,
                      controller.spa_port,
                      device.deviceSerialNumber
                    ),
                    localUser.name,
                    emailLinkExpiryInMinutes,
                    'client',
                    prepareOnPremiseDeviceIdLink(client.device_os, fkey),
                    prepareOffPremiseDeviceIdLink(client.device_os, fkey),
                    clientSetting.ios_link,
                    clientSetting.ios_link
                  ),
                  {
                    device_os: installerDescriptiveName[client.device_os]
                      ? installerDescriptiveName[client.device_os]
                      : client.device_os,
                    package_expiry_time: moment(link.link_expired_at).fromNow(
                      true
                    )
                  }
                );
                let qrJson = {};

                qrJson.url = prepareOnPremiseDownloadLinkForJSON(
                  controller,
                  client.device_os,
                  link.key,
                  fkey,
                  device.device_id,
                  controller.spa_port,
                  device.deviceSerialNumber
                );

                qrJson.api = getAPIUrlForDeviceOnPrem(fkey, controller);
                client.qr_json = JSON.stringify(qrJson);
                client.installer_status = 'success';
                log[client.id] += language.get['CLNT_PKG_SNT'];
                log[client.id] =
                  language.get['EMAIL_WLCM_TEXT'] +
                  prepareOnPremiseDownloadLink(
                    controller,
                    client.device_os,
                    link.key,
                    fkey,
                    device.device_id,
                    controller.spa_port,
                    device.deviceSerialNumber
                  ) +
                  language.get['EMAIL_WLCM_TEXT_OFFPREM'] +
                  prepareOnPremiseDownloadLink(
                    controller,
                    client.device_os,
                    link.key,
                    fkey,
                    device.device_id,
                    controller.spa_port,
                    device.deviceSerialNumber
                  ) +
                  language.get['EMAIL_WLCM_TEXT_1'] +
                  prepareOnPremiseDeviceIdLink(
                    controller,
                    client.device_os,
                    fkey,
                    device.device_id,
                    controller.spa_port,
                    device.deviceSerialNumber
                  ) +
                  language.get['EMAIL_WLCM_TEXT_2'] +
                  prepareOffPremiseDeviceIdLink(
                    controller,
                    client.device_os,
                    fkey,
                    device.device_id,
                    controller.spa_port,
                    device.deviceSerialNumber
                  ) +
                  language.get['EMAIL_WLCM_TEXT_3'];
                client.onPrenDownloadLink = prepareOnPremiseDownloadLink(
                  controller,
                  client.device_os,
                  link.key,
                  fkey,
                  device.device_id,
                  controller.spa_port,
                  device.deviceSerialNumber
                );
                client.installer_logs = log[client.id];
                client.save();
              }, 6000);
            } else if (client.device_os === 'android') {
              console.log('Calling to add android content to content');
              contentQueue.addtoQueue({
                os: client.device_os,
                component: 'client',
                folderPath: packageDir + '/',
                filename: `invisily-client-android-${client.id}`
              });
              setTimeout(async function() {
                const device = await models.DeviceInventories.findOne({
                  where: {
                    user_id: client.user_id,
                    id: client.device_id
                  }
                });
                if (!device) {
                  throw new Error('device not found');
                }
                const controller = await models.Controller.findOne({
                  where: {
                    roleId: 1
                  }
                });

                const clientSetting = await models.ClientSetting.findOne();

                helper.sendEmail(
                  prepareEmailOptions(
                    client.Email,
                    language.get['INV_CLIENT_SOFTWARE_INS'],
                    emailTemplateName,
                    prepareOnPremiseDownloadLink(
                      controller,
                      client.device_os,
                      link.key,
                      fkey,
                      device.device_id,
                      controller.spa_port,
                      device.deviceSerialNumber
                    ),
                    prepareOnPremiseDownloadLink(
                      controller,
                      client.device_os,
                      link.key,
                      fkey,
                      device.device_id,
                      controller.spa_port,
                      device.deviceSerialNumber
                    ),
                    localUser.name,
                    emailLinkExpiryInMinutes,
                    'client',
                    prepareOnPremiseDeviceIdLink(client.device_os, fkey),
                    prepareOffPremiseDeviceIdLink(client.device_os, fkey),
                    clientSetting.android_link,
                    clientSetting.android_link
                  ),
                  {
                    device_os: installerDescriptiveName[client.device_os]
                      ? installerDescriptiveName[client.device_os]
                      : client.device_os,
                    package_expiry_time: moment(link.link_expired_at).fromNow(
                      true
                    )
                  }
                );
                let qrJson = {};

                qrJson.url = prepareOnPremiseDownloadLinkForJSON(
                  controller,
                  client.device_os,
                  link.key,
                  fkey,
                  device.device_id,
                  controller.spa_port,
                  device.deviceSerialNumber
                );

                qrJson.api = getAPIUrlForDeviceOnPrem(fkey, controller);
                client.qr_json = JSON.stringify(qrJson);
                client.installer_status = 'success';
                log[client.id] += language.get['CLNT_PKG_SNT'];
                (log[client.id] =
                  language.get['EMAIL_WLCM_TEXT'] +
                  prepareOnPremiseDownloadLink(
                    controller,
                    client.device_os,
                    link.key,
                    fkey,
                    device.device_id,
                    controller.spa_port,
                    device.deviceSerialNumber
                  )),
                  +language.get['EMAIL_WLCM_TEXT_OFFPREM'] +
                    prepareOnPremiseDownloadLink(
                      controller,
                      client.device_os,
                      link.key,
                      fkey,
                      device.device_id,
                      controller.spa_port,
                      device.deviceSerialNumber
                    ),
                  +language.get['EMAIL_WLCM_TEXT_1'] +
                    prepareOnPremiseDeviceIdLink(client.device_os, fkey) +
                    language.get['EMAIL_WLCM_TEXT_2'] +
                    prepareOffPremiseDeviceIdLink(client.device_os, fkey) +
                    language.get['EMAIL_WLCM_TEXT_3'];
                console.log(log[client.id]);
                client.onPrenDownloadLink = prepareOnPremiseDownloadLink(
                  controller,
                  client.device_os,
                  link.key,
                  fkey,
                  device.device_id,
                  controller.spa_port,
                  device.deviceSerialNumber
                );
                client.installer_logs = log[client.id];
                client.save();
              }, 6000);
            } else {
              const controller = await models.Controller.findOne({
                where: {
                  roleId: 1
                }
              });
              helper.sendEmail(
                prepareEmailOptions(
                  client.Email,
                  language.get['INV_CLIENT_SOFTWARE_INS'],
                  'client_onboarding_email.html',
                  prepareOnPremiseDownloadLink(
                    controller,
                    client.device_os,
                    link.key,
                    fkey
                  ),
                  prepareOnPremiseDownloadLink(
                    controller,
                    client.device_os,
                    link.key,
                    fkey
                  ),
                  localUser.name,
                  emailLinkExpiryInMinutes,
                  'client',
                  prepareOnPremiseDeviceIdLink(client.device_os, fkey),
                  prepareOffPremiseDeviceIdLink(client.device_os, fkey)
                )
              );
              client.installer_status = 'success';
              log[client.id] += language.get['CLNT_PKG_SNT'];
              (log[client.id] =
                language.get['EMAIL_WLCM_TEXT'] +
                prepareOnPremiseDownloadLink(
                  controller,
                  client.device_os,
                  link.key,
                  fkey
                )),
                +language.get['EMAIL_WLCM_TEXT_OFFPREM'] +
                  prepareOnPremiseDownloadLink(
                    controller,
                    client.device_os,
                    link.key,
                    fkey
                  ),
                +language.get['EMAIL_WLCM_TEXT_1'] +
                  prepareOnPremiseDeviceIdLink(client.device_os, fkey) +
                  language.get['EMAIL_WLCM_TEXT_2'] +
                  prepareOffPremiseDeviceIdLink(client.device_os, fkey) +
                  language.get['EMAIL_WLCM_TEXT_3'];
              console.log(log[client.id]);
              client.onPrenDownloadLink = prepareOnPremiseDownloadLink(
                controller,
                client.device_os,
                link.key
              );
              client.installer_logs = log[client.id];
              client.save();
            }
          } else {
            const controller = await models.Controller.findOne({
              where: {
                roleId: 1
              }
            });

            client.installer_status = 'success';
            log[client.id] += language.get['CLNT_PKG_SNT'];
            (log[client.id] =
              language.get['EMAIL_WLCM_TEXT'] +
              prepareOnPremiseDownloadLink(
                controller,
                client.device_os,
                link.key,
                fkey
              )),
              +language.get['EMAIL_WLCM_TEXT_OFFPREM'] +
                prepareOnPremiseDownloadLink(
                  controller,
                  client.device_os,
                  link.key,
                  fkey
                ),
              +language.get['EMAIL_WLCM_TEXT_1'] +
                prepareOnPremiseDeviceIdLink(client.device_os, fkey) +
                language.get['EMAIL_WLCM_TEXT_2'] +
                prepareOffPremiseDeviceIdLink(client.device_os, fkey) +
                language.get['EMAIL_WLCM_TEXT_3'];
            client.onPrenDownloadLink = prepareOnPremiseDownloadLink(
              controller,
              client.device_os,
              link.key
            );
            client.installer_logs = log[client.id];
            client.save();
          }
        })
        .catch(err => {
          console.log(err && (err.message || err));
          log[client.id] += ',Error while creating package.';
          client.installer_logs = log[client.id];
          client.installer_status = 'error';

          if (err instanceof PackageCancelError) {
            log[client.id] += ',Cancel the package.';
            client.installer_logs = log[client.id];
            client.installer_status = 'cancel';
          }
          client.save();
          exec(
            `${projectConfig.serviceInstallerPath}/delete-temp-folder.sh ${installerDir}`,
            (error, stdout, stderr) => {
              console.log('Executing delete-temp-folder.sh');
              console.log(language.get['STD_OUT'] + stdout);
              if (error) {
                log[client.id] += ',Process Stoped while deleting temp folder.';
                client.installer_logs = log[client.id];
                client.save();
                console.log(error && (error.message || error));
              } else {
                log[client.id] += ',Temp folder deleted Successfully';
                client.installer_logs = log[client.id];
                client.save();
              }
            }
          );
        });
    });
}

function isMobileOS(client) {
  return client.device_os === 'android' || client.device_os === 'ios';
}

function isPackageWithInstallerPackage(type) {
  let result = true;

  if (
    type === installerTypeMeta['android'] ||
    type === installerTypeMeta['ios']
  ) {
    result = false;
  }

  return result;
}

function isPackageWithInstallerScript(type) {
  let result = false;

  if (
    type === installerTypeMeta['ubuntu'] ||
    type === installerTypeMeta['windows'] ||
    type === installerTypeMeta['macintosh'] ||
    type === installerTypeMeta['ubuntuserver'] ||
    type === installerTypeMeta['ubuntu18'] ||
    type === installerTypeMeta['ubuntuserver18'] ||
    type === installerTypeMeta['centosserver']
  ) {
    result = true;
  }

  return result;
}

function isMac(type) {
  return type.device_os === 'macintosh';
}

const updateClientRiskScore = async (req, res, next) => {
  try {
    const { agent_id, risk_score } = req.body;

    const client = await models.Client.findByPk(agent_id);

    if (!client) {
      throw new httpErrors.NotFound('No client found.');
    }

    await client.update({ risk_score });

    await publishClients(actionType.UPDATED, {
      ids: [client.id],
      mode: 0,
      data: { risk_score },
      type: client.client_type
    });

    return res.json({
      success: true,
      message: language.get['EDIT_SUCCESS']
    });
  } catch (error) {
    next(error);
  }
};

const sendOtpQrBulkUsers = clients => {
  const alreadySent = [];

  clients.forEach(async (client, index) => {
    const myClient = await models.Client.findByPk(client.id);
    if (alreadySent.includes(myClient.Email)) {
      return;
    } else {
      alreadySent.push(myClient.Email);
    }
    let localUser = null;
    if (myClient) {
      localUser = await myClient.getLocalUser();
    }
    helper.sendQRCode({
      OTP_image_data: client.OTP_image_data,
      recipient: client.Email,
      recipientName: localUser ? localUser.name : client.Name,
      emailSubject: 'Your Invisily QR Code',
      templateFileName: 'client_linotp_email.html',
      onPremApiUrl: projectConfig.onPremApiUrl,
      client: true
    });
  });
};

const updateClientAutoUpdates = async (req, res, next) => {
  try {
    const { agent_id } = req.params;
    const { auto_updates } = req.body;
    const client = await models.Client.findOne({
      where: {
        agent_id
      }
    });
    if (!client) return res.status(400).json({ message: 'Invalid agent_id' });

    const serverBridge = await models.sequelize.query(
      `SELECT client_id
       FROM ServerBridgeDeviceIds sbu
       JOIN ServerBridges sb ON sb.id = sbu.serverbridgeId
       WHERE sb.clientId = :agent_id`,
      {
        replacements: { agent_id },
        type: models.sequelize.QueryTypes.SELECT
      }
    );

    if (serverBridge.length) {
      await models.sequelize.query(
        `UPDATE Clients c
         SET c.auto_updates = :auto_updates
         WHERE c.id IN (
             SELECT client_id
             FROM ServerBridgeDeviceIds sbu
             JOIN ServerBridges sb ON sb.id = sbu.serverbridgeId
             WHERE sb.clientId = :agent_id
         )`,
        {
          replacements: { agent_id, auto_updates },
          type: models.sequelize.QueryTypes.UPDATE
        }
      );
    }

    await client.update({
      auto_updates
    });

    res.status(200).json({
      message: 'Client updated'
    });
  } catch (e) {
    next(e);
  }
};

const getClientId = async (req, res, next) => {
  try {
    const name = req.params.name;
    const client = await models.Client.findOne({
      where: {
        Name: name
      }
    });

    if (client) {
      res.status(200).json({ client: client.dataValues.id });
    } else {
      res.status(400).json({ message: 'Cient not found' });
    }
  } catch (error) {
    next(error);
  }
};

const bulkClientAutoUpdates = async (req, res, next) => {
  try {
    const { clientIds, state } = req.body;
    let clientsToUpdate;

    if (clientIds.selectAll && clientIds.selectAllExcept.length == 0) {
      clientsToUpdate = await models.Client.findAll({
        where: {
          client_type: clientIds.clientType
        }
      });
    } else if (clientIds.selectAll && clientIds.selectAllExcept.length > 0) {
      clientsToUpdate = await models.Client.findAll({
        where: {
          [Op.and]: [
            { agent_id: { [Op.notIn]: clientIds.selectAllExcept } },
            { client_type: clientIds.clientType }
          ]
        }
      });
    } else if (
      !clientIds.selectAll &&
      clientIds.selectAllExcept.length > 0 &&
      clientIds.selectedClients.length > 0
    ) {
      const differentIds = clientIds.selectedClients.filter(id => !clientIds.selectAllExcept.includes(id));
      clientsToUpdate = await models.Client.findAll({
        where: {
          [Op.and]: [
            { agent_id: { [Op.in]: differentIds } },
            // { client_type: differentIds.clientType }
          ]
        }
      });
    } else if (clientIds.selectedClients.length > 0) {
      clientsToUpdate = await models.Client.findAll({
        where: {
          agent_id: clientIds.selectedClients
        }
      });
    }

    clientsToUpdate.forEach(async client => {
      await client.update({ auto_updates: state });
    });

    res.status(200).json({
      message: 'Client(s) bulk updated',
    });
  } catch (error) {
    next(error);
  }
};

module.exports = {
  bulkActions,
  downloadService,
  status,
  approve,
  cancelPackage,
  sendOTPQrCode,
  verifyOtp,
  deregMFA,
  getInstallerStatus,
  sendLink,
  download,
  downloadService,
  updateClient,
  getClientDetail,
  unlockClient,
  generateService,
  getClients,
  generateLinOTPToken,
  addClient,
  deleteClient,
  updateClientRiskScore,
  sendOtpQrBulkUsers,
  sendPackage,
  generateOTPToken,
  sendOTPQr,
  generateZip,
  installerDescriptiveName,
  updateClientAutoUpdates,
  getQuickClients,
  getClientServicesWithGatewayGraph,
  getClientId,
  bulkClientAutoUpdates
};
