const models = require('../models');
const {Op} = require('sequelize');
const httpErrors = require('http-errors');
const {
    actionType,
    publishMicrosegmentAdd,
    publishMicrosegmentUpdate,
    publishMicrosegmentRemove,
    publishWorkload,
    publishStartDiscovery
} = require('../routes/common/event-publisher');
const {Sequelize} = require('../models');
Sequelize.options.deprecations = { operatorsAliases: false };

const { 
    getPublishMicrosegmentMessage,
} = require('../config/adminLogs.config');

const { insertIntoAdminLogs } = require('../routes/common/adminLogs');

const getWorkLoad = async (req, res, next) => {
    try {
        const {id} = req.params;
        let devices = [];
        const microsegments = await models.Microsegments.findAll({
            where: {
                workloadId: id
            },
            attributes: {
                include: ['id']
            }
        });

        let Ids = microsegments.map((item) => {
            return item.deviceId;
        });

        const workLoad = await models.Workloads.findOne({
            where: {
                id
            },
            include: [
                {
                    model: models.DeviceInventories,
                    attributes: ['id', 'device_name'],
                    // where:{
                    //   id: {
                    //     [sequelize.Op.notIn]: Ids
                    //   }
                    // },
                    raw: true
                }
            ]
        });
        workLoad.DeviceInventories.map((item) => {
            if (Ids.includes(item.id)) {
            } else {
                devices.push(item);
            }
        });
        res.json({workLoad, devices});
    } catch (e) {
        next(e);
    }
};

const getMicrosegment = async (req, res, next) => {
    try {
        const {id, deviceId} = req.params;
        const microsegment = await models.Microsegments.findOne({
            where: {
                workloadId: id,
                deviceId
            }
        });
        if (microsegment) {
            res.status(200).send(microsegment);
        } else {
            res.json('microsegment not found');
        }
    } catch (e) {
        next(e);
    }
};

const discoverMicrosegmentsOLD = async (req, res, next) => {
    const transaction = await models.sequelize.transaction();
    try {
        const {workloadId} = req.params;
        let micro;
        let data;
        let microIds = [];
        let allDiscovery;
        let destinationData;
        let counter = 1;
        let microsegment = {
            name: "",
            workloadId,
            deviceId: '',
            description: ""
        };
        const workloadName = await models.Workloads.findOne({
            where: {
                id: workloadId
            },
            attributes: ['name'],
            raw: true
        });
        const workloadPivots = await models.WorkloadDevicePivots.findAll({
            where: {
                workloadId
            },
            attributes: [
                [Sequelize.fn('DISTINCT', Sequelize.col('clientId')), 'clientId']
            ]
        });
        const clientIds = workloadPivots.map(pivot => pivot.clientId);
        data = await models.S2sDiscoveryData.findAll({
            where: {
                client_id: clientIds,
            },
            attributes: ['source_ip'],
            group: ['source_ip'],
            raw: true
        });
        const sourceIps = data.map(pivot => pivot.source_ip);
        let sourceDevices = await models.DeviceInventories.findAll({
            where: {
                ip_address: sourceIps
            },
            attributes: ['id'],
            raw: true
        });
        sourceDevices = sourceDevices.map(v => v.id);
        const microArr = await models.Microsegments.findAll({
            where: {
                workloadId,
                deviceId: {[Op.in]: sourceDevices}
            },
            raw: true
        });
        // if(microArr.length > 0){
        // const microsegmentIds = microArr.map(p=>p.id);
        // await models.MicrosegmentPivots.destroy({ where: { microsegmentId: {[Op.in]: microsegmentIds}}});
        for (const micro of microArr) {
            await models.MicrosegmentPivots.destroy({where: {microsegmentId: micro.id}});
            await models.Microsegments.destroy({where: {workloadId: micro.workloadId, deviceId: micro.deviceId}});
        }
        // }

        for (const d of sourceDevices) {
            microsegment.deviceId = d;
            microsegment.name = `${workloadName.name}-microsegment${counter++}`;
            micro = await models.Microsegments.create(microsegment);
            microIds.push(micro.id);
            allDiscovery = await models.S2sDiscoveryData.findAll({
                where: {
                    deviceId: d,
                },
                raw: true
            });
            for (const discovery of allDiscovery) {
                if (discovery && discovery.allowed_comm == 1) {
                    destinationData = {
                        ports: discovery.destination_port,
                        protocols: discovery.protocol,
                        microsegmentId: micro.id,
                        destination: discovery.destination_ip
                    };
                    await models.MicrosegmentPivots.create(destinationData);
                }
            }
        }
        // sourceDevices.map(async (d,index)=>{
        //   microsegment.deviceId = d;
        //   microsegment.name = `${workloadName.name}-microsegment${index+1}`;
        //   Promise.all([
        //     micro = models.Microsegments.create(microsegment),
        //     allDiscovery = models.S2sDiscoveryData.findAll({
        //       where: {
        //         deviceId:d,
        //       },
        //       raw:true
        //     })
        //   ]).then(async ([micro,allDiscovery])=>{
        //     allDiscovery.map((discovery)=>{
        //       microsegmentIds.push(micro.id)
        //         if(discovery && discovery.allowed_comm == 1){
        //           destinationData = {
        //             ports: discovery.destination_port,
        //             protocols: discovery.protocol,
        //             microsegmentId:micro.id,
        //             destination: discovery.destination_ip
        //           };
        //           return models.MicrosegmentPivots.create(destinationData);
        //         }
        //     })
        //   })
        // });
        await publishMicrosegmentAdd(actionType.ADDED, {
            id: microIds
        });
        res.status(200).json({message: "Microsegments created"})
    } catch (e) {
        await transaction.rollback();
        next(e);
    }
};

const discoverMicrosegments = async (req, res, next) => {
    try {
        const workload = await models.Workloads.findOne({
            where: {
                id: req.params.workloadId
            },
        });

        await publishWorkload(actionType.UPDATED, {});
        insertIntoAdminLogs(
            getPublishMicrosegmentMessage(workload.name),
            req
        )
        res.json('Workload updated successfully');
    } catch (e) {
        next(e);
    }
};

const discoverAllMicrosegments = async (req, res, next) => {
    try {
        await models.Workloads.update(
            { s2s_mode: 'spa' },
            { where: {} }
        );        

        await publishWorkload(actionType.UPDATED, {});
        insertIntoAdminLogs(
            getPublishMicrosegmentMessage('Default'),
            req
        )
        res.json('Workload updated successfully');
    } catch (e) {
        next(e);
    }
};


const getMicrosegmentDestinations = async (req, res, next) => {
    try {
        const {
            microsegmentId,
            limit,
            offset,
            sort = 'id',
            dir = 'desc',
            searchTerm,
        } = req.query;

        const options = {
            where: {
                microsegmentId
            },
            include: [
                {
                    model: models.DeviceInventories,
                    attributes: ['device_name'],
                    required: false
                }
            ],
            raw: true,
            limit,
            offset,
            order: []
        };

        if (searchTerm) {
            options.where = {
                microsegmentId,
                [Op.or]: {
                    ports: {
                        [Op.like]: `%${searchTerm.trim()}%`
                    },
                    protocols: {
                        [Op.like]: `%${searchTerm.trim()}%`
                    }
                }
            }
        }

        const result = await models.MicrosegmentPivots.paginate({
            ...options
        });
        result.data = result.data.map((d) => {
            let data = Object.values(d);
            let device_name = data[6] ? data[6] : d.destination;
            return {
                ports: d.ports,
                protocols: d.protocols,
                device_name,
                destination: d.destination
            }
        });
        res.json(result);
    } catch (error) {
        next(error);
    }
};

const getPorts = async (req, res, next) => {
    try {
        const {microsegmentId, destination} = req.params;
        const destIP = await models.DeviceInventories.findOne({
            where: {
                id: destination
            },
            attributes: ['ip_address'],
            raw: true
        })
        if (!destIP) {
            throw new httpErrors.NotFound('No data found');
        }
        const ports = await models.MicrosegmentPivots.findOne({
            where: {
                microsegmentId,
                destination: destIP.ip_address
            }
        });
        res.json(ports);
    } catch (e) {

    }
}

const getEditPorts = async (req, res, next) => {
    try {
        const {microsegmentId} = req.params;
        const {destination, ports, protocols, device_name} = req.body;
        const port = await models.MicrosegmentPivots.findOne({
            where: {
                microsegmentId,
                destination,
                ports,
                protocols
            },
            raw: true
        });
        port.dest_name = device_name;
        res.json(port);
    } catch (e) {

    }
}

// const rangeGenerator = (ports) =>{
//   let portsArr = ports.split(',').filter(n => n);
//   let tempArr = [...portsArr];
//   tempArr.map((data)=>{
//       if(data.indexOf('-') > -1){
//         var parms=data.split('-');
//         var start=parms[0];
//         var end=parms[1];
//         for (var i=start; i<=end; i++) {
//           portsArr.push(String(i));
//         }
//         var index = portsArr.indexOf(data);
//         portsArr.splice(index, 1);
//       }
//   });
//   return portsArr;
// }

// const addRangeProtocols = (ports,protocols) =>{
//   let portsArr = ports.split(',').filter(n => n);
//   let protocolsArr = protocols.split(',').filter(n => n);
//   let indexProtocol;
//   portsArr.map((data,index)=>{
//       if(data.indexOf('-') > -1){
//         var parms=data.split('-');
//         var start=parms[0];
//         var end=parms[1];
//         indexProtocol = protocolsArr[index];
//         for (var i=start; i<=end; i++) {
//           protocolsArr.push(String(indexProtocol));
//         }
//         protocolsArr.splice(index, 1);
//       }
//   });
//   return protocolsArr;
// }

const createDestinationPort = async (req, res, next) => {
    const transaction = await models.sequelize.transaction();
    try {
        const {ports, protocols, microsegmentId, destination} = req.body;
        const destIP = await models.DeviceInventories.findOne({
            where: {
                id: destination
            },
            attributes: ['ip_address'],
            raw: true
        })
        if (!destIP) {
            throw new httpErrors.NotFound('No data found');
        }
        const destinationPort = {
            ports,
            protocols,
            microsegmentId,
            destination: destIP.ip_address
        };
        const portsCheck = await models.MicrosegmentPivots.findOne({
            where: {
                microsegmentId,
                destination: destIP.ip_address
            }
        });
        if (portsCheck) {
            // let inputPorts = rangeGenerator(ports);
            //   let portsData = inputPorts.join(',');
            await models.MicrosegmentPivots.update({ports, protocols}, {
                where: {microsegmentId, destination: destIP.ip_address},
                transaction
            });
            await publishMicrosegmentUpdate(actionType.UPDATED, {
                id: parseInt(microsegmentId)
            });
            await transaction.commit();
            res.json({
                status: true,
                message: `Destination Device Updated Successfully`
            });
        } else {
            //   // let inputPorts = rangeGenerator(ports);
            //   // let data = addRangeProtocols(ports,protocols);
            //   // destinationPort.protocols = data.join(',');
            //   // destinationPort.ports = [...new Set(inputPorts)].join(',');
            await models.MicrosegmentPivots.create(destinationPort, {
                transaction
            });
            await transaction.commit();
            await publishMicrosegmentUpdate(actionType.UPDATED, {
                id: parseInt(microsegmentId)
            });
            res.json({
                status: true,
                message: `Ports Added Successfully`
            });
        }
    } catch (e) {
        await transaction.rollback();
        next(e);
    }
};

const createMicrosegment = async (req, res, next) => {
    const transaction = await models.sequelize.transaction();
    try {
        let microsegmentId;
        let {name, workloadId, selectedClientIds, deviceId, destinations, description} = req.body;
        const microsegment = {
            name,
            workloadId,
            deviceId,
            description
        };
        const item = await models.Microsegments.findOne({
            where: {
                workloadId,
                deviceId
            }
        });
        if (item) {
            microsegmentId = item.id;
            await models.Microsegments.update(microsegment, {
                where: {id: microsegmentId}
            });
            await transaction.commit();
            await publishMicrosegmentAdd(actionType.UPDATED, {
                id: parseInt(microsegmentId)
            });
            res.json({
                message: `Microsegment Updated Successfully`
            });
        } else {
            const micro = await models.Microsegments.create(microsegment, {
                transaction
            });
            microsegmentId = micro.id;
            let data = destinations.map(v => ({...v, microsegmentId: microsegmentId}));
            if (data.length <= 0) {
                throw new httpErrors.NotFound('Please Add at least 1 destination device');
            }
            await models.MicrosegmentPivots.bulkCreate(data, {
                transaction
            });
            await transaction.commit();
            await publishMicrosegmentAdd(actionType.ADDED, {
                id: parseInt(microsegmentId)
            });
            res.json({
                message: `Microsegment Created Successfully`
            });
        }


        // let destinationDevices = [];
        // if(ports.length === 0 && protocols.length === 0){
        //   throw new httpErrors.NotFound('Please Select Ports and Protocols');
        // }
        // destinationDevices = selectedClientIds.map((destination)=>{
        //   return {
        //     microsegmentId,
        //     destination
        //   }
        // });
        // await models.MicrosegmentPivots.update({ports,protocols},{
        //       where:{ microsegmentId, destination }
        // })

    } catch (e) {
        await transaction.rollback();
        next(e);
    }
};

const getMicrosegments = async (req, res, next) => {
    try {
        const {
            workloadId,
            limit,
            offset,
            sort = 'id',
            dir = 'desc',
            searchTerm,
        } = req.query;

        const options = {
            where: {
                workloadId
            },
            include: [
                {
                    model: models.DeviceInventories,
                    attributes: ['device_name'],
                    as: 'MPMicrosegementPivots'
                },
                {
                    model: models.DeviceInventories,
                    attributes: ['id', 'device_name'],
                    required: false
                }
            ],
            limit,
            offset,
            order: []
        };

        const allowedSortValues = [
            'id',
            'name'
        ];

        if (allowedSortValues.includes(sort)) {
            options.order.push([models.sequelize.literal(sort), dir]);
        }

        if (searchTerm) {
            options.where = {
                workloadId,
                [Op.or]: {
                    name: {
                        [Op.like]: `%${searchTerm.trim()}%`
                    },
                    description: {
                        [Op.like]: `%${searchTerm.trim()}%`
                    }
                }
            }
        }

        const result = await models.Microsegments.paginate({
            ...options
        });
        res.json(result);
    } catch (error) {
        next(error);
    }
};

const deleteMicrosegment = async (req, res, next) => {
    const transaction = await models.sequelize.transaction();
    try {
        const microsegment = await models.Microsegments.findOne({
            where: {
                id: req.params.id,
            }
        });
        if (!microsegment) {
            throw new httpErrors.NotFound('Microsegment not found');
        }
        // const microsegments = await models.Microsegments.findAll({
        //   where: {
        //     workloadId:req.params.id
        //   }
        // });
        // if(microsegments.length > 0){
        //   let mPivots = [];
        //   microsegments.map((item)=>{
        //     mPivots.push(item.id);
        //   });
        //   await models.MicrosegmentPivots.destroy({ where: { microsegmentId: mPivots }},{transaction});
        //   await models.Microsegments.destroy({ where: { workloadId: req.params.id }},{transaction});
        // }
        //await models.WorkloadDevicePivots.destroy({ where: { workloadId: req.params.id }});
        await models.MicrosegmentPivots.destroy({where: {microsegmentId: req.params.id}}, {transaction});
        await microsegment.destroy({transaction});
        await transaction.commit();
        await publishMicrosegmentRemove(actionType.DELETED, {
            id: parseInt(req.params.id)
        });
        res.json('Microsegment deleted successfully');
    } catch (error) {
        await transaction.rollback();
        next(error);
    }
}

const deleteMicrosegmentDestination = async (req, res, next) => {
    const transaction = await models.sequelize.transaction();
    try {
        const check = await models.MicrosegmentPivots.findAll({
            where: {
                microsegmentId: req.params.id
            }
        });
        if (check.length == 1) {
            throw new httpErrors.NotFound('destination device not deleted please delete microsegment');
        }
        const {destination, ports, protocols} = req.body;
        await models.MicrosegmentPivots.destroy({
            where: {
                ports,
                protocols,
                microsegmentId: req.params.id,
                destination
            }
        }, {transaction});
        // if (!microsegmentDestination) {
        //   throw new httpErrors.NotFound('Microsegment Destination not found');
        // }
        // //await models.MicrosegmentPivots.destroy({ where: { microsegmentId: req.params.id}},{transaction});
        // console.log(microsegmentDestination);
        // await microsegmentDestination.destroy({transaction});
        await transaction.commit();
        await publishMicrosegmentRemove(actionType.DELETED, {
            id: parseInt(req.params.id)
        });
        res.json('Microsegment destination device deleted successfully');
    } catch (error) {
        await transaction.rollback();
        next(error);
    }
}

const getWorkloadMicrosegments = async (req, res, next) => {
    try {
        const {id} = req.params;
        const microsegment = await models.Microsegments.findOne({
            where: {
                id
            },
            raw: true
        });

        let deviceId = microsegment.deviceId;

        const workloadMicrosegment = await models.DeviceInventories.findOne({
            where: {
                id: deviceId
            },
            attributes: ['id', 'device_name'],
            raw: true
        });
        workloadMicrosegment.microsegmentId = microsegment.id;
        workloadMicrosegment.name = microsegment.name;
        workloadMicrosegment.description = microsegment.description;

        let microsegments = await models.MicrosegmentPivots.findAll({
            where: {
                microsegmentId: id
            },
            attributes: ['ports', 'protocols', 'destination'],
            include: [
                {
                    model: models.DeviceInventories,
                    attributes: ['device_name'],
                    required: false
                }
            ],
            raw: true
        });
        microsegments = microsegments.map((d) => {
            let data = Object.values(d);
            let device_name = data[3] ? data[3] : d.destination;
            return {
                ports: d.ports,
                protocols: d.protocols,
                destination: d.destination,
                device_name,
            }
        });
        let devicesId = microsegments.map((item) => {
            return item.destination
        });

        const microsegmentDevices = await models.DeviceInventories.findAll({
            where: {
                id: devicesId
            },
            attributes: ['id', 'device_name'],
            raw: true
        });

        res.json({workloadMicrosegment, microsegmentDevices, microsegments});
    } catch (e) {
        next(e);
    }
};

module.exports = {
    getWorkLoad,
    getMicrosegment,
    getPorts,
    getEditPorts,
    createDestinationPort,
    createMicrosegment,
    getMicrosegments,
    deleteMicrosegment,
    getMicrosegmentDestinations,
    discoverAllMicrosegments,
    getWorkloadMicrosegments,
    deleteMicrosegmentDestination,
    discoverMicrosegments
};
