var models = require('../models');
const moment = require('moment');
var httpErrors = require('http-errors');

const getOsVersions = async (req, res, next) => {
    try {
        const dbRecords = await models.OsVersions.findAll({});
        let result = {};
        if(dbRecords.length && dbRecords.length != 0) {
            for(let j=0; j<dbRecords.length; j++) {
                if(result[dbRecords[j].tagName]) {
                    result[dbRecords[j].tagName] = [
                        ...result[dbRecords[j].tagName],
                        { 
                            id: dbRecords[j].id,
                            value: dbRecords[j].buildVersion, 
                            display: dbRecords[j].buildVersion
                        }
                    ];
                }
                else {
                    result[dbRecords[j].tagName] = [
                        {
                            id: dbRecords[j].id,
                            value: dbRecords[j].buildVersion, 
                            display: dbRecords[j].buildVersion
                        }
                    ];
                    
                }
            }
        }
        res.json(result);
    
    } catch (error) {
        next(error);
    }
}

const addOSVersion = async (req, res, next) => {
    const transaction = await models.sequelize.transaction();
    try {

        const { 
            deviceOS, newOSVersion
        } = req.body;
        let result = newOSVersion.trim();
        const existingRecord = await models.OsVersions.findOne({
            where: {
                tagName: deviceOS,
                buildVersion: result
            }
        });

        if(existingRecord){
            throw new httpErrors.Conflict(`OS version already exists.`);
        }

        const createdRecord = await models.OsVersions.create({ 
            tagName: deviceOS,
            buildVersion: result,
            displayName: '',
            createdAt: moment(),
            updatedAt: moment()
        }, { 
            transaction
        });

        await transaction.commit();
        res.json('OS version created successfully.');

    } catch (error) {
        await transaction.rollback();
        next(error);
    }
} 

const deleteOSVersion = async (req, res, next) => {
    try {

        const { 
            versionId
        } = req.params;

        const recordExist = await models.OsVersions.findByPk(versionId);

        if(!recordExist){
            throw new httpErrors.NotFound(`OS version not found.`);
        }

        await recordExist.destroy()

        res.json('OS version deleted successfully.');

    } catch (error) {
        next(error);
    }
} 


module.exports = {
    getOsVersions,
    addOSVersion,
    deleteOSVersion
};
