var httpErrors = require('http-errors');
var moment = require('moment');
var models = require('../models');
const { actionType, publishServices } = require('../routes/common/event-publisher');
const { Op } = require('sequelize');
const { getServiceCreateMessage, getServiceEditMessage, getServiceDeleteMessage } = require('../config/adminLogs.config');
const { insertIntoAdminLogs } = require('../routes/common/adminLogs');
var ENUM = require('../enum/enum.js');

/* GET service listing. */
const getAllServices = async (req, res, next) => {
  try {
    ConnectionOptions = ENUM.ServiceConnectionOptions
    let {
      limit,
      offset,
      active,
      sort='id',
      dir='desc',
      is_persistent = 'false',
      search,
      all,
      type,
      protocol,
      except = [],
      selectedConnection = ConnectionOptions.All
    } =  req.query;

    let secure_services = await models.sequelize.query(`
      SELECT DISTINCT * FROM ((SELECT s.id FROM Services s
      JOIN ServicePolicies sp ON sp.service_id = s.id
      JOIN PolicyTypeLookup ptl ON ptl.id = sp.policy_type_id WHERE ptl.policy_type = 'Routing Policy')
      UNION (SELECT s.id FROM Services s
      JOIN GroupServices gs ON gs.serviceId = s.id
      JOIN ServiceGroupPolicies sgp ON sgp.service_group_id = gs.groupId
      JOIN PolicyTypeLookup ptl ON ptl.id = sgp.policy_type_id WHERE ptl.policy_type = 'Routing Policy')
      UNION (SELECT s.id  FROM Services s WHERE EXISTS (SELECT 1 FROM RoutingPolicies WHERE all_services = 1)))
      AS PolicyServices`);
    secure_services = [...new Set(secure_services[0].map(item => item.id))];

    let options = {
      where: { },
      limit: all === 'true'? null: limit,
      offset: all === 'true'? null: offset,
      order:[]
    };

    if (is_persistent == 'true') {
      options.where.is_persistent = 1;
    }

    if (selectedConnection == ConnectionOptions.Gateway) {
      options.where.id = { [Op.in]: secure_services };
    }

    else if (selectedConnection == ConnectionOptions.Direct) {
      options.where.id = { [Op.notIn]: secure_services };
    }

    if (active === 'isactive') {
      options.where.is_active = 1;
    }

    const searchTerm = search.trim();

    if (searchTerm) {
      options.where[Op.or] = [
        {
          Name: {
            [Op.like]: `%${searchTerm}%`
          }
        },
        {
          protocol: {
            [Op.like]: `%${searchTerm}%`
          }
        },
        {
          ipv4_address: {
            [Op.like]: `%${searchTerm}%`
          }
        },
        {
          type: {
            [Op.like]: `%${searchTerm}%`
          }
        }
      ];
    }

    if (type.trim() && type != 'null' && type != 'undefined') {
      options.where.type = type
    }

    if (protocol.trim() && protocol != 'null' && protocol != 'undefined') {
      options.where.protocol = protocol
    }

    let skip = []
    try {
        skip = JSON.parse(except)
    } catch(e) {
        console.log('error')
    }

    if(Array.isArray(skip) && skip.length > 0) {
        options.where['id'] = {
            [Op.not]: skip
        }
    }

    const allowedSortValues = [
      'createdAt',
      'protocol',
      'ipv4_address',
      'type',
      'id',
      'Name'
    ];

    if (allowedSortValues.includes(sort)) {
      options.order.push([models.sequelize.literal(sort), dir]);
    }

    if(all == 'true'){
      const temp = JSON.parse(JSON.stringify(options))
      delete temp['offset']
      delete temp['limit']
      let count = await models.Service.count(options)
      temp.limit = count
      temp.offset = 0
      options = {
        where: options.where,
        limit: temp.limit,
        offset: temp.offset,
        order: options.order
      }
    }
    let result  = await models.Service.paginate(options);
    result.data.forEach(service => {
      let serviceData = service.dataValues;
      serviceData.is_secure = secure_services.includes(serviceData.id);
    });
    res.json(result);
  } catch (error) {
    next(error);
  }
}

const getServices = async (req, res, next) => {
  try {
    const services = await models.Service.paginate({
      where: { },
      limit: -1
    });

    const response = {
      services: services.data,
      total: services.recordsTotal
    };

    res.json(response);
  } catch (error) {
    next(error);
  }
}

const getDevices = async (req, res, next) => {
  try {
    const result = await models.sequelize.query(
      `SELECT DISTINCT di.id, di.device_name
       FROM WorkloadDevicePivots wldp
       INNER JOIN DeviceInventories di
       ON wldp.deviceId = di.id
       INNER JOIN Clients c ON di.id = c.device_id
       WHERE c.client_status = '4'`);

    let devices = result[0].map(row => ({ id: row.id, device_name: row.device_name }));

    const response = {
      devices
    };

    res.json(response);
  } catch (error) {
    next(error);
  }
}

const getAllTypesAndProtocols = async (req, res, next) => {
  try {

    const typesResults = await models.sequelize.query(`SELECT DISTINCT type FROM Services`)
    const protocolsResults = await models.sequelize.query(`SELECT DISTINCT protocol FROM Services`)


    const protocols = protocolsResults[0].map(row => {
      return row.protocol
    })
    const types = typesResults[0].map(row => {
      return row.type
    })

    res.json({
      protocols,
      types
    })


  } catch (error) {
    next(error);
  }
}


const getallforgateway = async (req, res, next) => {
  try {

    const options = {
      where: {
        isGateWayAssigned: 0
      },
      limit: -1
    };

    const services = await models.Service.paginate(options);
    const response = {
      services: services.data,
      total: services.recordsTotal
    }
    res.json(response);

  } catch (error) {
    next(error);
  }
}


const associated = async (req, res, next) => {
  try {

    const options = {
      include: [
        {
          model: models.GatewayService,
          required: true,
          attributes: ['gatewayId', 'serviceId']
        }
      ],
      where: { },
      limit: -1
    };

    const services = await models.Service.paginate(options);

    const response = {
      services: services.data,
      total: services.recordsTotal
    }

    res.json(response);
  } catch (error) {
    next(error);
  }
}


const notassociated = async (req , res, next) => {
  try {
    const options = {
      exclude: [
        {
          model: models.GatewayService,
          required: true,
          attributes: ['gatewayId', 'serviceId']
        }
      ],
      where: { },
      limit: -1
    };

    const services = await models.Service.paginate(options);

    const response = {
      services: services.data,
      total: services.recordsTotal
    }

    res.json(response);

  } catch (error) {
    next(error);
  }
}


// Assign service to all
const assignToAll = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {

    var id = req.body.serviceId;
    let serviceID = [];

    const clietnIds = await groupsIdOfService(id);

    serviceID = clietnIds;

    const service = await models.Service.findOne({
      where: {
        id: id
      }
    });

    if (!service) {
      throw new httpErrors.NotFound("Error Occured");
    }

    const clientIdOfService = await clientIdOfService(id);

    serviceID = [...serviceID, clientIdOfService];

    await models.ClientService.destroy({
      where: {
        serviceId: service.id
      },
      transaction
    })

    const clients = await models.Client.findAll({
      where: { }
    });

    const clientServices = clients.map(client=>({
        clientId: client.id,
        serviceId: service.id
    }));

    await Promise.all([
      models.ClientService.bulkCreate(clientServices, {
        transaction
      }),
      models.Client.update({
        updatedAt: moment().utc().local().format('YYYY-MM-DD HH:mm:ss')
      }, {
        where: {
          id: {
            $notIn: serviceID
          }
        },
        transaction
      }),
      models.Service.update({
        isClientAssigned: true
      }, {
        where: {
          id: id
        },
        transaction
      })
    ]);

    await publishServices(actionType.UPDATED, {
      ids: [
        id
      ]
    });

    await transaction.commit();

    res.json({
      success: true,
      message: 'Service Assigned Successfully!'
    });

  } catch (error) {
    await transaction.rollback();
    next(error);
  }
}


const clientIdOfService = serviceId => {
  return models.ClientService.findAll({
    where: {
      serviceId: serviceId
    },
    attributes: ['clientId'],
    raw: true
  }).then(data => {
    return Promise.resolve(data.map(service => service.clientId));
  });
};


const groupsIdOfService = serviceId => {
  let groupIds;
  return models.GroupService.findAll({
    where: {
      serviceId: serviceId
    },
    raw: true,
    attributes: ['groupId']
  })
    .then(groupId => {
      groupId = groupId.map(id => id.groupId);
      console.log('group ids,', groupId);
      groupIds = groupId;
      return models.GroupClient.findAll({
        where: {
          groupId: {
            $in: groupIds
          }
        },
        raw: true,
        attributes: ['clientId']
      });
    })
    .then(clientIds => {
      clientIds = clientIds.map(id => id.clientId);
      console.log('clietn ids', clientIds);
      return Promise.resolve(clientIds);
    });
};


const updateClients = (serviceId, transaction = null) => {
  return groupsIdOfService(serviceId).then(clientIds => {
    var time = moment()
      .utc()
      .local()
      .format('YYYY-MM-DD HH:mm:ss');
    return models.Client.update(
      { updatedAt: time },
      {
        where: {
          id: {
            $in: clientIds
          }
        },
        transaction
      }
    );
  });
};

const gatewayIdOfService = serviceId => {
  return models.GatewayService.findOne({
    where: {
      serviceId: serviceId
    },
    attributes: ['gatewayId'],
    raw: true
  }).then(data => {
    return Promise.resolve(data ? data.gatewayId : undefined);
  });
};


// Un-assign service to all
const unassignToAll = async (req, res, next)=>{
  const transaction = await models.sequelize.transaction();
  try {

    let id = req.body.serviceId;

    const service = await models.Service.findOne({
      where: {
        id: id
      }
    });

    if (!service) {
      throw new httpErrors.NotFound('Service not found.');
    }

    const serviceID = await clientIdOfService(id);

    await Promise.all([
      models.ClientService.destroy({
        where: {
          serviceId: service.id
        },
        transaction
      }),
      models.Client.update({
        updatedAt: moment().utc().local().format('YYYY-MM-DD HH:mm:ss')
      },
        {
          where: {
            id: {
              $in: serviceID
            }
          },
          transaction
        }
      ),
      models.Service.update(
        {
          isClientAssigned: false
        },
        {
          where: {
          id: id
        },
          transaction
        }
      ),
      updateClients(id,transaction),
      models.GroupService.destroy({
        where: {
          serviceId: service.id
        },
        transaction
      })
    ]);

    await publishServices(actionType.UPDATED, {
      ids: [
        service.id
      ]
    });

    await transaction.commit();

    res.json({
      success: true,
      message: 'Service Un-Assigned Successfully!'
    });

  } catch (error) {
    await transaction.rollback();
    next(error);
  }
}

const getQuickServices = async (req, res, next) => {
  try {
      const { searchKey, except, is_persistent = 'false' } = req.query

      const options = {
        where: { },
        attributes: ['id', 'Name']
      };

      if (is_persistent == 'true') {
        options.where.is_persistent = 1;
      }

      const searchTerm = searchKey && searchKey.trim();

      if (searchTerm) {
        options.where[Op.or] = [
          {
            Name: {
              [Op.like]: `%${searchTerm}%`
            }
          },
          {
            protocol: {
              [Op.like]: `%${searchTerm}%`
            }
          },
          {
            ipv4_address: {
              [Op.like]: `%${searchTerm}%`
            }
          },
          {
            type: {
              [Op.like]: `%${searchTerm}%`
            }
          }
        ];
      }

      let skip = []
        try {
            skip = JSON.parse(except)
        } catch(e) {
            console.log('error')
        }

      if(Array.isArray(skip) && skip.length > 0) {
        options.where['id'] = {
            [Op.not]: skip
        }
      }

      const services = await models.Service.findAll(options)

      res.status(200).json({
        data: services
      })

  } catch (error) {
    next(error);
  }
}



const updateService = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const services = req.body.service;
    const time = moment().utc().local().format('YYYY-MM-DD HH:mm:ss');

    const [
      // gatewayId,
      serviceID
    ] = await Promise.all([
      // gatewayIdOfService(req.params.id),
      clientIdOfService(req.params.id)
    ]);

    const service = await models.Service.findOne({
      where: {
        id: req.params.id
      }
    });

    if (!service) {
      throw new httpErrors.NotFound('Service not found.');
    }

    if(service.is_persistent !== services.is_persistent) {
      const groups = await service.getGroupServices()
      if(groups && groups.length > 0)
        throw new httpErrors('Cannot modify persistent state of service because this service is part of a servicegroup.')
    }

    const isExist = await checkAlreadyExistName(services, true);

    if (isExist) {
      throw new httpErrors.Conflict('Service with this name already exist.');
    }

    const isExistWithURL = await checkAlreadyExistURL(services, true);

    if (isExistWithURL) {
      throw new httpErrors.Conflict('Mapped URL/Host is already assigned to a service');
    }

    if(services.port && services.port.split(',').length > 1 && services.proxy_enabled)
    {
      throw new httpErrors.PreconditionFailed('service could not have more than 1 port when proxy is enabled')
    }

    if(services.proxy_enabled == true){

        // let proxiedGatewaysAttached = `
        //               SELECT services.id as serviceId, services.Name as serviceName, services.isGateWayAssigned AS isGateway, services.is_active AS isActive, gateways.proxy_enabled ,
	      //              routingPolicies.id AS routingPolicyId, routingPolicies.name AS routingPolicyName,
	      //              routingPolicies.id as RoutingPoliciesId, gateways.id as gatewayId, gateways.Name as gatewayName
	      //           FROM Services services
	      //           INNER JOIN ServicePolicies servicePolicies on servicePolicies.service_id = services.id
	      //           INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = servicePolicies.policy_id
	      //           LEFT JOIN GatewayPolicies AS gatewayPolicies on gatewayPolicies.policy_id =  routingPolicies.id
	      //           INNER JOIN Gateways AS gateways on gateways.id = gatewayPolicies.gateway_id
	      //       	 WHERE  services.id = ${req.params.id}
	
	      //           UNION
	
	      //           -- Services with Gateways Groups -> Routing Policy
	      //           SELECT services.id as serviceId, services.Name as serviceName, services.isGateWayAssigned AS isGateway, services.is_active AS isActive, gateways.proxy_enabled,
	      //              routingPolicies.id AS routingPolicyId, routingPolicies.name AS routingPolicyName,
	      //              routingPolicies.id as RoutingPoliciesId, gateways.id as gatewayId, gateways.Name as gatewayName
	      //           FROM Services services
	      //           LEFT JOIN GroupServices AS groupServices ON groupServices.serviceId = services.id
	      //           LEFT JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
	      //           LEFT JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
	      //           INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = serviceGroupPolicies.policy_id
	      //           LEFT JOIN GatewayGroupPolicies AS gatewayGroupPolicies on gatewayGroupPolicies.policy_id  =  routingPolicies.id
	      //           LEFT JOIN GatewayPolicies AS gatewayPolicies on gatewayPolicies.policy_id =  routingPolicies.id
	      //           INNER JOIN Gateways AS gateways on gateways.id = gatewayPolicies.gateway_id
	      //           WHERE  services.id = ${req.params.id}`;

          let proxiedGatewaysAttached = `
           SELECT services.id as serviceId, services.Name as serviceName, services.isGateWayAssigned AS isGateway, services.is_active AS isActive,gateways.proxy_enabled,
                   routingPolicies.id AS routingPolicyId, routingPolicies.name AS routingPolicyName,
                   routingPolicies.id as RoutingPoliciesId, gateways.id as gatewayId, gateways.Name as gatewayName
                FROM Services services
                INNER JOIN GroupServices AS groupServices ON groupServices.serviceId = services.id
                INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
                INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
                INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = serviceGroupPolicies.policy_id
                INNER JOIN GatewayGroupPolicies AS gatewayGroupPolicies on gatewayGroupPolicies.policy_id  =  routingPolicies.id
                INNER JOIN GatewayGroups AS gatewayGroups on gatewayGroups.id = gatewayGroupPolicies.gateway_group_id
                INNER JOIN GatewayGroupPivots gatewayGroupPivots on gatewayGroupPivots.gateway_group_id = gatewayGroups.id
                INNER JOIN Gateways AS gateways on gateways.id = gatewayGroupPivots.gateway_id
				WHERE services.id = ${req.params.id}
                UNION

                -- Services with Gateways Groups -> Routing Policy
                SELECT services.id as serviceId, services.Name as serviceName, services.isGateWayAssigned AS isGateway, services.is_active AS isActive,gateways.proxy_enabled, 
                   routingPolicies.id AS routingPolicyId, routingPolicies.name AS routingPolicyName,
                   routingPolicies.id as RoutingPoliciesId, gateways.id as gatewayId, gateways.Name as gatewayName
                FROM Services services
                INNER JOIN ServicePolicies servicePolicies on servicePolicies.service_id = services.id
                INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = servicePolicies.policy_id
                INNER JOIN GatewayGroupPolicies AS gatewayGroupPolicies on gatewayGroupPolicies.policy_id  =  routingPolicies.id
                INNER JOIN GatewayGroups AS gatewayGroups on gatewayGroups.id = gatewayGroupPolicies.gateway_group_id
                INNER JOIN GatewayGroupPivots gatewayGroupPivots on gatewayGroupPivots.gateway_group_id = gatewayGroups.id
                INNER JOIN Gateways AS gateways on gateways.id = gatewayGroupPivots.gateway_id
				WHERE services.id = ${req.params.id}
                UNION

                 -- Services with Gateways -> Routing Policy
                SELECT services.id as serviceId, services.Name as serviceName, services.isGateWayAssigned AS isGateway, services.is_active AS isActive,services.proxy_enabled, 
                   routingPolicies.id AS routingPolicyId, routingPolicies.name AS routingPolicyName,
                   routingPolicies.id as RoutingPoliciesId, gateways.id as gatewayId, gateways.Name as gatewayName
                FROM Services services
                INNER JOIN ServicePolicies servicePolicies on servicePolicies.service_id = services.id
                INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = servicePolicies.policy_id
                INNER JOIN GatewayPolicies AS gatewayPolicies on gatewayPolicies.policy_id =  routingPolicies.id
                INNER JOIN Gateways AS gateways on gateways.id = gatewayPolicies.gateway_id
				WHERE services.id = ${req.params.id}
                UNION

                -- Services with Gateways Groups -> Routing Policy
                SELECT services.id as serviceId, services.Name as serviceName, services.isGateWayAssigned AS isGateway, services.is_active AS isActive,gateways.proxy_enabled, 
                   routingPolicies.id AS routingPolicyId, routingPolicies.name AS routingPolicyName,
                   routingPolicies.id as RoutingPoliciesId, gateways.id as gatewayId, gateways.Name as gatewayName
                FROM Services services
                INNER JOIN GroupServices AS groupServices ON groupServices.serviceId = services.id
                INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
                INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
                INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = serviceGroupPolicies.policy_id
                INNER JOIN GatewayGroupPolicies AS gatewayGroupPolicies on gatewayGroupPolicies.policy_id  =  routingPolicies.id
                INNER JOIN GatewayPolicies AS gatewayPolicies on gatewayPolicies.policy_id =  routingPolicies.id
                INNER JOIN Gateways AS gateways on gateways.id = gatewayPolicies.gateway_id
	            WHERE services.id = ${req.params.id}`

           const gatewaysToAttachedResult = (
              await models.sequelize.query(proxiedGatewaysAttached)
              )[0];

              const hasProxiedService = gatewaysToAttachedResult.some(
                (row) => row.proxy_enabled === 0
              );
              
              if (hasProxiedService) {
                throw new httpErrors(`Non-proxied gateways cannot be assigned to proxied services.`);
              }
           
              console.log('gatewaysToAttachedResult :>> ', gatewaysToAttachedResult);
           if (hasProxiedService) {
            throw new httpErrors(`proxied services can not be assigned to non-proxied gateways`);
        }
    }

    await Promise.all([
      service.update(services, {
        transaction
      }),
      models.Client.update(
        {
          updatedAt: time
        },
        {
          where: {
            id: {
              $in: serviceID
            }
          },
          transaction
        }
      ),
      // models.Gateway.update(
      //   {
      //     updatedAt: time
      //    },
      //   {
      //     where: {
      //       id: gatewayId
      //     },
      //     transaction
      //   }
      // ),
      updateClients(req.params.id, transaction)
    ]);

    // delete existing endpoints and create again from the incoming payload
    await models.AllowedServiceEndpoints.destroy({
      where: { service_id: service.id },
      transaction: transaction
    });

    const endpoints = services.endpoints;

    for (let i = 0; i < endpoints.length; i++) {
      await models.AllowedServiceEndpoints.create(
        {
          endpoint: endpoints[i],
          service_id: service.id
        },
        { transaction: transaction }
      );
    }

    await publishServices(actionType.UPDATED, {
      ids: [
        service.id
      ]
    });

    await transaction.commit();
    insertIntoAdminLogs(
      getServiceEditMessage(service.Name),
      req
    )
    res.json({
      success: true,
      message: 'Update Successfully'
    });

  } catch (error) {
    await transaction.rollback();
    next(error);
  }
}


/* Get a single service */
const getService = async (req, res, next) => {
  try {

    const options = {
      where: {
        id: req.params.id
      }
    }

    const service = await models.Service.findAll(options);
    
    // fetch linked endpoints
    const queryForAllowedServiceEndpoints = `
      SELECT ase.endpoint, ase.service_id 
      FROM AllowedServiceEndpoints ase 
      INNER JOIN Services s ON s.id = ase.service_id 
      WHERE ase.service_id = :service_id
    `;

    let endpoints = (
      await models.sequelize.query(queryForAllowedServiceEndpoints, {
          replacements: { service_id: req.params.id },
          type: models.sequelize.QueryTypes.SELECT,
        })
    )

    const arrayOfEndpoints = endpoints.map(endpointObject => endpointObject.endpoint);

    service[0].dataValues.endpoints = arrayOfEndpoints;

    res.json(service);

  } catch (error) {
    next(error);
  }
}

/* Get a single service detail */
const getServiceDetail = async (req, res, next) => {
  try {

    const queryForServiceGroups = `
    SELECT serviceGroup.id , serviceGroup.name, serviceGroup.description, serviceGroup.is_persistent,
    services.id, services.Name 
    FROM ServiceGroups serviceGroup
    LEFT JOIN GroupServices groupServices On groupServices.groupId = serviceGroup.id
    LEFT JOIN Services services ON services.id = groupServices.serviceId 
    WHERE services.id = :services_id
    `
    const resultForServiceGroups = (
          await models.sequelize.query(queryForServiceGroups, {
            replacements: { services_id: req.params.id },
            type: models.sequelize.QueryTypes.SELECT,
            raw: false
          })
    );

    const queryForRoutingPolicy=`
    (
      SELECT services.id, services.Name, services.is_gateway_less  AS isGateway, services.is_active AS isActive,
         routingPolicies.id AS RoutingPolicyGroupId, routingPolicies.name AS RoutingPolicyGroupName,
         routingPolicies.id as RoutingPoliciesId, gateways.id as gatewayId, gateways.Name as gatewayName
      FROM Services services
      LEFT JOIN GroupServices AS groupServices ON groupServices.serviceId = services.id
      LEFT JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
      LEFT JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
      INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = serviceGroupPolicies.policy_id
      LEFT JOIN GatewayGroupPolicies AS gatewayGroupPolicies on gatewayGroupPolicies.policy_id  =  routingPolicies.id
      LEFT JOIN GatewayGroups AS gatewayGroups on gatewayGroups.id = gatewayGroupPolicies.gateway_group_id
      LEFT JOIN GatewayGroupPivots gatewayGroupPivots on gatewayGroupPivots.gateway_group_id = gatewayGroups.id
      LEFT JOIN Gateways AS gateways on gateways.id = gatewayGroupPivots.gateway_id
      where services.id = :services_id
    )
    UNION 
    (
      SELECT services.id, services.Name, services.isGateWayAssigned AS isGateway, services.is_active AS isActive,
         routingPolicies.id AS RoutingPolicyGroupId, routingPolicies.name AS RoutingPolicyGroupName,
         routingPolicies.id as RoutingPoliciesId, gateways.id as gatewayId, gateways.Name as gatewayName
      FROM Services services
      INNER JOIN ServicePolicies servicePolicies on servicePolicies.service_id = services.id 
      LEFT JOIN RoutingPolicies routingPolicies ON routingPolicies.id = servicePolicies.policy_id
      LEFT JOIN GatewayGroupPolicies AS gatewayGroupPolicies on gatewayGroupPolicies.policy_id  =  routingPolicies.id
      LEFT JOIN GatewayGroups AS gatewayGroups on gatewayGroups.id = gatewayGroupPolicies.gateway_group_id
      LEFT JOIN GatewayGroupPivots gatewayGroupPivots on gatewayGroupPivots.gateway_group_id = gatewayGroups.id
      LEFT JOIN Gateways AS gateways on gateways.id = gatewayGroupPivots.gateway_id
      where services.id = :services_id
    )
    UNION 
    (
      SELECT services.id, services.Name, services.isGateWayAssigned AS isGateway, services.is_active AS isActive,
         routingPolicies.id AS RoutingPolicyGroupId, routingPolicies.name AS RoutingPolicyGroupName,
       routingPolicies.id as RoutingPoliciesId, gateways.id as gatewayId, gateways.Name as gatewayName
      FROM Services services
      LEFT JOIN ServicePolicies servicePolicies on servicePolicies.service_id = services.id 
      INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = servicePolicies.policy_id
      LEFT JOIN GatewayPolicies AS gatewayPolicies on gatewayPolicies.policy_id =  routingPolicies.id
      LEFT JOIN Gateways AS gateways on gateways.id = gatewayPolicies.gateway_id 
      where services.id = :services_id
    )`

    const resultForRoutingPolicy = (
      await models.sequelize.query(queryForRoutingPolicy, {
        replacements: { services_id: req.params.id },
        type: models.sequelize.QueryTypes.SELECT,
        raw: false
      })
    );

    const queryForUserEntitlements = `
    (
      SELECT clientPolicies.policy_id , clientPolicies.client_id , clients.user as clientUserName, clients.Name as ClientName,
      localUsers.name as localName, localUsers.identity_provider, localUsers.id as localid, localUsers.userName as localusername,
      deviceInventories.device_name, deviceInventories.device_id, deviceInventories.device_type, deviceInventories.os , deviceInventories.inventory_sync_state,
      entitlements.access_restriction AS isRestricted, entitlements.name 
      FROM ClientPolicies clientPolicies  
      INNER JOIN Entitlements entitlements ON entitlements.id = clientPolicies.policy_id
      INNER JOIN Clients clients ON clients.id = clientPolicies.client_id 
      INNER JOIN LocalUsers localUsers ON localUsers.id = clients.user_id 
      INNER JOIN DeviceInventories deviceInventories ON deviceInventories.id = clients.device_id 
      WHERE clientPolicies.policy_type_id  = 1 AND entitlements.all_services = 1
    )
    UNION
    (
      SELECT clientPolicies.policy_id , clientPolicies.client_id , clients.user as clientUserName, clients.Name as ClientName,
      localUsers.name as localName, localUsers.identity_provider, localUsers.id as localid, localUsers.userName as localusername,
      deviceInventories.device_name, deviceInventories.device_id, deviceInventories.device_type, deviceInventories.os , deviceInventories.inventory_sync_state,
      entitlements.access_restriction AS isRestricted, entitlements.name 
      FROM Services services
      LEFT JOIN ServicePolicies servicePolicies ON servicePolicies.service_id = services.id
      LEFT JOIN ClientPolicies clientPolicies ON clientPolicies.policy_id = servicePolicies.policy_id 
      INNER JOIN Entitlements entitlements ON entitlements.id = clientPolicies.policy_id
      INNER JOIN Clients clients ON clients.id = clientPolicies.client_id
      INNER JOIN LocalUsers localUsers ON localUsers.id = clients.user_id 
      INNER JOIN DeviceInventories deviceInventories ON deviceInventories.id = clients.device_id 
      WHERE clientPolicies.policy_type_id  = 1 AND services.id = :services_id
    )
    UNION
    (
      SELECT clientPolicies.policy_id , clientPolicies.client_id , clients.user as clientUserName, clients.Name as ClientName,
      localUsers.name as localName, localUsers.identity_provider, localUsers.id as localid, localUsers.userName as localusername,
      deviceInventories.device_name, deviceInventories.device_id, deviceInventories.device_type, deviceInventories.os , deviceInventories.inventory_sync_state,
      entitlements.access_restriction AS isRestricted, entitlements.name 
      FROM Services services
      JOIN GroupServices groupServices ON groupServices.serviceId = services.id
      JOIN ServiceGroupPolicies serviceGroupPolicies ON serviceGroupPolicies.service_group_id = groupServices.groupId
      LEFT JOIN ClientPolicies clientPolicies ON clientPolicies.policy_id = serviceGroupPolicies.policy_id 
      INNER JOIN Entitlements entitlements ON entitlements.id = clientPolicies.policy_id
      INNER JOIN Clients clients ON clients.id = clientPolicies.client_id 
      INNER JOIN LocalUsers localUsers ON localUsers.id = clients.user_id 
      INNER JOIN DeviceInventories deviceInventories ON deviceInventories.id = clients.device_id 
      WHERE clientPolicies.policy_type_id  = 1 AND services.id = :services_id
    )
    `
    const resultForUserEntitlements = (
      await models.sequelize.query(queryForUserEntitlements, {
        replacements: { services_id: req.params.id },
        type: models.sequelize.QueryTypes.SELECT,
        raw: false
      })
    );

    res.json({
      serviceGroups: resultForServiceGroups,
      serviceGroupsTotal : resultForServiceGroups?.length || 0,
      routingPolicy:resultForRoutingPolicy,
      routingPolicyTotal : resultForRoutingPolicy?.length || 0,
      userEntitlements: resultForUserEntitlements,
      userEntitlementsTotal: resultForUserEntitlements?.length || 0,
    })

  } catch (error) {
    next(error);
  }
}

function checkAlreadyExistName(service, isEdit) {
  let whereClause = {
    Name: service.Name
  };
  if (isEdit) {
    whereClause.id = {
      $ne: service.id
    };
  }
  return new Promise((resolve, reject) => {
    models.Service.findOne({
      where: whereClause
    }).then(service => {
      if (service) {
        resolve(true);
      } else {
        resolve(false);
      }
    });
  });
}

async function checkAlreadyExistURL(service, isEdit) {
  let whereClause = {};
  if(service.mappedurl !== 'None'){
    whereClause = {
      mappedurl: service.mappedurl
    };
    if (isEdit) {
      whereClause.id = {
        $ne: service.id
      };
    }
    let data = await models.Service.findOne({
      where: whereClause
    })
    if(data){
      return true;
    }else{
      return false;
    }
  }else{
    return false;
  }
}


// add new service
const addService = async (req, res, next) => {
  const transaction = await models.sequelize.transaction();
  try {
    const { service } = req.body;
    const { id } = req.user;

    service['createdById'] = id;

    const isAlreadyExist = await checkAlreadyExistName(service, false);

    if(isAlreadyExist){
      throw new httpErrors.Conflict('Service with this name already exist.');
    }

    const isExistWithURL = await checkAlreadyExistURL(service, false);

    if (isExistWithURL) {
      throw new httpErrors.Conflict('Mapped URL/Host is already assigned to a service');
    }

    if(service.port && service.port.split(',').length > 1 && service.proxy_enabled)
    {
      throw new httpErrors.PreconditionFailed('service could not have more than 1 port when proxy is enabled')
    }

    const createdService = await models.Service.create({
      ...service,
      service_id: 0
    }, {
      transaction
    });

    createdService.service_id = createdService.id;
    createdService.createdById = id;

    await createdService.save({
      transaction
    });

    const endpoints = service.endpoints;

    for (let i = 0; i < endpoints.length; i++) {
      await models.AllowedServiceEndpoints.create(
        {
          endpoint: endpoints[i],
          service_id: createdService.id
        },
        { transaction: transaction }
      );
    }

    insertIntoAdminLogs(
      getServiceCreateMessage(service.Name),
      req
    )

    await publishServices(actionType.ADDED, {
      ids: [
        createdService.id
      ]
    });

    await transaction.commit();

    res.json({
      success: true,
      message: 'Service added Successfully.'
    });

  } catch (error) {
    await transaction.rollback();
    next(error);
  }
}


// delete a single service
const deleteService = async (req, res, next) => {
  try {
    const GatewayService = await models.GatewayService.findOne({
      where: {
        serviceId: req.params.id
      }
    });

    if (GatewayService) {
      throw new httpErrors.BadRequest('Cannot delete Service which is associated with Gateway');
    }

    const service = await models.Service.findOne({
      where: {
        id: req.params.id
      }
    });

    if(!service){
      throw new httpErrors.NotFound('Service not found.');
    }

    await service.destroy({ });

    await publishServices(actionType.DELETED, {
      ids: [
        service.id
      ]
    });
    insertIntoAdminLogs(
      getServiceDeleteMessage(service.Name),
      req
    )

    res.status(200).json({
      success: true,
      message: 'Deleted Successfully'
    });

  } catch (error) {
    next(error);
  }
}


deleteBulkServices = async (req, res, next) => {

    try {
        const {
          selectAll,
          selectedIds,
          search = '',
          is_persistent = false
        } = req.body;

        const options = {
          where: {
            [Op.and]: {
              isGateWayAssigned: false,
              ...is_persistent? { is_persistent }: {},
              [Op.or]: [
                {
                  Name: {
                    [Op.like]: `%${search.trim()}%`
                  }
                },
                {
                  protocol: {
                    [Op.like]: `%${search.trim()}%`
                  }
                },
                {
                  ipv4_address: {
                    [Op.like]: `%${search.trim()}%`
                  }
                },
                {
                  type: {
                    [Op.like]: `%${search.trim()}%`
                  }
                }
              ]
            }
          }
        }

        options.where[Op.and].id =  selectAll ? {
          [Op.notIn]: selectedIds
        } : {
          [Op.in]: selectedIds
        };

        const affectingServices = await models.Service.findAll({
          attributes: ['id'],
          where: options.where
        });

        const response = await models.Service.destroy(options);

        const ids = affectingServices.map(_service => _service.id);

        await publishServices(actionType.DELETED, {
          ids
        });

        return res.status(200).send({ message: 'Service(s) deleted successfully.'});

      } catch (error) {
        next(new httpErrors.InternalServerError('Something went wrong in deleting service(s).'));
    }
}



/**
 * Available services
 * @param {Request} req
 * @param {Response} res
 * @param {Function} next
 * @return {Object}
 */
const servicesAvailable = async (req, res, next) => {
  const { id } = req.params;
  const { force } =  req.query;
  try {
    if(force !== 'true')
    {
      const count = await models.GatewayGroups.count()

      if(count) {
        return res.json([]);
      }
    }

    const services = await models.Service.findAll({
      attributes: ['id', 'Name']
    });

    return res.json(services);
  } catch (error) {
    next(error);
  }
};

module.exports = {
    deleteBulkServices,
    getAllTypesAndProtocols,
    deleteService,
    addService,
    getService,
    notassociated,
    associated,
    getallforgateway,
    getServices,
    getDevices,
    getAllServices,
    assignToAll,
    unassignToAll,
    updateService,
    servicesAvailable,
    getQuickServices,
    getServiceDetail
};