'use strict';
var common = require('./common'),
  _ = require('lodash'),
  Sequelize = require('sequelize');
var schema = {};
_.assign(schema, common, {
  Name: {
    type: Sequelize.STRING(50),
    allowNull: false
  },
  agent_id: {
    type: Sequelize.INTEGER,
    allowNull: false
  },
  is_active: {
    type: Sequelize.BOOLEAN,
    defaultValue: true
  },
  ipv4_address: {
    type: Sequelize.STRING(50),
    allowNull: true
  },
  user: {
    type: Sequelize.STRING(50)
  },
  enc_key: {
    type: Sequelize.STRING,
    allowNull: false
  },
  HMacKey: {
    type: Sequelize.STRING,
    allowNull: false
  },
  is_deleted: {
    type: Sequelize.BOOLEAN,
    defaultValue: false
  },
  createdAt: {
    allowNull: false,
    type: Sequelize.DATE
  },
  updatedAt: {
    allowNull: false,
    type: Sequelize.DATE
  },
  ENC_KEY_LEN:{
    defaultValue: 32,
    type: Sequelize.INTEGER
  },
  HMAC_KEY_LEN:{
    defaultValue: 24,
    type: Sequelize.INTEGER
  },
  Email:{
    type: Sequelize.STRING(50)
  },
  device_os:{
    type: Sequelize.STRING
  },
  device_hardware_id:{
    type: Sequelize.STRING
  },
  cacert:{
    type:Sequelize.BLOB('medium')
  },
  clientcrt:{
    type:Sequelize.BLOB('medium')
  },
  clientpem:{
    type:Sequelize.BLOB('medium')
  },
  link_key:{
    type: Sequelize.STRING
  },
  link_key_expiry:{
    type: Sequelize.DATE
  },
  heart_beat:{
    type: Sequelize.BOOLEAN,
    defaultValue: false
  },
  heart_beat_sent_at:{
    type: Sequelize.DATE
  },
  access_restrictions:{
    type: Sequelize.BOOLEAN
  },
  installer_status:{
    type: Sequelize.STRING(50),
    allowNull: true,
    defaultValue: 'new'
  },
  cacer:{
    type:Sequelize.BLOB('medium')
  },
  clientcer:{
    type:Sequelize.BLOB('medium')
  },
  clientp12:{
    type:Sequelize.BLOB('medium')
  },
  counter:{
    type: Sequelize.STRING(50),
    allowNull: true,
    defaultValue: '1'
  },
  installer_logs:{
    type: Sequelize.TEXT('long'),
    allowNull: true
  },
  isDeviceIdSet:{
    type: Sequelize.BOOLEAN,
    allowNull: true,
    defaultValue: false
  },
  package_sent_at:{
    type: Sequelize.DATE
  },
  client_darkening:{
    type: Sequelize.BOOLEAN,
    allowNull: true,
    defaultValue: true
  },
  geoip_track:{
    type: Sequelize.BOOLEAN,
    allowNull: true,
    defaultValue: true
  },
  package_used:{
    type: Sequelize.BOOLEAN,
    defaultValue: true
  },
  qr_json:{
    type: Sequelize.TEXT('long'),
    allowNull: true
  },
  onPrenDownloadLink:{
    type: Sequelize.STRING,
    defaultValue: 'N/A'
  },
  isDownloadPackage:{
    type: Sequelize.BOOLEAN,
    allowNull: true,
    defaultValue: false
  },
  packageDownloaded:{
    type: Sequelize.BOOLEAN,
    allowNull: true,
    defaultValue: false
  },
  client_type:{
    type: Sequelize.STRING(50)
  },
  isCancelled:{
    type: Sequelize.BOOLEAN,
    defaultValue: 0
  },
  is_ldap_deleted:{
    type: Sequelize.BOOLEAN,
    defaultValue: 0
  },
  client_status:{
    type: Sequelize.BOOLEAN,
    defaultValue: 1
  },
  client_mfa:{
    type: Sequelize.BOOLEAN,
    defaultValue: 0
  },
  device_os_full_name:{
    type: Sequelize.STRING,
    defaultValue: ''
  },
  domain_name:{
    type: Sequelize.STRING,
    defaultValue: ''
  },
  serial_number:{
    type: Sequelize.STRING,
    defaultValue: ''
  },
  antivirus_installed:{
    type: Sequelize.STRING,
    defaultValue: ''
  },
  linOTP_token:{
    type:Sequelize.JSON
  },
  OTP_image_data:{
    type:Sequelize.TEXT('long')
  },
  original_Email:{
    type: Sequelize.STRING(50),
    nullable: true
  },
  is_local_deleted:{
    type: Sequelize.INTEGER,
    defaultValue: 0
  },
  is_local:{
    type: Sequelize.BOOLEAN,
    defaultValue: 0
  },
  client_av:{
    type: Sequelize.STRING,
    defaultValue: 'no'
  },
  client_os_version:{
    type: Sequelize.STRING,
    defaultValue: 'any'
  },
  waiting_reason:{
    defaultValue: null,
    type: Sequelize.TEXT('long')
  },
  recieved_os_version:{
    defaultValue: '',
    type: Sequelize.STRING
  },
  recieved_antivirus:{
    defaultValue: '',
    type: Sequelize.STRING
  },
  biometric_state:{
    type: Sequelize.BOOLEAN,
    defaultValue: false
  },
  client_epp:{
    type: Sequelize.BOOLEAN,
    defaultValue: false
  },
  recieved_epp:{
    defaultValue: '',
    type: Sequelize.STRING
  },
  wrong_attempts:{
    defaultValue:0,
    type:Sequelize.INTEGER
  },
  is_strong_mfa:{
    type: Sequelize.BOOLEAN,
    defaultValue: 0
  },
  client_version:{
    type: Sequelize.STRING,
    defaultValue: null
  },
  mfa_status:{
    type: Sequelize.STRING(50),
    defaultValue: 'unregistered'
  },
  auto_updates:{
    type: Sequelize.BOOLEAN,
    allowNull: false,
    defaultValue: true
  },
 risk_score:{
    type: Sequelize.INTEGER,
    allowNull: false,
    defaultValue: 1
  },
  device_id:{
    type: Sequelize.INTEGER
  },
  user_id:{
    type: Sequelize.INTEGER
  },
  os_tag:{
    type: Sequelize.STRING(50),
    defaultValue: null
  },
  inventory_sync_state: {
    type: Sequelize.STRING(50),
    defaultValue: 'Undetected'
  }
});

module.exports = {
  up: function(queryInterface, Sequelize) {
    return queryInterface.createTable('Clients', schema);
  },
  down: function(queryInterface, Sequelize) {
    return queryInterface.dropTable('Clients');
  }
};
