'use strict';
module.exports = {
  up: function(queryInterface, Sequelize) {
    return queryInterface.createTable('Settings', {
      id: {
        allowNull: false,
        autoIncrement: true,
        primaryKey: true,
        type: Sequelize.INTEGER
      },
      smtp_service: {
        type: Sequelize.STRING(100),
        allowNull: true
      },
      smtp_user: {
        type: Sequelize.STRING(50),
        allowNull: true
      },
      smtp_password: {
        type: Sequelize.STRING,
        allowNull: true
      },
      smtp_from: {
        type: Sequelize.STRING,
        allowNull: true
      },
      package_link_expiry: {
        type: Sequelize.INTEGER,
        defaultValue: 60
      },
      session_expiry: {
        type: Sequelize.INTEGER,
        defaultValue: 30
      },
      purge_allowed: {
        type: Sequelize.BOOLEAN,
        defaultValue: true
      },
      saas_allowed: {
        type: Sequelize.BOOLEAN,
        defaultValue: true
      },
      createdAt: {
        allowNull: true,
        type: Sequelize.DATE
      },
      updatedAt: {
        allowNull: true,
        type: Sequelize.DATE
      },
      isSettingsSet: {
        type: Sequelize.BOOLEAN,
        allowNull: true,
        defaultValue: false
      },
      last_purge_date: {
        type: Sequelize.DATE,
        allowNull: true
      },
      auto_purge_days: {
        type: Sequelize.INTEGER,
        allowNull: true,
        defaultValue: 30
      },
      date_format: {
        type: Sequelize.STRING,
        allowNull: true,
        defaultValue: 'dd/MM/yyyy HH:mm'
      },
      gapiKey: {
        type: Sequelize.STRING,
        allowNull: true,
        defaultValue: 'GOOGLEMAPAPIKEY'
      },
      password_expiry_days: {
        type: Sequelize.INTEGER,
        allowNull: true,
        defaultValue: 15
      },
      smtp_host: {
        type: Sequelize.STRING,
        allowNull: true,
        defaultValue: 'SMTPHOST'
      },
      smtp_port: {
        type: Sequelize.INTEGER,
        allowNull: true,
        defaultValue: '587'
      },
      isFirstLogin: {
        type: Sequelize.BOOLEAN,
        allowNull: true,
        defaultValue: true
      },
      mfa_server: {
        type: Sequelize.STRING
      },
      biometrics_state: {
        type: Sequelize.BOOLEAN,
        defaultValue: false
      },
      dirsync_enabled: {
        type: Sequelize.BOOLEAN,
        defaultValue: false
      },
      dirsync_time: {
        type: Sequelize.INTEGER,
        defaultValue: 30
      },
      dirclean_time: {
        type: Sequelize.INTEGER,
        defaultValue: 60
      },
      is_ldap_syncing: {
        type: Sequelize.BOOLEAN,
        defaultValue: false
      },
      last_ldap_sync_start_time: {
        type: Sequelize.DATE
      },
      last_ldap_sync_end_time: {
        type: Sequelize.DATE
      },
      admin_mfa: {
        defaultValue: false,
        type: Sequelize.BOOLEAN
      },
      max_clients_per_controller: {
        defaultValue: 10,
        type: Sequelize.INTEGER
      },
      siem_host: {
        type: Sequelize.STRING,
        defaultValue: '0.0.0.0'
      },
      siem_port: {
        type: Sequelize.INTEGER,
        defaultValue: 1
      },
      siem_protocol: {
        type: Sequelize.STRING,
        defaultValue: 'tls'
      },
      health_status: {
        defaultValue:900,
        type: Sequelize.INTEGER
      },
      services_access_logs: {
        defaultValue:3600,
        type: Sequelize.INTEGER
      },
      load_stats: {
        defaultValue:5,
        type: Sequelize.INTEGER
      },
      core_auto_update: {
        type: Sequelize.BOOLEAN
      },
      in_activity_timeout:{
        type: Sequelize.INTEGER,
        defaultValue: 150
      },
      mtls_frequency:{
        type: Sequelize.INTEGER,
        defaultValue: 1000
      },
      mtls_blocking_timeout:{
        type: Sequelize.INTEGER,
        defaultValue: 10
      },
      health_interval:{
        type: Sequelize.INTEGER,
        defaultValue: 1
      },
      load_stats_interval: {
        type: Sequelize.INTEGER,
        defaultValue: 60
      },
      device_validation_freq: {
        type: Sequelize.INTEGER,
        defaultValue: 300
      },
      recaptcha_site_key: {
        type: Sequelize.STRING,
        allowNull: true,
        defaultValue: 'RECAPTCHASITEKEY'
      }
    });
  },
  down: function(queryInterface, Sequelize) {
    return queryInterface.dropTable('Settings');
  }
};
