'use strict';
module.exports = {
  up: (queryInterface, Sequelize) => {
    return Promise.all([
      queryInterface.createTable('Entitlements', {
        id: {
          allowNull: false,
          autoIncrement: true,
          primaryKey: true,
          type: Sequelize.INTEGER
        },
        name: {
          type: Sequelize.STRING(50)
        },
        access_restriction: {
          type: Sequelize.BOOLEAN,
          defaultValue: false
        },
        access_start_time: {
          type: Sequelize.TIME
        },
        access_end_time: {
          type: Sequelize.TIME
        },
        months: {
          type: Sequelize.STRING(50)
        },
        days: {
          type: Sequelize.STRING(50)
        },
        is_enabled: {
          type: Sequelize.BOOLEAN,
          defaultValue: true,
        },
        all_clients: {
          type: Sequelize.BOOLEAN,
          defaultValue: false,
        },
        all_services: {
          type: Sequelize.BOOLEAN,
          defaultValue: false,
        },
        createdAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        },
        updatedAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        },
        policy_type: {
          type: Sequelize.STRING(50)
        }
      }),
      queryInterface.createTable('ServicePolicies', {
        id: {
          allowNull: false,
          autoIncrement: true,
          primaryKey: true,
          type: Sequelize.INTEGER
        },
        policy_id: {
          type: Sequelize.INTEGER
        },
        service_id: {
          type: Sequelize.INTEGER
        },
        createdAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        },
        updatedAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        }
      }),
      queryInterface.createTable('ServiceGroupPolicies', {
        id: {
          allowNull: false,
          autoIncrement: true,
          primaryKey: true,
          type: Sequelize.INTEGER
        },
        policy_id: {
          type: Sequelize.INTEGER
        },
        service_group_id: {
          type: Sequelize.INTEGER
        },
        createdAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        },
        updatedAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        }
      }),
      queryInterface.createTable('ClientPolicies', {
        id: {
          allowNull: false,
          autoIncrement: true,
          primaryKey: true,
          type: Sequelize.INTEGER
        },
        policy_id: {
          type: Sequelize.INTEGER
        },
        client_id: {
          type: Sequelize.INTEGER
        },
        createdAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        },
        updatedAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        }
      }),
      queryInterface.createTable('ClientGroupPolicies', {
        id: {
          allowNull: false,
          autoIncrement: true,
          primaryKey: true,
          type: Sequelize.INTEGER
        },
        policy_id: {
          type: Sequelize.INTEGER
        },
        client_group_id: {
          type: Sequelize.INTEGER
        },
        createdAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        },
        updatedAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        }
      })
    ])
  },
  down: (queryInterface, Sequelize) => {
    return Promise.all([
      queryInterface.dropTable('Entitlements'),
      queryInterface.dropTable('ServicePolicies'),
      queryInterface.dropTable('ServiceGroupPolicies'),
      queryInterface.dropTable('ClientPolicies'),
      queryInterface.dropTable('ClientGroupPolicies')
    ])
  }
};
