'use strict';
module.exports = {
  up: (queryInterface, Sequelize) => {
    return queryInterface.sequelize.transaction(function (t) {
      return Promise.all([
        queryInterface.sequelize.query('Delete from DevicePolicies', {transaction: t}),
        queryInterface.addColumn('DevicePolicies', 'all_clients', {
          type: Sequelize.BOOLEAN,
          defaultValue: false
        }, {transaction: t}),
        queryInterface.addColumn('DevicePolicies', 'all_services', {
          type: Sequelize.BOOLEAN,
          defaultValue: false
        }, {transaction: t}),

        queryInterface.addColumn('DevicePolicies', 'name', {
          type: Sequelize.STRING(50),
        }, {transaction: t}),
        queryInterface.createTable('ServiceDevicePolicies', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          device_policy_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'DevicePolicies',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          service_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'Services',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, {transaction: t}),
        queryInterface.createTable('ServiceGroupDevicePolicies', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          device_policy_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'DevicePolicies',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          service_group_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'ServiceGroups',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, {transaction: t}),
        queryInterface.createTable('ClientDevicePolicies', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          device_policy_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'DevicePolicies',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          client_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'Clients',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, {transaction: t}),
        queryInterface.createTable('ClientGroupDevicePolicies', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          device_policy_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'DevicePolicies',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          client_group_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'Groups',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, {transaction: t}),
        queryInterface.createTable('ServiceDevicePolicies', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          device_policy_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'DevicePolicies',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          service_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'Services',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, {transaction: t}),
        queryInterface.createTable('ServiceGroupDevicePolicies', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          device_policy_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'DevicePolicies',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          service_group_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'ServiceGroups',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, {transaction: t}),
        queryInterface.createTable('ClientDevicePolicies', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          device_policy_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'DevicePolicies',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          client_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'Clients',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, {transaction: t}),
        queryInterface.createTable('ClientGroupDevicePolicies', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          device_policy_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'DevicePolicies',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          client_group_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'Groups',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, {transaction: t}),
        queryInterface.createTable('OsPolicies', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          os: {
            allowNull: false,
            type: Sequelize.STRING(20),
            unique: true,
          },
          os_version: {
            allowNull: false,
            type: Sequelize.STRING(20)
          },

          is_blocked: {
            type: Sequelize.BOOLEAN,
            defaultValue: false
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          os_display_name: {
            type:Sequelize.STRING(50),
          }
        }, {transaction: t}),
      ])
    })
    
  },
  down: (queryInterface, Sequelize) => {
    return Promise.all([
      queryInterface.removeColumn('DevicePolicies','all_clients'),
      queryInterface.removeColumn('DevicePolicies','all_services'),
      queryInterface.removeColumn('DevicePolicies','name'),
      queryInterface.dropTable('ServiceDevicePolicies'),
      queryInterface.dropTable('ServiceGroupDevicePolicies'),
      queryInterface.dropTable('ClientDevicePolicies'),
      queryInterface.dropTable('ClientGroupDevicePolicies'),
      queryInterface.dropTable('OsPolicies'),
    ]).then(_ => queryInterface.sequelize.query(
        `Insert into DevicePolicies (os,os_version ,antivirus ,is_epp_client ,is_blocked ,is_enabled ) 
        Values ('ubuntuserver','Any','no',false,false,true),
        ('ubuntu18','Any','no',false,false,true),
        ('ubuntuserver18','Any','no',false,false,true),
        ('macintosh','Any','no',false,false,true),
        ('ubuntu','Any','no',false,false,true),
        ('windows','Any','no',false,false,true)`)
      )
  }
};
