'use strict';

module.exports = {
  up: (queryInterface, Sequelize) => {
    return queryInterface.sequelize.transaction(function (transaction) {
      return queryInterface.sequelize.query(`DELETE from AdminAccessLogs where userId Not In (SELECT Distinct ups.id from (select u.id from Users u inner join AdminAccessLogs ag on ag.userId = u.id) ups);`)
      .then(() => {
        
        return Promise.all([
          queryInterface.sequelize.query(`CREATE UNIQUE INDEX idx_name ON Entitlements(name)`, {transaction}),
          queryInterface.sequelize.query(`ALTER TABLE AdminAccessLogs ADD CONSTRAINT FK_LogUser FOREIGN KEY (userId) REFERENCES Users(id);`, {transaction})
            .then(_ => queryInterface.sequelize.query(`CREATE INDEX idx_createdAt ON AdminAccessLogs(createdAt)`, {transaction})),
          queryInterface.sequelize.query(`CREATE INDEX idx_firstName ON Users(firstName)`, {transaction})
            .then(_ => queryInterface.sequelize.query(`CREATE INDEX idx_lastName ON Users(lastName)`, {transaction})),
          queryInterface.sequelize.query(`CREATE INDEX idx_domain ON ValidDomains(domain)`, {transaction}),
        ])
      })
      .then(() => {
        
        return queryInterface.sequelize.query(`DELETE FROM UserRoles where userId In (SELECT ur.userId from (select * from UserRoles) ur left join Users u on u.id = ur.userId where u.id is NULL);`)
      }).then(() => {
  
        
        return Promise.all([
          queryInterface.sequelize.query(`ALTER TABLE UserRoles ADD (
            CONSTRAINT FK_user FOREIGN KEY (userId) REFERENCES Users(id),
            CONSTRAINT FK_role FOREIGN KEY (roleId) REFERENCES Roles(id)
          )`, {transaction}),
        ])
      }).then(() => 
      
        queryInterface.sequelize.query(`DELETE from UpdatePackage where client_id Not In (SELECT ups.client_id from (select up.client_id from Clients c inner join UpdatePackage up on up.client_id = c.id) ups)`))
      .then(() => {
  
        
        return Promise.all([
          queryInterface.sequelize.query(`ALTER TABLE UpdatePackage MODIFY client_id int null;`, {transaction})
            .then(_ => queryInterface.sequelize.query(`ALTER TABLE UpdatePackage ADD CONSTRAINT FK_client FOREIGN KEY (client_id) REFERENCES Clients(id);`, {transaction})),
          queryInterface.sequelize.query(`CREATE INDEX idx_username ON DeviceInventories(userName)`, {transaction}),
          queryInterface.sequelize.query(`CREATE INDEX idx_serialNumber ON DeviceInventories(deviceSerialNumber)`, {transaction}),
          queryInterface.sequelize.query(`CREATE INDEX idx_name ON Services(Name)`, {transaction}),
          queryInterface.sequelize.query(`CREATE INDEX idx_protocol ON Services(protocol)`, {transaction}),
          queryInterface.sequelize.query(`CREATE INDEX idx_ipv4Address ON Services(ipv4_address)`, {transaction}),
          queryInterface.sequelize.query(`CREATE INDEX idx_type ON Services(type)`, {transaction}),
          queryInterface.sequelize.query(`CREATE INDEX idx_name ON Controllers(Name)`, {transaction}),
          queryInterface.sequelize.query(`CREATE INDEX idx_name ON Gateways(Name)`, {transaction}),
          queryInterface.sequelize.query(`CREATE INDEX idx_name ON Idps(name)`, {transaction}),
          queryInterface.sequelize.query(`CREATE INDEX idx_name ON ServerBridges(name)`, {transaction}),
          // queryInterface.sequelize.query(`ALTER TABLE Clients DROP INDEX client_agent_id;`, {transaction}),
          queryInterface.sequelize.query(`CREATE INDEX idx_name ON Clients(name)`, {transaction}),
          queryInterface.sequelize.query(`CREATE INDEX idx_userName ON Clients(user)`, {transaction}),
        ])
      })
    })
    
  },

  down: (queryInterface, Sequelize) => {
    return Promise.all([
      queryInterface.sequelize.query(`ALTER TABLE Entitlements DROP INDEX idx_name;`),
      queryInterface.sequelize.query(`ALTER TABLE AdminAccessLogs DROP FOREIGN KEY FK_LogUser;`),
      queryInterface.sequelize.query(`ALTER TABLE AdminAccessLogs DROP INDEX idx_createdAt;`),
      queryInterface.sequelize.query(`ALTER TABLE Users DROP INDEX idx_firstName;`),
      queryInterface.sequelize.query(`ALTER TABLE Users DROP INDEX idx_lastName;`),
      queryInterface.sequelize.query(`ALTER TABLE ValidDomains DROP INDEX idx_domain;`),
      queryInterface.sequelize.query(`ALTER TABLE UserRoles DROP FOREIGN KEY FK_user;`),
      queryInterface.sequelize.query(`ALTER TABLE UserRoles DROP FOREIGN KEY FK_role;`),
      queryInterface.sequelize.query(`ALTER TABLE UpdatePackage DROP FOREIGN KEY FK_client;`),
      queryInterface.sequelize.query(`ALTER TABLE DeviceInventories DROP INDEX idx_username;`),
      queryInterface.sequelize.query(`ALTER TABLE DeviceInventories DROP INDEX idx_serialNumber;`),
      queryInterface.sequelize.query(`ALTER TABLE Services DROP INDEX idx_name;`),
      queryInterface.sequelize.query(`ALTER TABLE Services DROP INDEX idx_protocol;`),
      queryInterface.sequelize.query(`ALTER TABLE Services DROP INDEX idx_ipv4Address;`),
      queryInterface.sequelize.query(`ALTER TABLE Services DROP INDEX idx_type;`),
      queryInterface.sequelize.query(`ALTER TABLE Controllers DROP INDEX idx_name;`),
      queryInterface.sequelize.query(`ALTER TABLE Gateways DROP INDEX idx_name;`),
      queryInterface.sequelize.query(`ALTER TABLE Idps DROP INDEX idx_name;`),
      queryInterface.sequelize.query(`ALTER TABLE ServerBridges DROP INDEX idx_name;`),
      queryInterface.sequelize.query(`ALTER TABLE Clients DROP INDEX idx_name;`),
      queryInterface.sequelize.query(`ALTER TABLE Clients DROP INDEX idx_userName;`),
    ])
  }
};
