'use strict';

module.exports = {
  up: (queryInterface, Sequelize) => {
    return Promise.all([
      queryInterface.sequelize.query(`
      SELECT
        clientId,
        c.Name ,
        GROUP_CONCAT(serviceId) as serviceId
      FROM
        ClientServices cs
      JOIN Clients c ON
        c.id = cs.clientId and c.is_deleted = 0
      GROUP BY
        clientId;
    `, { type: queryInterface.sequelize.QueryTypes.SELECT }),
    queryInterface.sequelize.query(`
      SELECT
        g.name,
        gs.groupId ,
        GROUP_CONCAT(gs.serviceId) as serviceId
      FROM
        GroupServices gs
      JOIN \`Groups\` g ON
        g.id = gs.groupId
      GROUP BY
        gs.groupId ;
    `, { type: queryInterface.sequelize.QueryTypes.SELECT })
    ]).then(([data, groupData]) => {
      const values = data.reduce((acc, val) => {
        const obj = {
          name: val.Name + " - " + val.clientId,
          ClientPolicies: {
            client_id: val.clientId,
            policy_id: null,
          },
          ServicePolicies: (val.serviceId).split(',').map(v => ({
            policy_id: null,
            service_id: v
          }))
        };
        acc.push(obj);
        return acc;
      }, [])

      const groupValues = groupData.reduce((acc, val) => {
        const obj = {
          name: val.name + " - " + val.groupId,
          ClientGroupPolicies: {
            client_group_id: val.groupId,
            policy_id: null,
          },
          ServicePolicies: (val.serviceId).split(',').map(v => ({
            policy_id: null,
            service_id: v
          }))
        };
        acc.push(obj);
        return acc;
      }, [])

      return queryInterface.sequelize.transaction().then((transaction) => {
        return Promise.all([
          ...values.map(policy => {
            return queryInterface.sequelize.query(`
              INSERT INTO Entitlements (id, name) VALUES (DEFAULT, '${policy.name}')
            `, {type: queryInterface.sequelize.QueryTypes.INSERT, transaction})
            .then(([id]) => {
              return Promise.all([
                queryInterface.sequelize.query(`
                  INSERT INTO ClientPolicies (id, client_id, policy_id) 
                  VALUES (DEFAULT, ${policy.ClientPolicies.client_id}, ${id})
                `,{transaction}),
                queryInterface.sequelize.query(`
                  INSERT INTO ServicePolicies (id, service_id, policy_id) 
                  VALUES ${policy.ServicePolicies
                    .map(v => (`(DEFAULT, ${v.service_id}, ${id})`))
                    .join(',')}
                `,{transaction})
              ])
            })
          }),
          ...groupValues.map(policy => {
            return queryInterface.sequelize.query(`
              INSERT INTO Entitlements (id, name) VALUES (DEFAULT, '${policy.name}')
            `, {type: queryInterface.sequelize.QueryTypes.INSERT, transaction})
            .then(([id]) => {
              return Promise.all([
                queryInterface.sequelize.query(`
                  INSERT INTO ClientGroupPolicies (id, client_group_id, policy_id) 
                  VALUES (DEFAULT, ${policy.ClientGroupPolicies.client_group_id}, ${id})
                `,{transaction}),
                queryInterface.sequelize.query(`
                  INSERT INTO ServicePolicies (id, service_id, policy_id) 
                  VALUES ${policy.ServicePolicies
                    .map(v => (`(DEFAULT, ${v.service_id}, ${id})`))
                    .join(',')}
                `,{transaction})
              ])
            })
          }),
          queryInterface.sequelize.query('DELETE FROM GroupServices', {transaction}),
          queryInterface.sequelize.query(`
            INSERT INTO DevicePolicies 
            (antivirus,is_epp_client,is_blocked,is_enabled,all_clients,all_services,name) VALUES 
            ('no',0,0,1,1,1,'Default device policies')
          `, {type: queryInterface.sequelize.QueryTypes.INSERT, transaction})
        ])
        .then(
          _ => transaction.commit(),
          error => {
            return transaction.rollback().then(_ => {throw error});
          }
        )
        .then(_ => {
          queryInterface.sequelize.query(`
          Alter Table GroupServices Add (
            constraint service_FK foreign key(serviceId) references Services(ID) ON DELETE CASCADE,
            constraint group_FK foreign key(groupId) references ServiceGroups(ID) ON DELETE CASCADE
          )`
          ).catch(error => {
            if(error.original.code === 'ER_DUP_KEY' || error.original.code === 'ER_FK_DUP_NAME')
              return Promise.resolve();
            return Promise.reject(error);
          })
        })
      })
    })
  },

  down: (queryInterface, Sequelize) => {
    return queryInterface.sequelize.query(`DELETE FROM DevicePolicies where name = 'Default device policies'`,)
  }
};

