'use strict';

module.exports = {
  up: (queryInterface, Sequelize) => {
    return queryInterface.createTable('SaasBridges', {
      id: {
        allowNull: false,
        autoIncrement: true,
        primaryKey: true,
        type: Sequelize.INTEGER
      },
      name: {
        type: Sequelize.STRING(50),
        allowNull: false
      },
      clientId: {
        type: Sequelize.INTEGER,
        references:{
          model:'Clients',
          key:'id'
        },
        onDelete:'CASCADE'
      },
      serviceId: {
        type: Sequelize.INTEGER,
        references:{
          model:'Services',
          key:'id'
        },
        onDelete:'CASCADE'
      },
      mappedDomain: {
        type: Sequelize.STRING
      },
      authAgentDomain: {
        type: Sequelize.STRING,
      },
      authService: {
        type: Sequelize.ENUM('aws cognito', 'okta'),
        allowNull: false,
        defaultValue: 'aws cognito'
      },
      authServiceClientId: {
        type: Sequelize.STRING,
      },
      authServiceClientSecret: {
        type: Sequelize.STRING,
      },
      authServiceUrl: {
        type: Sequelize.STRING,
      },
      authServiceTokenUrl: {
        type: Sequelize.STRING,
      },
      authServiceUserInfoUrl: {
        type: Sequelize.STRING,
      },
      isEnable:{
        type:Sequelize.BOOLEAN,
        defaultValue:true
      },
      email_token: {
        type: Sequelize.STRING
      },
      email_token_expiry: {
        type: Sequelize.DATE
      },
      email_token_used: {
        type: Sequelize.BOOLEAN,
        defaultValue: false
      },
      createdAt: {
        allowNull: false,
        type: Sequelize.DATE,
        defaultValue:Sequelize.fn('NOW')
      },
      updatedAt: {
        allowNull: false,
        type: Sequelize.DATE,
        defaultValue:Sequelize.fn('NOW')
      }
    });
  },
  down: function(queryInterface, Sequelize) {
    return queryInterface.dropTable('SaasBridges');
  }
};
