'use strict';

module.exports = {
  up: (queryInterface, Sequelize) => {
    return Promise.all([

      queryInterface.sequelize.query(
        `DELETE FROM ClientPolicies WHERE policy_id NOT IN (
          SELECT id from Entitlements
        )`
      ).then(_ =>
        queryInterface.sequelize.query(
          `Alter Table ClientPolicies
            Add constraint Entitlements_ClientPolicies_FK foreign key(policy_id) 
            references Entitlements(ID) ON DELETE CASCADE`
        )
      ).catch(error => {
        if(error.original.code === 'ER_DUP_KEY' || error.original.code === 'ER_FK_DUP_NAME')
          return Promise.resolve();
        return Promise.reject(error);
      }),
      queryInterface.sequelize.query(
        `DELETE FROM ClientGroupPolicies WHERE client_group_id NOT IN (
          SELECT id from \`Groups\`
        )`
      ).then(_ =>
        queryInterface.sequelize.query(
          `Alter Table ClientGroupPolicies
            Add constraint Groups_FK foreign key(client_group_id) 
            references \`Groups\`(ID) ON DELETE CASCADE`
        )
      ).catch(error => {
        if(error.original.code === 'ER_DUP_KEY' || error.original.code === 'ER_FK_DUP_NAME')
          return Promise.resolve();
        return Promise.reject(error);
      }),

      queryInterface.sequelize.query(
        `DELETE FROM ClientGroupPolicies WHERE policy_id NOT IN (
          SELECT id from Entitlements
        )`
      ).then(_ =>
        queryInterface.sequelize.query(
          `Alter Table ClientGroupPolicies
            Add constraint Entitlements_ClientGroupPolicies_FK foreign key(policy_id) 
            references Entitlements(ID) ON DELETE CASCADE`
        )
      ).catch(error => {
        if(error.original.code === 'ER_DUP_KEY' || error.original.code === 'ER_FK_DUP_NAME')
          return Promise.resolve();
        return Promise.reject(error);
      }),

      queryInterface.sequelize.query(
        `DELETE FROM ServicePolicies WHERE policy_id NOT IN (
          SELECT id from Entitlements
        )`
      ).then(_ =>
        queryInterface.sequelize.query(
          `Alter Table ServicePolicies
            Add constraint Entitlements_ServicePolicies_FK foreign key(policy_id) 
            references Entitlements(ID) ON DELETE CASCADE`
        )
      ).catch(error => {
        if(error.original.code === 'ER_DUP_KEY' || error.original.code === 'ER_FK_DUP_NAME')
          return Promise.resolve();
        return Promise.reject(error);
      }),

      queryInterface.sequelize.query(
        `DELETE FROM ServicePolicies WHERE service_id NOT IN (
          SELECT id from Services
        )`
      ).then(_ =>
        queryInterface.sequelize.query(
          `Alter Table ServicePolicies
            Add constraint Services_FK foreign key(service_id) 
            references Services(ID) ON DELETE CASCADE`
        )
      ).catch(error => {
        if(error.original.code === 'ER_DUP_KEY' || error.original.code === 'ER_FK_DUP_NAME')
          return Promise.resolve();
        return Promise.reject(error);
      }),

      queryInterface.sequelize.query(
        `DELETE FROM ServiceGroupPolicies WHERE policy_id NOT IN (
          SELECT id from Entitlements
        )`
      ).then(_ =>
        queryInterface.sequelize.query(
          `Alter Table ServiceGroupPolicies
            Add constraint Entitlements_ServiceGroupPolicies_FK foreign key(policy_id) 
            references Entitlements(ID) ON DELETE CASCADE`
        )
      ).catch(error => {
        if(error.original.code === 'ER_DUP_KEY' || error.original.code === 'ER_FK_DUP_NAME')
          return Promise.resolve();
        return Promise.reject(error);
      }),

      queryInterface.sequelize.query(
        `DELETE FROM ServiceGroupPolicies WHERE service_group_id NOT IN (
          SELECT id from ServiceGroups
        )`
      ).then(_ =>
        queryInterface.sequelize.query(
          `Alter Table ServiceGroupPolicies
            Add constraint ServiceGroups_FK foreign key(service_group_id) 
            references ServiceGroups(ID) ON DELETE CASCADE`
        )
      ).catch(error => {
        if(error.original.code === 'ER_DUP_KEY' || error.original.code === 'ER_FK_DUP_NAME')
          return Promise.resolve();
        return Promise.reject(error);
      })
    ]);
  },

  down: (queryInterface, Sequelize) => {
    return Promise.all([
      queryInterface.sequelize.query(
        `Alter Table ClientPolicies
          drop FOREIGN KEY Entitlements_ClientPolicies_FK`
      ),
      queryInterface.sequelize.query(
        `Alter Table ClientGroupPolicies
          drop FOREIGN KEY Entitlements_ClientGroupPolicies_FK`
      ),
      queryInterface.sequelize.query(
        `Alter Table ServicePolicies
          drop FOREIGN KEY Entitlements_ServicePolicies_FK`
      ),
      queryInterface.sequelize.query(
        `Alter Table ServiceGroupPolicies
          drop FOREIGN KEY Entitlements_ServiceGroupPolicies_FK`
      ),

      queryInterface.sequelize.query(
        `Alter Table ClientGroupPolicies
          drop FOREIGN KEY Groups_FK`
      ),
      queryInterface.sequelize.query(
        `Alter Table ServicePolicies
          drop FOREIGN KEY Services_FK`
      ),
      queryInterface.sequelize.query(
        `Alter Table ServiceGroupPolicies
          drop FOREIGN KEY ServiceGroups_FK`
      ),      

    ]);
  }
};
