'use strict';
module.exports = {
  up: (queryInterface, Sequelize) => {
    return queryInterface.sequelize.transaction(function (t) {
      return Promise.all([
        queryInterface.createTable('ServiceRoutingPolicies', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          routing_policy_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'RoutingPolicies',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          service_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'Services',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, {transaction: t}),
        queryInterface.createTable('ServiceGroupRoutingPolicies', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          routing_policy_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'RoutingPolicies',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          service_group_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'ServiceGroups',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, {transaction: t}),
        queryInterface.createTable('GatewayRoutingPolicies', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          routing_policy_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'RoutingPolicies',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          gateway_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'Gateways',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, {transaction: t}),
        queryInterface.createTable('GatewayGroupRoutingPolicies', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          routing_policy_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'RoutingPolicies',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          gateway_group_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'GatewayGroups',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, {transaction: t})
      ])
    })
    
  },
  down: (queryInterface, Sequelize) => {
    return Promise.all([
      queryInterface.dropTable('ServiceRoutingPolicies'),
      queryInterface.dropTable('ServiceGroupRoutingPolicies'),
      queryInterface.dropTable('GatewayRoutingPolicies'),
      queryInterface.dropTable('GatewayGroupRoutingPolicies')
    ]);
  }
};
