'use strict';

module.exports = {
  up: (queryInterface, Sequelize) => {
    return queryInterface.createTable('OsVersions', {
      id: {
        allowNull: false,
        autoIncrement: true,
        primaryKey: true,
        type: Sequelize.INTEGER
      },
      tagName: {
        type: Sequelize.STRING(50)
      },
      displayName: {
        type: Sequelize.STRING(50)
      },
      buildVersion: {
        type: Sequelize.STRING(50)
      },
      createdAt: {
        allowNull: false,
        type: Sequelize.DATE
      },
      updatedAt: {
        allowNull: false,
        type: Sequelize.DATE
      }
    }).then(() => {
      return Promise.all([
        queryInterface.changeColumn('Updates', 'buildOs', {
          type: Sequelize.ENUM(
            'ubuntu',
            'windows10',
            'windows11',
            'windows',
            'macintosh',
            'ubuntuserver',
            'ubuntu18',
            'ubuntuserver18',
            'ios',
            'android',
            'windowsserver',
            'ubuntuserver20armv8',
            'centosserver'
          ),
          allowNull: false,
          defaultValue: 'ubuntu'
        }),
        queryInterface.changeColumn('Gateways', 'buildOs', {
          type: Sequelize.ENUM(
            'ubuntu',
            'windowsserver',
            'ubuntuserver20armv8',
            'centosserver'
          ),
          allowNull: false,
          defaultValue: 'ubuntu'
        }),
        queryInterface.sequelize.query(`
        INSERT INTO OsVersions (tagName, displayName, buildVersion, createdAt, updatedAt) VALUES
          ( 'windows6.1', 'Windows 7', '6.1.7601', NOW(), NOW() ),
          ( 'windows6.2', 'Windows 8', '6.2.9200', NOW(), NOW() ),
          ( 'windows6.3', 'Windows 8.1', '6.3.9600', NOW(), NOW() ),
          ( 'windows10.0', 'Windows 10', '10.0.10240', NOW(), NOW() ),
          ( 'windows10.0', 'Windows 10', '10.0.10586', NOW(), NOW() ),
          ( 'windows10.0', 'Windows 10', '10.0.14393', NOW(), NOW() ),
          ( 'windows10.0', 'Windows 10', '10.0.15063', NOW(), NOW() ),
          ( 'windows10.0', 'Windows 10', '10.0.16299', NOW(), NOW() ),
          ( 'windows10.0', 'Windows 10', '10.0.17134', NOW(), NOW() ),
          ( 'windows10.0', 'Windows 10', '10.0.17763', NOW(), NOW() ),
          ( 'windows10.0', 'Windows 10', '10.0.18362', NOW(), NOW() ),
          ( 'windows10.0', 'Windows 10', '10.0.18363', NOW(), NOW() ),
          ( 'windows10.0', 'Windows 10', '10.0.19041', NOW(), NOW() ),
          ( 'windows10.0', 'Windows 10', '10.0.19042', NOW(), NOW() ),
          ( 'windows10.0', 'Windows 10', '10.0.19044', NOW(), NOW() ),
          ( 'windowsserver6.1', 'Windows Server 2008 R2', '6.1.7601', NOW(), NOW() ),
          ( 'windowsserver6.2', 'Windows Server 2012', '6.2.9200', NOW(), NOW() ),
          ( 'windowsserver6.3', 'Windows Server 2012 R2', '6.3.9600', NOW(), NOW() ),
          ( 'windowsserver10.0', 'Windows Server 2016+', '10.0.14393', NOW(), NOW() ),
          ( 'windowsserver10.0', 'Windows Server 2016+', '10.0.17763', NOW(), NOW() ),
          ( 'windowsserver10.0', 'Windows Server 2016+', '10.0.18362', NOW(), NOW() ),
          ( 'windowsserver10.0', 'Windows Server 2016+', '10.0.18363', NOW(), NOW() ),
          ( 'windowsserver10.0', 'Windows Server 2016+', '10.0.19041', NOW(), NOW() ),
          ( 'windowsserver10.0', 'Windows Server 2016+', '10.0.19042', NOW(), NOW() ),
          ( 'windows11.0', 'Windows 11', '10.0.22000', NOW(), NOW() ),  
          ( 'centosserver7', 'CentOS 7 Server', '7.7', NOW(), NOW() ),
          ( 'centosserver7', 'CentOS 7 Server', '7.6', NOW(), NOW() ),
          ( 'centosserver7', 'CentOS 7 Server', '7.5', NOW(), NOW() ),
          ( 'centosserver7', 'CentOS 7 Server', '7.4', NOW(), NOW() ),
          ( 'centosserver7', 'CentOS 7 Server', '7.3', NOW(), NOW() ),
          ( 'centosserver7', 'CentOS 7 Server', '7.2', NOW(), NOW() ),
          ( 'centosserver7', 'CentOS 7 Server', '7.1', NOW(), NOW() ),
          ( 'centosserver7', 'CentOS 7 Server', '7.0', NOW(), NOW() ),
          ( 'centosserver8', 'CentOS 8 Server', '8.3', NOW(), NOW() ),
          ( 'centosserver8', 'CentOS 8 Server', '8.2', NOW(), NOW() ),
          ( 'centosserver8', 'CentOS 8 Server', '8.1', NOW(), NOW() ),
          ( 'centosserver8', 'CentOS 8 Server', '8.0', NOW(), NOW() ),
          ( 'macintosh10.14', 'macOS Mojave', '10.14.6', NOW(), NOW() ),
          ( 'macintosh10.14', 'macOS Mojave', '10.14.5', NOW(), NOW() ),
          ( 'macintosh10.14', 'macOS Mojave', '10.14.4', NOW(), NOW() ),
          ( 'macintosh10.14', 'macOS Mojave', '10.14.3', NOW(), NOW() ),
          ( 'macintosh10.14', 'macOS Mojave', '10.14.2', NOW(), NOW() ),
          ( 'macintosh10.14', 'macOS Mojave', '10.14.1', NOW(), NOW() ),
          ( 'macintosh10.14', 'macOS Mojave', '10.14', NOW(), NOW() ),
          ( 'macintosh10.15', 'macOS Catalina', '10.15.7', NOW(), NOW() ),
          ( 'macintosh10.15', 'macOS Catalina', '10.15.6', NOW(), NOW() ),
          ( 'macintosh10.15', 'macOS Catalina', '10.15.5', NOW(), NOW() ),
          ( 'macintosh10.15', 'macOS Catalina', '10.15.4', NOW(), NOW() ),
          ( 'macintosh10.15', 'macOS Catalina', '10.15.3', NOW(), NOW() ),
          ( 'macintosh10.15', 'macOS Catalina', '10.15.2', NOW(), NOW() ),
          ( 'macintosh10.15', 'macOS Catalina', '10.15.1', NOW(), NOW() ),
          ( 'macintosh10.15', 'macOS Catalina', '10.15', NOW(), NOW() ),
          ( 'macintosh11', 'macOS Big Sur', '11.5.1', NOW(), NOW() ),
          ( 'macintosh11', 'macOS Big Sur', '11.5', NOW(), NOW() ),
          ( 'macintosh11', 'macOS Big Sur', '11.4', NOW(), NOW() ),
          ( 'macintosh11', 'macOS Big Sur', '11.3.1', NOW(), NOW() ),
          ( 'macintosh11', 'macOS Big Sur', '11.3', NOW(), NOW() ),
          ( 'macintosh11', 'macOS Big Sur', '11.2.3', NOW(), NOW() ),
          ( 'macintosh11', 'macOS Big Sur', '11.2.2', NOW(), NOW() ),
          ( 'macintosh11', 'macOS Big Sur', '11.2.1', NOW(), NOW() ),
          ( 'macintosh11', 'macOS Big Sur', '11.2', NOW(), NOW() ),
          ( 'macintosh11', 'macOS Big Sur', '11.1', NOW(), NOW() ),
          ( 'macintosh11', 'macOS Big Sur', '11.0.1', NOW(), NOW() ),
          ( 'macintosh11', 'macOS Big Sur', '11', NOW(), NOW() ),
          ( 'macintosh12', 'macOS Monterey', '12.1', NOW(), NOW() ),
          ( 'macintosh12', 'macOS Monterey', '12.0.1', NOW(), NOW() ),
          ( 'macintosh12', 'macOS Monterey', '12.0', NOW(), NOW() ),
          ( 'macintosh12', 'macOS Monterey', '12.2', NOW(), NOW() ),
          ( 'macintosh12', 'macOS Monterey', '12.2.1', NOW(), NOW() ),
          ( 'macintosh12', 'macOS Monterey', '12.3', NOW(), NOW() ),
          ( 'macintosh12', 'macOS Monterey', '12.3.1', NOW(), NOW() ),   
          ( 'ubuntu', 'Ubuntu 16', '16.04.6', NOW(), NOW() ),
          ( 'ubuntu', 'Ubuntu 16', '16.04.5', NOW(), NOW() ),
          ( 'ubuntu', 'Ubuntu 16', '16.04.4', NOW(), NOW() ),
          ( 'ubuntu', 'Ubuntu 16', '16.04.3', NOW(), NOW() ),
          ( 'ubuntu', 'Ubuntu 16', '16.04.2', NOW(), NOW() ),
          ( 'ubuntu', 'Ubuntu 16', '16.04.1', NOW(), NOW() ),
          ( 'ubuntu', 'Ubuntu 16', '16.04', NOW(), NOW() ),
          ( 'ubuntu18', 'Ubuntu 18', '18.04.5', NOW(), NOW() ),
          ( 'ubuntu18', 'Ubuntu 18', '18.04.4', NOW(), NOW() ),
          ( 'ubuntu18', 'Ubuntu 18', '18.04.3', NOW(), NOW() ),
          ( 'ubuntu18', 'Ubuntu 18', '18.04.2', NOW(), NOW() ),
          ( 'ubuntu18', 'Ubuntu 18', '18.04.1', NOW(), NOW() ),
          ( 'ubuntu18', 'Ubuntu 18', '18.04', NOW(), NOW() ),
          ( 'android', 'Android', '11.0', NOW(), NOW() ),
          ( 'android', 'Android', '10.0', NOW(), NOW() ),
          ( 'android', 'Android', '9.0', NOW(), NOW() ),
          ( 'android', 'Android', '8.0', NOW(), NOW() ),
          ( 'android', 'Android', '7.0', NOW(), NOW() ),
          ( 'android', 'Android', '6.0', NOW(), NOW() ),
          ( 'ios', 'iOS', '14.0', NOW(), NOW() ),
          ( 'ios', 'iOS', '13.0', NOW(), NOW() ),
          ( 'ios', 'iOS', '12.0', NOW(), NOW() ),
          ( 'ios', 'iOS', '11.0', NOW(), NOW() ),
          ( 'ios', 'iOS', '10.0', NOW(), NOW() ),
          ( 'ios', 'iOS', '9.0', NOW(), NOW() ),
          ( 'ios', 'iOS', '8.0', NOW(), NOW() ),
          ( 'ubuntuserver20armv8', 'Ubuntu 20 Server (ARMv8)', '20.04.0', NOW(), NOW() )           
        `)
      ]);
    })
  },

  down: (queryInterface, Sequelize) => {
    return queryInterface.dropTable('OsVersions');
  }
};