'use strict';
module.exports = {
  up: (queryInterface, Sequelize) => {
    return Promise.all([
      queryInterface.createTable('SoftwareProcessPolicies', {
        id: {
          allowNull: false,
          autoIncrement: true,
          primaryKey: true,
          type: Sequelize.INTEGER
        },
        name: {
          type: Sequelize.STRING(50)
        },
        is_blocked: {
          type: Sequelize.BOOLEAN,
          defaultValue: false
        },
        is_enabled: {
          type: Sequelize.BOOLEAN,
          defaultValue: true,
        },
        all_clients: {
          type: Sequelize.BOOLEAN,
          defaultValue: false,
        },
        all_services: {
          type: Sequelize.BOOLEAN,
          defaultValue: false,
        },
        compliance: {
          type: Sequelize.ENUM('Any', 'All'),
          allowNull: false
        },
        createdAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        },
        updatedAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        }
      }),
      queryInterface.createTable('ServiceProcessPolicies', {
        id: {
          allowNull: false,
          autoIncrement: true,
          primaryKey: true,
          type: Sequelize.INTEGER
        },
        policy_id: {
          type: Sequelize.INTEGER
        },
        service_id: {
          type: Sequelize.INTEGER
        },
        createdAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        },
        updatedAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        }
      }),
      queryInterface.createTable('ServiceGroupProcessPolicies', {
        id: {
          allowNull: false,
          autoIncrement: true,
          primaryKey: true,
          type: Sequelize.INTEGER
        },
        policy_id: {
          type: Sequelize.INTEGER
        },
        service_group_id: {
          type: Sequelize.INTEGER
        },
        createdAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        },
        updatedAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        }
      }),
      queryInterface.createTable('ClientProcessPolicies', {
        id: {
          allowNull: false,
          autoIncrement: true,
          primaryKey: true,
          type: Sequelize.INTEGER
        },
        policy_id: {
          type: Sequelize.INTEGER
        },
        client_id: {
          type: Sequelize.INTEGER
        },
        createdAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        },
        updatedAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        }
      }),
      queryInterface.createTable('ClientGroupProcessPolicies', {
        id: {
          allowNull: false,
          autoIncrement: true,
          primaryKey: true,
          type: Sequelize.INTEGER
        },
        policy_id: {
          type: Sequelize.INTEGER
        },
        client_group_id: {
          type: Sequelize.INTEGER
        },
        createdAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        },
        updatedAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        }
      }),
      queryInterface.createTable('SoftwareProcessPolicyPivots', {
        id: {
          allowNull: false,
          autoIncrement: true,
          primaryKey: true,
          type: Sequelize.INTEGER
        },
        policy_id: {
          type: Sequelize.INTEGER
        },
        software_id: {
          type: Sequelize.INTEGER
        },
        createdAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        },
        updatedAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('now')
        }
      })
    ])
  },
  down: (queryInterface, Sequelize) => {
    return Promise.all([
      queryInterface.dropTable('SoftwareProcessPolicies'),
      queryInterface.dropTable('ServiceProcessPolicies'),
      queryInterface.dropTable('ServiceGroupProcessPolicies'),
      queryInterface.dropTable('ClientProcessPolicies'),
      queryInterface.dropTable('ClientGroupProcessPolicies'),
      queryInterface.dropTable('SoftwareProcessPolicyPivots')
    ])
  }
};
