'use strict';
module.exports = {
  up: (queryInterface, Sequelize) => {
    return queryInterface.sequelize.transaction().then((transaction) => {
      return Promise.all([
        queryInterface.createTable('PolicyTypeLookup', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          policy_type: {
            type: Sequelize.STRING
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, { transaction }),
        queryInterface.createTable('GatewayPolicies', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          policy_id: {
            type: Sequelize.INTEGER,
          },
          policy_type_id: {
            type: Sequelize.INTEGER,
          },
          gateway_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'Gateways',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, { transaction }),
        queryInterface.createTable('GatewayGroupPolicies', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          policy_id: {
            type: Sequelize.INTEGER,
          },
          policy_type_id: {
            type: Sequelize.INTEGER,
          },
          gateway_group_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'GatewayGroups',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, { transaction }),
        queryInterface.createTable('DataVolumePolicies', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          name: {
            type: Sequelize.STRING
          },
          is_blocked: {
            type: Sequelize.BOOLEAN,
            defaultValue: false
          },
          is_enabled: {
            type: Sequelize.BOOLEAN,
            defaultValue: true,
          },
          all_clients: {
            type: Sequelize.BOOLEAN,
            defaultValue: false,
          },
          all_services: {
            type: Sequelize.BOOLEAN,
            defaultValue: false,
          },
          client_download_rate: {
            type: Sequelize.INTEGER,
            defaultValue: 0,
          },
          client_upload_rate: {
            type: Sequelize.INTEGER,
            defaultValue: 0,
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, { transaction }),
        queryInterface.createTable('ControllerRuntimeHealths', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          controllerId: {
            type: Sequelize.INTEGER,
            references: {
              model: 'Controllers',
              key: 'id'
            },
            onDelete: 'CASCADE'
          },
          memTotal: {
            type: Sequelize.INTEGER
          },
          memFree: {
            type: Sequelize.INTEGER
          },
          storageTotal: {
            type: Sequelize.INTEGER
          },
          storageFree: {
            type: Sequelize.INTEGER
          },
          cpuUtilization: {
            type: Sequelize.FLOAT
          },
          mainProcessCPUUtilization: {
            type: Sequelize.FLOAT
          },
          snifferProcessCPUUtilization: {
            type: Sequelize.FLOAT
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, { transaction }),
        queryInterface.createTable('GatewayRuntimeHealths', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          gatewayId: {
            type: Sequelize.INTEGER,
            references: {
              model: 'Gateways',
              key: 'id'
            },
            onDelete: 'CASCADE'
          },
          memTotal: {
            type: Sequelize.INTEGER
          },
          memFree: {
            type: Sequelize.INTEGER
          },
          storageTotal: {
            type: Sequelize.INTEGER
          },
          storageFree: {
            type: Sequelize.INTEGER
          },
          cpuUtilization: {
            type: Sequelize.FLOAT
          },
          mainProcessCPUUtilization: {
            type: Sequelize.FLOAT
          },
          snifferProcessCPUUtilization: {
            type: Sequelize.FLOAT
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
          }
        }, { transaction }),
        queryInterface.createTable('ServerBridgeUserIds', {
          id: {
            allowNull: false,
            autoIncrement: true,
            primaryKey: true,
            type: Sequelize.INTEGER
          },
          serverBridgeId: {
            type: Sequelize.INTEGER,
            references:{
              model:'ServerBridges',
              key:'id'
            },
            onDelete:'CASCADE'
          },
          createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue:Sequelize.fn('now')
          },
          updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue:Sequelize.fn('now')
          },
          user_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'LocalUsers',
              key: 'id'
            },
            onDelete: 'cascade'
          },
          client_id: {
            type: Sequelize.INTEGER,
            references: {
              model: 'Clients',
              key: 'id'
            },
            onDelete: 'cascade'
          }
        }, { transaction }),
        queryInterface.addColumn('ServiceGroupPolicies', 'policy_type_id', {
          type: Sequelize.INTEGER
        }, { transaction }),
        queryInterface.addColumn('ClientGroupPolicies', 'policy_type_id', {
          type: Sequelize.INTEGER
        }, { transaction }),
        queryInterface.addColumn('ServicePolicies', 'policy_type_id', {
          type: Sequelize.INTEGER
        }, { transaction }),
        queryInterface.addColumn('ClientPolicies', 'policy_type_id', {
          type: Sequelize.INTEGER
        }, { transaction }),
        queryInterface.removeColumn('Users','is_deleted', { transaction }),
        queryInterface.removeColumn('Ldaps','is_deleted', { transaction }),
        queryInterface.removeColumn('Services','is_deleted', { transaction }),
        queryInterface.removeColumn('Gateways','is_deleted', { transaction }),
        queryInterface.removeColumn('Controllers','is_deleted', { transaction }),
        queryInterface.removeColumn('Clients','is_deleted', { transaction }),
        queryInterface.removeColumn('Idps','is_deleted', { transaction }),
        queryInterface.removeColumn('Groups','is_deleted', { transaction }),
        queryInterface.removeColumn('ServiceGroups','is_deleted', { transaction }),
        queryInterface.removeColumn('UserGroups','is_deleted', { transaction }),
        queryInterface.removeColumn('DeviceGroups','is_deleted', { transaction }),
        queryInterface.sequelize.query(`
          ALTER TABLE Settings ALTER max_clients_per_controller SET DEFAULT 15000;
        `, { transaction }),
        queryInterface.sequelize.query(`
          ALTER TABLE Settings ALTER health_status SET DEFAULT 1;
        `, { transaction }),
        queryInterface.sequelize.query(`
          ALTER TABLE Settings ALTER load_stats SET DEFAULT 60;
        `, { transaction }),
        queryInterface.sequelize.query(`
          ALTER TABLE Settings CHANGE services_access_logs data_reporting_interval INTEGER DEFAULT 1;
        `, { transaction }),
        queryInterface.addColumn('Settings', 'data_monitoring_interval', {
          defaultValue:5,
          type: Sequelize.INTEGER
        }, { transaction })
      ]).then(_ => transaction.commit(), error => {
        return transaction.rollback().then(_ => {throw error});
      })
    })
    .catch(error => {
      return Promise.resolve();
    });
  },
  down: (queryInterface, Sequelize) => {
    return Promise.all([
      queryInterface.dropTable('PolicyTypeLookup'),
      queryInterface.dropTable('GatewayPolicies'),
      queryInterface.dropTable('GatewayGroupPolicies'),
      queryInterface.dropTable('DataVolumePolicies'),
      queryInterface.dropTable('ControllerRuntimeHealths'),
      queryInterface.dropTable('GatewayRuntimeHealths'),
      queryInterface.dropTable('ServerBridgeUserIds'),
      queryInterface.removeColumn('ServiceGroupPolicies','policy_type_id'),
      queryInterface.removeColumn('ClientGroupPolicies','policy_type_id'),
      queryInterface.removeColumn('ServicePolicies','policy_type_id'),
      queryInterface.removeColumn('ClientPolicies','policy_type_id'),
      queryInterface.addColumn('Users', 'is_deleted', {
        type: Sequelize.BOOLEAN,
        defaultValue: false
      }),
      queryInterface.addColumn('Ldaps', 'is_deleted', {
        type: Sequelize.BOOLEAN,
        defaultValue: false
      }),
      queryInterface.addColumn('Services', 'is_deleted', {
        type: Sequelize.BOOLEAN,
        defaultValue: false
      }),
      queryInterface.addColumn('Gateways', 'is_deleted', {
        type: Sequelize.BOOLEAN,
        defaultValue: false
      }),
      queryInterface.addColumn('Controllers', 'is_deleted', {
        type: Sequelize.BOOLEAN,
        defaultValue: false
      }),
      queryInterface.addColumn('Clients', 'is_deleted', {
        type: Sequelize.BOOLEAN,
        defaultValue: false
      }),
      queryInterface.addColumn('Idps', 'is_deleted', {
        type: Sequelize.BOOLEAN,
        defaultValue: false
      }),
      queryInterface.addColumn('Groups', 'is_deleted', {
        type: Sequelize.BOOLEAN,
        defaultValue: false
      }),
      queryInterface.addColumn('ServiceGroups', 'is_deleted', {
        type: Sequelize.BOOLEAN,
        defaultValue: false
      }),
      queryInterface.addColumn('UserGroups', 'is_deleted', {
        type: Sequelize.BOOLEAN,
        defaultValue: false
      }),
      queryInterface.addColumn('DeviceGroups', 'is_deleted', {
        type: Sequelize.BOOLEAN,
        defaultValue: false
      }),
      queryInterface.sequelize.query(`
        ALTER TABLE Settings ALTER max_clients_per_controller SET DEFAULT 10;
      `),
      queryInterface.sequelize.query(`
        ALTER TABLE Settings ALTER health_status SET DEFAULT 900;
      `),
      queryInterface.sequelize.query(`
        ALTER TABLE Settings ALTER load_stats SET DEFAULT 5;
      `),
      queryInterface.sequelize.query(`
        ALTER TABLE Settings CHANGE data_reporting_interval services_access_logs INTEGER DEFAULT 3600;
      `),
      queryInterface.removeColumn('Settings','data_monitoring_interval')
    ])
  }
};