'use strict';

module.exports = {
  up: (queryInterface, Sequelize) => {
    const API_SERVER_PORT = process.env.API_SERVER_PORT;
    const API_SERVER_IP = process.env.API_SERVER_IP;

    return Promise.all([
      queryInterface.sequelize.query(`
        ALTER TABLE ClientPolicies
        DROP FOREIGN KEY Entitlements_ClientPolicies_FK;
      `),
      queryInterface.sequelize.query(`
        ALTER TABLE ClientGroupPolicies
        DROP FOREIGN KEY Entitlements_ClientGroupPolicies_FK;
      `),
      queryInterface.sequelize.query(`
        ALTER TABLE ServicePolicies
        DROP FOREIGN KEY Entitlements_ServicePolicies_FK;
      `),
      queryInterface.sequelize.query(`
        ALTER TABLE ServiceGroupPolicies
        DROP FOREIGN KEY Entitlements_ServiceGroupPolicies_FK;
      `)
    ]).then(_ => {
      return Promise.all([
        queryInterface.sequelize.query(`
          INSERT INTO PolicyTypeLookup (id, policy_type) VALUES
          (1, "Access Policy"),
          (2, "Device Policy"),
          (3, "Routing Policy"),
          (4, "Software Policy"),
          (5, "Software Process Policy"),
          (6, "Data Volume Policy");
        `),
        queryInterface.sequelize.query(`
          UPDATE Settings SET
            max_clients_per_controller = 15000,
            in_activity_timeout = 150,
            mtls_frequency = 1000,
            mtls_blocking_timeout = 10,
            health_interval = 1,
            load_stats_interval = 60,
            health_status = 1,
            data_reporting_interval = 1,
            load_stats = 60,
            data_monitoring_interval = 5;
        `),
        queryInterface.sequelize.query(`
          INSERT INTO OsVersions (
            tagName, displayName, buildVersion, createdAt, updatedAt
          ) VALUES (
            'macintosh12', 'macOS Monterey', '12.4', NOW(), NOW()
          );
        `),
        queryInterface.sequelize.query(`
          INSERT INTO Services (
            Name, port, protocol, service_id, ipv4_address, type,
            dummy_port, mappedurl, is_persistent, proxy_enabled,
            createdAt, updatedAt
          ) VALUES (
            'Invisily API Server',
            ${API_SERVER_PORT},
            'tcp',
            701,
            '${API_SERVER_IP}',
            'HTTPS',
            443,
            'api.invisily',
            true,
            false,
            NOW(),
            NOW()
          );
        `),
        queryInterface.sequelize.query(`
          INSERT INTO GatewayPolicies (policy_id, policy_type_id, gateway_id, createdAt, updatedAt)
          SELECT routing_policy_id, 3, gateway_id, NOW(), NOW()
          FROM GatewayRoutingPolicies
        `),
        queryInterface.sequelize.query(`
          INSERT INTO GatewayGroupPolicies (policy_id, policy_type_id, gateway_group_id, createdAt, updatedAt)
          SELECT routing_policy_id, 3, gateway_group_id, NOW(), NOW()
          FROM GatewayGroupRoutingPolicies
        `),
        queryInterface.sequelize.query(`
          UPDATE ServiceGroupPolicies SET policy_type_id = 1
        `),
        queryInterface.sequelize.query(`
          INSERT INTO ServiceGroupPolicies (policy_id, policy_type_id, service_group_id, createdAt, updatedAt)
          SELECT device_policy_id, 2, service_group_id, NOW(), NOW()
          FROM ServiceGroupDevicePolicies
          UNION
          SELECT routing_policy_id, 3, service_group_id, NOW(), NOW()
          FROM ServiceGroupRoutingPolicies
          UNION
          SELECT policy_id, 4, service_group_id, NOW(), NOW()
          FROM ServiceGroupSoftwarePolicies
          UNION
          SELECT policy_id, 5, service_group_id, NOW(), NOW()
          FROM ServiceGroupProcessPolicies
        `),
        queryInterface.sequelize.query(`
          UPDATE ClientGroupPolicies SET policy_type_id = 1
        `),
        queryInterface.sequelize.query(`
          INSERT INTO ClientGroupPolicies (policy_id, policy_type_id, client_group_id, createdAt, updatedAt)
          SELECT device_policy_id, 2, client_group_id, NOW(), NOW()
          FROM ClientGroupDevicePolicies
          UNION
          SELECT policy_id, 4, client_group_id, NOW(), NOW()
          FROM ClientGroupSoftwarePolicies
          UNION
          SELECT policy_id, 5, client_group_id, NOW(), NOW()
          FROM ClientGroupProcessPolicies
        `),
        queryInterface.sequelize.query(`
          UPDATE ServicePolicies SET policy_type_id = 1
        `),
        queryInterface.sequelize.query(`
          INSERT INTO ServicePolicies (policy_id, policy_type_id, service_id, createdAt, updatedAt)
          SELECT device_policy_id, 2, service_id, NOW(), NOW()
          FROM ServiceDevicePolicies
          UNION
          SELECT routing_policy_id, 3, service_id, NOW(), NOW()
          FROM ServiceRoutingPolicies
          UNION
          SELECT policy_id, 4, service_id, NOW(), NOW()
          FROM ServiceSoftwarePolicies
          UNION
          SELECT policy_id, 5, service_id, NOW(), NOW()
          FROM ServiceProcessPolicies
        `),
        queryInterface.sequelize.query(`
          UPDATE ClientPolicies SET policy_type_id = 1
        `),
        queryInterface.sequelize.query(`
          INSERT INTO ClientPolicies (policy_id, policy_type_id, client_id, createdAt, updatedAt)
          SELECT device_policy_id, 2, client_id, NOW(), NOW()
          FROM ClientDevicePolicies
          UNION
          SELECT policy_id, 4, client_id, NOW(), NOW()
          FROM ClientSoftwarePolicies
          UNION
          SELECT policy_id, 5, client_id, NOW(), NOW()
          FROM ClientProcessPolicies
        `)
      ]);
    })
  },

  down: async (queryInterface, Sequelize) => {
    return Promise.all([
      queryInterface.sequelize.query(`
        DELETE FROM PolicyTypeLookup WHERE id IN (1, 2, 3, 4, 5, 6);
      `),
      queryInterface.sequelize.query(`
        DELETE from Services where mappedurl = 'api.invisily';
      `),
      queryInterface.sequelize.query(`
        ALTER TABLE ServiceGroupPolicies
        ADD CONSTRAINT Entitlements_ServiceGroupPolicies_FK
        FOREIGN KEY(policy_id) REFERENCES Entitlements(ID) ON DELETE CASCADE;
      `),
      queryInterface.sequelize.query(`
        ALTER TABLE ServicePolicies
        ADD CONSTRAINT Entitlements_ServicePolicies_FK
        FOREIGN KEY(policy_id) REFERENCES Entitlements(ID) ON DELETE CASCADE;
      `),
      queryInterface.sequelize.query(`
        ALTER TABLE ClientGroupPolicies
        ADD CONSTRAINT Entitlements_ClientGroupPolicies_FK
        FOREIGN KEY(policy_id) REFERENCES Entitlements(ID) ON DELETE CASCADE;
      `),
      queryInterface.sequelize.query(`
        ALTER TABLE ClientPolicies
        ADD CONSTRAINT Entitlements_ClientPolicies_FK
        FOREIGN KEY(policy_id) REFERENCES Entitlements(ID) ON DELETE CASCADE;
      `)
    ])
  }
};