'use strict';

module.exports = {
  up: (queryInterface, Sequelize) => {
    return Promise.all([
      queryInterface.createTable('B2bPortals', {
        id: {
          allowNull: false,
          autoIncrement: true,
          primaryKey: true,
          type: Sequelize.INTEGER
        },
        serverBridgeId: {
          type: Sequelize.INTEGER,
          references: {
            model: 'ServerBridges',
            key: 'id'
          },
          onDelete: 'CASCADE'
        },
        portalUrl: {
          type: Sequelize.STRING
        },
        portalPort: {
          type: Sequelize.INTEGER
        },
        minVncPort: {
          type: Sequelize.INTEGER
        },
        maxVncPort: {
          type: Sequelize.INTEGER
        },
        createdAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
        },
        updatedAt: {
            allowNull: false,
            type: Sequelize.DATE,
            defaultValue: Sequelize.fn('now')
        },
      }),
      queryInterface.createTable('Workloads', {
        id: {
          allowNull: false,
          autoIncrement: true,
          primaryKey: true,
          type: Sequelize.INTEGER
        },
        name: {
          type: Sequelize.STRING
        },
        createdAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue:Sequelize.fn('NOW')
        },
        updatedAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue:Sequelize.fn('NOW')
        }
      }),
      queryInterface.createTable('WorkloadDevicePivots', {
        id: {
          allowNull: false,
          autoIncrement: true,
          primaryKey: true,
          type: Sequelize.INTEGER
        },
        workloadId: {
          type: Sequelize.INTEGER,
          allowNull: false
        },
        deviceId: {
          type: Sequelize.INTEGER,
          allowNull: false
        },
        clientId: {
          type: Sequelize.INTEGER
        },
        createdAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue:Sequelize.fn('NOW')
        },
        updatedAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue:Sequelize.fn('NOW')
        }
      }),
      queryInterface.createTable('Microsegments', {
        id: {
          allowNull: false,
          autoIncrement: true,
          primaryKey: true,
          type: Sequelize.INTEGER
        },
        workloadId: {
          type: Sequelize.INTEGER,
          allowNull: false,
        },
        deviceId: {
          type: Sequelize.INTEGER,
          allowNull: false
        },
        createdAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('NOW')
        },
        updatedAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('NOW')
        }
      }),
      queryInterface.createTable('MicrosegmentPivots', {
        id: {
          allowNull: false,
          autoIncrement: true,
          primaryKey: true,
          type: Sequelize.INTEGER
        },
        ports: {
          type: Sequelize.TEXT('long')
        },
        protocols: {
          type: Sequelize.TEXT('long')
        },
        microsegmentId: {
          type: Sequelize.INTEGER,
          allowNull: false
        },
        destination: {
          type: Sequelize.INTEGER,
          allowNull: false
        },
        createdAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('NOW')
        },
        updatedAt: {
          allowNull: false,
          type: Sequelize.DATE,
          defaultValue: Sequelize.fn('NOW')
        }
      }),
      queryInterface.addColumn('ServiceGroups', 'is_persistent', {
        type: Sequelize.BOOLEAN,
        defaultValue: false
      }),
      queryInterface.changeColumn('Updates', 'buildComponent', {
        type: Sequelize.ENUM('client', 's2s-client', 'controller', 'gateway')
      })
    ])
  },

  down: async (queryInterface, Sequelize) => {
    return Promise.all([
      queryInterface.dropTable('B2bPortals'),
      queryInterface.dropTable('Workloads'),
      queryInterface.dropTable('WorkloadDevicePivots'),
      queryInterface.dropTable('Microsegments'),
      queryInterface.dropTable('MicrosegmentPivots'),
      queryInterface.removeColumn('ServiceGroups', 'is_persistent'),
      queryInterface.changeColumn('Updates', 'buildComponent', {
        type: Sequelize.ENUM('client', 'controller', 'gateway')
      })
    ])
  }
};
