'use strict';

/** @type {import('sequelize-cli').Migration} */

module.exports = {
  async up (queryInterface, Sequelize) {
    await queryInterface.createTable('S2sManualRules', {
      id: {
        allowNull: false,
        autoIncrement: true,
        primaryKey: true,
        type: Sequelize.INTEGER
      },
      source_device: {
        type: Sequelize.STRING(50),
        allowNull: false
      },
      source_ip: {
        type: Sequelize.STRING(50),
        allowNull: false
      },
      destination_device: {
        type: Sequelize.STRING(50),
        allowNull: false
      },
      destination_ip: {
        type: Sequelize.STRING(50),
        allowNull: false
      },
      destination_port: {
        type: Sequelize.STRING(50),
        allowNull: false
      },
      protocol:{
        type: Sequelize.STRING(50),
        allowNull: false
      },
      workloadId:{
        type: Sequelize.INTEGER,
        allowNull: false
      },
      client_id:{
        type: Sequelize.INTEGER,
        allowNull:true
      },
      allowed_comm:{
        type: Sequelize.BOOLEAN,
        allowNull: false,
        defaultValue: true
      },
      deviceId:{
        type: Sequelize.INTEGER,
        allowNull:true
      },
      createdAt: {
        allowNull: false,
        type: Sequelize.DATE
      },
      updatedAt: {
        allowNull: false,
        type: Sequelize.DATE
      }
    });

    await queryInterface.addConstraint('S2sManualRules', {
      type: 'foreign key',
      fields: ['workloadId'],
      name: 'fk_workloadId',
      references: {
        table: 'Workloads',
        field: 'id', 
      },
      onDelete: 'CASCADE', 
      onUpdate: 'CASCADE',
    });
  },
  
  async down (queryInterface, Sequelize) {
    await queryInterface.removeConstraint('S2sManualRules', 'fk_workloadId');
    return queryInterface.dropTable('S2sManualRules');
  }
};