'use strict';
const moment = require('moment');

module.exports = {
  async up(queryInterface, DataTypes) {
    const timeZones = moment.tz.names();

  const timeFormatData = timeZones.map((tz) => {
    const offset = moment.tz(tz).format('Z'); 
    const format = `GMT${offset}`;
    const uniqueIdentifier = `${tz}_${format}`; // Combine time zone name and format

    return { format:format , uniqueIdentifier: uniqueIdentifier };
  });
    await queryInterface.createTable('TimeFormats', {
      id: {
        allowNull: false,
        autoIncrement: true,
        primaryKey: true,
        type: DataTypes.INTEGER,
      },
      format: {
        type: DataTypes.STRING,
        allowNull: false,
      },
      createdAt: DataTypes.DATE,
      updatedAt: DataTypes.DATE,
      uniqueIdentifier: { // Add a new column for the unique identifier
        type: DataTypes.STRING,
        allowNull: false,
        unique: true, // Ensure uniqueness
      },
    }).then(() => {
      return queryInterface.bulkInsert('TimeFormats', timeFormatData, {});
    });;

  },

  async down(queryInterface, DataTypes) {
    await queryInterface.dropTable('TimeFormats');
  },
};
