'use strict';

const { DataTypes } = require('sequelize');

/** @type {import('sequelize-cli').Migration} */
module.exports = {
  async up(queryInterface, Sequelize) {
    await queryInterface.createTable('ClientSoftwareInventory', {
      id: {
        type: Sequelize.INTEGER,
        primaryKey: true,
        autoIncrement: true
      },
      client_id: {
        type: Sequelize.INTEGER,
        allowNull: false,
        unique: true
      },
      state: {
        type: Sequelize.STRING(50),
        allowNull: false,
        defaultValue: 'Undetected'
      },
      ocs_agent_enabled: {
        type: Sequelize.TINYINT,
        allowNull: false,
        defaultValue: false
      },
      createdAt: DataTypes.DATE,
      updatedAt: DataTypes.DATE
    });

    await queryInterface.addConstraint('ClientSoftwareInventory', {
      fields: ['client_id'],
      type: 'foreign key',
      name: 'client_id_client_Software_inventory_fk',
      references: {
        table: 'Clients',
        field: 'id'
      },
      onDelete: 'CASCADE',
      onUpdate: 'CASCADE'
    });
  },

  async down(queryInterface, Sequelize) {
    await queryInterface.removeConstraint(
      'ClientSoftwareInventory',
      'client_id_client_Software_inventory_fk'
    );
    await queryInterface.dropTable('ClientSoftwareInventory');
  }
};
