'use strict';

/** @type {import('sequelize-cli').Migration} */
module.exports = {
  async up (queryInterface, Sequelize) {
    // Entitlements:
    await queryInterface.sequelize.query(
      `
        -- Entitlements:
        CREATE PROCEDURE GetEntitlementsForUser(IN user_id INT)
        BEGIN

          -- Entitlement: Services With Users --
          SELECT services.id as id, services.Name as name, 
          entitlements.id  as policy_id, entitlements.name as entitlement_name, entitlements.access_restriction as time_restricted, entitlements.all_services  as all_services 
          FROM Services services
          INNER JOIN ServicePolicies servicePolicies ON servicePolicies.service_id = services.id 
          INNER JOIN Entitlements entitlements ON entitlements.id  = servicePolicies.policy_id
          INNER JOIN UserPolicies userPolicies ON userPolicies.policy_id = entitlements.id
          INNER JOIN LocalUsers localUsers ON localUsers.id = userPolicies.user_id
          WHERE ((userPolicies.policy_type_id = 1 ) AND (localUsers.id = user_id)) 

          UNION

          -- Entitlement: Services With Users Groups --
          SELECT services.id as id, services.Name as name, 
          entitlements.id  as policy_id, entitlements.name as entitlement_name, entitlements.access_restriction as time_restricted, entitlements.all_services  as all_services
          FROM Services services
          INNER JOIN ServicePolicies servicePolicies ON servicePolicies.service_id = services.id 
          INNER JOIN Entitlements entitlements ON entitlements.id  = servicePolicies.policy_id
          INNER JOIN UserGroupPolicies userGroupPolicies ON userGroupPolicies.policy_id = entitlements.id
          INNER JOIN CustomUserGroups customUserGroups ON customUserGroups.id  = userGroupPolicies.user_group_id
          INNER JOIN UserGroupsPivot userGroupsPivot ON userGroupsPivot.group_id = customUserGroups.id
          INNER JOIN LocalUsers localUsers ON localUsers.id = userGroupsPivot.user_id
          WHERE ((userGroupPolicies.policy_type_id  = 1 ) AND (localUsers.id  = user_id))

          UNION

          -- Entitlement: Service Groups With Users Groups -- 
          SELECT serviceGroups.id as id, serviceGroups.name as name,
          entitlements.id  as policy_id, entitlements.name as entitlement_name, entitlements.access_restriction as time_restricted, entitlements.all_services  as all_services
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN Entitlements entitlements ON entitlements.id  = serviceGroupPolicies.policy_id
          INNER JOIN UserGroupPolicies userGroupPolicies ON userGroupPolicies.policy_id = entitlements.id
          INNER JOIN CustomUserGroups customUserGroups ON customUserGroups.id  = userGroupPolicies.user_group_id
          INNER JOIN UserGroupsPivot userGroupsPivot ON userGroupsPivot.group_id = customUserGroups.id
          INNER JOIN LocalUsers localUsers ON localUsers.id = userGroupsPivot.user_id
          WHERE ((userGroupPolicies.policy_type_id  = 1 ) AND (localUsers.id  = user_id))
          
          UNION

          -- Entitlement: Service Groups With Users --  
          SELECT serviceGroups.id as id, serviceGroups.name as name,
          entitlements.id  as policy_id, entitlements.name as entitlement_name, entitlements.access_restriction as time_restricted, entitlements.all_services  as all_services
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN Entitlements entitlements ON entitlements.id  = serviceGroupPolicies.policy_id
          INNER JOIN UserPolicies userPolicies ON userPolicies.policy_id = entitlements.id 
          INNER JOIN LocalUsers localUsers ON localUsers.id = userPolicies.user_id
          WHERE ((userPolicies.policy_type_id = 1 ) AND (localUsers.id  = user_id))

          UNION

          -- Entitlement: Service With Clients --
          SELECT services.id as id, services.Name as name, 
          entitlements.id  as policy_id, entitlements.name as entitlement_name, entitlements.access_restriction as time_restricted, entitlements.all_services  as all_services
          FROM Services services
          INNER JOIN ServicePolicies servicePolicies ON servicePolicies.service_id = services.id 
          INNER JOIN Entitlements entitlements ON entitlements.id  = servicePolicies.policy_id
          INNER JOIN ClientPolicies clientPolicies ON clientPolicies.policy_id = entitlements.id
          INNER JOIN Clients clients ON clients.id = clientPolicies.client_id 
          WHERE ((clientPolicies.policy_type_id  = 1 ) AND (clients.user_id = user_id))

          UNION

          -- Entitlement: Service With Client Groups --
          SELECT services.id as id, services.Name as name, 
          entitlements.id  as policy_id, entitlements.name as entitlement_name, entitlements.access_restriction as time_restricted, entitlements.all_services  as all_services
          FROM Services services
          INNER JOIN ServicePolicies servicePolicies ON servicePolicies.service_id = services.id 
          INNER JOIN Entitlements entitlements ON entitlements.id  = servicePolicies.policy_id
          INNER JOIN ClientGroupPolicies clientGroupPolicies ON clientGroupPolicies.policy_id = entitlements.id 
          INNER JOIN \`Groups\` g ON g.id = clientGroupPolicies.client_group_id  
          INNER JOIN GroupClients groupClients ON groupClients.groupId  = g.id 
          INNER JOIN Clients clients ON clients.id = groupClients.clientId 
          WHERE ((clientGroupPolicies.policy_type_id  = 1 ) AND (clients.user_id = user_id))

          UNION

          -- Entitlement: Service Groups With Client Groups -- 
          SELECT serviceGroups.id as id, serviceGroups.name as name,
          entitlements.id  as policy_id, entitlements.name as entitlement_name, entitlements.access_restriction as time_restricted, entitlements.all_services  as all_services
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN Entitlements entitlements ON entitlements.id  = serviceGroupPolicies.policy_id
          INNER JOIN ClientGroupPolicies clientGroupPolicies ON clientGroupPolicies.policy_id = entitlements.id 
          INNER JOIN \`Groups\` g ON g.id = clientGroupPolicies.client_group_id  
          INNER JOIN GroupClients groupClients ON groupClients.groupId  = g.id 
          INNER JOIN Clients clients ON clients.id = groupClients.clientId 
          WHERE ((clientGroupPolicies.policy_type_id  = 1 ) AND (clients.user_id = user_id))

          UNION

          -- Entitlement: Service Groups With Clients --  
          SELECT serviceGroups.id as id, serviceGroups.name as name,
          entitlements.id  as policy_id, entitlements.name as entitlement_name, entitlements.access_restriction as time_restricted, entitlements.all_services  as all_services
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN Entitlements entitlements ON entitlements.id  = serviceGroupPolicies.policy_id
          INNER JOIN ClientPolicies clientPolicies ON clientPolicies.policy_id = entitlements.id
          INNER JOIN Clients clients ON clients.id = clientPolicies.client_id 
          WHERE ((clientPolicies.policy_type_id  = 1 ) AND (clients.user_id  = user_id)) 
          
          UNION
              
          -- Entitlement: Services With All Users / Clients --
          SELECT services.id as id, services.Name as name, 
          entitlements.id  as policy_id, entitlements.name AS entitlement_name, entitlements.access_restriction as time_restricted, entitlements.all_services  as all_services
          FROM Services services
          INNER JOIN ServicePolicies servicePolicies ON servicePolicies.service_id = services.id 
          INNER JOIN Entitlements entitlements ON entitlements.id  = servicePolicies.policy_id
          WHERE all_clients = 1 OR all_users = 1

          UNION

          -- Entitlement: Service Groups With All Users / Clients --
          SELECT serviceGroups.id as id, serviceGroups.name as name, 
          entitlements.id  as policy_id, entitlements.name AS entitlement_name, entitlements.access_restriction as time_restricted, entitlements.all_services  as all_services
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN Entitlements entitlements ON entitlements.id  = serviceGroupPolicies.policy_id
          WHERE all_clients = 1 OR all_users = 1
          
          UNION
          
          -- Entitlement: User With All Services --
          SELECT 0 as id, "All Services" as name, 
          entitlements.id  as policy_id, entitlements.name AS entitlement_name, entitlements.access_restriction as time_restricted, entitlements.all_services  as all_services
          FROM Entitlements entitlements
          INNER JOIN UserPolicies userPolicies ON userPolicies.policy_id = entitlements.id
          INNER JOIN LocalUsers localUsers ON localUsers.id = userPolicies.user_id
          WHERE ((userPolicies.policy_type_id = 1 ) AND (entitlements.all_services = 1) AND (localUsers.id = user_id))
          
          UNION
          
          -- Entitlement: User Group With All Services -- 
          SELECT 0 as id, "All Services" as name, 
          entitlements.id  as policy_id, entitlements.name AS entitlement_name, entitlements.access_restriction as time_restricted, entitlements.all_services  as all_services
          FROM Entitlements entitlements
          INNER JOIN UserGroupPolicies userGroupPolicies ON userGroupPolicies.policy_id = entitlements.id
          INNER JOIN CustomUserGroups customUserGroups ON customUserGroups.id  = userGroupPolicies.user_group_id
          INNER JOIN UserGroupsPivot userGroupsPivot ON userGroupsPivot.group_id = customUserGroups.id
          INNER JOIN LocalUsers localUsers ON localUsers.id = userGroupsPivot.user_id
          WHERE ((userGroupPolicies.policy_type_id = 1 ) AND (entitlements.all_services = 1) AND (localUsers.id = user_id))
          
          UNION
          
          -- Entitlement: Client With All Services --
          SELECT 0 as id, "All Services" as name, 
          entitlements.id  as policy_id, entitlements.name AS entitlement_name, entitlements.access_restriction as time_restricted, entitlements.all_services  as all_services
          FROM Entitlements entitlements
          INNER JOIN ClientPolicies clientPolicies ON clientPolicies.policy_id = entitlements.id
          INNER JOIN Clients clients ON clients.id = clientPolicies.client_id 
          WHERE ((clientPolicies.policy_type_id  = 1 ) AND (entitlements.all_services = 1) AND (clients.user_id = user_id))
          
          UNION
          
          -- Entitlement: Client Group With All Services --
          SELECT 0 as id, "All Services" as name, 
          entitlements.id  as policy_id, entitlements.name AS entitlement_name, entitlements.access_restriction as time_restricted, entitlements.all_services  as all_services
          FROM Entitlements entitlements
          INNER JOIN ClientGroupPolicies clientGroupPolicies ON clientGroupPolicies.policy_id = entitlements.id 
          INNER JOIN \`Groups\` g ON g.id = clientGroupPolicies.client_group_id  
          INNER JOIN GroupClients groupClients ON groupClients.groupId  = g.id 
          INNER JOIN Clients clients ON clients.id = groupClients.clientId 
          WHERE ((clientGroupPolicies.policy_type_id  = 1 ) AND (entitlements.all_services = 1) AND (clients.user_id = user_id));
          
        END ;
      `
    );

    // Services
    await queryInterface.sequelize.query(
      `
        -- Services:
        CREATE PROCEDURE GetServicesForUser(IN user_id INT)
        BEGIN
        
          -- Service With Clients --
          SELECT services.id as id, services.Name as name, services.ipv4_address, services.type, services.mappedurl
          FROM Services services
          INNER JOIN ServicePolicies servicePolicies ON servicePolicies.service_id = services.id 
          INNER JOIN Entitlements entitlements ON entitlements.id  = servicePolicies.policy_id
          INNER JOIN ClientPolicies clientPolicies ON clientPolicies.policy_id = entitlements.id
          INNER JOIN Clients clients ON clients.id = clientPolicies.client_id 
          WHERE ((clientPolicies.policy_type_id  = 1 ) AND (clients.user_id = user_id))
        
          UNION
        
          -- Service With Client Groups --
          SELECT services.id as id, services.Name as name, services.ipv4_address, services.type, services.mappedurl
          FROM Services services
          INNER JOIN ServicePolicies servicePolicies ON servicePolicies.service_id = services.id 
          INNER JOIN Entitlements entitlements ON entitlements.id  = servicePolicies.policy_id
          INNER JOIN ClientGroupPolicies clientGroupPolicies ON clientGroupPolicies.policy_id = entitlements.id 
          INNER JOIN \`Groups\` g ON g.id = clientGroupPolicies.client_group_id  
          INNER JOIN GroupClients groupClients ON groupClients.groupId  = g.id 
          INNER JOIN Clients clients ON clients.id = groupClients.clientId 
          WHERE ((clientGroupPolicies.policy_type_id  = 1 ) AND (clients.user_id = user_id))
        
          UNION
        
          -- Service With Users --
          SELECT services.id as id, services.Name as name, services.ipv4_address, services.type, services.mappedurl
          FROM Services services
          INNER JOIN ServicePolicies servicePolicies ON servicePolicies.service_id = services.id 
          INNER JOIN Entitlements entitlements ON entitlements.id  = servicePolicies.policy_id
          INNER JOIN UserPolicies userPolicies ON userPolicies.policy_id = entitlements.id
          INNER JOIN LocalUsers localUsers ON localUsers.id = userPolicies.user_id
          WHERE ((userPolicies.policy_type_id  = 1 ) AND (localUsers.id  = user_id))
        
          UNION
        
          -- Service With User Groups --
          SELECT services.id as id, services.Name as name, services.ipv4_address, services.type, services.mappedurl
          FROM Services services
          INNER JOIN ServicePolicies servicePolicies ON servicePolicies.service_id = services.id 
          INNER JOIN Entitlements entitlements ON entitlements.id  = servicePolicies.policy_id
          INNER JOIN UserGroupPolicies userGroupPolicies ON userGroupPolicies.policy_id = entitlements.id
          INNER JOIN CustomUserGroups customUserGroups ON customUserGroups.id  = userGroupPolicies.user_group_id
          INNER JOIN UserGroupsPivot userGroupsPivot ON userGroupsPivot.group_id = customUserGroups.id
          INNER JOIN LocalUsers localUsers ON localUsers.id = userGroupsPivot.user_id
          WHERE ((userGroupPolicies.policy_type_id  = 1 ) AND (localUsers.id  = user_id))
        
          UNION
        
          -- Service In Service Groups With Clients --
          SELECT DISTINCT groupServices.serviceId as id, services.Name as name, services.ipv4_address, services.type, services.mappedurl
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN Entitlements entitlements ON entitlements.id  = serviceGroupPolicies.policy_id
          INNER JOIN ClientPolicies clientPolicies ON clientPolicies.policy_id = entitlements.id
          INNER JOIN Clients clients ON clients.id = clientPolicies.client_id 
          LEFT JOIN Services services ON services.id = groupServices.serviceId
          WHERE ((clientPolicies.policy_type_id  = 1 ) AND (clients.user_id  = user_id))
        
          UNION
        
          -- Service In Service Groups With Client Groups --
          SELECT DISTINCT groupServices.serviceId as id, services.Name as name, services.ipv4_address, services.type, services.mappedurl
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN Entitlements entitlements ON entitlements.id  = serviceGroupPolicies.policy_id
          INNER JOIN UserPolicies userPolicies ON userPolicies.policy_id = entitlements.id 
          INNER JOIN LocalUsers localUsers ON localUsers.id = userPolicies.user_id
          LEFT JOIN Services services ON services.id = groupServices.serviceId 
          WHERE ((userPolicies.policy_type_id = 1 ) AND (localUsers.id  = user_id))
        
          UNION
        
          -- Service In Service Groups With Users --
          SELECT DISTINCT groupServices.serviceId as id, services.Name as name, services.ipv4_address, services.type, services.mappedurl
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN Entitlements entitlements ON entitlements.id  = serviceGroupPolicies.policy_id
          INNER JOIN ClientGroupPolicies clientGroupPolicies ON clientGroupPolicies.policy_id = entitlements.id 
          INNER JOIN \`Groups\` g ON g.id = clientGroupPolicies.client_group_id  
          INNER JOIN GroupClients groupClients ON groupClients.groupId  = g.id 
          INNER JOIN Clients clients ON clients.id = groupClients.clientId 
          LEFT JOIN Services services ON services.id = groupServices.serviceId 
          WHERE ((clientGroupPolicies.policy_type_id  = 1 ) AND (clients.user_id = user_id))
        
          UNION
        
          -- Service In Service Groups With User Groups --
          SELECT DISTINCT groupServices.serviceId as id, services.Name as name, services.ipv4_address, services.type, services.mappedurl
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN Entitlements entitlements ON entitlements.id  = serviceGroupPolicies.policy_id
          INNER JOIN UserGroupPolicies userGroupPolicies ON userGroupPolicies.policy_id = entitlements.id
          INNER JOIN CustomUserGroups customUserGroups ON customUserGroups.id  = userGroupPolicies.user_group_id
          INNER JOIN UserGroupsPivot userGroupsPivot ON userGroupsPivot.group_id = customUserGroups.id
          INNER JOIN LocalUsers localUsers ON localUsers.id = userGroupsPivot.user_id
          LEFT JOIN Services services ON services.id = groupServices.serviceId 
          WHERE ((userGroupPolicies.policy_type_id  = 1 ) AND (localUsers.id  = user_id))
        
          UNION 
          
          -- Service With All Users / Clients --
          SELECT services.id as id, services.Name as name, services.ipv4_address, services.type, services.mappedurl
          FROM Services services
          INNER JOIN ServicePolicies servicePolicies ON servicePolicies.service_id = services.id 
          INNER JOIN Entitlements entitlements ON entitlements.id  = servicePolicies.policy_id
          WHERE all_clients = 1 OR all_users = 1
        
          UNION 
          
          -- Service In Service Groups With All Users / Clients --
          SELECT groupServices.serviceId as id, services.Name as name, services.ipv4_address, services.type, services.mappedurl
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN Entitlements entitlements ON entitlements.id  = serviceGroupPolicies.policy_id
          LEFT JOIN Services services ON services.id = groupServices.serviceId  
          WHERE all_clients = 1 OR all_users = 1;
        
        END ;
      `
    );


    // Service Groups
    await queryInterface.sequelize.query(
      `
        -- Service Groups:
        CREATE PROCEDURE GetServiceGroupsForUser(IN user_id INT)
        BEGIN
            
          -- Service Groups With Clients --
          SELECT DISTINCT serviceGroups.id as id, serviceGroups.name as name, groupServices.serviceId as service_id, services.Name as service_name, groupServices.createdAt as created_at, serviceGroups.description as description
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN Entitlements entitlements ON entitlements.id  = serviceGroupPolicies.policy_id
          INNER JOIN ClientPolicies clientPolicies ON clientPolicies.policy_id = entitlements.id
          INNER JOIN Clients clients ON clients.id = clientPolicies.client_id 
          LEFT JOIN Services services ON services.id = groupServices.serviceId
          WHERE ((clientPolicies.policy_type_id  = 1 ) AND (clients.user_id  = user_id))
        
          UNION
        
          -- Service Groups With Users --
          SELECT DISTINCT serviceGroups.id as id, serviceGroups.name as name, groupServices.serviceId as service_id, services.Name as service_name, groupServices.createdAt as created_at, serviceGroups.description as description
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN Entitlements entitlements ON entitlements.id  = serviceGroupPolicies.policy_id
          INNER JOIN UserPolicies userPolicies ON userPolicies.policy_id = entitlements.id 
          INNER JOIN LocalUsers localUsers ON localUsers.id = userPolicies.user_id
          LEFT JOIN Services services ON services.id = groupServices.serviceId 
          WHERE ((userPolicies.policy_type_id = 1 ) AND (localUsers.id  = user_id))
        
          UNION
        
          -- Service Groups With Client Groups --
          SELECT DISTINCT serviceGroups.id as id, serviceGroups.name as name, groupServices.serviceId as service_id, services.Name as service_name, groupServices.createdAt as created_at, serviceGroups.description as description
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN Entitlements entitlements ON entitlements.id  = serviceGroupPolicies.policy_id
          INNER JOIN ClientGroupPolicies clientGroupPolicies ON clientGroupPolicies.policy_id = entitlements.id 
          INNER JOIN \`Groups\` g ON g.id = clientGroupPolicies.client_group_id  
          INNER JOIN GroupClients groupClients ON groupClients.groupId  = g.id 
          INNER JOIN Clients clients ON clients.id = groupClients.clientId 
          LEFT JOIN Services services ON services.id = groupServices.serviceId 
          WHERE ((clientGroupPolicies.policy_type_id  = 1 ) AND (clients.user_id = user_id))
        
          UNION
          
          -- Service Groups With User Groups --
          SELECT DISTINCT serviceGroups.id as id, serviceGroups.name as name, groupServices.serviceId as service_id, services.Name as service_name, groupServices.createdAt as created_at, serviceGroups.description as description
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN Entitlements entitlements ON entitlements.id  = serviceGroupPolicies.policy_id
          INNER JOIN UserGroupPolicies userGroupPolicies ON userGroupPolicies.policy_id = entitlements.id
          INNER JOIN CustomUserGroups customUserGroups ON customUserGroups.id  = userGroupPolicies.user_group_id
          INNER JOIN UserGroupsPivot userGroupsPivot ON userGroupsPivot.group_id = customUserGroups.id
          INNER JOIN LocalUsers localUsers ON localUsers.id = userGroupsPivot.user_id
          LEFT JOIN Services services ON services.id = groupServices.serviceId 
          WHERE ((userGroupPolicies.policy_type_id  = 1 ) AND (localUsers.id  = user_id))
              
          UNION 
          
          -- Service Groups With All Users / Clients --
          SELECT DISTINCT serviceGroups.id as id, serviceGroups.name as name, groupServices.serviceId as service_id, services.Name as service_name, groupServices.createdAt as created_at, serviceGroups.description as description
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN Entitlements entitlements ON entitlements.id  = serviceGroupPolicies.policy_id
          LEFT JOIN Services services ON services.id = groupServices.serviceId  
          WHERE all_clients = 1 OR all_users = 1;
          
        END ;
      `
    )
  },

  async down (queryInterface, Sequelize) {
    await queryInterface.sequelize.query('DROP PROCEDURE IF EXISTS GetEntitlementsForUser');
    await queryInterface.sequelize.query('DROP PROCEDURE IF EXISTS GetServicesForUser');
    await queryInterface.sequelize.query('DROP PROCEDURE IF EXISTS GetServiceGroupsForUser');
  }
};
