'use strict';

/** @type {import('sequelize-cli').Migration} */
module.exports = {
  async up (queryInterface, Sequelize) {
    // Policies:
    await queryInterface.sequelize.query(
      `
        -- Policies:
        CREATE PROCEDURE GetPoliciesForGateway(IN gatewayId INT)
        BEGIN
        
          -- Policy: Service with Gateway --
          SELECT services.id as id, services.Name as name, 
          routingPolicies.id  as policy_id, routingPolicies.name AS policy_name, routingPolicies.all_services  as all_services, routingPolicies.all_gateways  as all_gateways
          FROM Services services
          INNER JOIN ServicePolicies servicePolicies ON servicePolicies.service_id = services.id 
          INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = servicePolicies.policy_id 
          INNER JOIN GatewayPolicies gatewayPolicies ON gatewayPolicies.policy_id = routingPolicies.id 
          INNER JOIN Gateways gateways ON gateways.id = gatewayPolicies.gateway_id 
          WHERE ((gatewayPolicies.policy_type_id  = 3 ) AND (servicePolicies.policy_type_id  = 3 ) AND (gateways.id = gatewayId))
          
          UNION
          
          -- Policy: Service Groups With Gateway --  
          SELECT serviceGroups.id as id, serviceGroups.name as name,
          routingPolicies.id  as policy_id, routingPolicies.name AS policy_name, routingPolicies.all_services  as all_services, routingPolicies.all_gateways  as all_gateways
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = serviceGroupPolicies.policy_id 
          INNER JOIN GatewayPolicies gatewayPolicies ON gatewayPolicies.policy_id = routingPolicies.id 
          INNER JOIN Gateways gateways ON gateways.id = gatewayPolicies.gateway_id
          WHERE ((gatewayPolicies.policy_type_id  = 3 ) AND (serviceGroupPolicies.policy_type_id  = 3 ) AND (gateways.id = gatewayId))
          
          UNION
          
          -- Policy: Service With All Gateways --
          SELECT services.id as id, services.Name as name, 
          routingPolicies.id  as policy_id, routingPolicies.name AS policy_name, routingPolicies.all_services  as all_services, routingPolicies.all_gateways  as all_gateways
          FROM Services services
          INNER JOIN ServicePolicies servicePolicies ON servicePolicies.service_id = services.id 
          INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = servicePolicies.policy_id 
          WHERE all_gateways = 1
          
          UNION
          
          -- Policy: Service Groups With All Gateways --  
          SELECT serviceGroups.id as id, serviceGroups.name as name,
          routingPolicies.id  as policy_id, routingPolicies.name AS policy_name, routingPolicies.all_services  as all_services, routingPolicies.all_gateways  as all_gateways
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = serviceGroupPolicies.policy_id 
          WHERE all_gateways = 1
          
          UNION
          
          -- Policy: Gateway With All Services --
          SELECT 0 as id, "All Services" as name, 
          routingPolicies.id  as policy_id, routingPolicies.name AS policy_name, routingPolicies.all_services  as all_services, routingPolicies.all_gateways  as all_gateways
          FROM RoutingPolicies routingPolicies 
          INNER JOIN GatewayPolicies gatewayPolicies ON gatewayPolicies.policy_id = routingPolicies.id 
          INNER JOIN Gateways gateways ON gateways.id = gatewayPolicies.gateway_id 
          WHERE ((gatewayPolicies.policy_type_id  = 3 ) AND (routingPolicies.all_services = 1) AND (gateways.id = gatewayId))
          
          UNION
          
          -- Policy: All Gateways With All Services --
          SELECT 0 as id, "All Gateways" as name, 
          routingPolicies.id  as policy_id, routingPolicies.name AS policy_name, routingPolicies.all_services  as all_services, routingPolicies.all_gateways  as all_gateways
          FROM RoutingPolicies routingPolicies 
          WHERE ((routingPolicies.all_gateways = 1) AND (routingPolicies.all_services = 1));
          
        END ;
      `
    );

    // Services:
    await queryInterface.sequelize.query(
      `
        -- Services:
        CREATE PROCEDURE GetServicesForGateway(IN gatewayId INT)
        BEGIN

            -- Service with Gateway --
          SELECT services.id as id, services.Name as name, services.ipv4_address, services.type, services.mappedurl
          FROM Services services 
          INNER JOIN ServicePolicies servicePolicies ON servicePolicies.service_id = services.id 
          INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = servicePolicies.policy_id 
          INNER JOIN GatewayPolicies gatewayPolicies ON gatewayPolicies.policy_id = routingPolicies.id 
          INNER JOIN Gateways gateways ON gateways.id = gatewayPolicies.gateway_id 
          WHERE ((gatewayPolicies.policy_type_id  = 3 ) AND (servicePolicies.policy_type_id  = 3 ) AND (gateways.id = gatewayId))
          
          UNION
          
          -- Service In Service Groups With Gateway --
          SELECT DISTINCT groupServices.serviceId as id, services.Name as name, services.ipv4_address, services.type, services.mappedurl
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = serviceGroupPolicies.policy_id 
          INNER JOIN GatewayPolicies gatewayPolicies ON gatewayPolicies.policy_id = routingPolicies.id 
          INNER JOIN Gateways gateways ON gateways.id = gatewayPolicies.gateway_id 
          LEFT JOIN Services services ON services.id = groupServices.serviceId
          WHERE ((gatewayPolicies.policy_type_id  = 3 ) AND (serviceGroupPolicies.policy_type_id  = 3 ) AND (gateways.id = gatewayId))
          
          UNION
          
          -- Service With All Gateways --
          SELECT services.id as id, services.Name as name, services.ipv4_address, services.type, services.mappedurl
          FROM Services services
          INNER JOIN ServicePolicies servicePolicies ON servicePolicies.service_id = services.id 
          INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = servicePolicies.policy_id 
          WHERE all_gateways = 1
          
          UNION 
            
          -- Service In Service Groups With All Gateways --
          SELECT groupServices.serviceId as id, services.Name as name, services.ipv4_address, services.type, services.mappedurl
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = serviceGroupPolicies.policy_id 
          LEFT JOIN Services services ON services.id = groupServices.serviceId  
          WHERE all_gateways = 1;
          
        END ;
      `
    );

    // Service Groups
    await queryInterface.sequelize.query(
      `
        -- Service Groups:
        CREATE PROCEDURE GetServiceGroupsForGateway(IN gatewayId INT)
        BEGIN
        
          -- Service Groups With Gateway --
          SELECT DISTINCT serviceGroups.id as id, serviceGroups.name as name, groupServices.serviceId as service_id, services.Name as service_name, groupServices.createdAt as created_at, serviceGroups.description as description
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = serviceGroupPolicies.policy_id 
          INNER JOIN GatewayPolicies gatewayPolicies ON gatewayPolicies.policy_id = routingPolicies.id 
          INNER JOIN Gateways gateways ON gateways.id = gatewayPolicies.gateway_id 
          LEFT JOIN Services services ON services.id = groupServices.serviceId
          WHERE ((gatewayPolicies.policy_type_id  = 3 ) AND (serviceGroupPolicies.policy_type_id  = 3 ) AND (gateways.id = gatewayId))
          
          UNION
          
          -- Service Groups With All Gateways --
          SELECT DISTINCT serviceGroups.id as id, serviceGroups.name as name, groupServices.serviceId as service_id, services.Name as service_name, groupServices.createdAt as created_at, serviceGroups.description as description
          FROM GroupServices AS groupServices
          INNER JOIN ServiceGroups AS serviceGroups ON serviceGroups.id = groupServices.groupId
          INNER JOIN ServiceGroupPolicies AS serviceGroupPolicies ON serviceGroupPolicies.service_group_id = serviceGroups.id
          INNER JOIN RoutingPolicies routingPolicies ON routingPolicies.id = serviceGroupPolicies.policy_id 
          LEFT JOIN Services services ON services.id = groupServices.serviceId  
          WHERE all_gateways = 1;
          
        END ;
      `
    );
  },

  async down (queryInterface, Sequelize) {
    await queryInterface.sequelize.query('DROP PROCEDURE IF EXISTS GetPoliciesForGateway');
    await queryInterface.sequelize.query('DROP PROCEDURE IF EXISTS GetServicesForGateway');
    await queryInterface.sequelize.query('DROP PROCEDURE IF EXISTS GetServiceGroupsForGateway');
  }
};
