'use strict';

/** @type {import('sequelize-cli').Migration} */
module.exports = {
  async up (queryInterface, Sequelize) {
    let transaction;
    try {
      transaction = await queryInterface.sequelize.transaction();

      // Fetch all admin users
      const [adminUsers] = await queryInterface.sequelize.query(
        `
          SELECT DISTINCT u.id
            FROM Users u 
            INNER JOIN UserTemplates ut ON ut.user_id = u.id
            INNER JOIN TemplatePermissions tp ON tp.id = ut.temp_permission_id 
            INNER JOIN Templates t ON t.id = tp.temp_id 
            INNER JOIN Permissions p ON p.id = tp.permission_id 
            WHERE t.id = 2
        `,
        { transaction }
      );

      // S2S Devices:
      // insert in privileges
      await queryInterface.sequelize.query(
        `
          INSERT INTO Privilege (id, name, is_crud_allowed, privilege_id) VALUES
            (64, "Workload Devices", 1, 41)
        `, 
        { transaction }
      );

      // insert in permissions
      const [devicePermission] = await queryInterface.sequelize.query(
        `
          INSERT INTO Permissions (create_permission, read_permission, update_permission, delete_permission, privilege_id) VALUES
            (1, 1, 1, 1, 64)
        `,
        { transaction }
      );

      // insert in template permissions of admin template (id = 2)
      const [deviceTempPermissions] = await queryInterface.sequelize.query(
        `
          INSERT INTO TemplatePermissions (temp_id, permission_id) VALUES
            (2, ${devicePermission})
        `,
        { transaction }
      );

      // insert in  user template
      if (adminUsers.length > 0) {
        const deviceAdminUserTemplateData = adminUsers.map(user => ({
          user_id: user.id,
          temp_permission_id: deviceTempPermissions
        }));

        await queryInterface.bulkInsert('UserTemplates', deviceAdminUserTemplateData, { transaction });
      }

      // Microsegments:
      // insert in privileges
      await queryInterface.sequelize.query(
        `
          INSERT INTO Privilege (id, name, is_crud_allowed, privilege_id) VALUES
            (65, "Microsegments", 1, 41)
        `, 
        { transaction }
      );

      // insert in permissions
      const [microsegPermission] = await queryInterface.sequelize.query(
        `
          INSERT INTO Permissions (create_permission, read_permission, update_permission, delete_permission, privilege_id) VALUES
            (1, 1, 1, 1, 65)
        `,
        { transaction }
      );

      // insert in template permissions of admin template (id = 2)
      const [microsegTempPermissions] = await queryInterface.sequelize.query(
        `
          INSERT INTO TemplatePermissions (temp_id, permission_id) VALUES
            (2, ${microsegPermission})
        `,
        { transaction }
      );

      // insert in  user template
      if (adminUsers.length > 0) {
        const microsegAdminUserTemplateData = adminUsers.map(user => ({
          user_id: user.id,
          temp_permission_id: microsegTempPermissions
        }));
        
        await queryInterface.bulkInsert('UserTemplates', microsegAdminUserTemplateData, { transaction });
      }

      // Workloads:
      // insert in privileges
      await queryInterface.sequelize.query(
        `
          INSERT INTO Privilege (id, name, is_crud_allowed, privilege_id) VALUES
            (66, "Workloads", 1, 41)
        `, 
        { transaction }
      );

      // insert in permissions
      const [workloadsPermission] = await queryInterface.sequelize.query(
        `
          INSERT INTO Permissions (create_permission, read_permission, update_permission, delete_permission, privilege_id) VALUES
            (1, 1, 1, 1, 66)
        `,
        { transaction }
      );

      // insert in template permissions of admin template (id = 2)
      const [workloadsTempPermissions] = await queryInterface.sequelize.query(
        `
          INSERT INTO TemplatePermissions (temp_id, permission_id) VALUES
            (2, ${workloadsPermission})
        `,
        { transaction }
      );

      // insert in  user template
      if (adminUsers.length > 0) {
        const workloadsAdminUserTemplateData = adminUsers.map(user => ({
          user_id: user.id,
          temp_permission_id: workloadsTempPermissions
        }));
        
        await queryInterface.bulkInsert('UserTemplates', workloadsAdminUserTemplateData, { transaction });
      }

      await transaction.commit();
    } catch (error) {
      if (transaction) await transaction.rollback();
      throw error;
    }
  },

  async down (queryInterface, Sequelize) {
    let transaction;
    try {
      transaction = await queryInterface.sequelize.transaction();

      // delete S2S Devices
      await queryInterface.sequelize.query(
        `
          DELETE FROM Privilege
          WHERE id = 64
        `
      );

      // delete Worloads
      await queryInterface.sequelize.query(
        `
          DELETE FROM Privilege
          WHERE id = 65
        `
      );

      // delete Micro
      await queryInterface.sequelize.query(
        `
          DELETE FROM Privilege
          WHERE id = 66
        `
      );

      await transaction.commit();
    } catch (error) {
      if (transaction) await transaction.rollback();
      throw error;
    }
  }
};
