'use strict';
const crypto = require('crypto');
const Openssl = require('../openssl/openssl.js');
const OpensslObj = new Openssl();

module.exports = {
  async up(queryInterface, Sequelize) {
    let transaction;
    try {
      // Start a new transaction
      transaction = await queryInterface.sequelize.transaction();

      // Fetch devices without S2S clients
      const [devicesWithoutS2sClients] = await queryInterface.sequelize.query(
        `SELECT * FROM DeviceInventories
        WHERE device_type IN ('Physical Server', 'Virtual Server')
        AND id NOT IN (
            SELECT DISTINCT device_id
            FROM Clients
            WHERE user LIKE '%workload%'
        );`,
        { transaction }
      );

      const getCurrentDateTime = () => {
        const now = new Date();
        return now.toISOString().slice(0, 19).replace('T', ' ');
      };

      // Generate encryption and HMAC keys
      const [encKeys, hmacKeys] = await Promise.all([
        Promise.all(devicesWithoutS2sClients.map(() => OpensslObj.generateEncKey())),
        Promise.all(devicesWithoutS2sClients.map(() => OpensslObj.generateHMacKey()))
      ]);

      const workloadName = 'Default';
      
      // Create a new workload and retrieve its ID
      const [workload] = await queryInterface.sequelize.query(
        `INSERT INTO Workloads (name) VALUES (:workloadName);`,
        {
          replacements: {
            workloadName: workloadName,
          },
          type: Sequelize.QueryTypes.INSERT,
          transaction,
        }
      );
      const workloadId = workload;
      let pivots = [];
      for (const [i, device] of devicesWithoutS2sClients.entries()) {
        const clientsInsertQuery = `
          INSERT INTO Clients (
            Name,
            ipv4_address,
            user,
            device_os,
            client_type,
            client_status,
            risk_score,
            client_darkening,
            client_os_version,
            serial_number,
            device_hardware_id,
            agent_id,
            enc_key,
            HMacKey,
            is_local,
            device_id,
            auto_updates,
            user_auth_required,
            createdAt,
            updatedAt
          )
          VALUES (
            '{${device.device_name}}@{${workloadName}}',
            '${device.ip_address}',
            'workload${Date.now()}',
            '${device.os}',
            'Server',
            1,
            ${device.risk_score},
            ${device.darkening_state},
            '',
            '${device.device_id}',
            '${crypto.createHash('md5').update(device.device_id.toString()).digest('hex')}',
            0,
            '${encKeys[i].key}',
            '${hmacKeys[i].key}',
            1,
            '${device.id}',
            false,
            false,
            '${getCurrentDateTime()}',
            '${getCurrentDateTime()}'
          );
        `;

        const [dbClients] = await queryInterface.sequelize.query(clientsInsertQuery, { });

        const [clientsData] = await queryInterface.sequelize.query(
          `SELECT id, device_id FROM Clients WHERE id = ${dbClients};`
        );

        const insertQuery = `
          INSERT INTO WorkloadDevicePivots (workloadId, deviceId, clientId)
          VALUES ('${workload}', '${clientsData[0].device_id}', '${clientsData[0].id}');
        `;

        await queryInterface.sequelize.query(insertQuery);

        const clientSoftwareInventoryQuery = `
          INSERT INTO ClientSoftwareInventory (client_id, createdAt, updatedAt)
          VALUES ('${dbClients}', '${getCurrentDateTime()}', '${getCurrentDateTime()}');
        `;

        await queryInterface.sequelize.query(clientSoftwareInventoryQuery);

        pivots.push(dbClients);
      }
      
      // Update the agent_id column in the Clients table
      await queryInterface.sequelize.query(
        `UPDATE Clients SET agent_id = id;`,
        { transaction }
      );

      await transaction.commit();

    } catch (error) {
      console.error('Error in migration up:', error);
      await transaction.rollback();
    }
  },

  async down(queryInterface, Sequelize) {
  }
};
