'use strict';

/** @type {import('sequelize-cli').Migration} */
module.exports = {
  async up (queryInterface, Sequelize) {
    let transaction;
    try {
      transaction = await queryInterface.sequelize.transaction();

      const [duplicateS2SDiscovery] = await queryInterface.sequelize.query(
        `
          SELECT id
          FROM (
              SELECT 
                  id, 
                  source_ip, 
                  destination_ip, 
                  destination_port, 
                  protocol,
                  ROW_NUMBER() OVER (PARTITION BY source_ip, destination_ip, destination_port, protocol ORDER BY id) AS row_num
              FROM S2sDiscoveryData
          ) AS subquery
          WHERE row_num > 1
          ORDER By id;
        `,
        { transaction }
      );

      for (let i = 0; i < duplicateS2SDiscovery.length; i++) {
        await queryInterface.sequelize.query(
          `
            DELETE FROM S2sDiscoveryData
            WHERE id = ${duplicateS2SDiscovery[i].id}
          `,
          { transaction }
        );
      }
      
      const [duplicateS2SManual] = await queryInterface.sequelize.query(
        `
          SELECT id
          FROM (
              SELECT 
                  id, 
                  source_ip, 
                  destination_ip, 
                  destination_port, 
                  protocol,
                  ROW_NUMBER() OVER (PARTITION BY source_ip, destination_ip, destination_port, protocol ORDER BY id) AS row_num
              FROM S2sManualRules
          ) AS subquery
          WHERE row_num > 1
          ORDER By id;
        `,
        { transaction }
      );

      for (let i = 0; i < duplicateS2SManual.length; i++) {
        await queryInterface.sequelize.query(
          `
            DELETE FROM S2sManualRules
            WHERE id = ${duplicateS2SManual[i].id}
          `,
          { transaction }
        );
      }

      await transaction.commit();
    } catch (error) {
      if (transaction) await transaction.rollback();
      throw error;
    }

  },

  async down (queryInterface, Sequelize) {
    /**
     * Add reverting commands here.
     *
     * Example:
     * await queryInterface.dropTable('users');
     */
  }
};
