'use strict';

/** @type {import('sequelize-cli').Migration} */
module.exports = {
  async up (queryInterface, Sequelize) {
    const indexesToRemove = [
      { tableName: 'ServiceAccessLogs', indexName: 'idx_service_id' },
      { tableName: 'ServiceAccessLogs', indexName: 'idx_log_start_time' },
      { tableName: 'ServiceAccessLogs', indexName: 'idx_service_id_log_start_time' },
      { tableName: 'ServiceAccessLogs', indexName: 'idx_agent_log_start_time' },
      { tableName: 'Clients', indexName: 'idx_client_id' },
      { tableName: 'ServiceAccessLogs', indexName: 'idx_agent_id' },
      { tableName: 'ServiceAccessLogs', indexName: 'idx_agent_log_start_time_bytes' },
      { tableName: 'Clients', indexName: 'idx_client_id_name' },
      { tableName: 'ForbiddenSoftwareLists', indexName: 'idx_fsl_device_id' },
      { tableName: 'Templates', indexName: 'idx_templates_name' },
      { tableName: 'Templates', indexName: 'idx_templates_desc' },
      { tableName: 'Services', indexName: 'idx_is_gateway_assigned' },
      { tableName: 'Entitlements', indexName: 'idx_all_services' },
    ];

    // Check and remove existing indexes
    for (const index of indexesToRemove) {
      const [results] = await queryInterface.sequelize.query(
        `SHOW INDEX FROM ${index.tableName} WHERE Key_name = :indexName`,
        {
          replacements: { indexName: index.indexName },
          type: Sequelize.QueryTypes.SELECT
        }
      );
      
      if (results) {
        await queryInterface.removeIndex(index.tableName, index.indexName);
      }
    }

    await queryInterface.addIndex('ServiceAccessLogs', ['service_id'], { name: 'idx_service_id' });
    await queryInterface.addIndex('ServiceAccessLogs', ['log_start_time'], { name: 'idx_log_start_time' });
    await queryInterface.addIndex('ServiceAccessLogs', ['service_id', 'log_start_time'], { name: 'idx_service_id_log_start_time' });
    await queryInterface.addIndex('ServiceAccessLogs', ['agent_id', 'log_start_time'], { name: 'idx_agent_log_start_time' });
    await queryInterface.addIndex('Clients', ['id'], { name: 'idx_client_id' });
    await queryInterface.addIndex('ServiceAccessLogs', ['agent_id'], { name: 'idx_agent_id' });
    await queryInterface.addIndex('ServiceAccessLogs', ['agent_id', 'log_start_time', 'client_tx_bytes', 'service_tx_bytes'], { name: 'idx_agent_log_start_time_bytes' });
    await queryInterface.addIndex('Clients', ['id', 'Name'], { name: 'idx_client_id_name' });
    await queryInterface.addIndex('ForbiddenSoftwareLists', ['device_id'], { name: 'idx_fsl_device_id' });
    await queryInterface.addIndex('Templates', ['name'], { name: 'idx_templates_name' });
    await queryInterface.addIndex('Templates', ['description'], { name: 'idx_templates_desc' });
    await queryInterface.addIndex('Services', ['isGateWayAssigned'], { name: 'idx_is_gateway_assigned' });
    await queryInterface.addIndex('Entitlements', ['all_services'], { name: 'idx_all_services' });
  },

  async down (queryInterface, Sequelize) {
    await queryInterface.removeIndex('ServiceAccessLogs', 'idx_service_id');
    await queryInterface.removeIndex('ServiceAccessLogs', 'idx_log_start_time');
    await queryInterface.removeIndex('ServiceAccessLogs', 'idx_service_id_log_start_time');
    await queryInterface.removeIndex('ServiceAccessLogs', 'idx_agent_log_start_time');
    await queryInterface.removeIndex('Clients', 'idx_client_id');
    await queryInterface.removeIndex('ServiceAccessLogs', 'idx_agent_id');
    await queryInterface.removeIndex('ServiceAccessLogs', 'idx_agent_log_start_time_bytes');
    await queryInterface.removeIndex('Clients', 'idx_client_id_name');
    await queryInterface.removeIndex('ForbiddenSoftwareLists', 'idx_fsl_device_id');
    await queryInterface.removeIndex('Templates', 'idx_templates_name');
    await queryInterface.removeIndex('Templates', 'idx_templates_desc');
    await queryInterface.removeIndex('Services', 'idx_is_gateway_assigned');
    await queryInterface.removeIndex('Entitlements', 'idx_is_gateway_assigned');    
  }
};
