'use strict';
const httpErrors = require('http-errors');
const crypto = require('crypto');
module.exports = function(sequelize, DataTypes) {
  var Gateway = sequelize.define(
    'Gateway',
    {
      Name: DataTypes.STRING,
      agent_id: DataTypes.INTEGER,
      ipv4_address: DataTypes.STRING,
      protocol: DataTypes.STRING,
      port: DataTypes.INTEGER,
      public_ipv4_address: DataTypes.STRING,
      public_ipv4_port: DataTypes.INTEGER,
      spa_port: DataTypes.INTEGER,
      is_active: DataTypes.BOOLEAN,
      counter: DataTypes.STRING,
      enc_key: DataTypes.STRING,
      HMacKey: DataTypes.STRING,
      email: DataTypes.STRING,
      hardware_id: DataTypes.STRING,
      cacert: DataTypes.BLOB('medium'),
      clientcrt: DataTypes.BLOB('medium'),
      clientpem: DataTypes.BLOB('medium'),
      link_key: DataTypes.STRING,
      link_key_expiry: DataTypes.DATE,
      heart_beat: DataTypes.BOOLEAN,
      heart_beat_sent_at: DataTypes.DATE,
      deployment_location: DataTypes.INTEGER,
      installer_status: DataTypes.STRING,
      onPrenDownloadLink: DataTypes.STRING,
      security_protocol: DataTypes.STRING,
      isDownloadPackage: DataTypes.BOOLEAN,
      packageDownloaded: DataTypes.BOOLEAN,
      device_binding: DataTypes.BOOLEAN,
      tls_encryption: DataTypes.BOOLEAN,
      connected_clients: DataTypes.INTEGER,
      is_unique: DataTypes.BOOLEAN,
      subnet_ip: DataTypes.STRING,
      pkg_url: DataTypes.STRING,
      // gateway_group_id: DataTypes.INTEGER,
      proxy_enabled: {
        type: DataTypes.BOOLEAN,
        allowNull: false,
        defaultValue: false
      },
      is_device_profiler_enabled: {
        type: DataTypes.BOOLEAN,
        allowNull: false,
        defaultValue : false
      },
      buildOs: {
        type: DataTypes.ENUM(
          'ubuntuserver18',
          'windowsserver',
          'ubuntuserver20armv8',
          'centosserver'
        ),
        allowNull: false,
        defaultValue: 'ubuntu'
      }
    },
    {
      defaultScope: {
        where: {}
      }
    }
  );
  Gateway.associate = function(models) {
    Gateway.hasOne(models.GatewayHealth, { foreignKey: 'gatewayId' });

    Gateway.belongsToMany(models.RoutingPolicies, {
      foreignKey: 'gateway_id',
      through: models.GatewayPolicies
    });

    Gateway.belongsToMany(models.RoutingPolicies, {
      foreignKey: 'gateway_id',
      through: models.GatewayPolicies
    });

    Gateway.belongsToMany(models.Service, {
      through: 'GatewayService',
      foreignKey: 'gatewayId'
    });

    Gateway.hasMany(models.GatewayGroupPivot, {
      foreignKey: 'gateway_id',
      as: 'GatewayGatewayGroups',
      onDelete: 'CASCADE',
      hooks: true
    });

    Gateway.belongsToMany(models.GatewayGroups, {
      through: models.GatewayGroupPivot,
      foreignKey: 'gateway_id'
    });
  };

  // Gateway.addHook('beforeUpdate', checkGroupField);
  Gateway.addHook('beforeCreate', async function(gateway, options) {
    if (!process.env.LOCAL_USER || !process.env.LOCAL_PASSWORD) {
      throw new httpErrors.NotAcceptable('Local user credentials is missing.');
    }

    const exist = await sequelize.models.LocalUsers.findOne({
      where: {
        userName: process.env.LOCAL_USER
      }
    });

    if (!exist) {
      const hashPassword = await crypto
        .createHash('sha256')
        .update(process.env.LOCAL_PASSWORD)
        .digest('hex');

      await sequelize.models.LocalUsers.create({
        name: 'Gateway User',
        email: process.env.LOCAL_USER + '_invisily@local.com',
        userName: process.env.LOCAL_USER,
        password: hashPassword,
        runtime_bind_id: Math.floor(Date.now() / 1000)
      });
    }

    // await checkGroupField(gateway, options);
  });
  // async function checkGroupField(gateway, options) {
  //   const count = await sequelize.models.GatewayGroups.count({
  //     where: {
  //       all_gateways: true
  //     }
  //   });
  // }
  return Gateway;
};
