const { actionType, publishLDAP } = require('../routes/common/event-publisher');

'use strict';
module.exports = function(sequelize, DataTypes) {
  var Ldap = sequelize.define('Ldap', {
    id: {
      allowNull: false,
      autoIncrement: true,
      primaryKey: true,
      type: DataTypes.INTEGER
    },
    LDAP_url: DataTypes.STRING,
    LDAP_timeout: DataTypes.INTEGER,
    LDAP_connectTimeout: DataTypes.INTEGER,
    LDAP_reconnect: DataTypes.BOOLEAN,
    LDAP_user: DataTypes.STRING,
    LDAP_password: DataTypes.STRING,
    LDAP_dc: DataTypes.STRING,
    LDAP_ca: DataTypes.BLOB,
    LDAP_cert: DataTypes.BLOB,
    LDAP_ca_name: DataTypes.STRING,
    LDAP_cert_name: DataTypes.STRING,
    LDAP_clients_dn: DataTypes.STRING,
    LDAP_groups_dn: DataTypes.STRING,
    directory_type: DataTypes.STRING
  });
  Ldap.associate = function(models) {};

  Ldap.addHook('afterCreate', sendCreateMessage);
  Ldap.addHook('afterUpdate', sendUpdateMessage);

  async function sendCreateMessage(ldap, options){
    const { dataValues } = ldap;
    const message = {
      data: {
        LDAP_url: dataValues.LDAP_url,
        LDAP_user: dataValues.LDAP_user,
        LDAP_ca: dataValues.LDAP_ca,
        LDAP_ca_name: dataValues.LDAP_ca_name,
        LDAP_timeout: dataValues.LDAP_timeout,
        LDAP_connectTimeout: dataValues.LDAP_connectTimeout,
        directory_type: dataValues.directory_type,
        LDAP_id: dataValues.id
      }
    };

    await publishLDAP(actionType.ADDED, message);
  }

  async function sendUpdateMessage(ldap, options){

    if(options.event == 'delete'){
      const { dataValues } = ldap;
      const message = {
        data: {
          LDAP_url: dataValues.LDAP_url,
          LDAP_user: dataValues.LDAP_user,
          LDAP_ca: dataValues.LDAP_ca,
          LDAP_ca_name: dataValues.LDAP_ca_name,
          LDAP_timeout: dataValues.LDAP_timeout,
          LDAP_connectTimeout: dataValues.LDAP_connectTimeout,
          directory_type: dataValues.directory_type,
          LDAP_id: dataValues.id
        }
      };
  
      await publishLDAP(actionType.DELETED, message);

    }else{
      const { dataValues } = ldap;
      const message = {
        data: {
          LDAP_url: dataValues.LDAP_url,
          LDAP_user: dataValues.LDAP_user,
          LDAP_ca: dataValues.LDAP_ca,
          LDAP_ca_name: dataValues.LDAP_ca_name,
          LDAP_timeout: dataValues.LDAP_timeout,
          LDAP_connectTimeout: dataValues.LDAP_connectTimeout,
          directory_type: dataValues.directory_type,
          LDAP_id: dataValues.id
        }
      };
  
      await publishLDAP(actionType.UPDATED, message);
    }
  }

  return Ldap;
};
