const amqplib = require('amqplib');
const connectionString = {
  protocol: 'amqp',
  hostname: process.env.RABBITMQ_HOST || 'localhost',
  port: process.env.RABBITMQ_PORT || 5672,
  username: process.env.RABBITMQ_USER,
  password: process.env.RABBITMQ_PASS
}
const HEARTBEAT_THRESHOLD_IN_SECONDS = 60;
const HEARTBEAT_IN_SECONDS = 10;
let retries = 0;

async function createConnection(cb) {
  try {
    const connection = await amqplib.connect(connectionString);

    connection.on("error", function (err) {
      if (err.message !== "Connection closing") {
        console.error("RabbitMQ connection error: ", (err || {}).message);
      }
    });

    connection.on("close", function () {
      console.error("RabbitMQ reconnecting...");
      retries += 1;
      setTimeout(
        createConnection.bind(null, cb),
        Math.min((retries * HEARTBEAT_IN_SECONDS), HEARTBEAT_THRESHOLD_IN_SECONDS) * 1000
      );
    });

    if(cb && typeof cb === 'function') cb(connection);
    retries = 0;
    console.log(`RabbitMQ connected`)
    return connection;
  } catch (error) {
    console.error(error.message);
    retries += 1;
    console.log(`RabbitMQ reconnecting in ${Math.min((retries * HEARTBEAT_IN_SECONDS), HEARTBEAT_THRESHOLD_IN_SECONDS)}s...`);
    setTimeout(
      createConnection.bind(null, cb), 
      Math.min((retries * HEARTBEAT_IN_SECONDS), HEARTBEAT_THRESHOLD_IN_SECONDS) * 1000
    );
  }
}

module.exports = {
  createConnection,
}