const crypto = require('crypto');
const moment = require('moment');
const models = require('../models');
const Queue = require('./queue');
const helper = require('../routes/common/helper');
const { generateRandomPassword } = require('../utils/common');

class PasswordChangeQueue extends Queue {
  constructor(queueName, opts) {
    super(queueName, opts)
  }

  async consumer(msg) {

    const { user } = msg;

    const localUser = await models.LocalUsers.findOne({
        where: {
          userName: user
        }
    });

    const generatePasword = generateRandomPassword();

    let newPassword = crypto.createHash('sha256')
                            .update(generatePasword)
                            .digest('hex');

    if (localUser.last_password_hash1) {
      await localUser.update({
        password: newPassword,
        last_password_hash1: localUser.password,
        last_password_hash2: localUser.last_password_hash1,
        change_pass: true,
        password_updated_at: moment(),
        temporary_password_expiry: moment().add(1, 'hours')
      });
    }

    if (!localUser.last_password_hash1 && !localUser.last_password_hash2) {
      await localUser.update({
          password: newPassword,
          last_password_hash1: localUser.password,
          change_pass: true,
          password_updated_at: moment(),
          temporary_password_expiry: moment().add(1, 'hours')
      });
    }

    await models.Client.update({
      is_active: false
      }, {
      where: {
          client_type: 'Server',
          user_id: localUser.id,
          device_id: null
          }
    });

    console.log('Password reset Consumer: ', msg);

    console.log('Sending email...');

    helper.sendLocalUserPasswordResetMail({
        recipient: localUser.email,
        emailSubject: 'Invisily Password Reset',
        templateFileName: 'client-password-reset.html',
        name: localUser.name,
        username: localUser.userName,
        password: generatePasword,
    });

  }
}

module.exports = new PasswordChangeQueue('reset-password-queue');