var express = require('express');
var router = express.Router();
var httpErrors = require('http-errors');
var models = require('../../models');
var jwt = require('jsonwebtoken');
var mailer = require('../../utils/mailer');
var moment = require('moment');
var dotenv = require('dotenv').config({ path: '/var/local/env/sdp' });
var blacklist = require('express-jwt-blacklist');
var uuid = require('uuid');
const crypto = require('crypto');
var organizationId = process.env.Organization_DB_ID
  ? parseInt(process.env.Organization_DB_ID)
  : 1;
var projectConfig = require('../../config/config')['project'][
  process.env.NODE_ENV
];
let helper = require('./../common/helper');
let language = require('./../common/language');
var argon2 = require('argon2');
var validator = require('validator');
let settings = require('./../common/settings');
var linOTPSession = require('../common/getLinOTPSession');
var axios = require('axios');
var https = require('https');
const httpError = require('http-errors');
const { verifyJwtToken } = require('./../common/helper');
const { insertIntoAdminLogs } = require('../common/adminLogs');
const {
  getLoginMessage,
  logsPriority,
  getLogoutMessage
} = require('../../config/adminLogs.config');

const httpsAgent = new https.Agent({
  rejectUnauthorized: false // (NOTE: this will disable client ssl verification)
});

/* Login Form */
router.post('/login', function(req, res, next) {
  var email = req.body.email;
  var password = req.body.password;

  var month_access = true;
  var day_access = true;
  var time_access = true;
  var now = moment(new Date());

  var month = now.format('MMM');
  var day = now.format('ddd');

  models.User.findOne({
    include: [{ model: models.Role }],
    attributes: {
      exclude: ['public_key', 'secret_key', 'answer_1', 'answer_2', 'answer_3']
    },
    where: { email: email }
  }).then(function(user) {
    if (user) {
      if (!user.is_locked && !user.is_locked_auto) {
        argon2
          .verify(user.password, password)
          .then(match => {
            if (match) {
              user.update({ wrong_login_attempts: 0 }).then(function(data) {
                if (user.access_restrictions) {
                  if (
                    typeof user.week_days != 'undefined' &&
                    user.week_days != null
                  ) {
                    var week = user.week_days.split(',');

                    if (
                      user.week_days.split(',').indexOf(day) > -1 &&
                      user.week_days.split(',').length > 1
                    ) {
                      day_access = false;
                    } else if (
                      day === week[0] &&
                      user.week_days.split(',').length === 1
                    ) {
                      day_access = false;
                    }
                  }
                  if (
                    typeof user.months != 'undefined' &&
                    user.months != null
                  ) {
                    var months = user.months.split(',');

                    if (
                      user.months.split(',').indexOf(month) > -1 &&
                      user.months.split(',').length > 1
                    ) {
                      month_access = false;
                    } else if (
                      month === months[0] &&
                      user.months.split(',').length === 1
                    ) {
                      month_access = false;
                    }
                  }
                  var start_time =
                    user.access_time_from_hour +
                    ':' +
                    user.access_time_from_minute;
                  var end_time =
                    user.access_time_to_hour + ':' + user.access_time_to_minute;
                  var currentTime = moment(); // e.g. 11:00 pm
                  var startTime = moment(start_time, 'HH:mm');
                  var endTime = moment(end_time, 'HH:mm');
                  var amIBetween = currentTime.isBetween(startTime, endTime);
                  if (amIBetween) {
                    time_access = false;
                  }
                } //jfhtyr#aF@rdtE
                if (user.active === true) {
                  if (month_access || day_access || time_access) {
                    // create a token
                    var token = jwt.sign(
                      { user: user.id },
                      process.env.JWT_SECRET_KEY
                    );
                    user.updateAttributes({ last_login: Date.now() });
                    // return the information including token as JSON
                    res.json({
                      success: true,
                      message: language.get['LOGIN_SUCCESS'],
                      token: token,
                      user: user
                    });
                  } else {
                    res.json({
                      success: false,
                      message: language.get['ACCESS_DENIED_AT_TIME']
                    });
                  }
                } else {
                  res.json({
                    success: false,
                    message: language.get['ACCOUNT_DEACTIVATED']
                  });
                }
              });
            } else {
              let wrong_att = user.wrong_login_attempts + 1;
              if (wrong_att >= 5) {
                user.update({ is_locked_auto: true }).then(function(data) {
                  res.json({
                    success: false,
                    message: language.get['INVALID_EMAIL_PASSWORD']
                  });
                });
              } else {
                user
                  .update({ wrong_login_attempts: wrong_att })
                  .then(function(data) {
                    res.json({
                      success: false,
                      message: language.get['INVALID_EMAIL_PASSWORD']
                    });
                  });
              }
            }
          })
          .catch(err => {
            res.json({
              success: false,
              message: language.get['INVALID_EMAIL_PASSWORD']
            });
          });
      } else {
        if (user.is_locked_auto) {
          models.Settings.findOne({
            limit: 1,
            order: [['id', 'DESC']]
          }).then(function(setting) {
            const recaptchaConfigured =
              !!setting.recaptcha_site_key &&
              setting.recaptcha_site_key != 'RECAPTCHASITEKEY';
            models.Questions.findOne({
              where: {
                id: user.question_1_id
              }
            }).then(function(question1) {
              models.Questions.findOne({
                where: {
                  id: user.question_2_id
                }
              }).then(function(question2) {
                models.Questions.findOne({
                  where: {
                    id: user.question_3_id
                  }
                }).then(function(question3) {
                  res.json({
                    success: false,
                    message: language.get['ACCOUNT_LOCKED_AUTO'],
                    question1: question1.question,
                    question2: question2.question,
                    question3: question3.question,
                    recaptchaConfigured
                  });
                });
              });
            });
          });
        } else {
          res.json({
            success: false,
            message: language.get['ACCOUNT_LOCKED']
          });
        }
      }
    } else {
      res.json({
        success: false,
        message: language.get['INVALID_EMAIL_PASSWORD']
      });
    }
  });
});
let prepareEmailOptions = (
  recipient,
  emailSubject,
  templateFileName,
  downloadLink,
  recipientName,
  linkExpiryInMinutes,
  component
) => {
  return {
    recipient: recipient,
    emailSubject: emailSubject,
    templateFileName: templateFileName,
    downloadLink: downloadLink,
    recipientName: recipientName,
    linkExpiryInMinutes: linkExpiryInMinutes,
    component: component
  };
};

class UserForgotLinkNotValidError extends Error {
  constructor(status, allowedStatuses) {
    super('Link may have been already expired or you have taken a wrong turn.');
    this.name = 'ForgotLinkNotValidError';
  }
}

class UserForgotLinkAlreadyUsed extends Error {
  constructor() {
    super('Link is already used, please request for a new link');
    this.name = 'ForgotLinkAlreadyUsed';
  }
}

/* ForGot Password Form */
router.post('/forgotPassword', function(req, res, next) {
  var email = req.body.username;
  let isValid = true;
  //Checking Email
  isValid = isValid && validator.isEmail(email);
  isValid = isValid && validator.isLength(email, { min: 5, max: 100 });
  if (!isValid) {
    res.json({
      success: false,
      message: language.get['INVALID_DATA_PROVIDED']
    });
    return 0;
  }
  models.User.findOne({
    include: [{ model: models.Role }],
    where: { email: email }
  }).then(function(user) {
    if (user) {
      if (!user.is_locked_auto) {
        crypto.randomBytes(20, function(err, buffer) {
          var token = buffer.toString('hex');
          models.User.update(
            {
              reset_password_token: token,
              reset_password_expires: moment().add(60, 'm')
            },
            { where: { email: email } }
          )
            .then(function(data) {
              let link = '';
              let breakingLink = projectConfig.onPremUiUrl.split('api');
              link =
                breakingLink[0] +
                `/recover-password/` +
                new Buffer(token).toString('base64');
              helper.sendEmail(
                prepareEmailOptions(
                  user.email,
                  'Invisily | Password Reset',
                  'forgot_password.html',
                  link,
                  'User',
                  2,
                  ''
                )
              );
              res.json({
                success: true,
                message: language.get['EMAIL_SENT_SUCCESS']
              });
            })
            .catch(function(error) {
              console.log(error && (error.message || error));
              res.json({
                success: false,
                message: language.get['FORGOT_PASSWORD_ERROR']
              });
            });
        });
      } else {
        res.json({
          success: false,
          message: language.get['FORGOT_PASSWORD_ERROR']
        });
      }
    } else {
      res.json({ success: true, message: language.get['EMAIL_SENT_SUCCESS'] });
    }
  });
});

router.post('/changePassword', function(req, res, next) {
  console.log('req.user', req.user);
  var password = req.body.password;

  let isValid = true;
  //Checking Password
  isValid = isValid && validator.isLength(password, { min: 8, max: 100 });
  let regex = /^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[^A-z0-9]).{8,100}$/;
  isValid = isValid && regex.test(password);
  if (!isValid) {
    res.json({
      success: false,
      message: language.get['INVALID_DATA_PROVIDED']
    });
    return 0;
  }

  models.User.findOne({
    where: {
      id: req.user.id
    }
  }).then(function(user) {
    if (user) {
      argon2
        .verify(user.password, password)
        .then(match => {
          if (match) {
            res.send({
              success: false,
              message: language.get['SAME_PASSWORD_AS_PREVIOUS'],
              token: ''
            });
          } else {
            argon2
              .hash(password)
              .then(hash => {
                password = hash;
                var time = moment()
                  .utc()
                  .local()
                  .format('YYYY-MM-DD HH:mm:ss');
                models.User.update(
                  {
                    reset_password_token: null,
                    reset_password_expires: null,
                    password: password,
                    reset_password_time: time,
                    is_first_login: false
                  },
                  { where: { id: user.id } }
                )
                  .then(function(data) {
                    helper.sendEmail(
                      prepareEmailOptions(
                        user.email,
                        'Invisily | Password Changed',
                        'password_changed.html',
                        '',
                        user.firstName,
                        2,
                        ''
                      )
                    );
                    res.json({
                      success: true,
                      message: language.get['PASSWORD_CHANGE_SUCCESS']
                    });
                  })
                  .catch(function(error) {
                    console.log(error && (error.message || error));
                    res.json({
                      success: false,
                      message: language.get['SOME_ERROR']
                    });
                  });
              })
              .catch(err => {
                res.json({
                  success: false,
                  message: language.get['PASSWORD_HASH_FAILED']
                });
              });
          }
        })
        .catch(err => {});
    } else {
      res.json({ success: false, message: language.get['TOKEN_INVALID'] });
    }
  });
});

router.post('/confirmToken/', function(req, res, next) {
  var token_temp = req.body.token;
  var token = new Buffer(token_temp, 'base64').toString('ascii');
  models.User.findOne({
    where: {
      reset_password_token: token,
      reset_password_expires: {
        $gt: Date.now()
      }
    }
  })
    .then(function(user) {
      if (user) {
        res.json({
          status: true,
          message: language.get['TOKEN_FOUND']
        });
      } else {
        res.json({
          status: false,
          message: language.get['TOKEN_NOT_FOUND']
        });
      }
    })
    .catch(err => {
      res.json({
        status: false,
        message: language.get['TOKEN_NOT_FOUND']
      });
    });
});

/* ForGot PAssword Form */
router.post('/confirmPassword/', function(req, res, next) {
  var token_temp = req.body.token;
  var password = req.body.password;

  let isValid = true;
  //Checking Password
  isValid = isValid && validator.isLength(password, { min: 8, max: 100 });
  let regex = /^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[^A-z0-9]).{8,100}$/;
  isValid = isValid && regex.test(password);
  if (!isValid) {
    res.json({
      success: false,
      message: language.get['INVALID_DATA_PROVIDED']
    });
    return 0;
  }

  var token = new Buffer(token_temp, 'base64').toString('ascii');
  models.User.findOne({
    where: {
      reset_password_token: token,
      reset_password_expires: {
        $gt: Date.now()
      }
    }
  }).then(function(user) {
    if (user) {
      argon2
        .hash(password)
        .then(hash => {
          password = hash;
          var time = moment()
            .utc()
            .local()
            .format('YYYY-MM-DD HH:mm:ss');
          models.User.update(
            {
              reset_password_token: null,
              reset_password_expires: null,
              password: password,
              reset_password_time: time
            },
            { where: { id: user.id } }
          )
            .then(function(data) {
              helper.sendEmail(
                prepareEmailOptions(
                  user.email,
                  'Password Changed',
                  'password_changed.html',
                  '',
                  'User',
                  2,
                  ''
                )
              );
              res.json({
                success: true,
                message: language.get['EMAIL_SENT_SUCCESS']
              });
            })
            .catch(function(error) {
              console.log(error && (error.message || error));
              res.json({
                success: false,
                message: language.get['SOME_ERROR']
              });
            });
        })
        .catch(err => {
          res.json({
            success: false,
            message: language.get['PASSWORD_HASH_FAILED']
          });
        });
    } else {
      res.json({ success: false, message: language.get['TOKEN_INVALID'] });
    }
  });
});

router.post('/agents', function(req, res, next) {
  var username = req.body.username;
  var password = req.body.password;
  var mac_address = req.body.mac_address;

  models.Organization.findOne({
    where: { username: username, password: password }
  }).then(function(organization) {
    if (organization) {
      // create a token
      var token = jwt.sign(
        { user: username + password + mac_address },
        process.env.JWT_SECRET_KEY
      );
      // return the information including token as JSON
      res.status(200).json({ token: token });
    } else {
      res.status(401).json({ message: language.get['CREDENTIAL_INVALID'] });
    }
  });
});

router.post('/checkPassword', function(req, res, next) {
  let response = req.body;
  var password = response.password;
  let isValid = true;
  //Checking Password
  isValid = isValid && validator.isLength(password, { min: 8, max: 100 });
  let regex = /^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[^A-z0-9]).{8,100}$/;
  isValid = isValid && regex.test(password);
  if (!isValid) {
    res.json({
      success: false,
      message: language.get['INVALID_DATA_PROVIDED']
    });
    return 0;
  }
  models.User.findOne({
    where: {
      id: response.id
    }
  }).then(function(user) {
    if (user) {
      try {
        argon2
          .verify(user.password, password)
          .then(match => {
            if (match) {
              res.send({
                success: true,
                message: language.get['PASSWORD_MATCH']
              });
            } else {
              res.send({
                success: false,
                message: language.get['PASSWORD_INVALID'],
                token: ''
              });
            }
          })
          .catch(err => {
            res.send({
              success: false,
              message: language.get['PASSWORD_INVALID'],
              token: ''
            });
          });
      } catch (err) {
        res.send({
          success: false,
          message: language.get['PASSWORD_INVALID'],
          token: ''
        });
      }
    } else {
      res.send({
        success: false,
        message: language.get['PASSWORD_INVALID'],
        token: ''
      });
    }
  });
});

router.post('/checkPasswordWithEmail', function(req, res, next) {
  let response = req.body;
  var password = response.password;
  var email = response.email;

  let isValid = true;
  //Checking Email
  isValid = isValid && validator.isEmail(email);
  isValid = isValid && validator.isLength(email, { min: 5, max: 100 });
  //Checking Password
  isValid = isValid && validator.isLength(password, { min: 8, max: 100 });
  let regex = /^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[^A-z0-9]).{8,100}$/;
  isValid = isValid && regex.test(password);
  if (!isValid) {
    res.json({
      success: false,
      message: language.get['INVALID_DATA_PROVIDED']
    });
    return 0;
  }

  models.User.findOne({
    where: {
      email: response.email
    }
  }).then(function(user) {
    if (user) {
      try {
        argon2
          .verify(user.password, password)
          .then(match => {
            if (match) {
              res.send({
                success: true,
                message: language.get['PASSWORD_MATCH']
              });
            } else {
              res.send({
                success: false,
                message: language.get['PASSWORD_INVALID'],
                token: ''
              });
            }
          })
          .catch(err => {
            res.send({
              success: false,
              message: language.get['PASSWORD_INVALID'],
              token: ''
            });
          });
      } catch (err) {
        res.send({
          success: false,
          message: language.get['PASSWORD_INVALID'],
          token: ''
        });
      }
    } else {
      res.send({
        success: false,
        message: language.get['PASSWORD_INVALID'],
        token: ''
      });
    }
  });
});

router.post('/organizations', function(req, res, next) {
  var username = req.body.username;
  var password = req.body.password;
  var apiCall = req.body.apiCall;
  var organizationCheck = {
    $ne: organizationId
  };
  if (apiCall !== null && apiCall !== undefined) {
    organizationCheck = {
      $eq: organizationId
    };
  }
  models.Organization.findOne({
    where: { id: organizationCheck, username: username, password: password }
  }).then(function(organization) {
    if (organization) {
      let orgId = organization.id;
      let orgName = organization.name;
      // create a token
      var token = jwt.sign(
        { user: username + password + orgId + orgName },
        process.env.JWT_SECRET_KEY
      );
      // return the information including token as JSON
      res.status(200).json({ token: token });
    } else {
      res.status(401).json({ message: language.get['CREDENTIAL_INVALID'] });
    }
  });
});

function xwwwfurlenc(srcjson) {
  if (typeof srcjson !== 'object')
    if (typeof console !== 'undefined') {
      console.log('"srcjson" is not a JSON object');
      return null;
    }
  u = encodeURIComponent;
  var urljson = '';
  var keys = Object.keys(srcjson);
  for (var i = 0; i < keys.length; i++) {
    urljson += u(keys[i]) + '=' + u(srcjson[keys[i]]);
    if (i < keys.length - 1) urljson += '&';
  }
  return urljson;
}

router.post('/validateLinToken', async (req, res, next) => {
  try {
    const { email, password, token } = req.body;
    const user = await models.User.findOne({
      where: {
        email
      }
    });

    if (!user) {
      httpError.NotFound('User not found.');
    }
    if (!user.verifyOtp(token))
      res.status(400).json({ status: false, message: 'Invalid OTP' });

    loginUser(email, password, false, res, req);
  } catch (error) {
    next(error);
  }
});
router.post('/validateLinTokenModal', async (req, res, next) => {
  try {
    const { email, token } = req.body;
    const user = await models.User.findOne({
      where: {
        email
      }
    });

    if (!user) {
      httpError.NotFound('User not found.');
    }

    let is_otp_verified;
    if (!user.verifyOtp(token)) {
      is_otp_verified = false;
    } else {
      console.log('TRUE');
      is_otp_verified = true;
    }
    // res.status(400).json({ status: false, message: 'Invalid OTP' });
    res.status(200).json({ status: true, is_otp_verified: is_otp_verified });
  } catch (error) {
    next(error);
  }
});

router.post('/validateOTPToken', async (req, res, next) => {
  try {
    const { email, token } = req.body;
    let user = await models.User.findOne({
      where: {
        email
      }
    });

    if (!user) {
      httpError.NotFound('User not found.');
    }

    if (!user.verifyOtp(token)) {
      res.status(400).json({ status: false, message: 'Invalid OTP' });
    } else {
      const settings = await models.Settings.findOne({
        attributes: ['admin_mfa', 'recaptcha_site_key']
      });

      const recaptchaConfigured =
        !!settings.recaptcha_site_key &&
        settings.recaptcha_site_key != 'RECAPTCHASITEKEY';

      const recaptchaCheck =
        !recaptchaConfigured ||
        (recaptchaConfigured && user.recaptcha_verified);

      if (user.questions_verified && settings.admin_mfa && recaptchaCheck) {
        await user.update({
          is_locked_auto: false,
          questions_verified: false,
          recaptcha_verified: false
        });
      }
      res.status(200).json({
        status: true,
        message: 'Valid OTP',
        mfa_enabled: settings.admin_mfa
      });
    }
  } catch (error) {
    next(error);
  }
});

router.get('/getRecaptchaSiteKey', async (req, res, next) => {
  try {
    const settings = await models.Settings.findOne({
      attributes: ['recaptcha_site_key']
    });

    res.status(200).json({
      recaptcha_site_key: settings.recaptcha_site_key
    });
  } catch (error) {
    next(error);
  }
});

router.post('/passRecaptchaStatus', async (req, res, next) => {
  try {
    await models.User.update(
      { recaptcha_verified: true },
      { where: { email: req.body.email } }
    );
    res.status(200).json({ message: 'Recaptcha successfully verified.' });
  } catch (error) {
    next(error);
  }
});

router.post('/createLinToken', async (req, res, next) => {
  const { email, resendQR } = req.body;

  try {
    const user = await models.User.findOne({
      where: {
        email
      }
    });

    if (!user) {
      httpError.NotFound('User not found.');
    }

    const updateUser = await user.generateOtp();

    if (!updateUser.is_mfa_assigned || resendQR) {
      helper.sendQRCode({
        OTP_image_data: updateUser.otp_image_data,
        recipient: updateUser.email,
        recipientName: `${updateUser.firstName} ${updateUser.lastName}`,
        emailSubject: 'Your Invisily Admin QR Code',
        templateFileName: 'admin_opt_mail.html',
        is_mfa_assigned: updateUser.is_mfa_assigned
      });
      await updateUser.update({
        is_mfa_assigned: true
      });
    }

    res.send({ message: 'Check your email for OTP QR.' });
  } catch (error) {
    next(error);
  }
});

router.post('/validateAnswers', async (req, res, next) => {
  const { email, answer1, answer2, answer3 } = req.body;

  try {
    const user = await models.User.findOne({
      where: {
        email
      }
    });

    if (!user) {
      httpError.NotFound('User not found.');
    }

    const answer1Validate =
      answer1 && (await argon2.verify(user && user.answer_1, answer1));
    const answer2Validate =
      answer2 && (await argon2.verify(user && user.answer_2, answer2));
    const answer3Validate =
      answer3 && (await argon2.verify(user && user.answer_3, answer3));
    const validation = answer1Validate && answer2Validate && answer3Validate;

    if (validation) {
      await user.update({
        questions_verified: true
      });
    } else {
      throw new httpErrors.Forbidden('Error Occurred.');
    }

    const settings = await models.Settings.findOne({
      attributes: ['admin_mfa', 'recaptcha_site_key']
    });

    const recaptchaConfigured =
      !!settings.recaptcha_site_key &&
      settings.recaptcha_site_key != 'RECAPTCHASITEKEY';

    const recaptchaCheck =
      !recaptchaConfigured || (recaptchaConfigured && user.recaptcha_verified);

    if (user.questions_verified && !settings.admin_mfa && recaptchaCheck) {
      await user.update({
        is_locked_auto: false,
        questions_verified: false,
        recaptcha_verified: false
      });
    }

    res.send({
      status: validation,
      mfa_enabled: settings.admin_mfa
    });
  } catch (error) {
    next(error);
  }
});

const loginUser = (username, password, isOtpVerificationRequired, res, req) => {
  models.User.findOne({
    include: [{ model: models.Role }],
    attributes: {
      exclude: ['public_key', 'secret_key', 'answer_1', 'answer_2', 'answer_3']
    },
    where: {
      email: username
    }
  }).then(async user => {
    if (user) {
      query = `
        SELECT DISTINCT t.id as temp_id, t.name
        FROM Templates t
        INNER JOIN TemplatePermissions tp ON tp.temp_id = t.id
        INNER JOIN Permissions p ON p.id = tp.permission_id
        INNER JOIN Privilege pv ON pv.id = p.privilege_id
        INNER JOIN UserTemplates ut ON ut.temp_permission_id = tp.id
        INNER JOIN Users u ON u.id = ut.user_id
        WHERE u.id = ${user.id}
      `;

      let [userTemplate] = (await models.sequelize.query(query))[0];

      if (userTemplate.temp_id === 6) {
        return res.json({
          success: false,
          message: language.get['LOGIN_NOT_ALLOWED']
        });
      }

      if (user.email_verified) {
        if (!user.is_locked && !user.is_locked_auto) {
          try {
            argon2
              .verify(user.password, password)
              .then(match => {
                if (match) {
                  user
                    .update({ wrong_login_attempts: 0 })
                    .then(async function(data) {
                      models.Settings.findAll({
                        limit: 1,
                        order: [['id', 'DESC']]
                      })
                        .then(function(setting) {
                          let dat = {};
                          if (setting && setting[0]) {
                            dat.session_expiry = setting[0].session_expiry;
                            dat.purge_allowed = setting[0].purge_allowed;
                            dat.date_format = setting[0].date_format;
                            dat.gapiKey = setting[0].gapiKey;
                            dat.recaptcha_site_key =
                              setting[0].recaptcha_site_key;
                            dat.isFirstLogin = setting[0].isFirstLogin;
                          } else {
                            dat.session_expiry = 15;
                            dat.purge_allowed = true;
                            dat.date_format = 'dd/MM/yyyy HH:mm';
                            dat.gapiKey = 'GOOGLEMAPAPIKEY';
                            dat.recaptcha_site_key = 'RECAPTCHASITEKEY';
                            dat.isFirstLogin = false;
                          }

                          if (user.reset_password_time) {
                            let expiryTimeForPassword = moment().subtract(
                              ((setting[0] &&
                                setting[0].password_expiry_days) ||
                                15) * 24,
                              'hours'
                            );
                            const passwordExpiry = moment(
                              user.reset_password_time
                            );
                            if (
                              !expiryTimeForPassword.isBefore(passwordExpiry)
                            ) {
                              return res.send({
                                success: false,
                                message: language.get['PASSWORD_EXPIRED'],
                                token: ''
                              });
                            }
                          }

                          user.reset_password_time = moment();
                          user.save();

                          var expiryTime = dat;
                          var iat = moment().unix();
                          let jsonUser = {
                            id: user.id,
                            jti: uuid.v4(),
                            iat,
                            exp: iat + dat.session_expiry * 60
                          };
                          let unixExpiryTime = jsonUser.exp;

                          if (user.Roles) {
                            jsonUser.role = user.Roles[0].title;
                          }
                          jsonUser.roleId = user.roles;
                          jsonUser.tempId = userTemplate.temp_id;
                          var token = jwt.sign(
                            JSON.stringify(jsonUser),
                            process.env.JWT_SECRET_KEY
                          );
                          let usr = {
                            firstName: user.firstName,
                            email: user.email,
                            email_verified: user.email_verified,
                            is_mfa_assigned: user.is_mfa_assigned,
                            otp_image_data: user.otp_image_data,
                            is_first_login: user.is_first_login,
                            is_question_changed: user.is_question_changed,
                            is_dark: user?.is_dark
                          };
                          if (user.Roles) {
                            usr.role = user.Roles[0].title;
                          }
                          //this is the user id field
                          usr.usd = user.id;
                          Promise.resolve()
                            .then(() => {
                              return models.Settings.findOne({
                                order: [['id', 'DESC']],
                                attributes: ['admin_mfa']
                              });
                            })
                            .then(setting => {
                              let userResponse = {
                                success: true,
                                message:
                                  language.get['OTP_VERIFICATION_REQUIRED'],
                                admin_mfa: setting.admin_mfa,
                                user: {
                                  is_mfa_assigned: user.is_mfa_assigned
                                }
                              };

                              if (
                                !setting.admin_mfa ||
                                !isOtpVerificationRequired
                              ) {
                                userResponse = {
                                  success: true,
                                  message: language.get['LOGIN_SUCCESS'],
                                  token: token,
                                  user: usr,
                                  expiryTime: expiryTime,
                                  unixTime: unixExpiryTime,
                                  is_dark: user?.is_dark
                                };
                              }
                              const myRole =
                                user.Roles[0] && user.Roles[0].title;
                              req.user = {
                                id: user.id,
                                role: myRole
                              };
                              insertIntoAdminLogs(
                                getLoginMessage(
                                  user.firstName + ' ' + user.lastName,
                                  myRole
                                ),
                                req
                              );

                              res.status(200).send(userResponse);
                            });
                        })
                        .catch(error => {
                          console.log(error && (error.message || error));
                          res.send({
                            success: false,
                            message: language.get['USERNAME_PASSWORD_INVALID'],
                            token: ''
                          });
                        });
                    });
                } else {
                  let wrong_att = user.wrong_login_attempts + 1;
                  if (wrong_att >= 5) {
                    user.update({ is_locked_auto: true }).then(function(data) {
                      res.send({
                        success: false,
                        message: language.get['USERNAME_PASSWORD_INVALID'],
                        token: ''
                      });
                    });
                  } else {
                    user
                      .update({ wrong_login_attempts: wrong_att })
                      .then(function(data) {
                        res.send({
                          success: false,
                          message: language.get['USERNAME_PASSWORD_INVALID'],
                          token: ''
                        });
                      });
                  }
                }
              })
              .catch(err => {
                res.send({
                  success: false,
                  message: language.get['USERNAME_PASSWORD_INVALID'],
                  token: ''
                });
              });
          } catch (err) {
            res.send({
              success: false,
              message: language.get['USERNAME_PASSWORD_INVALID'],
              token: ''
            });
          }
        } else {
          if (user.is_locked_auto) {
            models.Settings.findOne({
              limit: 1,
              order: [['id', 'DESC']]
            }).then(function(setting) {
              const recaptchaConfigured =
                !!setting.recaptcha_site_key &&
                setting.recaptcha_site_key != 'RECAPTCHASITEKEY';
              models.Questions.findOne({
                where: {
                  id: user.question_1_id
                }
              }).then(function(question1) {
                models.Questions.findOne({
                  where: {
                    id: user.question_2_id
                  }
                }).then(function(question2) {
                  models.Questions.findOne({
                    where: {
                      id: user.question_3_id
                    }
                  }).then(function(question3) {
                    res.json({
                      success: false,
                      message: language.get['ACCOUNT_LOCKED_AUTO'],
                      question1: question1.question,
                      question2: question2.question,
                      question3: question3.question,
                      recaptchaConfigured
                    });
                  });
                });
              });
            });
          } else {
            res.json({
              success: false,
              message: language.get['ACCOUNT_LOCKED']
            });
          }
        }
      } else {
        res.send({
          success: false,
          message: language.get['VERIFY_EMAIL'],
          token: ''
        });
      }
    } else {
      res.send({
        success: false,
        message: language.get['USERNAME_PASSWORD_INVALID'],
        token: ''
      });
    }
  });
};

router.post('/', function(req, res, next) {
  const { username, password } = req.body;
  loginUser(username, password, true, res, req);
});

router.post('/logout', async function(req, res) {
  let reqUser = req.user;

  if (!reqUser) {
    reqUser = verifyJwtToken(req.header('authorization'));
  }

  let message = 'logout';
  let priority = logsPriority.PRIORITY_LOW;
  if (reqUser.id) {
    const myUser = await models.User.findOne({
      include: [{ model: models.Role }],
      attributes: { exclude: ['public_key', 'secret_key'] },
      where: {
        id: reqUser.id
      }
    });

    if (myUser) {
      const tempObj = getLogoutMessage(
        myUser.firstName + ' ' + myUser.lastName,
        myUser.Roles[0] && myUser.Roles[0].title
      );
      message = tempObj.action;
      priority = tempObj.process;
    }
  }

  if (reqUser) {
    models.AdminAccessLogs.create({
      userId: reqUser.id,
      action: message,
      priority
    })
      .then(adminAccessLog => {
        console.log(reqUser);
        blacklist.revoke(reqUser);
        res.json('logout');
      })
      .catch(error => {
        res.status(500).send(error);
      });
  } else {
    res.json('logout');
  }
});

module.exports = router;
