var express = require('express');
var router = express.Router();
const clientValidators = require('../../validators/clients');
const validate = require('../../validators/validate');
const validatePassword = require('../../middlewares/validatePassword');
const publicServiceAuthenticate = require('../../middlewares/publicServiceAuthenticate')
const verifyPermissions = require('../../middlewares/verifyPermissions');

var { 
  bulkActions,
  downloadService,
  status,
  approve,
  cancelPackage,
  sendOTPQrCode,
  verifyOtp,
  deregMFA,
  getInstallerStatus,
  sendLink,
  download,
  downloadService,
  updateClient,
  getClientDetail,
  unlockClient,
  generateService,
  getClients,
  generateLinOTPToken,
  addClient,
  deleteClient,
  updateClientRiskScore,
  generateOTPToken,
  sendOTPQr,
  updateClientAutoUpdates,
  getQuickClients,
  getClientServicesWithGatewayGraph,
  getClientId,
  bulkClientAutoUpdates
} = require('../../dao/clients');

router.get('/download/service/:key', downloadService);
router.get('/download/service-qrcode/:key', downloadService);
router.put('/status', verifyPermissions(['End User Clients', 'Server Side Client']), status);
router.put('/approve', verifyPermissions(['End User Clients', 'Server Side Client']), approve);
router.put('/cancelPackage', verifyPermissions(['End User Clients', 'Server Side Client']), cancelPackage);

router.post('/send/OTP_QR', verifyPermissions(['End User Clients', 'Server Side Client']), sendOTPQr);

// Get Client Graph
router.post('/graph', verifyPermissions(['End User Clients', 'Server Side Client']), getClientServicesWithGatewayGraph);

router.post('/sendOTPQrCode', verifyPermissions(['End User Clients', 'Server Side Client']), sendOTPQrCode);
router.post('/quick', verifyPermissions(['End User Clients', 'Server Side Client']), getQuickClients);
router.get('/verify/otp', verifyOtp);
router.post('/deregMFA', verifyPermissions(['End User Clients', 'Server Side Client']), deregMFA);
/* Get a single client detail */
router.post('/getInstallerStatus', verifyPermissions(['End User Clients', 'Server Side Client']), getInstallerStatus);
/* Get a single client detail */
router.post('/unlockClient', verifyPermissions(['End User Clients', 'Server Side Client']), unlockClient);
router.post('/sendLink', verifyPermissions(['End User Clients', 'Server Side Client']), sendLink);
router.get('/download/:key', download);
// generate installer package for client
router.get('/generate/service/:id/:state', generateService);
/* GET clients listing. */
router.get('/', verifyPermissions(['End User Clients','Server Side Client','Groups','Infrastructure Dashboard']), getClients);
router.get('/getClientId/:name', verifyPermissions(['Microsegments','Workloads']), getClientId);
router.put('/generateOTPToken/:device_id', verifyPermissions(['End User Clients', 'Server Side Client']), generateOTPToken);
router.put('/updates/:agent_id', clientValidators.updateClientAutoUpdates(), validate, verifyPermissions(['End User Clients', 'Server Side Client']), updateClientAutoUpdates);
router.put('/bulkUpdate', verifyPermissions(['End User Clients', 'Server Side Client']), bulkClientAutoUpdates);


router.put('/riskScore', publicServiceAuthenticate, clientValidators.updateClientRiskScore(), validate, verifyPermissions(['End User Clients', 'Server Side Client']), updateClientRiskScore)
router.put('/:type/:id', clientValidators.updateClient(), validate, verifyPermissions(['End User Clients', 'Server Side Client']), updateClient);
/* Get a single client detail */
router.get('/:type/:id', verifyPermissions(['End User Clients','Server Side Client']), getClientDetail);

router.post('/generateLinOTPToken/:id', verifyPermissions(['End User Clients', 'Server Side Client']), generateLinOTPToken);
//Bulk delete clients
router.delete('/deleteBulkClients', validatePassword, verifyPermissions(['End User Clients', 'Server Side Client']), bulkActions);
router.post('/bulkActions', verifyPermissions(['End User Clients', 'Server Side Client']), bulkActions);
// add new client
router.post('/:type', clientValidators.addClient(), validate, verifyPermissions(['End User Clients', 'Server Side Client']), addClient);
// delete a single user
router.delete('/:type/:id/:user', validatePassword, verifyPermissions(['End User Clients', 'Server Side Client']), deleteClient);


module.exports = router;
