var express = require('express');
var router = express.Router();
let path = require('path');
let appDir = path.dirname(require.main.filename);
appDir = appDir.substring(0, appDir.lastIndexOf('/'));
const validatePassword = require('../../middlewares/validatePassword');
const controllerValidars = require('../../validators/controllers');
var validate = require('../../validators/validate');
const verifyPermissions = require('../../middlewares/verifyPermissions');

const {
  getControllers,
  getController,
  status,
  deleteController,
  updateController,
  downloadUpgrade,
  downloadService,
  generateService,
  addController,
  getControllerHealthStats
} = require('../../dao/controllers');

/* GET controller listing. */
router.get('/', verifyPermissions(['Controllers','Gateways']), getControllers);

router.put('/status', verifyPermissions('Controllers'), status);

router.put('/:id', controllerValidars.updateController(), validate, verifyPermissions('Controllers'), updateController);

/* Get a single controller detail */
router.get('/:id', verifyPermissions('Controllers'), getController);

// add new controller
router.post('/', controllerValidars.addController(), validate, verifyPermissions('Controllers'), addController);

// delete a single controller
router.delete('/:id', validatePassword, verifyPermissions('Controllers'), deleteController);

router.get('/download/upgrade/:os', downloadUpgrade);

router.get('/download/service/:key', downloadService);

router.get('/generate/service/:id/:state', verifyPermissions('Controllers'), generateService);

router.get('/health/graph/:controllerId', getControllerHealthStats);

module.exports = router;
