var express = require('express');
var router = express.Router();
var models = require('../../models');
var path = require('path');
var url = require('url');
var http = require('http');
var https = require('https');
var archiver = require('archiver');
let language = require('./../common/language');
let helper = require('./../common/helper');
var projectConfig = require('../../config/config')['project'][
  process.env.NODE_ENV
];
var appDir = path.dirname(require.main.filename);
const moment = require('moment');
//For LDAP
const ldapjs = require('ldapjs');

router.get('/add/:token/', function(req, res, next) {
  console.log(req.params.token);
  let key = req.params.token;
  let message = '';
  let notfoundurl = 'https://' + req.headers.host + '/web/notfound';
  let validUrl =
    'https://' +
    req.headers.host +
    '/web/deviceid/' +
    encodeURI(req.params.token);
  if (key) {
    models.DownloadPackage.find({ where: { link_key: key, used: 0 } })
      .then(function(DP) {
        if (DP) {
          models.Client.find({
            where: {
              id: DP.client_id
            }
          }).then(function(client) {
            if (client) {
              const linkExpiry = moment(DP.link_key_expiry);
              if (moment().isBefore(linkExpiry)) {
                res.redirect(validUrl);
              } else {
                message =
                  'Device Onboarding page has expired, please request for a new link';
                helper.sendEmailLinkErrorHtmlResponse(message, 400, res);
              }
            }
          });
        } else {
          res.redirect(notfoundurl);
        }
      })
      .catch(err => {
        console.log(err && (err.message || err));
        res.redirect(notfoundurl);
      });
  } else {
    res.redirect(notfoundurl);
  }
});

router.post('/add/serve/', (req, res) => {
  var id = req.body.deviceId;
  let key = req.body.token;
  console.log(key);
  models.DownloadPackage.find({ where: { link_key: key, used: 0 } }).then(
    function(DP) {
      if (DP) {
        models.Client.find({
          where: {
            id: DP.client_id
          }
        })
          .then(function(client) {
            if (client) {
              const linkExpiry = moment.tz(
                DP.link_key_expiry,
                moment.tz.guess()
              );
              if (moment().isBefore(linkExpiry)) {
                client
                  .update({ device_hardware_id: id })
                  .then(function() {
                    DP.update({ updatedAt: moment(), used: 1 }).then(
                      function() {
                        client.isDeviceIdSet = true;
                        client.save();
                        res.json({
                          success: true,
                          message: 'Device id updated.'
                        });
                      }
                    );
                  })
                  .catch(err => {
                    res.json({
                      success: false,
                      message:
                        'Request can not be completed at this moment kindly try again.'
                    });
                  });
              } else {
                res.json({
                  success: false,
                  message: 'Token is expired, kindly resend the package again.'
                });
              }
            } else {
              res.json({
                success: false,
                message:
                  'Request can not be completed at this moment kindly try again.'
              });
            }
          })
          .catch(err => {
            console.log(err && (err.message || err));
            res.json({
              success: false,
              message: 'Client id against this request is not valid.'
            });
          });
      } else {
        res.json({
          success: false,
          message: 'Token is not valid.'
        });
      }
    }
  );
});

module.exports = router;
