const router = require('express').Router();
const verifyPermissions = require('../../middlewares/verifyPermissions');

const {
  getDevices,
  getWorkloadDevices,
  deleteDevice,
  getAvailableDevices,
  clientsforAvailableDevices,
  getDevice,
  createDevice,
  updateDevice,
  getDevicesForUser,
  toggleDarkeningState,
  downloadCsv,
  uploadCsv,
  bulkActions,
  getGlobalDarkening,
  devicesValidationTask,
  getAllLocations,
  getClients
} = require('../../dao/devices');

router.get('/csv/download', downloadCsv);

router.get('/workload', verifyPermissions('Workload Devices'), getWorkloadDevices);

router.get('/user/:id',verifyPermissions('Users'), getDevicesForUser);

router.get('/available/:serverSideDeviceFound/:mobileDeviceFound/:departmentId', verifyPermissions('Users'), getAvailableDevices);

router.put('/available-devices', verifyPermissions(['Devices','Users']), clientsforAvailableDevices);

router.get('/devicesValidation', verifyPermissions('Devices'), devicesValidationTask);

router.get('/locations', verifyPermissions(['Devices','Workload Devices']), getAllLocations);

router.get('/globalDarkening', verifyPermissions(['Devices','Workload Devices']), getGlobalDarkening);

router.get('/:id', verifyPermissions(['Devices','Workload Devices']), getDevice);

router.get('/', verifyPermissions('Devices'), getDevices);

router.post('/csv/upload', verifyPermissions('Devices'), uploadCsv);

router.post('/', verifyPermissions('Devices'), createDevice);

router.put('/bulk-actions', verifyPermissions('Devices'), bulkActions);

router.put('/toggleDarkening/:id', verifyPermissions('Devices'), toggleDarkeningState);

router.get('/clients/:id', verifyPermissions(['Devices','Workload Devices']), getClients);

router.put('/:id', verifyPermissions('Devices'), updateDevice);

router.delete('/:id/', verifyPermissions('Devices'), deleteDevice);

module.exports = router;
