var express = require('express');
var router = express.Router();
let path = require('path');
let appDir = path.dirname(require.main.filename);
appDir = appDir.substring(0, appDir.lastIndexOf('/'));
const validatePassword = require('../../middlewares/validatePassword');
const gatewatValidators = require('../../validators/gateways');
var validate = require('../../validators/validate');
const verifyPermissions = require('../../middlewares/verifyPermissions');
const rateLimiter = require('../../middlewares/rateLimiter')

const {
  downloadGateway,
  deleteGateway,
  getGateway,
  status,
  getGateways,
  getGatewayServices,
  updateGateway,
  addGateway,
  downloadPackage,
  gatewaysAvailable,
  getGatewaysOSAndProtocols,
  cloneGateway,
  getQuickGateways,
  getGatewaysHealth,
  getGatewayHealthStats
} = require('../../dao/gateways');

/* GET gateways listing. */
router.get('/', verifyPermissions(['Gateways', 'Gateways Groups']), getGateways);

router.get('/:id/services',verifyPermissions('Gateways'), getGatewayServices);

router.get('/quick', getQuickGateways);

router.get('/os/protocol', verifyPermissions('Gateways Groups'), getGatewaysOSAndProtocols);

router.get('/available', gatewaysAvailable);

router.put('/status', verifyPermissions('Gateways'), status);

router.put('/:id(\\d+)', gatewatValidators.updateGateway(), validate, verifyPermissions('Gateways'), updateGateway);

/* Get a single gateway detail */
router.get('/:id(\\d+)', verifyPermissions('Gateways'), getGateway);

router.post('/clone', verifyPermissions('Gateways'), cloneGateway);

// add new gateway
router.post('/', gatewatValidators.addGateway(), validate, verifyPermissions('Gateways'), addGateway);

// delete a single gateway
router.delete('/:id(\\d+)', validatePassword, verifyPermissions('Gateways'), deleteGateway);

router.delete('/otp/:id(\\d+)', verifyPermissions('Gateways'), deleteGateway);

//for download
router.get('/download/service/:key', downloadPackage);

//for download
router.get('/generate/service/:id/:state', verifyPermissions('Gateways'), rateLimiter ,downloadGateway);

//for summary table
router.get('/health/summary', verifyPermissions('Infrastructure Dashboard'), getGatewaysHealth);

// for graph
router.get('/health/graph/:gatewayId', getGatewayHealthStats);

module.exports = router;
