var express = require('express');
var router = express.Router();
var _ = require('lodash');
var models = require('../../models');
var commonUtils = require('../../reports/external-threat-integration/common-utils');
var dotenv = require('dotenv').config({ path: '/var/local/env/erisk' });
var organizationId = process.env.Organization_DB_ID
  ? parseInt(process.env.Organization_DB_ID)
  : 1;
var baseUrl = process.env.BASE_URL;
let language = require('./../common/language');

router.post('/', function(req, res, next) {
  var settings = req.body;
  models.GlobalSetting.create(settings)
    .then(function(data) {
      res.status(200).json(data);
    })
    .catch(function(error) {
      res.status(500).json(error.message);
    });
});

router.put('/:id', function(req, res) {
  models.GlobalSetting.find({
    where: {
      id: req.params['id']
    }
  }).then(function(settings) {
    if (settings) {
      settings.update(req.body.settings).then(function(data) {
        res.json({
          success: true,
          message: 'Update Successfully'
        });
      });
    }
  });
});

// fetch all the global settings
router.get('/', function(req, res, next) {
  models.GlobalSetting.findOne({
    attributes: ['summaryRiskScore']
  })
    .then(function(data) {
      res.status(200).json(data);
    })
    .catch(function(error) {
      res.status(500).json(error.message);
    });
});

router.get('/updateScore/:startDate/:endDate', function(req, res, next) {
  var data = req.params;
  var start_date = data.startDate;
  var end_date = data.endDate;
  data.apiUrl = baseUrl;
  data.start = start_date;
  data.end = end_date;
  _.unset(data, 'startDate');
  _.unset(data, 'endDate');
  if (
    start_date !== undefined &&
    start_date !== null &&
    start_date !== '' &&
    end_date !== undefined &&
    end_date !== null &&
    end_date !== ''
  ) {
    models.Organization.findOne({
      where: {
        id: organizationId,
        status: 1
      },
      attributes: ['username', 'password']
    }).then(function(organization) {
      data.username = organization.username;
      data.password = organization.password;
      return commonUtils
        .getRiskScore(data)
        .then(function(riskScore) {
          riskScore = parseInt(riskScore);
          models.GlobalSetting.findOne().then(function(settings) {
            if (settings) {
              settings
                .update({ summaryRiskScore: riskScore })
                .then(function(data) {
                  res.json({
                    summaryRiskScore: riskScore
                  });
                })
                .catch(function(error) {
                  res.json(error.message);
                });
            }
          });
        })
        .catch(function(error) {
          res.json(error.message);
        });
    });
  } else {
    res.json({ message: 'start and end dates required' });
  }
});

module.exports = router;
