const express = require('express');
const router = express.Router();
const groupValidators = require('../../validators/groups');
const validate = require('../../validators/validate');
const verifyPermissions = require('../../middlewares/verifyPermissions');

const {
  deleteGroup,
  updateGroup,
  getAvailableGroups,
  getClients,
  getGroup,
  addClient,
  deleteClient,
  addGroup,
  getGroups,
  getGroupClientsWithPagination,
  bulkDeleteGroups,
  getGroupClients
} = require('../../dao/groups');


router.get('/', verifyPermissions('Groups'), getGroups);

router.get('/clients/all/:groupId',verifyPermissions('Groups'), getGroupClients);

router.get('/available', getAvailableGroups);

router.post('/membership/:groupId/:clientId', verifyPermissions('Groups'), addClient);

router.delete('/membership/:groupId/:clientId', verifyPermissions('Groups'), deleteClient);

router.get('/:groupId', verifyPermissions('Groups'), getGroup);

router.get('/clients/:groupId', verifyPermissions('Groups'), getGroupClientsWithPagination);

router.get('/:groupId/:term', getClients);

router.post('/', groupValidators.addGroup(), validate, verifyPermissions('Groups'), addGroup);

router.put('/:id', groupValidators.updateGroup(), validate, verifyPermissions('Groups'), updateGroup);

router.delete('/bulk', verifyPermissions('Groups'), bulkDeleteGroups);

router.delete('/:id', verifyPermissions('Groups'), deleteGroup);


module.exports = router;