var express = require('express');
var router = express.Router();
var models = require('../../models');
let language = require('./../common/language');
const validatePassword = require('../../middlewares/validatePassword');
const idpValidations = require('../../validators/idps');
var validate = require('../../validators/validate');
const verifyPermissions = require('../../middlewares/verifyPermissions');

const {
  deleteIdp,
  addIdp,
  getIdp,
  udpateIdp,
  getIdps
} = require('../../dao/idps');

/* GET idps listing. */
router.get('/', verifyPermissions('IDP Management'), getIdps);

/* GET update idp. */
router.put('/:id', idpValidations.updateIdp(), validate, verifyPermissions('IDP Management'), udpateIdp);

/* Get a single idp detail */
router.get('/:id',  verifyPermissions('IDP Management'), getIdp);

// add new idp
router.post('/', idpValidations.addIdp(), validate, verifyPermissions('IDP Management'), addIdp);

// delete a single idp
router.delete('/:id', validatePassword, idpValidations.deleteIdp(), validate, verifyPermissions('IDP Management'), deleteIdp);

router.delete('/otp/:id', idpValidations.deleteIdp(), validate, deleteIdp);

module.exports = router;
