var express = require('express');
var router = express.Router();
var models = require('../../models');
let language = require('./../common/language');
const moment = require('moment');
const crypto = require('crypto');
var dirSyncManager = require('../common/dirsync-manager');
const fs = require('fs');
const execSync = require('child_process').execSync;
const projectConfig = require('../../config/config')['project'][
  process.env.NODE_ENV
];
const validatePassword = require('../../middlewares/validatePassword');
const verifyPermissions = require('../../middlewares/verifyPermissions');
const ldapValidators = require('../../validators/ldaps');
var validate = require('../../validators/validate');

const {
  getClients,
  updateLdap,
  duplicationCheck,
  addUser,
  getLdapUser,
  deleteLdap
} = require('../../dao/ldaps');

/*
const key = '5e4fa8357435e62533043862f2125787f1b2efc24c5bdca153cca418a08dea3e';
const iv = '02158c0cc4baa00d69a718d757e4d243';

function encrypt(text) {
  let cipher = crypto.createCipheriv(
    'aes-256-cbc',
    Buffer.from(key, 'hex'),
    Buffer.from(iv, 'hex')
  );
  let encrypted = cipher.update(text);
  encrypted = Buffer.concat([encrypted, cipher.final()]);
  return encrypted.toString('hex');
}

function decrypt(text) {
  let encryptedText = Buffer.from(text, 'hex');
  let decipher = crypto.createDecipheriv(
    'aes-256-cbc',
    Buffer.from(key, 'hex'),
    Buffer.from(iv, 'hex')
  );
  let decrypted = decipher.update(encryptedText);
  decrypted = Buffer.concat([decrypted, decipher.final()]);
  return decrypted.toString();
}
*/

/* GET users listing. */
router.get('/', verifyPermissions('LDAP Configuration'), getClients);

router.put('/:id', ldapValidators.updateLdapUser(), validate, verifyPermissions('LDAP Configuration'), updateLdap);

/* Get a single user detail */
router.get('/:id', verifyPermissions('LDAP Configuration'), getLdapUser);

// add new user
router.post('/', verifyPermissions('LDAP Configuration'), addUser);

router.post('/duplicationCheck', verifyPermissions('LDAP Configuration'), duplicationCheck);

// delete a single user
router.delete('/:id', validatePassword, ldapValidators.deleteLdap(), validate, verifyPermissions('LDAP Configuration'), deleteLdap);

router.delete('/otp/:id', ldapValidators.deleteLdap(), validate, deleteLdap);

module.exports = router;
