const router = require('express').Router();
const validatePassword = require('../../middlewares/validatePassword');
const localUserValidators = require('../../validators/localUsers');
const validate = require('../../validators/validate');
const verifyUser = require('../../middlewares/verifyUser');
const verifyPermissions = require('../../middlewares/verifyPermissions');

const {
  getLocalUser,
  getLocalUsers,
  resetPassword,
  createLocalUser,
  searchClient,
  updateLocalUser,
  deleteLocalUser,
  sendResetLink,
  token,
  checkValidToken,
  updatePassword,
  bulkPasswordChange,
  bulkDeleteUser,
  deviceAssociationWithUser,
  bulkActions,
  createCustomUserGroups,
  getCustomUserGroups,
  makeLDAPChanges,
  getUserDeviceRelatedPolicies,
  getAvailableLocalUsers,
  getLocalUserServices,
  getLocalUsersSearch,
  customGroupData,
  updateCustomGroup,
  deleteCustomGroup,
  deleteUserFromCustomGoup,
  getCustomUserGroupById,
  getUsersAndUserGroups,
  updateUserGroup,
  handleBulkdelete,
  handleBulkdeleteCurrentPage,
  paginationGroups,
  getLocalUsersFromDropDown,
  getUserGroupSearch,
  deleteCurrentPage,
  uploadCsv
} = require('../../dao/localUsers');

router.get('/checkValidToken/:token', checkValidToken);

router.get('/token/:token', token);

router.get('/customGroupData',verifyPermissions(['Users','User Groups']), customGroupData);

router.get('/paginationGroups', verifyPermissions('User Groups'), paginationGroups);

router.post(
  '/updatePassword/:token',
  localUserValidators.forgotPassword(),
  validate,
  verifyPermissions('Users'),
  updatePassword
);

router.get('/', verifyPermissions(['RDI Bridges','User Groups']), makeLDAPChanges, getLocalUsers);

router.get('/getLocalUsersFromDropDown', makeLDAPChanges, getLocalUsersFromDropDown);

router.get('/jql',verifyPermissions(['Users', 'User Groups']), makeLDAPChanges, getLocalUsersSearch);

router.get('/search', getUserGroupSearch);

router.get('/userdropdown', getUsersAndUserGroups);

router.put('/updateCustomGroup/:id', verifyPermissions('User Groups'), updateCustomGroup);

router.put('/updateUserGroup/:id', verifyPermissions(['Users','User Groups']), updateUserGroup);

router.delete('/deleteCustomGroup/:id', verifyPermissions('User Groups'), deleteCustomGroup);

router.delete('/deleteUserFromCustomGoup/:userId/:groupId', verifyPermissions('User Groups'), deleteUserFromCustomGoup);

router.delete('/handleBulkdelete', verifyPermissions('User Groups'), handleBulkdelete);

router.delete('/handleBulkdeleteCurrentPage', verifyPermissions('User Groups'), handleBulkdeleteCurrentPage);

router.get('/:id/services', verifyPermissions('Users'), getLocalUserServices);

router.get('/getCustomUserGroupData/:id', verifyPermissions('User Groups'), getCustomUserGroupById);

router.get('/available/:departmentId',verifyPermissions(['Devices','Workload Devices']), getAvailableLocalUsers);

router.get('/:id([0-9]+)', verifyPermissions(['Users','Devices','Workload Devices']), getLocalUser);

router.get('/resetPassword', resetPassword);

router.post('/sendResetLink', verifyUser, verifyPermissions('Users'), sendResetLink);

router.post(
  '/',
  localUserValidators.createLocalUser(),
  validate,
  verifyPermissions('Users'),
  createLocalUser
);

router.post('/searchClient', verifyPermissions('Users'), searchClient);

router.put('/bulk-actions', verifyPermissions('Users'), bulkActions);

router.put('/bulk-actions-delete-current', validatePassword, verifyPermissions('Users'), deleteCurrentPage);

router.put('/bulk-actions-delete-current-otp', verifyPermissions('Users'), deleteCurrentPage);

router.post('/customUserGroups', verifyPermissions('User Groups'), createCustomUserGroups);

router.get('/getCustomUserGroups', getCustomUserGroups);

router.put('/bulk/delete-users', validatePassword, verifyPermissions('Users'), bulkDeleteUser);

router.put('/bulk/password-change', validatePassword, verifyPermissions('Users'), bulkPasswordChange);

router.put('/:id/device', verifyPermissions('Users'), deviceAssociationWithUser);

router.put(
  '/:id',
  localUserValidators.updateLocalUser(),
  validate,
  verifyPermissions('Users'),
  updateLocalUser
);

router.delete('/:id/:userName', validatePassword, verifyPermissions('Users'), deleteLocalUser);
router.delete('/:id/:userName/otp', verifyPermissions('Users'), deleteLocalUser);

router.get('/device/policies/:userId/:deviceId', getUserDeviceRelatedPolicies);

router.post('/csv/upload', uploadCsv);

module.exports = router;
