const router = require('express').Router();
const verifyPermissions = require('../../middlewares/verifyPermissions');

const {  
  getOrganizations,
  deleteOrganization,
  getDepartments,
  createDepartment,
  deleteDepartment,
  updateDepartment,
  getOrganization,
  updateOrganization,
  createOrganization,
  getDepartment,
  getOrganizationNames,
  getOrganizationByName,
  getAllDepartments
} = require('../../dao/organizations');

router.get('/departments/:id', verifyPermissions(['Users','Devices','Workload Devices']), getDepartment);

router.get('/departments', verifyPermissions(['Workload Devices','Organizations']), getDepartments);

router.get('/all-depts', verifyPermissions(['Users','Devices']), getAllDepartments);

router.get('/names/:name', verifyPermissions(['Users','Devices']), getOrganizationByName);

router.get('/names',verifyPermissions(['Users','Devices','Workload Devices']), getOrganizationNames);

router.get('/:id', verifyPermissions(['Users','Devices','Workload Devices','Organizations']), getOrganization);

router.get('/', verifyPermissions('Organizations'), getOrganizations);

router.delete('/departments/:id', verifyPermissions('Organizations'), deleteDepartment);

router.delete('/:id', verifyPermissions('Organizations'), deleteOrganization);

router.post('/departments', verifyPermissions('Organizations'), createDepartment);

router.post('/', verifyPermissions('Organizations'), createOrganization);

router.put('/departments/:id', verifyPermissions('Organizations'), updateDepartment);

router.put('/:id', verifyPermissions('Organizations'), updateOrganization);

module.exports = router;
