const express = require('express');
const saasBridge = require('../../dao/saasBridge');
const saasValidors = require('../../validators/saasBridge');
const validate = require('../../validators/validate');
const validatePassword = require('../../middlewares/validatePassword');
const verifyPermissions = require('../../middlewares/verifyPermissions');

const router = express.Router();

router.get('/clients',verifyPermissions('SaaS Bridges'), saasValidors.getClients(), validate, saasBridge.getClients);

router.get('/services/:id',verifyPermissions('SaaS Bridges'), saasValidors.getServices(), validate, saasBridge.getClientServices);

router.get('/all', verifyPermissions('SaaS Bridges'), saasBridge.getBridges);

router.get('/:id',verifyPermissions('SaaS Bridges'), saasValidors.getBridge(), validate, saasBridge.getBridge);

router.post('/', saasValidors.createBridge(), validate, verifyPermissions('SaaS Bridges'), saasBridge.createBridge);

router.put('/:id', saasValidors.updateBridge(), validate, verifyPermissions('SaaS Bridges'), saasBridge.updateBridge);

router.delete('/:id', validatePassword, saasValidors.deleteBridge(), validate, verifyPermissions('SaaS Bridges'), saasBridge.deleteBridge);

router.get('/download/:id(\\d+)', saasBridge.downloadBridge);

router.get('/downloadPackage/:token', saasBridge.downloadBridgeFromLink);

module.exports = router;