var express = require('express');
var router = express.Router();
var path = require('path');
let appDir = path.dirname(require.main.filename);
appDir = appDir.substring(0, appDir.lastIndexOf('/'));
const settingsValidators = require('../../validators/settings');
var validate = require('../../validators/validate');
var verifyUser = require('../../middlewares/verifyUser');
const verifyPermissions = require('../../middlewares/verifyPermissions');

const {
  updateClientSettings,
  clientSettings,
  updateSettings,
  createNewSettings,
  getSettings,
  getTimeZones,
  mfaSettings,
  updateCoreSettings,
  forceLogOut,
  getAddressSpaceSettings,
  updateAddressSpaceSettings,
  getTimeSettings
} = require('../../dao/settings');

/* Get Setting */
router.get('/', verifyUser, getSettings);

router.get('/timesetttings', verifyUser, getTimeSettings);

router.get('/timeZones', getTimeZones);

router.get('/addressSpace', verifyPermissions(['Portal Settings','Client Settings','Core Settings']), getAddressSpaceSettings);

router.post('/addressSpace', verifyPermissions(['Core Settings']), updateAddressSpaceSettings);

router.post('/mfaSettings', mfaSettings);

/* CREATE new settings. */
router.post('/', createNewSettings);

//router.post('/addressSpace', changeAddressSpaceSettings);

router.put(
  '/:id(\\d+)',
  settingsValidators.updateSettings(),
  validate,
  verifyUser,
  verifyPermissions(['Portal Settings']),
  updateSettings
);

router.get('/clientSetting', verifyPermissions('Client Settings'), clientSettings);

router.put(
  '/clientSetting',
  settingsValidators.updateClientSettings(),
  validate,
  verifyPermissions('Client Settings'),
  updateClientSettings
);

router.put(
  '/coreSetting',
  settingsValidators.updateCoreSettings(),
  validate,
  verifyPermissions('Core Settings'),
  updateCoreSettings
);

router.patch('/force/logout', verifyPermissions('Client Settings'), forceLogOut);

module.exports = router;
