const router = require('express').Router();
const verifyPermissions = require('../../middlewares/verifyPermissions');

const { 
  getSoftwares,
  getSoftwaresProcessBinding,
  getForbiddenSoftwares,
  deleteSoftware,
  deleteSoftwareProcessBinding,
  getSoftware,
  getSoftwareProcessBinding,
  createSoftware,
  createSoftwareProcessBinding,
  updateSoftware,
  updateSoftwareProcessBinding,
  downloadCsv,
  uploadCsv,
  bulkActionsForbidden,
  bulkActions,
  deleteCurrentSoftwareMasterList,
  deleteCurrentSoftwareProcessBinding,
  bulkActionsSoftwareProcessBinding,
  importSoftwareList,
  getDistinctOS,
  removeObseleteVersions
} = require('../../dao/softwares');
router.get('/remove-obselete-versions', verifyPermissions('Software Master List'), removeObseleteVersions);

router.get('/oslist', verifyPermissions('Software Master List'), getDistinctOS);

router.get('/import',verifyPermissions('Software Master List'), importSoftwareList);

router.get('/', verifyPermissions('Software Master List') , getSoftwares);

router.get('/software-process-binding', verifyPermissions('Process Binding'), getSoftwaresProcessBinding);

router.get('/forbidden-softwares', verifyPermissions('Unauthorized Software'), getForbiddenSoftwares);

router.post('/', verifyPermissions('Software Master List'), createSoftware);

router.post('/create-software-process-binding', verifyPermissions('Process Binding'), createSoftwareProcessBinding);

// router.get('/csv/download', downloadCsv);

// router.post('/csv/upload', uploadCsv);

router.get('/:id', verifyPermissions('Software Master List'), getSoftware);

router.get('/software-process-binding/:id', verifyPermissions('Process Binding'), getSoftwareProcessBinding);

router.put('/bulk-actions-forbidden', verifyPermissions('Unauthorized Software'), bulkActionsForbidden);

router.put('/bulk-actions', verifyPermissions('Software Master List'), bulkActions);

router.put('/bulk-actions-delete-current', verifyPermissions('Software Master List'), deleteCurrentSoftwareMasterList);

router.put('/bulk-actions-software-process-binding', verifyPermissions('Process Binding'), bulkActionsSoftwareProcessBinding);

router.put('/bulk-actions-delete-current-process-binding', verifyPermissions('Process Binding'), deleteCurrentSoftwareProcessBinding);

router.put('/:id', verifyPermissions('Software Master List'), updateSoftware);

router.put('/software-process-binding/:id', verifyPermissions('Process Binding'), updateSoftwareProcessBinding);

router.delete('/:id/', verifyPermissions('Software Master List'), deleteSoftware);

router.delete('/software-process-binding/:id/', verifyPermissions('Process Binding'), deleteSoftwareProcessBinding);



module.exports = router;
