const express = require('express');
const router = express.Router();
const trustedHostValidators = require('../../validators/trusted-hosts');
var validate = require('../../validators/validate');
const {
  getHosts,
  createHost,
  updateHost,
  deleteHost,
  deleteTrustedHost
} = require('../../dao/trusted-hosts');
const verifyPermisions = require('../../middlewares/verifyPermissions');

router.get('/', verifyPermisions(['End User Clients','Server Side Client','Client Settings','Groups']), getHosts);

router.post('/:parentType/:parentId', trustedHostValidators.createHost(), validate, createHost);

router.put('/:parentType/:parentId/:id', trustedHostValidators.editHost(), validate, updateHost);

router.delete('/:parentType/:parentId/:id/:host/:port/:protocol', deleteHost);

router.delete('/:address/:port/:protocol/:parentId', deleteTrustedHost);

module.exports = router;
