var express = require('express');
var router = express.Router();
const {
    getUsers,
    resetMfa,
    deleteUser,
    duplicationCheck,
    getUser,
    updateUser,
    updateEmail,
    lockUser,
    unlockUser,
    resendVerification,
    createUser,
    regenerateUserKeys,
    updateSecurityQuestions,
    updateUserTheme
} = require('../../dao/users');

const validatePassword = require('../../middlewares/validatePassword');
const verifyPermissions = require('../../middlewares/verifyPermissions');
const userValidators = require('../../validators/users');
const validate = require('../../validators/validate');
const helper = require("../common/helper");
const moment = require("moment/moment");
const rateLimiter = require('../../middlewares/rateLimiter');

const onlyMe = (req, res, next) => {
    var sessionUser = req.user;
    var paramsId = req.params.id;

    if (sessionUser.id != paramsId) {
        return res.status(403).json('Not allowed');
    }
    next();
}


// GET users listing.
router.get('/', verifyPermissions('Admin Users'), getUsers);

router.post('/resetMfa', validatePassword, verifyPermissions('Admin Users'), resetMfa);

router.put('/updateTheme', validate, updateUserTheme);

router.put('/keys/:id', userValidators.regenerateKeys(), validate, verifyPermissions('Admin Users'), regenerateUserKeys)

// update single user
router.put('/:id', onlyMe, validatePassword, userValidators.updateUser(), validate, verifyPermissions('Admin Users'), updateUser);


// update user email
router.put('/email/:id', onlyMe, userValidators.updateEmail(), validate, verifyPermissions('Admin Users'), updateEmail);

// Lock single user
router.post('/lockUser/', validatePassword, verifyPermissions('Admin Users'), lockUser);

router.post('/updateSecurityQuestions/:id', updateSecurityQuestions);

// Unlock single user
router.post('/unlockUser/', validatePassword, verifyPermissions('Admin Users'), unlockUser);

/* Get a single user detail */
router.get('/:id', onlyMe, getUser);

router.post('/resend-verification/', rateLimiter, verifyPermissions('Admin Users'), resendVerification);

// add new user
router.post('/', rateLimiter,validatePassword, userValidators.createUser(), validate, verifyPermissions('Admin Users'), createUser);

// check email validity
router.post('/duplicationCheck', verifyPermissions('Admin Users'), duplicationCheck);

// delete a single user
router.delete('/delete/:id', validatePassword, verifyPermissions('Admin Users'), deleteUser);





module.exports = router;