var express = require('express');
var router = express.Router();
var models = require('../../models');
const argon2 = require('argon2');
var validator = require('validator');
var moment = require('moment');
let helper = require('./../common/helper');
let settings = require('./../common/settings');
let language = require('./../common/language');
var projectConfig = require('../../config/config')['project'][
  process.env.NODE_ENV
];

var emailLinkExpiryInMinutes = 60;

async function setExpiryTime() {
  emailLinkExpiryInMinutes = await settings.getExpirySettings();
}

router.get('/user/:key', (req, res) => {
  let key = new Buffer(req.params.key, 'base64').toString('ascii');
  let uri = req.headers.host;
  if (projectConfig.offPremUiUrl.includes(req.headers.host)) {
    uri = projectConfig.offPremUiUrl;
  } else {
    uri = projectConfig.onPremUiUrl;
  }

  models.User.find({
    where: {
      email_verification_link: key
      //email_verified: false
    }
  }).then(function(user) {
    if (user) {
      if (user.email_verified) {
        console.log('Already verified');
        res.redirect(uri + '/verification/verified');
        return;
      }
      const linkExpiry = moment.tz(
        user.email_verification_link_expiry,
        moment.tz.guess()
      );
      console.log(
        'User [' + user.firstName + '] verification expiry is: ',
        linkExpiry.format()
      );
      if (moment().isBefore(linkExpiry)) {
        console.log(
          'User [' + user.firstName + '] verification link is not expired'
        );
        user
          .update({
            email_verification_link_expiry: moment(),
            email_verified: true
          })
          .then(function() {
            console.log(
              'Updated expiry of user [' + user.firstName + '] to now'
            );
            res.redirect(uri + '/verification/success');
          });
      } else {
        res.redirect(uri + '/verification/failed');
      }
    } else {
      res.redirect(uri + '/verification/failed');
    }
  });
});

module.exports = router;
