const express = require('express');
const { createWhiteListedHosts, getWhiteListedHosts, updateWhiteListedHost, deleteWhiteListedHost } = require('../../dao/whiteListedHosts');
const router = express.Router();
const validator = require('../../validators/whitelisted-hosts');
const validate = require('../../validators/validate');
const verifyPermisions = require('../../middlewares/verifyPermissions');

router.route('/hosts/:controllerId')
    .post(
        validator.createHost(),
        validate,
        createWhiteListedHosts
    )
    .get(
        verifyPermisions('Controllers'),
        validator.getHosts(),
        validate,
        getWhiteListedHosts
    )
router.route('/:hostId')
        .put(
            validator.editHost(),
            validate,
            updateWhiteListedHost
        )
        .delete(
            validator.deleteHost(),
            validate,
            deleteWhiteListedHost
        )

module.exports = router;