var https = require('https');
var axios = require('axios');

const httpErrors = require('http-errors');
const FormData = require('form-data');
const fs = require('fs');
const jwt = require('jsonwebtoken');
const md5File = require('md5-file');
const models = require('../../models')

const httpsAgent = new https.Agent({
  rejectUnauthorized: false // (NOTE: this will disable client ssl verification)
});

const CONTENT_BASE_URL = process.env.CONTENT_URL + '/api/v1/links';

function generateToken(data) {
  return jwt.sign(data, process.env.JWT_SECRET_KEY, { expiresIn: '1m' });
}


module.exports.upload = async function ({os, folderPath, filename, component, originalInstaller}) {
  try {
    let ext = 'zip';

    if (os === 'android' && originalInstaller) {
      ext = 'apk';
    } else if (os === 'ios' && originalInstaller){
      ext = 'ipa';
    }

    const filePath = `${folderPath}${filename || 'installer'}.${ext}`;

    console.log('Uploading to location: ', filePath);

    if (!fs.existsSync(filePath)) {
      throw new httpErrors.NotFound('Package not available');
    }

    const hash = md5File.sync(filePath);

    const data = new FormData();
    data.append('os', os);
    data.append('component', component);
    data.append('hash', hash);
    data.append('file', fs.createReadStream(filePath));
    
    const token = generateToken({os});

    const config = {
      method: 'post',
      url: CONTENT_BASE_URL + '/upload',
      headers: { 
        ...data.getHeaders(),
        'Authorization': `Bearer ${token}`,
      },
      maxContentLength: Infinity,
      maxBodyLength: Infinity,
      data : data,
      httpsAgent
    };
    
    console.log('Sending upload request');
    const res = await axios(config);
    console.log('Request complete');
    return res.data;
  } catch (error) {
    throw error;
  }
}

module.exports.generateLink = async function (data) {
  // adding build version to save on content portal
  
  
  if(!['android', 'ios'].includes(data.os) ){
    const update = await models.Update.findOne({
      where: {
        buildOs: data.os,
        buildComponent: data.component,
      }
    })
    if(update) {
      data.version = update.buildVersion
    }
  }
  const token = generateToken({});
  try {

    const config = {
      method: 'post',
      url: CONTENT_BASE_URL + '/generate',
      headers: { 
        'Authorization': `Bearer ${token}`,
      },
      data : data,
      httpsAgent
    };
    const res = await axios(config);

    return res.data;
  } catch (error) {
    throw error;
  }
}