const { authenticator } = require('otplib');
const qrcode = require('qrcode');
const { promisify } = require('util');
const toDataURL = promisify(qrcode.toDataURL);

let fsExtra = require('fs-extra');
let projectConfig = require('../../config/config')['project'][
  process.env.NODE_ENV
];
let path = require('path');
let appDir = path.dirname(require.main.filename);
appDir = appDir.substring(0, appDir.lastIndexOf('/'));
let mailer = require('../../utils/mailer');
const { exec } = require('child_process');
const https = require('https');
const emailQueue = require('../../queues/email');
var jwt = require('jsonwebtoken');
const { cloneGateway } = require('../../dao/gateways');

class VerficationFailed extends Error {
  constructor() {
    super('Package verification failed');
    this.name = 'VerificationFailed';
  }
}

module.exports = {

  
  getHttpAgent:() => {
    return new https.Agent({
      rejectUnauthorized: false // (NOTE: this will disable client ssl verification)
    });
  },

  notEmptyQueryParam: function(data) {
    return data && data.trim() && data !== '' && data != 'undefined' && data != 'null' ? true: false;
  },

  isEmpty: function(data) {
    if (typeof data === 'object') {
      if (JSON.stringify(data) === '{}' || JSON.stringify(data) === '[]') {
        return true;
      } else if (!data) {
        return true;
      }
      return false;
    } else if (typeof data === 'string') {
      if (!data.trim()) {
        return true;
      }
      return false;
    } else if (typeof data === 'undefined') {
      return true;
    } else {
      return false;
    }
  },

  mkDir: function(dirPath) {
    return fsExtra
      .ensureDir(dirPath)
      .then(() => {
        console.log('created directory strcuture successfully ' + dirPath);
      })
      .catch(err => {
        console.log('Error saving attached file for Updates' + err);
        throw err;
      });
  },

  exec: function(fileName, parameters) {
    return new Promise((resolve, reject) => {
      exec(
        `${fileName}${parameters ? ` ${parameters.join(' ')}` : ''}`,
        err => {
          if (err) {
            return reject(err);
          }

          resolve();
        }
      );
    });
  },

  writeFile: function(fileName, data) {
    return new Promise((resolve, reject) => {
      fsExtra.writeFile(fileName, data, err => {
        if (err) {
          return reject(err);
        }

        resolve();
      });
    });
  },

  readFile: function(fileName) {
    return new Promise((resolve, reject) => {
      fsExtra.readFile(fileName, (err, data) => {
        if (err) {
          return reject(err);
        }

        resolve(data.toString());
      });
    });
  },
  /**
   * @description options should have recipient,
   * recipientName, emailSubject, templateFileName,
   * linkExpiryInMinutes, downloadLink keys.
   * */
  sendEmail: (options, extraOptions = {}) => {
    if (options.recipient) {
      
      let mailOptions = {
        recipients:options.recipient,
        subject:options.emailSubject,
        file:`${appDir}/templates/` + options.templateFileName,
        context:{
          user_name: options.recipientName,
          link_expiry_in_minutes: options.linkExpiryInMinutes,
          confirm_link: options.downloadLink,
          on_premise_confirm_link: options.onPremiseDownloadLink,
          off_premise_confirm_link: options.offPremiseDownloadLink,
          on_premise_Device_link: options.onPremDeviceIdLink,
          off_premise_Device_link: options.offPremDeviceIdLink,
          component: options.component,
          onPremApiUrl: options.onPremUiUrl,
          device_os: extraOptions.device_os,
          package_expiry_time: extraOptions.package_expiry_time,
          onPremWebMobileUrl: options.onPremWebMobileUrl,
          offPremWebMobileUrl: options.offPremWebMobileUrl
        }
      }

      emailQueue.addtoQueue(mailOptions);
      // mailer.sendMail(
      //   options.recipient,
      //   options.emailSubject,
      //   `${appDir}/templates/` + options.templateFileName,
      //   {
      //     user_name: options.recipientName,
      //     link_expiry_in_minutes: options.linkExpiryInMinutes,
      //     confirm_link: options.downloadLink,
      //     on_premise_confirm_link: options.onPremiseDownloadLink,
      //     off_premise_confirm_link: options.offPremiseDownloadLink,
      //     on_premise_Device_link: options.onPremDeviceIdLink,
      //     off_premise_Device_link: options.offPremDeviceIdLink,
      //     component: options.component,
      //     onPremApiUrl: options.onPremApiUrl,
      //     device_os: extraOptions.device_os,
      //     package_expiry_time: extraOptions.package_expiry_time,
      //     onPremWebMobileUrl: options.onPremWebMobileUrl,
      //     offPremWebMobileUrl: options.offPremWebMobileUrl
      //   }
      // );
    }
  },

  sendPackage:(options) => {
    if(options.recipient){

      let mailOptions = {
        recipients:options.recipient,
        subject:options.emailSubject,
        file:`${appDir}/templates/` + options.templateFileName,
        context:{
          user_name:options.recipientName
        },
        callback:undefined,
        attachments:options.attachments
      }

      emailQueue.addtoQueue(mailOptions);

      // mailer.sendMail(
      //   options.recipient,
      //   options.emailSubject,
      //   `${appDir}/templates/` + options.templateFileName,
      //   {
      //     user_name:options.recipientName
      //   },
      //   undefined,
      //   options.attachments
      // )
    }
  },

  sendQRCode: async (options) => {

    if (options.recipient) {
      if (options.OTP_image_data == null) {

        let otp_secret = authenticator.generateSecret();
        const otpauth = authenticator.keyuri(options.recipient, 'INVISILY ADMIN', otp_secret);
        const img = await toDataURL(otpauth);
        options.OTP_image_data = img.split(';base64,').pop().slice(0, -5);
        

        let mailOptions = {
          recipients:options.recipient,
          subject:options.emailSubject,
          file:`${appDir}/templates/` + options.templateFileName,
          context:{
            onPremApiUrl: options.onPremApiUrl,
            user_name: options.recipientName
          },
          callback:undefined,
          attachments:[
            {
              filename: 'code.png',
              content: options.OTP_image_data,
              encoding: 'base64'
            }
          ],
          client: options?.client
        }
        emailQueue.addtoQueue(mailOptions);
      }else{

      let mailOptions = {
        recipients:options.recipient,
        subject:options.emailSubject,
        file:`${appDir}/templates/` + options.templateFileName,
        context:{
          onPremApiUrl: options.onPremUiUrl,
          user_name: options.recipientName
        },
        callback:undefined,
        attachments:[
          {
            filename: 'code.png',
            content: options.OTP_image_data,
            encoding: 'base64'
          }
        ],
        client: options?.client
      }
      emailQueue.addtoQueue(mailOptions);
    }
      

      // mailer.sendMail(
      //   options.recipient,
      //   options.emailSubject,
      //   `${appDir}/templates/` + options.templateFileName,
      //   {
      //     onPremApiUrl: options.onPremApiUrl,
      //     user_name: options.recipientName
      //   },
      //   undefined,
      //   [
      //     {
      //       filename: 'code.png',
      //       content: options.OTP_image_data,
      //       encoding: 'base64'
      //     }
      //   ]
      // );
  }
  },

  sendLocalUserResetEmail: options => {
    if (options.recipient) {

      let mailOptions = {
        recipients:options.recipient,
        subject:options.emailSubject,
        file:`${appDir}/templates/` + options.templateFileName,
        context:{
          user_name: options.recipientName,
          email_expiry_time: options.expiryTime,
          on_premise_installer_link: options.onPremiseDownloadLink
        }
      }

      emailQueue.addtoQueue(mailOptions);

      // mailer.sendMail(
      //   options.recipient,
      //   options.emailSubject,
      //   `${appDir}/templates/` + options.templateFileName,
      //   {
      //     user_name: options.recipientName,
      //     email_expiry_time: options.expiryTime,
      //     on_premise_installer_link: options.onPremiseDownloadLink
      //   }
      // );
    }
  },

  sendInstallerEmail: options => {
    if (options.recipient) {
     
      let mailOptions = {
        recipients:options.recipient,
        subject:options.emailSubject,
        file:`${appDir}/templates/` + options.templateFileName,
        context:{
          user_name: options.recipientName,
          on_premise_installer_link: options.onPremiseDownloadLink,
          off_premise_installer_link: options.offPremiseDownloadLink,
          onPremApiUrl: options.onPremUiUrl,
          device_os: options.device_os,
          package_expiry_time: options.package_expiry_time,
          portal_link:options.portal_link
        }
      }

      emailQueue.addtoQueue(mailOptions);
      
      // mailer.sendMail(
      //   options.recipient,
      //   options.emailSubject,
      //   `${appDir}/templates/` + options.templateFileName,
      //   {
      //     user_name: options.recipientName,
      //     on_premise_installer_link: options.onPremiseDownloadLink,
      //     off_premise_installer_link: options.offPremiseDownloadLink,
      //     onPremApiUrl: options.onPremApiUrl,
      //     device_os: options.device_os,
      //     package_expiry_time: options.package_expiry_time
      //   }
      // );
    }
  },

  sendVerificationEmail: options => {
    if (options.recipient) {

      let mailOptions = {
        recipients:options.recipient,
        subject:options.emailSubject,
        file:`${appDir}/templates/` + options.templateFileName,
        context:{
          user_name: options.recipientName,
          link_expiry_in_minutes: options.linkExpiryInMinutes,
          confirm_link: options.downloadLink,
          on_premise_confirm_link: options.on_premise_confirm_link,
          off_premise_confirm_link: options.off_premise_confirm_link,
          inv_logo_url: options.invLogoLink,
          ebryx_logo_url: options.ebrLogoLink
        }
      }

      emailQueue.addtoQueue(mailOptions);
      // mailer.sendMail(
      //   options.recipient,
      //   options.emailSubject,
      //   `${appDir}/templates/` + options.templateFileName,
      //   {
      //     user_name: options.recipientName,
      //     link_expiry_in_minutes: options.linkExpiryInMinutes,
      //     confirm_link: options.downloadLink,
      //     on_premise_confirm_link: options.on_premise_confirm_link,
      //     off_premise_confirm_link: options.off_premise_confirm_link,
      //     inv_logo_url: options.invLogoLink,
      //     ebryx_logo_url: options.ebrLogoLink
      //   }
      // );
    }
  },


  sendLocalUserPasswordResetMail: options => {
    if (options.recipient) {

      let mailOptions = {
        recipients:options.recipient,
        subject:options.emailSubject,
        file:`${appDir}/templates/` + options.templateFileName,
        context:{
          name: options.name,
          username: options.username,
          password: options.password
        }
      }

      emailQueue.addtoQueue(mailOptions);
    }
  },

  sendPasswordResetMail: options => {
    if (options.recipient) {

      let mailOptions = {
        recipients:options.recipient,
        subject:options.emailSubject,
        file:`${appDir}/templates/` + options.templateFileName,
        context:{
          name: options.name,
          userName: options.userName,
          password: options.password
        }
      }

      emailQueue.addtoQueue(mailOptions);
    }
  },

  verifySignatureFile: (packageFilePath, sigFilePath, publicKeyPath) => {
    return new Promise((resolve, reject) => {
      exec(
        `openssl dgst -verify ${publicKeyPath} -keyform PEM -sha256 -signature ${sigFilePath} -binary ${packageFilePath}`,
        (error, stdout, stderr) => {
          if (error) {
            console.log('Error in verifying package ', error);
            fsExtra.removeSync(`${sigFilePath}`);
            reject(new VerficationFailed());
          } else {
            console.log('Package File verified with sign file uploaded');
            resolve();
          }
        }
      );
    });
  },

  sendEmailLinkErrorHtmlResponse: (message, code, res) => {
    console.log(message);
    res.writeHead(code, { 'Content-Type': 'text/html' });
    res.write(
      '<!DOCTYPE html><html><body><div style="text-align:center; color: red; font-size: 20px;">' +
        message +
        '</div></body></html>'
    );
    res.end();
  },

  sendNewLocalUserMail: (email, name, username, password) => {
    let mailOptions = {
      recipients: email,
      subject: `Invisily New User`,
      file: `${appDir}/templates/new-client-password.html`,
      context: {
        name: name,
        username: username,
        password: password
      }
    }
    emailQueue.addtoQueue(mailOptions);
  },

  sendPasswordExpiryMail: (email, name) => {
    let mailOptions = {
      recipients: email,
      subject: `Invisily User Password Expired`,
      file: `${appDir}/templates/client-password-expiry.html`,
      context: {
        name: name,
        version: '4.7.0'
      }
    }
    
    emailQueue.addtoQueue(mailOptions);
  },

  verifyJwtToken: (token) => {
    try {
      if (typeof token !== 'string') {
        throw new Error('Invalid token format');
      }

      if(token.includes('Bearer')) { 
        token = token.replace('Bearer ',''); 
      }
      const decoded = jwt.verify(token, process.env.JWT_SECRET_KEY)
      return decoded
    }
    catch(e) {
      if (e.name === 'TokenExpiredError') {
        console.log('JWT Token has expired')
      }
      else {
        console.log('jwt decryption error ', e)
      }

      return false
    }
  }
};
