let fsExtra = require('fs-extra');
let projectConfig = require('../../config/config')['project'][
  process.env.NODE_ENV
];
let path = require('path');
let appDir = path.dirname(require.main.filename);
appDir = appDir.substring(0, appDir.lastIndexOf('/'));
var models = require('../../models');
const moment = require('moment');

module.exports = {
  purgeClientLog: () => {
    let CONNECTION_ATTEMPT_TIME = {};
    let whereClause = {};
    return models.Settings.findAll({
      limit: 1,
      order: [['id', 'DESC']]
    })
      .then(function(data) {
        if (data[0].purge_allowed) {
          console.log('----------------AUTO PURGE TURNED ON----------------');
          if (data[0].auto_purge_days) {
            let purge_time = moment().subtract(
              data[0].auto_purge_days * 24,
              'hours'
            );
            CONNECTION_ATTEMPT_TIME.lte = purge_time;
            whereClause.CONNECTION_ATTEMPT_TIME = CONNECTION_ATTEMPT_TIME;
            //purge
            models.Connection.destroy({
              where: whereClause
            }).then(function(client) {
              data[0].last_purge_date = moment().add(0, 'hours');
              data[0].save().then(function(x) {
                console.log('----------------PURGED-------------------');
                return true;
              });
            });
            return true;
          } else {
            console.log(
              '------------------No time to purge--------------------'
            );
            return true;
          }
        } else {
          console.log('----------------AUTO PURGE TURNED OFF----------------');
        }
      })
      .catch(error => {
        console.log(error && (error.message || error));
        console.log("Can't purge logs");
        return false;
      });
  },

  getSmtpSettings: () => {
    return models.Settings.findAll({
      limit: 1,
      order: [['id', 'DESC']]
    })
      .then(function(data) {
        let jsonD = {
          transporter: {
            auth: {}
          }
        };
        jsonD.transporter.service = data[0].smtp_service;
        if (data[0].smtp_service === 'local') {
          jsonD.transporter.service = '';
          jsonD.transporter.port = data[0].smtp_port;
          jsonD.transporter.host = data[0].smtp_host;
          jsonD.transporter.tls = {};
          jsonD.transporter.tls.rejectUnauthorized = false;
        }else if(data[0].smtp_service === 'Gmail') {
          jsonD.transporter.secure = true;
          jsonD.transporter.tls = {};
          jsonD.transporter.tls.servername = 'smtp.gmail.com';
        }
        jsonD.transporter.auth.user = data[0].smtp_user;
        jsonD.transporter.auth.pass = data[0].smtp_password;
        jsonD.from = data[0].smtp_from;
        jsonD.transporter.pool = true;
        jsonD.transporter.maxConnections = 1;
        return jsonD;
      })
      .catch(error => {
        var jsonD = {
          transporter: {
            auth: {}
          }
        };
        jsonD.transporter.service = '';
        jsonD.transporter.auth.user = '';
        jsonD.transporter.auth.pass = '';
        jsonD.from = '';
        return jsonD;
      });
  },

  getExpirySettings: () => {
    return models.Settings.findAll({
      limit: 1,
      order: [['id', 'DESC']]
    })
      .then(function(data) {
        if (data && data[0]) {
          return data[0].package_link_expiry;
        } else {
          return 60;
        }
      })
      .catch(error => {
        console.log(error && (error.message || error));
        return 60;
      });
  },

  getUpdateSettings: () => {
    return models.Settings.findAll({
      limit: 1,
      order: [['id', 'DESC']]
    })
      .then(function(data) {
        if (data) {
          return data[0].auto_update_state;
        } else {
          return false;
        }
      })
      .catch(error => {
        console.log(error && (error.message || error));
        return false;
      });
  },

  getSessionSettings: () => {
    return models.Settings.findAll({
      limit: 1,
      order: [['id', 'DESC']]
    })
      .then(function(data) {
        if (data) {
          return {
            session_expiry: data[0].session_expiry,
            purge_allowed: data[0].purge_allowed,
            date_format: data[0].date_format,
            gapiKey: data[0].gapiKey,
            recaptcha_site_key: data[0].recaptcha_site_key
          };
        } else {
          return {
            session_expiry: 15,
            purge_allowed: true,
            date_format: 'dd/MM/yyyy HH:mm',
            gapiKey: 'GOOGLEMAPAPIKEY',
            recaptcha_site_key: 'RECAPTCHASITEKEY'
          };
        }
      })
      .catch(error => {
        console.log(error && (error.message || error));
        return {
          session_expiry: 15,
          purge_allowed: true,
          date_format: 'dd/MM/yyyy HH:mm',
          gapiKey: 'GOOGLEMAPAPIKEY',
          recaptcha_site_key: 'RECAPTCHASITEKEY'
        };
      });
  }
};
