#!/bin/bash

# Set SQL Server connection parameters
# Get the Creds from the Env File
SQL_HOST=$(< /var/local/env/sdp  grep "PRODUCTION_HOST=" | awk -F '=' '{print $2}')
SQL_USER=$(< /var/local/env/sdp  grep "PRODUCTION_USERNAME=" | awk -F '=' '{print $2}')
SQL_PASSWORD=$(< /var/local/env/sdp  grep "PRODUCTION_PASSWORD=" | awk -F '=' '{print $2}')
SQL_PORT="3306"
SQL_SDP_DATABASE="sdp"
SQL_OCS_DATABASE="ocs"

# Writing Query
SQL_QUERY_MASTER_LIST="SELECT sml.id
    FROM ${SQL_SDP_DATABASE}.SoftwareMasterLists sml
    LEFT JOIN (
        SELECT DISTINCT software_version.VERSION as software_version_name, software_version.ID, DeviceInventories.is_reference_machine
        FROM ${SQL_SDP_DATABASE}.DeviceInventories as DeviceInventories
        LEFT JOIN ${SQL_OCS_DATABASE}.hardware  as hardware on hardware.UUID = DeviceInventories.deviceSerialNumber
        LEFT JOIN ${SQL_OCS_DATABASE}.software as software on software.HARDWARE_ID = hardware.id
        LEFT JOIN ${SQL_OCS_DATABASE}.software_version software_version on software_version.ID = software.VERSION_ID 
        WHERE DeviceInventories.is_reference_machine= true
    ) r
    ON sml.software_version = r.software_version_name
    LEFT JOIN ${SQL_SDP_DATABASE}.DeviceInventories di
    ON r.is_reference_machine = di.is_reference_machine
    WHERE r.software_version_name IS NULL"


# Connect to MySQL and execute the query, and output only the IDs
to_delete_software=($(mysql -u ${SQL_USER} -p${SQL_PASSWORD} -h ${SQL_HOST} ${SQL_SDP_DATABASE} -N -B -e "${SQL_QUERY_MASTER_LIST}" 2>/dev/null | cut -f1))

# Check if the array is not empty and delete the records if it is not
if [ ${#to_delete_software[@]} -gt 0 ]; then
  # Convert the array to a comma-separated list
  to_delete_list=$(IFS=, ; echo "${to_delete_software[*]}")
  
  # Delete the records using the list of IDs
  mysql -u ${SQL_USER} -p${SQL_PASSWORD} -h ${SQL_HOST} ${SQL_SDP_DATABASE} -e "DELETE FROM ${SQL_SDP_DATABASE}.SoftwareMasterLists WHERE id IN (${to_delete_list})" 2>/dev/null
fi

# Print the contents of the file to the console and delete them
# mysql -u ${SQL_USER} -p${SQL_PASSWORD} -h ${SQL_HOST} ${SQL_SDP_DATABASE} -e "${SQL_QUERY_MASTER_LIST}" > masterlist.txt
# cat masterlist.txt | xargs -I{} mysql -u ${SQL_USER} -p${SQL_PASSWORD} -h ${SQL_HOST} ${SQL_SDP_DATABASE} -e "DELETE FROM ${SQL_SDP_DATABASE}.SoftwareMasterLists WHERE id={};"
# rm -rf masterlist.txt

