#!/bin/bash

SCRIPT_NAME="$0"

CONTEXT="[INSTALL_CENTOS_SERVER]"

log() {
  echo "$CONTEXT" "$@"
}

# error handling
set -eE -o functrace

failure() {
  local LINE=$1
  local CMD=$2
  echo "$SCRIPT_NAME: $LINE | $CMD"
}
trap 'failure ${LINENO} "$BASH_COMMAND"' ERR

log "Running [$SCRIPT_NAME]"

log "Checking if user is root"
if (( EUID != 0 )); then
  log "Please run with sudo. Exiting..."
  exit 1
fi

log "Checking if client is already installed"
if rpm -q invisily-client; then
  log "Invisily Client is already installed!"
  log "Please uninstall existing package first and try again."
  log "Run: sudo yum remove -y invisily-client"
  exit 1
fi

log "Checking if settings and rpm files exist"
for FILE in settings.cfg invisily_client.rpm; do
  if [[ ! -f $FILE ]]; then
    log "$FILE does not exist! Exiting..."
    exit 1
  fi
done

log "Installing client"
if ! rpm -ivh ./invisily_client.rpm --nodeps; then
  log "Error while installing invisily-client. Exiting..."
  exit 1
fi

log "Placing settings.cfg file"
cp ./settings.cfg /opt/invisily/linuxclientd/bin

log "Restarting Invisily service to load updated settings"
systemctl restart invisily_backend.service

log "Invisily Client is installed and running!"
