const { io } = require('./bin/www');
const { getAlertLogs, getInfraPermissionsNotification, getAdminPermissionsNotification } = require('./cronjobs/alert-logs');

let clientState = {}; // Object to store client state { socketId: tempId }

// Socket.io connection event
console.log('This is the SocketIo Code');
io.on('connection', socket => {
  console.log('Socket connection established');

  socket.on('client_connected', data => {
    console.log('Client connected with TemplateID: ' + data.value + ' and SocketID: ' + socket.id);
    clientState[socket.id] = data.value;
  });

  // Join a channel
  socket.on('join', channel => {
    socket.join(channel);
    console.log(`User joined channel: ${channel}`);
  });

  // Leave a channel
  socket.on('leave', channel => {
    socket.leave(channel);
    console.log(`User left channel: ${channel}`);
  });

  // Handle disconnection
  socket.on('disconnect', () => {
    console.log('Client disconnected with SocketID:', socket.id);
    delete clientState[socket.id];
  });
});

setInterval(async function() {
  let alerts;
  for (const [socketId, tempId] of Object.entries(clientState)) {
    if (tempId === 2) {
      if (!alerts) {
        alerts = await getAlertLogs(tempId);
      }
      if (alerts && alerts.length > 0) {
        console.log("Sending alerts to Client having SocketID:", socketId);
        io.to(socketId).emit('msg_to_client', alerts);
      }
    }
  }
}, 60000);

setInterval(async function() {
  let infraAlerts;
  let adminAlerts;
  for (const [socketId, tempId] of Object.entries(clientState)) {
    if (tempId === 1) {
      if (!infraAlerts) {
        infraAlerts = await getInfraPermissionsNotification(tempId);
      }
      if (infraAlerts && infraAlerts.length > 0) {
        console.log("Sending additional permissions alerts to Infra Client with SocketID:", socketId);
        io.to(socketId).emit('permissions', infraAlerts, tempId);
      }
    }
    if (tempId === 2) {
      if (!adminAlerts) {
        adminAlerts = await getAdminPermissionsNotification(tempId);
      }
      if (adminAlerts && adminAlerts.length > 0) {
        console.log("Sending additional permissions alerts to Admin Client with SocketID:", socketId);
        io.to(socketId).emit('permissions', adminAlerts, tempId);
      }
    }
  }
}, 60000);
