const { body, param } = require('express-validator');

const addClient = () => [
    ...name(body),
    ...device_hardware_id(body),
    ...device_os(body),
    ...user(body)
];

const updateClient = () => [
    ...name(body),
    ...device_hardware_id(body),
    ...device_os(body),
    ...user(body),
    ...id(param),
    ...onboarding_required(param)
];

const updateClientRiskScore = () => [
    ...agentId(body),
    ...riskScore(body)
];

const updateClientAutoUpdates = () => [
    ...agentId(param),
    ...auto_updates(body)
];

const id = check => [
    check('id')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .withMessage('Id is required.')
]

const agentId = check => [
    check('agent_id')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .withMessage('agent_id is required.')
]

const auto_updates = check => [
    check('auto_updates')
    .exists()
    .not()
    .isEmpty()
    .isBoolean()
    .withMessage('auto_updates is required and must be a boolean value')
]

const name = check => [
    check('client.Name')
    .exists()
    .not()
    .isEmpty()
    .trim()
    .withMessage('Name is required.')
]


const riskScore = check => [
    check('risk_score')
    .exists()
    .not()
    .isEmpty()
    .isInt({min: 1, max: 10})
    .withMessage('Risk score must be an integer and ranges between 1 to 10.')
]

const email = check => [
    check('client.Email')
    .exists()
    .not()
    .isEmpty()
    .isEmail()
    .normalizeEmail({ gmail_remove_dots: false })
    .withMessage('Email is required.')
]

const device_hardware_id = check => [
    check('client.device_hardware_id')
    .exists()
    .not()
    .isEmpty()
    .trim()
];

const user = check => [
    check('client.user')
    .exists()
    .not()
    .isEmpty()
    .trim()
];

const device_os = check => [
    check('client.device_os')
    .exists()
    .not()
    .isEmpty()
    .trim()
];

const onboarding_required = check => [
    check('client.onboarding_required')
    .exists()
    .not()
    .isEmpty()
    .isBoolean()
]

module.exports = {
    addClient,
    updateClient,
    updateClientRiskScore,
    updateClientAutoUpdates
}
