const { body, param } = require('express-validator');

const addController = () => [
    ...Name(body),
    ...email(body),
    ...ipv4_address(body),
    ...port(body),
    ...public_ipv4_address(body),
    ...public_ipv4_port(body),
    ...spa_port(body),
]

const updateController = () => [
    ...Name(body),
    ...email(body),
    ...ipv4_address(body),
    ...port(body),
    ...public_ipv4_address(body),
    ...public_ipv4_port(body),
    ...spa_port(body),
    ...Id(param)
]

const Id = check => [
    check('id')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .withMessage('Id is required.')
]

const Name = check => [
    check('controller.Name')
    .exists()
    .not()
    .isEmpty()
    .withMessage('Name is required.')
]

const email = check => [
    check('controller.email')
    .exists()
    .not()
    .isEmpty()
    .isEmail()
    .normalizeEmail({ gmail_remove_dots: false })
    .withMessage('Email is required.')
]

const ipv4_address = check => [
    check('controller.ipv4_address')
    .exists()
    .not()
    .isEmpty()
    .custom(value => {
        const validIpExpression = /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/;
        if (!validIpExpression.test(value)) {
            throw new Error("Invalid private IP.")
        }
        return true;
    })
    .withMessage('Private IP is required.')
]

const port = check => [
    check('controller.port')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .custom(value => {
        const validPortExpression = /^([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])$/;
        if (!validPortExpression.test(value)) {
            throw new Error('Invalid private port.');
        }
        return true;
    })
    .withMessage('Private port is required.')
]

const public_ipv4_address = check => [
    check('controller.public_ipv4_address')
    .exists()
    .not()
    .isEmpty()
    .custom(value => {
        const validIpExpression = /^(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.(?:25[0-5]|2[0-4]\d|[01]?\d\d?)|\*)(?:,\s*(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.(?:25[0-5]|2[0-4]\d|[01]?\d\d?)|\*))*$/;
        if (!validIpExpression.test(value)) {
            throw new Error("Public IP list contain invalid IP(s).")
        }
    
        return true;
    })
    .withMessage('public IP is required.')
]

const public_ipv4_port = check => [
    check('controller.public_ipv4_port')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .custom(value => {
        const validPortExpression = /^([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])$/;
        if (!validPortExpression.test(value)) {
            throw new Error('Invalid public port.');
        }
        return true;
    })
    .withMessage('public port is required.')
]

const spa_port = check => [
    check('controller.spa_port')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .custom(value => {
        const validPortExpression = /^([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])$/;

        if (!validPortExpression.test(value)) {
            throw new Error('Invalid SPA port.');
        }

        return true;
    })
    .withMessage('SPA port is required.')
]

module.exports = {
    addController,
    updateController,
}