const { body, param} = require('express-validator');


const addInventory = () => [
    ...deviceSerialNumber(body),
    ...deviceTag(body),
    ...userName(body)
];

const updateInventory = () => [
    ...deviceSerialNumber(body),
    ...deviceTag(body),
    ...userName(body),
    ...id(param)
];

const id = check => [
    check('id')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
]

const deviceSerialNumber = check => [
    check('inventory.deviceSerialNumber')
    .exists()
    .not()
    .isEmpty()
    .trim()
    .withMessage('Invalid Serial Number')
];


const deviceTag = check => [
    check('inventory.deviceTag')
    .exists()
    .not()
    .isEmpty()
    .trim()
    .custom(value => {
        const deviceTagExpression = /^[0-9a-zA-Z-]{1,200}$/;

        if (!deviceTagExpression.test(value)) {
            throw new Error('Invalid device tag');
        }

        return true;
    })
]


const userName = check => [
    check('inventory.userName')
    .exists()
    .not()
    .isEmpty()
    .trim()
    .withMessage('Invalid username.')
]


module.exports = {
    addInventory,
    updateInventory
}