const { body, param } = require('express-validator');

const createGatewayGroup = () => {
  return [
    ...name(body),
    ...description(body),
    ...gateways(body),
    ...addAllGateways(body)
  ];
};

const updateGatewayGroup = () => {
  return [
    ...id(param),
    ...description(body),
    ...name(body),
    ...gateways(body),
    ...appendGateways(body),
    ...addAllGateways(body)
  ];
};

const deleteGatewayGroup = () => {
  return [...id(param)];
};


const id = check => [
  check('id')
    .exists()
    .isNumeric()
    .not()
    .isEmpty()
    .withMessage('Id is required.')
];

const name = check => [
  check('name')
    .exists()
    .not()
    .isEmpty()
    .withMessage('Name is required.')
];

const description = check => [
  check('description')
  .exists()
  .not()
  .isEmpty()
  .withMessage('Description is required.')
];

const gateways = check => [
  check('gateways')
    .exists()
    .custom((value, { req }) => {
      if (!req.body.all_gateways && value && !Array.isArray(value)) {
        throw new Error('Gateways should be an array');
      }
      return true;
    })
];

const addAllGateways = check => [
  check('addAllGateways')
  .isBoolean()
  .withMessage('addAllGateways must be a boolean value')
]


const appendGateways = check => [
  check('appendGateways')
  .isBoolean()
  .exists()
  .withMessage('appendGateways must be a boolean value')
]


module.exports = {
  createGatewayGroup,
  updateGatewayGroup,
  deleteGatewayGroup,
}
