const { body, param, query } = require('express-validator');
const httpErrors = require('http-errors');

const getBridge = () => [
    ...Id(param)
];

const createBridge = () => [
    ...name(body),
    ...clientId(body),
    ...serviceId(body),
    ...authServiceClientSecret(body),
    ...authServiceTokenUrl(body),
    ...authServiceUrl(body),
    ...authServiceClientId(body),
    ...authService(body),
    ...authAgentDomain(body),
    ...authServiceTokenUrl(body),
    ...authServiceUserInfoUrl(body),
    ...mappedDomain(body)
];

const updateBridge = () => [
    ...Id(param),
    ...name(body),
    ...clientId(body),
    ...serviceId(body),
    ...authServiceClientSecret(body),
    ...authServiceTokenUrl(body),
    ...authServiceUrl(body),
    ...authServiceClientId(body),
    ...authService(body),
    ...authAgentDomain(body),
    ...authServiceTokenUrl(body),
    ...authServiceUserInfoUrl(body),
    ...mappedDomain(body)
];

const deleteBridge = () => [
    ...Id(param)
];

const getClients = () => [
    ...limit(query),
    ...offet(query),
    ...search(query)
]

const getServices = () => [
    ...Id(param)
]


const limit = check => [
    check('limit')
    .isNumeric()
    .withMessage('Invalid limit')
]

const offet = check => [
    check('offset')
    .isNumeric()
    .withMessage('Invalid offset')
]

const search = check => [
    check('search')
    .escape()
]

const name = check => [
    check('name')
    .trim()
    .exists()
    .not()
    .isEmpty()
    .withMessage('Name is required.')
]

const clientId = check => [
    check('clientId')
    .trim()
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .isInt({ min: 1})
    .escape()
    .withMessage('Client is required.')

]

const serviceId = check => [
    check('serviceId')
    .trim()
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .isInt({ min: 1})
    .withMessage('Service is required.')
]

const authServiceClientSecret = check => [
    check('authServiceClientSecret')
    .trim()
    .exists()
    .not()
    .isEmpty()
    .withMessage('Authentication Service Client Secret is required.')
]

const authServiceClientId = check => [
    check('authServiceClientId')
    .trim()
    .exists()
    .not()
    .isEmpty()
    .withMessage('Authentication Service Client ID is required.')
]

const authService = check => [
    check('authService')
    .trim()
    .exists()
    .not()
    .isEmpty()
    .custom(value => {
        const services = ['aws cognito', 'okta', 'keycloak'];
        if(!services.includes(value)){
            throw new httpErrors.UnprocessableEntity('Invalid Authentication Service.');
        }
        return true;
    })
    .withMessage('Authentication Service is required.')
]

const authAgentDomain = check => [
    check('authAgentDomain')
    .trim()
    .exists()
    .not()
    .isEmpty()
    .withMessage('Auth Agent domain is required.')
]

const authServiceTokenUrl = check => [
    check('authServiceTokenUrl')
    .trim()
    .exists()
    .not()
    .isEmpty()
    .withMessage('Authentication Service Token URL is required.')
]

const authServiceUrl = check => [
    check('authServiceUrl')
    .trim()
    .exists()
    .not()
    .isEmpty()
    .withMessage('Authentication Service URL is required.')
]

const authServiceUserInfoUrl = check => [
    check('authServiceUserInfoUrl')
    .trim()
    .exists()
    .not()
    .isEmpty()
    .withMessage('Authentication Service User Info URL is required.')
]

const mappedDomain = check => [
    check('mappedDomain')
    .trim()
    .exists()
    .not()
    .isEmpty()
    .custom(value => {
        const validDomainExpression = /^[a-zA-Z0-9][a-zA-Z0-9.-]+[a-zA-Z0-9]$/;
        if(!validDomainExpression.test(value)){
            throw httpErrors.UnprocessableEntity('Invalid Mapped Domain.');
        }

        return true;
    })
    .withMessage('Mapped domain is required.')
]

const Id = check => [
    check('id')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .escape()
    .withMessage('Id is required.')
]

module.exports = {
    getBridge,
    createBridge,
    updateBridge,
    deleteBridge,
    getClients,
    getServices
};