const { body, param } = require('express-validator');

const addServiceGroup = () => [
    ...name(body),
    ...description(body),
    ...serviceIds(body),
    ...addAllServices(body)
];

const description = check => [
    check('description')
    .exists()
    .not()
    .isEmpty()
    .withMessage('Description is required.')
]

const addAllServices = check => [
    check('addAllServices')
    .isBoolean()
    .withMessage('addAllServices must be a boolean value')
]


const appendServices = check => [
    check('appendServices')
    .isBoolean()
    .exists()
    .withMessage('appendServices must be a boolean value')
]


const serviceIds = check => [
    check('serviceIds')
    .exists()
    .isArray()
    .withMessage('serviceIds are required.')
]

const updateServiceGroup = () => [
    ...name(body),
    ...id(param),
    ...description(body),
    ...serviceIds(body),
    ...appendServices(body),
    ...addAllServices(body),
];

const deleteServiceGroup = () => [
    ...id(param)
]

const name = check => [
    check('name')
    .trim()
    .exists()
    .not()
    .isEmpty()
    .withMessage('Name is required.')
]

const id = check => [
    check('id')
    .trim()
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .withMessage('Invalid Id.')
]

module.exports = {
    addServiceGroup,
    updateServiceGroup,
    deleteServiceGroup,
    serviceIds,
    description
}