const { body, param } = require('express-validator');
var httpErrors = require('http-errors');

const createHost = () => [
    ...address(body),
    ...port(body),
    ...isPortInRange(body),
    ...label(body),
    ...port(body),
    ...protocol(body),
    ...parentId(param),
    ...parentType(param)
];

const editHost = () => [
    ...address(body),
    ...port(body),
    ...isPortInRange(body),
    ...label(body),
    ...port(body),
    ...protocol(body),
    ...parentId(param),
    ...parentType(param),
    ...id(param)
];

const id = check => [
    check('id')
    .exists()
    .isNumeric()
    .withMessage('Invalid Id.')
];

const address = check => [
    check('address')
    .exists()
    .not()
    .isEmpty()
    .custom(value => {
      validExpression = /^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)(\/(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?))*$|^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)+([A-Za-z]|[A-Za-z][A-Za-z0-9\-]*[A-Za-z0-9])$/;
      // const validIpExpression = /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/;
      // const validHostnameExpression = /^(([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]*[a-zA-Z0-9])\.)*([A-Za-z0-9]|[A-Za-z0-9][A-Za-z0-9\-]*[A-Za-z0-9])$/;

      if(!validExpression.test(value)){
        throw new httpErrors.UnprocessableEntity('Invalid hostname.');
      }
      // if (
      //   !(validIpExpression.test(value) || validHostnameExpression.test(value))
      // ) {
      //   throw new httpErrors.UnprocessableEntity('Invalid address.');
      // }
      return true;
    })
    .withMessage('Invalid hostname.')
];

const direction = check => [
    check('direction')
    .exists()
    .not()
    .isEmpty()
    .withMessage('direction is required.')
];

const isPortInRange = check => [
    check('isPortInRange')
    .exists()
    .not()
    .isEmpty()
    .withMessage('isPortInRange is required.')
];

const label = check => [
    check('label')
    .exists()
    .not()
    .isEmpty()
    .withMessage('label is required.')
];

const protocol = check => [
    check('protocol')
    .exists()
    .not()
    .isEmpty()
    .withMessage('Invalid protocol.')
];

const parentType = check => [
    check('parentType')
    .exists()
    .not()
    .isEmpty()
    .withMessage('parentType is required.')
];

const parentId = check => [
    check('parentId')
    .exists()
    .not()
    .isEmpty()
    .withMessage('parent Id is required.')
];

const port = check => [
  check('port').custom((value, { req }) => {
    const validPortExpress = /^([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])$/;
    const { isPortInRange } = req.body;

    if (isPortInRange) {
      let { startPort, endPort } = req.body;
      startPort = +startPort;
      endPort = +endPort;
      if (startPort < endPort) {
        if (
          !validPortExpress.test(startPort) ||
          !validPortExpress.test(endPort)
        ) {
          throw new Error('Invalid port.');
        } else {
          return true;
        }
      } else {
        throw new Error('Invalid port range.');
      }
    } else {
      let { port } = req.body;
      port = +port;
      if (!validPortExpress.test(port)) {
        throw new Error('Invalid port.');
      } else {
        return true;
      }
    }
  })
];

module.exports = {
  createHost,
  editHost
};
