const { body, param } = require('express-validator');


const createUser = () => [
    ...firstName(body),
    ...lastName(body),
    ...email(body),
    ...password(body)
]


const updateUser = () => [
    ...firstName(body),
    ...lastName(body),
    ...email(body),
    ...updatePassword(body),
    ...id(param)
]

const regenerateKeys = () => [
    ...id(param)
]


const updateEmail = () => [
    ...useremail(body)
]


const useremail = check => [
    check('useremail')
    .exists()
    .not()
    .isEmpty()
    .isEmail()
    .normalizeEmail({ gmail_remove_dots: false })
    .withMessage('Invalid Email.')
]

const id = check => [
    check('id')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .withMessage('Invalid Id.')
]

const firstName = check => [
    check('user.firstName')
    .exists()
    .not()
    .isEmpty()
    .withMessage('First name is required.')
]

const lastName = check => [
    check('user.lastName')
    .exists()
    .not()
    .isEmpty()
    .withMessage('Last name is required.')
]

const email = check => [
    check('user.email')
    .exists()
    .not()
    .isEmpty()
    .isEmail()
    .normalizeEmail({ gmail_remove_dots: false })
    .withMessage('Invalid Email.')
];

const password = check => [
    check('user.password')
    .exists()
    .not()
    .isEmpty()
    .custom((value) => {

        if(!isValidPassword(value)){
            throw new httpErrors.UnprocessableEntity('Use between 8 to 31 characters with a mix of letters, numbers and symbols e.g. 1nvisilyR@cks');
        }

        return true;
    })
    .withMessage('Password is required.')
]

const updatePassword = check => [
    check('localUser.password')
    .custom((value) => {
        if(value && value.trim()){
            if(!isValidPassword(value)){
                throw new httpErrors.UnprocessableEntity('Use between 8 to 31 characters with a mix of letters, numbers and symbols e.g. 1nvisilyR@cks');
            }
        }
        return true;
    })
    .withMessage('Password is required.')
]

const isValidPassword = (password) => {
    const passwordExpression = /^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[^A-z0-9]).{8,31}$/;
    if(!passwordExpression.test(password)){
        return false;
    }
    return true;
}


module.exports = {
    createUser,
    updateUser,
    updateEmail,
    regenerateKeys
}