import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { AccessPolicyService } from '../create-access-policy/access-policy.service';
import { Pagination } from '../models/pagination';
import { Subject } from 'rxjs/internal/Subject';
import { PreserveSearch } from '../models/preserve-search';

import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';

import { GlobalMessageService } from '../global-messages/global-messages.service';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { of } from 'rxjs';
import { DeviceOSEnum } from '../shared/emuns/client.enum';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import moment from 'moment';

@Component({
  selector: 'app-access-policy',
  templateUrl: './access-policy.component.html',
  styleUrls: ['./access-policy.component.css']
})
export class AccessPolicyComponent implements OnInit {
  showHelpText: boolean = false;
  pagination = new Pagination();
  searchKeyword: any = '';
  isSearching: boolean = false;
  policies: any = {};
  $searchPolicy = new Subject();
  loading: boolean = true;
  private Search: PreserveSearch = new PreserveSearch();
  isNavigatingToSeach: boolean = false;

  preserveSearch = new PreserveSearch();
  navigationToEdit: boolean = false;

  pageSize = [10, 25, 50, 100, 200];

  clients: any = [];
  $clientSearch = new Subject();
  isClientSearching = false;
  clientOffset = 0;
  selectedClientIds = [];
  currentClientSearch = '';
  notAllCientsInSearch = true

  public alert: GlobalMessageClass;

  sort: string;
  sortDir: string;

  allClientPolicies: boolean = false;
  isDarkMode : any;
  timezones = [
    { name: 'UTC', value: 'UTC', moment: 'Etc/UCT' },
    { name: 'GMT', value: 'GMT', moment: 'Europe/London' },
    { name: 'AEDT', value: 'AEDT', moment: 'Antarctica/Macquarie' },
    { name: 'JST', value: 'JST', moment: 'Asia/Tokyo' },
    { name: 'IST', value: 'IST', moment: 'Asia/Calcutta' },
    { name: 'EET', value: 'EET', moment: 'Africa/Cairo' },
    { name: 'CAT', value: 'CAT', moment: 'Africa/Lusaka' },
    { name: 'ART', value: 'ART', moment: 'America/Argentina/Buenos_Aires' },
    { name: 'NST', value: 'NST', moment: 'Canada/Newfoundland' },
    { name: 'AST', value: 'AST', moment: 'Canada/Atlantic' },
    { name: 'CET', value: 'CET', moment: 'Europe/Amsterdam' },
    { name: 'AWST', value: 'AWST', moment: 'Australia/Perth' },
    { name: 'NZST', value: 'NZST', moment: 'Pacific/Auckland' },
    { name: 'SAST', value: 'SAST', moment: 'Africa/Johannesburg' },
    { name: 'WET', value: 'WET', moment: 'Atlantic/Canary' },
    { name: 'PST', value: 'PST', moment: 'America/Vancouver' },
    { name: 'MST', value: 'MST', moment: 'US/Mountain' },
    { name: 'CST', value: 'CST', moment: 'Mexico/General' },
    { name: 'EST', value: 'EST', moment: 'America/Cayman' },
    { name: 'HST', value: 'HST', moment: 'Pacific/Honolulu' },
    { name: 'AKST', value: 'AKST', moment: 'US/Alaska' }
  ];

  constructor(
    private router: Router,
    private accessService: AccessPolicyService,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog,
    private titleService: Title
  ) {
    this.titleService.setTitle('Policy Management | Entitlements');
  }

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    let saveSearch = this.preserveSearch.getSearch('accessPolicySaveSearch');
    if (saveSearch) {
      this.pagination = saveSearch.pagination;
      this.searchKeyword = saveSearch.searchKeyword;
    }
    this.subscibeToSearch();
    this.pagination.id = 'policies';
    let accessPolicySearch = this.Search.getSearch('accessPolicySearch');
    if (accessPolicySearch) {
      this.pagination.currentPage = accessPolicySearch.currentPage;
      this.searchKeyword = accessPolicySearch.searchKey;
      this.getAllPolicies(this.pagination);
    } else {
      this.getAllPolicies(this.pagination);
    }
    this.getClients();
    this.subscibeToSearch();
    this.subscribeClientSearch();
  }

  getClients() {
    this.currentClientSearch = this.currentClientSearch == null ? '' : this.currentClientSearch;
    this.accessService
      .getClients(this.clientOffset, this.currentClientSearch)
      .subscribe((data: any) => {
        this.notAllCientsInSearch = false
        this.clients = [...this.clients, ...data];
      });
  }

  fetchMoreClients() {
    if(this.notAllCientsInSearch){
      this.clientOffset += 1;
      this.getClients();
    }
  }

  clientFocus() {
    this.currentClientSearch = '';
    this.clients = [];
    this.notAllCientsInSearch = true
    this.clientOffset = 0
    this.getClients();
  }

  searchPolicyByClientName() {
    this.pagination.currentPage = 1;
    this.getAllPolicies(this.pagination);
  }

  subscribeClientSearch() {
    this.$clientSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.isClientSearching = true;
          this.currentClientSearch = term;
          this.clientOffset = 0;
          return this.accessService.getClients(
            this.clientOffset,
            this.currentClientSearch
          );
        })
      )
      .subscribe(data => {
        this.isClientSearching = false;
        this.clients = data;
      });
  }

  getAllPolicies(pagination: Pagination) {
    this.loading = true;
    this.accessService
      .getAllPolicies(
        pagination,
        this.searchKeyword,
        this.sort,
        this.sortDir,
        this.selectedClientIds,
        this.allClientPolicies
      )
      .subscribe({
        next: policies => {
          this.loading = false;
          this.pagination.totalItems = policies.recordsTotal;
          this.policies = policies;
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  clearfun() {
    this.searchKeyword = '';
    this.getAllPolicies(this.pagination);
  }

  deletePolicy(id: any) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'Are you sure you want to delete?',
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        this.accessService.deletePolicy(id).subscribe(res => {
          this.alert = {
            closed: false,
            type: 'success',
            message: 'Policy has been deleted successfully.'
          };
          this.globalMessageService.pushAlert(this.alert);
          this.getAllPolicies(this.pagination);
        });
      }
    });
  }

  subscibeToSearch() {
    this.$searchPolicy
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap(data => {
          this.loading = true;
          this.isSearching = true;
          this.searchKeyword = data;
          this.pagination.currentPage = 1;
          this.getAllPolicies(this.pagination);
          return of('Fetching data...');
        })
      )
      .subscribe(() => {
        this.loading = false;
        this.isSearching = false;
      });
  }
  enablePolicy(id: any, value: any) {
    this.accessService.enablePolicy(id, value).subscribe(res => {
      this.getAllPolicies(this.pagination);
    });
  }

  editHandler(policy) {
    const search = {
      pagination: this.pagination,
      searchKeyword: this.searchKeyword
    };
    this.preserveSearch.saveSearch('accessPolicySaveSearch', search);
    this.navigationToEdit = true;
    this.router.navigate([`accessPolicy/${policy.id}`]);
  }

  ngOnDestroy(): void {
    if (!this.navigationToEdit) {
      this.preserveSearch.saveSearch('accessPolicySaveSearch', '');
    }
  }

  pageChange(page: any) {
    this.pagination.currentPage = page;
    this.getAllPolicies(this.pagination);
  }

  addAccessPolicyHandler() {
    this.router.navigate(['/create-access-policy']);
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getAllPolicies(this.pagination);
  }

  sortRecord(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
    this.getAllPolicies(this.pagination);
  }

  allClientPoliciesToggle(check) {
    this.allClientPolicies = check;
    this.getAllPolicies(this.pagination);
  }

  getMomentTimeZone(selectedTimeZone) {
    const timezone = this.timezones.find(tz => tz.value === selectedTimeZone);
    return timezone ? timezone.moment : null;
  }

  convertFromUTC(timeString, timeZoneMomentValue) {
    if (!timeZoneMomentValue) {
      return null;
    }
    const utcTime = moment.utc(`1970-01-01 ${timeString}`, 'YYYY-MM-DD HH:mm:ss');
    const localTime = utcTime.tz(timeZoneMomentValue);
    return localTime.format('HH:mm:ss');
  }

  current_time_permitted(access_restriction, start_time, end_time, months, days,time_zone) {
    var access_permitted: boolean;
    if (!access_restriction) {
      return true;
    } else {
      // let convertedTimeZone = this.getMomentTimeZone(time_zone);
      const today = new Date();
      const currentTime = `${today.getHours()}:${today.getMinutes()}:${today.getSeconds()}`;

      let convertedCurrentTime = this.getCurrentTimeInUTC()

      const [startHour, startMinute, startSecond] = start_time.split(':');
      const [endHour, endMinute, endSecond] = end_time.split(':');

      access_permitted =
        months?.includes(today.getMonth() + 1) &&
        days?.includes(today.getDay() - 1) &&
        convertedCurrentTime >= `${startHour}:${startMinute}:${startSecond}` &&
        convertedCurrentTime <= `${endHour}:${endMinute}:${endSecond}`;
    }
    return access_permitted;
  }

  getCurrentTimeInUTC = () => {
    const today = new Date();
    const hours = String(today.getUTCHours()).padStart(2, '0');
    const minutes = String(today.getUTCMinutes()).padStart(2, '0');
    const seconds = String(today.getUTCSeconds()).padStart(2, '0');
    return `${hours}:${minutes}:${seconds}`;
  };

  getOsName(os) {
    switch (os) {
      case DeviceOSEnum.Ubuntu18:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer18:
        return 'Ubuntu Server';
      case DeviceOSEnum.Macintosh:
        return 'Macintosh';
      case DeviceOSEnum.Windows:
        return 'Windows';
      case DeviceOSEnum.Ubuntu:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer:
        return 'Ubuntu Server';
      case DeviceOSEnum.WindowsServer:
        return 'Windows Server';
      case DeviceOSEnum.IOS:
        return 'iOS';
      case DeviceOSEnum.Android:
        return 'Android';
      case DeviceOSEnum.UbuntuServer20armv8:
        return 'Ubuntu Server (ARMv8)';
      case DeviceOSEnum.CentosServer:
        return 'CentOS Server';
      case DeviceOSEnum.Unknown:
        return 'Unknown';
      default:
        return os;
    }
  }
}
