import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable, map } from 'rxjs';
import { ConfigService } from '../services/config.service';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { Router } from '@angular/router';

@Injectable({
  providedIn: 'root'
})
export class AdditionalPermissionsService {
  private apiBaseUrl: string;
  private rolesAndPermissionsApiUrl = '/templates';
  private assignedUserTemplatesSubject = new BehaviorSubject<any>(
    this.loadInitialValue()
  );
  assignedUserTemplates$ = this.assignedUserTemplatesSubject.asObservable();

  constructor(
    private authHttp: AuthHttp,
    private configService: ConfigService,
    private router: Router
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.rolesAndPermissionsApiUrl =
      this.apiBaseUrl + this.rolesAndPermissionsApiUrl;
  }

  setAssignedUserTemplates(data: any) {
    console.log(this.assignedUserTemplatesSubject.value);
    this.assignedUserTemplatesSubject.next(data);
  }

  getAdditionalPermissions(
    page: number = 1,
    pageSize: number = 10,
    sort: string = 'id',
    dir: string = 'desc',
    searchKey = ''
  ): any {
    let queryString = `?limit=${pageSize}&offset=${page -
      1}&sort=${sort}&dir=${dir}&searchKey=${searchKey}`;
    return this.authHttp
      .get(`${this.rolesAndPermissionsApiUrl}/additional-permissions/${queryString}`)
      .pipe(map(response => response));
  }

  loadInitialValue() {
    return JSON.parse(localStorage.getItem('Roles')) || {};
  }

  createAdditionalTemplate(permissions) {
    return this.authHttp
      .post(`${this.rolesAndPermissionsApiUrl}/additional`, permissions)
      .pipe(map(response => response));
  }

  getTemplatebyId(id) {
    return this.authHttp
      .get(`${this.rolesAndPermissionsApiUrl}/${id}`)
      .pipe(map(response => response));
  }

  getAssignedUserTemplates() {
    return this.authHttp
      .get(`${this.rolesAndPermissionsApiUrl}/user`)
      .pipe(map(response => response));
  }

  getAvailableAdditionalPermissions() {
    return this.authHttp
      .get(`${this.rolesAndPermissionsApiUrl}/additional-possible`)
      .pipe(map(response => response));
  }

  getCreatedAdditionalPermission(id) {
    return this.authHttp
      .get(`${this.rolesAndPermissionsApiUrl}/additional-requests/${id}`)
      .pipe(map(response => response));
  }

  updateAdditionalTemplate(id, permissions) {
    return this.authHttp
      .put(`${this.rolesAndPermissionsApiUrl}/additional/${id}`, permissions)
      .pipe(map(response => response));
  }

  deleteAdditionalTemplate(id) {
    return this.authHttp
      .delete(`${this.rolesAndPermissionsApiUrl}/additional/${id}`)
      .pipe(map(response => response));
  }

  getCrudData(privId): Observable<any> {
    return this.getAssignedUserTemplates().pipe(
      map((template: any) => {
        const templateCrud = template.filter(
          data => data.privilege_id === privId
        )[0];
        return templateCrud;
      })
    );
  }
}
