import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { ConfigService } from '../services/config.service';
import { Pagination } from '../models/pagination';
import * as moment from 'moment';
import { AuthShortlivedCookieService } from '../auth-shortlived-cookie.service';
import { map } from 'rxjs/operators';
@Injectable()
export class AdminAccessLogService {
  private apiBaseUrl: string;
  private adminLogsApiUrl: string = '/adminAccessLog';

  colorsByPriority = {
    low: {
      label: 'Low',
      value: 'label label-success'
    },
    medium: {
      label: 'Medium',
      value: 'label label-warning'
    },
    high: {
      label: 'High',
      value: 'label label-danger'
    }
  };
  constructor(
    private authHttp: AuthHttp,
    private congigService: ConfigService,
    private authShortlivedCookie: AuthShortlivedCookieService
  ) {
    this.apiBaseUrl = this.congigService.getConfiguration().apiBaseUrl;
    this.adminLogsApiUrl = this.apiBaseUrl + this.adminLogsApiUrl;
  }

  // getAllLogs(pagination: Pagination, sort: String, sortOrderDesc:boolean=true) {

  //   let query = '';

  //   let sortOrder = sortOrderDesc?'DESC':'ASC';
  //   query = query+`sort=${sort}&dir=${sortOrder}&limit=${pagination.itemsPerPage}&offset=${pagination.currentPage-1}`;

  //   return this.authHttp
  //     .get(
  //       `${this.adminLogsApiUrl}?${query}`
  //     )
  //     .map(response => response);
  // }

  getAllColors() {
    return this.colorsByPriority;
  }

  searchLog(
    searchTerm: string = '',
    pagination: Pagination,
    startDate: Date,
    endDate: Date,
    sort: string = 'createdAt',
    dir: string = 'desc',
    priority = ''
  ) {
    let _searchTerm = searchTerm || '';
    let _startDate = '1900-01-01';
    let _endDate = '9999-01-01';
    if (startDate) {
      _startDate = moment(startDate)
        .startOf('day')
        .format('YYYY-MM-DD HH:mm:ss');
    }
    if (endDate) {
      _endDate = moment(endDate)
        .endOf('day')
        .format('YYYY-MM-DD HH:mm:ss');
    }

    const base64Encoded = encodeURIComponent(_searchTerm);
    let query = `?searchKey=${base64Encoded}&startDate=${_startDate}`;
    query = query + `&endDate=${_endDate}&sort=${sort}&dir=${dir}`;
    query =
      query +
      `&limit=${pagination.itemsPerPage}&offset=${pagination.currentPage - 1}`;
    if (priority) {
      query = query + `&priority=${priority}`;
    }
    return this.authHttp.get(`${this.adminLogsApiUrl}${query}`).pipe(
      map(response => {
        return response;
      })
    );
  }

  exportCSV(
    searchKeyword: string = '',
    startDate: Date,
    endDate: Date,
    priority = '',
    portalVersionInfo
  ) {
    const urlEncoded = encodeURIComponent(searchKeyword);
    let _startDate = '1900-01-01';
    let _endDate = '9999-01-01';
    if (startDate) {
      _startDate = moment(startDate)
        .startOf('day')
        .format('YYYY-MM-DD HH:mm:ss');
    }
    if (endDate) {
      _endDate = moment(endDate)
        .endOf('day')
        .format('YYYY-MM-DD HH:mm:ss');
    }

    let query = `?searchKey=${urlEncoded}&startDate=${_startDate}&endDate=${_endDate}&version=${portalVersionInfo}`;
    if (priority) {
      query = query + `&priority=${priority}`;
    }
    this.authShortlivedCookie.setFor(5);
    window.location.href = `${this.adminLogsApiUrl}/export-csv${query}`;
  }
}
