import { Component, OnInit } from '@angular/core';
import { AlertServices } from './alerts.service';
import { Pagination } from '../models/pagination';
import { Subscription, timer } from 'rxjs';
import { Subject } from 'rxjs';
import { Observable } from 'rxjs';
import { Title } from '@angular/platform-browser';
import {
  debounceTime,
  distinctUntilChanged,
  first,
  switchMap
} from 'rxjs/operators';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { RoleEnum } from '../shared/emuns/global.enum';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { VersionService } from '../version/version.service';
import { RolesAndPermissionService } from '../roles-and-permission/roles-and-permission.service';
import { ViewModalComponent } from './view-modal/view-modal.component';
import { SettingsService } from '../settings/settings.service';
import { ThemeService } from '../theme.service';

@Component({
  selector: 'app-alert-list',
  templateUrl: './alert-list.component.html',
  styleUrls: ['./alert-list.component.css']
})
export class AlertListComponent implements OnInit {
  private timerSubscription: Subscription;
  pagination = new Pagination();
  alerts: any = {};
  $searchAlerts = new Subject();
  start_date: Date = null;
  end_date: Date = null;
  searchKeyword;
  userrole;
  sort = 'createdAt';
  sortDir = 'desc';
  public dateFormat: string = 'dd/MM/yyyy HH:mm';
  selectedType = null;
  selectedSeverity = null;
  pageSize = [10, 25, 50, 100, 200];

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  loading: boolean = true;
  dateRangePicker: Date[] = [];
  distinctType = [];
  distinctSeverity = [];
  timeZone = '';
  today: Date;
  public alert: GlobalMessageClass;
  Role = RoleEnum;
  portalVersionInfo: string;
  isDarkMode : any;
  constructor(
    private alertService: AlertServices,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog,
    private versionService: VersionService,
    private titleService: Title,
    private rolesPermissionService: RolesAndPermissionService,
    public settingService : SettingsService,
    public themeService : ThemeService
  ) {
    this.titleService.setTitle('Alerts');
    this.today = new Date();
    this.userrole = localStorage.getItem('ur');
    if (typeof this.userrole != 'undefined' || this.userrole != null) {
      this.userrole = atob(this.userrole);
    }
    var localDateFormat = localStorage.getItem('dateFormat');
    if (typeof localDateFormat != 'undefined' || localDateFormat != null) {
      this.dateFormat = localDateFormat;
    }

    let timeZoneOffset = new Date().getTimezoneOffset();

    this.timeZone =
      'UTC' + (timeZoneOffset > 0 ? '-0' : '+0') + (timeZoneOffset / 60) * -1;
  }

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.refreshData();
    this.getCRUDdata(9);
    this.subscibeToSearch();
    this.portalVersionInfo = this.versionService.portalVersion;
  }

  public refreshData(): void {
    // if (this.timerSubscription) {
    //   this.timerSubscription.unsubscribe();
    // }
    this.getAlerts();
    // this.subscribeToData();
  }

  getCRUDdata(privId){
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 
      this.readPermission = res?.read_permission === 1 
      this.updatePermission = res?.update_permission === 1 
      this.deletePermission = res?.delete_permission === 1 
    });
  }

  clearfun() {
    this.searchKeyword = '';
    this.getAlerts();
  }

  private subscribeToData(): void {
    this.timerSubscription = timer(10000)
      .pipe(first())
      .subscribe(() => this.refreshData());
  }

  public ngOnDestroy(): void {
    this.$searchAlerts.unsubscribe();
    if (this.timerSubscription) {
      this.timerSubscription.unsubscribe();
    }
  }

  getAlerts() {
    this.loading = true;
    this.alertService.getDistinctType().subscribe(
      (response: any) => {
        this.distinctType = response;
      },
      error => {}
    );
    this.alertService.getDistinctSeverity().subscribe(
      (response: any) => {
        this.distinctSeverity = response;
      },
      error => {}
    );
    this.alertService
      .getAlerts(
        this.pagination.itemsPerPage,
        this.pagination.currentPage,
        this.searchKeyword,
        this.start_date,
        this.end_date,
        this.selectedType,
        this.selectedSeverity,
        this.sort,
        this.sortDir
      )
      .subscribe({
        next: (res: any) => {
          this.loading = false;
          this.alerts = res;
          this.pagination.totalItems = res.recordsTotal;
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  purgeData() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'Are you sure you want to delete all data?',
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        this.alertService.purgeData().subscribe(res => {
          this.alert = {
            closed: false,
            type: 'success',
            message: 'Alerts have been deleted successfully.'
          };
          this.globalMessageService.pushAlert(this.alert);
          this.alertService.setNotificationPurge(true)
          this.alerts.data = [];
          this.pagination.totalItems = 0;
        });
      }
    });
  }

  sortData() {
    this.sortDir = this.sortDir === 'desc' ? 'asc' : 'desc';
    this.refreshData();
  }

  subscibeToSearch() {
    this.$searchAlerts
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap(data => {
          this.loading = true;
          this.pagination.currentPage = 1;
          return this.alertService.getAlerts(
            this.pagination.itemsPerPage,
            this.pagination.currentPage,
            this.searchKeyword,
            this.start_date,
            this.end_date,
            this.selectedType,
            this.selectedSeverity
          );
        })
      )
      .subscribe({
        next: (alerts: any) => {
          this.loading = false;
          const { recordsTotal } = alerts;
          this.alerts = alerts;
          this.pagination.totalItems = recordsTotal;
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  pageChange(event) {
    this.pagination.currentPage = event;
    this.getAlerts();
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getAlerts();
  }

  sortRecords(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
    this.getAlerts();
  }

  dateChange() {
    this.$searchAlerts.next({
      searchWord: this.searchKeyword,
      typing: false,
      date_change: true,
      sort_change: false
    });
  }

  dateRangePickerChange() {
    if (this.dateRangePicker[0]) {
      this.start_date = this.dateRangePicker[0];
    }

    if (this.dateRangePicker[1]) {
      this.end_date = this.dateRangePicker[1];
    }

    this.dateChange();
  }

  clearDate() {
    this.dateRangePicker = [];
    this.start_date = null;
    this.end_date = null;
    this.dateChange();
  }

  alertTypeChange(type) {
    this.pagination.currentPage = 1;
    this.getAlerts();
  }

  alertSeverityChange(type) {
    this.pagination.currentPage = 1;
    this.getAlerts();
  }

  exportCSV() {
    this.alertService.exportCSV(
      this.searchKeyword,
      this.start_date,
      this.end_date,
      this.selectedType,
      this.portalVersionInfo
    );
  }

  viewAlert(alert) {
    this.settingService.userSettingsLoaded.subscribe(() => {
      console.log('HEllo');
      this.themeService.isDarkMode$.subscribe((res: any) => {
        this.isDarkMode = res
      });
    });

    console.log('this.isDarkMode :>> ', this.isDarkMode);
    const dialogRef = this.dialog.open(ViewModalComponent, {
      width: '400px',
      height: '350px',
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        id: alert.id,
        type: alert.type,
        description: alert.message,
        severity: alert.severity,
        timestamp: alert.createdAt,
      },
    });

    dialogRef.afterClosed().subscribe(result => {
      console.log("BACK TO ALERTS");
    });
  }
}
