import {
  Component,
  HostListener,
  AfterViewInit,
  Renderer2
} from '@angular/core';
import { Router, NavigationEnd, ActivatedRoute } from '@angular/router';
import { AuthenticationService } from './login/authentication.service';
import { Subscription } from 'rxjs';
import * as moment from 'moment';
import { filter, tap } from 'rxjs/operators';
import { BehaviorSubject, Observable } from 'rxjs';
import { ThemeService } from './theme.service';
import { SettingsService } from './settings/settings.service';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: [
    './app.component.css'
    // '../../node_modules/anychart/dist/css/anychart-ui.min.css',
    // '../../node_modules/anychart/dist/fonts/css/anychart-font.min.css'
  ]
})
export class AppComponent {
  title = 'SDP';
  showMargin: boolean = false;
  public littleShort: boolean = false;
  hideNavbar: boolean = false;
  mobileView: boolean = false;
  screenHeight: string = '';
  currentRoute;
  routeCheck: string;
  // Subscription of the observer of the screen size
  mediaQuery$: Subscription;
  // The active media query (xs | sm | md | lg | xl)
  activeMediaQuery: string;
  isDarkMode: any;

  constructor(
    private router: Router,
    private activatedRoute: ActivatedRoute,
    private authenticationManager: AuthenticationService,
    private render: Renderer2,
    public themeService: ThemeService,
    private settingService: SettingsService
  ) {
    router.events
      .pipe(filter(event => event instanceof NavigationEnd))
      .subscribe(event => {
        this.currentRoute = event;
        this.routeCheck = this.currentRoute.url.split(';')[0];
        this.routeCheck = this.routeCheck.split('?')[0];
        this.routeCheck = this.routeCheck.split('/', 2).join('/');
      });

    this.authenticationManager.showNavBarEmitter
      .pipe(
        tap(mode => {
          let currentUser = JSON.parse(localStorage.getItem('currentUser'));
          if (typeof currentUser != 'undefined' && currentUser != null) {
            if (mode !== null || currentUser.token) {
              this.showMargin = true;
              this.AuthenticatedMode();
            }
          }
        })
      )
      .subscribe({
        next: () => {
          var currentUser = JSON.parse(localStorage.getItem('currentUser'));
          if (typeof currentUser == 'undefined' || currentUser == null) {
            this.showMargin = false;
            this.unAuthenticatedMode();
          }
        },
        error: err => {
          console.log(`Oops... ${err}`);
        },
        complete: () => {}
      });

    this.authenticationManager.showLogOutEmitter.subscribe(mode => {
      if (mode) {
        this.showMargin = false;
        this.unAuthenticatedMode();
      }
    });
    this.findLittleShort();

    const currentUser = JSON.parse(localStorage.getItem('currentUser'));
    if (currentUser && currentUser.token) {
      this.authenticationManager.autologout(moment(currentUser.tokenExpiry));
    }
  }

  ngOnInit() {
    console.log('This Componenet is redered');
    this.themeService.isDarkMode$.subscribe((res: any) => {
      console.log('Here We Goo');
      this.isDarkMode = res ? 'dark-theme' : '';
    });

    this.settingService.userSettingsLoaded.subscribe(() => {
      console.log('HEllo');
      this.themeService.isDarkMode$.subscribe((res: any) => {
        this.isDarkMode = res ? 'dark-theme' : '';
      });
    });

    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.isDarkMode = this.isDarkMode ? 'dark-theme' : '';

    this.screenHeight = window.innerHeight + 'px';
  }

  public findLittleShort() {
    let isLittleShort = JSON.parse(localStorage.getItem('littleShort'));
    if (isLittleShort) {
      this.littleShort = true;
    } else {
      if (!isLittleShort) {
        this.littleShort = false;
      }
    }
  }

  onActivate(event) {
    let scrollToTop = window.setInterval(() => {
      let pos = window.pageYOffset;
      if (pos > 0) {
        window.scrollTo(0, pos - 20); // how far to scroll on each step
      } else {
        window.clearInterval(scrollToTop);
      }
    }, 16);
  }

  onChangeScreenStatus(Screenstatus: {
    hide: boolean;
    mini: boolean;
    mobileView: boolean;
  }) {
    this.littleShort = Screenstatus.mini;
    this.hideNavbar = Screenstatus.hide;
    this.mobileView = Screenstatus.mobileView;
  }

  @HostListener('window:resize', ['$event'])
  onResize(event) {
    this.screenHeight = window.innerHeight + 'px';
  }

  AuthenticatedMode() {
    this.render.removeClass(document.body, 'login');
    this.render.addClass(document.body, 'page-header-fixed');
    this.render.addClass(document.body, 'page-sidebar-closed-hide-logo');
    this.render.addClass(document.body, 'page-content-white');
    this.render.addClass(document.body, 'page-sidebar-fixed');
  }

  unAuthenticatedMode() {
    this.render.removeClass(document.body, 'page-header-fixed');
    this.render.removeClass(document.body, 'page-sidebar-closed-hide-logo');
    this.render.removeClass(document.body, 'page-content-white');
    this.render.removeClass(document.body, 'page-sidebar-fixed');
    this.render.addClass(document.body, 'login');
  }
}
