import { NgModule, APP_INITIALIZER } from '@angular/core';
import {
  HttpClient,
  HttpClientModule,
  HTTP_INTERCEPTORS
} from '@angular/common/http';
import { BrowserModule } from '@angular/platform-browser';
import { FormsModule, ReactiveFormsModule } from '@angular/forms';
import { AppRoutingModule } from './app-routing.module';
import { DragDropModule } from '@angular/cdk/drag-drop';
import { AuthGuard } from './guard/auth.guard';
// import { ModalModule } from 'angular2-modal';
// import { BootstrapModalModule } from 'angular2-modal/plugins/bootstrap';
import { ModalModule } from 'ngx-bootstrap/modal';
import { AppComponent } from './app.component';
import { UserService } from './users/user.service';
import { AuthenticationService } from './login/authentication.service';
import { GauthService } from './gauth/gauth.service';
import { MatMenuModule } from '@angular/material/menu';
import { MatButtonModule } from '@angular/material/button';
import { DeviceIdService } from './deviceid/deviceid.service';
import { ForgotService } from './forgot-password/forgot.service';
import { DashboardService } from './dashboard/dashboard.service';
import { RecoverService } from './recover-password/recover.service';
import { environment } from '../environments/environment';
import { ConfigService } from './services/config.service';
import { CheckToken } from './auth-guard.service';
import { RoleGuard } from './role-guard.service';
import { NgxPaginationModule } from 'ngx-pagination';
import { DragulaModule } from 'ng2-dragula';
import { RolesResolve } from './users/roles.resolve';
import { LdapService } from './ldap/ldap.service';
import { ManageService } from './manage-service/manage.service';
import { GatewayService } from './gateway/gateway.service';
import { WorkloadService } from './workload/workload.service';
import { GatewayGroupsService } from './gateway-groups/gateway-groups.service';
import { ControllerService } from './controller/controller.service';
import { ClientService } from './client/client.service';
import { FreshLoginService } from './fresh-login/fresh-login.service';
import { GlobalMessageService } from './global-messages/global-messages.service';
import { UpdateService } from './updates/update.service';
import { SettingsService } from './settings/settings.service';
import { LicenseService } from './license/license.service';
// import { ChartModule } from 'angular2-highcharts';
// import { HighchartsStatic } from 'angular2-highcharts/dist/HighchartsService';
// import { AgmCoreModule, GoogleMapsAPIWrapper } from '@agm/core';
import { QRCodeModule } from 'angularx-qrcode';
import { IdpService } from './idp-management/idp.service';
import { WithoutSpaceValidator } from './validators/without-space.validator.directive';
import { DigitOnlyDirective } from './directives/digit-only.directive';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import { RecoverPasswordGuard } from './guard/recover-password.guard';
import { TrustedHostsService } from './trusted-hosts/trusted-hosts.service';
import { GroupService } from './group/group.service';
import { LocalTrustedHostsService } from './trusted-hosts/local-trusted-hosts.service';
import { NgSelectModule } from '@ng-select/ng-select';
import { BackupService } from './backups/backup.service';
import { RestoreService } from './restore/restore.service';
import { AdminAccessLogService } from './admin-access-log/admin-access-log.service';
import { DeviceInventoryService } from './device-inventory/device-inventory.service';
import { AuthShortlivedCookieService } from './auth-shortlived-cookie.service';
import { LocalUserService } from './create-local-user/local-user.service';
import { DeviceService } from './devices-list/device.service';
import { SoftwareService } from './softwares-list/software.service';
import { SoftwareProcessBindingService } from './software-process-binding/software-process-binding.service';
import { OrganizationService } from './organization-list-view/organization.service';
import { OsTransformPipe } from './pipes/os-transform.pipe';
import { ValidDomainsService } from './valid-domains.service';
import { AmChartsModule } from '@amcharts/amcharts3-angular';
import { ReportingService } from '../app/reportings/reporting.service';
import { BsDatepickerModule } from 'ngx-bootstrap/datepicker';
import { PaginationModule } from 'ngx-bootstrap/pagination';
import { TooltipModule } from 'ngx-bootstrap/tooltip';

import { TimepickerModule } from 'ngx-bootstrap/timepicker';
import { PolicyService } from './policy/policy.service';
import { AccessPolicyService } from './create-access-policy/access-policy.service';
import { DevicePolicyService } from './create-device-policy/device-policy.service';
import { SoftwareProcessPolicyService } from './create-software-process-policy/software-process-policy.service';
import { SoftwarePolicyService } from './create-software-policy/software-policy.service';

import { OsPolicyService } from './create-os-policy/os-policy.service';
import { OsPolicyServiceSoftware } from './create-software/os-software.service';

import { AutofocusDirective } from './directives/autofocus.directive';
import { AlertServices } from './alert-list/alerts.service';
import { ForbiddenSoftwareServices } from './forbidden-software-list/forbidden-software.service';
import { IotService } from './iot.service';
import { SaasBridgeService } from './saas-bridge/saas-bridge.service';
import { MicrosegmentService } from './microsegmentation/microsegmentation.service';
import { RoutingPolicyService } from './routing-policy/routing-policy.service';
import { OsService } from './common/os-service.service';
import { UsersReportsService } from '../app/users-reports/reporting.service';
import { DevicesReportsService } from '../app/devices-reports/reporting.service';
import { ServicesReportsService } from './services-reports/reporting.service';
import { ClientsReportsService } from './clients-reports/reporting.service';
import { PolicyReportsService } from './policy-reports/reporting.service';
import { InfrastructureReportsService } from './infrastructure-reports/reporting.service';
import { RecaptchaModule } from 'ng-recaptcha';
import { DataVolumePolicyService } from './data-volume-policy/data-volume-policy.service';
import { OverlayscrollbarsModule } from 'overlayscrollbars-ngx';
import { ToastrModule } from 'ngx-toastr';
import {
  NgbModule,
  NgbDropdownModule,
  NgbDropdownConfig,
  NgbAlertModule
} from '@ng-bootstrap/ng-bootstrap';
import { JwtInterceptor } from '../jwt.intercepter';
import { SharedModule } from './shared/shared.module';
import { ViewDetailsServiceComponent } from './manage-service/manage-list/view-detail-service/view-detail-service.component';
import { CommonModule, DatePipe } from '@angular/common';
import { UserGroupsComponent } from './users/user-groups/user-groups.component';
import { MatAutocompleteModule } from '@angular/material/autocomplete';
import { ChangePasswordGuard } from './guard/change-password.guard';
import { SecrityQuestionsComponent } from './security-questions/security-questions.component';
import { ChangeQuestionGuard } from './guard/change-question.guard';

//***************** Components Modules ********************//
import { ActivityIndicatorModule } from './activity-indicator/activity-indicator.module';
import { AccessPolicyModule } from './access-policy/access-policy.module';
import { WorkloadListModule } from './workload/workload-list/workload-list.module';
import { ValidDomainsModule } from './valid-domains/valid-domains.module';
import { UserListModule } from './users/user-list/user-list.module';
import { UsersReportsModule } from './users-reports/users-reports.module';
import { UserAgentAccessPolicyModule } from './user-agent-access-policy/user-agent-access-policy.module';
import { UpdatesListModule } from './updates/updates-list/updates-list.module';
import { TrustedHostsModule } from './trusted-hosts/trusted-hosts.module';
import { SoftwaresListModule } from './softwares-list/softwares-list.module';
import { SoftwareProcessPolicyModule } from './software-process-policy/software-process-policy.module';
import { SoftwareProcessBindingModule } from './software-process-binding/software-process-binding.module';
import { ServicesReportsModule } from './services-reports/services-reports.module';
import { SaasBridgeModule } from './saas-bridge/saas-bridge.module';
import { RoutingPolicyModule } from './routing-policy/routing-policy.module';
import { ReportingsModule } from './reportings/reportings.module';
import { ServiceGroupDetailModule } from './policy/service-group-detail/service-group-detail.module';
import { PolicyReportsModule } from './policy-reports/policy-reports.module';
import { NewPaginationModule } from './pagination/pagination.module';
import { OsPolicyModule } from './os-policy/os-policy.module';
import { OrganizationListViewModule } from './organization-list-view/organization-list-view.module';
import { MicrosegmentModule } from './create-microsegment/microsegment.module';
import { MicrosegmentationModule } from './microsegmentation/microsegmentation.module';
import { SpinnerModule } from './spinner/spinner.module';
import { CreateWorkloadModule } from './workload/create-workload/create-workload.module';
import { SoftwarePolicyModule } from './software-policy/software-policy.module';
import { ManageListModule } from './manage-service/manage-list/manage-list.module';
import { FilterCheckButtonModule } from './filter-check-button/filter-check-button.module';
import { LocalUsersListModule } from './local-users-list/local-users-list.module';
import { LdapuserListModule } from './ldap/ldapuser-list/ldapuser-list.module';
import { IotModule } from './iot/iot.module';
import { InfrastructureReportsModule } from './infrastructure-reports/infrastructure-reports.module';
import { IdpListModule } from './idp-management/idp-list/idp-list.module';
import { GroupDetailModule } from './group/group-detail/group-detail.module';
import { CreateGroupModule } from './group/create-group/create-group.module';
import { GatewayListModule } from './gateway/gateway-list/gateway-list.module';
import { CreateGatwayGroupModule } from './gateway-groups/create-gatway-group/create-gatway-group.module';
import { ForbiddenSoftwareListModule } from './forbidden-software-list/forbidden-software-list.module';
import { DevicesReportsModule } from './devices-reports/devices-reports.module';
import { DevicesListModule } from './devices-list/devices-list.module';
import { DevicePolicyModule } from './device-policy/device-policy.module';
import { DeviceInventoryModule } from './device-inventory/device-inventory.module';
import { DeviceAccessPolicyModule } from './device-access-policy/device-access-policy.module';
import { DataVolumePolicyModule } from './data-volume-policy/data-volume-policy.module';
import { GatewayHealthModule } from './dashboard/gateway-health/gateway-health.module';
import { DashboardModule } from './dashboard/dashboard.module';
import { CreatePolicyModule } from './create-policy/create-policy.module';
import { HostsByOSModule } from './charts/hosts_by_OS.module';
import { CreateOrganizationModule } from './create-organization/create-organization.module';
import { ControllerListModule } from './controller/controller-list/controller-list.module';
import { WhitelistedHostsModule } from './controller/whitelisted-hosts/whitelisted-hosts.module';
import { ClientsReportsModule } from './clients-reports/clients-reports.module';
import { ClientListModule } from './client/client-list/client-list.module';
import { ClientAccessLogModule } from './client/client-access-log/client-access-log.module';
import { BackupsModule } from './backups/backups.module';
import { AlertListModule } from './alert-list/alert-list.module';
import { AdminAccessLogModule } from './admin-access-log/admin-access-log.module';
import { AssignListModule } from './assign-service/assign-list/assign-list.module';
import { CreateAccessPolicyModule } from './create-access-policy/create-access-policy.module';
import { CreateB2bBridgeModule } from './create-b2b-bridge/create-b2b-bridge.module';
import { CreateDataVolumePolicyModule } from './create-data-volume-policy/create-data-volume-policy.module';
import { CreateDevicePolicyModule } from './create-device-policy/create-device-policy.module';
import { CreateDeviceModule } from './create-device/create-device.module';
import { ConfirmDeleteDeviceModule } from './create-local-user/confirm-delete-device/confirm-delete-device-module';
import { CreateLocalUserModule } from './create-local-user/create-local-user.module';
import { CreateOsPolicyModule } from './create-os-policy/create-os-policy.module';
import { CreateServerBridgeModule } from './create-server-bridge/create-server-bridge.module';
import { CreateSoftwareModule } from './create-software/create-software.module';
import { CreateSoftwarePolicyModule } from './create-software-policy/create-software-policy.module';
import { CreateSoftwareProcessBindingModule } from './create-software-process-binding/create-software-process-binding.module';
import { CreateSoftwareProcessPolicyModule } from './create-software-process-policy/create-software-process-policy.module';
import { DeleteConfirmationModule } from './delete-confirmation/delete-confirmation.module';
import { DeviceidModule } from './deviceid/deviceid.module';
import { FooterModule } from './footer/footer.module';
import { ForgotPasswordModule } from './forgot-password/forgot-password.module';
import { VersionModule } from './version/version.module';
import { NavbarModule } from './navbar/navbar.module';
import { FreshLoginModule } from './fresh-login/fresh-login.module';
import { GauthModule } from './gauth/gauth.module';
import { GlobalMessagesModule } from './global-messages/global-messages.module';
import { HealthStatusModule } from './health-status/health-status.module';
import { LicenseModule } from './license/license.module';
import { LocalUserForgotPasswordModule } from './local-user-forgot-password/local-user-forgot-password.module';
import { LoginModule } from './login/login.module';
import { LogoutModule } from './logout/logout.module';
import { MobileClientModule } from './mobile-client/mobile-client.module';
import { ViewDeatilServiceModule } from './manage-service/manage-list/view-detail-service/view-detail-service.module';
import { GatewayHealthStatusModule } from './src/app/gateway-health-status/gateway-health-status.module';
import { FailedUploadSoftwaresModule } from './softwares-list/failed-upload-softwares/failed-upload-softwares.module';
import { FailedUploadDevicesModule } from './devices-list/failed-upload-devices/failed-upload-devices.module';
import { CreateQuickGroupModule } from './group/create-quick-group/create-quick-group.module';
import { CreateQuickGatewayGroupModule } from './gateway/gateway-list/create-quick-gateway-group/create-quick-gateway-group.module';
import { CreateQuickServiceGroupModule } from './manage-service/manage-list/create-quick-service-group/create-quick-service-group.module';
import { CreateRoutingPolicyModule } from './routing-policy/create-routing-policy/create-routing-policy.module';
import { GatewayGroupsModule } from './gateway-groups/gateway-group-list/gateway-groups.module';
import { PolicyModule } from './policy/policy.module';
import { RestoreModule } from './restore/restore.module';
import { GroupListModule } from './group/group-list/group-list.module';
import { CreateIdpModule } from './idp-management/create-idp/create-idp.module';
import { TopbarModule } from './topbar/topbar.module';
import { NotfoundModule } from './notfound/notfound.module';
import { OnboardModule } from './onboard/onboard.module';
import { VerificationModule } from './verification/verification.module';
import { RecoverModule } from './recover-password/recover.module';
import { CreateUserModule } from './users/create-user/create-user.module';
import { CreateLdapuserModule } from './ldap/creat-ldapuser/create-ldapuser.module';
import { CreateManageModule } from './manage-service/create-manage/create-manage.module';
import { CreateGatewayModule } from './gateway/create-gateway/create-gateway.module';
import { CreateControllerModule } from './controller/create-controller/create-controller.module';
import { CreateClientModule } from './client/create-client/create-client.module';
import { SettingsModule } from './settings/settings.module';
import { CreateUpdateModule } from './updates/create-update/create-update.module';
import { UserGroupsModule } from './users/user-groups/user-groups.module';
import { SecrityQuestionsModule } from './security-questions/security-questions.module';
import { StringTransformPipe } from './pipes/string.pipe';
import { OsTransformPipeModule } from './shared/os-transform-pipe/os-transform-pipe.module';
import { UserGroupsEditModule } from './users/user-groups-edit/user-groups-edit.module';
import { GatewayDetailModule } from './gateway/gateway-detail/gateway-detail.module';
import { OtpVerificationModule } from './otp-verification/otp-verification.module';
import { ViewRolesAndPermissionModule } from './roles-and-permission/view-roles-and-permission/view-roles-and-permission.module';
import { CreateRolesAndPermissionModule } from './roles-and-permission/create-roles-and-permission/create-roles-and-permission.module';
import { EditRolesAndPermissionModule } from './roles-and-permission/edit-roles-and-permission/edit-roles-and-permission.module';
import { RolesAndPermissionService } from './roles-and-permission/roles-and-permission.service';
import { S2SGuard } from './guard/s2s.guard';
import { FreshLogin } from './guard/fresh-login.guard';
import { ViewAdditionalPermissionsModule } from './additional-permissions/view-additional-permissions/view-additional-permissions.module';
import { CreateAdditionalPermissionsModule } from './additional-permissions/create-additional-permissions/create-additional-permissions.module';
import { AdditionalPermissionsService } from './additional-permissions/additional-permissions.service';
import { ApprovalModalModule } from './additional-permissions/approval-modal/approval-modal.module';
import { ViewModalModule } from './alert-list/view-modal/view-modal.module'
import { AdditionalPermissionsDetailsModule } from './additional-permissions/additional-permissions-details/additional-permissions-details.module';
import { AdditionalPermissionsGuard } from './guard/additional-permissions.guard';
import { CategoryService } from './topbar/nav.service';
import { OnboardingReportsModule } from './onboarding-reports/onboarding-reports.module';
import { CreateRbiBridgeModule } from './create-rbi-bridge/create-rbi-bridge.module';
import { WorkloadDevicesComponent } from './workload/workload-devices/workload-devices.component';
import { TableModule } from "./table/table.module";

import { MicrosegmentsModule } from './workload/microsegments/microsegments.module';
import { WorkloadDevicesModule } from './workload/workload-devices/workload-devices.module';
import { CreateWorkloadDevicesModule } from './workload/create-workload-devices/create-workload-devices.module';

declare let require: any;
let gapiKey = localStorage.getItem('gapiKey') || 'fakekey';

export function ConfigLoader(configService: ConfigService) {
  return () => configService.load(environment.configFile);
}

@NgModule({
  imports: [
    TimepickerModule.forRoot(),
    BrowserModule,
    BrowserAnimationsModule,
    FormsModule,
    AppRoutingModule,
    ModalModule.forRoot(),
    HttpClientModule,
    NgxPaginationModule,
    MatMenuModule,
    MatButtonModule,
    DragulaModule,
    DragDropModule,
    // ChartModule,
    QRCodeModule,
    NgSelectModule,
    AmChartsModule,
    BsDatepickerModule.forRoot(),
    PaginationModule.forRoot(),
    TooltipModule.forRoot(),
    RecaptchaModule,
    SecrityQuestionsModule,
    NgbModule,
    NgbDropdownModule,
    NgbAlertModule,
    OverlayscrollbarsModule,
    SharedModule,
    OsTransformPipeModule,
    ActivityIndicatorModule,
    AccessPolicyModule,
    WorkloadListModule,
    ValidDomainsModule,
    UserGroupsModule,
    ReactiveFormsModule,
    UserListModule,
    UsersReportsModule,
    UserAgentAccessPolicyModule,
    UpdatesListModule,
    TrustedHostsModule,
    SoftwaresListModule,
    SoftwareProcessPolicyModule,
    SoftwareProcessBindingModule,
    ServicesReportsModule,
    SaasBridgeModule,
    RoutingPolicyModule,
    ReportingsModule,
    ServiceGroupDetailModule,
    PolicyReportsModule,
    NewPaginationModule,
    OsPolicyModule,
    OrganizationListViewModule,
    MicrosegmentModule,
    MicrosegmentationModule,
    SpinnerModule,
    CreateWorkloadModule,
    SoftwarePolicyModule,
    ManageListModule,
    FilterCheckButtonModule,
    LocalUsersListModule,
    LdapuserListModule,
    IotModule,
    InfrastructureReportsModule,
    IdpListModule,
    GroupDetailModule,
    CreateGroupModule,
    GatewayListModule,
    GatewayDetailModule,
    UserGroupsEditModule,
    CreateGatwayGroupModule,
    ForbiddenSoftwareListModule,
    DevicesReportsModule,
    DevicesListModule,
    DevicePolicyModule,
    DeviceInventoryModule,
    DeviceAccessPolicyModule,
    DataVolumePolicyModule,
    GatewayHealthModule,
    DashboardModule,
    CreatePolicyModule,
    HostsByOSModule,
    CreateOrganizationModule,
    ControllerListModule,
    WhitelistedHostsModule,
    ClientsReportsModule,
    ClientListModule,
    ClientAccessLogModule,
    BackupsModule,
    AlertListModule,
    AdminAccessLogModule,
    AssignListModule,
    CreateAccessPolicyModule,
    CreateB2bBridgeModule,
    CreateDataVolumePolicyModule,
    CreateDevicePolicyModule,
    CreateDeviceModule,
    ConfirmDeleteDeviceModule,
    CreateLocalUserModule,
    CreateOsPolicyModule,
    CreateServerBridgeModule,
    CreateSoftwareModule,
    CreateSoftwarePolicyModule,
    CreateSoftwareProcessBindingModule,
    CreateSoftwareProcessPolicyModule,
    DeleteConfirmationModule,
    DeviceidModule,
    FooterModule,
    ForgotPasswordModule,
    VersionModule,
    NavbarModule,
    FreshLoginModule,
    GauthModule,
    GlobalMessagesModule,
    HealthStatusModule,
    LicenseModule,
    LocalUserForgotPasswordModule,
    LoginModule,
    LogoutModule,
    MobileClientModule,
    ViewDeatilServiceModule,
    WorkloadListModule,
    GatewayHealthStatusModule,
    FailedUploadSoftwaresModule,
    FailedUploadDevicesModule,
    CreateQuickGroupModule,
    CreateQuickGatewayGroupModule,
    CreateQuickServiceGroupModule,
    CreateRoutingPolicyModule,
    GatewayGroupsModule,
    PolicyModule,
    RestoreModule,
    GroupListModule,
    CreateIdpModule,
    TopbarModule,
    NotfoundModule,
    OnboardModule,
    VerificationModule,
    RecoverModule,
    CreateUserModule,
    CreateLdapuserModule,
    CreateManageModule,
    CreateGatewayModule,
    CreateControllerModule,
    CreateClientModule,
    SettingsModule,
    CreateUpdateModule,
    OtpVerificationModule,
    ToastrModule.forRoot({
      timeOut: 5000,
      positionClass: 'toast-bottom-right'
    }),
    AdditionalPermissionsDetailsModule,
    ViewRolesAndPermissionModule,
    ViewAdditionalPermissionsModule,
    CreateRolesAndPermissionModule,
    CreateAdditionalPermissionsModule,
    EditRolesAndPermissionModule,
    ApprovalModalModule,
    ViewModalModule,
    OnboardingReportsModule,
    CreateRbiBridgeModule,
    TableModule,
    MicrosegmentsModule,
    WorkloadDevicesModule,
    CreateWorkloadDevicesModule
  ],
  declarations: [
    AppComponent,
    WithoutSpaceValidator,
    DigitOnlyDirective,
    AutofocusDirective,
  ],
  providers: [
    DatePipe,
    DevicePolicyService,
    CategoryService,
    SoftwareProcessPolicyService,
    SoftwarePolicyService,
    OsPolicyService,
    OsPolicyServiceSoftware,
    IotService,
    AccessPolicyService,
    ValidDomainsService,
    PolicyService,
    UserService,
    ManageService,
    GatewayService,
    WorkloadService,
    GatewayGroupsService,
    AuthenticationService,
    GauthService,
    DeviceIdService,
    FreshLoginService,
    ForgotService,
    DashboardService,
    RecoverService,
    LdapService,
    ControllerService,
    ClientService,
    UpdateService,
    SettingsService,
    LicenseService,
    GlobalMessageService,
    IdpService,
    GroupService,
    BackupService,
    RestoreService,
    AdminAccessLogService,
    DeviceInventoryService,
    AuthShortlivedCookieService,
    LocalUserService,
    DeviceService,
    SoftwareService,
    SoftwareProcessBindingService,
    OrganizationService,
    ReportingService,
    UsersReportsService,
    DevicesReportsService,
    ServicesReportsService,
    ClientsReportsService,
    PolicyReportsService,
    InfrastructureReportsService,
    AlertServices,
    ForbiddenSoftwareServices,
    CheckToken,
    SaasBridgeService,
    MicrosegmentService,
    HttpClient,
    { provide: HTTP_INTERCEPTORS, useClass: JwtInterceptor, multi: true },
    RoleGuard,
    ConfigService,
    {
      provide: APP_INITIALIZER,
      useFactory: ConfigLoader,
      deps: [ConfigService],
      multi: true
    },
    AuthGuard,
    RecoverPasswordGuard,
    S2SGuard,
    AdditionalPermissionsGuard,
    ChangePasswordGuard,
    ChangeQuestionGuard,
    RolesResolve,
    TrustedHostsService,
    LocalTrustedHostsService,
    RoutingPolicyService,
    OsService,
    DataVolumePolicyService,
    NgbDropdownConfig,
    StringTransformPipe,
    RolesAndPermissionService,
    AdditionalPermissionsService,
    FreshLogin
  ],
  bootstrap: [AppComponent]
})
export class AppModule {}
