import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { UserService } from '../../users/user.service';
import { User } from '../../models/user';
import { ManageService } from '../../manage-service/manage.service';
import { Service } from '../../models/service';
import { Pagination } from '../../models/pagination';
import { PreserveSearch } from '../../models/preserve-search';
import { DragulaService } from 'ng2-dragula';
import { LdapUser } from '../../models/ldapuser';
import { Assign } from '../../models/assign';
import { AuthenticationService } from '../../login/authentication.service';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import * as _ from 'lodash';
import { Observable } from 'rxjs/internal/Observable';
import { tap } from 'rxjs/operators';
@Component({
  selector: 'app-assign-list',
  templateUrl: './assign-list.component.html',
  styleUrls: ['./assign-list.component.css']
})
export class AssignListComponent implements OnInit {
  private users: Observable<User[]>;
  public alert: GlobalMessageClass;
  private usersList = new Array();
  private selectedUser: LdapUser;
  services: Service[];
  private servicesList = new Array();
  private selectedService: Service;
  private pagination: Pagination = new Pagination();
  private Search: PreserveSearch = new PreserveSearch();
  ldapusers: LdapUser[];
  private assignedService = new Assign();
  selectedRowIndex: any = -1;
  assignedServices: any;
  private originalServices: any;
  constructor(
    private router: Router,
    private userService: UserService,
    private manageService: ManageService,
    private route: ActivatedRoute,
    private dragulaService: DragulaService,
    private authenticationService: AuthenticationService,
    private globalMessageService: GlobalMessageService
  ) {
    // this.dragulaService.drag('service-bag').subscribe(() => {
    //   // Do something on drag
    // });
    // dragulaService.drop().subscribe(({ name, el, target, source, sibling }) => {
    //   this.onDrop([el, target, source, sibling]);
    // });
    dragulaService.drop.subscribe(value => {
      this.onDrop(value.slice(1));
    });
    /*dragulaService.setOptions('service-bag', {
			copy: true
		});*/
  }

  private onDrop(args) {
    let [el, target, source, sibling] = args;

    if (source !== target) {
      if (target.id == 'assigned-services') {
        this.assignedService.ldapUserName = this.selectedUser.cn;
        this.assignedService.serviceid = el.id;
        this.manageService
          .assignService(this.assignedService)
          .subscribe(response => {
            if (response.success) {
              alert('Assigned');
            }
          });
      }

      if (target.id == 'available-services') {
        this.assignedService.ldapUserName = this.selectedUser.cn;
        this.assignedService.serviceid = el.id;
        this.manageService
          .unassignService(this.assignedService)
          .subscribe(response => {
            if (response.success) {
              alert('removed');
            }
          });
      }
    }
  }

  // getPage(page: number) {
  //   this.pagination.currentPage = page;
  //   this.manageService
  //     .getServices(this.pagination.currentPage, this.pagination.itemsPerPage)
  //     pipe(tap((res :any) => {
  //       this.services = res.services;
  //       this.originalServices = this.services;
  //       this.pagination.totalItems = res.total;
  //       this.pagination.currentPage = page;
  //     }))
  //     .subscribe({
  //       next: next => {},
  //       error: err => {
  //         if (err.status === 403) {
  //           if (err._body == LicenseErrorEnum.Invalid) {
  //             this.authenticationService.showLicenseOnly(true);
  //             this.router.navigate(['/license']);
  //           } else {
  //             this.alert = {
  //               closed: false,
  //               type: 'danger',
  //               message: 'You are not authorized to access this page.'
  //             };
  //             this.globalMessageService.pushAlert(this.alert);
  //           }
  //         }
  //       },
  //       complete: () => {
  //       }
  //     });
  // }
  getLdapUsers() {
    this.manageService
      .getLdapUsers()
      .pipe(
        tap(res => {
          this.ldapusers = res['ldapusers'];
        })
      )
      .subscribe();
  }
  public ngOnInit(): void {
    this.getLdapUsers();

    let serviceSearch = this.Search.getSearch('serviceSearch');
    if (serviceSearch) {
      this.pagination.currentPage = serviceSearch.current_page;
    }
    //this.getPage(this.pagination.currentPage);
  }

  private selectUser(user: LdapUser): void {
    this.selectedUser = user;
    this.selectedRowIndex = user.cn;
    //this.getPage(1);
    var ref = this;
    this.manageService
      .getAssignedServices(user.cn)
      .subscribe(assignedServices => {
        this.assignedServices = assignedServices;
        assignedServices.forEach(function(assignedService, index) {
          for (var i = 0; i < ref.services.length; i++) {
            if (ref.services[i]['id'] == assignedService.serviceid) {
              ref.services.splice(i, 1);
            }
          }
        });
      });
  }
}
