import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { ConfigService } from '../services/config.service';
import * as moment from 'moment';
import { AuthShortlivedCookieService } from '../auth-shortlived-cookie.service';
import { map } from 'rxjs/operators';
import { Title } from '@angular/platform-browser';

@Injectable()
export class BackupService {
  private apiBaseUrl: string;
  private backupApiUrl: string = '/backup';
  private downlaodBackupUrl: string = '/download';
  private downloadSpecificUrl: string = '/downloadUrl';
  private restoreBackupUrl: string = '/restore';

  constructor(
    private authHttp: AuthHttp,
    private congigService: ConfigService,
    private authShortlivedCookie: AuthShortlivedCookieService,
    private titleService: Title
  ) {
    this.titleService.setTitle('Settings | System Backup');
    this.apiBaseUrl = this.congigService.getConfiguration().apiBaseUrl;
    this.backupApiUrl = this.apiBaseUrl + this.backupApiUrl;
  }

  takeBackup(portalVersion) {
    return this.authHttp.post(this.backupApiUrl + this.downlaodBackupUrl, { portalVersion });
  }

  downloadFile(file: any): void {
    const timeDate = moment().format('Do MMMM YYYY, h:m A');
    const fileName = 'dump-' + timeDate + '.sql';
    var newBlob = new Blob([file], { type: 'application/sql' });
    var downloadUrl = window.URL.createObjectURL(newBlob);
    var link = document.createElement('a');
    link.href = downloadUrl;
    link.download = fileName;
    link.click();
  }

  getBackupsList(
    pageSize: number = 10,
    page: number = 1,
    sort: string = 'createdAt',
    dir: string = 'desc'
  ) {
    return this.authHttp
      .get(
        `${this.backupApiUrl}?limit=${pageSize}&offset=${page -
          1}&sort=${sort}&dir=${dir}`
      )
      .pipe(map(response => response));
  }

  downloadSpecificBackup(id) {
    this.authShortlivedCookie.setFor(5);

    window.location.href =
      this.backupApiUrl + this.downloadSpecificUrl + `/${id}`;
  }

  deleteBackup(id) {
    return this.authHttp
      .delete(this.backupApiUrl + `/${id}`)
      .pipe(map(response => response));
  }

  restoreBackup(id, verificationPassword) {
    return this.authHttp
      .post(this.backupApiUrl + this.restoreBackupUrl + `/${id}`, { verificationPassword });
  }
}
