import {
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  NgZone,
  ViewChild
} from '@angular/core';
import { ActivatedRoute, NavigationEnd, Router } from '@angular/router';
import { ClientService } from '../client.service';
import { Client } from '../../models/client';
import { Pagination } from '../../models/pagination';
import { PreserveSearch } from '../../models/preserve-search';
import * as _ from 'lodash';
import * as moment from 'moment';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { AuthenticationService } from '../../login/authentication.service';
import { Popup } from '../../Popup';
import { Subscription, Subject, timer } from 'rxjs';
import { of } from 'rxjs';
import {
  debounceTime,
  distinctUntilChanged,
  first,
  switchMap,
  tap
} from 'rxjs/operators';
import { ErrorCodeEnum } from '../../shared/emuns/error.enum';
import {
  ClientStatusEnum,
  ClientTypeEnum,
  DeviceOSEnum
} from '../../shared/emuns/client.enum';
import { LicenseErrorEnum } from '../../shared/emuns/license.enum';
import { InstallerStatusEnum } from '../../shared/emuns/global.enum';
import { ConfirmModalComponent } from '../../../app/shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { RolesAndPermissionService } from '../../roles-and-permission/roles-and-permission.service';

declare var anychart: any;

@Component({
  selector: 'app-client-list',
  templateUrl: './client-list.component.html',
  styleUrls: ['./client-list.component.css']
})
export class ClientListComponent implements OnInit, OnDestroy {
  showHelpText: boolean = false;
  public downloadInitialtedFor = new Array();
  public currentTime = new Date();
  public searchFiltersForm: any = {};
  public searchFilters: any = {};
  installerStatus: Popup = new Popup();
  logStatus: Popup = new Popup();
  qrStatus: Popup = new Popup();
  private timerSubscription: Subscription;
  public alert: GlobalMessageClass;
  clients: any = {};
  isLdapSyncing: boolean = false;
  isBiometric: boolean = false;
  isOtp: boolean = false;
  noShow: boolean = false;
  noShowMessage: string;
  noClientShow: boolean = false;
  private clientsList = new Array();
  private selectedClient: Client;
  private activeStatus = true;
  pagination: Pagination = new Pagination();
  private Search: PreserveSearch = new PreserveSearch();
  private hearbeatTimeThresholdInSeconds: number = 120;
  public showFilter: boolean = false;
  packageStatuses = ['', 'New', 'Waiting', 'Registered', 'Provisioned'];
  public isIdSort: boolean = true;
  public sortByNameOrder: boolean = false; //true DESC, false ASC
  public sortByIdOrder: boolean = true; //true DESC, false ASC

  public userrole: string = '';
  private navigatingToEditClient: boolean = false;
  public clientType;
  public clientTypePlural;
  public helpText: String;
  public clientTypeLabel;
  public actionsDropdown: number = 0;
  OTPQcodeImageData: string = null;
  showOtpImage: boolean = true;
  clientListSubscription;
  clientListCheckedIds = [];
  clientListCheckedNames = [];
  selectAll = false;
  auto_updates = true;
  autoUpdatesSwitchValue: boolean;

  selectAllExcept: any = new Set();
  selectedClients: any = new Set();

  isDeleting = false;
  showPasswordConfimation = false;
  adOrphand = false;
  localOrphand = false;
  isSearching = false;
  $searchClient = new Subject();
  searchKeyword;
  quickFilterValues = [];
  showQuickGroup: boolean = false;
  normalizedSelectedClients = {};

  ClientStatus = ClientStatusEnum;
  InstallerStatus = InstallerStatusEnum;
  ClientType = ClientTypeEnum;
  DeviceOS = DeviceOSEnum;

  public toggleDisplayStyle = 'none';
  myChart = null;
  @ViewChild('clientgraph') container;
  isDataLoaded = false;
  public createdOnce = false;

  quickFilters = [
    {
      isChecked: false,
      title: 'Allowed',
      value: 'is_active',
      status_value: 1,
      arrayValue: 1
    },
    {
      isChecked: false,
      title: 'Blocked',
      value: 'is_active',
      status_value: 0,
      arrayValue: 2
    },
    // {
    //   isChecked: false,
    //   title: 'Darkened',
    //   value: 'isDarkened',
    //   status_value: 1,
    //   arrayValue: 3
    // },
    // {
    //   isChecked: false,
    //   title: 'Not Darkened',
    //   value: 'isDarkened',
    //   status_value: 0,
    //   arrayValue: 4
    // },
    // {
    //   isChecked: false,
    //   title: 'MFA Enabled',
    //   value: 'isMfa',
    //   status_value: 1,
    //   arrayValue: 5
    // },
    // {
    //   isChecked: false,
    //   title: 'MFA Disabled',
    //   value: 'isMfa',
    //   status_value: 0,
    //   arrayValue: 6
    // },
    {
      isChecked: false,
      title: 'Offline',
      value: 'isOnline',
      status_value: 0,
      arrayValue: 7
    },
    {
      isChecked: false,
      title: 'Online',
      value: 'isOnline',
      status_value: 1,
      arrayValue: 8
    }
    // {
    //   isChecked: false,
    //   title: 'Local Orphans',
    //   value: 'localOrphand',
    //   status_value: 0,
    //   arrayValue: 9
    // },
    // {
    //   isChecked: false,
    //   title: 'AD Orphans',
    //   value: 'adOrphand',
    //   status_value: 0,
    //   arrayValue: 10
    // }
  ];
  quickFilterStatusValues = [];
  quickFiltersStatus = [
    {
      isChecked: true,
      title: 'New',
      values: 1
    },
    {
      isChecked: false,
      title: 'Waiting',
      values: 2
    },
    {
      isChecked: false,
      title: 'Registered',
      values: 3
    },
    {
      isChecked: false,
      title: 'Provisioned',
      values: 4
    }
  ];
  quickFilterOsValues = [];
  quickFiltersOs = [
    // {
    //   isChecked: false,
    //   title: 'Ubuntu 16 Desktop',
    //   value: DeviceOSEnum.Ubuntu
    // },
    // {
    //   isChecked: false,
    //   title: 'Ubuntu 16 Server',
    //   value: DeviceOSEnum.UbuntuServer
    // },
    {
      isChecked: false,
      title: 'Ubuntu Desktop',
      value: DeviceOSEnum.Ubuntu18
    },
    {
      isChecked: false,
      title: 'Ubuntu Server',
      value: DeviceOSEnum.UbuntuServer18
    },
    {
      isChecked: false,
      title: 'Ubuntu Server (ARMv8)',
      value: DeviceOSEnum.UbuntuServer20armv8
    },
    {
      isChecked: false,
      title: 'Windows',
      value: DeviceOSEnum.Windows
    },
    {
      isChecked: false,
      title: 'Windows Server',
      value: DeviceOSEnum.WindowsServer
    },
    {
      isChecked: false,
      title: 'Android',
      value: DeviceOSEnum.Android
    },
    {
      isChecked: false,
      title: 'iOS',
      value: DeviceOSEnum.IOS
    },
    {
      isChecked: false,
      title: 'macOS',
      value: DeviceOSEnum.Macintosh
    },
    {
      isChecked: false,
      title: 'CentOS Server',
      value: DeviceOSEnum.CentosServer
    }
  ];
  quickFiltersShow = false;
  quickFiltersShowState = false;
  quickFiltersShowOs = false;

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  quickFilterStatusValue = -1;
  quickFilterOsValue = '-1';

  loading: Boolean = true;
  noClientError: String = '';

  pageSize = [10, 25, 50, 100, 200];

  sort: string;
  sortDir: string;

  _status = {
    New: 1,
    Waiting: 2,
    Registered: 3,
    Provisioned: 4
  };

  isDarkMode: boolean = false;
  constructor(
    private router: Router,
    private clientService: ClientService,
    private route: ActivatedRoute,
    private globalMessageService: GlobalMessageService,
    private authenticationService: AuthenticationService,
    private ref: ChangeDetectorRef,
    public dialog: MatDialog,
    private titleService: Title,
    private rolesPermissionService: RolesAndPermissionService
  ) {
    this.route.params.subscribe(params => {
      this.clientType = params.type;
      this.parseFilters();
      this.clientTypeLabel =
        params.type === ClientTypeEnum.Server
          ? 'Server Client'
          : ClientTypeEnum.Client;
      this.clientTypePlural =
        params.type === ClientTypeEnum.Client
          ? 'End User Clients'
          : 'Server Side Clients';

      const clientGraphPage = localStorage.getItem('clientGraphPage');
      if (
        !clientGraphPage ||
        (clientGraphPage == 'Server' &&
          this.clientTypePlural == 'End User Clients')
      ) {
        localStorage.setItem('clientGraphPage', 'Desktop');
        // location.reload();
      } else if (
        !clientGraphPage ||
        (clientGraphPage == 'Desktop' &&
          this.clientTypePlural == 'Server Side Clients')
      ) {
        localStorage.setItem('clientGraphPage', 'Server');
        // location.reload();
      }

      this.helpText =
        params.type === ClientTypeEnum.Client
          ? 'Manage Invisily clients for desktop devices created during device and user binding. Please note that most of the client details are not editable.'
          : 'Manage Invisily clients for servers created during device and user binding or during creation of a bridge.';
      this.titleService.setTitle(
        'Client Management | ' + this.clientTypePlural
      );
      this.router.routeReuseStrategy.shouldReuseRoute = () => false;
      let clientSearch = this.Search.getSearch('clientSearch');

      let filterStatus = this.Search.getSearch('filterStatus');
      let filterOS = this.Search.getSearch('filterOs');
      let filterState = this.Search.getSearch('filterState');

      if (filterStatus) {
        this.quickFilterStatusValues = filterStatus;
      }

      if (filterOS) {
        this.quickFilterOsValues = filterOS;
      }

      if (filterState) {
        this.quickFilterValues = filterState;
      }

      if (clientSearch) {
        this.updateAllFilters(clientSearch.search);
        this.pagination = clientSearch.pagination;
        this.searchFiltersForm = { ...clientSearch.search };
        this.searchFilters = { ...clientSearch.search };
      }
      this.searchFormInit();
      this.refreshData();
      this.getSettingsRole();
    });

    this.userrole = localStorage.getItem('ur');
    if (typeof this.userrole != 'undefined' || this.userrole != null) {
      this.userrole = atob(this.userrole);
    }
  }

  private parseFilters() {
    if (this.clientType === ClientTypeEnum.Client) {
      //client
      this.quickFiltersOs = this.quickFiltersOs.filter(os => {
        return os.value !== DeviceOSEnum.WindowsServer;
      });
    } else {
      //false
      this.quickFiltersOs = this.quickFiltersOs.filter(os => {
        return os.value.includes('server');
      });
    }
  }

  private updateAllFilters(search) {
    if (search['name']) {
      this.searchKeyword = search['name'];
    }
    this.quickFiltersOs = this.quickFiltersOs.map(singleFilter => {
      if (
        search['device_os'] &&
        search['device_os'].includes(singleFilter.value)
      ) {
        return {
          ...singleFilter,
          isChecked: true
        };
      }
      return singleFilter;
    });
    this.quickFiltersStatus = this.quickFiltersStatus.map(singleFilter => {
      if (
        search['package_status'] &&
        search['package_status'] !== '-1' &&
        search['package_status'].includes(singleFilter.values)
      ) {
        return {
          ...singleFilter,
          isChecked: true
        };
      }
      return singleFilter;
    });

    this.quickFilters = this.quickFilters.map(singleFilter => {
      if (search[singleFilter.value]) {
        if (
          singleFilter.value === 'localOrphand' ||
          singleFilter.value === 'adOrphand'
        ) {
          if (search['localOrphand'] || search['adOrphand']) {
            return {
              ...singleFilter,
              isChecked: true
            };
          }
        }
        const searchArray = search[singleFilter.value].split(',');
        if (searchArray.length == 2) {
          return {
            ...singleFilter,
            isChecked: true
          };
        } else {
          return {
            ...singleFilter,
            isChecked:
              search[singleFilter.value] == singleFilter.status_value
                ? true
                : false
          };
        }
      } else {
        return singleFilter;
      }
    });
  }

  private resetDropdowns(e) {
    let elem = e.target;
    while (elem && elem !== window.document) {
      elem = elem?.parentNode;
    }
    this.actionsDropdown = 0;
    this.ref.markForCheck();
  }

  quickFilterHandler() {
    this.quickFiltersShowState = false;
    this.quickFiltersShowOs = false;
    this.quickFiltersShow = !this.quickFiltersShow;
  }

  quickFilterStateHandler() {
    this.quickFiltersShowState = !this.quickFiltersShowState;
    this.quickFiltersShow = false;
    this.quickFiltersShowOs = false;
  }

  quickFilterOsHandler() {
    this.quickFiltersShowState = false;
    this.quickFiltersShow = false;
    this.quickFiltersShowOs = !this.quickFiltersShowOs;
  }

  inputChanged(event) {
    event.stopPropagation();
  }

  clearFirstFilter(event) {
    event.stopPropagation();
    this.quickFilters = this.quickFilters.map(filter => {
      return {
        ...filter,
        isChecked: false
      };
    });
  }

  clearSecondFilter(event) {
    event.stopPropagation();
    this.quickFiltersStatus = this.quickFiltersStatus.map(filter => {
      return {
        ...filter,
        isChecked: false
      };
    });
    // this.quickFilterStatusValue = -1;
  }

  clearThirdFilter(event) {
    event.stopPropagation();
    this.quickFiltersOs = this.quickFiltersOs.map(filter => {
      return {
        ...filter,
        isChecked: false
      };
    });
    // this.quickFilterOsValue = '-1';
  }

  bulkAutoUpdate(state) {
    let clientsId = this.clients.data.map((data: any) => data.id);
    let bulkDataSet = {
      selectAll: !this.searchKeyword && this.selectAll ? true : false,
      selectAllExcept: [...this.selectAllExcept],
      selectedClients:
        !this.searchKeyword ||
        !this.quickFilterOsValues ||
        !this.quickFilterValues ||
        !this.quickFilterStatusValues ||
        this.selectedClients.size > 0
          ? [...this.selectedClients]
          : [...clientsId],
      switch: this.autoUpdatesSwitchValue,
      clientType: this.clientType,
      searchKeyword: this.searchKeyword
    };

    this.clientService.bulkAutoUpdate(bulkDataSet, state).subscribe(
      response => {
        this.selectAll = false;
        this.selectAllExcept.clear();
        this.selectedClients.clear();
        this.clientListCheckedIds = [];
        this.clientListCheckedNames = [];
        this.refreshData();
        this.alert = {
          closed: false,
          type: 'success',
          message: `Successfully auto-updated selected client(s)`
        };
        this.globalMessageService.pushAlert(this.alert);
      },
      error => {
        if (error) {
          this.alert = {
            closed: false,
            type: 'error',
            message: 'Failed to auto-update selected client(s)'
          };
          this.globalMessageService.pushAlert(this.alert);
        }
      }
    );
  }

  applyFirstFilter() {
    this.adOrphand = false;
    this.localOrphand = false;
    const { device_os, package_status } = this.searchFiltersForm;
    this.searchFiltersForm = {
      device_os,
      package_status
    };
    if (this.quickFilterValues.indexOf(1) !== -1) {
      //Allowed
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        is_active: '1'
      };
    }
    if (this.quickFilterValues.indexOf(2) !== -1) {
      //Blockoed
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        is_active: '0'
      };
    }
    if (
      this.quickFilterValues.indexOf(1) !== -1 &&
      this.quickFilterValues.indexOf(2) !== -1
    ) {
      //Allow + Blockoed
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        is_active: '0,1'
      };
    }
    if (this.quickFilterValues.indexOf(3) !== -1) {
      //Darkened
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isDarkened: '1'
      };
    }
    if (this.quickFilterValues.indexOf(4) !== -1) {
      //Not Darkened
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isDarkened: '0'
      };
    }
    if (
      this.quickFilterValues.indexOf(3) !== -1 &&
      this.quickFilterValues.indexOf(4) !== -1
    ) {
      //dark + not dark
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isDarkened: '0,1'
      };
    }
    if (this.quickFilterValues.indexOf(5) !== -1) {
      //mfa
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isMfa: '1'
      };
    }
    if (this.quickFilterValues.indexOf(6) !== -1) {
      //Not mfa
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isMfa: '0'
      };
    }
    if (
      this.quickFilterValues.indexOf(6) !== -1 &&
      this.quickFilterValues.indexOf(5) !== -1
    ) {
      //mfa + not mfa
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isMfa: '0,1'
      };
    }
    if (this.quickFilterValues.indexOf(8) !== -1) {
      //online
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isOnline: '1'
      };
    }
    if (this.quickFilterValues.indexOf(7) !== -1) {
      //Not online
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isOnline: '0'
      };
    }
    if (
      this.quickFilterValues.indexOf(8) !== -1 &&
      this.quickFilterValues.indexOf(7) !== -1
    ) {
      //online + not online
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isOnline: '0,1'
      };
    }

    if (this.quickFilterValues.indexOf(9) !== -1) {
      this.localOrphand = true;
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        localOrphand: true
      };
    }
    if (this.quickFilterValues.indexOf(10) !== -1) {
      this.adOrphand = true;
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        adOrphand: true
      };
    }

    this.searchData();

    /*event.stopPropagation();
    this.quickFilters.forEach(singleFilter => {
      const filterValue = singleFilter.value;
      if (singleFilter.isChecked) {
        this.searchFiltersForm = {
          ...this.searchFiltersForm,
          [filterValue]: true
        };
      } else {
        if (this.searchFiltersForm[filterValue]) {
          delete this.searchFiltersForm[filterValue];
        }
      }
    });*/
  }

  statusFilterChanged() {
    const packageStatusArray = this.quickFilterStatusValues;
    delete this.searchFiltersForm.package_status;
    this.searchFiltersForm = {
      ...this.searchFiltersForm,
      package_status: packageStatusArray.toString()
    };
    if (!packageStatusArray.length) {
      // delete this.searchFiltersForm.package_status;
      this.searchFiltersForm.package_status = '-1';
    }
    this.searchData();
  }

  osFilterChanged() {
    const osArray = this.quickFilterOsValues;
    delete this.searchFiltersForm.device_os;
    this.searchFiltersForm = {
      ...this.searchFiltersForm,
      device_os: osArray.toString()
    };
    if (!osArray.length) {
      // delete this.searchFiltersForm.device_os;
      this.searchFiltersForm.device_os = '-1';
    }
    this.searchData();
  }

  applySecondHandler(event) {
    event.stopPropagation();
    const packageStatusArray = [];
    delete this.searchFiltersForm.package_status;
    this.quickFiltersStatus.forEach(singleStatus => {
      if (singleStatus.isChecked) {
        packageStatusArray.push(singleStatus.values);
      }
    });
    this.searchFiltersForm = {
      ...this.searchFiltersForm,
      package_status: packageStatusArray.toString()
    };
    if (!packageStatusArray.length) {
      delete this.searchFiltersForm.package_status;
    }
    this.clearAllFiltersDropDowns();
    this.searchData();
  }

  applyThirdHandler(event) {
    event.stopPropagation();
    const osArray = [];
    delete this.searchFiltersForm.device_os;
    this.quickFiltersOs.forEach(singleOs => {
      if (singleOs.isChecked) {
        osArray.push(singleOs.value);
      }
    });
    this.searchFiltersForm = {
      ...this.searchFiltersForm,
      device_os: osArray.toString()
    };
    if (!osArray.length) {
      delete this.searchFiltersForm.device_os;
    }
    this.clearAllFiltersDropDowns();
    this.searchData();
  }

  clearAllFiltersDropDowns() {
    this.quickFiltersShow = false;
    this.quickFiltersShowState = false;
    this.quickFiltersShowOs = false;
  }

  private searchFormInit() {
    if (!this.searchFiltersForm.device_os) {
      this.searchFiltersForm.device_os = '-1';
      this.searchFilters.device_os = '-1';
    }

    if (!this.searchFiltersForm.package_status) {
      this.searchFiltersForm.package_status = '-1';
      this.searchFilters.package_status = '-1';
    }

    if (!this.searchFiltersForm.is_active) {
      this.searchFiltersForm.is_active = '-1';
      this.searchFilters.is_active = '-1';
    }
  }

  ngOnInit() {
    //this.getPage(this.pagination.currentPage);
    //this.refreshData();
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.subscibeToSearch();
    window.document.addEventListener('click', this.resetDropdowns.bind(this));

    // resetSearch() not called when the browser reloads
    window.onbeforeunload = () => this.ngOnDestroy();
  }

  getSettingsRole() {
    this.router.events.subscribe(event => {
      if (event instanceof NavigationEnd) {
        const currentUrl = this.router.url;
        if (currentUrl == '/clients/Client') {
          this.getCRUDdata(44);
        } else if (currentUrl == '/clients/Server') {
          this.getCRUDdata(45);
        }
      }
    });
  }

  subscibeToSearch() {
    this.$searchClient
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap(data => {
          this.loading = true;
          this.isSearching = true;
          this.searchFiltersForm = {
            ...this.searchFiltersForm,
            name: data
          };
          this.searchKeyword = data;
          this.searchData();
          return of('client fetching...');
        })
      )
      .subscribe(() => {
        this.loading = false;
        this.isSearching = false;
      });
  }

  getCRUDdata(privId) {
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission == 1 ? true : false;
      this.readPermission = res?.read_permission == 1 ? true : false;
      this.updatePermission = res?.update_permission == 1 ? true : false;
      this.deletePermission = res?.delete_permission == 1 ? true : false;
    });
  }

  private onDownload(client: Client): void {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message:
          'Are you sure you want to request a new service installer and download it?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        if (!this.downloadInitialtedFor.includes(client.id)) {
          this.downloadInitialtedFor.push(client.id);
        }
        this.selectedClient = client;

        this.clientService
          .generate_client(client.id, 1)
          .pipe(
            tap((response: any) => {
              if (response.status) {
                this.alert = {
                  closed: false,
                  type: 'success',
                  message:
                    'Service Installer request has been initiated. Your package download will start within two minutes meanwhile, you can view the status of the request by clicking on the status button.'
                };
                this.globalMessageService.pushAlert(this.alert);
                this.getPage(this.pagination.currentPage);
              }
            })
          )
          .subscribe({
            next: next => {},
            error: err => {
              if (err.status === ErrorCodeEnum.Forbidden) {
                if (err._body == LicenseErrorEnum.Invalid) {
                  this.authenticationService.showLicenseOnly(true);
                  this.router.navigate(['/license']);
                } else {
                  // this.alert = {
                  //   closed: false,
                  //   type: 'danger',
                  //   message: 'You are not authorized to access this page.'
                  // };
                  // this.globalMessageService.pushAlert(this.alert);
                }
              }
            },
            complete: () => {}
          });
      }
    });
  }

  cancelInstaller(client) {
    client.installer_status = InstallerStatusEnum.Canceled;
    this.clientService.cancelInstallerStatus(client).subscribe(
      data => {},
      error => {
        this.alert = {
          closed: false,
          type: 'danger',
          message: 'Something went wrong in canceling'
        };
        this.globalMessageService.pushAlert(this.alert);
      }
    );
  }

  public showFilters(): void {
    if (this.showFilter) {
      this.showFilter = false;
      this.timerSubscription.unsubscribe();
    } else {
      this.showFilter = true;
    }
  }

  private onSelect(client: Client): void {
    let clientSearch = {
      pagination: this.pagination,
      search: this.searchFilters
    };
    this.Search.saveSearch('clientSearch', clientSearch);
    this.Search.saveSearch('filterStatus', this.quickFilterStatusValues);
    this.Search.saveSearch('filterOs', this.quickFilterOsValues);
    this.Search.saveSearch('filterState', this.quickFilterValues);
    this.selectedClient = client;
    this.navigatingToEditClient = true;
    this.router.navigate(['/clients', this.clientType, this.selectedClient.id]);
  }

  private onClone(client: Client): void {
    let clientSearch = {
      pagination: this.pagination,
      search: this.searchFilters
    };
    this.Search.saveSearch('clientSearch', clientSearch);
    this.Search.saveSearch('filterStatus', this.quickFilterStatusValues);
    this.Search.saveSearch('filterOs', this.quickFilterOsValues);
    this.Search.saveSearch('filterState', this.quickFilterValues);
    this.selectedClient = client;
    this.navigatingToEditClient = true;
    this.router.navigate([
      '/clients',
      this.clientType,
      this.selectedClient.id,
      true
    ]);
  }

  public searchData(): void {
    this.selectAll = false;
    this.unsubsctibeFromList();
    this.searchFilters = { ...this.searchFiltersForm };
    this.refreshData(1);
    this.clientListCheckedIds = [];
  }

  private unsubsctibeFromList() {
    if (this.clientListSubscription) {
      this.clientListSubscription.unsubscribe();
    }
  }

  public searchDataEnter(event: any) {}

  adOrphandsHandler(status) {
    this.adOrphand = status;
    this.getLocalAdClients();
  }

  private getLocalAdClients() {
    this.unsubsctibeFromList();
    this.selectAll = false;
    this.clientListCheckedIds = [];
    this.clientListCheckedNames = [];
    this.pagination.currentPage = 1;
    this.getPage(this.pagination.currentPage);
  }

  localOrphandsHandler(status) {
    this.localOrphand = status;
    this.getLocalAdClients();
  }

  public refreshData(number = null): void {
    if (this.timerSubscription) {
      this.timerSubscription.unsubscribe();
    }
    if (number != null) {
      this.pagination.currentPage = number;
    }
    this.getPage(this.pagination.currentPage);
    this.subscribeToData();
  }

  private subscribeToData(): void {
    this.timerSubscription = timer(10000)
      .pipe(first())
      .subscribe(() => this.refreshData());
  }

  public ngOnDestroy(): void {
    window.document.removeEventListener('click', this.resetDropdowns);
    if (this.clientListSubscription) {
      this.clientListSubscription.unsubscribe();
    }
    if (this.timerSubscription) {
      this.timerSubscription.unsubscribe();
    }

    /**
     * Reset search only if the admin is not going to edit
     * a client from the search results
     **/
    if (!this.navigatingToEditClient) {
      this.clearSearch();
    }
  }

  clearSearch() {
    this.searchFiltersForm = {};
    this.searchFilters = {};
    this.searchFormInit();
    this.showFilter = false;

    this.pagination.currentPage = 1;
    this.pagination.itemsPerPage = 10;
    let clientSearch = {
      pagination: this.pagination,
      search: this.searchFiltersForm
    };
    this.Search.saveSearch('clientSearch', clientSearch);
    this.Search.saveSearch('filterStatus', null);
    this.Search.saveSearch('filterOs', null);
    this.Search.saveSearch('filterState', null);
  }

  resetSearch() {
    this.clearSearch();
    if (this.timerSubscription) {
      this.timerSubscription.unsubscribe();
    }
    this.getPage(1);
    this.subscribeToData();
    this.clientListCheckedIds = [];
    this.clientListCheckedNames = [];
    this.selectAll = false;
  }

  copyText() {
    var copyText = document.getElementById('code-here');
    let innerText: any = copyText.innerText;
    let selBox = document.createElement('textarea');
    selBox.style.height = '0';
    selBox.value = innerText;
    document.body.appendChild(selBox);
    selBox.focus();
    selBox.select();
    document.execCommand('copy');
    this.installerStatus.heading = 'Installer Status';
    this.installerStatus.body = InstallerStatusEnum.Copied;
    this.installerStatus.action = 'Confirm';
    this.installerStatus.show = true;
    setTimeout(() => {
      this.cancelStatusPopup();
    }, 2000);
  }

  private generateNoShowMessage() {
    const search = this.searchFilters;
    const messages = [
      `${search.name && search.name.length > 0 ? `name "${search.name}"` : ''}`,
      `${
        search.package_status !== '-1'
          ? `this package status "${
              this.packageStatuses[search.package_status]
            }"`
          : ''
      }`,
      `${
        search.agent_id && search.agent_id !== '-1'
          ? `agent ID "${search.agent_id}"`
          : ''
      }`,
      `${search.device_os !== '-1' ? `OS "${search.device_os}"` : ''}`,
      `${
        search.is_active !== '-1'
          ? search.is_active == '1'
            ? 'status "active"'
            : 'status "inactive"'
          : ''
      }`
    ];

    const validMessages = messages.filter(message => message !== '');
    return validMessages.join(' and ');
  }

  public hasSearchFilters() {
    if (this.searchFilters) {
      let hasAnyFilter = false;

      for (let filter in this.searchFilters) {
        if (
          this.searchFilters[filter] &&
          this.searchFilters[filter].length &&
          this.searchFilters[filter] !== '-1'
        ) {
          hasAnyFilter = true;
        }
      }

      if (hasAnyFilter) {
        return hasAnyFilter;
      }
    }
  }

  showOTPQrCode(client) {
    this.showOtpImage = true;
    this.clientService.generateOTPToken(client).subscribe(
      (data: any) => {
        client.OTP_image_data = data.OTP_image_data;
        this.OTPQcodeImageData = client.OTP_image_data;
        this.showOtpImage = false;
      },
      error => {
        this.alert = {
          closed: false,
          type: 'danger',
          message: 'Error in generating linOTP Token'
        };
        this.globalMessageService.pushAlert(this.alert);
      }
    );
  }

  cancelOTPQrCode() {
    this.showOtpImage = true;
  }

  private initiateDownload(clients) {
    var index = -1;
    clients.forEach(client => {
      if (this.downloadInitialtedFor.includes(client.id)) {
        if (
          client.installer_status === InstallerStatusEnum.Success &&
          client.isDownloadPackage === true &&
          client.packageDownloaded === false
        ) {
          index = this.downloadInitialtedFor.indexOf(client.id);
          if (index > -1) {
            this.downloadInitialtedFor.splice(index, 1);
          }
          window.location.assign(client.onPrenDownloadLink);
        }
      }
    });
  }

  pageChangeHandler(page) {
    //this.clientListCheckedIds = [];
    this.getPage(page);
  }

  getOSName(osName) {
    if (!osName) return '';
    switch (osName) {
      case DeviceOSEnum.Ubuntu:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer:
        return 'Ubuntu Server';
      case DeviceOSEnum.Ubuntu18:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer18:
        return 'Ubuntu Server';
      case DeviceOSEnum.UbuntuServer20armv8:
        return 'Ubuntu Server (ARMv8)';
      case DeviceOSEnum.Windows:
        return 'Windows';
      case DeviceOSEnum.WindowsServer:
        return 'Windows Server';
      case DeviceOSEnum.Macintosh:
        return 'Macintosh';
      case DeviceOSEnum.IOS:
        return 'iOS';
      case DeviceOSEnum.Android:
        return 'Android';
      case DeviceOSEnum.CentosServer:
        return 'CentOS Server';
      case DeviceOSEnum.CentosServer7:
        return 'CentOS 7 Server';
      case DeviceOSEnum.CentosServer8:
        return 'CentOS 8 Server';
      case 'unknown':
        return 'Unknown';
      default:
        return osName;
    }
  }

  getPage(page: number) {
    this.loading = true;
    this.currentTime = new Date();
    let prevPage = this.pagination.currentPage;
    this.pagination.currentPage = page;

    // if (this.hasSearchFilters()) {
    //   this.showFilter = true;
    // }

    if (this.searchKeyword) {
      this.searchFilters['name'] = this.searchKeyword;
    }

    this.clientListSubscription = this.clientService
      .getClients(
        this.pagination.currentPage,
        this.pagination.itemsPerPage,
        this.searchFilters,
        this.sort,
        this.sortDir,
        this.clientType,
        this.adOrphand,
        this.localOrphand
      )
      .pipe(
        tap((res: any) => {
          //this.computeOnlineStatus(res.client);
          this.loading = false;
          this.clients = res;

          res.data = res.data.map(client => {
            return { ...client, isChecked: false };
          });

          this.clients = res;

          setTimeout(() => {
            this.changeCheckStatusOfAllClients(this.selectAll);
          }, 10);

          this.addIsCheckedInClients(this.clients.data);
          this.isLdapSyncing =
            res.isLdapSyncing == null ? false : res.isLdapSyncing;
          this.isBiometric = res.biometric == null ? false : res.biometric;
          this.isOtp = res.otp == null ? false : res.otp;
          this.initiateDownload(res.data);
          this.pagination.totalItems = res.recordsTotal;
          if (res.recordsTotal == 0 && this.hasSearchFilters()) {
            this.noClientError = '';

            this.noShow = true;

            let selectedPackageStatus = this.quickFiltersStatus.filter(x =>
              this.quickFilterStatusValues.includes(x.values)
            );
            let selectedOS = this.quickFiltersOs.filter(x =>
              this.quickFilterOsValues.includes(x.value)
            );
            let selectedQuickFilters = this.quickFilters.filter(x =>
              this.quickFilterValues.includes(x.arrayValue)
            );

            if (this.searchKeyword) {
              this.noClientError = 'name ' + this.searchKeyword;
            }

            if (selectedPackageStatus.length > 0) {
              this.noClientError =
                this.noClientError +
                (this.noClientError ? ' and' : '') +
                ' package status ' +
                selectedPackageStatus
                  .map(status => {
                    return '"' + status.title + '"';
                  })
                  .join(', ');
            }

            if (selectedOS.length > 0) {
              this.noClientError =
                this.noClientError +
                (this.noClientError ? ' and' : '') +
                ' OS ' +
                selectedOS
                  .map(os => {
                    return '"' + os.title + '"';
                  })
                  .join(', ');
            }

            if (selectedQuickFilters.length > 0) {
              this.noClientError =
                this.noClientError +
                (this.noClientError ? ' and' : '') +
                ' status ' +
                selectedQuickFilters
                  .map(status => {
                    return '"' + status.title + '"';
                  })
                  .join(', ');
            }

            this.noShowMessage = this.generateNoShowMessage();
          } else if (res.recordsTotal == 0) {
            this.noClientShow = true;
            this.noShow = false;
          } else {
            this.noClientShow = false;
            this.noShow = false;
          }
          this.pagination.currentPage = page;
          if (prevPage != this.pagination.currentPage) {
            let scrollToTop = window.setInterval(() => {
              let pos = window.pageYOffset;
              if (pos > 0) {
                window.scrollTo(0, pos - 20); // how far to scroll on each step
              } else {
                window.clearInterval(scrollToTop);
              }
            }, 16);
          }
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          this.loading = false;
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              // this.alert = {
              //   closed: false,
              //   type: 'danger',
              //   message: 'You are not authorized to access this page.'
              // };
              // this.globalMessageService.pushAlert(this.alert);
            }
          }
        },
        complete: () => {}
      });
  }

  clearfun() {
    this.searchKeyword = '';
    this.clearSearch();
    this.getPage(this.pagination.currentPage);
  }

  private addIsCheckedInClients(clients) {
    let count = 0;
    this.clients.data = clients.map(client => {
      //find client id if exist in check list array
      const check = this.clientListCheckedIds.find(id => id === client.id)
        ? true
        : false;
      if (check) {
        count = count + 1;
      }
      return {
        ...client,
        isChecked: check
      };
    });
    // if (count === this.clients.length) {
    //   this.selectAll = false;
    // }
  }

  changeCheckBox(client) {
    const { isChecked, id } = client;

    const targetClient = this.clients.data.find(c => c.agent_id === id);
    this.autoUpdatesSwitchValue = targetClient.auto_updates;

    if (this.selectAll) {
      if (isChecked) {
        this.selectAllExcept.delete(id);
      } else {
        this.selectAllExcept.add(id);
      }
    } else {
      if (!isChecked) {
        delete this.normalizedSelectedClients[id];
        this.selectedClients.delete(id);
      } else {
        this.normalizedSelectedClients[id] = client;
        this.selectedClients.add(id);
      }
    }
  }

  cancelVerificationPassword(event) {
    this.showPasswordConfimation = false;
  }

  submitVerificationPassword(event) {
    this.isDeleting = true;
    if (this.clientListSubscription) {
      this.clientListSubscription.unsubscribe();
    }
    const selectedIds = this.selectAll
      ? this.selectAllExcept
      : this.selectedClients;
    const filters = {
      status: this.quickFilterStatusValues,
      os: this.quickFilterOsValues,
      quickFilters: Object.keys(this.searchFiltersForm)
        .filter(key => /^is|^localOrphand|^adOrphand/.test(key))
        .reduce((obj, key) => {
          obj[key] = this.searchFiltersForm[key];
          return obj;
        }, {})
    };

    this.clientService
      .deleteBulkClients(
        this.selectAll,
        selectedIds,
        this.clientType,
        filters,
        this.searchKeyword,
        event
      )
      .subscribe(
        data => {
          this.isDeleting = false;
          this.showPasswordConfimation = false;
          this.selectAll = false;
          this.clientListCheckedIds = [];
          this.clientListCheckedNames = [];
          this.refreshData();
          this.alert = {
            closed: false,
            type: 'success',
            message: 'Client(s) deleted successfully.'
          };
          this.globalMessageService.pushAlert(this.alert);
        },
        error => {
          //this.showPasswordConfimation = false;
          this.isDeleting = false;
          // this.alert = {
          //   closed: false,
          //   type: 'danger',
          //   message: 'Something went wrong in deleting clients.'
          // };
          // this.globalMessageService.pushAlert(this.alert);
        }
      );
  }

  selectAllAction() {
    if (!this.selectAll) {
      this.selectedClients.clear();
    } else {
      this.selectedClients.clear();
    }
    this.selectAllExcept.clear();
    this.normalizedSelectedClients = {};
    this.changeCheckStatusOfAllClients(this.selectAll);
    // if (this.selectAll) {
    //   this.clientListCheckedIds.push(...this.clients.data.map(client => client.id));
    //   this.clientListCheckedNames.push(
    //     ...this.clients.data.map(c => {
    //       return { id: c.id, user: c.user };
    //     })
    //   );
    //   this.changeCheckStatusOfAllClients(true);
    // } else {
    //   const Ids = this.clients.data.map(c => c.id);
    //   this.clientListCheckedIds = this.clientListCheckedIds.filter(id => {
    //     return Ids.indexOf(id) === -1;
    //   });
    //   this.clientListCheckedNames = this.clientListCheckedNames.filter(c => {
    //     return Ids.indexOf(c.id) === -1;
    //   });
    //   this.changeCheckStatusOfAllClients(false);
    // }
    // this.selectAll = !this.selectAll;
  }

  private changeCheckStatusOfAllClients(check) {
    this.clients.data = this.clients.data.map(client => {
      this.autoUpdatesSwitchValue = client.auto_updates;
      return {
        ...client,
        isChecked:
          (!this.selectAllExcept.has(client.id) && this.selectAll) ||
          (this.selectedClients.has(client.id) && !this.selectAll)
      };
    });
  }

  deleteSelectedClients() {
    this.showPasswordConfimation = true;
  }

  blockSelectedClients() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'Are you sure you want to block selected client(s) ?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        if (this.clientListSubscription) {
          this.clientListSubscription.unsubscribe();
        }

        const filters = {
          status: this.quickFilterStatusValues,
          os: this.quickFilterOsValues,
          quickFilters: Object.keys(this.searchFiltersForm)
            .filter(key => /^is|^localOrphand|^adOrphand/.test(key))
            .reduce((obj, key) => {
              obj[key] = this.searchFiltersForm[key];
              return obj;
            }, {})
        };

        const selectedIds = this.selectAll
          ? this.selectAllExcept
          : this.selectedClients;
        this.clientService
          .bulkActions(
            this.selectAll,
            selectedIds,
            this.clientType,
            'changeStatusBulkClients',
            filters,
            this.searchKeyword,
            0
          )
          .subscribe(
            data => {
              this.showPasswordConfimation = false;
              this.selectAll = false;
              this.selectAllExcept.clear();
              this.selectedClients.clear();
              this.clientListCheckedIds = [];
              this.clientListCheckedNames = [];
              this.refreshData();
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Client(s) updated successfully.'
              };
              this.globalMessageService.pushAlert(this.alert);
            },
            error => {
              this.showPasswordConfimation = false;
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'Something went wrong in updating client(s).'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          );
      }
    });
  }

  forceOnboard() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'Are you sure you want to allow selected client(s) ?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        if (this.clientListSubscription) {
          this.clientListSubscription.unsubscribe();
        }

        const filters = {
          status: this.quickFilterStatusValues,
          os: this.quickFilterOsValues,
          quickFilters: Object.keys(this.searchFiltersForm)
            .filter(key => /^is|^localOrphand|^adOrphand/.test(key))
            .reduce((obj, key) => {
              obj[key] = this.searchFiltersForm[key];
              return obj;
            }, {})
        };

        const selectedIds = this.selectAll
          ? this.selectAllExcept
          : this.selectedClients;
        this.clientService
          .bulkActions(
            this.selectAll,
            selectedIds,
            this.clientType,
            'forceOnboard',
            filters,
            this.searchKeyword,
            1
          )
          .subscribe(
            (response: any) => {
              this.showPasswordConfimation = false;
              this.selectAll = false;
              this.selectAllExcept.clear();
              this.selectedClients.clear();
              this.clientListCheckedIds = [];
              this.clientListCheckedNames = [];
              this.refreshData();
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Client(s) updated successfully.'
              };
              this.globalMessageService.pushAlert(this.alert);
            },
            (error: any) => {
              this.showPasswordConfimation = false;
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'Something went wrong in updating client(s).'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          );
      }
    });
  }

  unblockSelectedClients() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'Are you sure you want to allow selected client(s) ?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        if (this.clientListSubscription) {
          this.clientListSubscription.unsubscribe();
        }

        const filters = {
          status: this.quickFilterStatusValues,
          os: this.quickFilterOsValues,
          quickFilters: Object.keys(this.searchFiltersForm)
            .filter(key => /^is|^localOrphand|^adOrphand/.test(key))
            .reduce((obj, key) => {
              obj[key] = this.searchFiltersForm[key];
              return obj;
            }, {})
        };

        const selectedIds = this.selectAll
          ? this.selectAllExcept
          : this.selectedClients;
        this.clientService
          .bulkActions(
            this.selectAll,
            selectedIds,
            this.clientType,
            'changeStatusBulkClients',
            filters,
            this.searchKeyword,
            1
          )
          .subscribe(
            data => {
              this.showPasswordConfimation = false;
              this.selectAll = false;
              this.selectAllExcept.clear();
              this.selectedClients.clear();
              this.clientListCheckedIds = [];
              this.clientListCheckedNames = [];
              this.refreshData();
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Client(s) updated successfully.'
              };
              this.globalMessageService.pushAlert(this.alert);
            },
            error => {
              this.showPasswordConfimation = false;
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'Something went wrong in updating client(s).'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          );
      }
    });
  }

  sendInstallerLinkToSelected() {
    if (this.clientListSubscription) {
      this.clientListSubscription.unsubscribe();
    }
    if (
      Object.values(this.normalizedSelectedClients).some(
        (client: any) => !client.device_os
      )
    ) {
      const alert = {
        closed: false,
        type: 'danger',
        message: "Can't send installer link to a ARM Platform client"
      };
      this.globalMessageService.pushAlert(alert);
      return;
    }
    const selectedIds = Object.keys(this.normalizedSelectedClients);
    const allSelected = this.selectAll;
    this.sendInstallerLink(selectedIds, allSelected);
  }

  enableDarkeningSelectedClients() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message:
          'Are you sure you want to enable darkening for selected client(s) ?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        if (this.clientListSubscription) {
          this.clientListSubscription.unsubscribe();
        }

        const filters = {
          status: this.quickFilterStatusValues,
          os: this.quickFilterOsValues,
          quickFilters: Object.keys(this.searchFiltersForm)
            .filter(key => /^is|^localOrphand|^adOrphand/.test(key))
            .reduce((obj, key) => {
              obj[key] = this.searchFiltersForm[key];
              return obj;
            }, {})
        };

        const selectedIds = this.selectAll
          ? this.selectAllExcept
          : this.selectedClients;
        this.clientService
          .bulkActions(
            this.selectAll,
            selectedIds,
            this.clientType,
            'changeDarkeningStatusBulkClients',
            filters,
            this.searchKeyword,
            1
          )
          .subscribe(
            data => {
              this.showPasswordConfimation = false;
              this.selectAll = false;
              this.selectAllExcept.clear();
              this.selectedClients.clear();
              this.clientListCheckedIds = [];
              this.clientListCheckedNames = [];
              this.refreshData();
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Client(s) updated successfully.'
              };
              this.globalMessageService.pushAlert(this.alert);
            },
            error => {
              this.showPasswordConfimation = false;
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'Something went wrong in updating client(s).'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          );
      }
    });
  }

  disableDarkeningSelectedClients() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message:
          'Are you sure you want to disable darkening for selected client(s) ?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        if (this.clientListSubscription) {
          this.clientListSubscription.unsubscribe();
        }

        const filters = {
          status: this.quickFilterStatusValues,
          os: this.quickFilterOsValues,
          quickFilters: Object.keys(this.searchFiltersForm)
            .filter(key => /^is|^localOrphand|^adOrphand/.test(key))
            .reduce((obj, key) => {
              obj[key] = this.searchFiltersForm[key];
              return obj;
            }, {})
        };

        const selectedIds = this.selectAll
          ? this.selectAllExcept
          : this.selectedClients;
        this.clientService
          .bulkActions(
            this.selectAll,
            selectedIds,
            this.clientType,
            'changeDarkeningStatusBulkClients',
            filters,
            this.searchKeyword,
            0
          )
          .subscribe(
            data => {
              this.showPasswordConfimation = false;
              this.selectAll = false;
              this.selectAllExcept.clear();
              this.selectedClients.clear();
              this.clientListCheckedIds = [];
              this.clientListCheckedNames = [];
              this.refreshData();
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Client(s) updated successfully.'
              };
              this.globalMessageService.pushAlert(this.alert);
            },
            error => {
              this.showPasswordConfimation = false;
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'Something went wrong in updating client(s).'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          );
      }
    });
  }

  enableMfaSelectedClients() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'Are you sure you want to enable MFA for selected client(s) ?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        if (this.clientListSubscription) {
          this.clientListSubscription.unsubscribe();
        }

        const filters = {
          status: this.quickFilterStatusValues,
          os: this.quickFilterOsValues,
          quickFilters: Object.keys(this.searchFiltersForm)
            .filter(key => /^is|^localOrphand|^adOrphand/.test(key))
            .reduce((obj, key) => {
              obj[key] = this.searchFiltersForm[key];
              return obj;
            }, {})
        };

        const selectedIds = this.selectAll
          ? this.selectAllExcept
          : this.selectedClients;
        this.clientService
          .bulkActions(
            this.selectAll,
            selectedIds,
            this.clientType,
            'changeMfaStatusBulkClients',
            filters,
            this.searchKeyword,
            1
          )
          .subscribe(
            data => {
              this.showPasswordConfimation = false;
              this.selectAll = false;
              this.selectAllExcept.clear();
              this.selectedClients.clear();
              this.clientListCheckedIds = [];
              this.clientListCheckedNames = [];
              this.refreshData();
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Client(s) updated successfully.'
              };
              this.globalMessageService.pushAlert(this.alert);
            },
            error => {
              this.showPasswordConfimation = false;
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'Something went wrong in updating client(s).'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          );
      }
    });
  }

  disableMfaSelectedClients() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'Are you sure you want to disable MFA for selected client(s) ?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        if (this.clientListSubscription) {
          this.clientListSubscription.unsubscribe();
        }

        const filters = {
          status: this.quickFilterStatusValues,
          os: this.quickFilterOsValues,
          quickFilters: Object.keys(this.searchFiltersForm)
            .filter(key => /^is|^localOrphand|^adOrphand/.test(key))
            .reduce((obj, key) => {
              obj[key] = this.searchFiltersForm[key];
              return obj;
            }, {})
        };

        const selectedIds = this.selectAll
          ? this.selectAllExcept
          : this.selectedClients;
        this.clientService
          .bulkActions(
            this.selectAll,
            selectedIds,
            this.clientType,
            'changeMfaStatusBulkClients',
            filters,
            this.searchKeyword,
            0
          )
          .subscribe(
            data => {
              this.showPasswordConfimation = false;
              this.selectAll = false;
              this.selectAllExcept.clear();
              this.selectedClients.clear();
              this.clientListCheckedIds = [];
              this.clientListCheckedNames = [];
              this.refreshData();
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Client(s) updated successfully.'
              };
              this.globalMessageService.pushAlert(this.alert);
            },
            error => {
              this.showPasswordConfimation = false;
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'Something went wrong in updating client(s).'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          );
      }
    });
  }

  enableBiometricMfaSelectedClients() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message:
          'Are you sure you want to enable Biometric MFA for selected client(s) ?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        if (this.clientListSubscription) {
          this.clientListSubscription.unsubscribe();
        }

        const filters = {
          status: this.quickFilterStatusValues,
          os: this.quickFilterOsValues,
          quickFilters: Object.keys(this.searchFiltersForm)
            .filter(key => /^is|^localOrphand|^adOrphand/.test(key))
            .reduce((obj, key) => {
              obj[key] = this.searchFiltersForm[key];
              return obj;
            }, {})
        };

        const selectedIds = this.selectAll
          ? this.selectAllExcept
          : this.selectedClients;
        this.clientService
          .bulkActions(
            this.selectAll,
            selectedIds,
            this.clientType,
            'changeBiometricMfaStatusBulkClients',
            filters,
            this.searchKeyword,
            1
          )
          .subscribe(
            data => {
              this.showPasswordConfimation = false;
              this.selectAll = false;
              this.selectAllExcept.clear();
              this.selectedClients.clear();
              this.clientListCheckedIds = [];
              this.clientListCheckedNames = [];
              this.refreshData();
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Client(s) updated successfully.'
              };
              this.globalMessageService.pushAlert(this.alert);
            },
            error => {
              this.showPasswordConfimation = false;
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'Something went wrong in updating client(s).'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          );
      }
    });
  }

  disableBiometricMfaSelectedClients() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message:
          'Are you sure you want to disable Biometric MFA for selected client(s) ?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        if (this.clientListSubscription) {
          this.clientListSubscription.unsubscribe();
        }

        const filters = {
          status: this.quickFilterStatusValues,
          os: this.quickFilterOsValues,
          quickFilters: Object.keys(this.searchFiltersForm)
            .filter(key => /^is|^localOrphand|^adOrphand/.test(key))
            .reduce((obj, key) => {
              obj[key] = this.searchFiltersForm[key];
              return obj;
            }, {})
        };

        const selectedIds = this.selectAll
          ? this.selectAllExcept
          : this.selectedClients;
        this.clientService
          .bulkActions(
            this.selectAll,
            selectedIds,
            this.clientType,
            'changeBiometricMfaStatusBulkClients',
            filters,
            this.searchKeyword,
            0
          )
          .subscribe(
            data => {
              this.showPasswordConfimation = false;
              this.selectAll = false;
              this.selectAllExcept.clear();
              this.selectedClients.clear();
              this.clientListCheckedIds = [];
              this.clientListCheckedNames = [];
              this.refreshData();
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Client(s) updated successfully.'
              };
              this.globalMessageService.pushAlert(this.alert);
            },
            error => {
              this.showPasswordConfimation = false;
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'Something went wrong in updating client(s).'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          );
      }
    });
  }

  enableStrongMfaSelectedClients() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message:
          'Are you sure you want to enable Strong MFA for selected client(s) ?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        if (this.clientListSubscription) {
          this.clientListSubscription.unsubscribe();
        }

        const filters = {
          status: this.quickFilterStatusValues,
          os: this.quickFilterOsValues,
          quickFilters: Object.keys(this.searchFiltersForm)
            .filter(key => /^is|^localOrphand|^adOrphand/.test(key))
            .reduce((obj, key) => {
              obj[key] = this.searchFiltersForm[key];
              return obj;
            }, {})
        };

        const selectedIds = this.selectAll
          ? this.selectAllExcept
          : this.selectedClients;
        this.clientService
          .bulkActions(
            this.selectAll,
            selectedIds,
            this.clientType,
            'changeStrongMfaStatusBulkClients',
            filters,
            this.searchKeyword,
            1
          )
          .subscribe(
            data => {
              this.showPasswordConfimation = false;
              this.selectAll = false;
              this.selectAllExcept.clear();
              this.selectedClients.clear();
              this.clientListCheckedIds = [];
              this.clientListCheckedNames = [];
              this.refreshData();
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Client(s) updated successfully.'
              };
              this.globalMessageService.pushAlert(this.alert);
            },
            error => {
              this.showPasswordConfimation = false;
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'Something went wrong in updating client(s).'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          );
      }
    });
  }

  disableStrongMfaSelectedClients() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message:
          'Are you sure you want to disable Strong MFA for selected client(s) ?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        if (this.clientListSubscription) {
          this.clientListSubscription.unsubscribe();
        }

        const filters = {
          status: this.quickFilterStatusValues,
          os: this.quickFilterOsValues,
          quickFilters: Object.keys(this.searchFiltersForm)
            .filter(key => /^is|^localOrphand|^adOrphand/.test(key))
            .reduce((obj, key) => {
              obj[key] = this.searchFiltersForm[key];
              return obj;
            }, {})
        };

        const selectedIds = this.selectAll
          ? this.selectAllExcept
          : this.selectedClients;
        this.clientService
          .bulkActions(
            this.selectAll,
            selectedIds,
            this.clientType,
            'changeStrongMfaStatusBulkClients',
            filters,
            this.searchKeyword,
            0
          )
          .subscribe(
            data => {
              this.showPasswordConfimation = false;
              this.selectAll = false;
              this.selectAllExcept.clear();
              this.selectedClients.clear();
              this.clientListCheckedIds = [];
              this.clientListCheckedNames = [];
              this.refreshData();
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Client(s) updated successfully.'
              };
              this.globalMessageService.pushAlert(this.alert);
            },
            error => {
              this.showPasswordConfimation = false;
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'Something went wrong in updating client(s).'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          );
      }
    });
  }

  sendOtpQrSelectedClients() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message:
          'Are you sure you want to send OTP QR Code to selected client(s) ?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        if (this.clientListSubscription) {
          this.clientListSubscription.unsubscribe();
        }

        const filters = {
          status: this.quickFilterStatusValues,
          os: this.quickFilterOsValues,
          quickFilters: Object.keys(this.searchFiltersForm)
            .filter(key => /^is|^localOrphand|^adOrphand/.test(key))
            .reduce((obj, key) => {
              obj[key] = this.searchFiltersForm[key];
              return obj;
            }, {})
        };

        const selectedIds = this.selectAll
          ? this.selectAllExcept
          : this.selectedClients;
        this.clientService
          .bulkActions(
            this.selectAll,
            selectedIds,
            this.clientType,
            'sendOtpQrBulkClients',
            filters,
            this.searchKeyword
          )
          .subscribe(
            data => {
              this.showPasswordConfimation = false;
              this.selectAll = false;
              this.selectAllExcept.clear();
              this.selectedClients.clear();
              this.clientListCheckedIds = [];
              this.clientListCheckedNames = [];
              this.refreshData();
              this.alert = {
                closed: false,
                type: 'success',
                message: 'OTP QR code sent.'
              };
              this.globalMessageService.pushAlert(this.alert);
            },
            error => {
              this.showPasswordConfimation = false;
              this.alert = {
                closed: false,
                type: 'danger',
                message:
                  'Something went wrong in sending OTP QR Code to client(s).'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          );
      }
    });
  }

  /*private computeOnlineStatus(clients) {
    let heartBeatSentAt;
    let allowedTime: moment.Moment = moment().subtract(
      this.hearbeatTimeThresholdInSeconds,
      'seconds'
    );
    let diffInSeconds: number;
    let heartBeat;

    clients.forEach(client => {
      heartBeat = client.heart_beat;
      heartBeatSentAt = moment(client.heart_beat_sent_at);
      diffInSeconds = heartBeatSentAt.diff(allowedTime, 'second');
      if (heartBeat) {
        if (diffInSeconds >= 0) {
          client.is_online = true;
        } else {
          client.is_online = false;
        }
      } else {
        client.is_online = false;
      }
    });
  }*/

  private onDelete(client: Client): void {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'Are you sure you want to delete it?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.selectedClient = client;

        this.clientService
          .delete_client(client.id, this.clientType, client.user)
          .pipe(
            tap((response: any) => {
              if (response) {
                if (response.success) {
                  this.alert = {
                    closed: false,
                    type: 'success',
                    message:
                      'Client deleted successfully.You have to generate new OTP for other clients of this user.'
                  };
                  this.globalMessageService.pushAlert(this.alert);
                  this.getPage(this.pagination.currentPage);
                } else {
                  this.alert = {
                    closed: false,
                    type: 'danger',
                    message:
                      'Something went wrong while processing this request kindly request again.'
                  };
                  this.globalMessageService.pushAlert(this.alert);
                  this.getPage(this.pagination.currentPage);
                }
              }
            })
          )
          .subscribe({
            next: next => {},
            error: err => {
              if (err.status === ErrorCodeEnum.Forbidden) {
                if (err._body == LicenseErrorEnum.Invalid) {
                  this.authenticationService.showLicenseOnly(true);
                  this.router.navigate(['/license']);
                } else {
                  // this.alert = {
                  //   closed: false,
                  //   type: 'danger',
                  //   message: 'You are not authorized to access this page.'
                  // };
                  // this.globalMessageService.pushAlert(this.alert);
                }
              }
            },
            complete: () => {}
          });
      }
    });
  }

  public unLock(client: Client): void {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'Are you sure you want to unlock this client?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.selectedClient = client;

        this.clientService
          .unlock_client(client.id)
          .pipe(
            tap(response => {
              if (response.success) {
                this.alert = {
                  closed: false,
                  type: 'success',
                  message: 'Client unlocked successfully.'
                };
                this.globalMessageService.pushAlert(this.alert);
                this.getPage(this.pagination.currentPage);
              }
            })
          )
          .subscribe({
            next: next => {},
            error: err => {
              if (err.status === ErrorCodeEnum.Forbidden) {
                if (err._body == LicenseErrorEnum.Invalid) {
                  this.authenticationService.showLicenseOnly(true);
                  this.router.navigate(['/license']);
                } else {
                  // this.alert = {
                  //   closed: false,
                  //   type: 'danger',
                  //   message: 'You are not authorized to access this page.'
                  // };
                  // this.globalMessageService.pushAlert(this.alert);
                }
              }
            },
            complete: () => {}
          });
      }
    });
  }

  cancelStatusPopup() {
    this.installerStatus.show = false;
  }

  cancelLogPopup() {
    this.logStatus.show = false;
  }

  cancelQRPopup() {
    this.qrStatus.show = false;
  }

  isExpired(client) {
    return (
      client.package_used || moment(client.link_key_expiry).isBefore(moment())
    );
  }

  allowButton(client: Client): boolean {
    let utcTime = new Date(client.package_sent_at);
    if (client.installer_status === InstallerStatusEnum.Processing) {
      if (client.isDeviceIdSet) {
        return false;
      }
      if (utcTime < this.currentTime) {
        return false;
      }
      return true;
    } else {
      return false;
    }
  }

  private onStatus(client: Client): void {
    this.selectedClient = client;
    this.installerStatus.heading = 'Installer Status';
    this.installerStatus.body = client.installer_status;
    this.installerStatus.action = 'Confirm';
    this.installerStatus.show = true;
    setTimeout(() => {
      this.cancelStatusPopup();
    }, 2000);
  }

  private showLogs(client: Client): void {
    if (client.installer_logs) {
      this.logStatus.heading = 'Installer Logs';
      this.logStatus.body = client.installer_logs;
      if (client.installer_status == InstallerStatusEnum.Success) {
        document.getElementById('code-here').innerHTML =
          '' + this.logStatus.body.replace(/,/g, '<br/>');
      } else {
        document.getElementById('code-here').innerHTML =
          '>> ' + this.logStatus.body.replace(/,/g, '<br/>>> ');
      }
      this.logStatus.action = 'Confirm';
      this.logStatus.show = true;
    }
  }

  showQR(client: Client): void {
    if (!this.isExpired(client)) {
      if (client.installer_logs) {
        this.qrStatus.heading = 'Installer Logs';
        this.qrStatus.body = client.qr_json;
        this.qrStatus.action = 'Confirm';
        this.qrStatus.show = true;
      }
    }
  }

  onGenerate(client: Client): void {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'Are you sure you want to request a new service installer ?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.selectedClient = client;
        client.installer_status = InstallerStatusEnum.Processing;
        client.package_used = false;

        this.clientService
          .generate_client(client.id, 0)
          .pipe(
            tap((response: any) => {
              if (response.status) {
                this.alert = {
                  closed: false,
                  type: 'success',
                  message:
                    "Service Installer request initiated you'll receive an email within two minutes. Scan QR code after one minute."
                };
                this.globalMessageService.pushAlert(this.alert);
                this.getPage(this.pagination.currentPage);
              }
            })
          )
          .subscribe({
            next: next => {},
            error: err => {
              if (err.status === ErrorCodeEnum.Forbidden) {
                if (err._body == LicenseErrorEnum.Invalid) {
                  this.authenticationService.showLicenseOnly(true);
                  this.router.navigate(['/license']);
                } else {
                  // this.alert = {
                  //   closed: false,
                  //   type: 'danger',
                  //   message: 'You are not authorized to access this page.'
                  // };
                  // this.globalMessageService.pushAlert(this.alert);
                }
              }
            },
            complete: () => {}
          });
      }
    });
  }

  private sendLink(client: Client): void {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Are you sure you want to send installer link to ${client.Name} via email?`
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.sendInstallerLink([client.id], false);
      }
    });
  }

  sendInstallerLink(ids, allSelected) {
    const clientSubject = ids.length > 0 || allSelected ? 'Clients' : 'Client';
    const is_online = this.searchFilters.isOnline;
    const is_active = this.searchFilters.is_active;
    this.clientService
      .sendLink(
        ids,
        allSelected,
        Array.from(this.selectAllExcept),
        this.clientType,
        this.quickFilterOsValues,
        this.searchKeyword,
        this.quickFilterStatusValues,
        is_online,
        is_active
      )
      .pipe(
        tap((response: any) => {
          if (response.status) {
            const alert = {
              closed: false,
              type: 'success',
              message: `${clientSubject} will receive an email with installer link within two minutes.`
            };
            this.globalMessageService.pushAlert(alert);
            this.getPage(this.pagination.currentPage);
          }
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              // this.alert = {
              //   closed: false,
              //   type: 'danger',
              //   message: 'You are not authorized to access this page.'
              // };
              // this.globalMessageService.pushAlert(this.alert);
            }
          }
        },
        complete: () => {}
      });
  }

  add_client(): void {
    this.router.navigate([`/create-client`, this.clientType]);
  }

  deregMFA(client: Client) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Do you really want to deregister ${client.Name} from MFA server?`
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.clientService
          .deregMFA(client)
          .pipe(
            tap((response: any) => {
              if (response.statusCode === 1200) {
                const alert = {
                  closed: false,
                  type: 'success',
                  message: 'Client will deregister within few seconds.'
                };
                this.globalMessageService.pushAlert(alert);
              } else {
                const alert = {
                  closed: false,
                  type: 'danger',
                  message: 'Deregisteration from MFA failed.'
                };
                this.globalMessageService.pushAlert(alert);
              }
            })
          )
          .subscribe({
            next: next => {},
            error: err => {
              if (err.status === ErrorCodeEnum.Forbidden) {
                if (err._body == LicenseErrorEnum.Invalid) {
                  this.authenticationService.showLicenseOnly(true);
                  this.router.navigate(['/license']);
                } else {
                  // this.alert = {
                  //   closed: false,
                  //   type: 'danger',
                  //   message: 'You are not authorized to access this page.'
                  // };
                  // this.globalMessageService.pushAlert(this.alert);
                }
              }
            },
            complete: () => {}
          });
      }
    });
  }

  sendOTPQrCode(client: Client) {
    this.clientService
      .sendOTPQrCode(client)
      .pipe(
        tap((response: any) => {
          if (response.success) {
            const alert = {
              closed: false,
              type: 'success',
              message: 'Client will receive OTP Code within 2 minutes.'
            };
            this.globalMessageService.pushAlert(alert);
          }
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              // this.alert = {
              //   closed: false,
              //   type: 'danger',
              //   message: 'You are not authorized to access this page.'
              // };
              // this.globalMessageService.pushAlert(this.alert);
            }
          } else {
            const alert = {
              closed: false,
              type: 'danger',
              message: 'Something went wrong while requesting OTP QR Code.'
            };
            this.globalMessageService.pushAlert(alert);
          }
        },
        complete: () => {}
      });
  }

  toggle(client: Client, index) {
    this.clientService
      .change_status(client)
      .pipe(
        tap(response => {
          if (response.success) {
            this.clients.data[index].is_active = !this.clients.data[index]
              .is_active;
            if (this.clients.data[index].is_active) {
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Client access allowed'
              };
              this.globalMessageService.pushAlert(this.alert);
            } else {
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Client access blocked'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          }
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              // this.alert = {
              //   closed: false,
              //   type: 'danger',
              //   message: 'You are not authorized to access this page.'
              // };
              // this.globalMessageService.pushAlert(this.alert);
            }
          }
        },
        complete: () => {}
      });
  }

  approve(client: Client, index) {
    this.clientService
      .approve(client)
      .pipe(
        tap(response => {
          if (response.success) {
            client.client_status = ClientStatusEnum.Registered;
            const alert = {
              closed: false,
              type: 'success',
              message: 'Client has been approved for registration successfully.'
            };
            this.globalMessageService.pushAlert(alert);
          }
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              // this.alert = {
              //   closed: false,
              //   type: 'danger',
              //   message: 'You are not authorized to access this page.'
              // };
              // this.globalMessageService.pushAlert(this.alert);
            }
          }
        },
        complete: () => {}
      });
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getPage(this.pagination.currentPage);
  }

  sortRecord(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
    this.getPage(this.pagination.currentPage);
  }

  popOverCheckOne(client) {
    return (
      (client.client_status === ClientStatusEnum.Waiting ||
        client.client_status === ClientStatusEnum.Provisioned ||
        client.client_status === ClientStatusEnum.Registered) &&
      (client.device_os !== DeviceOSEnum.IOS ||
        client.device_os !== DeviceOSEnum.Android)
    );
  }

  popOverCheckTwo(client) {
    return (
      !(
        (client.client_status === ClientStatusEnum.Waiting ||
          client.client_status === ClientStatusEnum.Provisioned ||
          client.client_status === ClientStatusEnum.Registered) &&
        (client.device_os !== DeviceOSEnum.IOS ||
          client.device_os !== DeviceOSEnum.Android)
      ) &&
      (client.is_ldap_deleted ||
        client.client_status === ClientStatusEnum.Registered ||
        client.is_local_deleted)
    );
  }

  async handleAddToGroup() {
    if (
      this.selectAll &&
      (this.selectAllExcept.size < 1 || this.searchKeyword)
    ) {
      this.normalizedSelectedClients = {};
    }
    if (
      this.selectAll &&
      (this.isFilterApplied() || this.selectAllExcept.size > 0)
    ) {
      const is_online = this.searchFilters.isOnline;
      const is_active = this.searchFilters.is_active;
      const response = (await this.clientService
        .getQuickClients(
          this.selectAll,
          Array.from(this.selectAllExcept),
          this.clientType,
          this.quickFilterOsValues,
          this.searchKeyword,
          this.quickFilterStatusValues,
          is_online,
          is_active
        )
        .toPromise()) as any;
      const clients = response.data;
      if (clients) {
        let normalize = {};
        clients.forEach(client => {
          normalize[client.id] = client;
        });
        this.normalizedSelectedClients = normalize;
      }
    }
    this.showQuickGroup = true;
  }

  closeQuickGroup() {
    this.showQuickGroup = false;
  }

  successCloseQuickGroup() {
    this.showQuickGroup = false;
    this.selectAll = false;
    this.selectedClients = new Set();
    this.normalizedSelectedClients = {};
    this.changeCheckStatusOfAllClients(this.selectAll);
  }

  isFilterApplied() {
    const applied =
      this.quickFilterOsValues.length > 0 ||
      this.quickFilterValues.length > 0 ||
      this.searchKeyword ||
      this.quickFilterStatusValues.length > 0 ||
      this.selectAllExcept.size > 0;
    return applied;
  }

  closeTogglePopup() {
    this.toggleDisplayStyle = 'none';
    this.isDataLoaded = false;
  }

  clientGraph(client) {
    this.clientService.getClientGraph(client.id).subscribe(data => {
      this.isDataLoaded = true;
      this.getGraph(data, client);
    });
  }

  zoomIn() {
    this.myChart && this.myChart.zoomIn();
  }

  zoomOut() {
    this.myChart && this.myChart.zoomOut();
  }

  getGraph(clientData, client) {
    let elem = this.container.nativeElement;
    let routingPolicyExist = false;

    if (clientData.length === 0) {
      this.alert = {
        closed: false,
        type: 'danger',
        message: 'No service is attached to this client'
      };
      this.globalMessageService.pushAlert(this.alert);
      return;
    }

    const uniqueNodes = new Set();
    uniqueNodes.add(
      JSON.stringify({
        id: client.id,
        group: 'client',
        name: client.Name,
        height: 60,

        fill: {
          src: '/assets/images/client.png'
        }
      })
    );

    if (this.isDataLoaded) {
      // Iterate over each object in the JSON array
      clientData.forEach(obj => {
        // Add service node
        uniqueNodes.add(
          JSON.stringify({
            id: obj.serviceId,
            group: 'service',
            name: obj.serviceName,
            height: 30,
            fill: {
              src: '/assets/images/service.jpg'
            }
          })
        );

        // Add gateway node
        if (obj.routingPolicyId) {
          routingPolicyExist = true;
          uniqueNodes.add(
            JSON.stringify({
              id: obj.gatewayId,
              group: 'gateway',
              name: obj.gatewayName,
              height: 30,
              fill: {
                src: '/assets/images/router.svg'
              }
            })
          );
        }
      });

      // Convert the Set back to an array of objects
      const nodes = Array.from(uniqueNodes).map((data: string) =>
        JSON.parse(data)
      );

      const edges = new Set();

      // Iterate over each object in the clientData array
      clientData.forEach(obj => {
        // Add edge from client to gateway if gateway exists in service
        if (obj.routingPolicyId) {
          edges.add(
            JSON.stringify({
              from: client.id,
              to: obj.gatewayId
            })
          );
        }

        // Add edge from gateway to service
        edges.add(
          JSON.stringify({
            from: obj.gatewayId || client.id,
            to: obj.serviceId
          })
        );
      });

      const uniqueEdges = Array.from(edges).map((data: string) =>
        JSON.parse(data)
      );

      // create data
      var data = {
        nodes: nodes,
        edges: uniqueEdges
      };

      if (this.createdOnce) {
        this.myChart.edges().arrows({
          enabled: false
        });
        this.myChart.data(data);
        this.toggleDisplayStyle = 'block';

        return;
      }

      var chart = anychart.graph(data);

      // chart.background().fill('transparent');
      chart.credits(false);
      // enable labels of nodes
      chart
        .nodes()
        ?.labels()
        .enabled(true);

      // configure labels of nodes
      chart
        .nodes()
        ?.labels()
        .format('{%id}');
      chart
        .nodes()
        ?.labels()
        .fontSize(12);
      chart
        .nodes()
        ?.labels()
        .fontWeight(600);

      // configure labels of nodes in groups
      chart
        .group('service')
        ?.labels()
        .format('{%name}');
      if (routingPolicyExist)
        chart
          .group('gateway')
          ?.labels()
          .format('{%name}');
      // chart.group("client").labels().format("{%name}");

      if (routingPolicyExist)
        chart
          .group('gateway')
          ?.labels()
          .fontColor('#fffff');
      chart
        .group('client')
        ?.labels()
        .fontColor('#fffff');
      chart
        .group('service')
        ?.labels()
        .fontColor('#fffff');

      chart
        .nodes()
        .normal()
        .stroke(null);
      chart.nodes().shape('circle');
      // chart.nodes().width(100);
      // chart.nodes().height(100);
      // chart.autoRedraw(false);
      chart
        .edges()
        .normal()
        .stroke('#32a852', 2);
      chart
        .edges()
        .hovered()
        .stroke('#32a852', 3);
      chart
        .edges()
        .selected()
        .stroke('#32a852', 3);

      // configure tooltips of nodes
      chart
        .nodes()
        .tooltip()
        .useHtml(true);
      chart
        .nodes()
        .tooltip()
        .format(
          "<span style='font-weight:bold'>ID: {%id} <br>Type: {%group} <br>Status: Active"
        );

      // configure tooltips of edges
      chart
        .edges()
        .tooltip()
        .format('{%from} -> {%to}');

      chart.edges().tooltip(false);
      // chart.title("Client Connectivity Graph");

      // chart.layout().type("fixed");

      // create a chart and set the data
      chart.container(elem);
      chart.draw();
      // this.createdOnce = true;
      this.myChart = chart;
      this.createdOnce = true;

      this.toggleDisplayStyle = 'block';
    }
  }

  doRerender() {
    this.ref.detectChanges();
  }
}
