import { Injectable, ɵConsole } from '@angular/core';
import { HttpClient, HttpHeaders, HttpResponse } from '@angular/common/http';
import { ConfigService } from '../services/config.service';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { AuthenticationService } from '../login/authentication.service';
import { PaginationResponse } from '../models/pagination-response';
import { Client } from '../models/client';
import { AuthShortlivedCookieService } from '../auth-shortlived-cookie.service';
import { ClientAccessRestriction } from '../models/clientaccessrestriction';
import { ManageService } from '../manage-service/manage.service';
import { Service } from '../models/service';
import { TrustedHost } from '../models/trusted-host';
import { Observable } from 'rxjs/internal/Observable';
import { map } from 'rxjs/operators';
import { BehaviorSubject } from 'rxjs';
declare var require: any;
const moment = require('../../../node_modules/moment');
@Injectable()
export class ClientService {
  private clientsUrl = 'http://localhost:3000/clients';
  private clientsApiUrl = '/clients';
  private clientsgatewayApiUrl = '/clientsgateway';
  private clientsChangeStatusApiUrl = '/clients/status';
  private clientsAccessLogsApiUrl = '/clients-access-logs';
  private cancelInstallerStatusUrl = '/clients/cancelPackage';
  private clients: Client[];
  private apiBaseUrl: string;
  private settingApiUrl = '/settings';
  private settingGlobalIpApiUrl = '/settings/checkRule/';
  private clientTypeSource = new BehaviorSubject<string>('');
  clientType$ = this.clientTypeSource.asObservable();
  httpOptions = {
    headers: new HttpHeaders({
      'Content-Type': 'application/json'
    })
  };
  constructor(
    private http: HttpClient,
    private authenticationService: AuthenticationService,
    private authShortlivedCookie: AuthShortlivedCookieService,
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.clientsApiUrl = this.apiBaseUrl + this.clientsApiUrl;
    this.clientsChangeStatusApiUrl =
      this.apiBaseUrl + this.clientsChangeStatusApiUrl;
    this.clientsgatewayApiUrl = this.apiBaseUrl + this.clientsgatewayApiUrl;
    this.clientsAccessLogsApiUrl =
      this.apiBaseUrl + this.clientsAccessLogsApiUrl;
    this.settingGlobalIpApiUrl = this.apiBaseUrl + this.settingGlobalIpApiUrl;
    this.cancelInstallerStatusUrl =
      this.apiBaseUrl + this.cancelInstallerStatusUrl;
  }

  /** GET from the server */

  getClients(
    page: number = 1,
    pageSize: number = 10,
    search: any = {},
    sort: string = 'id',
    dir: string = 'desc',
    type: string,
    adOrphand: boolean,
    localOrphand: boolean,
    all: boolean = false,
    except = []
  ): any {
    const skip = JSON.stringify(except);

    let queryString = `?type=${type}&limit=${pageSize}&offset=${page -
      1}&sort=${sort}&dir=${dir}&all=${all}&except=${skip}`;
    let multiSet = false;

    if (search) {
      multiSet = true;
      if (multiSet) {
        queryString += '&';
      }

      let hasAnyFilter = false;
      for (let filter in search) {
        if (
          search[filter] &&
          search[filter].length &&
          search[filter] !== '-1'
        ) {
          queryString += `${filter}=${search[filter]}&`;
          hasAnyFilter = true;
        } else if (search[filter] && search[filter] !== '-1') {
          queryString += `${filter}=${search[filter]}&`;
        }
      }

      if (hasAnyFilter) {
        queryString = queryString.substring(0, queryString.length - 1);
        multiSet = true;
      }
    }

    // if (multiSet) {
    //   queryString += '&';
    // }
    //queryString += '&adOrphand=' + adOrphand;
    //queryString += '&localOrphand=' + localOrphand;

    return this.authHttp
      .get(this.clientsApiUrl + queryString)
      .pipe(map((response: Response) => response));
  }

  add_client(
    client: Client,
    clientaccessrestrictions: ClientAccessRestriction,
    ips: Array<TrustedHost>,
    type: string,
    isClone: boolean,
    parentId: number = 0 //To clone trusted hosts of client in clone scenario
  ): any {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    let queryString = '';

    if (client.id > 0 && typeof client.id != 'undefined' && isClone === false) {
      return this.authHttp
        .put(
          `${this.clientsApiUrl}/${type}/${client.id}`,
          {
            client: client,
            clientaccessrestrictions: clientaccessrestrictions
          },
          options
        )
        .pipe(map(this.extractData));
      //.catch(this.handleError);
    } else {
      if (isClone) {
        delete client.id;
        delete clientaccessrestrictions.id;
      }
      return this.authHttp
        .post(
          `${this.clientsApiUrl}/${type}`,
          {
            client: client,
            clientaccessrestrictions: clientaccessrestrictions,
            gips: ips,
            parentId,
            isClone
          },
          options
        )
        .pipe(map((response: Response) => response));
    }
  }

  check_rule(client: Client, Ip, direction, port, protocol): any {
    let queryString =
      '?clientId=' +
      client.id +
      '&ip=' +
      Ip +
      '&direction=' +
      direction +
      '&port=' +
      port +
      '&protocol=' +
      protocol;
    return this.authHttp
      .get(this.settingGlobalIpApiUrl + queryString)
      .pipe(map((response: Response) => response));
  }

  change_status(client: Client): Observable<any> {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    return this.authHttp
      .put(this.clientsChangeStatusApiUrl, { client: client }, options)
      .pipe(map((response: Response) => response));
  }

  approve(client: Client): Observable<any> {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    return this.authHttp
      .put(this.apiBaseUrl + '/clients/approve', { client: client }, options)
      .pipe(map((response: Response) => response));
  }

  cancelInstallerStatus(client: Client) {
    return this.authHttp
      .put(this.cancelInstallerStatusUrl, {
        client: client
      })
      .pipe(map(response => response));
  }

  deleteBulkClients(
    selectAll,
    selectedIds,
    client_type,
    filters,
    search,
    event
  ) {
    this.httpOptions['body'] = {
      selectAll,
      selectedIds: Array.from(selectedIds),
      verificationPassword: event,
      client_type,
      filters: JSON.stringify(filters),
      search,
      type: 'deleteClients'
    };
    return this.authHttp.delete(
      `${this.clientsApiUrl}/deleteBulkClients`,
      this.httpOptions
    );
  }

  getQuickClients(
    allSelected: boolean = false,
    except: number[],
    clientType: string,
    os: string[] = [],
    search: string = '',
    status: number[] = [],
    is_online: string,
    is_active: string
  ) {
    return this.authHttp
      .post(`${this.clientsApiUrl}/quick`, {
        allSelected,
        except,
        clientType,
        os,
        search,
        status,
        quickFilters: {
          is_online,
          is_active
        }
      })
      .pipe(map((response: Response) => response));
  }

  bulkActions(
    selectAll,
    selectedIds,
    client_type,
    type,
    filters,
    search,
    status = null
  ) {
    return this.authHttp.post(`${this.clientsApiUrl}/bulkActions`, {
      selectAll,
      selectedIds: Array.from(selectedIds),
      client_type,
      status: status,
      type,
      filters: JSON.stringify(filters),
      search
    })
    .pipe(map((response: Response) => response));
  }

  delete_client(id: number, type: string, user: string): any {
    var queryString = '';

    if (id > 0) {
      queryString = `/${type}/${id}/${user}`;
    }
    return this.authHttp
      .delete(this.clientsApiUrl + queryString)
      .pipe(map((response: Response) => response));
  }

  generateOTPToken(client) {
    return this.authHttp
      .post(`${this.clientsApiUrl}/generateLinOTPToken/${client.id}`, {
        client
      })
      .pipe(map(response => response));
  }

  generateOTP(device_id) {
    return this.authHttp
      .put(`${this.clientsApiUrl}/generateOTPToken/${device_id}`, {})
      .pipe(map(response => response));
  }

  generate_client(id: number, isDownload = 0): any {
    var queryString = '';

    if (id > 0) {
      queryString = '/' + id;
    }
    if (isDownload == 1) {
      queryString += '/' + 1;
    } else {
      queryString += '/' + 0;
    }
    return this.authHttp
      .get(`${this.clientsApiUrl}/generate/service${queryString}`)
      .pipe(map(response => response));
  }

  sendLink(
    ids: number,
    allSelected: false,
    except: number[],
    clientType: string,
    os: string[] = [],
    search: string = '',
    status: number[] = [],
    is_online: string,
    is_active: string
  ): any {
    return this.authHttp
      .post(`${this.clientsApiUrl}/sendLink`, {
        ids,
        allSelected,
        except,
        clientType,
        os,
        search,
        status,
        quickFilters: {
          is_online,
          is_active
        }
      })
      .pipe(map(response => response));
  }

  deregMFA(client: any): any {
    return this.authHttp
      .post(`${this.clientsApiUrl}/deregMFA`, { id: client.id })
      .pipe(map(response => response));
  }

  sendOTPQrCode(client: any): any {
    return this.authHttp
      .post(`${this.clientsApiUrl}/sendOTPQRCode`, {
        email: client.Email,
        user: client.user,
        name: client.Name
      })
      .pipe(map(response => response));
  }

  sendOTPQr(device_id, user_id): any {
    return this.authHttp
      .post(`${this.clientsApiUrl}/send/OTP_QR`, {
        device_id,
        user_id
      })
      .pipe(map(response => response));
  }

  getclientdetail(id, type: string): Observable<Client | any> {
    var queryString = '';

    if (id != '') {
      queryString = `/${type}/${id}`;
    }
    return this.authHttp
      .get(this.clientsApiUrl + queryString)
      .pipe(map(response => response));
  }

  getClientStatus(id): Observable<Client | any> {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    return this.authHttp
      .post(this.clientsApiUrl + '/getInstallerStatus/', { id: id }, options)
      .pipe(map(response => response));
  }

  unlock_client(id): Observable<any> {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    return this.authHttp
      .post(this.clientsApiUrl + '/unlockClient/', { id: id }, options)
      .pipe(map(response => response));
  }

  getDate(date: Object): any {
    var newDate = null;
    if (date) {
      newDate = moment(date).format('YYYY-MM-DD');
    }
    return newDate;
  }

  getClientAccessLogs(
    page: number = 1,
    pageSize: number = 10,
    logtype: any = null,
    clientid: any = null,
    clientos: any = null,
    start_date: Object = null,
    end_date: Object = null,
    sort: string = 'id',
    dir: string = 'desc',
    search_key: String = null
  ): any {
    let queryString = `?limit=${pageSize}&offset=${page - 1}&`;
    let multiSet = false;
    var start_date_unix = this.getDate(start_date);
    var end_date_unix = this.getDate(end_date);

    if (start_date_unix && end_date_unix) {
      queryString +=
        'start_date=' + start_date_unix + '&end_date=' + end_date_unix;
      multiSet = true;
    }

    if (search_key != null && search_key != 'null') {
      if (multiSet) {
        queryString += '&';
      }
      queryString += 'searchKey=' + search_key;
      multiSet = true;
    }

    if (logtype != null && logtype != 'null') {
      if (multiSet) {
        queryString += '&';
      }
      queryString += 'logtype=' + logtype;
      multiSet = true;
    }

    if (clientid != null && clientid != 'null') {
      if (multiSet) {
        queryString += '&';
      }
      queryString += 'clientid=' + clientid;
      multiSet = true;
    }

    if (clientos != null && clientos != 'null') {
      if (multiSet) {
        queryString += '&';
      }
      queryString += 'clientos=' + clientos;
      multiSet = true;
    }

    if (sort != null && sort != 'null') {
      if (multiSet) {
        queryString += '&';
      }
      queryString += 'sort=' + sort + '&dir=' + dir;
      multiSet = true;
    }

    return this.authHttp
      .get(this.clientsAccessLogsApiUrl + queryString)
      .pipe(map(response => response));
  }

  purgeClientAccessLogs(): any {
    return this.authHttp
      .delete(this.clientsAccessLogsApiUrl + '/purge')
      .pipe(map(response => response));
    // .catch(this.handleError);
  }

  updateClientAutoUpdates(agentId: number, auto_updates: boolean): any {
    return this.authHttp
      .put(this.clientsApiUrl + '/updates/' + agentId, { auto_updates })
      .pipe(map(response => response));
    // .catch(this.handleError);
  }

  /*getservicedetail(id): Observable<Service> {
    var queryString = "";

    if (id!="") {
      queryString = "/"+id;
    }
    return this.authHttp.get(this.clientsserviceApiUrl+queryString).map((response: Response) => response);
  }*/
  private extractData(res: Response) {
    let body = res;
    return body || {};
  }

  // private handleError(error: any) {
  //   // In a real world app, we might use a remote logging infrastructure
  //   let errMsg: string;
  //   if (error) {
  //     const body = error || '';
  //     const err = body.error || JSON.stringify(body);
  //     errMsg = `${error.status} - ${error.statusText || ''} ${err}`;
  //   } else {
  //     errMsg = error.message ? error.message : error.toString();
  //   }
  //   return Observable.throw(errMsg);
  // }

  exportCSV(
    logtype: any = null,
    clientid: any = null,
    clientos: any = null,
    start_date: Object = null,
    end_date: Object = null,
    search_key: String = null,
    portalVersionInfo
  ): any {
    let queryString = '?';
    let multiSet = false;
    var start_date_unix = this.getDate(start_date);
    var end_date_unix = this.getDate(end_date);

    if(portalVersionInfo){
      queryString += `version=${portalVersionInfo}`
    }

    if (start_date_unix && end_date_unix) {
      queryString +=
        'start_date=' + start_date_unix + '&end_date=' + end_date_unix;
      multiSet = true;
    }

    if (search_key != null && search_key != 'null') {
      if (multiSet) {
        queryString += '&';
      }
      queryString += 'searchKey=' + search_key;
      multiSet = true;
    }

    if (logtype != null && logtype != 'null') {
      if (multiSet) {
        queryString += '&';
      }
      queryString += 'logtype=' + logtype;
      multiSet = true;
    }

    if (clientid != null && clientid != 'null') {
      if (multiSet) {
        queryString += '&';
      }
      queryString += 'clientid=' + clientid;
      multiSet = true;
    }

    if (clientos != null && clientos != 'null') {
      if (multiSet) {
        queryString += '&';
      }
      queryString += 'clientos=' + clientos;
      multiSet = true;
    }

    this.authShortlivedCookie.setFor(5);
    window.location.href = `${this.clientsAccessLogsApiUrl}/export-csv${queryString}`;
  }

  getClientGraph(clientId) {
    return this.authHttp
      .post(`${this.clientsApiUrl}/graph`, {
        clientId
      })
      .pipe(map(response => response));

  }

  bulkAutoUpdate(clientIds, state) {
    return this.authHttp
      .put(`${this.clientsApiUrl}/bulkUpdate`, { clientIds, state })
      .pipe(map(response => response));

  }

  getClientId(name){
    return this.authHttp
      .get(`${this.clientsApiUrl}/getClientId/${name}`)
      .pipe(map(response => response));
  }

  

  setClientType(clientType: string) {
    this.clientTypeSource.next(clientType);
  }

  getClientServices(id): Observable<Client | any> {
    const apiUrl = `${this.clientsAccessLogsApiUrl}/clientServices/${id}`;

    return this.authHttp
      .get(apiUrl)
      .pipe(map(response => response));
  } 
}
