import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { ClientService } from '../client.service';
import { Client } from '../../models/client';
import { ClientAccessRestriction } from '../../models/clientaccessrestriction';
import { ManageService } from '../../manage-service/manage.service';
import { Service } from '../../models/service';
import { DragulaService } from 'ng2-dragula';
import { Pagination } from '../../models/pagination';
import { PreserveSearch } from '../../models/preserve-search';
import { LdapUser } from '../../models/ldapuser';
import { forkJoin, fromEvent, Observable, of } from 'rxjs';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { TrustedHost } from '../../models/trusted-host';
import { AuthenticationService } from '../../login/authentication.service';
import { LicenseService } from '../../license/license.service';
import { BehaviorSubject, Subject } from 'rxjs';
import { debounceTime, map, switchMap, tap } from 'rxjs/operators';
import dragula, { Drake } from 'dragula';
import { ErrorCodeEnum } from '../../shared/emuns/error.enum';
import { ContextHelpEnum } from '../../shared/emuns/global.enum';
import { LicenseErrorEnum } from '../../shared/emuns/license.enum';
import { ClientStatusEnum, ClientTypeEnum, DeviceOSEnum } from '../../shared/emuns/client.enum';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-client',
  templateUrl: './create-client.component.html',
  styleUrls: ['./create-client.component.css']
})
export class CreateClientComponent implements OnInit {
  public alert: GlobalMessageClass;
  public malert: GlobalMessageClass;
  public hide_loader = true;
  public user_has_error = false;
  public user_error =
    'Either Host is down or you may have entered a wrong host in your LDAP settings.';
  public tid: number = null;
  changingMade: boolean = false;
  isEdit = false;
  client = new Client();
  clientDeviceOs;
  refclient = new Client();
  public removeServiceList: Service[] = [];
  public addServiceList: Service[] = [];
  public creatingClient: boolean = false;
  hideDarkening: boolean = false;
  hideWhiteListing: boolean = false;
  clientaccessrestrictions = new ClientAccessRestriction();
  license: any;
  selected_months = new Array();
  selected_weekdays = new Array();
  isMfaDisbaled: boolean = false;
  isClone: boolean = false;
  disableAddHostButton: boolean = false;
  paramClientId;
  //isEnableBiometric;

  ClientStatus = ClientStatusEnum;
  ClientType = ClientTypeEnum;
  DeviceOS = DeviceOSEnum;

  searchTerm$ = new Subject();
  searchTerm;

  public clientType;
  public clientTypePlural;

  loading: Boolean = false;

  clientAv = [
    { display: 'No', value: 'no' },
    { display: 'Any', value: 'any' },
    { display: 'Avast', value: 'avast' },
    { display: 'Avira', value: 'avira' },
    { display: 'Bitdefender', value: 'bitdefender' },
    { display: 'ESET', value: 'eset' },
    { display: 'Kaspersky', value: 'kaspersky' },
    { display: 'McAfee', value: 'mcafee' },
    { display: 'Norton', value: 'norton' },
    { display: 'Sophos', value: 'sophos' },
    { display: 'Total AV', value: 'total' },
    { display: 'Trend Micro', value: 'trend' },
    { display: 'Webroot', value: 'webroot' },
    { display: 'Windows Defender', value: 'defender' }
  ];

  windowsOsVersion = [
    { value: '6.1.7601', display: 'Windows 7' },
    { value: '6.2.9200', display: 'Windows 8' },
    { value: '6.3.9600', display: 'Windows 8.1' },
    { value: '10.0.18363', display: 'Windows 10' },
    { value: '10.0.22000', display: 'Windows 11' }
  ];

  macOsVersions = [
    { value: '10.15.4', display: 'macOS Catalina' },
    { value: '10.14.6', display: 'macOS Mojave' },
    { value: '10.13.6', display: 'macOS High Sierra' }
  ];

  ubuntu18OsVersions = [
    { value: '18.04.4', display: '' },
    { value: '18.04.3', display: '' },
    { value: '18.04.2', display: '' },
    { value: '18.04.1', display: '' },
    { value: '18.04', display: '' }
  ];

  ubuntu16OsVersions = [
    { value: '16.04.6', display: '' },
    { value: '16.04.5', display: '' },
    { value: '16.04.4', display: '' },
    { value: '16.04.3', display: '' },
    { value: '16.04.2', display: '' },
    { value: '16.04.1', display: '' },
    { value: '16.04', display: '' }
  ];

  ubuntu20OsVersions = [
    { value: '20.04.0', display: '' },
    { value: '20.04.1', display: '' },
    { value: '20.04.2', display: '' },
    { value: '20.10.0', display: '' }
  ];

  centosserv7OsVersions = [
    { value: '7.7', display: '' },
    { value: '7.6', display: '' },
    { value: '7.5', display: '' },
    { value: '7.4', display: '' },
    { value: '7.3', display: '' },
    { value: '7.2', display: '' },
    { value: '7.1', display: '' },
    { value: '7.0', display: '' }
  ];

  centosserv8OsVersions = [
    { value: '8.3', display: '' },
    { value: '8.2', display: '' },
    { value: '8.1', display: '' },
    { value: '8.0', display: '' }
  ];

  oss = [
    { value: DeviceOSEnum.Ubuntu18, display: 'Ubuntu Desktop', type: 'client' },
    { value: DeviceOSEnum.UbuntuServer18, display: 'Ubuntu Server', type: 'server' },
    {
      value: DeviceOSEnum.UbuntuServer20armv8,
      display: 'Ubuntu Server (ARMv8)',
      type: 'server'
    },
    { value: DeviceOSEnum.Windows, display: 'Windows', type: 'client' },
    { value: DeviceOSEnum.WindowsServer, display: 'Windows Server', type: 'server' },
    { value: DeviceOSEnum.Macintosh, display: 'macOS', type: 'client' },
    { value: DeviceOSEnum.IOS, display: 'iOS', type: 'client' },
    { value: DeviceOSEnum.Android, display: 'Android', type: 'client' },
    { value: DeviceOSEnum.CentosServer, display: 'CentOS Server', type: 'server' }
  ];

  clientOsVersion = [];

  // Updated via child component
  private localModeHosts: TrustedHost[] = [];

  public radio_button_options = [
    { value: true, display: 'Yes' },
    { value: false, display: 'No' }
  ];

  //Contextual Help
  public contextualHelp: Array<any>;
  public showContext: boolean = false;
  //Contextual Help

  public hours = [
    '00',
    '01',
    '02',
    '03',
    '04',
    '05',
    '06',
    '07',
    '08',
    '09',
    '10',
    '11',
    '12',
    '13',
    '14',
    '15',
    '16',
    '17',
    '18',
    '19',
    '20',
    '21',
    '22',
    '23'
  ];
  public minutes = [
    '00',
    '01',
    '02',
    '03',
    '04',
    '05',
    '06',
    '07',
    '08',
    '09',
    '10',
    '11',
    '12',
    '13',
    '14',
    '15',
    '16',
    '17',
    '18',
    '19',
    '20',
    '21',
    '22',
    '23',
    '24',
    '25',
    '26',
    '27',
    '28',
    '29',
    '30',
    '31',
    '32',
    '33',
    '34',
    '35',
    '36',
    '37',
    '38',
    '39',
    '40',
    '41',
    '42',
    '43',
    '44',
    '45',
    '46',
    '47',
    '48',
    '49',
    '50',
    '51',
    '52',
    '53',
    '54',
    '55',
    '56',
    '57',
    '58',
    '59'
  ];

  public levels = [
    { value: 'Secret', display: 'Secret' },
    { value: 'TopSecret', display: 'Top Secret' },
    { value: 'Confidential', display: 'Confidential' },
    { value: 'PublicTrust', display: 'Public Trust' }
  ];

  public access_from = [
    { value: 'Secret', display: 'Secret' },
    { value: 'TopSecret', display: 'Top Secret' },
    { value: 'Confidential', display: 'Confidential' },
    { value: 'PublicTrust', display: 'Public Trust' }
  ];

  public months = [
    { value: 'All', display: 'All', isChecked: false },
    { value: 'Jan', display: 'Jan', isChecked: false },
    { value: 'Feb', display: 'Feb', isChecked: false },
    { value: 'Mar', display: 'Mar', isChecked: false },
    { value: 'Apr', display: 'Apr', isChecked: false },
    { value: 'May', display: 'May', isChecked: false },
    { value: 'Jun', display: 'Jun', isChecked: false },
    { value: 'Jul', display: 'Jul', isChecked: false },
    { value: 'Aug', display: 'Aug', isChecked: false },
    { value: 'Sep', display: 'Sep', isChecked: false },
    { value: 'Oct', display: 'Oct', isChecked: false },
    { value: 'Nov', display: 'Nov', isChecked: false },
    { value: 'Dec', display: 'Dec', isChecked: false }
  ];

  public weeks = [
    { value: 'All', display: 'All', isChecked: false },
    { value: 'Sun', display: 'Sun', isChecked: false },
    { value: 'Mon', display: 'Mon', isChecked: false },
    { value: 'Tue', display: 'Tue', isChecked: false },
    { value: 'Wed', display: 'Wed', isChecked: false },
    { value: 'Thu', display: 'Thu', isChecked: false },
    { value: 'Fri', display: 'Fri', isChecked: false },
    { value: 'Sat', display: 'Sat', isChecked: false }
  ];

  deviceOsSelected(fromTempalate = false): void {
    const { device_os } = this.client;
    if (device_os == DeviceOSEnum.Android || device_os == DeviceOSEnum.IOS) {
      this.hideDarkening = true;
      this.client.client_darkening = false;
      this.hideWhiteListing = true;
    } else {
      if (device_os === DeviceOSEnum.Windows) {
        this.clientOsVersion = this.windowsOsVersion;
      } else if (device_os === DeviceOSEnum.Macintosh) {
        this.clientOsVersion = this.macOsVersions;
      } else if (device_os === DeviceOSEnum.CentosServer) {
        this.clientOsVersion = this.centosserv7OsVersions;
      } else {
        if (device_os === DeviceOSEnum.Ubuntu || device_os === DeviceOSEnum.UbuntuServer) {
          this.clientOsVersion = this.ubuntu18OsVersions;
        } else {
          this.clientOsVersion = this.ubuntu20OsVersions;
        }
      }
      if (fromTempalate) {
        this.client.client_os_version = 'any';
      }
      this.hideDarkening = false;
      this.hideWhiteListing = false;
    }
  }

  selectedUser: string = '';
  users = new Array();
  services: Service[];
  private servicesList = new Array();
  private selectedService: Service;
  pagination: Pagination = new Pagination();
  private Search: PreserveSearch = new PreserveSearch();
  public ldapusers: LdapUser[];
  public ldapNextPage = null;
  public assignedServices = {};
  public fetchedservices = [];
  oldservices;
  public pageTitle: string;
  public clientTypeLabel: string;

  private clientToBeClonedId = 0;
  public clientTypeRouterLink = "";
  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private clientService: ClientService,
    private dragulaService: DragulaService,
    private manageService: ManageService,
    private globalMessageService: GlobalMessageService,
    private authenticationService: AuthenticationService,
    private licenseService: LicenseService,
    private titleService: Title
  ) {
    this.route.params.subscribe(params => {
      this.clientType = params.type;
      this.clientTypeLabel =
        params.type === ClientTypeEnum.Server ? 'Server Client' : ClientTypeEnum.Client;

      if(params.type === ClientTypeEnum.Client){
        this.clientTypePlural = 'End User Clients'
        this.clientTypeRouterLink = '/clients/Client'
      }else{
        this.clientTypePlural = 'Server Side Clients'
        this.clientTypeRouterLink = '/clients/Server'
      }

      if (params.type === ClientTypeEnum.Server){
        this.clientService.setClientType('server');
      } else {
        this.clientService.setClientType('client');
      }
      this.pageTitle = !params.id
        ? `Add ${this.clientTypeLabel}`
        : params.clone
        ? `Clone ${this.clientTypeLabel}`
        : `View ${this.clientTypeLabel}`;

      if (params.type === ClientTypeEnum.Server) {
        this.titleService.setTitle('Client Management | Server Side Client | ' + this.pageTitle);
      } else {
        this.titleService.setTitle('Client Management | End User Client | ' + this.pageTitle);
      }

      this.oss =
        params.type == ClientTypeEnum.Server
          ? this.oss.filter(os => os.type == 'server')
          : this.oss.filter(
              os =>
                os.value != DeviceOSEnum.WindowsServer && os.value != DeviceOSEnum.UbuntuServer20armv8
            );
      this.client.device_os = this.oss[0].value;
    });

    this.contextualHelp = ['', ''];
    this.malert = {
      closed: true,
      type: 'danger',
      message: 'Time must be valid in access restriction.'
    };
    const bag: any = this.dragulaService.find('service-bag');
    if (bag !== undefined) this.dragulaService.destroy('service-bag');

    const containers = [
      document.getElementById('container-1'),
      document.getElementById('container-2'),
      document.getElementById('container-3')
    ];
    const drake: Drake = dragula(containers);

    drake.on('drop', (el, target, source) => {
      const items = Array.from(target.children).slice(1);
      this.onDrop(items);
    });
  }

  showContextHelp(name: any): void {
    if (name == 'selectedUser') {
      this.contextualHelp = ['selectedUser', 'Active Directory user ID.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.name) {
      this.contextualHelp = ['name', 'Username fetched from Active Directory.'];
      this.showContext = true;
    } else if (name == 'Email') {
      this.contextualHelp = [
        'Email',
        'Client package will be sent to this email address.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.device_os) {
      this.contextualHelp = ['device_os', 'OS of the client device.'];
      this.showContext = true;
    } else if (name == 'device_hardware_id') {
      this.contextualHelp = [
        'device_hardware_id',
        'Unique ID of the machine the client is bound to. This will be updated when client is installed.'
      ];
      this.showContext = true;
    } else if (name == 'device_av') {
      this.contextualHelp = [
        'device_av',
        'Endpoint security software installed on client machine'
      ];
      this.showContext = true;
    } else if (name == 'device_os_version') {
      this.contextualHelp = [
        'device_os_version',
        'Version of OS installed in client machine'
      ];
      this.showContext = true;
    }
  }

  closeContextualHelp(): void {
    this.showContext = false;
  }

  updateHosts(hosts) {
    this.localModeHosts = hosts;
  }

  private onDrop(args) {
    let [el, target, source, sibling] = args;

    if (source !== target) {
      this.changingMade = true;
      if (target.id == 'assigned-service') {
        this.assignedServices[el.id] = el.id;
        let index = this.services.findIndex(service => service.id == el.id);
        let findex = this.fetchedservices.findIndex(fetchedservice =>
          index > -1 ? fetchedservice.id == this.services[index].id : false
        );
        if (findex === -1) {
          this.fetchedservices.push(this.services[index]);
        }
        this.services.splice(index, 1);
      }

      if (target.id == 'available-service') {
        delete this.assignedServices[el.id];
        let index = this.fetchedservices.findIndex(
          fetchedservice => fetchedservice.id == el.id
        );
        let sindex = this.services.findIndex(service =>
          index > -1 ? service.id == this.fetchedservices[index].id : false
        );
        if (sindex === -1) {
          this.services.push(this.fetchedservices[index]);
        }
        this.fetchedservices.splice(index, 1);
      }

      let indexForCheckList = this.removeServiceList.findIndex(
        cs => cs.id == el.id
      );
      this.removeServiceList.splice(indexForCheckList, 1);
      indexForCheckList = this.addServiceList.findIndex(cs => cs.id == el.id);
      this.addServiceList.splice(indexForCheckList, 1);
    }
  }

  removeService(ev, service: Service) {
    if (ev.target.checked) {
      let index = this.removeServiceList.findIndex(
        removeServiceIndex => removeServiceIndex.id == service.id
      );
      if (index === -1) {
        this.removeServiceList.push(service);
      }
    } else {
      let index = this.removeServiceList.findIndex(
        removeServiceIndex => removeServiceIndex.id == service.id
      );
      if (index != -1) {
        this.removeServiceList.splice(index, 1);
      }
    }

    if (this.removeServiceList.length == 0) {
      this.removeServiceList = [];
    }
  }

  addService(ev, service: Service) {
    if (ev.target.checked) {
      let index = this.addServiceList.findIndex(
        addServiceIndex => addServiceIndex.id == service.id
      );
      if (index === -1) {
        this.addServiceList.push(service);
      }
    } else {
      let index = this.addServiceList.findIndex(
        addServiceIndex => addServiceIndex.id == service.id
      );
      if (index != -1) {
        this.addServiceList.splice(index, 1);
      }
    }
    if (this.addServiceList.length == 0) {
      this.addServiceList = [];
    }
  }

  addSelectedIndex() {
    this.changingMade = true;
    for (var i in this.addServiceList) {
      this.assignedServices[this.addServiceList[i].id] = this.addServiceList[
        i
      ].id;

      let index = this.services.findIndex(
        service => service.id == this.addServiceList[i].id
      );

      let findex = this.fetchedservices.findIndex(
        fetchedservice => fetchedservice.id == this.services[index].id
      );
      if (findex === -1) {
        this.fetchedservices.push(this.services[index]);
      }
      this.services.splice(index, 1);
    }
    this.addServiceList = [];
  }

  removeSelectedIndex() {
    this.changingMade = true;
    for (var i in this.removeServiceList) {
      let obj: Service = new Service();
      obj.id = this.assignedServices[this.removeServiceList[i].id];
      delete this.assignedServices[obj.id];

      let index = this.fetchedservices.findIndex(
        fetchedservice => fetchedservice.id == this.removeServiceList[i].id
      );
      let sindex = this.services.findIndex(
        service => service.id == this.fetchedservices[index].id
      );
      if (sindex === -1) {
        this.services.push(this.fetchedservices[index]);
      }
      this.fetchedservices.splice(index, 1);
    }
    this.removeServiceList = [];
  }

  selectAll() {
    this.changingMade = true;
    for (var i in this.services) {
      this.assignedServices[this.services[i].id] =
        '' + this.services[i].id + '';
    }
    for (var j = 0; j < this.services.length; j++) {
      this.fetchedservices.push(this.services[j]);
    }
    this.services = [];
  }

  deSelectAll() {
    this.changingMade = true;
    for (var i in this.assignedServices) {
      let obj: Service = new Service();
      obj.id = this.assignedServices[this.assignedServices[i]];
      let index = this.fetchedservices.findIndex(
        fetchedService => fetchedService.id == obj.id
      );
      if (index != -1) {
        this.services.push(this.fetchedservices[index]);
      }
      delete this.assignedServices[this.assignedServices[i]];
    }
    this.fetchedservices = [];
  }

  getPage(page: number) {
    this.loading = true;
    let ref = this;
    this.pagination.currentPage = page;
    this.manageService
      .getAllAssociatedServices()
      .pipe(
        tap((res: any) => {
          this.services = res.services;
          this.pagination.totalItems = res.total;
          this.pagination.currentPage = page;
          this.licenseService.getLicense().subscribe(response => {
            this.license = response.license;
            this.isMfaDisbaled =
              this.license.mfa_feature == null
                ? true
                : !this.license.mfa_feature;
            this.paramClientId = this.route.params['_value']['id'];

            if (typeof this.paramClientId != 'undefined') {
              //this.pageTitle = 'Edit Client';
              this.clientService
                .getclientdetail(this.paramClientId, this.clientType)
                .subscribe(client => {
                  this.loading = false;
                  if (!client) {
                    this.alert = {
                      closed: false,
                      type: 'danger',
                      message: 'No client found with the specified ID.'
                    };
                    this.globalMessageService.pushAlert(this.alert);
                    this.router.navigate(['/clients', this.clientType]);
                  }
                  this.isEdit = true;
                  this.client = client;
                  this.clientDeviceOs = this.client.device_os;
                  if (
                    !this.client.client_os_version &&
                    !this.client.client_av
                  ) {
                    this.client.client_av = 'no';
                    this.client.client_os_version = 'any';
                  }

                  this.deviceOsSelected();
                  this.tid = this.client.id;
                  this.refclient = Object.assign({}, this.client);
                  /*if (this.client.ClientAccessRestrictions.length > 0) {
                  this.clientaccessrestrictions = this.client.ClientAccessRestrictions[0];

                  this.selected_months = this.clientaccessrestrictions.months.split(
                    ','
                  );
                  if (this.clientaccessrestrictions.months == '') {
                    this.selected_months = [];
                  }
                  this.selected_weekdays = this.clientaccessrestrictions.week_days.split(
                    ','
                  );
                  if (this.clientaccessrestrictions.week_days == '') {
                    this.selected_weekdays = [];
                  }
                }*/
                  this.selectedUser = this.client.user;
                  // this.fetchedservices = client.Services;
                  // this.oldservices = client.Services;
                  // this.fetchedservices.forEach(function(fetchedservice, index) {
                  //   ref.assignedServices[fetchedservice.id] = fetchedservice.id;
                  //   for (var i = 0; i < ref.services.length; i++) {
                  //     if (ref.services[i]['id'] == fetchedservice.id) {
                  //       ref.services.splice(i, 1);
                  //     }
                  //   }
                  // });
                });
            } else {
              //this.isEnableBiometric = false;
              // this.client.access_restrictions = false;
              // this.client.device_os = '';
              this.loading = false;
              this.client.geoip_track = true;
              this.client.auto_updates = true;
              this.client.client_darkening = false;
              this.client.device_hardware_id = '0'.repeat(32);
              this.client.client_av = 'no';
              this.client.client_os_version = 'any';
            }
          });
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          this.loading = false;
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              // this.alert = {
              //   closed: false,
              //   type: 'danger',
              //   message: 'You are not authorized to access this page.'
              // };
              // this.globalMessageService.pushAlert(this.alert);
            }
          }
        },
        complete: () => {}
      });
  }

  makeRandomId(length: number) {
    var text = '';
    var possible =
      'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    for (var i = 0; i < length; i++) {
      text += possible.charAt(Math.floor(Math.random() * possible.length));
    }
    return text;
  }

  getLdapUsers(term = null, ldapNextPage = null) {
    return this.manageService.getLdapUsers(term, ldapNextPage);
  }

  getLocalUsers(term) {
    return this.manageService.getLocalUsers(term);
  }

  searchMore() {
    if (this.ldapNextPage && this.ldapNextPage.cookie) {
      this.hide_loader = false;
      this.getLdapUsers(this.searchTerm, this.ldapNextPage).subscribe({
        next: (res: any) => {
          if (res.entries.length > 0) {
            this.ldapusers = [...this.ldapusers, ...res.entries];
            this.ldapNextPage = res.nextPage;
          }
          this.hide_loader = true;
        }
      });
    }
  }

  setupSearch() {
    this.searchTerm$
      .pipe(
        tap(() => {
          this.hide_loader = false;
        }),
        debounceTime(1000),
        switchMap((term: any) => {
          this.searchTerm = term;

          if (term.length) {
            return forkJoin(this.getLdapUsers(term), this.getLocalUsers(term));
          } else {
            return of([{ entries: [] }, []]);
          }
        })
      )
      .subscribe({
        next: (res: any) => {
          if (res[0].entries && res[1]) {
            const ldapUsers = res[0].entries.map(client => {
              return {
                ...client,
                isLdapClient: true
              };
            });
            const localUsers = res[1].map(localClient => {
              return {
                uid: localClient.userName,
                mail: localClient.email,
                cn: localClient.name,
                isLdapClient: false
              };
            });
            this.ldapusers = [...ldapUsers, ...localUsers];
            this.ldapNextPage = res.nextPage;
            this.hide_loader = true;
          }
        },
        error: err => {
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            }
          } else {
            this.user_error =
              'Either Host is down or you may have entered a wrong host in your LDAP settings.';
            this.user_has_error = true;
          }
          this.hide_loader = true;
        }
      });
  }

  ngOnInit() {
    this.route.params.subscribe(params => {
      if (params.clone) {
        this.isClone = true;
        this.clientToBeClonedId = this.route.snapshot.params.id;
      }
    });

    let serviceSearch = this.Search.getSearch('serviceSearch');
    if (serviceSearch) {
      this.pagination.currentPage = serviceSearch.current_page;
    }
    this.getPage(this.pagination.currentPage);
    this.setupSearch();
  }

  navigate(pageTitle:string){}
  onsubmit() {
    this.loading = true;
    this.creatingClient = true;
    // if (this.client.access_restrictions) {
    //   let fh = this.clientaccessrestrictions.access_time_from_hour;
    //   let th = this.clientaccessrestrictions.access_time_to_hour;
    //   let dif = parseInt(th) - parseInt(fh);
    //   if (dif < 1) {
    //     this.alert = {
    //       closed: false,
    //       type: "error",
    //       message: "Time must be valid in access restriction."
    //     };
    //     this.globalMessageService.pushAlert(this.alert);
    //     return 0;
    //   }
    //   if (dif <= 0) {
    //     let fm = this.clientaccessrestrictions.access_time_from_minute;
    //     let tm = this.clientaccessrestrictions.access_time_to_minute;
    //     let difm = parseInt(tm) - parseInt(fm);
    //     if (difm < 1) {
    //       this.malert = {
    //         closed: false,
    //         type: 'danger',
    //         message: 'Time must be valid in access restriction.'
    //       };
    //       this.creatingClient = false;
    //       // this.globalMessageService.pushAlert(this.malert);
    //       return 0;
    //     }
    //   }
    //   if (
    //     this.selected_weekdays.length < 1 &&
    //     this.selected_months.length < 1
    //   ) {
    //     this.malert = {
    //       closed: false,
    //       type: 'danger',
    //       message: 'Weekdays and Months are required in access restriction.'
    //     };
    //     this.creatingClient = false;
    //     // this.globalMessageService.pushAlert(this.malert);
    //     return 0;
    //   } else if (this.selected_weekdays.length < 1) {
    //     this.malert = {
    //       closed: false,
    //       type: 'danger',
    //       message: 'Weekdays are required in access restriction.'
    //     };
    //     this.creatingClient = false;
    //     // this.globalMessageService.pushAlert(this.malert);
    //     return;
    //   } else if (this.selected_months.length < 1) {
    //     this.malert = {
    //       closed: false,
    //       type: 'danger',
    //       message: 'Months are required in access restriction.'
    //     };
    //     this.creatingClient = false;
    //     // this.globalMessageService.pushAlert(this.malert);
    //     return 0;
    //   }
    // }
    if (this.isClone) {
      this.changingMade = true;
    }
    if (
      JSON.stringify(this.client) !== JSON.stringify(this.refclient) ||
      this.changingMade
      // || JSON.stringify(this.ips) !== JSON.stringify(this.refips)
    ) {
      //this.clientaccessrestrictions.months = this.selected_months.join();
      //this.clientaccessrestrictions.week_days = this.selected_weekdays.join();

      this.client.user = this.selectedUser;
      this.client.services = this.assignedServices;
      if (
        this.client.device_os === DeviceOSEnum.Android ||
        this.client.device_os === DeviceOSEnum.IOS
      ) {
        this.client.client_av = '';
        this.client.client_os_version = '';
      }
      if (this.isClone) {
        var prop = [
          'user',
          'Email',
          'services',
          'Name',
          'is_local',
          'client_os_version',
          'client_av',
          'installer_status',
          'access_restrictions',
          'device_os',
          'geoip_track',
          'client_darkening',
          'client_mfa',
          'device_hardware_id',
          'biometric_state',
          'client_epp'
        ];
        for (var k in this.client) {
          if (prop.indexOf(k) < 0) {
            delete this.client[k];
          }
        }
        this.client.device_hardware_id = '00000000000000000000000000000000';
      }
      delete this.client.client_os_version;
      delete this.client.client_epp;
      delete this.client.client_av;
      this.clientService
        .add_client(
          this.client,
          this.clientaccessrestrictions,
          this.localModeHosts,
          this.clientType,
          this.isClone,
          this.clientToBeClonedId
        )
        .subscribe(
          response => {
            this.loading = false;
            if (response.success) {
              let message = 'Client added successfully.';
              if (
                this.client.device_os === DeviceOSEnum.Android ||
                this.client.device_os === DeviceOSEnum.IOS
              ) {
                message =
                  'Client added/updated successfully. Adding a new mobile client would automatically send the package.';
              } else if (this.isEdit && !this.isClone) {
                message = 'Client updated successfully.';
              } else if (this.isClone) {
                message = 'Client cloned successfully.';
              }
              this.alert = {
                closed: false,
                type: 'success',
                message: message
              };
              this.globalMessageService.pushAlert(this.alert);
              this.router.navigate(['/clients', this.clientType]);
            } else {
              alert(response.message);
              this.creatingClient = false;
            }
          },
          error => {
            this.loading = false;
            this.creatingClient = false;
            // this.alert = {
            //   closed: false,
            //   type: 'danger',
            //   message: error._body || 'Something went wrong in creating client'
            // };
            // this.globalMessageService.pushAlert(this.alert);
          }
        );
    } else {
      this.loading = false;
      this.creatingClient = false;
      // this.onCancel();
    }
  }

  changeClientDarkening() {
    if (this.client.client_darkening) {
      this.client.client_darkening = false;
    } else {
      this.client.client_darkening = true;
    }
  }

  changeIPTracking() {
    if (this.client.geoip_track) {
      this.client.geoip_track = false;
    } else {
      this.client.geoip_track = true;
    }
  }

  handleAutoUpdatesChange() {
    this.client.auto_updates = !this.client.auto_updates;
    this.clientService
      .updateClientAutoUpdates(this.client.agent_id, this.client.auto_updates)
      .subscribe(response => {
        const alert = {
          closed: false,
          type: 'success',
          message: 'Client updated'
        };
        this.globalMessageService.pushAlert(alert);
      });
  }

  changeClientMfa() {
    // this.client.client_mfa = !this.client.client_mfa;
    if (this.client.client_mfa === false) {
      this.client.biometric_state = false;
      this.client.is_strong_mfa = false;
    }
  }

  changeBiometricMfa() {
    if (this.client.biometric_state === false) {
      this.client.is_strong_mfa = false;
    }
  }

  changeAccessRestrictions() {
    if (this.client.access_restrictions) {
      this.client.access_restrictions = false;
      this.malert = {
        closed: true,
        type: 'danger',
        message: 'Months are required in access restriction.'
      };
    } else {
      this.client.access_restrictions = true;
    }
  }

  updateCheckedOptions(option, event) {
    this.malert = {
      closed: true,
      type: 'danger',
      message: 'Time must be valid in access restriction.'
    };
    this.changingMade = true;
    if (option.value == 'All' && event.target.checked == true) {
      this.selected_months = new Array();
      this.months.forEach(month => {
        this.selected_months.push(month.value);
        month.isChecked = true;
      });
    } else if (option.value == 'All' && event.target.checked == false) {
      this.months.forEach(month => {
        var i = this.selected_months.indexOf(month.value);
        if (i != -1) {
          this.selected_months.splice(i, 1);
        }
        month.isChecked = false;
      });
    } else if (option.value != 'All' && event.target.checked == false) {
      var i = this.selected_months.indexOf(option.value);
      if (i != -1) {
        this.selected_months.splice(i, 1);
      }
      var i = this.selected_months.indexOf('All');
      if (i != -1) {
        this.selected_months.splice(i, 1);
      }
      this.months[0].isChecked = false;
      option.isChecked = false;
    } else if (option.value != 'All' && event.target.checked == true) {
      this.selected_months.push(option.value);
      option.isChecked = true;
    }
  }

  changedTime() {
    this.malert = {
      closed: true,
      type: 'danger',
      message: 'Time must be valid in access restriction.'
    };
    this.changingMade = true;
  }

  updateCheckedWeekends(option, event) {
    this.malert = {
      closed: true,
      type: 'danger',
      message: 'Time must be valid in access restriction.'
    };
    this.changingMade = true;
    if (option.value == 'All' && event.target.checked == true) {
      this.selected_weekdays = new Array();
      this.weeks.forEach(week => {
        this.selected_weekdays.push(week.value);
        week.isChecked = true;
      });
    } else if (option.value == 'All' && event.target.checked == false) {
      this.weeks.forEach(week => {
        var i = this.selected_weekdays.indexOf(week.value);
        if (i != -1) {
          this.selected_weekdays.splice(i, 1);
        }
        week.isChecked = false;
      });
    } else if (option.value != 'All' && event.target.checked == false) {
      var i = this.selected_weekdays.indexOf(option.value);
      if (i != -1) {
        this.selected_weekdays.splice(i, 1);
      }
      var i = this.selected_weekdays.indexOf('All');
      if (i != -1) {
        this.selected_weekdays.splice(i, 1);
      }
      this.weeks[0].isChecked = false;
      option.isChecked = false;
    } else if (option.value != 'All' && event.target.checked == true) {
      this.selected_weekdays.push(option.value);
      option.isChecked = true;
    }
  }

  onCancel(): void {
    this.router.navigate(['/clients', this.clientType]);
  }

  userSelected(event): void {
    this.client.is_local = !event.isLdapClient;
    this.changingMade = true;
    let index = 0;
    index = this.ldapusers.findIndex(x => x['uid'] == this.selectedUser);
    this.client.Name = this.ldapusers[index].cn;
    if (this.clientType === ClientTypeEnum.Client)
      this.client.Email = this.ldapusers[index].mail;
  }

}
