import { AfterViewInit, Component, OnInit, ViewChild } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { AccessPolicyService } from './access-policy.service';
import { Subject, forkJoin } from 'rxjs';
import {
  catchError,
  debounceTime,
  distinctUntilChanged,
  map,
  switchMap
} from 'rxjs/operators';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { DeviceOSEnum } from '../shared/emuns/client.enum';
import { APIDataTypeEnum } from '../shared/emuns/global.enum';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { NgForm } from '@angular/forms';
import moment from 'moment-timezone';

@Component({
  selector: 'app-create-access-policy',
  templateUrl: './create-access-policy.component.html',
  styleUrls: ['./create-access-policy.component.css']
})
export class CreateAccessPolicyComponent implements OnInit, AfterViewInit {
  policyName;
  accessChecked = false;
  spaces = 0;
  months = [
    {
      id: 1,
      isChecked: false,
      name: 'Jan'
    },
    {
      id: 2,
      isChecked: false,
      name: 'Feb'
    },
    {
      id: 3,
      isChecked: false,
      name: 'Mar'
    },
    {
      id: 4,
      isChecked: false,
      name: 'Apr'
    },
    {
      id: 5,
      isChecked: false,
      name: 'May'
    },
    {
      id: 6,
      isChecked: false,
      name: 'Jun'
    },
    {
      id: 7,
      isChecked: false,
      name: 'Jul'
    },
    {
      id: 8,
      isChecked: false,
      name: 'Aug'
    },
    {
      id: 9,
      isChecked: false,
      name: 'Sep'
    },
    {
      id: 10,
      isChecked: false,
      name: 'Oct'
    },
    {
      id: 11,
      isChecked: false,
      name: 'Nov'
    },
    {
      id: 12,
      isChecked: false,
      name: 'Dec'
    }
  ];
  weekdays = [
    {
      id: 6,
      isChecked: false,
      name: 'S'
    },
    {
      id: 0,
      isChecked: false,
      name: 'M'
    },
    {
      id: 1,
      isChecked: false,
      name: 'T'
    },
    {
      id: 2,
      isChecked: false,
      name: 'W'
    },
    {
      id: 3,
      isChecked: false,
      name: 'T'
    },
    {
      id: 4,
      isChecked: false,
      name: 'F'
    },
    {
      id: 5,
      isChecked: false,
      name: 'S'
    }
  ];

  timezones = [
    { name: 'UTC', value: 'UTC', moment: 'Etc/UCT' },
    { name: 'GMT', value: 'GMT', moment: 'Europe/London' },
    { name: 'AEDT', value: 'AEDT', moment: 'Antarctica/Macquarie' },
    { name: 'JST', value: 'JST', moment: 'Asia/Tokyo' },
    { name: 'IST', value: 'IST', moment: 'Asia/Calcutta' },
    { name: 'EET', value: 'EET', moment: 'Africa/Cairo' },
    { name: 'CAT', value: 'CAT', moment: 'Africa/Lusaka' },
    { name: 'ART', value: 'ART', moment: 'America/Argentina/Buenos_Aires' },
    { name: 'NST', value: 'NST', moment: 'Canada/Newfoundland' },
    { name: 'AST', value: 'AST', moment: 'Canada/Atlantic' },
    { name: 'CET', value: 'CET', moment: 'Europe/Amsterdam' },
    { name: 'AWST', value: 'AWST', moment: 'Australia/Perth' },
    { name: 'NZST', value: 'NZST', moment: 'Pacific/Auckland' },
    { name: 'SAST', value: 'SAST', moment: 'Africa/Johannesburg' },
    { name: 'WET', value: 'WET', moment: 'Atlantic/Canary' },
    { name: 'PST', value: 'PST', moment: 'America/Vancouver' },
    { name: 'MST', value: 'MST', moment: 'US/Mountain' },
    { name: 'CST', value: 'CST', moment: 'Mexico/General' },
    { name: 'EST', value: 'EST', moment: 'America/Cayman' },
    { name: 'HST', value: 'HST', moment: 'Pacific/Honolulu' },
    { name: 'AKST', value: 'AKST', moment: 'US/Alaska' }
  ];

  selectedTimezone: string;
  clients = [];
  services = [];
  selectedClientIds = [];
  selectedServiceIds = [];
  selectedClientTypes = [];
  selectedServiceTypes = [];
  selectedClientCount = 0;
  selectedServiceCount = 0;
  selectedDays = [];
  selectedMonths = [];
  client = [];
  clientGroup = [];
  service = [];
  serviceGroup = [];
  startDate = new Date(`14 Dec 2020 00:00:00`);
  endDate = new Date(`14 Dec 2020 23:59:00`);
  valid = true;
  saveButtonDisabled = false;
  clientOffset = 0;
  serviceOffset = 0;
  currentClientSearch = '';
  currentServiceSearch = '';
  $clientSearch = new Subject();
  $serviceSearch = new Subject();
  isClientSearching = false;
  isServiceSearching = false;
  title: string;
  isEditing = false;
  currentEditingId;
  allClientsSelected = false;
  allServicesSelected = false;
  clientsPlaceHolder = 'Search Client or Client Group';
  servicesPlaceHolder = 'Search Service or Service Group';

  users = [];
  $userSearch = new Subject();
  isUserSearching = true;
  selectedUserIds = [];
  allUsersSelected = false;
  selectedUserCount = 0;
  usersPlaceHolder = 'Search User or User Group';
  currentUserSearch = '';
  userOffset = 0;
  selectedUserTypes = [];
  user = [];
  userGroup = [];
  selectedTimezoneLabel: string;

  loading: boolean = false;
  policyType = '';
  withUser: boolean;
  disableSaveButton = false;

  shouldDisableClientInput = false;
  shouldDisableUserInput = false;
  @ViewChild('accessForm') accessForm: NgForm;

  DataType = APIDataTypeEnum;
  isDarkMode: boolean;
  dynamicRoute: string = '';
  dynamicTitle: string = '';
  selectedCondition: any = {
    label: '',
    value: ''
  };
  conditionOptions = [
    { label: 'None', value: 'null' },
    { label: 'Event = true', value: 'true' },
    { label: 'Event = false', value: 'false' }
  ];
  constructor(
    public router: Router,
    private activatedRoute: ActivatedRoute,
    private accessService: AccessPolicyService,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog,
    private titleService: Title
  ) {}

  ngOnInit() {
    this.selectedTimezoneLabel = this.selectedTimezone;
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.disableSaveButton = false;
    this.policyType = this.activatedRoute.snapshot.params.policyType;
    this.withUser =
      this.policyType == 'user-agent-policy' || this.policyType == 'user-agent';
    this.activatedRoute.params.subscribe(data => {
      if (!data.id) {
        //create
        this.title = 'Add';
        this.getAllDropDownsData();
        this.selectedCondition.label = null
      } else {
        //Edit
        this.loading = true;
        this.isEditing = true;
        this.title = 'Edit';
        this.currentEditingId = data.id;
        this.getSinglePolicy(data.id);
      }
    });
    this.title += this.withUser ? ' User' : ' Device';
    this.subscribeClientSearch();
    this.subscribeServiceSearch();
    if (this.withUser) {
      this.titleService.setTitle(
        'Policy Management | Entitlements | User Entitlements | ' +
          this.title +
          ' Entitlements'
      );
      this.dynamicTitle = 'User Entitlements';
      this.dynamicRoute = '/user-agent-access-policy';
    } else {
      this.titleService.setTitle(
        'Policy Management | Entitlements | Device Entitlements | ' +
          this.title +
          ' Entitlements'
      );
      this.dynamicTitle = 'Device Entitlements';
      this.dynamicRoute = '/device-access-policy';
    }
    this.buttonShouldBeDisabled();
  }

  ngAfterViewInit(): void {}

  private getSinglePolicy(id) {
    this.accessService.getSinglePolicy(id).subscribe(
      (data: any) => {
        this.loading = false;
        const {
          name,
          access_restriction,
          access_end_time,
          access_start_time,
          time_zone,
          event
        } = data;

        this.accessChecked = access_restriction;
        this.policyName = name;
        const selectedOption = this.conditionOptions.find(
          option => option.value == JSON.stringify(event)
        );
        // Assign the found option to selectedCondition
        this.selectedCondition = this.conditionOptions.filter(op => op.value == JSON.stringify(event))[0]
        this.selectedTimezone = time_zone;
        
        const selectedTimezoneObj = this.timezones.find(
          timezone => timezone.value === this.selectedTimezone
        );
        this.selectedTimezoneLabel = selectedTimezoneObj
          ? selectedTimezoneObj.name
          : null;

        if(this.accessChecked){
          let convertedTimeZone = this.getMomentTimeZone(time_zone);
          let coverted_access_start_time = this.convertFromUTC(access_start_time,convertedTimeZone)
          let coverted_access_end_time = this.convertFromUTC(access_end_time,convertedTimeZone)
          // this.startDate = new Date(`14 Dec 2020 ${access_start_time}`);
          // this.endDate = new Date(`14 Dec 2020 ${access_end_time}`);
          this.startDate = new Date(`14 Dec 2020 ${coverted_access_start_time}`);
          this.endDate = new Date(`14 Dec 2020 ${coverted_access_end_time}`);
         
        }
        this.parseAllEditData(data);
      },
      err => {
        this.loading = false;
      }
    );
  }

  private parseAllEditData(data) {
    if (data.all_clients) {
      this.selectAllClients();
    }
    if (data.all_services) {
      this.selectAllServices();
    }
    if (data?.all_users) {
      this.selectAllUsers();
    }
    if (data.access_restriction) {
      this.selectedDays = data.days;
      this.selectedMonths = data.months;
      this.months.forEach(m => {
        if (this.selectedMonths.indexOf(m.id) !== -1) {
          m.isChecked = true;
        }
      });
      this.weekdays.forEach(w => {
        if (this.selectedDays.indexOf(w.id) !== -1) {
          w.isChecked = true;
        }
      });
    } else {
      this.startDate = new Date(`14 Dec 2020 00:00:00`);
      this.endDate = new Date(`14 Dec 2020 23:59:00`);
    }

    this.selectedCondition = this.conditionOptions.filter(op => op.value == JSON.stringify(data.event))[0];
    this.selectedClientIds = [
      ...data.Clients.map(client => client.id),
      ...data.Groups.map(g => g.id)
    ];

    this.selectedClientTypes = [
      ...data.Clients.map(({ id, Name, device_os }) => {
        return {
          id,
          name: Name,
          type: 'client',
          device_os
        };
      }),
      ...data.Groups.map(({ id, name }) => {
        return {
          id,
          name,
          type: 'clientGroup'
        };
      })
    ];

    this.selectedServiceIds = [
      ...data.Services.map(client => client.id),
      ...data.ServiceGroups.map(g => g.id)
    ];
    this.selectedServiceTypes = [
      ...data.Services.map(({ id, Name }) => {
        return {
          id,
          name: Name,
          type: 'service'
        };
      }),
      ...data.ServiceGroups.map(({ id, name }) => {
        return {
          id,
          name,
          type: 'serviceGroup'
        };
      })
    ];

    this.selectedClientCount = this.selectedClientIds.length;
    this.selectedServiceCount = this.selectedServiceIds.length;

    this.clients = [...this.clients, ...this.selectedClientTypes];
    this.services = [...this.services, ...this.selectedServiceTypes];

    this.selectedUserIds = [
      ...data.Users.map(user => user.runtime_bind_id),
      ...data.UserGroups.map(g => g.runtime_bind_id)
    ];

    this.selectedUserTypes = [
      ...data.Users.map(({ id, Name, runtime_bind_id }) => {
        return {
          id,
          name: Name,
          type: 'user',
          runtime_bind_id
        };
      }),
      ...data.UserGroups.map(({ id, name, runtime_bind_id }) => {
        return {
          id,
          name,
          type: 'userGroup',
          runtime_bind_id
        };
      })
    ];

    this.selectedUserCount = this.selectedUserIds.length;
    this.users = [...this.users, ...this.selectedUserTypes];

    this.getAllDropDownsData();
  }

  private subscribeClientSearch() {
    this.$clientSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.isClientSearching = true;
          this.spaces = 0;
          this.currentClientSearch = term;
          this.clientOffset = 0;
          return this.accessService.getAllDropDownDataWithoutPagination(
            this.clientOffset,
            this.serviceOffset,
            this.currentClientSearch,
            this.currentServiceSearch,
            this.currentEditingId,
            ''
          );
        })
      )
      .subscribe((data: any) => {
        this.isClientSearching = false;
        this.clients = data.filter(
          d =>
            d.type === APIDataTypeEnum.Client ||
            d.type === APIDataTypeEnum.ClientGroup
        );
      });
  }

  appendrightOs(os) {
    switch (os) {
      case DeviceOSEnum.Ubuntu18:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer18:
        return 'Ubuntu Server';
      case DeviceOSEnum.Macintosh:
        return 'macOS';
      case DeviceOSEnum.Windows:
        return 'Windows';
      case DeviceOSEnum.Ubuntu:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer:
        return 'Ubuntu Server';
      case DeviceOSEnum.WindowsServer:
        return 'Windows Server';
      case DeviceOSEnum.IOS:
        return 'iOS';
      case DeviceOSEnum.Android:
        return 'Android';
      case DeviceOSEnum.UbuntuServer20armv8:
        return 'Ubuntu Server (ARMv8)';
      case DeviceOSEnum.Unknown:
        return 'Unknown';
      case DeviceOSEnum.CentosServer:
        return 'CentOS Server';
      default:
        return os;
    }
  }

  private subscribeServiceSearch() {
    this.$serviceSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.serviceOffset = 0;
          this.isServiceSearching = true;
          this.currentServiceSearch = term;
          return this.accessService.getAllDropDownDataWithoutPagination(
            this.clientOffset,
            this.serviceOffset,
            this.currentClientSearch,
            this.currentServiceSearch,
            this.currentEditingId,
            ''
          );
        })
      )
      .subscribe((data: any) => {
        this.isServiceSearching = false;
        this.services = data.filter(
          d =>
            d.type === APIDataTypeEnum.Service ||
            d.type === APIDataTypeEnum.ServiceGroup
        );
      });
  }

  clientChange() {
    if (this.selectedClientIds.length >= this.selectedClientCount) {
      this.selectedClientCount = this.selectedClientCount + 1;
      const lastId = this.selectedClientIds[this.selectedClientIds.length - 1];
      const client = this.clients.find(item => item.id === lastId);
      this.selectedClientTypes = [...this.selectedClientTypes, client];
    } else {
      this.selectedClientCount = this.selectedClientIds.length;
      this.selectedClientTypes = this.selectedClientTypes.filter(client => {
        return this.selectedClientIds.indexOf(client.id) !== -1;
      });
    }

    this.buttonShouldBeDisabled();
  }

  serviceChange() {
    if (this.selectedServiceIds.length >= this.selectedServiceCount) {
      this.selectedServiceCount = this.selectedServiceCount + 1;
      const lastId = this.selectedServiceIds[
        this.selectedServiceIds.length - 1
      ];
      const client = this.services.find(item => item.id === lastId);
      this.selectedServiceTypes = [...this.selectedServiceTypes, client];
    } else {
      this.selectedServiceCount = this.selectedServiceIds.length;
      this.selectedServiceTypes = this.selectedServiceTypes.filter(client => {
        return this.selectedServiceIds.indexOf(client.id) !== -1;
      });
    }
  }

  clientFocus() {
    this.clientOffset = 0;
    if (this.currentClientSearch) {
      this.currentClientSearch = '';
      this.$clientSearch.next('');
    } else {
      this.currentClientSearch = '';
    }
  }

  serviceFocus() {
    this.serviceOffset = 0;
    if (this.currentServiceSearch) {
      this.currentServiceSearch = '';
      this.$serviceSearch.next('');
    } else {
      this.currentServiceSearch = '';
    }
  }

  getAllDropDownsData() {
    this.accessService
      .getAllDropDownDataWithoutPagination(
        this.clientOffset,
        this.serviceOffset,
        this.currentClientSearch,
        this.currentServiceSearch,
        this.currentEditingId,
        ''
      )
      .subscribe((data: any) => {
        const clientData = data.filter(
          d =>
            (d.type === APIDataTypeEnum.Client ||
              d.type === APIDataTypeEnum.ClientGroup) &&
            !this.selectedClientIds?.includes(d.id) &&
            (this.withUser ? d.user_id !== null : d.user_id === null)
        );
        clientData.forEach(c => (this.clients = [...this.clients, c]));
        this.buttonShouldBeDisabled();

        const serviceData = data.filter(
          d =>
            (d.type === APIDataTypeEnum.Service ||
              d.type === APIDataTypeEnum.ServiceGroup) &&
            !this.selectedServiceIds?.includes(d.id)
        );
        serviceData.forEach(s => (this.services = [...this.services, s]));
      });

    this.isUserSearching = true;
    forkJoin([this.accessService.getUserDropDown()])
      .pipe(
        map((response: any) => {
          return response[0]?.data;
        }),
        catchError((error: any) => {
          // Handle errors here
          return error;
        })
      )
      .subscribe({
        next: data => {
          this.users = data;
          this.isUserSearching = false;
        },
        error: error =>
          catchError((error: any) => {
            // Handle errors here
            return error;
          })
      });
  }

  // User Section Started

  selectAllUsers() {
    this.selectedUserIds = null;
    this.selectedUserCount = 0;
    this.allUsersSelected = !this.allUsersSelected;
    this.usersPlaceHolder = !this.allUsersSelected
      ? 'Search User or User Group'
      : 'All Users are selected';

    this.buttonShouldBeDisabled();
  }

  selectAllUserClick(selectall) {
    if (
      (!this.selectedClientIds || this.selectedClientIds?.length <= 0) &&
      !this.allClientsSelected
    ) {
      if (selectall) {
        if (this.selectedClientCount > 0) {
          const dialogRef = this.dialog.open(ConfirmModalComponent, {
            panelClass: this.isDarkMode ? 'dark-theme' : '',
            data: {
              title: 'Confirmation',
              message: 'Are you sure you want to select all User?'
            }
          });

          dialogRef.afterClosed().subscribe(result => {
            if (result) {
              this.selectAllUsers();
            }
          });
        } else {
          this.selectAllUsers();
        }
      } else {
        this.selectAllUsers();
      }
    }
    this.buttonShouldBeDisabled();
  }

  userFocus() {
    this.userOffset = 0;
    if (this.currentServiceSearch) {
      this.currentServiceSearch = '';
      this.$userSearch.next('');
    } else {
      this.currentServiceSearch = '';
    }
  }

  userChange() {
    if (this.selectedUserIds.length >= this.selectedUserCount) {
      this.selectedUserCount = this.selectedUserCount + 1;
      const lastId = this.selectedUserIds[this.selectedUserIds.length - 1];
      const user = this.users.find(item => item.runtime_bind_id === lastId);
      this.selectedUserTypes = [...this.selectedUserTypes, user];
    } else {
      this.selectedUserCount = this.selectedUserIds.length;
      this.selectedUserTypes = this.selectedUserTypes.filter(user => {
        return this.selectedUserIds.indexOf(user.runtime_bind_id) !== -1;
      });
    }

    this.buttonShouldBeDisabled();
  }

  condtionChange() {
    console.log('selectedCondition :>> ', this.selectedCondition);
    // if (this.selectedUserIds && this.selectedUserIds.length > 0) {
    //   this.selectedCondition = 'true';
    // } else {
    //   this.selectedCondition = 'false';
    // }
  }
  private parseAllSelectedUser() {
    if (!this.allUsersSelected) {
      this.user = this.selectedUserTypes
        .filter(u => u.type === APIDataTypeEnum.User)
        .map(u => u.id);
      this.userGroup = this.selectedUserTypes
        .filter(u => u.type === APIDataTypeEnum.UserGroup)
        .map(u => u.id);
    }
  }

  // User Section Ended

  fetchMoreServices() {
    this.serviceOffset += 1;
    this.accessService
      .getAllDropDownDataWithoutPagination(
        this.clientOffset,
        this.serviceOffset,
        this.currentClientSearch,
        this.currentServiceSearch,
        this.currentEditingId,
        ''
      )
      .subscribe((data: any) => {
        const serviceData = data.filter(
          d =>
            (d.type === APIDataTypeEnum.Service ||
              d.type === APIDataTypeEnum.ServiceGroup) &&
            !this.selectedServiceIds?.includes(d.id)
        );
        serviceData.forEach(s => (this.services = [...this.services, s]));
      });
  }

  fetchMoreClients() {
    this.clientOffset += 1;
    this.accessService
      .getAllDropDownDataWithoutPagination(
        this.clientOffset,
        this.serviceOffset,
        this.currentClientSearch,
        this.currentServiceSearch,
        this.currentEditingId,
        ''
      )
      .subscribe((data: any) => {
        const clientData = data.filter(
          d =>
            (d.type === APIDataTypeEnum.Client ||
              d.type === APIDataTypeEnum.ClientGroup) &&
            !this.selectedClientIds?.includes(d.id)
        );
        if (this.spaces < 5 && clientData.length === 0) {
          this.spaces += 1;
          this.clients = [
            ...this.clients,
            { type: 'clientGroup', id: '', name: '', disabled: true }
          ];
        }
        clientData.forEach(c => (this.clients = [...this.clients, c]));
      });
  }

  buttonHandler(event, button) {
    event.preventDefault();
    button.isChecked = !button.isChecked;
    const index = this.selectedDays.indexOf(button.id);
    if (index !== -1) {
      this.selectedDays.splice(index, 1);
    } else {
      this.selectedDays.push(button.id);
    }
    if (this.selectedDays.length === 0 || this.selectedMonths.length === 0) {
      this.saveButtonDisabled = true;
    } else {
      this.saveButtonDisabled = false;
    }
  }

  monthButtonHandler(event, month) {
    event.preventDefault();
    month.isChecked = !month.isChecked;
    const index = this.selectedMonths.indexOf(month.id);
    if (index !== -1) {
      this.selectedMonths.splice(index, 1);
    } else {
      this.selectedMonths.push(month.id);
    }
    if (this.selectedDays.length === 0 || this.selectedMonths.length === 0) {
      this.saveButtonDisabled = true;
    } else {
      this.saveButtonDisabled = false;
    }
  }

  selectAllClients() {
    this.selectedClientIds = null;
    this.selectedClientCount = 0;
    this.allClientsSelected = !this.allClientsSelected;
    this.clientsPlaceHolder = !this.allClientsSelected
      ? 'Search Client or Client Group'
      : 'All Clients are selected';
    this.buttonShouldBeDisabled();
  }

  selectAllServices() {
    this.selectedServiceIds = null;
    this.selectedServiceCount = 0;
    this.allServicesSelected = !this.allServicesSelected;
    this.servicesPlaceHolder = !this.allServicesSelected
      ? 'Search Service or Service Group'
      : 'All services are selected';
  }

  selectAllServicesClick(selectall) {
    if (selectall) {
      if (this.selectedServiceCount > 0) {
        const dialogRef = this.dialog.open(ConfirmModalComponent, {
          panelClass: this.isDarkMode ? 'dark-theme' : '',
          data: {
            title: 'Confirmation',
            message: 'Are you sure you want to select all Services?'
          }
        });

        dialogRef.afterClosed().subscribe(result => {
          if (result) {
            this.selectAllServices();
          }
        });
      } else {
        this.selectAllServices();
      }
    } else {
      this.selectAllServices();
    }
  }

  selectAllClientsClick(selectall) {
    if (
      (!this.selectedUserIds || this.selectedUserIds?.length <= 0) &&
      !this.allUsersSelected
    ) {
      if (selectall) {
        if (this.selectedClientCount > 0) {
          const dialogRef = this.dialog.open(ConfirmModalComponent, {
            panelClass: this.isDarkMode ? 'dark-theme' : '',
            data: {
              title: 'Confirmation',
              message: 'Are you sure you want to select all Clients?'
            }
          });

          dialogRef.afterClosed().subscribe(result => {
            if (result) {
              this.selectAllClients();
            }
          });
        } else {
          this.selectAllClients();
        }
      } else {
        this.selectAllClients();
      }
    }
    this.buttonShouldBeDisabled();
  }

  accessChanged() {
    if (!this.accessChecked) {
      this.weekdays?.forEach(d => (d.isChecked = false));
      this.selectedDays = [];
    }
    this.saveButtonDisabled = this.accessChecked;
  }

  isValid(event: boolean): void {
    this.valid = event;
    this.disableSaveButton = !this.valid;
  }
  convertFromUTC(timeString, timeZoneMomentValue) {
    const utcTime = moment.utc(`1970-01-01 ${timeString}`, 'YYYY-MM-DD HH:mm');
    const localTime = utcTime.tz(timeZoneMomentValue);
    return localTime.format('HH:mm:ss');
  }
  convertToUTC(timeString, timeZoneMomentValue) {
    const localTime = moment.tz(
      `1970-01-01 ${timeString}`,
      'YYYY-MM-DD HH:mm',
      timeZoneMomentValue
    );
    const utcTime = localTime.utc();
    return utcTime.format('HH:mm');
  }

  getMomentTimeZone(selectedTimeZone) {
    const timezone = this.timezones.find(tz => tz.value === selectedTimeZone);
    return timezone ? timezone.moment : null;
  }

  savePolicyHandler(event) {
    this.loading = true;
    event.stopPropagation();
    const startTime = `${
      this.startDate.getHours() < 10
        ? '0' + this.startDate.getHours()
        : this.startDate.getHours()
    }:${
      this.startDate.getMinutes() < 10
        ? '0' + this.startDate.getMinutes()
        : this.startDate.getMinutes()
    }`;
    const endTime = `${
      this.endDate.getHours() < 10
        ? '0' + this.endDate.getHours()
        : this.endDate.getHours()
    }:${
      this.endDate.getMinutes() < 10
        ? '0' + this.endDate.getMinutes()
        : this.endDate.getMinutes()
    }`;
    this.parseAllSelectedClients();
    this.parseAllSelectedUser();

    if (this.selectedCondition.label.value == 'null') {
      this.selectedCondition.label.value = null;
    }

    let timeZoneMomentValue
    let convertedStartTime
    let convertedEndTime

    if(this.accessChecked){
       timeZoneMomentValue = this.getMomentTimeZone(this.selectedTimezone);
       convertedStartTime = this.convertToUTC(startTime, timeZoneMomentValue);
       convertedEndTime = this.convertToUTC(endTime, timeZoneMomentValue);
    }

    const policyObject = {
      name: this.policyName,
      access_restriction: this.accessChecked,
      time_zone: (this.accessChecked && this.selectedTimezone) ? this.selectedTimezone : null,
      access_start_time: (this.accessChecked && convertedStartTime) ? convertedStartTime : '',
      access_end_time: (this.accessChecked && convertedEndTime) ? convertedEndTime : '',
      months: this.selectedMonths,
      days: this.selectedDays,
      is_enabled: true,
      client: this.client,
      clientGroup: this.clientGroup,
      service: this.service,
      serviceGroup: this.serviceGroup,
      all_clients: this.allClientsSelected,
      all_services: this.allServicesSelected,
      user: this.user,
      userGroup: this.userGroup,
      all_users: this.allUsersSelected,
      event: this.selectedCondition.label.value,
      policy_type:
        this.policyType == 'device-policy'
          ? 'Device'
          : this.policyType == 'user-agent-policy'
          ? 'User Agent'
          : ''
    };
    this.disableSaveButton = true;
    if (!this.isEditing) {
      this.accessService.createService(policyObject).subscribe(
        data => {
          this.loading = false;
          this.showAlert('success', 'Entitlements Created', false);
          this.router.navigate(['/access-policy']);
        },
        error => {
          this.loading = false;
          this.disableSaveButton = false;
          // this.showAlert('danger', JSON.parse(error._body).message, false);
        }
      );
    } else {
      this.accessService
        .updateService(policyObject, this.currentEditingId)
        .subscribe(
          data => {
            this.loading = false;
            this.showAlert('success', 'Entitlements Updated', false);
            this.router.navigate(['/access-policy']);
          },
          error => {
            this.loading = false;
            this.disableSaveButton = false;
            // this.showAlert('danger', JSON.parse(error._body).message, false);
          }
        );
    }
  }

  timezoneChangeHandler() {
    this.selectedTimezoneLabel = this.selectedTimezone;
  }

  private parseAllSelectedClients() {
    if (!this.allClientsSelected) {
      this.client = this.selectedClientTypes
        .filter(client => {
          return client.type === APIDataTypeEnum.Client;
        })
        .map(c => c.id);
      this.clientGroup = this.selectedClientTypes
        .filter(client => {
          return client.type === APIDataTypeEnum.ClientGroup;
        })
        .map(cg => cg.id);
    }
    if (!this.allServicesSelected) {
      this.service = this.selectedServiceTypes
        .filter(s => s.type === APIDataTypeEnum.Service)
        .map(s => s.id);
      this.serviceGroup = this.selectedServiceTypes
        .filter(s => s.type === APIDataTypeEnum.ServiceGroup)
        .map(s => s.id);
    }
  }

  private showAlert(type: string, message: string, sticky: boolean) {
    const alert = {
      closed: false,
      type: type,
      message: message,
      sticky: sticky
    };
    this.globalMessageService.pushAlert(alert);
  }

  buttonShouldBeDisabled() {
    if (this.selectedUserIds?.length > 0 || this.allUsersSelected) {
      this.shouldDisableClientInput = true;
      this.shouldDisableUserInput = false;
    } else if (this.selectedClientIds?.length > 0 || this.allClientsSelected) {
      this.shouldDisableUserInput = true;
      this.shouldDisableClientInput = false;
    } else {
      this.shouldDisableUserInput = false;
      this.shouldDisableClientInput = false;
    }
  }
}
