import { Component, OnInit } from '@angular/core';
import { OsPolicyService } from './os-policy.service';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { Router, ActivatedRoute } from '@angular/router';
import { OsService } from '../common/os-service.service';
import { DeviceOSEnum } from '../shared/emuns/client.enum';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-os-policy',
  templateUrl: './create-os-policy.component.html',
  styleUrls: ['./create-os-policy.component.css']
})
export class CreateOsPolicyComponent implements OnInit {
  policyName;
  client = {
    device_os: '',
    client_os_version: 'any',
    is_blocked: false
  };
  DeviceOS = DeviceOSEnum;
  
  win61OsVersion;
  win62OsVersion;
  win63OsVersion;
  win10OsVersion;
  win11OsVersion;
  winserv61OsVersion;
  winserv62OsVersion;
  winserv63OsVersion;
  winserv10OsVersion;
  centosserv7Version;
  centosserv8Version;
  macMojaveVersions;
  macCatalinaVersions;
  macBigSurVersions;
  macMontereyVersions;
  macVenturaVersions;
  macSonomaVersions;
  ubuntu18OsVersions;
  ubuntu16OsVersions;
  androidOsVersions;
  iOsOsVersions;
  ubuntuserver20armv8Versions;
  windowsserver;

  clientAllOs = [
    { value: DeviceOSEnum.Ubuntu18, display: 'Ubuntu Desktop' },
    { value: DeviceOSEnum.UbuntuServer18, display: 'Ubuntu Server' },
    { value: DeviceOSEnum.UbuntuServer20armv8, display: 'Ubuntu Server (ARMv8)' },
    { value: DeviceOSEnum.Windows6_1, display: 'Windows 7' },
    { value: DeviceOSEnum.Windows6_2, display: 'Windows 8' },
    { value: DeviceOSEnum.Windows6_3, display: 'Windows 8.1' },
    { value: DeviceOSEnum.Windows10, display: 'Windows 10' },
    { value: DeviceOSEnum.Windows11, display: 'Windows 11' },
    { value: DeviceOSEnum.WindowsServer6_1, display: 'Windows Server 2008 R2' },
    { value: DeviceOSEnum.WindowsServer6_2, display: 'Windows Server 2012' },
    { value: DeviceOSEnum.WindowsServer6_3, display: 'Windows Server 2012 R2' },
    { value: DeviceOSEnum.WindowsServer10, display: 'Windows Server 2016+' },
    { value: DeviceOSEnum.CentosServer7, display: 'CentOS 7 Server' },
    { value: DeviceOSEnum.CentosServer8, display: 'CentOS 8 Server' },
    { value: DeviceOSEnum.Macintosh10_14, display: 'macOS Mojave' },
    { value: DeviceOSEnum.Macintosh10_15, display: 'macOS Catalina' },
    { value: DeviceOSEnum.Macintosh11, display: 'macOS Big Sur' },
    { value: DeviceOSEnum.Macintosh12, display: 'macOS Monterey' },
    { value: DeviceOSEnum.Macintosh13, display: 'macOS Ventura' },
    { value: DeviceOSEnum.Macintosh14, display: 'macOS Sonoma' },
    { value: DeviceOSEnum.IOS, display: 'iOS' },
    { value: DeviceOSEnum.Android, display: 'Android' },
    { value: DeviceOSEnum.WindowsServer, display: 'Windows Server'}
  ];

  policies = [
    { value: false, display: 'Warn Only' },
    { value: true, display: 'Block' }
  ];

  deviceOs = [];

  clientOsVersion = [];
  title;
  isEditing = false;
  editingId = '';
  loading: boolean = false;
  showOSVersionModal = false;
  newOSVersion = '';
  versionLoading: boolean = false;
  disableSaveButton = false;
  isDarkMode :boolean;
  constructor(
    private osService: OsPolicyService,
    public router: Router,
    private globalMessageService: GlobalMessageService,
    private activatedRoute: ActivatedRoute,
    private osVersionService: OsService,
    public dialog: MatDialog,
    private titleService: Title
  ) {
    this.client.is_blocked = this.policies[0].value;
    this.deviceOsSelected(false);
  }

  async ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.disableSaveButton = false;
    const osVersions = await this.osVersionService.getOsVersions().toPromise();
    this.updateOSVersionArrays(osVersions);

    const id = this.activatedRoute.snapshot.params.id;
    if (!id) {
      //create
      this.title = 'Add';
      this.getAllOs();
    } else {
      //edit
      this.title = 'Edit';
      this.isEditing = true;
      this.editingId = id;
      this.getDetails();
    }
    this.titleService.setTitle('Policy Management | Device Policy | OS Policy | ' + this.title + ' OS Policy');
  }

  openModal() {
    if (this.client.client_os_version == 'addNew') {
      this.showOSVersionModal = true;
    }
  }

  closeModal() {
    this.client.client_os_version = 'any';
    this.newOSVersion = '';
    this.showOSVersionModal = false;
  }

  saveNewOSVersion() {
    if (this.newOSVersion != '') {
      if (this.newOSVersion.includes('..')) {
        this.globalMessageService.pushAlert({
          closed: false,
          type: 'danger',
          message: 'Invalid OS Version',
          sticky: false
        });
        return;
      }
      this.versionLoading = true;
      let selectedOS;
      if (this.client.device_os == DeviceOSEnum.UbuntuServer) selectedOS = DeviceOSEnum.Ubuntu;
      else if (this.client.device_os == DeviceOSEnum.UbuntuServer18)
        selectedOS = DeviceOSEnum.Ubuntu18;
      else selectedOS = this.client.device_os;
      this.osVersionService
        .saveNewOSVersion(selectedOS, this.newOSVersion)
        .subscribe(
          _ => {
            this.osVersionService.getOsVersions().subscribe(osVersions => {
              this.updateOSVersionArrays(osVersions);
              this.deviceOsSelected(false);
            });
            this.globalMessageService.pushAlert({
              closed: false,
              type: 'success',
              message: 'OS Version added successfully',
              sticky: false
            });
            this.client.client_os_version = 'any';
            this.newOSVersion = '';
            this.showOSVersionModal = false;
            this.versionLoading = false;
          },
          _error => {
            this.versionLoading = false;
          }
        );
    }
  }

  private getAllOs() {
    this.osService.getAllOs(this.editingId).subscribe((data: any) => {
      this.deviceOs = this.clientAllOs.filter(os => {
        return data.indexOf(os.value) === -1;
      });
      if (this.isEditing) {
        this.deviceOs = [
          this.clientAllOs.find(os => os.value === this.client.device_os),
          ...this.deviceOs
        ];
      } else if (!this.isEditing && this.deviceOs[0] != undefined) {
        this.client.device_os = this.deviceOs[0].value;
      } else {
        this.client.device_os = '';
      }
      this.deviceOsSelected(false);
    });
  }

  getDetails() {
    this.loading = true;
    this.osService.getSinglePolicy(this.editingId).subscribe(
      (data: any) => {
        this.loading = false;
        const { os, os_version, is_blocked } = data;
        this.client = {
          device_os: os,
          client_os_version: os_version,
          is_blocked: is_blocked
        };
        this.getAllOs();
      },
      err => {
        this.loading = false;
      }
    );
  }

  deviceOsSelected(fromTemplate = true): void {
    const { device_os } = this.client;
    if (device_os === DeviceOSEnum.Windows6_1) {
      this.clientOsVersion = this.win61OsVersion;
    } else if (device_os === DeviceOSEnum.Windows6_2) {
      this.clientOsVersion = this.win62OsVersion;
    } else if (device_os === DeviceOSEnum.Windows6_3) {
      this.clientOsVersion = this.win63OsVersion;
    } else if (device_os === DeviceOSEnum.Windows10) {
      this.clientOsVersion = this.win10OsVersion;
    } else if (device_os === DeviceOSEnum.Windows11) {
      this.clientOsVersion = this.win11OsVersion;
    } else if (device_os === DeviceOSEnum.WindowsServer6_1) {
      this.clientOsVersion = this.winserv61OsVersion;
    } else if (device_os === DeviceOSEnum.WindowsServer6_2) {
      this.clientOsVersion = this.winserv62OsVersion;
    } else if (device_os === DeviceOSEnum.WindowsServer6_3) {
      this.clientOsVersion = this.winserv63OsVersion;
    } else if (device_os === DeviceOSEnum.WindowsServer10) {
      this.clientOsVersion = this.winserv10OsVersion;
    } else if (device_os === DeviceOSEnum.CentosServer7) {
      this.clientOsVersion = this.centosserv7Version;
    } else if (device_os === DeviceOSEnum.CentosServer8) {
      this.clientOsVersion = this.centosserv8Version;
    } else if (device_os === DeviceOSEnum.Macintosh10_14) {
      this.clientOsVersion = this.macMojaveVersions;
    } else if (device_os === DeviceOSEnum.Macintosh10_15) {
      this.clientOsVersion = this.macCatalinaVersions;
    } else if (device_os === DeviceOSEnum.Macintosh11) {
      this.clientOsVersion = this.macBigSurVersions;
    } else if (device_os === DeviceOSEnum.Macintosh12) {
      this.clientOsVersion = this.macMontereyVersions;
    } else if (device_os === DeviceOSEnum.Macintosh13) {
      this.clientOsVersion = this.macVenturaVersions;
    } else if (device_os === DeviceOSEnum.Macintosh14) {
      this.clientOsVersion = this.macSonomaVersions;
    } else if (device_os === DeviceOSEnum.IOS) {
      this.clientOsVersion = this.iOsOsVersions;
    } else if (device_os === DeviceOSEnum.Android) {
      this.clientOsVersion = this.androidOsVersions;
    } else if (device_os === DeviceOSEnum.Ubuntu || device_os === DeviceOSEnum.UbuntuServer) {
      this.clientOsVersion = this.ubuntu16OsVersions;
    } else if (device_os === DeviceOSEnum.Ubuntu18 || device_os === DeviceOSEnum.UbuntuServer18) {
      this.clientOsVersion = this.ubuntu18OsVersions;
    } else if (device_os === DeviceOSEnum.UbuntuServer20armv8) {
      this.clientOsVersion = this.ubuntuserver20armv8Versions;
    }else if (device_os === DeviceOSEnum.WindowsServer) {
      this.clientOsVersion = this.windowsserver;
    } 

    this.clientOsVersion = [
      { value: 'any', display: 'Any' },
      ...(this.clientOsVersion || []).sort(
        (a, b) => parseInt(b.value) - parseInt(a.value)
      ),
      { value: 'addNew', display: '+ Add new version' }
    ];

    if (fromTemplate) {
      this.client.client_os_version = 'any';
    }
  }

  handleDelete(item) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Are you sure want to delete version ${item.display}?`,
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        this.osVersionService.deleteOsVersions(item.id).subscribe(_response => {
          this.clientOsVersion = this.clientOsVersion.filter(
            version => version.id !== item.id
          );
          this.client.client_os_version = 'any';
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'success',
            message: 'OS version deleted successfully',
            sticky: false
          });
        });
      }
    });
  }

  submitHandler() {
    const { device_os, client_os_version, is_blocked } = this.client;
    if (!is_blocked && client_os_version == 'any') {
      this.globalMessageService.pushAlert({
        closed: false,
        type: 'danger',
        message: "'Warn Only' policy cannot be defined with 'any' OS version.",
        sticky: false
      });
      return;
    }
    this.loading = true;
    this.disableSaveButton = true;
    const policy = {
      os: device_os,
      os_version: client_os_version,
      is_blocked
    };
    if (!this.isEditing) {
      this.osService.createService(policy).subscribe(
        data => {
          this.loading = false;
          this.showAlert('success', 'OS Policy Created', false);
          this.router.navigate(['/os-policy']);
        },
        error => {
          this.disableSaveButton = false;
          this.loading = false;
          // this.showAlert('danger', JSON.parse(error._body).message, false);
        }
      );
    } else {
      this.osService.updateService(policy, this.editingId).subscribe(
        data => {
          this.loading = false;
          this.showAlert('success', 'Os Policy Updated', false);
          this.router.navigate(['/os-policy']);
        },
        error => {
          this.disableSaveButton = false;
          this.loading = false;
          // this.showAlert('danger', JSON.parse(error._body).message, false);
        }
      );
    }
  }

  private showAlert(type: string, message: string, sticky: boolean) {
    const alert = {
      closed: false,
      type: type,
      message: message,
      sticky: sticky
    };
    this.globalMessageService.pushAlert(alert);
  }

  private updateOSVersionArrays(osVersions) {
    this.win62OsVersion = osVersions[DeviceOSEnum.Windows6_2] || [];
    this.win61OsVersion = osVersions[DeviceOSEnum.Windows6_1] || [];
    this.win63OsVersion = osVersions[DeviceOSEnum.Windows6_3] || [];
    this.win10OsVersion = osVersions[DeviceOSEnum.Windows10] || [];
    this.win11OsVersion = osVersions[DeviceOSEnum.Windows11] || [];
    this.winserv61OsVersion = osVersions[DeviceOSEnum.WindowsServer6_1] || [];
    this.winserv62OsVersion = osVersions[DeviceOSEnum.WindowsServer6_2] || [];
    this.winserv63OsVersion = osVersions[DeviceOSEnum.WindowsServer6_3] || [];
    this.winserv10OsVersion = osVersions[DeviceOSEnum.WindowsServer10] || [];
    this.centosserv7Version = osVersions[DeviceOSEnum.CentosServer7] || [];
    this.centosserv8Version = osVersions[DeviceOSEnum.CentosServer8] || [];
    this.macMojaveVersions = osVersions[DeviceOSEnum.Macintosh10_14] || [];
    this.macCatalinaVersions = osVersions[DeviceOSEnum.Macintosh10_15] || [];
    this.macBigSurVersions = osVersions[DeviceOSEnum.Macintosh11] || [];
    this.macMontereyVersions = osVersions[DeviceOSEnum.Macintosh12] || [];
    this.macVenturaVersions = osVersions[DeviceOSEnum.Macintosh13] || [];
    this.macSonomaVersions = osVersions[DeviceOSEnum.Macintosh14] || [];
    this.ubuntu18OsVersions = osVersions[DeviceOSEnum.Ubuntu18] || [];
    this.ubuntu16OsVersions = osVersions[DeviceOSEnum.Ubuntu] || [];
    this.androidOsVersions = osVersions[DeviceOSEnum.Android] || [];
    this.iOsOsVersions = osVersions[DeviceOSEnum.IOS] || [];
    this.ubuntuserver20armv8Versions = osVersions[DeviceOSEnum.UbuntuServer20armv8] || [];
    this.windowsserver = osVersions[DeviceOSEnum.WindowsServer] || [];
  }
}
