import { Component, OnInit } from '@angular/core';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { Subject } from 'rxjs';
import { Router, ActivatedRoute } from '@angular/router';
import { AccessPolicyService } from '../create-access-policy/access-policy.service';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { IotService } from '../iot.service';
import { DeviceOSEnum } from '../shared/emuns/client.enum';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-server-bridge',
  templateUrl: './create-server-bridge.component.html',
  styleUrls: ['./create-server-bridge.component.css']
})
export class CreateServerBridgeComponent implements OnInit {
  type: string = 'server-bridge';
  typeTitle: string = 'Server Bridges';
  dynamicRoute: string = '';
  clients: any = [];
  selectedClientIds;
  clientOffset = 0;
  currentClientSearch = '';
  $clientSearch = new Subject();
  isClientSearching = false;

  physicalDevices: any = [];
  selectedPhysicalDeviceId;
  physicalDeviceOffset = 0;
  currentPhysicalDeviceSearch = '';
  $physicalDeviceSearch = new Subject();
  isPhysicalDeviceSearching = false;

  devices: any = [];
  selecteddeviceId;
  deviceOffset = 0;
  currentdeviceSearch = '';
  $deviceSearch = new Subject();
  isdeviceSearching = false;

  selectedDevices = [];

  selectedDeviceIds = [];
  title: string;
  isEditing = false;
  currentEditingId;
  gatewayName;
  deviceIdsMatch = false;

  loading: boolean = false;

  pane: number = 0;
  disableSaveButton = false;

  constructor(
    public router: Router,
    private activatedRoute: ActivatedRoute,
    private iotService: IotService,
    private globalMessageService: GlobalMessageService,
    private titleService: Title
  ) {
    this.type = router.url.replace('/create/', '');
    this.type = this.type.replace('/edit/', '');
    this.type = this.type.replace(/[0-9]/g, '');
    this.type = this.type.replace('/', '');

    switch (this.type) {
      case 'iot-bridge':
        this.typeTitle = 'Device Bridge (ARM)';
        this.dynamicRoute = "/iot-bridge"
        break;
      case 'client-bridge':
        this.typeTitle = 'Device Bridge (x86)';
        this.dynamicRoute = "/client-bridge"
        break;
      default:
        this.typeTitle = 'Server Bridge';
        this.dynamicRoute = "/server-bridge"
        break;
    }
  }

  ngOnInit() {
    this.disableSaveButton = false;
    this.activatedRoute.params.subscribe(data => {
      this.getPhysicalDevices();
      this.getDevices();
      if (!data.id) {
        //create
        this.title = 'Add';
        this.getServerClients();
      } else {
        //Edit
        this.deviceIdsMatch = true;
        this.isEditing = true;
        this.title = 'Edit';
        this.currentEditingId = data.id;
        this.getSinglePolicy(data.id);
      }
      this.titleService.setTitle('Infrastructure | ' + this.typeTitle + ' | ' + this.title + ' ' + this.typeTitle);
    });
    this.subscribeClientSearch();
  }

  getServerClients() {
    this.loading = true;
    this.iotService
      .getAllDropDownData(
        this.clientOffset,
        this.currentClientSearch,
        this.type
      )
      .subscribe(
        (data: any) => {
          this.clients = [...this.clients, ...data];
          this.loading = false;
        },
        err => {
          this.loading = false;
        }
      );
  }

  getPhysicalDevices() {
    this.loading = true;
    this.iotService
      .getPhysicalDevices(
        this.physicalDeviceOffset,
        this.currentPhysicalDeviceSearch,
        this.type
      )
      .subscribe(
        (data: any) => {
          this.physicalDevices = [...this.physicalDevices, ...data];
          this.loading = false;
        },
        err => {
          this.loading = false;
        }
      );
  }

  getDevices() {
    this.loading = true;
    this.iotService
      .getDevices(this.deviceOffset, this.currentdeviceSearch, this.type)
      .subscribe(
        (data: any) => {
          this.devices = [...this.devices, ...data];
          this.loading = false;
        },
        err => {
          this.loading = false;
        }
      );
  }

  private subscribeClientSearch() {
    this.$clientSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.isClientSearching = true;
          if (term !== null) {
            this.currentClientSearch = term;
          } else {
            this.currentClientSearch = ''
          }
          this.clientOffset = 0;
          return this.iotService.getAllDropDownData(
            this.clientOffset,
            this.currentClientSearch,
            this.type
          );
        })
      )
      .subscribe((data: any) => {
        this.isClientSearching = false;
        if (this.currentClientSearch === '') {
          const combined = [...this.clients, ...data];
          const uniqueSet = new Set(combined.map(item => JSON.stringify(item)));
          this.clients = Array.from(uniqueSet).map(item => JSON.parse(item));
        }
        else {
          this.clients = data;
        }
      });

    this.$physicalDeviceSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.isPhysicalDeviceSearching = true;
          this.currentPhysicalDeviceSearch = term;
          this.physicalDeviceOffset = 0;
          return this.iotService.getPhysicalDevices(
            this.physicalDeviceOffset,
            this.currentPhysicalDeviceSearch,
            this.type
          );
        })
      )
      .subscribe(data => {
        this.isPhysicalDeviceSearching = false;
        this.physicalDevices = data;
      });

    this.$deviceSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.isdeviceSearching = true;
          if (term !== null) {
            this.currentdeviceSearch = term;
          } else {
            this.currentdeviceSearch = ''
          }
          this.deviceOffset = 0;
          return this.iotService.getDevices(
            this.deviceOffset,
            this.currentdeviceSearch,
            this.type
          );
        })
      )
      .subscribe((data: any) => {
        this.isdeviceSearching = false;
        if (this.currentdeviceSearch === '') {
          const combined = [...this.devices, ...data];
          const uniqueSet = new Set(combined.map(item => JSON.stringify(item)));
          this.devices = Array.from(uniqueSet).map(item => JSON.parse(item));        }
        else {
          this.devices = data;
        }
      });
  }

  fetchMoreClients() {
    this.clientOffset += 1;
    this.getServerClients();
  }

  fetchMorePhysicalDevices() {
    this.physicalDeviceOffset += 1;
    this.getPhysicalDevices();
  }

  fetchMoredevices() {
    this.deviceOffset += 1;
    this.getDevices();
  }

  private getSinglePolicy(id) {
    this.loading = true;
    this.iotService.getSinglePolicy(id).subscribe(
      (data: any) => {
        this.loading = false;
        const { name, serverBridgeIds, Client, clientId, device_id } = data;
        this.clients = [Client];
        this.selectedClientIds = clientId;
        this.gatewayName = name;
        // this.selectedDeviceIds = serverBridgeIds.map(device => device.deviceId);
        this.selectedPhysicalDeviceId = device_id;
        this.selectedDevices = serverBridgeIds.map(s => s.DeviceInventory);
      },
      err => {
        this.loading = false;
      }
    );
  }

  savePolicyHandler(event) {
    this.loading = true;
    event.stopPropagation();
    const policyObject = {
      name: this.gatewayName,
      clientId: this.selectedClientIds,
      deviceIds: this.selectedDeviceIds,
      physicalDevice: this.selectedPhysicalDeviceId,
      devices: this.selectedDevices.map(x => x.id)
    };
    this.disableSaveButton = true;
    if (!this.isEditing) {
      this.iotService.createService(policyObject, this.type).subscribe(
        data => {
          this.loading = false;
          this.showAlert('success', 'Server Bridge Created', false);
          this.router.navigate([`/${this.type}`]);
        },
        error => {
          this.disableSaveButton = false;
          this.loading = false;
          // this.showAlert('danger', JSON.parse(error._body).message, false);
        }
      );
    } else {
      this.iotService
        .updateService(policyObject, this.currentEditingId)
        .subscribe(
          data => {
            this.loading = false;
            this.showAlert('success', 'Server Bridge Updated', false);
            this.router.navigate([`/${this.type}`]);
          },
          error => {
            this.disableSaveButton = false;
            this.loading = false;
            // this.showAlert('danger', JSON.parse(error._body).message, false);
          }
        );
    }
  }

  cancelHandler() {
    this.router.navigate([`/${this.type}`]);
  }

  private showAlert(type: string, message: string, sticky: boolean) {
    const alert = {
      closed: false,
      type: type,
      message: message,
      sticky: sticky
    };
    this.globalMessageService.pushAlert(alert);
  }

  deviceIdSelected() {
    this.deviceIdsMatch = true;
    const regex = /^([0-9A-Fa-f]{2}[:]){5}([0-9A-Fa-f]{2})$/;
    this.selectedDeviceIds.forEach(id => {
      if (!regex.test(id)) {
        this.deviceIdsMatch = false;
      }
    });
  }

  addDevice() {
    if (this.selecteddeviceId && !this.isAlreadyExist(this.selecteddeviceId)) {
      this.selectedDevices.push(
        this.devices.filter(x => x.id === this.selecteddeviceId)[0]
      );
    }
  }

  isAlreadyExist(id) {
    for (let i = 0; i < this.selectedDevices.length; i++) {
      if (this.selectedDevices[i].id == id) {
        return true;
      }
    }
    return false;
  }

  removeDevice(id) {
    this.selectedDevices = this.selectedDevices.filter(x => x.id !== id);
  }

  appendrightOs(os) {
    switch (os) {
      case DeviceOSEnum.Ubuntu18:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer18:
        return 'Ubuntu Server';
      case DeviceOSEnum.Macintosh:
        return 'Macintosh';
      case DeviceOSEnum.Windows:
        return 'Windows';
      case DeviceOSEnum.Ubuntu:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer:
        return 'Ubuntu Server';
      case DeviceOSEnum.WindowsServer:
        return 'Windows Server';
      case DeviceOSEnum.IOS:
        return 'iOS';
      case DeviceOSEnum.Android:
        return 'Android';
      case DeviceOSEnum.UbuntuServer20armv8:
        return 'Ubuntu Server (ARMv8)';
      case DeviceOSEnum.CentosServer:
        return 'CentOS Server';
      case 'unknown':
        return 'Unknown';
      default:
        return os;
    }
  }
}
