import { Injectable } from '@angular/core';
import { ConfigService } from '../services/config.service';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { Pagination } from '../models/pagination';
import { map } from 'rxjs/operators';

@Injectable()
export class SoftwarePolicyService {
  private policyApiUrl: string;
  constructor(
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    const baseUrl = configService.getConfiguration().apiBaseUrl;
    this.policyApiUrl = baseUrl + '/policies/software';
  }

  getAllOs(id) {
    return this.authHttp
      .get(this.policyApiUrl + '/Os' + `?id=${id}`)
      .pipe(map(res => res));
  }

  getAllPolicies(
    pagination: Pagination,
    name: string = '',
    selectedClientIds = [],
    allClientPolicies: boolean = false
  ) {
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;

    let _name: string = '';

    if (name.trim()) {
      _name = name.trim();
    }
    let _clients = selectedClientIds.join(',');
    return this.authHttp
      .get(
        this.policyApiUrl +
          `?offset=${currentPage}&limit=${itemsPerPage}&searchValue=${_name}&clients=${_clients}&allClientPolicies=${allClientPolicies}`
      )
      .pipe(
        map(res => res),
        map((res: any) => {
          return {
            ...res,
            data: (res.data || []).map(v => ({
              ...v,
              clientName: v.all_clients
                ? 'All Clients'
                : v.Clients[0] && v.Clients[0].Name
                ? v.Clients[0].Name +
                  `${
                    v.Clients.length + (v.Groups || []).length > 1 ? '...' : ''
                  }`
                : v.Groups[0] && v.Groups[0].name
                ? v.Groups[0].name + `${v.Groups.length > 1 ? '...' : ''}`
                : 'Unknown',
              serviceName: v.all_services
                ? 'All Services'
                : v.Services[0] && v.Services[0].Name
                ? v.Services[0].Name +
                  `${
                    v.Services.length + (v.ServiceGroups || []).length > 1
                      ? '...'
                      : ''
                  }`
                : v.ServiceGroups[0] && v.ServiceGroups[0].name
                ? v.ServiceGroups[0].name +
                  `${v.ServiceGroups.length > 1 ? '...' : ''}`
                : 'Unknown',
              userName: v.all_users
                ? 'All Users'
                : v.Users && v.Users[0] && v.Users[0].Name
                ? v.Users[0].Name +
                  `${
                    v.Users.length + (v.UserGroups || []).length > 1
                      ? '...'
                      : ''
                  }`
                : v.UserGroups && v.UserGroups[0] && v.UserGroups[0].name
                ? v.UserGroups[0].name +
                  `${v.UserGroups.length > 1 ? '...' : ''}`
                : 'Unknown'
            }))
          };
        })
      );
  }

  deletePolicy(id) {
    return this.authHttp
      .delete(this.policyApiUrl + `/${id}`)
      .pipe(map(res => res));
  }

  enableSoftwarePolicy(id, value) {
    return this.authHttp
      .patch(this.policyApiUrl + `/${id}/enable/${value}`, {})
      .pipe(map(response => response));
  }

  enablePolicy(id, value) {
    return this.authHttp
      .patch(this.policyApiUrl + `/${id}/enable/${value}`, {})
      .pipe(map(res => res));
  }

  getSinglePolicy(id) {
    return this.authHttp
      .get(`${this.policyApiUrl}/${id}`)
      .pipe(map(res => res));
  }

  createService(policyOption) {
    return this.authHttp
      .post(`${this.policyApiUrl}/create`, policyOption)
      .pipe(map(response => response));
  }

  updateService(policyOption, id) {
    return this.authHttp.put(`${this.policyApiUrl}/${id}`, policyOption);
  }
}
