import { Component, OnInit } from '@angular/core';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { Router, ActivatedRoute } from '@angular/router';
import { SoftwareProcessBindingService } from '../software-process-binding/software-process-binding.service';
import { OsPolicyService } from '../create-os-policy/os-policy.service';
import { DeviceOSEnum } from '../shared/emuns/client.enum';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-software-process-binding',
  templateUrl: './create-software-process-binding.component.html',
  styleUrls: ['./create-software-process-binding.component.css']
})
export class CreateSoftwareProcessBindingComponent implements OnInit {
  viewMode = false;
  inputValue: string;
  errorMessage = false;
  isEdr: boolean = false;

  client = {
    softwareName: '',
    device_os: '',
    processList: '',
    is_edr: false
  };

  deviceOs = [];

  clientAllOs = [
    { value: DeviceOSEnum.Ubuntu18, display: 'Ubuntu Desktop' },
    { value: DeviceOSEnum.UbuntuServer18, display: 'Ubuntu Server' },
    {
      value: DeviceOSEnum.UbuntuServer20armv8,
      display: 'Ubuntu Server (ARMv8)'
    },
    { value: DeviceOSEnum.Windows6_1, display: 'Windows 7' },
    { value: DeviceOSEnum.Windows6_2, display: 'Windows 8' },
    { value: DeviceOSEnum.Windows6_3, display: 'Windows 8.1' },
    { value: DeviceOSEnum.Windows10, display: 'Windows 10' },
    { value: DeviceOSEnum.Windows11, display: 'Windows 11' },
    { value: DeviceOSEnum.WindowsServer6_1, display: 'Windows Server 2008 R2' },
    { value: DeviceOSEnum.WindowsServer6_2, display: 'Windows Server 2012' },
    { value: DeviceOSEnum.WindowsServer6_3, display: 'Windows Server 2012 R2' },
    { value: DeviceOSEnum.WindowsServer10, display: 'Windows Server 2016+' },
    { value: DeviceOSEnum.CentosServer7, display: 'CentOS 7 Server' },
    { value: DeviceOSEnum.CentosServer8, display: 'CentOS 8 Server' },
    { value: DeviceOSEnum.Macintosh10_14, display: 'macOS Mojave' },
    { value: DeviceOSEnum.Macintosh10_15, display: 'macOS Catalina' },
    { value: DeviceOSEnum.Macintosh11, display: 'macOS Big Sur' },
    { value: DeviceOSEnum.Macintosh12, display: 'macOS Monterey' },
    { value: DeviceOSEnum.Macintosh13, display: 'macOS Ventura' },
    { value: DeviceOSEnum.Macintosh14, display: 'macOS Sonoma' },
    { value: DeviceOSEnum.IOS, display: 'iOS' },
    { value: DeviceOSEnum.Android, display: 'Android' },
    { value: DeviceOSEnum.WindowsServer, display: 'Windows Server' }
  ];

  clientMajorOS = [
    { value: DeviceOSEnum.Windows, display: 'Windows' },
    { value: DeviceOSEnum.Linux, display: 'Linux' },
    { value: DeviceOSEnum.MacOS, display: 'macOS' },
    { value: DeviceOSEnum.Android, display: 'Android' },
    { value: DeviceOSEnum.IOS, display: 'iOS' }
  ];

  title;
  isEditing = false;
  editingId = '';
  loading: Boolean = false;
  disableSaveButton = false;
  patternError = false;
  constructor(
    private osService: OsPolicyService,
    private SoftwareService: SoftwareProcessBindingService,
    public router: Router,
    private globalMessageService: GlobalMessageService,
    private activatedRoute: ActivatedRoute,
    private titleService: Title
  ) {}

  async ngOnInit() {
    this.disableSaveButton = false;
    // this.viewMode = this.router.url.includes('/view-software')
    const id = this.activatedRoute.snapshot.params.id;
    if (!id) {
      //create
      this.title = 'Bind';
    } else {
      //edit
      this.title = 'Edit Bind';
      this.isEditing = true;
      this.editingId = id;
      this.getDetails();
    }
    this.titleService.setTitle(
      'Device Health Check | Process Binding | ' +
        this.title +
        ' Software Processes'
    );
  }

  validateLength(input: string) {
    const words: string[] = [];

    // If there are commas, split the input string
    if (input.includes(',')) {
      words.push(...input.split(','));
    } else {
      // If no commas, consider the entire input as a single word
      words.push(input);
    }

    this.patternError = false; // Reset error flag

    // const pattern = /^(?:[\/\\,]?[a-zA-Z0-9\s_.\-]+)*[\/\\,]?[a-zA-Z0-9\s_.\-]+$/;
    // const pattern = /^(?:\/?[a-zA-Z0-9\s_.\-]+)+\/?$/;
    const pattern = /^(?:\/?[a-zA-Z0-9\s_.\-]+(?:\/[a-zA-Z0-9\s_.\-]+)*)\/?$/;

    console.log('words :>> ', words);
    this.patternError = false;

    words.forEach(word => {
      if (!pattern.test(word.trim()) || word.trim().match(/[\/\\]$/)) {
        this.patternError = true;
        return;
      }

      if (word.length > 1024) {
        this.errorMessage = true;
        return;
      }
    });

    if (input.length == 0) {
      this.patternError = false;
    }

    if (this.errorMessage) {
      this.errorMessage = true;
      document.getElementById('processList').style.border = '1px solid #a94442';
      document.getElementById('processList').style.borderLeft =
        '5px solid #a94442';
      document.getElementById('iconn').style.border = '1px solid #a94442';
      document.getElementById('iconn').style.backgroundColor = '#f2dede';
      return;
    } else {
      this.errorMessage = false;
      document.getElementById('processList').style.borderLeft = '';
      document.getElementById('processList').style.border = '';
      document.getElementById('iconn').style.border = '';
      document.getElementById('iconn').style.backgroundColor = '';
    }
  }

  getDetails() {
    this.loading = true;
    this.SoftwareService.getSoftware(this.editingId).subscribe(
      (data: any) => {
        this.loading = false;
        const { os, software, ProcessesList, is_edr } = data;
        this.isEdr = is_edr;
        this.client = {
          device_os: os,
          softwareName: software,
          processList: ProcessesList,
          is_edr: is_edr
        };
      },
      err => {
        this.loading = false;
      }
    );
  }

  submitHandler() {
    const { softwareName, device_os, processList } = this.client;
    this.loading = true;
    this.disableSaveButton = true;
    const software = {
      software: softwareName,
      os: device_os,
      process_list: processList,
      is_edr: this.isEdr
    };

    if (!this.isEditing) {
      this.SoftwareService.addSoftware(software, this.isEditing).subscribe(
        data => {
          this.loading = false;
          this.showAlert('success', 'Software Processes Added', false);
          this.router.navigate(['/software_process_binding']);
        },
        error => {
          this.disableSaveButton = false;
          this.loading = false;
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'danger',
            message: error.error
          });
        }
      );
    } else {
      this.SoftwareService.addSoftware(software, this.editingId).subscribe(
        data => {
          this.loading = false;
          this.showAlert('success', 'Software Processes Updated', false);
          this.router.navigate(['/software_process_binding']);
        },
        error => {
          this.disableSaveButton = false;
          this.loading = false;
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'danger',
            message: error.error
          });
        }
      );
    }
  }

  private showAlert(type: string, message: string, sticky: boolean) {
    const alert = {
      closed: false,
      type: type,
      message: message,
      sticky: sticky
    };
    this.globalMessageService.pushAlert(alert);
  }

  edrToggle() {
    this.isEdr = !this.isEdr;
    if (this.client.device_os) {
      this.client.device_os = null;
    }
  }

  navigate(title) {}
}
