import { Component, OnInit } from '@angular/core';
import { SoftwareProcessPolicyService } from './software-process-policy.service';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { Router, ActivatedRoute } from '@angular/router';
import { Subject, forkJoin } from 'rxjs';
import {
  catchError,
  debounceTime,
  distinctUntilChanged,
  map,
  switchMap
} from 'rxjs/operators';
import { AccessPolicyService } from '../create-access-policy/access-policy.service';
import { DeviceOSEnum } from '../shared/emuns/client.enum';
import { APIDataTypeEnum } from '../shared/emuns/global.enum';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-software-process-policy',
  templateUrl: './create-software-process-policy.component.html',
  styleUrls: ['./create-software-process-policy.component.css']
})
export class CreateSoftwareProcessPolicyComponent implements OnInit {
  showHelpText: boolean = false;
  policyName;
  spaces = 0;
  selectedSoftwares = [];
  compliances = ['All', 'Any'];
  S;
  DataType = APIDataTypeEnum;

  users = [];
  $userSearch = new Subject();
  isUserSearching = true;
  selectedUserIds = [];
  allUsersSelected = false;
  selectedUserCount = 0;
  usersPlaceHolder = 'Search User or User Group';
  currentUserSearch = '';
  userOffset = 0;
  selectedUserTypes = [];
  user = [];
  userGroup = [];

  client = {
    name: '',
    device_os: '',
    is_blocked: false,
    compliance: 'Any'
  };

  windowsOsVersion = [
    { value: '6.1.7601', display: 'Windows 7' },
    { value: '6.2.9200', display: 'Windows 8' },
    { value: '6.3.9600', display: 'Windows 8.1' },
    { value: '10.0.18363', display: 'Windows 10' },
    { value: '10.0.22000', display: 'Windows 11' }
  ];

  macOsVersions = [
    { value: '10.15.4', display: 'macOS Catalina' },
    { value: '10.14.6', display: 'macOS Mojave' },
    { value: '10.13.6', display: 'macOS High Sierra' },
    { value: '11', display: 'macOS Big Sur' },
    { value: '12', display: 'macOS Monterey' },
    { value: '13', display: 'macOS Ventura' }
  ];

  ubuntu18OsVersions = [
    { value: '18.04.4', display: '' },
    { value: '18.04.3', display: '' },
    { value: '18.04.2', display: '' },
    { value: '18.04.1', display: '' },
    { value: '18.04', display: '' }
  ];

  ubuntu16OsVersions = [
    { value: '16.04.6', display: '' },
    { value: '16.04.5', display: '' },
    { value: '16.04.4', display: '' },
    { value: '16.04.3', display: '' },
    { value: '16.04.2', display: '' },
    { value: '16.04.1', display: '' },
    { value: '16.04', display: '' }
  ];

  centosserv8OsVersions = [
    { value: '8.3', display: '' },
    { value: '8.2', display: '' },
    { value: '8.1', display: '' },
    { value: '8.0', display: '' }
  ];

  clientAllOs = [
    { value: DeviceOSEnum.Ubuntu, display: 'Ubuntu Desktop' },
    { value: DeviceOSEnum.UbuntuServer, display: 'Ubuntu Server' },
    { value: DeviceOSEnum.Ubuntu18, display: 'Ubuntu 18 Desktop' },
    { value: DeviceOSEnum.UbuntuServer18, display: 'Ubuntu 18 Server' },
    { value: DeviceOSEnum.Windows, display: 'Windows' },
    { value: DeviceOSEnum.Macintosh, display: 'macOS' },
    { value: DeviceOSEnum.CentosServer, display: 'CentOS Server' }
  ];

  policies = [
    { value: false, display: 'Warn Only' },
    { value: true, display: 'Block' }
  ];

  deviceOs = [];

  clientOsVersion = [];
  title;
  isEditing = false;
  editingId = '';

  clients = [];
  services = [];
  softwares = [];
  selectedClientIds = [];
  selectedServiceIds = [];
  selectedClientTypes = [];
  selectedServiceTypes = [];
  selectedClientCount = 0;
  selectedServiceCount = 0;
  clientOffset = 0;
  serviceOffset = 0;
  currentClientSearch = '';
  currentServiceSearch = '';
  $clientSearch = new Subject();
  $serviceSearch = new Subject();
  isClientSearching = false;
  isServiceSearching = false;
  currentEditingId;
  clientsPlaceHolder = 'Search Client or Client Group';
  servicesPlaceHolder = 'Search Service or Service Group';
  selectSoftwarePlaceHolder = 'Select Software';
  allClientsSelected = false;
  allServicesSelected = false;
  loading: boolean = false;
  riskScoreThreshold: number = 1;
  disableSaveButton = false;

  _client = [];
  clientGroup = [];
  service = [];
  serviceGroup = [];

  shouldDisableClientInput = false;
  shouldDisableUserInput = false;
  isDarkMode : boolean;

  constructor(
    private softwareProcessPolicyService: SoftwareProcessPolicyService,
    public router: Router,
    private globalMessageService: GlobalMessageService,
    private activatedRoute: ActivatedRoute,
    private accessService: AccessPolicyService,
    public dialog: MatDialog,
    private titleService: Title
  ) {
    this.client.is_blocked = this.policies[0].value;
  }

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.disableSaveButton = false;
    this.activatedRoute.params.subscribe(data => {
      if (!data.id) {
        //create
        this.title = 'Add';
        this.getAllDropDownsData();
      } else {
        //edit
        this.title = 'Edit';
        this.isEditing = true;
        this.editingId = data.id;
        this.getDetails();
      }
      this.titleService.setTitle(
        'Policy Management | Device Policy | Process Policy | ' +
          this.title +
          '  Running Process Policy'
      );
    });

    this.getAllOs();
    this.subscribeClientSearch();
    this.subscribeServiceSearch();
    this.buttonShouldBeDisabled();
  }

  private getAllOs() {
    this.deviceOs = this.clientAllOs;
  }

  getDetails() {
    this.loading = true;
    this.softwareProcessPolicyService.getSinglePolicy(this.editingId).subscribe(
      (data: any) => {
        this.loading = false;
        const { os, is_blocked, name, compliance } = data;
        this.client = {
          name: name,
          device_os: os,
          is_blocked: is_blocked,
          compliance: compliance
        };
        this.policyName = name;
        // this.compliance = compliance
        this.deviceOsSelected(false);
        this.parseAllEditData(data);
      },
      err => {
        this.loading = false;
      }
    );
  }

  deviceOsSelected(fromTemplate = true): void {
    const { device_os } = this.client;
    if (device_os === DeviceOSEnum.Windows) {
      this.clientOsVersion = this.windowsOsVersion;
    } else if (device_os === DeviceOSEnum.Macintosh) {
      this.clientOsVersion = this.macOsVersions;
    } else if (device_os === DeviceOSEnum.CentosServer) {
      this.clientOsVersion = this.centosserv8OsVersions;
    } else {
      if (
        device_os === DeviceOSEnum.Ubuntu ||
        device_os === DeviceOSEnum.UbuntuServer
      ) {
        this.clientOsVersion = this.ubuntu16OsVersions;
      } else {
        this.clientOsVersion = this.ubuntu18OsVersions;
      }
    }
  }

  submitHandler() {
    this.loading = true;
    this.parseAllSelectedClients();
    this.parseAllSelectedUser();
    const { is_blocked, compliance } = this.client;
    const policy = {
      name: this.policyName,
      selectedSoftwares: this.selectedSoftwares,
      is_blocked,
      compliance,
      client: this._client,
      clientGroup: this.clientGroup,
      service: this.service,
      serviceGroup: this.serviceGroup,
      all_clients: this.allClientsSelected,
      all_services: this.allServicesSelected,
      user: this.user,
      userGroup: this.userGroup,
      all_users: this.allUsersSelected
    };
    this.disableSaveButton = true;
    if (!this.isEditing) {
      this.softwareProcessPolicyService.createService(policy).subscribe(
        data => {
          this.loading = false;
          this.showAlert('success', 'Running Process Policy Created', false);
          this.router.navigate(['/software-process-policy']);
        },
        error => {
          this.disableSaveButton = false;
          this.loading = false;
        }
      );
    } else {
      this.softwareProcessPolicyService
        .updateService(policy, this.editingId)
        .subscribe(
          data => {
            this.loading = false;
            this.showAlert('success', 'Running Process Policy Updated', false);
            this.router.navigate(['/software-process-policy']);
          },
          error => {
            this.disableSaveButton = false;
            this.loading = false;
          }
        );
    }
  }

  private showAlert(type: string, message: string, sticky: boolean) {
    const alert = {
      closed: false,
      type: type,
      message: message,
      sticky: sticky
    };
    this.globalMessageService.pushAlert(alert);
  }

  clientChange() {
    if (this.selectedClientIds.length >= this.selectedClientCount) {
      this.selectedClientCount = this.selectedClientCount + 1;
      const lastId = this.selectedClientIds[this.selectedClientIds.length - 1];
      const client = this.clients.find(item => item.id === lastId);
      this.selectedClientTypes = [...this.selectedClientTypes, client];
    } else {
      this.selectedClientCount = this.selectedClientIds.length;
      this.selectedClientTypes = this.selectedClientTypes.filter(client => {
        return this.selectedClientIds.indexOf(client.id) !== -1;
      });
    }
    this.buttonShouldBeDisabled();
  }

  serviceChange() {
    if (this.selectedServiceIds.length >= this.selectedServiceCount) {
      this.selectedServiceCount = this.selectedServiceCount + 1;
      const lastId = this.selectedServiceIds[
        this.selectedServiceIds.length - 1
      ];
      const client = this.services.find(item => item.id === lastId);
      this.selectedServiceTypes = [...this.selectedServiceTypes, client];
    } else {
      this.selectedServiceCount = this.selectedServiceIds.length;
      this.selectedServiceTypes = this.selectedServiceTypes.filter(client => {
        return this.selectedServiceIds.indexOf(client.id) !== -1;
      });
    }
  }

  clientFocus() {
    this.clientOffset = 0;
    if (this.currentClientSearch) {
      this.currentClientSearch = '';
      this.$clientSearch.next('');
    } else {
      this.currentClientSearch = '';
    }
  }

  serviceFocus() {
    this.serviceOffset = 0;
    if (this.currentServiceSearch) {
      this.currentServiceSearch = '';
      this.$serviceSearch.next('');
    } else {
      this.currentServiceSearch = '';
    }
  }

  fetchMoreServices() {
    this.serviceOffset += 1;
    this.accessService
      .getAllDropDownData(
        this.clientOffset,
        this.serviceOffset,
        this.currentClientSearch,
        this.currentServiceSearch,
        this.currentEditingId,
        '',
        true
      )
      .subscribe((data: any) => {
        const serviceData = data.filter(
          d =>
            (d.type === APIDataTypeEnum.Service ||
              d.type === APIDataTypeEnum.ServiceGroup) &&
            !this.selectedServiceIds.includes(d.id)
        );
        serviceData.forEach(s => (this.services = [...this.services, s]));
      });
  }

  fetchMoreClients() {
    this.clientOffset += 1;
    this.accessService
      .getAllDropDownData(
        this.clientOffset,
        this.serviceOffset,
        this.currentClientSearch,
        this.currentServiceSearch,
        this.currentEditingId,
        '',
        true
      )
      .subscribe((data: any) => {
        const clientData = data.filter(
          d =>
            (d.type === APIDataTypeEnum.Client ||
              d.type === APIDataTypeEnum.ClientGroup) &&
            !this.selectedClientIds.includes(d.id)
        );
        if (this.spaces < 5 && clientData.length === 0) {
          this.spaces += 1;
          this.clients = [
            ...this.clients,
            { type: 'clientGroup', id: '', name: '', disabled: true }
          ];
        }
        clientData.forEach(c => (this.clients = [...this.clients, c]));
      });
  }

  selectAllClients() {
    this.selectedClientIds = null;
    this.selectedClientCount = 0;
    this.allClientsSelected = !this.allClientsSelected;
    this.clientsPlaceHolder = !this.allClientsSelected
      ? 'Search Client or Client Group'
      : 'All Clients are selected';
    this.buttonShouldBeDisabled();
  }

  selectAllServices() {
    this.selectedServiceIds = null;
    this.selectedServiceCount = 0;
    this.allServicesSelected = !this.allServicesSelected;
    this.servicesPlaceHolder = !this.allServicesSelected
      ? 'Search Service or Service Group'
      : 'All services are selected';
  }

  selectAllServicesClick(selectall) {
    if (selectall) {
      if (this.selectedServiceCount > 0) {
        const dialogRef = this.dialog.open(ConfirmModalComponent, {
          panelClass: this.isDarkMode ? 'dark-theme' : '',
          data: {
            title: 'Confirmation',
            message: `Are you sure you want to select all Services?`
          }
        });

        dialogRef.afterClosed().subscribe(result => {
          if (result) {
            this.selectAllServices();
          }
        });
      } else {
        this.selectAllServices();
      }
    } else {
      this.selectAllServices();
    }
  }

  selectAllClientsClick(selectall) {
    if (
      (!this.selectedUserIds || this.selectedUserIds?.length <= 0) &&
      !this.allUsersSelected
    ) {
      if (selectall) {
        if (this.selectedClientCount > 0) {
          const dialogRef = this.dialog.open(ConfirmModalComponent, {
            panelClass: this.isDarkMode ? 'dark-theme' : '',
            data: {
              title: 'Confirmation',
              message: `Are you sure you want to select all Clients?`
            }
          });

          dialogRef.afterClosed().subscribe(result => {
            if (result) {
              this.selectAllClients();
            }
          });
        } else {
          this.selectAllClients();
        }
      } else {
        this.selectAllClients();
      }
    }
    this.buttonShouldBeDisabled();
  }

  private subscribeClientSearch() {
    this.$clientSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.isClientSearching = true;
          this.spaces = 0;
          this.currentClientSearch = term;
          this.clientOffset = 0;
          return this.accessService.getAllDropDownData(
            this.clientOffset,
            this.serviceOffset,
            this.currentClientSearch,
            this.currentServiceSearch,
            this.currentEditingId,
            '',
            true
          );
        })
      )
      .subscribe((data: any) => {
        this.isClientSearching = false;
        this.clients = data.filter(
          d =>
            d.type === APIDataTypeEnum.Client ||
            d.type === APIDataTypeEnum.ClientGroup
        );
      });
  }

  private subscribeServiceSearch() {
    this.$serviceSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.serviceOffset = 0;
          this.isServiceSearching = true;
          this.currentServiceSearch = term;
          return this.accessService.getAllDropDownData(
            this.clientOffset,
            this.serviceOffset,
            this.currentClientSearch,
            this.currentServiceSearch,
            this.currentEditingId,
            '',
            true
          );
        })
      )
      .subscribe((data: any) => {
        this.isServiceSearching = false;
        this.services = data.filter(
          d =>
            d.type === APIDataTypeEnum.Service ||
            d.type === APIDataTypeEnum.ServiceGroup
        );
      });
  }

  private parseAllEditData(data) {
    if (data.all_clients) {
      this.selectAllClients();
    }
    if (data.all_services) {
      this.selectAllServices();
    }
    this.selectedClientIds = [
      ...data.Clients.map(client => client.id),
      ...data.Groups.map(g => g.id)
    ];
    this.selectedClientTypes = [
      ...data.Clients.map(({ id, Name, device_os }) => {
        return {
          id,
          name: Name,
          type: 'client',
          device_os
        };
      }),
      ...data.Groups.map(({ id, name }) => {
        return {
          id,
          name,
          type: 'clientGroup'
        };
      })
    ];

    this.selectedServiceIds = [
      ...data.Services.map(client => client.id),
      ...data.ServiceGroups.map(g => g.id)
    ];
    this.selectedServiceTypes = [
      ...data.Services.map(({ id, Name }) => {
        return {
          id,
          name: Name,
          type: 'service'
        };
      }),
      ...data.ServiceGroups.map(({ id, name }) => {
        return {
          id,
          name,
          type: 'serviceGroup'
        };
      })
    ];

    this.selectedSoftwares = [
      ...data.SoftwareInstances.map(software => software.id)
    ];

    this.selectedClientCount = this.selectedClientIds.length;
    this.selectedServiceCount = this.selectedServiceIds.length;

    this.clients = [...this.clients, ...this.selectedClientTypes];
    this.services = [...this.services, ...this.selectedServiceTypes];
    this.softwares = [
      ...this.softwares,
      ...[
        ...data.SoftwareInstances.map(({ id, binding_name }) => {
          return {
            id,
            binding_name
          };
        })
      ]
    ];

    this.selectedUserIds = [
      ...data.Users.map(user => user.runtime_bind_id),
      ...data.UserGroups.map(g => g.runtime_bind_id)
    ];

    this.selectedUserTypes = [
      ...data.Users.map(({ id, Name, runtime_bind_id }) => {
        return {
          id,
          name: Name,
          type: 'user',
          runtime_bind_id
        };
      }),
      ...data.UserGroups.map(({ id, name, runtime_bind_id }) => {
        return {
          id,
          name,
          type: 'userGroup',
          runtime_bind_id
        };
      })
    ];

    this.selectedUserCount = this.selectedUserIds.length;
    this.users = [...this.users, ...this.selectedUserTypes];

    this.getAllDropDownsData();
  }

  private parseAllSelectedClients() {
    if (!this.allClientsSelected) {
      this._client = this.selectedClientTypes
        .filter(client => {
          return client.type === APIDataTypeEnum.Client;
        })
        .map(c => c.id);
      this.clientGroup = this.selectedClientTypes
        .filter(client => {
          return client.type === APIDataTypeEnum.ClientGroup;
        })
        .map(cg => cg.id);
    }
    if (!this.allServicesSelected) {
      this.service = this.selectedServiceTypes
        .filter(s => s.type === APIDataTypeEnum.Service)
        .map(s => s.id);
      this.serviceGroup = this.selectedServiceTypes
        .filter(s => s.type === APIDataTypeEnum.ServiceGroup)
        .map(s => s.id);
    }
  }

  getAllDropDownsData() {
    this.softwareProcessPolicyService
      .getSoftwaresProcessBindingList()
      .subscribe((data: any) => {
        const softwareData = data.filter(
          d => !this.selectedSoftwares.includes(d.id)
        );
        softwareData.forEach(c => (this.softwares = [...this.softwares, c]));
        // this.softwares = response;
      });
    this.accessService
      .getAllDropDownDataWithoutPagination(
        this.clientOffset,
        this.serviceOffset,
        this.currentClientSearch,
        this.currentServiceSearch,
        this.currentEditingId,
        ''
      )
      .subscribe((data: any) => {
        const clientData = data.filter(
          d =>
            (d.type === APIDataTypeEnum.Client ||
              d.type === APIDataTypeEnum.ClientGroup) &&
            !this.selectedClientIds.includes(d.id)
        );
        clientData.forEach(c => (this.clients = [...this.clients, c]));
        this.buttonShouldBeDisabled();

        const serviceData = data.filter(
          d =>
            (d.type === APIDataTypeEnum.Service ||
              d.type === APIDataTypeEnum.ServiceGroup) &&
            !this.selectedServiceIds.includes(d.id)
        );
        serviceData.forEach(s => (this.services = [...this.services, s]));
      });

    this.isUserSearching = true;
    forkJoin([this.accessService.getUserDropDown()])
      .pipe(
        map((response: any) => {
          console.log(response);
          return response[0]?.data;
        }),
        catchError((error: any) => {
          // Handle errors here
          return error;
        })
      )
      .subscribe({
        next: data => {
          this.users = data;
          this.isUserSearching = false;

          console.log(data);
        },
        error: error =>
          catchError((error: any) => {
            // Handle errors here
            return error;
          })
      });
  }

  // User Section Started

  selectAllUsers() {
    this.selectedUserIds = null;
    this.selectedUserCount = 0;
    this.allUsersSelected = !this.allUsersSelected;
    this.usersPlaceHolder = !this.allUsersSelected
      ? 'Search User or User Group'
      : 'All Users are selected';
    this.buttonShouldBeDisabled();
  }

  selectAllUserClick(selectall) {
    if (
      (!this.selectedClientIds || this.selectedClientIds?.length <= 0) &&
      !this.allClientsSelected
    ) {
      if (selectall) {
        if (this.selectedClientCount > 0) {
          const dialogRef = this.dialog.open(ConfirmModalComponent, {
            panelClass: this.isDarkMode ? 'dark-theme' : '',
            data: {
              title: 'Confirmation',
              message: 'Are you sure you want to select all User?'
            }
          });

          dialogRef.afterClosed().subscribe(result => {
            if (result) {
              this.selectAllUsers();
            }
          });
        } else {
          this.selectAllUsers();
        }
      } else {
        this.selectAllUsers();
      }
    }
    this.buttonShouldBeDisabled();
  }

  userFocus() {
    this.userOffset = 0;
    if (this.currentServiceSearch) {
      this.currentServiceSearch = '';
      this.$userSearch.next('');
    } else {
      this.currentServiceSearch = '';
    }
  }

  userChange() {
    if (this.selectedUserIds.length >= this.selectedUserCount) {
      this.selectedUserCount = this.selectedUserCount + 1;
      const lastId = this.selectedUserIds[this.selectedUserIds.length - 1];
      const user = this.users.find(item => item.runtime_bind_id === lastId);
      this.selectedUserTypes = [...this.selectedUserTypes, user];
    } else {
      this.selectedUserCount = this.selectedUserIds.length;
      this.selectedUserTypes = this.selectedUserTypes.filter(user => {
        return this.selectedUserIds.indexOf(user.runtime_bind_id) !== -1;
      });
    }
    this.buttonShouldBeDisabled();
  }

  private parseAllSelectedUser() {
    if (!this.allUsersSelected) {
      this.user = this.selectedUserTypes
        .filter(u => u.type === APIDataTypeEnum.User)
        .map(u => u.id);
      this.userGroup = this.selectedUserTypes
        .filter(u => u.type === APIDataTypeEnum.UserGroup)
        .map(u => u.id);
    }
  }

  // User Section Ended

  getOsName(os) {
    switch (os) {
      case DeviceOSEnum.Ubuntu18:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer18:
        return 'Ubuntu Server';
      case DeviceOSEnum.Macintosh:
        return 'Macintosh';
      case DeviceOSEnum.Windows:
        return 'Windows';
      case DeviceOSEnum.Ubuntu:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer:
        return 'Ubuntu Server';
      case DeviceOSEnum.WindowsServer:
        return 'Windows Server';
      case DeviceOSEnum.IOS:
        return 'iOS';
      case DeviceOSEnum.Android:
        return 'Android';
      case DeviceOSEnum.UbuntuServer20armv8:
        return 'Ubuntu Server (ARMv8)';
      case DeviceOSEnum.CentosServer:
        return 'CentOS Server';
      case 'unknown':
        return 'Unknown';
      default:
        return os;
    }
  }

  buttonShouldBeDisabled() {
    if (this.selectedUserIds?.length > 0 || this.allUsersSelected) {
      this.shouldDisableClientInput = true;
      this.shouldDisableUserInput = false;
    } else if (this.selectedClientIds?.length > 0 || this.allClientsSelected) {
      this.shouldDisableUserInput = true;
      this.shouldDisableClientInput = false;
    } else {
      this.shouldDisableUserInput = false;
      this.shouldDisableClientInput = false;
    } 
  }
}
