import { Component, OnInit } from '@angular/core';
// import { OsPolicyServiceSoftware } from './os-software.service';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { Router, ActivatedRoute } from '@angular/router';
// import { OsService } from '../common/os-service.service';
import { SoftwareService } from '../softwares-list/software.service';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-software',
  templateUrl: './create-software.component.html',
  styleUrls: ['./create-software.component.css']
})
export class CreateSoftwareComponent implements OnInit {
  viewMode = false;

  client = {
    softwareName: '',
    softwareVersion: '',
    softwarePublisher: '',
    device_os: '',
    client_os_version: 'any',
    osComment: ''
  };

  softwareOs = [
    'macOS',
    'Microsoft Windows 10 Pro',
    'Ubuntu',
    'Ubuntu 18.04',
    'Ubuntu 18.04.6 LTS',
    'Ubuntu 20.04',
    'iOS',
    'Android'
  ];

  clientOsVersion = [];
  title;
  isEditing = false;
  editingId = '';
  loading: Boolean = false;
  disableSaveButton = false;

  constructor(
    // private osService: OsPolicyServiceSoftware,
    private SoftwareService: SoftwareService,
    public router: Router,
    private globalMessageService: GlobalMessageService,
    private activatedRoute: ActivatedRoute, // private osVersionService: OsService,
    private titleService: Title
  ) {}

  async ngOnInit() {
    this.disableSaveButton = false;
    // const osVersions = await this.osVersionService.getOsVersions().toPromise();
    // this.updateOSVersionArrays(osVersions);
    // this.viewMode = this.router.url.includes('/view-software')
    const id = this.activatedRoute.snapshot.params.id;
    if (!id) {
      //create
      this.title = 'Add';
    } else {
      //edit
      this.title = 'Edit';
      this.isEditing = true;
      this.editingId = id;
      this.getDetails();
    }
    this.titleService.setTitle('Device Health Check | Software Master List | ' + this.title + ' Software');
  }

  getDetails() {
    this.loading = true;
    this.SoftwareService.getSoftware(this.editingId).subscribe(
      (data: any) => {
        this.loading = false;
        const {
          os,
          os_version,
          software_name,
          software_version,
          software_publisher,
          os_comments
        } = data;
        this.client = {
          device_os: os,
          client_os_version: os_version,
          softwareName: software_name,
          softwareVersion: software_version,
          softwarePublisher: software_publisher,
          osComment: os_comments
        };
      },
      err => {
        this.loading = false;
      }
    );
  }

  submitHandler() {
    const {
      softwareName,
      softwareVersion,
      softwarePublisher,
      device_os,
      client_os_version,
      osComment
    } = this.client;
    if (this.softwareNameValid(softwareName) == false) {
      this.showAlert('danger', 'Invalid Software Name', false);
      return;
    }
    this.loading = true;
    this.disableSaveButton = true;
    const software = {
      software_name: softwareName,
      software_version: softwareVersion,
      software_publisher: softwarePublisher,
      os: device_os,
      os_version: client_os_version,
      os_comments: osComment
    };
    if (!this.isEditing) {
      this.SoftwareService.addSoftware(software, this.isEditing).subscribe(
        data => {
          this.loading = false;
          this.showAlert('success', 'Software Created', false);
          this.router.navigate(['/softwares']);
        },
        error => {
          this.disableSaveButton = false;
          this.loading = false;
        }
      );
    } else {
      this.SoftwareService.addSoftware(software, this.editingId).subscribe(
        data => {
          this.loading = false;
          this.showAlert('success', 'Software Updated', false);
          this.router.navigate(['/softwares']);
        },
        error => {
          this.disableSaveButton = false;
          this.loading = false;
        }
      );
    }
  }

  softwareNameValid(softwareName: string) {
    let alphaNumeric =
      'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    let nameValid = false;
    for (let j = 0; j < softwareName.length; j++) {
      nameValid = nameValid || alphaNumeric.indexOf(softwareName[j]) != -1;
    }
    return nameValid;
  }

  private showAlert(type: string, message: string, sticky: boolean) {
    const alert = {
      closed: false,
      type: type,
      message: message,
      sticky: sticky
    };
    this.globalMessageService.pushAlert(alert);
  }

  navigate(title:String){}
}
