import { AmChart, AmChartsService } from '@amcharts/amcharts3-angular';
import {
  ChangeDetectorRef,
  Component,
  ElementRef,
  OnInit,
  ViewChild
} from '@angular/core';
import { Router } from '@angular/router';
import { Subscription, timer, zip } from 'rxjs';
import { ClientService } from '../client/client.service';
import { ManageService } from '../manage-service/manage.service';
import { GatewayService } from '../gateway/gateway.service';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { AuthenticationService } from '../login/authentication.service';
import { Client } from '../models/client';
import { Service } from '../models/service';
import { DashboardService } from './dashboard.service';
import { Device } from '../models/device';
import { DeviceInventoryService } from '../device-inventory/device-inventory.service';
import { first, tap } from 'rxjs/operators';
import { ErrorCodeEnum } from '../shared/emuns/error.enum';
import { ClientAccessStatusEnum } from '../shared/emuns/client.enum';
import { DeviceOSEnum } from '../shared/emuns/client.enum';
import { APIDataTypeEnum } from '../shared/emuns/global.enum';
import { LicenseErrorEnum } from '../shared/emuns/license.enum';
import { ToastrService } from 'ngx-toastr';
import { Title } from '@angular/platform-browser';
import { ThemeService } from '../theme.service';
import { LicenseService } from '../license/license.service';
import { RolesAndPermissionService } from '../roles-and-permission/roles-and-permission.service';
import { RolesData } from '../shared/roles/roles';
declare var google: any;

@Component({
  selector: 'dashboard',
  templateUrl: './dashboard.component.html',
  styleUrls: ['./dashboard.component.css']
})
export class DashboardComponent implements OnInit {
  private logtimerSubscription: Subscription;
  private alertsSubscription: Subscription;
  private timerSubscription: Subscription;
  public statusColor: Array<string> = [
    'label label-danger',
    'label label-warning',
    'label label-success bg-success',
    'label label-success',
    'label label-info'
  ];
  public alert: GlobalMessageClass;
  public pageTitle: string = 'Dashboard';
  private allowedBounds;
  public accessLogs: any = {};
  public dateFormat: string = 'dd/MM/yyyy HH:mm';
  clients: number = 0;
  ldaps: number = 1;
  clientsActive: number;
  userrole: string = '';
  mfaServerActive: number;
  otpServerStatus: number;
  contentPortalStatus: Number;
  onboardingPotalStatus: Number;
  adminsCount: number;
  servicesCount: number;
  policiesCount: number;
  alertLogsCount: number;
  gateways: number = 1;
  gatewaysActive: number;
  controllers: number = 1;
  controllersActive: number;
  countFetched: boolean = false;
  hostByOS: Object = {};
  lat: number = 51.678418;
  lng: number = 7.809007;
  isSettingsSet: boolean = true;
  public displayMapDashboard: boolean = false;
  controllerMarkers: Array<Object>;
  gatewayMarkers: Array<Object>;
  selectedGraphFilter: any = {
    selectedServicesByUser: null,
    selectedServicesByVolume: null,
    selectedUsersByServices: null,
    selectedUsersByVolume: null,
    deviceStatistics: null
  };
  private map;
  minZoom: number = 2;
  dataExist: Array<any> = [];
  private top10servicesByUser: AmChart;
  private top10servicesByVolume: AmChart;
  private devicesCounts: AmChart;
  private top10usersByServices: AmChart;
  private top10usersByVolume: AmChart;
  private totalNumberOfUserAccessedServices: AmChart;
  private deviceStatistics: AmChart;
  private totalNumberOfServiceAccessed: AmChart;
  private accessByHostOS: AmChart;
  private desktopPercentage: AmChart;
  private mobilePercentage: AmChart;
  private serverPercentage: AmChart;
  private utillizations: AmChart;
  private graphData: any;

  utilizationByClientLoading: Boolean = true;
  utilizationByServiceAccessedLoading: Boolean = true;
  accessByHostOsLoading: Boolean = true;
  devicesStatisticsLoading: Boolean = true;
  topTenServiceByClientsLoading: Boolean = true;
  topTenServicesByVolumeOfDataLoading: Boolean = true;
  devicesCountLoading: Boolean = true;
  topTenClientsByNumberOfServicesAccessed: Boolean = true;
  topTenClientsByAggregateVolumeOfData: Boolean = true;
  accessLogLoading: Boolean = true;
  utillizationLoading: boolean = true;
  secondaryControllersCount: number;
  onlineSecondaryControllers: number;
  uniqueClientsCount: number = 0;
  primaryControllerStatus: Boolean;
  selectedUtillizationType: 'client' | 'service' = APIDataTypeEnum.Client;
  graphTitle: string;
  adminDashboard: boolean = true;
  InfrastructureDashboard: boolean = false;

  onlineClients: Client[] = [];
  onlineClientsLoading: boolean = false;

  topServices: Service[] = [];
  servicesLoading: boolean = false;

  topDevices: Device[] = [];
  devicesLoading: boolean = false;

  timeZone = '';
  isDarkMode: boolean = false;
  OShashMap = {
    ubuntu: 'Ubuntu Desktop',
    ubuntuserver: 'Ubuntu Server',
    ubuntu18: 'Ubuntu Desktop',
    ubuntuserver18: 'Ubuntu Server',
    ubuntuserver20armv8: 'Ubuntu Server (ARMv8)',
    centosserver: 'CentOS Server',
    windows: 'Windows',
    windowsserver: 'Windows Server',
    macintosh: 'Macintosh',
    ios: 'iOS',
    android: 'Android',
    unknown: 'Unknown'
  };

  versions = [
    {
      label: 'Latest Version',
      value: false
    },
    {
      label: 'Old Versions',
      value: true
    }
  ];
  selectedVersion = this.versions[0];

  portalTime: string = '';
  systemTime: string = '';
  customTime: string = '';
  portalTimeLogin: string = '';
  selectedInterval: string = 'days';
  servicesSelectedInterval: string = 'month';
  clientsSelectedInterval: string = 'month';
  servicesByDataSelectedInterval: string = 'month';
  utilizationSelectedInterval : string = 'client';

  @ViewChild('daysLabel') daysLabel: any;
  @ViewChild('monthLabel') monthLabel: any;
  @ViewChild('yearLabel') yearLabel: any;

  @ViewChild('servicesWeekLabel') servicesWeekLabel: any;
  @ViewChild('servicesMonthLabel') servicesMonthLabel: any;
  @ViewChild('servicesYearLabel') servicesYearLabel: any;

  @ViewChild('clientsWeekLabel') clientsWeekLabel: any;
  @ViewChild('clientsMonthLabel') clientsMonthLabel: any;
  @ViewChild('clientsYearLabel') clientsYearLabel: any;

  @ViewChild('servicesByDataWeekLabel') servicesByDataWeekLabel: any;
  @ViewChild('servicesByDataMonthLabel') servicesByDataMonthLabel: any;
  @ViewChild('servicesByDataYearLabel') servicesByDataYearLabel: any;

  @ViewChild('utilizationServicesLabel') utilizationServicesLabel: any;
  @ViewChild('utilizationClientsLabel') utilizationClientsLabel: any;

  constructor(
    private dashboardService: DashboardService,
    private gatewaysService: GatewayService,
    private globalMessageService: GlobalMessageService,
    private clientService: ClientService,
    private manageService: ManageService,
    private devicesService: DeviceInventoryService,
    private router: Router,
    private authenticationService: AuthenticationService,
    private AmCharts: AmChartsService,
    private titleService: Title,
    public themeService: ThemeService,
    private roles: RolesAndPermissionService,
    private changeDetector: ChangeDetectorRef
  ) {
    //overlay.defaultViewContainer = vcRef;

    this.titleService.setTitle('Dashboard');
    var localDateFormat = localStorage.getItem('dateFormat');
    if (typeof localDateFormat != 'undefined' || localDateFormat != null) {
      this.dateFormat = localDateFormat;
    }

    let timeZoneOffset = new Date().getTimezoneOffset();
    this.timeZone =
      'UTC' + (timeZoneOffset > 0 ? '-0' : '+0') + (timeZoneOffset / 60) * -1;
  }

  changeState() {
    this.roles.setAssignedUserTemplates(
      JSON.parse(localStorage.getItem('Roles')) || {}
    );
  }

  moveUp() {
    this.displayMapDashboard = false;
    localStorage.setItem('displayMap', '' + this.displayMapDashboard + '');
  }

  moveDown() {
    this.displayMapDashboard = true;
    localStorage.setItem('displayMap', '' + this.displayMapDashboard + '');
  }

  private refreshData(): void {
    if (this.timerSubscription) {
      this.timerSubscription.unsubscribe();
    }
    this.subscribeToData();
  }

  private subscribeToData(): void {
    this.timerSubscription = timer(30000)
      .pipe(first())
      .subscribe(() => this.refreshData());
  }

  countOnline() {
    this.dashboardService
      .countHost()
      .pipe(
        tap((res: any) => {
          this.countFetched = true;
          if (res.status == true) {
            this.clients = res.clients;
            this.clientsActive = res.clientsActive;
            this.gateways = res.gateways;
            this.gatewaysActive = res.gatewaysActive;
            this.secondaryControllersCount = res.secondaryControllersCount;
            this.mfaServerActive = res.mfaServerStatus;
            this.controllersActive = res.controllersActive;
            this.ldaps = res.ldaps;
            this.isSettingsSet = res.isSettingsSet;
            this.contentPortalStatus = res.contentPortalStatus;
            this.onboardingPotalStatus = res.onboardingPotalStatus;
            this.adminsCount = res.adminsCount;
            this.servicesCount = res.servicesCount;
            this.policiesCount = res.policiesCount;
            this.alertLogsCount = res.alertLogsCount;
            this.onlineSecondaryControllers = res.onlineSecondaryControllers;
            this.primaryControllerStatus = res.primaryControllerStatus;
          } else {
            this.clients = 0;
            this.clientsActive = 0;
            this.gateways = 0;
            this.gatewaysActive = 0;
            this.controllers = 0;
            this.controllersActive = 0;
            this.ldaps = 0;
            this.mfaServerActive = 0;
            this.isSettingsSet = false;
            this.contentPortalStatus = 0;
            this.onboardingPotalStatus = 0;
          }
        })
      )
      .subscribe({
        error: err => {
          console.log(err && (err.message || err));
        },
        complete: () => {}
      });
  }

  public ngOnDestroy(): void {
    if (this.timerSubscription) {
      this.timerSubscription.unsubscribe();
    }
    if (this.logtimerSubscription) {
      this.logtimerSubscription.unsubscribe();
    }
    if (this.top10servicesByUser) {
      this.AmCharts.destroyChart(this.top10servicesByUser);
    }
    if (this.top10servicesByVolume) {
      this.AmCharts.destroyChart(this.top10servicesByVolume);
    }
    if (this.devicesCounts) {
      this.AmCharts.destroyChart(this.devicesCounts);
    }
    if (this.top10usersByServices) {
      this.AmCharts.destroyChart(this.top10usersByServices);
    }
    if (this.top10usersByVolume) {
      this.AmCharts.destroyChart(this.top10usersByVolume);
    }
    if (this.mobilePercentage) {
      this.AmCharts.destroyChart(this.mobilePercentage);
    }
    if (this.desktopPercentage) {
      this.AmCharts.destroyChart(this.desktopPercentage);
    }
    if (this.serverPercentage) {
      this.AmCharts.destroyChart(this.serverPercentage);
    }
    if (this.utillizations) {
      this.AmCharts.destroyChart(this.utillizations);
    }
  }

  ngOnInit() {
    
    this.themeService.isDarkMode$.subscribe((res: any) => {
      this.isDarkMode = res;
    });
    this.roles.assignedUserTemplates$.subscribe((next: any) => {
      if (next.length > 0) {
        this.adminDashboard = next.some(priv => priv.privilege_id === 1);
        this.InfrastructureDashboard = next.some(
          priv => priv.privilege_id === 62
        );
      }
    });

    this.themeService.darkModeToggled.subscribe((darkMode: boolean) => {
      this.isDarkMode = darkMode;
      // Call the method to update the chart or perform any other actions
      this.updateChart(darkMode);
      this.ngAfterViewInit();

      this.selectedInterval = 'days';
      this.servicesSelectedInterval = 'month';

      this.updateDevicesStatistics('days');
      this.updateGraphData(
        'getGraphServicesByUsers',
        'month',
        'top10servicesByUser'
      );
      this.updateGraphData(
        'getGraphUsersByVolume',
        'month',
        'top10usersByVolume'
      )
      this.updateGraphData(
        'getGraphServicesByVolume',
        'month',
        'top10servicesByVolume'
      )
      this.updateUtillizationGraph('client');

      this.toggleActiveLabel(this.selectedInterval);
      this.toggleActiveLabelServices(this.servicesSelectedInterval);
      this.toggleActiveLabelClientsByData(this.clientsSelectedInterval);
      this.toggleActiveLabelServicesByData(this.servicesByDataSelectedInterval);
      this.toggleActiveLabelUtilization(this.utilizationSelectedInterval);
    });

    this.setAmChartFont();
    this.updateUtillizationGraph(this.selectedUtillizationType);
    document.querySelector('#test');
    var displayMap = localStorage.getItem('displayMap');
    if (typeof displayMap != 'undefined' || displayMap != null) {
      if (displayMap == 'true') {
        this.displayMapDashboard = true;
      } else {
        this.displayMapDashboard = false;
      }
    } else {
      this.displayMapDashboard = false;
    }
    this.refreshData();
    // this.refreshLogData();

    this.userrole = localStorage.getItem('ur');
    if (typeof this.userrole != 'undefined' && this.userrole != null) {
      this.userrole = atob(this.userrole);
    }

    this.refreshLogs();
    this.refreshClients();
    this.refreshServices();
    this.refreshDevices();
    this.countOnline();

    this.dashboardService
      .getOS()
      .pipe(
        tap(res => {
          if (res.status == true) {
            this.hostByOS = [
              {
                name: 'Clients',
                colorByPoint: true,
                data: res.data
              }
            ];
            this.AmCharts.updateChart(this.accessByHostOS, () => {
              this.accessByHostOS.dataProvider = res.data.map(x => ({
                ...x,
                name: this.OShashMap[x.name]
              }));
              this.accessByHostOsLoading = false;
            });
          } else {
            this.hostByOS = null;
            this.accessByHostOsLoading = false;
          }
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          this.accessByHostOsLoading = false;
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'You are not authorized to access this page.'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          }
        }
      });

    this.dashboardService
      .getControllerMarkers()
      .pipe(
        tap(res => {
          if (res) {
            var i,
              x,
              index = 0;
            let marks = res as any;
            let alreadyExist = false;
            let prevIndex = -1;
            this.controllerMarkers = new Array();
            for (x in marks) {
              if (marks[x].length > 1) {
                if (this.dataExist[marks[x][0].lt + marks[x][0].ln]) {
                  prevIndex = this.dataExist[marks[x][0].lt + marks[x][0].ln];
                  alreadyExist = true;
                }
                let Cinfo = '';
                let Ginfo = '';
                let CLinfo = '';
                let controller_inner_index = 0;
                let gateway_inner_index = 0;
                let client_inner_index = 0;
                for (i in marks[x]) {
                  if (marks[x][i].controller == 1) {
                    if (controller_inner_index != 0) {
                      Cinfo += ', ';
                    }
                    if (alreadyExist) {
                      Cinfo += marks[x][i].name + ' (' + marks[x][i].ip + ')';
                    } else {
                      Cinfo += marks[x][i].name;
                    }
                    this.lat = marks[x][i].lt;
                    this.lng = marks[x][i].ln;
                    controller_inner_index++;
                  }
                  if (marks[x][i].gateway == 1) {
                    if (gateway_inner_index != 0) {
                      Ginfo += ', ';
                    }
                    if (alreadyExist) {
                      Ginfo += marks[x][i].name + ' (' + marks[x][i].ip + ')';
                    } else {
                      Ginfo += marks[x][i].name;
                    }
                    gateway_inner_index++;
                  }
                  if (marks[x][i].client == 1) {
                    if (client_inner_index != 0) {
                      CLinfo += ', ';
                    }
                    if (alreadyExist) {
                      CLinfo += marks[x][i].name + ' (' + marks[x][i].ip + ')';
                    } else {
                      CLinfo += marks[x][i].name;
                    }
                    client_inner_index++;
                  }
                }
                if (!alreadyExist) {
                  marks[x][0].ip = marks[x][0].ip;
                } else {
                  marks[x][0].ip =
                    marks[prevIndex][0].ip + ', ' + marks[x][0].ip;
                }
                if (!alreadyExist) {
                  marks[x][0].controllerInformation = Cinfo;
                  marks[x][0].gatewayInformation = Ginfo;
                  marks[x][0].clientInformation = CLinfo;
                } else {
                  if (marks[prevIndex][0].controllerInformation) {
                    if (Cinfo != '') {
                      marks[x][0].controllerInformation =
                        marks[prevIndex][0].controllerInformation +
                        ', ' +
                        Cinfo;
                    } else {
                      marks[x][0].controllerInformation =
                        marks[prevIndex][0].controllerInformation;
                    }
                  } else {
                    marks[x][0].controllerInformation = Cinfo;
                  }
                  if (marks[prevIndex][0].gatewayInformation) {
                    if (Ginfo != '') {
                      marks[x][0].gatewayInformation =
                        marks[prevIndex][0].gatewayInformation + ', ' + Ginfo;
                    } else {
                      marks[x][0].gatewayInformation =
                        marks[prevIndex][0].gatewayInformation;
                    }
                  } else {
                    marks[x][0].gatewayInformation = Ginfo;
                  }
                  if (marks[prevIndex][0].clientInformation) {
                    if (CLinfo != '') {
                      marks[x][0].clientInformation =
                        marks[prevIndex][0].clientInformation + ', ' + CLinfo;
                    } else {
                      marks[x][0].clientInformation =
                        marks[prevIndex][0].clientInformation;
                    }
                  } else {
                    marks[x][0].clientInformation = CLinfo;
                  }
                  marks[x].ip = marks[prevIndex].ip + ', ' + marks[x][i].ip;
                }
                this.controllerMarkers[index] = marks[x][0];
              } else {
                if (this.dataExist[marks[x][0].lt + marks[x][0].ln]) {
                  prevIndex = this.dataExist[marks[x][0].lt + marks[x][0].ln];
                  alreadyExist = true;
                }
                if (marks[x][0].controller == 1) {
                  if (!alreadyExist) {
                    marks[x][0].controllerInformation = marks[x][0].name;
                  } else {
                    if (marks[prevIndex][0].controllerInformation) {
                      marks[x][0].controllerInformation =
                        marks[prevIndex][0].controllerInformation +
                        ', ' +
                        marks[x][0].name +
                        ' (' +
                        marks[x][0].ip +
                        ')';
                    } else {
                      marks[x][0].controllerInformation =
                        marks[x][0].name + ' (' + marks[x][0].ip + ')';
                    }
                  }
                  this.lat = marks[x][0].lt;
                  this.lng = marks[x][0].ln;
                }
                if (alreadyExist && marks[x][0].controller != 1) {
                  if (marks[prevIndex][0].controllerInformation) {
                    marks[x][0].controllerInformation =
                      marks[prevIndex][0].controllerInformation;
                  }
                }
                if (marks[x][0].gateway == 1) {
                  if (!alreadyExist) {
                    marks[x][0].gatewayInformation = marks[x][0].name;
                  } else {
                    if (marks[prevIndex][0].gatewayInformation) {
                      marks[x][0].gatewayInformation =
                        marks[prevIndex][0].gatewayInformation +
                        ', ' +
                        marks[x][0].name +
                        ' (' +
                        marks[x][0].ip +
                        ')';
                    } else {
                      marks[x][0].gatewayInformation =
                        marks[x][0].name + ' (' + marks[x][0].ip + ')';
                    }
                  }
                }
                if (alreadyExist && marks[x][0].gateway != 1) {
                  if (marks[prevIndex][0].gatewayInformation) {
                    marks[x][0].gatewayInformation =
                      marks[prevIndex][0].gatewayInformation;
                  }
                }
                if (marks[x][0].client == 1) {
                  if (!alreadyExist) {
                    marks[x][0].clientInformation = marks[x][0].name;
                  } else {
                    if (marks[prevIndex][0].clientInformation) {
                      marks[x][0].clientInformation =
                        marks[prevIndex][0].clientInformation +
                        ', ' +
                        marks[x][0].name +
                        ' (' +
                        marks[x][0].ip +
                        ')';
                    } else {
                      marks[x][0].clientInformation =
                        marks[x][0].name + ' (' + marks[x][0].ip + ')';
                    }
                  }
                }
                if (alreadyExist && marks[x][0].client != 1) {
                  if (marks[prevIndex][0].clientInformation) {
                    marks[x][0].clientInformation =
                      marks[prevIndex][0].clientInformation;
                  }
                }
                if (!alreadyExist) {
                  marks[x][0].ip = marks[x][0].ip;
                } else {
                  marks[x][0].ip =
                    marks[prevIndex][0].ip + ', ' + marks[x][0].ip;
                }
                this.controllerMarkers[index] = marks[x][0];
              }
              alreadyExist = false;
              this.dataExist[marks[x][0].lt + marks[x][0].ln] = x;
              index++;
            }
          }
        })
      )
      .subscribe({
        error: err => {
          console.log(err && (err.message || err));
        },
        complete: () => {}
      });
  }

  toggleActiveLabel(interval: string) {
    // Toggle the active class on the labels based on the selected interval
    if (interval === 'days') {
      this.daysLabel.nativeElement.classList.add('active');
      this.monthLabel.nativeElement.classList.remove('active');
      this.yearLabel.nativeElement.classList.remove('active');
    } else if (interval === 'month') {
      this.daysLabel.nativeElement.classList.remove('active');
      this.monthLabel.nativeElement.classList.add('active');
      this.yearLabel.nativeElement.classList.remove('active');
    } else if (interval === 'year') {
      this.daysLabel.nativeElement.classList.remove('active');
      this.monthLabel.nativeElement.classList.remove('active');
      this.yearLabel.nativeElement.classList.add('active');
    }
  }

  toggleActiveLabelServices(interval: string) {
    // Toggle the active class on the labels based on the selected interval
    if (interval === 'week') {
      this.servicesWeekLabel.nativeElement.classList.add('active');
      this.servicesMonthLabel.nativeElement.classList.remove('active');
      this.servicesYearLabel.nativeElement.classList.remove('active');
    } else if (interval === 'month') {
      this.servicesWeekLabel.nativeElement.classList.remove('active');
      this.servicesMonthLabel.nativeElement.classList.add('active');
      this.servicesYearLabel.nativeElement.classList.remove('active');
    } else if (interval === 'year') {
      this.servicesWeekLabel.nativeElement.classList.remove('active');
      this.servicesMonthLabel.nativeElement.classList.remove('active');
      this.servicesYearLabel.nativeElement.classList.add('active');
    }
  }
  toggleActiveLabelClientsByData(interval: string) {
    // Toggle the active class on the labels based on the selected interval
    if (interval === 'week') {
      this.clientsWeekLabel.nativeElement.classList.add('active');
      this.clientsMonthLabel.nativeElement.classList.remove('active');
      this.clientsYearLabel.nativeElement.classList.remove('active');
    } else if (interval === 'month') {
      this.clientsWeekLabel.nativeElement.classList.remove('active');
      this.clientsMonthLabel.nativeElement.classList.add('active');
      this.clientsYearLabel.nativeElement.classList.remove('active');
    } else if (interval === 'year') {
      this.clientsWeekLabel.nativeElement.classList.remove('active');
      this.clientsMonthLabel.nativeElement.classList.remove('active');
      this.clientsYearLabel.nativeElement.classList.add('active');
    }
  }
  toggleActiveLabelServicesByData(interval: string) {
    // Toggle the active class on the labels based on the selected interval
    if (interval === 'week') {
      this.servicesByDataWeekLabel.nativeElement.classList.add('active');
      this.servicesByDataMonthLabel.nativeElement.classList.remove('active');
      this.servicesByDataYearLabel.nativeElement.classList.remove('active');
    } else if (interval === 'month') {
      this.servicesByDataWeekLabel.nativeElement.classList.remove('active');
      this.servicesByDataMonthLabel.nativeElement.classList.add('active');
      this.servicesByDataYearLabel.nativeElement.classList.remove('active');
    } else if (interval === 'year') {
      this.servicesWeekLabel.nativeElement.classList.remove('active');
      this.servicesByDataMonthLabel.nativeElement.classList.remove('active');
      this.servicesByDataYearLabel.nativeElement.classList.add('active');
    }
  }
  toggleActiveLabelUtilization(interval: string) {
    // Toggle the active class on the labels based on the selected interval
    if (interval === 'services') {
      this.utilizationServicesLabel.nativeElement.classList.add('active');
      this.utilizationClientsLabel.nativeElement.classList.remove('active');
    } else if (interval === 'client') {
      this.utilizationServicesLabel.nativeElement.classList.remove('active');
      this.utilizationClientsLabel.nativeElement.classList.add('active');
    }
  }

  updateChart(isDarkMode: boolean) {
    if (this.devicesCounts) {
      this.devicesCounts.color = isDarkMode ? '#000000' : '#CCCCCC';
      this.devicesCounts.gridColor = isDarkMode ? '#000' : '#fff';
      this.devicesCounts.validateNow();
    }

    if (this.top10servicesByVolume) {
      this.top10servicesByVolume.color = isDarkMode ? '#CCCCCC' : '#000000';
      this.top10servicesByVolume.gridColor = isDarkMode ? '#CCCCCC' : '#000000';
      this.top10servicesByVolume.validateNow();
    }

    if (this.top10servicesByUser) {
      this.top10servicesByUser.color = isDarkMode ? '#CCCCCC' : '#000000';
      this.top10servicesByUser.gridColor = isDarkMode ? '#CCCCCC' : '#000000';
      this.top10servicesByUser.validateNow();
    }

    if (this.top10usersByVolume) {
      this.top10usersByVolume.color = isDarkMode ? '#CCCCCC' : '#000000';
      this.top10usersByVolume.gridColor = isDarkMode ? '#CCCCCC' : '#000000';
      this.top10usersByVolume.validateNow();
    }

    if (this.top10usersByServices) {
      this.top10usersByServices.color = isDarkMode ? '#CCCCCC' : '#000000';
      this.top10usersByServices.gridColor = isDarkMode ? '#CCCCCC' : '#000000';
      this.top10usersByServices.validateNow();
    }
    if (this.deviceStatistics) {
      this.deviceStatistics.backgroundColor = isDarkMode
        ? '#CCCCCC'
        : '#000000';
      this.deviceStatistics.color = isDarkMode ? '#CCCCCC' : '#000000';
      this.deviceStatistics.gridColor = isDarkMode ? '#CCCCCC' : '#000000';
      this.deviceStatistics.validateNow();
    }

    // if(this.desktopPercentage){
    //   this.desktopPercentage.color = '#dad'
    // }
  }

  getGmtOffset(): string {
    this.systemTime = localStorage.getItem('systemTime');
    this.portalTime = localStorage.getItem('portalTime');
    this.customTime = localStorage.getItem('customTime');
    this.portalTimeLogin = localStorage.getItem('portalTimeZone');
    console.log(
      'ALL VALUES',
      this.systemTime,
      this.portalTime,
      this.customTime
    );

    if (
      this.systemTime != null &&
      this.portalTime == null &&
      this.customTime == null
    ) {
      console.log('IN SYSTEM', this.systemTime);
      return this.systemTime;
    }

    if (
      this.systemTime == null &&
      this.portalTime != null &&
      this.customTime == null
    ) {
      console.log('IN PORTAL', this.portalTime);
      return this.portalTime;
    }

    if (
      this.systemTime == null &&
      this.portalTime == null &&
      this.customTime != null
    ) {
      console.log('IN CUSTOM', this.customTime);
      return this.customTime;
    }

    // return this.portalTimeLogin
  }

  setAmChartFont() {
    this.AmCharts.addInitHandler(
      function(chart) {
        // Set the default fontFamily for the chart
        chart.fontFamily = 'Poppins';
      },
      ['serial', 'pie', 'xy']
    );
  }

  // public refreshLogData(): void {
  //   if (this.logtimerSubscription) {
  //     this.logtimerSubscription.unsubscribe();
  //   }
  //   this.getPage(1);
  //   this.subscribeToLogData();
  // }

  // private subscribeToLogData(): void {
  //   this.logtimerSubscription = Observable.timer(10000)
  //     .first()
  //     .subscribe(() => this.refreshLogData());
  // }

  refreshLogs() {
    if (this.userrole !== 'Report-Viewer') {
      this.getPage(1);
    }
  }

  refreshClients() {
    if (this.userrole !== 'Report-Viewer') {
      this.onlineClientsLoading = true;
      this.clientService
        .getClients(
          1,
          5,
          { isOnline: '1' },
          'id',
          'desc',
          '',
          false,
          false,
          false,
          []
        )
        .subscribe(
          response => {
            this.onlineClients = response.data;
            this.onlineClientsLoading = false;
          },
          () => {
            this.onlineClientsLoading = false;
          }
        );
    }
  }

  refreshServices() {
    if (this.userrole !== 'Report-Viewer') {
      this.servicesLoading = true;
      this.manageService.getServices(1, 5).subscribe(
        response => {
          this.topServices = response.data;
          this.servicesLoading = false;
        },
        () => {
          this.servicesLoading = false;
        }
      );
    }
  }

  refreshDevices() {
    if (this.userrole !== 'Report-Viewer') {
      this.devicesLoading = true;
      this.devicesService
        .getDeviceInventories(5, 1, 'desc', 'created_at')
        .subscribe(
          (response: any) => {
            this.topDevices = response.data;
            this.devicesLoading = false;
          },
          () => {
            this.devicesLoading = false;
          }
        );
    }
  }

  addLdap() {
    if (!this.ldaps) {
      this.router.navigate(['/create-ldapuser']);
    }
    if (!this.isSettingsSet) {
      this.router.navigate(['/settings']);
    }
  }

  addController() {
    if (this.ldaps !== 0) {
      this.router.navigate(['/create-controller']);
    }
  }

  addGateway() {
    if (this.controllers !== 0) {
      this.router.navigate(['/create-gateway']);
    }
  }

  getPage(page: number) {
    this.accessLogLoading = true;
    this.clientService
      .getClientAccessLogs(1, 6)
      .pipe(
        tap((res: any) => {
          /*****************************
			/*  0 => Failed Login
			/*  1 => Exited Abruptly
			/*  2 => Gracefully Closed
			/*  3 => Running
			/*  4 => Failed
			******************************/
          for (let log of res.data || []) {
            if (!log.is_login_successfull) {
              log.status = ClientAccessStatusEnum.FailedLogin;
              log.event = 'Failed Login';
            } else if (
              !log.is_logout_successfull &&
              log.is_logout_successfull != null
            ) {
              log.status = ClientAccessStatusEnum.ForcefullyClosed;
              log.event = 'Forcefully Closed';
            } else {
              if (
                log.CONNECTION_ATTEMPT_TIME != null &&
                log.DISCONNECTION_TIME != null
              ) {
                log.status = ClientAccessStatusEnum.GracefullyClosed;
                log.event = 'Gracefully Closed';
              } else if (log.DISCONNECTION_TIME == null) {
                log.status = ClientAccessStatusEnum.Running;
                log.event = 'Running';
              } else {
                log.status = ClientAccessStatusEnum.Failed;
                log.event = 'Failed';
              }
            }
            if (
              log.CONNECTION_ATTEMPT_TIME != null &&
              log.DISCONNECTION_TIME != null
            ) {
              let diffInMs: number =
                Date.parse(log.DISCONNECTION_TIME) -
                Date.parse(log.CONNECTION_ATTEMPT_TIME);
              let diffInHours: number = diffInMs / 1000 / 60 / 60;
              let flatHours = Math.floor(diffInHours);
              log.sessionTime = '';
              if (flatHours > 0) {
                log.sessionTime = flatHours + ' Hours ';
              }
              let totalMinutesUsed = flatHours * 60;
              let minutesleft = diffInMs / 1000 / 60 - totalMinutesUsed;
              let flatMinutes = Math.floor(minutesleft);
              if (flatMinutes > 0) {
                log.sessionTime += flatMinutes + ' Minutes ';
              }
              if (flatMinutes <= 0) {
                let totalSecondsUsed = flatMinutes * 60;
                let secondsleft =
                  diffInMs / 1000 - (totalSecondsUsed + totalMinutesUsed * 60);
                let flatSeconds = Math.floor(secondsleft);
                if (secondsleft > 0) {
                  log.sessionTime += secondsleft + ' Seconds ';
                }
              }
            }
          }
          this.accessLogs = res;
        })
      )
      .subscribe({
        next: res => {
          this.accessLogLoading = false;
        },
        error: err => {
          this.accessLogLoading = false;
        }
      });
  }

  onLoadMap($event: any) {
    this.map = $event;
    let startPoint = new google.maps.LatLng(
      36.90731625763393,
      -86.51778523864743
    );
    let endPoint = new google.maps.LatLng(
      37.02763411292923,
      -86.37183015289304
    );
    this.allowedBounds = new google.maps.LatLngBounds(startPoint, endPoint);
    let map = this.map;
    let allowedBounds = this.allowedBounds;
    google.maps.event.addListener(map, 'dragend', function() {
      if (this.zoom == this.minZoom) {
        if (allowedBounds.contains(map.getCenter())) return;
        var c = map.getCenter(),
          x = c.lng(),
          y = c.lat(),
          maxX = allowedBounds.getNorthEast().lng(),
          maxY = allowedBounds.getNorthEast().lat(),
          minX = allowedBounds.getSouthWest().lng(),
          minY = allowedBounds.getSouthWest().lat();

        if (x < minX) x = minX;
        if (x > maxX) x = maxX;
        if (y < minY) y = minY;
        if (y > maxY) y = maxY;

        map.setCenter(new google.maps.LatLng(y, x));
      }
    });
  }

  ngAfterViewInit() {
    this.top10servicesByUser = this.AmCharts.makeChart('servicesByUser', {
      type: 'serial',
      theme: 'light',
      dataProvider: [],
      hideCredits: true,
      valueAxes: [
        {
          title: 'Number of users',
          gridColor: '#FFFFFF',
          gridAlpha: 0.2,
          dashLength: 0,
          fontFamily: 'Poppins',
          color: this.isDarkMode ? '#FFF' : '#000',
          titleColor: this.isDarkMode ? '#FFF' : '#000'
        }
      ],
      gridAboveGraphs: false,
      startDuration: 1,
      graphs: [
        {
          balloonText: "[[category]]: <b style='color:#111111;'>[[value]]</b>",
          fillAlphas: 1,
          lineAlpha: 0.2,
          type: 'column',
          lineColor: '#39C6A5',
          lineThickness: 2,
          negativeLineColor: '#39C6A5',
          valueField: 'value',
          color: this.isDarkMode ? '#FFF' : '#000'
        }
      ],
      chartCursor: {
        categoryBalloonEnabled: false,
        cursorAlpha: 0,
        zoomable: false
      },
      categoryField: 'name',
      categoryAxis: {
        gridPosition: 'middle',
        gridAlpha: 0,
        tickPosition: 'start',
        tickLength: 5,
        color: this.isDarkMode ? '#FFF' : '#000',
        labelFunction: function(label, item, axis) {
          var chart = axis.chart;
          if (label.length > 8) {
            return label.substr(0, 9) + '...';
          }
          if (label.length > 11) {
            return label.substr(0, 12) + '...';
          }
          return label;
        }
      },
      rotate: true
    });
    this.top10servicesByVolume = this.AmCharts.makeChart('servicesByVolume', {
      type: 'serial',
      theme: 'light',
      dataProvider: [],
      hideCredits: true,
      valueAxes: [
        {
          gridColor: this.isDarkMode ? '#FFF' : '#000',
          title: 'Data consumed (MB)',
          gridAlpha: 0.2,
          dashLength: 0,
          fontFamily: 'Poppins',
          color: this.isDarkMode ? '#FFF' : '#000',
          titleColor: this.isDarkMode ? '#FFF' : '#000'
        }
      ],
      gridAboveGraphs: false,
      startDuration: 1,
      graphs: [
        {
          balloonText: "[[category]]: <b style='color:#111111;'>[[value]]</b>",
          fillAlphas: 1,
          lineAlpha: 1,
          type: 'column',
          valueField: 'value',
          lineColor: '#39C6A5',
          lineThickness: 2,
          negativeLineColor: '#39C6A5',
          color: this.isDarkMode ? '#FFF' : '#000',
          gridColor: this.isDarkMode ? '#FFF' : '#000'
        }
      ],
      chartCursor: {
        categoryBalloonEnabled: false,
        cursorAlpha: 0,
        zoomable: false
      },
      categoryField: 'name',
      categoryAxis: {
        labelRotation: 90,
        gridPosition: 'start',
        gridAlpha: 0,
        tickPosition: 'start',
        tickLength: 20,
        color: this.isDarkMode ? '#FFF' : '#000',
        gridColor: this.isDarkMode ? '#FFF' : '#000',
        labelFunction: function(label, item, axis) {
          var chart = axis.chart;
          if (label.length > 8) {
            return label.substr(0, 9) + '...';
          }
          if (label.length > 11) {
            return label.substr(0, 12) + '...';
          }
          return label;
        }
      }
    });
    this.devicesCounts = this.AmCharts.makeChart('devicesCount', {
      type: 'serial',
      hideCredits: true,
      theme: 'light',
      dataProvider: [],
      gridAboveGraphs: false,
      categoryField: 'name',
      // "categoryAxis": {
      // "gridAlpha": 0.07,
      // "axisColor": "#eaf0f4",
      // "gridColor": "green",
      // "startOnAxis": false,
      // "title": "Year",
      // },
      categoryAxis: {
        gridPosition: 'start',
        gridAlpha: 0,
        tickPosition: 'start',
        tickLength: 20,
        color: this.isDarkMode ? '#FFF' : '#000'
      },
      // "valueAxes": [{
      //   "gridColor": "#eaf0f4",
      //   "gridAlpha": 0.5,
      //   "dashLength": 0
      // }],
      valueAxes: [
        {
          stackType: 'regular',
          gridColor: '#eaf0f4',
          // "title": "Data consumed (MB)",
          gridAlpha: 1,
          dashLength: 0,
          fontFamily: 'Poppins',
          color: this.isDarkMode ? '#FFF' : '#000'
        }
      ],
      startDuration: 1,
      graphs: [
        {
          id: 'g1',
          type: 'column',
          title: 'Articles',
          columnWidth: 0.2,
          valueField: 'yesterday',
          lineColor: '#39C6A5',
          lineAlpha: 0,
          fillAlphas: 1
        },
        {
          id: 'g2',
          type: 'column',
          columnWidth: 0.2,
          title: 'Blogs',
          valueField: 'today',
          lineAlpha: 0,
          lineColor: '#B0E8DB',
          fillAlphas: 1
        }
      ]
    });
    this.top10usersByServices = this.AmCharts.makeChart('usersByServices', {
      type: 'serial',
      theme: 'dark',
      hideCredits: true,
      dataProvider: [],
      gridAboveGraphs: false,
      startDuration: 1,
      valueAxes: [
        {
          stackType: 'regular',
          title: 'Number of services',
          gridColor: '#eaf0f4',
          gridAlpha: 1,
          dashLength: 0,
          fontFamily: 'Poppins',
          color: this.isDarkMode ? '#FFF' : '#000',
          titleColor: this.isDarkMode ? '#FFF' : '#000'
        }
      ],
      graphs: [
        {
          balloonText: "[[category]]: <b style='color:#111111;'>[[value]]</b>",
          fillAlphas: 1,
          lineAlpha: 0,
          columnWidth: 0.5,
          type: 'column',
          valueField: 'value',
          lineColor: '#39C6A5',
          color: this.isDarkMode ? '#FFF' : '#000'
        }
      ],
      // chartCursor: {
      //   categoryBalloonEnabled: false,
      //   cursorAlpha: 0,
      //   zoomable: false
      // },
      categoryField: 'name',
      categoryAxis: {
        labelRotation: 90,
        gridPosition: 'start',
        gridAlpha: 1,
        gridColor: '#eaf0f4',
        dashLength: 1,
        minorGridEnabled: true,
        tickPosition: 'start',
        tickLength: 20,
        color: this.isDarkMode ? '#FFF' : '#000',

        labelFunction: function(label, item, axis) {
          var chart = axis.chart;
          if (label.length > 8) {
            return label.substr(0, 9) + '...';
          }
          if (label.length > 11) {
            return label.substr(0, 12) + '...';
          }
          return label;
        }
      }
    });
    this.top10usersByVolume = this.AmCharts.makeChart('usersByVolume', {
      type: 'serial',
      theme: 'light',
      dataProvider: [],
      hideCredits: true,
      valueAxes: [
        {
          title: 'Data consumed (MB)',
          gridColor: '#FFFFFF',
          gridAlpha: 0.2,
          dashLength: 0,
          fontFamily: 'Poppins',
          color: this.isDarkMode ? '#FFF' : '#000',
          titleColor: this.isDarkMode ? '#FFF' : '#000'
        }
      ],
      gridAboveGraphs: false,
      startDuration: 1,
      graphs: [
        {
          balloonText: "[[category]]: <b style='color:#111111;'>[[value]]</b>",
          fillAlphas: 1,
          lineColor: '#39C6A5',
          lineAlpha: 0.2,
          type: 'column',
          valueField: 'value',
          color: this.isDarkMode ? '#FFF' : '#000'
        }
      ],
      chartCursor: {
        categoryBalloonEnabled: false,
        cursorAlpha: 0,
        zoomable: false
      },
      categoryField: 'name',
      categoryAxis: {
        gridPosition: 'middle',
        gridAlpha: 0,
        tickPosition: 'start',
        color: this.isDarkMode ? '#FFF' : '#000',
        tickLength: 5,
        labelFunction: function(label, item, axis) {
          var chart = axis.chart;
          if (label.length > 8) {
            return label.substr(0, 9) + '...';
          }
          if (label.length > 11) {
            return label.substr(0, 12) + '...';
          }
          return label;
        }
      },
      rotate: true
    });
    this.totalNumberOfServiceAccessed = this.AmCharts.makeChart('chartdiv4', {
      type: 'gauge',
      theme: 'none',
      hideCredits: true,
      axes: [
        {
          axisAlpha: 0,
          tickAlpha: 0,
          labelsEnabled: false,
          startValue: 0,
          endValue: 100,
          startAngle: 0,
          endAngle: 360,
          bands: [
            {
              color: this.isDarkMode ? '#8e8e8e' : '#eee',
              startValue: 0,
              endValue: 100,
              radius: '80%',
              innerRadius: '60%'
            },
            {
              color: '#84b761',
              startValue: 0,
              endValue: 0,
              radius: '80%',
              innerRadius: '60%'
            }
          ]
        }
      ],
      allLabels: [
        {
          text: '',
          y: '40%',
          size: 40,
          bold: true,
          color: '#aaa',
          align: 'center',
          fontFamily: 'Poppins'
        }
      ]
    });
    this.desktopPercentage = this.AmCharts.makeChart('chartdiv5', {
      type: 'gauge',
      theme: 'none',
      startDuration: 1,
      hideCredits: true,
      fontFamily: 'Poppins',
      axes: [
        {
          axisAlpha: 0,
          tickAlpha: 0,
          labelsEnabled: false,
          startValue: 0,
          endValue: 100,
          startAngle: 0,
          endAngle: 360,
          bands: [
            {
              color: this.isDarkMode ? '#353d4a' : '#eee',
              startValue: 0,
              endValue: 100,
              radius: '120%',
              innerRadius: '105%'
            },
            {
              color: '#39C6A5',
              startValue: 0,
              endValue: 0,
              radius: '120%',
              innerRadius: '105%'
            }
          ]
        }
      ],
      axisLabels: {
        fontFamily: 'Poppins'
      },
      allLabels: [
        {
          text: '',
          y: '32%',
          size: 25,
          bold: true,
          align: 'center',
          fontFamily: 'Poppins',
          color: this.isDarkMode ? '#FFF' : '#364150'
        },
        {
          text: '',
          y: '60%',
          x: '2%',
          size: 14,
          bold: false,
          align: 'center',
          fontFamily: 'Poppins',
          color: this.isDarkMode ? '#FFF' : '#364150'
        }
      ]
    });
    this.deviceStatistics = this.AmCharts.makeChart('device-statistics', {
      type: 'serial',
      theme: 'light',
      startDuration: 1,
      sequencedAnimation: false,
      zoomOutText: '',
      hideCredits: true,
      marginRight: 20,
      marginLeft: 50,
      autoMarginOffset: 20,
      mouseWheelZoomEnabled: false,

      dataDateFormat: 'YYYY-MM-DD',
      valueAxes: [
        {
          id: 'v1',
          axisAlpha: 0,
          position: 'left',
          fontFamily: 'Poppins',
          gridColor: this.isDarkMode ? '#FFF' : '#000',
          color: this.isDarkMode ? '#FFF' : '#000'
        }
      ],
      balloon: {
        borderThickness: 1,
        shadowAlpha: 0
      },
      graphs: [
        {
          id: 'g1',
          balloon: {
            adjustBorderColor: false,
            color: '#ffffff',
            horizontalPadding: 20,
            verticalPadding: 20
          },
          lineColor: '#9460FF',
          bullet: 'round',
          bulletBorderAlpha: 1,
          bulletColor: '#FFFFFF',
          bulletSize: 6,
          hideBulletsCount: 100,
          lineThickness: 2,
          title: 'Licensed Users',
          useLineColorForBulletBorder: true,
          valueField: 'users',
          balloonText: "<span style='font-size:12x;'>[[users]]</span>"
        },
        {
          id: 'g3',
          balloon: {
            adjustBorderColor: false,
            color: '#ffffff'
          },
          lineColor: '#2CD9C5',
          bullet: 'round',
          bulletBorderAlpha: 1,
          bulletColor: '#FFFFFF',
          bulletSize: 5,
          hideBulletsCount: 50,
          lineThickness: 2,
          title: 'Total Devices',
          useLineColorForBulletBorder: true,
          valueField: 'devices',
          balloonText: "<span style='font-size:12x;'>[[devices]]</span>"
        },
        {
          id: 'g2',
          balloon: {
            adjustBorderColor: false,
            color: '#ffffff'
          },
          lineColor: '#E32D2A',
          bullet: 'round',
          bulletBorderAlpha: 1,
          bulletColor: '#FFFFFF',
          bulletSize: 5,
          hideBulletsCount: 50,
          lineThickness: 2,
          title: 'User With Devices',
          useLineColorForBulletBorder: true,
          valueField: 'clients',
          balloonText: "<span style='font-size:12px;'>[[clients]]</span>"
        }
      ],

      categoryField: 'date',
      categoryAxis: {
        parseDates: true,
        dashLength: 1,
        minorGridEnabled: true,
        color: this.isDarkMode ? '#FFF' : '#000', // Set the color for x-axis labels
        gridColor: this.isDarkMode ? '#FFF' : '#000',
        dateFormats: [
          {
            period: 'fff',
            format: 'JJ:NN:SS'
          },
          {
            period: 'ss',
            format: 'JJ:NN:SS'
          },
          {
            period: 'mm',
            format: 'JJ:NN'
          },
          {
            period: 'hh',
            format: 'JJ:NN'
          },
          {
            period: 'DD',
            format: 'MMM DD'
          },
          {
            period: 'WW',
            format: 'MMM DD'
          },
          {
            period: 'MM',
            format: 'MMM DD'
          },
          {
            period: 'YYYY',
            format: 'YYYY'
          }
        ]
      },
      dataProvider: [],
      legend: {
        position: 'bottom',
        useMarkerColorForLabels: true
      }
    });
    // const hideMessage = setInterval(() => {
    //   const tag: any = document.querySelector("a[href='http://www.amcharts.com']");
    //   tag.style.visibility = 'hidden';
    //   clearInterval(hideMessage);
    // }, 1000);
    this.mobilePercentage = this.AmCharts.makeChart('chartdiv9', {
      type: 'gauge',
      theme: 'none',
      startDuration: 1,
      hideCredits: true,
      fontFamily: 'Poppins',
      axes: [
        {
          axisAlpha: 0,
          tickAlpha: 0,
          labelsEnabled: false,
          startValue: 0,
          endValue: 100,
          startAngle: 0,
          endAngle: 360,
          bands: [
            {
              color: this.isDarkMode ? '#353d4a' : '#eee',
              startValue: 0,
              endValue: 100,
              radius: '120%',
              innerRadius: '105%'
            },
            {
              color: '#39C6A5',
              startValue: 0,
              endValue: 0,
              radius: '120%',
              innerRadius: '105%'
            }
          ]
        }
      ],
      axisLabels: {
        fontFamily: 'Poppins'
      },
      allLabels: [
        {
          text: '',
          y: '32%',
          size: 25,
          bold: true,
          color: this.isDarkMode ? '#FFF' : '#364150',
          align: 'center',
          fontFamily: 'Poppins'
        },
        {
          text: '',
          y: '60%',
          x: '2%',
          size: 14,
          bold: false,
          color: this.isDarkMode ? '#FFF' : '#364150',
          align: 'center',
          fontFamily: 'Poppins'
        }
      ]
    });

    this.serverPercentage = this.AmCharts.makeChart('chartdiv10', {
      type: 'gauge',
      theme: 'none',
      startDuration: 1,
      hideCredits: true,
      fontFamily: 'Poppins',
      axes: [
        {
          axisAlpha: 0,
          tickAlpha: 0,
          labelsEnabled: false,
          startValue: 0,
          endValue: 100,
          startAngle: 0,
          endAngle: 360,
          bands: [
            {
              color: this.isDarkMode ? '#353d4a' : '#eee',
              startValue: 0,
              endValue: 100,
              radius: '120%',
              innerRadius: '105%'
            },
            {
              color: '#39C6A5',
              startValue: 0,
              endValue: 0,
              radius: '120%',
              innerRadius: '105%'
            }
          ]
        }
      ],
      axisLabels: {
        fontFamily: 'Poppins'
      },
      allLabels: [
        {
          text: '',
          y: '32%',
          size: 25,
          bold: true,
          color: this.isDarkMode ? '#FFF' : '#364150',
          align: 'center',
          fontFamily: 'Poppins'
        },
        {
          text: '',
          y: '60%',
          x: '2%',
          size: 14,
          bold: false,
          color: this.isDarkMode ? '#FFF' : '#364150',
          align: 'center',
          fontFamily: 'Poppins'
        }
      ]
    });

    this.accessByHostOS = this.AmCharts.makeChart('accessByHostOS', {
      type: 'pie',
      theme: 'none',
      dataProvider: [],
      hideCredits: true,
      titleField: 'name',
      valueField: 'y',
      labelRadius: 5,

      radius: '40%',
      innerRadius: '60%',
      labelText: '[[title]]'
    });

    this.utillizations = this.AmCharts.makeChart('utillizations', {
      type: 'serial',
      hideCredits: true,
      zoomOutText: '',
      dataDateFormat: 'YYYY-MM-DD HH',
      theme: 'dark',
      dataProvider: [],
      gridAboveGraphs: false,
      categoryField: 'hours',
      categoryAxis: {
        parseDates: true,
        minPeriod: 'hh',
        dashLength: 1,
        minorGridEnabled: true,
        dateFormats: [
          {
            period: 'fff',
            format: 'JJ:NN:SS'
          },
          {
            period: 'ss',
            format: 'JJ:NN:SS'
          },
          {
            period: 'mm',
            format: 'JJ:NN'
          },
          {
            period: 'hh',
            format: 'JJ:NN A'
          },
          {
            period: 'DD',
            format: 'MMM DD\nHH:NN'
          },
          {
            period: 'WW',
            format: 'MMM DD\nHH:NN'
          },
          {
            period: 'MM',
            format: 'MMM DD\nHH:NN'
          },
          {
            period: 'YYYY',
            format: 'YYYY'
          }
        ],
        gridPosition: 'start',
        gridAlpha: 0,
        tickPosition: 'start',
        tickLength: 20,
        color: this.isDarkMode ? '#FFF' : '#000'
      },
      // "valueAxes": [{
      //   "gridColor": "#eaf0f4",
      //   "gridAlpha": 0.5,
      //   "dashLength": 0
      // }],
      valueAxes: [
        {
          stackType: 'regular',
          gridColor: '#eaf0f4',
          title: 'Data consumed (KB)',
          gridAlpha: 1,
          dashLength: 0,
          fontFamily: 'Poppins',
          color: this.isDarkMode ? '#FFF' : '#000',
          titleColor: this.isDarkMode ? '#FFF' : '#000'
        }
      ],
      startDuration: 1,
      graphs: [
        {
          id: 'g1',
          type: 'column',
          title: 'Articles',
          columnWidth: 0.5,
          valueField: 'consumed',
          lineColor: '#39C6A5',
          lineAlpha: 0,
          fillAlphas: 1,
          color: this.isDarkMode ? '#FFF' : '#000'
        }
      ]
    });

    zip(
      this.dashboardService.getGraphServicesByUsers(),
      this.dashboardService.getGraphServicesByVolume(),
      this.dashboardService.getGraphUsersByServices(),
      this.dashboardService.getGraphUsersByVolume(),
      this.dashboardService.getDevicesCount(),
      this.dashboardService.getDevicesStatistics(),
      this.dashboardService.getUtillizationsData('client')
    ).subscribe(
      ([
        getGraphServicesByUsers,
        getGraphServicesByVolume,
        getGraphUsersByServices,
        getGraphUsersByVolume,
        devicesCount,
        devicesStatistics,
        utillizations
      ]: any) => {
        this.graphData = {
          getGraphServicesByUsers,
          getGraphServicesByVolume,
          getGraphUsersByServices,
          getGraphUsersByVolume
        };
        this.selectedGraphFilter.selectedServicesByUser =
          getGraphServicesByUsers.month.length;
        this.selectedGraphFilter.selectedServicesByVolume =
          getGraphServicesByVolume.month.length;
        this.selectedGraphFilter.selectedUsersByServices =
          getGraphUsersByServices.month.length;
        this.selectedGraphFilter.selectedUsersByVolume =
          getGraphUsersByVolume.month.length;

        this.selectedGraphFilter.utillizations = utillizations.data.length;

        this.AmCharts.updateChart(this.top10servicesByUser, () => {
          this.top10servicesByUser.dataProvider = getGraphServicesByUsers.month;
          this.topTenServiceByClientsLoading = false;
        });

        this.AmCharts.updateChart(this.utillizations, () => {
          this.utillizations.dataProvider = utillizations.data;
          this.uniqueClientsCount = utillizations.uniqueClientsCount;
          this.utillizationLoading = false;
        });
        this.AmCharts.updateChart(this.top10servicesByVolume, () => {
          this.top10servicesByVolume.dataProvider =
            getGraphServicesByVolume.month;
          this.topTenServicesByVolumeOfDataLoading = false;
        });
        this.AmCharts.updateChart(this.devicesCounts, () => {
          this.devicesCounts.dataProvider = devicesCount.devices;
          this.devicesCountLoading = false;
        });
        this.AmCharts.updateChart(this.mobilePercentage, () => {
          const value = devicesCount.percentageByTypes.mobile;
          const value2 = devicesCount.percentageByTypes.totalMobile;
          const value3 = devicesCount.percentageByTypes.mobileCountWithVersion;
          this.mobilePercentage.axes[0].bands[1].endValue = value;
          this.mobilePercentage.allLabels[0].text = value + '%';
          this.mobilePercentage.allLabels[1].text = value3 + '/' + value2;
        });
        this.AmCharts.updateChart(this.desktopPercentage, () => {
          const value = devicesCount.percentageByTypes.desktop;
          const value2 = devicesCount.percentageByTypes.totalDesktop;
          const value3 = devicesCount.percentageByTypes.desktopCountWithVersion;
          this.desktopPercentage.axes[0].bands[1].endValue = value;
          this.desktopPercentage.allLabels[0].text = value + '%';
          this.desktopPercentage.allLabels[1].text = value3 + '/' + value2;
        });
        this.AmCharts.updateChart(this.serverPercentage, () => {
          const value = devicesCount.percentageByTypes.server;
          const value2 = devicesCount.percentageByTypes.totalServer;
          const value3 = devicesCount.percentageByTypes.serverCountWithVersion;
          this.serverPercentage.axes[0].bands[1].endValue = value;
          this.serverPercentage.allLabels[0].text = value + '%';
          this.serverPercentage.allLabels[1].text = value3 + '/' + value2;
        });
        this.AmCharts.updateChart(this.top10usersByServices, () => {
          this.top10usersByServices.dataProvider =
            getGraphUsersByServices.month;
          this.topTenClientsByNumberOfServicesAccessed = false;
        });
        this.AmCharts.updateChart(this.top10usersByVolume, () => {
          this.top10usersByVolume.dataProvider = getGraphUsersByVolume.month;
          this.topTenClientsByAggregateVolumeOfData = false;
        });
        this.AmCharts.updateChart(this.deviceStatistics, () => {
          this.deviceStatistics.dataProvider = devicesStatistics;
          this.selectedGraphFilter.deviceStatistics = devicesStatistics.length;
          this.devicesStatisticsLoading = false;
        });
      }
    );

    this.dashboardService.getGraphData().subscribe({
      next: data => {
        // this.AmCharts.updateChart(this.totalNumberOfUserAccessedServices, () => {
        //   // debugger;
        //   this.totalNumberOfUserAccessedServices.axes[0].bands[1].endValue = data.utilizationByUsers;
        //   this.totalNumberOfUserAccessedServices.allLabels[0].text = data.utilizationByUsers + '%';
        //   this.utilizationByClientLoading = false;
        // });
        this.AmCharts.updateChart(this.totalNumberOfServiceAccessed, () => {
          this.totalNumberOfServiceAccessed.axes[0].bands[1].endValue =
            data.utilizationByServices;
          this.totalNumberOfServiceAccessed.allLabels[0].text =
            data.utilizationByServices + '%';
          this.utilizationByServiceAccessedLoading = false;
        });
      },
      error: err => {
        this.utilizationByClientLoading = false;
        this.utilizationByServiceAccessedLoading = false;
        // this.topTenServiceByClientsLoading = false;
        // this.topTenServicesByVolumeOfDataLoading = false;
        // this.topTenClientsByNumberOfServicesAccessed = false;
        // this.topTenClientsByAggregateVolumeOfData = false;
      }
    });
  }

  updateGraphData(type: string, filter: string, chart: any) {
    if (chart === 'top10servicesByUser') {
      this.handleServiceByUserChange(type, filter);
    } else if (chart === 'top10servicesByVolume') {
      this.handleServicesByDataVolumeChange(type, filter);
    } else if (chart === 'top10usersByServices') {
      this.handleUserServiceGraphChange(type, filter);
    } else if (chart === 'top10usersByVolume') {
      this.handleUserByVolumeGraphChange(type, filter);
    }

    this.AmCharts.updateChart(this[chart], () => {
      this[chart].dataProvider = this.graphData[type][filter];
    });
  }

  handleUserServiceGraphChange(type, filter) {
    this.topTenClientsByNumberOfServicesAccessed = true;
    this.dashboardService.getGraphUsersByServices(filter).subscribe(data => {
      this.graphData[type] = data;
      this.selectedGraphFilter.selectedUsersByServices = data[filter].length;
      this.AmCharts.updateChart(this.top10usersByServices, () => {
        this.top10usersByServices.dataProvider = data[filter];
        this.topTenClientsByNumberOfServicesAccessed = false;
      });
    });
  }

  handleUserByVolumeGraphChange(type, filter) {
    this.topTenClientsByAggregateVolumeOfData = true;
    this.dashboardService.getGraphUsersByVolume(filter).subscribe(data => {
      this.graphData[type] = data;
      this.selectedGraphFilter.selectedUsersByVolume = this.graphData[type][
        filter
      ].length;
      this.AmCharts.updateChart(this.top10usersByVolume, () => {
        this.top10usersByVolume.dataProvider = data[filter];
        this.topTenClientsByAggregateVolumeOfData = false;
      });
    });
  }

  handleServicesByDataVolumeChange(type, filter) {
    this.topTenServicesByVolumeOfDataLoading = true;
    this.dashboardService.getGraphServicesByVolume(filter).subscribe(data => {
      this.graphData[type] = data;
      this.selectedGraphFilter.selectedServicesByVolume = this.graphData[type][
        filter
      ].length;
      this.AmCharts.updateChart(this.top10servicesByVolume, () => {
        this.top10servicesByVolume.dataProvider = data[filter];
        this.topTenServicesByVolumeOfDataLoading = false;
      });
    });
  }

  handleServiceByUserChange(type, filter) {
    this.topTenServiceByClientsLoading = true;
    this.dashboardService.getGraphServicesByUsers(filter).subscribe(data => {
      this.graphData[type] = data;
      this.selectedGraphFilter.selectedServicesByUser = this.graphData[type][
        filter
      ].length;

      this.AmCharts.updateChart(this.top10servicesByUser, () => {
        this.top10servicesByUser.dataProvider = data[filter];
        this.topTenServiceByClientsLoading = false;
      });
    });
  }

  updateDevicesStatistics(interval) {
    this.devicesStatisticsLoading = true;
    this.dashboardService.getDevicesStatistics(interval).subscribe({
      next: response => {
        this.AmCharts.updateChart(this.deviceStatistics, () => {
          this.deviceStatistics.dataProvider = response;
          this.selectedGraphFilter.deviceStatistics = response.length;
        });
      },
      complete: () => {
        this.devicesStatisticsLoading = false;
      }
    });
  }

  updateUtillizationGraph(type: 'client' | 'service') {
    this.utillizationLoading = true;
    this.selectedUtillizationType = type;
    this.dashboardService.getUtillizationsData(type).subscribe(data => {
      this.AmCharts.updateChart(this.utillizations, () => {
        this.utillizations.dataProvider = data.data;
        this.uniqueClientsCount = data.uniqueClientsCount;
        this.utillizationLoading = false;
        this.selectedGraphFilter.utillizations = data.data.length || 0;
      });
    });
    const capitalizedType =
      this.selectedUtillizationType.charAt(0).toUpperCase() +
      this.selectedUtillizationType.slice(1);
    this.graphTitle = `Utilization by ${capitalizedType}s`;
  }

  versionChange() {
    this.dashboardService
      .getDevicesCount(this.selectedVersion.value || false)
      .subscribe(devicesCount => {
        this.AmCharts.updateChart(this.mobilePercentage, () => {
          const value = devicesCount.percentageByTypes.mobile;
          const value2 = devicesCount.percentageByTypes.totalMobile;
          const value3 = devicesCount.percentageByTypes.mobileCountWithVersion;
          this.mobilePercentage.axes[0].bands[1].endValue = value;
          this.mobilePercentage.allLabels[0].text = value + '%';
          this.mobilePercentage.allLabels[1].text = value3 + '/' + value2;
        });
        this.AmCharts.updateChart(this.desktopPercentage, () => {
          const value = devicesCount.percentageByTypes.desktop;
          const value2 = devicesCount.percentageByTypes.totalDesktop;
          const value3 = devicesCount.percentageByTypes.desktopCountWithVersion;
          this.desktopPercentage.axes[0].bands[1].endValue = value;
          this.desktopPercentage.allLabels[0].text = value + '%';
          this.desktopPercentage.allLabels[1].text = value3 + '/' + value2;
        });
        this.AmCharts.updateChart(this.serverPercentage, () => {
          const value = devicesCount.percentageByTypes.server;
          const value2 = devicesCount.percentageByTypes.totalServer;
          const value3 = devicesCount.percentageByTypes.serverCountWithVersion;
          this.serverPercentage.axes[0].bands[1].endValue = value;
          this.serverPercentage.allLabels[0].text = value + '%';
          this.serverPercentage.allLabels[1].text = value3 + '/' + value2;
        });
      });
  }

  getOSName(osName) {
    switch (osName) {
      case DeviceOSEnum.Ubuntu18:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer18:
        return 'Ubuntu Server';
      case DeviceOSEnum.UbuntuServer20armv8:
        return 'Ubuntu Server (ARMv8)';
      case DeviceOSEnum.Windows:
        return 'Windows';
      case DeviceOSEnum.WindowsServer:
        return 'Windows Server';
      case DeviceOSEnum.Macintosh:
        return 'Macintosh';
      case DeviceOSEnum.IOS:
        return 'iOS';
      case DeviceOSEnum.Android:
        return 'Android';
      case DeviceOSEnum.CentosServer:
        return 'CentOS Server';
      default:
        return osName;
    }
  }
}
