import { Injectable } from '@angular/core';

import { ConfigService } from '../services/config.service';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { AuthenticationService } from '../login/authentication.service';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';

@Injectable()
export class DashboardService {
  private dashboardUrl = '/dashboard';
  private graphUrl = '/graph';
  private countApiUrl = '/count';
  private osApiUrl = '/os';
  private markerController = '/markers-controllers';
  private markerGateway = '/markers-gateways';
  private markerClient = '/markers-clients';
  private apiBaseUrl: string;

  constructor(
    private authenticationService: AuthenticationService,
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.dashboardUrl = this.apiBaseUrl + this.dashboardUrl;
    this.countApiUrl = this.dashboardUrl + this.countApiUrl;
    this.osApiUrl = this.dashboardUrl + this.osApiUrl;
    this.markerController = this.dashboardUrl + this.markerController;
    this.markerClient = this.dashboardUrl + this.markerClient;
    this.markerGateway = this.dashboardUrl + this.markerGateway;
    this.graphUrl = this.dashboardUrl + this.graphUrl;
  }

  /** Count User */

  countHost(): any {
    return this.authHttp
      .get(this.countApiUrl)
      .pipe(map((response: Response) => response));
  }

  getOS(): Observable<any> {
    return this.authHttp
      .get(this.osApiUrl)
      .pipe(map((response: Response) => response));
  }

  getGraphData(): any {
    return this.authHttp
      .get(this.graphUrl)
      .pipe(map((response: Response) => response));
  }

  getControllerMarkers(): any {
    return this.authHttp
      .get(this.markerController)
      .pipe(map((response: Response) => response));
  }

  getGraphUsersByServices(filter?: string): any {
    let url = this.graphUrl + '/users/services';
    if (filter) {
      url = `${url}?filter=${filter}`;
    }
    return this.authHttp.get(url).pipe(map((response: Response) => response));
  }

  getGraphUsersByVolume(filter?: string): any {
    let url = this.graphUrl + '/users/volume';
    if (filter) {
      url = `${url}?filter=${filter}`;
    }
    return this.authHttp.get(url).pipe(map((response: Response) => response));
  }

  getGraphServicesByUsers(filter?: string): any {
    let url = this.graphUrl + '/services/users';
    if (filter) {
      url = `${url}?filter=${filter}`;
    }
    return this.authHttp.get(url).pipe(map((response: Response) => response));
  }

  getGraphServicesByVolume(filter?: string): any {
    let url = this.graphUrl + '/services/volume';
    if (filter) {
      url = `${url}?filter=${filter}`;
    }
    return this.authHttp.get(url).pipe(map((response: Response) => response));
  }

  getDevicesCount(version = false): any {
    let url = this.graphUrl + '/devices/count' + `?oldVersion=${version}`;
    return this.authHttp.get(url).pipe(map((response: Response) => response));
  }

  getDevicesStatistics(interval: 'days' | 'month' | 'year' = 'days'): any {
    let url = this.graphUrl + '/devices/statistics' + `?interval=${interval}`;
    return this.authHttp.get(url).pipe(map((response: Response) => response));
  }

  getUtillizationsData(type: 'client' | 'service'): any {
    let url = this.graphUrl + '/utillization/' + type;
    return this.authHttp.get(url).pipe(map((response: Response) => response));
  }

  getTopDataSummary(): any {
    let url = this.dashboardUrl + '/top/summary';
    return this.authHttp.get(url).pipe(map((response: Response) => response));
  }
}
