import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { ConfigService } from '../services/config.service';
import { AuthShortlivedCookieService } from '../auth-shortlived-cookie.service';
import { map } from 'rxjs/operators';

@Injectable()
export class DeviceInventoryService {
  private apiBaseUrl: string;
  private deviceInventoriesUrl = '/deviceInventory';

  constructor(
    private authHttp: AuthHttp,
    private configService: ConfigService,
    private authShortlivedCookie: AuthShortlivedCookieService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.deviceInventoriesUrl = this.apiBaseUrl + this.deviceInventoriesUrl;
  }

  getDeviceInventories(
    pageSize: number = 10,
    page: number = 1,
    sort: string = 'id',
    dir: string
  ) {
    let query = `?limit=${pageSize}&offset=${page - 1}&sort=${sort}&dir=${dir}`;
    return this.authHttp
      .get(`${this.deviceInventoriesUrl + query}`)
      .pipe(map(response => response));
  }

  deleteInventory(inventory) {
    return this.authHttp
      .delete(`${this.deviceInventoriesUrl}/${inventory.id}`)
      .pipe(map(response => response));
  }

  updateInventory(inventory) {
    return this.authHttp
      .put(`${this.deviceInventoriesUrl}/${inventory.id}`, { inventory })
      .pipe(map(response => response));
  }

  addNewInventory(inventory) {
    return this.authHttp
      .post(`${this.deviceInventoriesUrl}/addInventory`, { inventory })
      .pipe(map(response => response));
  }

  uploadCsvFile(fileData) {
    return this.authHttp
      .post(this.deviceInventoriesUrl, { fileData: fileData })
      .pipe(map(response => response));
  }

  searchInventory(
    searchTerm,
    pageSize: number = 10,
    page: number = 1,
    sort: string = 'id',
    dir: string = 'desc'
  ) {
    const base64Encoded = encodeURIComponent(searchTerm);
    let query = `?searchTerm=${base64Encoded}&limit=${pageSize}&offset=${page -
      1}$sort=${sort}&dir=${dir}`;
    return this.authHttp
      .get(`${this.deviceInventoriesUrl + query}`)
      .pipe(map(response => response));
  }

  downloadCsv() {
    this.authShortlivedCookie.setFor(5);

    window.location.href = `${this.deviceInventoriesUrl}/downloadCsv`;
  }
}
