import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { Pagination } from '../models/pagination';
import { DevicePolicyService } from '../create-device-policy/device-policy.service';
import { PreserveSearch } from '../models/preserve-search';
import { Subject, Subscription } from 'rxjs';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { AccessPolicyService } from '../create-access-policy/access-policy.service';
import { DeviceOSEnum } from '../shared/emuns/client.enum';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { OtpAuthenticationService } from '../otp-verification/otp-authentication.service';

@Component({
  selector: 'app-device-policy',
  templateUrl: './device-policy.component.html',
  styleUrls: ['./device-policy.component.css']
})
export class DevicePolicyComponent implements OnInit {
  showHelpText: boolean = false;
  spaces = 0;
  pagination = new Pagination();
  $searchPolicy = new Subject();
  policies: any = {};
  searchKeyword;
  private Search: PreserveSearch = new PreserveSearch();
  isNavigatingToSeach: boolean = false;
  loading: Boolean = true;
  notAllCientsInSearch = true;

  pageSize = [10, 25, 50, 100, 200];

  preserveSearch = new PreserveSearch();
  navigationToEdit: boolean = false;

  clients: any = [];
  $clientSearch = new Subject();
  isClientSearching = false;
  clientOffset = 0;
  selectedClientIds = [];
  currentClientSearch = '';

  allClientPolicies: boolean = false;

  public alert: GlobalMessageClass;
  isDarkMode: boolean;
  showOtpDialog: boolean = false;
  otptoken: string = '';
  option: string;
  private otpStatusSubscription: Subscription;

  constructor(
    private router: Router,
    private deviceService: DevicePolicyService,
    private accessService: AccessPolicyService,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog,
    private titleService: Title,
    private OtpAuthenticationService: OtpAuthenticationService
  ) {}

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.titleService.setTitle(
      'Policy Management | Device Policy | Security Policy'
    );
    let saveSearch = this.preserveSearch.getSearch('devicePolicySaveSearch');
    if (saveSearch) {
      this.pagination = saveSearch.pagination;
      this.searchKeyword = saveSearch.searchKeyword;
    }
    this.subscibeToSearch();
    this.pagination.id = 'device';
    let devicePolicySearch = this.Search.getSearch('devicePolicySearch');
    if (devicePolicySearch) {
      this.pagination.currentPage = devicePolicySearch.currentPage;
      this.searchKeyword = devicePolicySearch.searchKey;
      this.$searchPolicy.next(this.searchKeyword);
    } else {
      this.getDevicePolicies();
    }
    this.getClients();
    this.subscribeClientSearch();
  }

  getClients() {
    this.currentClientSearch == null ? '' : this.currentClientSearch;
    this.accessService
      .getClients(this.clientOffset, this.currentClientSearch)
      .subscribe((data: any) => {
        if (this.spaces < 5 && data.length === 0) {
          this.notAllCientsInSearch = false;
          this.spaces += 1;
          this.clients = [
            ...this.clients,
            { type: 'clientGroup', id: '', name: '', disabled: true }
          ];
        }
        this.clients = [...this.clients, ...data];
      });
  }

  fetchMoreClients() {
    if (this.notAllCientsInSearch) {
      this.clientOffset += 1;
      this.getClients();
    }
  }

  clientFocus() {
    this.currentClientSearch = '';
    this.clients = [];
    this.notAllCientsInSearch = true;
    this.clientOffset = 0;
    this.getClients();
  }

  searchPolicyByClientName() {
    this.getDevicePolicies();
  }

  subscribeClientSearch() {
    this.$clientSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.isClientSearching = true;
          this.spaces = 0;
          this.currentClientSearch = term;
          this.clientOffset = 0;
          return this.accessService.getClients(
            this.clientOffset,
            this.currentClientSearch
          );
        })
      )
      .subscribe(data => {
        this.isClientSearching = false;
        this.clients = data;
      });
  }

  subscibeToSearch() {
    this.$searchPolicy
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap(data => {
          this.loading = true;
          this.searchKeyword = data;
          this.pagination.currentPage = 1;
          return this.deviceService.getAllPolicies(
            this.pagination,
            this.searchKeyword,
            this.selectedClientIds
          );
        })
      )
      .subscribe(policies => {
        const { data, recordsTotal } = policies as any;
        this.policies = policies;
        this.pagination.totalItems = recordsTotal;
        this.loading = false;
      });
  }

  private getDevicePolicies() {
    this.loading = true;
    this.deviceService
      .getAllPolicies(
        this.pagination,
        this.searchKeyword,
        this.selectedClientIds,
        this.allClientPolicies
      )
      .subscribe(policies => {
        this.loading = false;
        const { data, recordsTotal } = policies;
        this.policies = policies || {};
        this.pagination.totalItems = recordsTotal || 0;
      });
  }

  removeLastComma(name: string): string {
    return name && name.substring(0, name && name.length - 2);
  }

  clearfun() {
    this.searchKeyword = '';
    this.getDevicePolicies();
  }

  deleteHandler(id) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Are you sure you want to delete?`
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        let mfa = localStorage.getItem('adminMfaValue');
        if (mfa === 'true') {
          this.showOtpDialog = true;
          this.option = 'delete';
          this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckDel$.subscribe(
            (res: any) => {
              if (res == true) {
                this.deviceService.deletePolicy(id).subscribe(res => {
                  this.alert = {
                    closed: false,
                    type: 'success',
                    message: 'Policy has been deleted successfully.'
                  };
                  this.globalMessageService.pushAlert(this.alert);
                  this.getDevicePolicies();
                });
                this.showOtpDialog = false;
                this.OtpAuthenticationService.setOtpStatusValueDel(false);
              }
            }
          );
        } else {
          this.deviceService.deletePolicy(id).subscribe(res => {
            this.alert = {
              closed: false,
              type: 'success',
              message: 'Policy has been deleted successfully.'
            };
            this.globalMessageService.pushAlert(this.alert);
            this.getDevicePolicies();
          });
        }
      }
    });
  }

  enablePolicy(policy: any, value: any) {
    this.deviceService.enablePolicy(policy.id, value).subscribe(res => {
      policy.is_enabled = !policy.is_enabled;
    });
  }

  addDevicePolicy() {
    this.router.navigate(['/create-device-policy']);
  }

  editPolicy(id) {
    let mfa = localStorage.getItem('adminMfaValue');
    if (mfa === 'true') {
      this.showOtpDialog = true;
      this.option = 'edit';
      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckEdit$.subscribe(
        (res: any) => {
          if (res == true) {
            const search = {
              pagination: this.pagination,
              searchKeyword: this.searchKeyword
            };
            this.preserveSearch.saveSearch('devicePolicySaveSearch', search);
            this.navigationToEdit = true;
            this.router.navigate([`/devicePolicy/${id}`]);
            this.OtpAuthenticationService.setOtpStatusValueEdit(false);
          }
        }
      );
    } else {
      const search = {
        pagination: this.pagination,
        searchKeyword: this.searchKeyword
      };
      this.preserveSearch.saveSearch('devicePolicySaveSearch', search);
      this.navigationToEdit = true;
      this.router.navigate([`/devicePolicy/${id}`]);
    }
  }

  ngOnDestroy(): void {
    if (!this.navigationToEdit) {
      this.preserveSearch.saveSearch('devicePolicySaveSearch', '');
    }

    if (this.otpStatusSubscription) {
      this.otpStatusSubscription.unsubscribe();
    }
  }

  pageChange(event) {
    this.pagination.currentPage = event;
    this.getDevicePolicies();
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getDevicePolicies();
  }

  allClientPoliciesToggle(check) {
    this.allClientPolicies = check;
    this.getDevicePolicies();
  }

  getOsName(os) {
    switch (os) {
      case DeviceOSEnum.Ubuntu18:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer18:
        return 'Ubuntu Server';
      case DeviceOSEnum.Macintosh:
        return 'Macintosh';
      case DeviceOSEnum.Windows:
        return 'Windows';
      case DeviceOSEnum.Ubuntu:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer:
        return 'Ubuntu Server';
      case DeviceOSEnum.WindowsServer:
        return 'Windows Server';
      case DeviceOSEnum.IOS:
        return 'iOS';
      case DeviceOSEnum.Android:
        return 'Android';
      case DeviceOSEnum.UbuntuServer20armv8:
        return 'Ubuntu Server (ARMv8)';
      case DeviceOSEnum.CentosServer:
        return 'CentOS Server';
      case DeviceOSEnum.Unknown:
        return 'Unknown';
      default:
        return os;
    }
  }

  clearSearch() {
    this.currentClientSearch = '';
    this.clients = [];
    this.notAllCientsInSearch = true;
    this.clientOffset = 0;
    this.getClients();
  }

  cancelVerificationPassword(event: any) {
    this.showOtpDialog = false;
    this.OtpAuthenticationService.setOtpStatusValueEdit(false);
    this.OtpAuthenticationService.setOtpStatusValueDel(false);
  }

  submitVerificationPassword(event: any) {
    this.otptoken = event;
    // this.otptoken = event
  }

  getClosePopup(event: any) {
    this.showOtpDialog = false;
  }
}
