import { Component, OnInit } from '@angular/core';
import { DevicesReportsService } from './reporting.service';
import { Pagination } from '../models/pagination';
import { Router } from '@angular/router';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-devices-reports',
  templateUrl: './devices-reports.component.html',
  styleUrls: ['./devices-reports.component.css']
})
export class DevicesReportsComponent implements OnInit {
  ReportsOfAllUnassignedDevices: any = [];

  serviceSortDir: string = 'asc';
  serviceSort: string;

  summaryOfAllServicesPagination: Pagination = new Pagination();
  reportsOfAllUnassignedDevicesLoading: boolean = true;
  pageSizeSummaryOfServices = [10, 25, 50, 100, 200];

  constructor(
    private router: Router,
    private DevicesReportsService: DevicesReportsService,
    private titleService: Title
  ) {}

  ngOnInit() {
    this.titleService.setTitle('Reports | Devices Reports');
    this.summaryOfAllServicesPagination.id = 'devices_reports';
    this.getReportsOfAllUnassignedDevices(this.summaryOfAllServicesPagination);
  }

  ngAfterViewInit() {}

  ngOnDestroy() {}

  getReportsOfAllUnassignedDevices(pagination: Pagination) {
    this.reportsOfAllUnassignedDevicesLoading = true;
    this.DevicesReportsService.getDevicesReports(
      pagination,
      this.serviceSortDir,
      this.serviceSort
    ).subscribe({
      next: summary => {
        this.summaryOfAllServicesPagination.totalItems = summary.recordsTotal;
        this.ReportsOfAllUnassignedDevices = summary;
        this.reportsOfAllUnassignedDevicesLoading = false;
      },
      error: err => {
        this.reportsOfAllUnassignedDevicesLoading = false;
      }
    });
  }

  pageChangeSummaryOfAllServices(page: any) {
    this.summaryOfAllServicesPagination.currentPage = page;
    this.getReportsOfAllUnassignedDevices(this.summaryOfAllServicesPagination);
  }

  sortServiceRecord(term: string) {
    this.serviceSortDir = this.serviceSortDir == 'asc' ? 'desc' : 'asc';
    this.serviceSort = term;
    
    this.getReportsOfAllUnassignedDevices(this.summaryOfAllServicesPagination);
  }

  exportCSV() {
    this.DevicesReportsService.exportCSV();
  }

  viewDevice(id) {
    this.router.navigate(['/view-device', id]);
  }
}
