import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { ConfigService } from '../services/config.service';
import * as moment from 'moment';
import { AuthShortlivedCookieService } from '../auth-shortlived-cookie.service';
import { Pagination } from '../models/pagination';
import { map } from 'rxjs/operators';

interface date {
  year: string;
  month: string;
  day: string;
}
@Injectable()
export class DevicesReportsService {
  private apiBaseUrl: string;
  private reportingApiUrl: string = '/reports';

  constructor(
    private authHttp: AuthHttp,
    private congigService: ConfigService,
    private authShortlivedCookie: AuthShortlivedCookieService
  ) {
    this.apiBaseUrl = this.congigService.getConfiguration().apiBaseUrl;
    this.reportingApiUrl = this.apiBaseUrl + this.reportingApiUrl;
  }

  getDevicesReports(pagination: Pagination, dir: string, sort: string): any {
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;

    return this.authHttp
      .get(
        this.reportingApiUrl +
          '/devices/reports?offset=' +
          currentPage +
          '&limit=' +
          itemsPerPage +
          '&dir=' +
          dir +
          '&sort=' +
          sort
      )
      .pipe(map((response: Response) => response));
  }

  exportCSV() {
    this.authShortlivedCookie.setFor(5);
    window.location.href = `${this.reportingApiUrl}/devices/export-csv`;
  }
}
