import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { map } from 'rxjs/operators';
import { ConfigService } from '../services/config.service';

@Injectable()
export class ForbiddenSoftwareServices {
  private forbiddenSoftwareApiUrl: string;
  constructor(
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    const baseUrl = configService.getConfiguration().apiBaseUrl;
    this.forbiddenSoftwareApiUrl = baseUrl + '/softwares';
  }

  getForbiddenSoftwares(
    pageSize: number = 10,
    page: number = 1,
    searchTerm: string = '',
    sort = 'id',
    dir = 'desc'
  ) {
    let query = `?limit=${pageSize}&offset=${page -
      1}&searchTerm=${searchTerm}&sort=${sort}&dir=${dir}`;
    return this.authHttp
      .get(`${this.forbiddenSoftwareApiUrl}/forbidden-softwares${query}`)
      .pipe(
        map(response => response),
        map((response: any) => {
          response.data = response.data.map(d => {
            return {
              ...d
            };
          });
          return response;
        })
      );
  }

  searchForbiddenSoftwares(
    searchQuery: string,
    pageSize: number = 10,
    page: number = 1,
    sort: string = 'id',
    dir: string = 'desc'
  ) {
    const encodedUrlComponent = encodeURIComponent(searchQuery);
    let query =
      `?limit=${pageSize}` +
      `&offset=${page - 1}` +
      `&sort=${sort}` +
      `&dir=${dir}` +
      `&searchTerm=${encodedUrlComponent}`;

    return this.authHttp
      .get(`${this.forbiddenSoftwareApiUrl}/forbidden-softwares${query}`)
      .pipe(map(response => response));
  }

  bulkActions(selectAll = false, selectedIds, searchKeyword) {
    return this.authHttp.put(
      `${this.forbiddenSoftwareApiUrl}/bulk-actions-forbidden`,
      {
        selectAll,
        selectedIds: Array.from(selectedIds),
        searchKeyword
      }
    );
  }
}
