import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { User } from '../models/user';
import { UserService } from '../users/user.service';
import { ForgotService } from '../forgot-password/forgot.service';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import {ConfigService} from '../services/config.service';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-forgot-password',
  templateUrl: './forgot.component.html',
  styleUrls: ['./forgot.component.css']
})
export class ForgotComponent implements OnInit {
  userEmail = null;
  loading = false;
  error = null;
  isDarkMode: any;
  logo: any;
  constructor(
    private router: Router,
    private forgotService: ForgotService,
    private globalMessageService: GlobalMessageService,
    private configService: ConfigService,
    private titleService: Title
  ) {
    this.titleService.setTitle('Reset Password');
  }

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('modeDark'));
    this.logo = this.isDarkMode ? '/assets/images/logo-dark.png' : '/assets/images/invisily-logo.png'
  }

  forgot() {
    this.loading = true;
    this.error = null;

    this.forgotService.sendMail(this.userEmail).subscribe((result: any) => {
      this.loading = false;

      if (result.success) {
        const alert = {
          type: 'success',
          message: `An email with further instructions for account recovery has been sent to ${this.userEmail}.`,
          closed: true
        };

        this.globalMessageService.clearAlerts();
        this.globalMessageService.pushAlert(alert);
        this.router.navigate(['/login']);
      } else {
        this.globalMessageService.clearAlerts();
        this.error = result.message;
      }
    }, error => {
      console.log(error);
    });
  }
}
