import { Injectable } from '@angular/core';

import { Router } from '@angular/router';
import { ConfigService } from '../services/config.service';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { HttpClient, HttpHeaders } from '@angular/common/http';

@Injectable()
export class ForgotService {
  private authenticationUrl = '/authentication';
  public allow: string;
  private apiBaseUrl: string;
  httpOptions = {
    headers: new HttpHeaders({ 'Content-Type': 'application/json' })
  };

  constructor(
    private httpClient: HttpClient,
    private router: Router,
    private configService: ConfigService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.authenticationUrl = this.apiBaseUrl + this.authenticationUrl;
  }

  sendMail(username: string): Observable<boolean | any> {
    return this.httpClient
      .post(
        this.authenticationUrl + '/forgotPassword',
        JSON.stringify({ username: username }),
        this.httpOptions
      )
      .pipe(map((response: Response) => response));
  }
}
