import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { finalize } from 'rxjs/operators';
import { GatewayService } from '../../gateway/gateway.service';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { ManageService } from '../../manage-service/manage.service';
import { Gateway } from '../../models/gateway';
import { Pagination } from '../../models/pagination';
import { Service } from '../../models/service';
import { GatewayGroupsService } from '../gateway-groups.service';
import { DeviceOSEnum } from '../../shared/emuns/client.enum';
import { ConfirmModalComponent } from '../../../app/shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-gatway-group',
  templateUrl: './create-gatway-group.component.html',
  styleUrls: ['./create-gatway-group.component.css']
})
export class CreateGatwayGroupComponent implements OnInit {
  groupName = '';
  groupDescription = '';
  groupId: number;
  gateways: Gateway[] = [];
  selectedGateways = [];
  services: Service[] = [];
  selectedServices = [];
  loading: boolean = false;
  updateMode: boolean = false;
  Object = Object;
  
  // allServicesSelected: boolean = false
  // allGatewaysSelected: boolean = false

  // All Gateways component
  allGateways = [];
  allGatewaysPagination: Pagination = new Pagination();
  allGatewaysSort: string = 'createdAt';
  allGatewaysSortDir: string = 'desc';
  allGatewaysSearch: string = '';
  allSelectedGateways = {};
  selectAllExcept = new Set();
  allGatewaysSelected: boolean = false;
  disableAddAllGateways: boolean = true;
  gatewaysLoading: boolean = false;
  selectedProtocol = null;
  selectedOs = null;

  // All Current Gateways variables

  currentGateways = [];
  filteredCurrentGateways = [];
  currentGatewaysPagination: Pagination = new Pagination();
  currentGatewaysSort: string = 'createdAt';
  currentGatewaysSortDir: string = 'desc';
  currentGatewaysSearch: string = '';
  currentSelectedGateways = {};
  selectCurrentExcept = new Set();
  allCurrentGatewaysSelected: boolean = false;
  disableDeleteCurrentGateways: boolean = true;
  currentLoading: boolean = false;
  currentSelectedProtocol = null;
  currentSelectedOs = null;

  pageSize = [10, 25, 50, 100, 200];
  protocols = [];
  os = [];

  isDarkMode : boolean;
  
  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private manageServices: ManageService,
    private gatewayServices: GatewayService,
    private gateWayGroupServices: GatewayGroupsService,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog,
    private titleService: Title
  ) {}

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    const groupId = this.route.snapshot.params.id;
    if (groupId) {
      this.gateWayGroupServices
        .getGatewaysGroupdetail(groupId)
        .subscribe(gatewayGroup => {
          this.groupName = gatewayGroup.name;
          this.groupId = gatewayGroup.id;
          this.groupDescription = gatewayGroup.description;
          this.selectedGateways = gatewayGroup.gateways.map(
            gateway => gateway.id
          );
          this.currentGateways = gatewayGroup.gateways;
          this.getPaginatedCurrentGateways();
        });
      this.updateMode = true;
    }

    if (this.updateMode) {
      this.titleService.setTitle('Infrastructure | Gateways Groups | Edit Gateway Group');
    }
    else {
      this.titleService.setTitle('Infrastructure | Gateways Groups | Add Gateway Group');
    }

    this.gatewayServices
      .getGatewayOsAndProtocols()
      .subscribe((response: any) => {
        this.protocols = response.protocols;
        this.os = (response.os || []).map(x => this.appendrightOs(x));
      });

    this.getGatewaysWithPagination();
  }

  getGatewaysOs() {
    const allOss = [
      { value: DeviceOSEnum.UbuntuServer18, display: 'Ubuntu Server' },
      { value: DeviceOSEnum.UbuntuServer20armv8, display: 'Ubuntu Server (ARMv8)' },
      { value: DeviceOSEnum.CentosServer, display: 'CentOS Server' }
    ];
    return (
      allOss.find(myOs => myOs.display === this.selectedOs) || { value: null }
    );
  }

  getGatewaysWithPagination() {
    const os = this.getGatewaysOs();

    this.gatewayServices
      .getGateways(
        this.allGatewaysPagination.currentPage,
        this.allGatewaysPagination.itemsPerPage,
        'active',
        this.allGatewaysSort,
        this.allGatewaysSortDir,
        this.allGatewaysSearch,
        [],
        this.selectedProtocol,
        os.value
      )
      .subscribe(result => {
        this.allGatewaysPagination.totalItems = result.recordsTotal;
        const self = this;
        this.allGateways = result.data.map(service => {
          return {
            ...service,
            isChecked:
              self.allGatewaysSelected && !self.selectAllExcept.has(service.id)
                ? true
                : self.allSelectedGateways[service.id]
                ? true
                : false
          };
        });
        // this.allGatewaysSelected = this.allGateways.every(service => service.isChecked)
        this.gatewaysLoading = false;
        this.checkAlreadyAddedGateways();
      });
  }

  allGatewaysPageChange(page) {
    this.allGatewaysPagination.currentPage = page;
    this.getGatewaysWithPagination();
  }

  handleAllGatewaysSearch() {
    this.allGatewaysSelected = false;
    this.selectAllExcept.clear();
    this.allGatewaysPagination.currentPage = 1;
    this.getGatewaysWithPagination();
    // this.checkAlreadyAddedServices()
  }

  sortAllGateways(key) {
    this.allGatewaysSort = key;
    this.allGatewaysSortDir =
      this.allGatewaysSortDir == 'desc' ? 'asc' : 'desc';
    this.allGatewaysPagination.currentPage = 1;
    this.getGatewaysWithPagination();
    this.checkAlreadyAddedGateways();
  }

  allGatewaysPageSizeChange(event) {
    this.allGatewaysPagination.itemsPerPage = event;
    this.getGatewaysWithPagination();
    this.checkAlreadyAddedGateways();
  }

  selectAllGateways() {
    this.allGateways = this.allGateways.map(service => {
      return {
        ...service,
        isChecked: this.allGatewaysSelected
      };
    });
    // if(this.allGatewaysSelected)
    // {
    //   // normalizing selected to save
    //   const normalize = this.allGateways.reduce((prev, current) => {
    //     prev[current.id] = current;
    //     return prev;
    //   }, {})
    //   const self = this
    //   this.allSelectedGateways = {...normalize, ...self.allSelectedGateways}
    // }
    // else{
    //   this.allGateways.forEach(service => {
    //     delete this.allSelectedGateways[service.id]
    //   })
    // }
    this.selectAllExcept.clear();
    this.allSelectedGateways = {};
    this.updateSelectedCount();
    if (this.allGatewaysSelected) {
      this.disableAddAllGateways = false;
    }
  }

  updateSelectedCount() {
    let disable = Object.keys(this.allSelectedGateways).length < 1;
    if (this.allGatewaysSelected) {
      disable = false;
    }
    this.disableAddAllGateways = disable;
  }

  changeAllGatewayCheckBox(gateway) {
    if (this.allGatewaysSelected) {
      if (this.selectAllExcept.has(gateway.id)) {
        this.allGateways = this.allGateways.map(myGateway => {
          return {
            ...myGateway,
            isChecked: gateway.id === myGateway.id ? true : myGateway.isChecked
          };
        });
        this.selectAllExcept.delete(gateway.id);
      } else {
        this.allGateways = this.allGateways.map(myGateway => {
          return {
            ...myGateway,
            isChecked: gateway.id === myGateway.id ? false : myGateway.isChecked
          };
        });
        this.selectAllExcept.add(gateway.id);
      }
    } else {
      if (gateway.isChecked) {
        this.allSelectedGateways[gateway.id] = gateway;
      } else {
        delete this.allSelectedGateways[gateway.id];
      }
    }

    this.updateSelectedCount();
  }

  addSelectedGateways() {
    if (this.allGatewaysSelected) {
      const os = this.getGatewaysOs();

      this.gatewayServices
        .getGateways(
          this.allGatewaysPagination.currentPage,
          this.allGatewaysPagination.itemsPerPage,
          'active',
          this.allGatewaysSort,
          this.allGatewaysSortDir,
          this.allGatewaysSearch,
          [],
          this.selectedProtocol,
          os.value,
          false,
          true,
          Array.from(this.selectAllExcept)
        )
        .subscribe(response => {
          const currentIds = this.currentGateways.map(service => service.id);
          let data = response.data.filter(
            service => !currentIds.includes(service.id)
          );
          this.allGateways = this.allGateways.map(gateway => {
            return {
              ...gateway,
              isChecked: false
            };
          });
          this.currentGateways = [...this.currentGateways, ...data];
          this.allGatewaysSelected = false;
          this.allSelectedGateways = {};
          this.updateCurrentGatewaysCount();
          this.updateSelectedCount();
          this.gatewaysLoading = false;
          this.getPaginatedCurrentGateways();
          this.checkAlreadyAddedGateways();
          // this.allGatewaysSelected = false
        });
    } else {
      this.gatewaysLoading = true;
      const currentIds = this.currentGateways.map(service => service.id);
      let data = Object.values(this.allSelectedGateways).map((gateway: any) => {
        return {
          ...gateway,
          isChecked: false
        };
      });
      data = data.filter(gateway => !currentIds.includes(gateway.id));
      this.allGateways = this.allGateways.map(gateway => {
        return {
          ...gateway,
          isChecked: false
        };
      });
      this.currentGateways = [...this.currentGateways, ...data];
      this.allGatewaysSelected = false;
      this.allSelectedGateways = {};
      this.updateCurrentGatewaysCount();
      this.updateSelectedCount();
      this.gatewaysLoading = false;
      this.getPaginatedCurrentGateways();
      this.checkAlreadyAddedGateways();
    }
    this.selectAllExcept = new Set();
  }

  // current gateways

  updateCurrentGatewaysCount() {
    this.currentGatewaysPagination.totalItems = this.currentGateways.length;
  }

  updateCurrentSelectedCount() {
    let disable = Object.keys(this.currentSelectedGateways).length < 1;
    if (this.allCurrentGatewaysSelected) {
      disable = false;
    }
    this.disableDeleteCurrentGateways = disable;
  }

  getFilteredCurrentGateways() {
    this.currentLoading = true;
    let gateways = [...this.currentGateways];
    if (this.currentSelectedProtocol) {
      gateways = gateways.filter(
        gateway => gateway.protocol === this.currentSelectedProtocol
      );
    }
    if (this.currentSelectedOs) {
      gateways = gateways.filter(
        gateway =>
          this.appendrightOs(gateway.buildOs) === this.currentSelectedOs
      );
    }

    this.currentGatewaysPagination.totalItems = gateways.length;
    gateways.filter(
      gateway =>
        gateway.Name.toLowerCase().indexOf(
          this.currentGatewaysSearch.toLowerCase()
        ) > -1
    );
    const result = gateways.filter(gateway => gateway.is_unique);
    return result;
  }

  getPaginatedCurrentGateways() {
    this.currentLoading = true;
    const offset =
      this.currentGatewaysPagination.itemsPerPage *
      (this.currentGatewaysPagination.currentPage - 1);
    let gateways = this.getFilteredCurrentGateways();
    this.currentGatewaysPagination.totalItems = gateways.length;
    this.filteredCurrentGateways = gateways.slice(
      offset,
      offset + this.currentGatewaysPagination.itemsPerPage
    );

    if (this.allCurrentGatewaysSelected) {
      this.allCurrentGatewaysSelected = true;
      this.filteredCurrentGateways = this.filteredCurrentGateways.map(
        gateway => {
          return {
            ...gateway,
            isChecked: !this.selectCurrentExcept.has(gateway.id)
          };
        }
      );
    }
    // else{
    //   this.allCurrentGatewaysSelected = false
    // }

    this.currentLoading = false;
    this.checkAlreadyAddedGateways();
    this.updateCurrentGatewaysCount();
    this.updateCurrentSelectedCount();
  }

  checkAlreadyAddedGateways() {
    const currentIds = this.currentGateways.map(gateway => gateway.id);
    this.allGateways = this.allGateways.map(gateway => {
      return {
        ...gateway,
        alreadySelected: currentIds.includes(gateway.id)
      };
    });
  }

  selectAllCurrentGateways() {
    // const paginatedIds = this.filteredCurrentGateways.map(service => service.id)
    // this.currentGateways = this.currentGateways.map(service => {
    //   return paginatedIds.includes(service.id)?
    //     ({...service, isChecked: this.allCurrentGatewaysSelected })
    //       : service
    // })

    // if(this.allCurrentGatewaysSelected)
    // {
    //   // saving all selected services in normalize form
    //   this.filteredCurrentGateways.forEach(gateway => {
    //     this.currentSelectedGateways[gateway.id] = gateway
    //   })
    // }
    // else{
    //   this.currentGateways.forEach(service => {
    //     delete this.currentSelectedGateways[service.id]
    //   })
    // }
    this.currentGateways = this.currentGateways.map(gateway => {
      return {
        ...gateway,
        isChecked: false
      };
    });
    this.currentSelectedGateways = {};
    this.selectCurrentExcept.clear();
    this.updateCurrentSelectedCount();
    this.getPaginatedCurrentGateways();
  }

  changeCurrentCurrentCheckBox(gateway) {
    if (this.allCurrentGatewaysSelected) {
      if (this.selectCurrentExcept.has(gateway.id)) {
        this.selectCurrentExcept.delete(gateway.id);
      } else {
        this.selectCurrentExcept.add(gateway.id);
      }
    } else {
      if (gateway.isChecked) {
        this.currentSelectedGateways[gateway.id] = gateway;
      } else {
        delete this.currentSelectedGateways[gateway.id];
      }
      // this.allCurrentClientsSelected = false
    }
    this.updateCurrentSelectedCount();
  }

  sortCurrentGateways(sort) {
    this.currentGatewaysSort = sort;
    this.currentGatewaysSortDir =
      this.currentGatewaysSortDir == 'desc' ? 'asc' : 'desc';
    this.currentGatewaysPagination.currentPage = 1;
    this.currentGateways = this.currentGateways.sort((a, b) => {
      return this.currentGatewaysSortDir == 'desc'
        ? a[sort].toString().localeCompare(b[sort])
        : b[sort].toString().localeCompare(a[sort]);
    });
    this.getPaginatedCurrentGateways();
  }

  deleteSelectedGateways() {
    this.currentLoading = true;
    let temp = [];
    if (!this.allCurrentGatewaysSelected) {
      this.currentGateways.forEach((gateway, i) => {
        if (gateway.isChecked) {
          delete this.currentSelectedGateways[gateway.id];
        }
        if (!gateway.isChecked) {
          temp.push(gateway);
        }
      });
      this.currentGatewaysPagination.totalItems = temp.length;
    }

    if (this.allCurrentGatewaysSelected) {
      let gatewaysToDelete = this.getFilteredCurrentGateways();
      gatewaysToDelete = gatewaysToDelete.filter(
        gateway => !this.selectCurrentExcept.has(gateway.id)
      );
      let gatewaysIds = gatewaysToDelete.map(gateway => gateway.id);
      temp = this.currentGateways.filter(
        gateway => !gatewaysIds.includes(gateway.id)
      );
    }
    setTimeout(() => {
      this.currentGateways = [...temp];
      this.allCurrentGatewaysSelected = false;
      this.updateCurrentSelectedCount();
      this.currentGatewaysPagination.currentPage = 1;
      this.getPaginatedCurrentGateways();
      this.currentLoading = false;
      this.checkAlreadyAddedGateways();
    }, 200);
    this.selectCurrentExcept = new Set();
  }

  currentGatewaysPageSizeChange(event) {
    if (event) {
      this.currentGatewaysPagination.itemsPerPage = event;
      this.getPaginatedCurrentGateways();
      this.checkAlreadyAddedGateways();
    }
  }

  currentGatewaysPageChange(page) {
    this.currentGatewaysPagination.currentPage = page;
    this.getPaginatedCurrentGateways();
    this.checkAlreadyAddedGateways();
  }

  createGatewayGroup() {
    this.loading = true;
    const currentIds = this.currentGateways.map(gateway => gateway.id);
    this.gateWayGroupServices
      .add_gatewayGroup(this.groupName, currentIds, this.groupDescription)
      .pipe(
        finalize(() => {
          this.loading = false;
        })
      )
      .subscribe(
        response => {
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'success',
            message: 'Gateway group added successfully'
          });
          this.goToGatewayGroups();
        },
        err => {
          this.loading = false;
        }
      );
  }

  handleUpdate() {
    this.loading = true;
    const currentIds = this.currentGateways.map(gateway => gateway.id);
    this.gateWayGroupServices
      .update_gatewayGroup(
        this.groupId,
        this.groupName,
        currentIds,
        this.groupDescription
      )
      .pipe(
        finalize(() => {
          this.loading = false;
        })
      )
      .subscribe(
        response => {
          this.goToGatewayGroups();
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'success',
            message: 'GatewayGroup updated successfully'
          });
        },
        err => {
          this.loading = false;
        }
      );
  }

  saveHandler() {
    if (this.updateMode) {
      this.handleUpdate();
    } else {
      this.createGatewayGroup();
    }
  }

  confirmSelect(): boolean {
    let confirm = false;
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Do you want to select all gateways and services?`,
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
       confirm = true
      }
    });
    return confirm;
  }

  goToGatewayGroups() {
    this.router.navigate(['/gatewayGroups']);
  }

  cancelHandler() {
    this.goToGatewayGroups();
  }

  handleCurrentOsChange(os) {
    this.selectCurrentExcept.clear();
    this.allCurrentGatewaysSelected = false;
    this.currentGatewaysPagination.currentPage = 1;
    this.getPaginatedCurrentGateways();
  }

  handleProtocolChange(protocol) {
    this.selectAllExcept.clear();
    this.allGatewaysSelected = false;
    this.allGatewaysPagination.currentPage = 1;
    this.getGatewaysWithPagination();
  }

  handleCurrentProtocolChange(protocol) {
    this.selectCurrentExcept.clear();
    this.allCurrentGatewaysSelected = false;
    this.currentGatewaysPagination.currentPage = 1;
    this.getPaginatedCurrentGateways();
    this.checkAlreadyAddedGateways();
  }

  handleOsChange(os) {
    this.selectAllExcept.clear();
    this.allGatewaysSelected = false;
    this.allGatewaysPagination.currentPage = 1;
    this.getGatewaysWithPagination();
    this.checkAlreadyAddedGateways();
  }

  appendrightOs(os) {
    switch (os) {
      case DeviceOSEnum.Ubuntu18:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer18:
        return 'Ubuntu Server';
      case DeviceOSEnum.Macintosh:
        return 'Macintosh';
      case DeviceOSEnum.Windows:
        return 'Windows';
      case DeviceOSEnum.Ubuntu:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer:
        return 'Ubuntu Server';
      case DeviceOSEnum.WindowsServer:
        return 'Windows Server';
      case DeviceOSEnum.IOS:
        return 'iOS';
      case DeviceOSEnum.Android:
        return 'Android';
      case DeviceOSEnum.UbuntuServer20armv8:
        return 'Ubuntu Server (ARMv8)';
      case DeviceOSEnum.CentosServer:
        return 'CentOS Server';
      case 'unknown':
        return 'Unknown';
      default:
        return os;
    }
  }
}
