import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { Pagination } from '../../models/pagination';
import { PolicyService } from '../../policy/policy.service';
import { GatewayGroupsService } from '../gateway-groups.service';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-gateway-group-detail',
  templateUrl: './gateway-group-detail.component.html',
  styleUrls: ['./gateway-group-detail.component.css']
})
export class GatewayGroupDetailComponent implements OnInit {
  gatewayGroupId;
  gatewayGroupName;
  gateways = [];
  associatedGroups = [];
  gatewaysPagination: Pagination = new Pagination();
  groupServicePagination: Pagination = new Pagination();

  gatewaysSort: string = 'name';
  gatewaysSortDir: string = 'asc';

  gatewaysSearch: string = '';

  pageSize = [10, 25, 50, 100];
  showConfirm = false;

  showGateways = true;
  selectedGroupId;

  groupServices = [];
  groupServiceSort = 'Name';
  groupServiceSortDir = 'asc';
  groupsLoading = false;

  associatedServices = [];
  associatedServicesPagination: Pagination = new Pagination();
  associatedServicesSort = 'Name';
  associatedServicesSortDir = 'asc';
  allServices: boolean = false;
  allServiceGroups: boolean = false;

  constructor(
    private gateWayGroupServices: GatewayGroupsService,
    private serviceGroupServices: PolicyService,
    private router: Router,
    private route: ActivatedRoute,
    private globalMessageService: GlobalMessageService,
    private titleService: Title
  ) {}

  ngOnInit() {
    this.gatewayGroupId = this.route.snapshot.params.id;
    this.getGatewaysOfGroups();
    // this.getAssociatedServiceGroups()
    this.getAssociatedServices();
  }

  getAssociatedServices() {
    this.gateWayGroupServices
      .getAssociatedServices(
        this.gatewayGroupId,
        this.associatedServicesPagination.currentPage - 1,
        this.associatedServicesPagination.itemsPerPage,
        this.associatedServicesSort,
        this.associatedServicesSortDir
      )
      .subscribe((response: any) => {

        // if(response.all) {
        //   this.allServices = true;
        // } else {
        this.associatedServices = response.data.data;
        this.associatedServicesPagination.totalItems =
          response.data.recordsTotal;
        // }
        this.getAllAssignedServices();
      });
  }

  getAllAssignedServices() {
    for (let i = 0; i < this.associatedGroups.length; i++) {
      if (
          !this.associatedServices.some(
          item => item.id === this.associatedGroups[i].id
        )
      ) {
        this.associatedServices.push(this.associatedGroups[i]);
      }
    }
  }

  getGatewaysOfGroups() {
    this.gateWayGroupServices
      .getGroupsGatewayById(
        this.gatewayGroupId,
        this.gatewaysPagination.currentPage - 1,
        this.gatewaysPagination.itemsPerPage,
        this.gatewaysSort,
        this.gatewaysSortDir,
        this.gatewaysSearch
      )
      .subscribe(response => {
        this.gateways = response.data;
        this.gatewaysPagination.totalItems = response.total;
        this.gatewayGroupName = response.name;
        this.titleService.setTitle('Infrastructure | Gateways Groups | ' + this.gatewayGroupName);
      });
  }

  gatewaysPaginationChange(page) {
    this.gatewaysPagination.currentPage = page;
    this.getGatewaysOfGroups();
  }

  gatewaysPageSizeChange(size) {
    this.gatewaysPagination.itemsPerPage = size;
    this.gatewaysPagination.currentPage = 1;
    this.getGatewaysOfGroups();
  }

  handleGatewaysSearch() {
    this.gatewaysPagination.currentPage = 1;
    this.getGatewaysOfGroups();
  }

  editHandler() {
    this.router.navigate([`/create-gateway-group/${this.gatewayGroupId}`]);
  }

  goBack() {
    this.router.navigate(['/gatewayGroups']);
  }

  deleteHandler() {
    this.showConfirm = false;
    this.gateWayGroupServices.deleteGatewayGroup(this.gatewayGroupId).subscribe(
      response => {
        this.globalMessageService.pushAlert({
          message: `'Gateway Group deleted`,
          type: 'success'
        });
        this.goBack();
      },
      err => {
        this.showConfirm = false;
      }
    );
  }

  showConfirmation() {
    this.showConfirm = true;
  }

  cancelHandler() {
    this.showConfirm = false;
  }

  handlGroupClick(groupId) {
    if (this.selectedGroupId !== groupId) {
      this.groupServicePagination.currentPage = 1;
      this.selectedGroupId = groupId;
      this.groupServices = [];
      this.getServiceByServiceGroupId();
    } else {
      this.selectedGroupId = null;
    }
  }

  getServiceByServiceGroupId() {
    this.groupsLoading = true;
    this.serviceGroupServices
      .getGroupServicesWithPagination(
        this.selectedGroupId,
        this.groupServicePagination.currentPage - 1,
        this.groupServicePagination.itemsPerPage,
        this.groupServiceSort,
        this.groupServiceSortDir
      )
      .subscribe(
        response => {
          this.groupServicePagination.totalItems = response.total;
          this.groupServices = response.data;
          this.groupsLoading = false;
        },
        err => {
          this.groupsLoading = false;
        }
      );
  }

  groupServicePaginationChange(page) {
    this.groupServicePagination.currentPage = page;
    this.getServiceByServiceGroupId();
  }

  associatedServicePaginationChange(page) {
    this.associatedServicesPagination.currentPage = page;
    this.getAssociatedServices();
  }

  sortRecord(term) {
    this.groupServiceSort = term;
    this.groupServiceSortDir =
      this.groupServiceSortDir == 'desc' ? 'asc' : 'desc';
    this.groupServicePagination.currentPage = 1;
    this.getServiceByServiceGroupId();
  }

  sortAssociatedServices(term) {
    this.associatedServicesSort = term;
    this.associatedServicesSortDir =
      this.associatedServicesSortDir == 'desc' ? 'asc' : 'desc';
    this.associatedServicesPagination.currentPage = 1;
    this.getAssociatedServices();
  }

  sortGateways(sort) {
    this.gatewaysSort = sort;
    this.gatewaysSortDir = this.gatewaysSortDir == 'desc' ? 'asc' : 'desc';
    this.gatewaysPagination.currentPage = 1;
    this.getGatewaysOfGroups();
  }
}
