import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp, HttpHeaders } from '@angular/common/http';
import { map } from 'rxjs/operators';
import { AuthenticationService } from '../login/authentication.service';
import { GatewayGroup } from '../models/gatewayGroup';
import { ConfigService } from '../services/config.service';

@Injectable()
export class GatewayGroupsService {
  private gatewaysApiUrl = '/gatewaysGroups';
  private apiBaseUrl: string;
  httpOptions = {
    headers: new HttpHeaders({
      'Content-Type': 'application/json'
    })
  };
  constructor(
    private configService: ConfigService,
    private authenticationService: AuthenticationService,
    private authHttp: AuthHttp
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.gatewaysApiUrl = this.apiBaseUrl + this.gatewaysApiUrl;
  }

  getGatewaysGroups(
    page: number = 1,
    pageSize: number = 10,
    sort: string,
    ascending: boolean,
    searchKey: string = ''
  ): any {
    let query = `?limit=${pageSize}&offset=${page -
      1}&name=${searchKey}&sort=${sort}&dir=${ascending ? 'asc' : 'desc'}`;
    return this.authHttp
      .get(this.gatewaysApiUrl + query)
      .pipe(map((response: Response) => response));
  }

  getAssociatedServiceGroups(groupId) {
    return this.authHttp
      .get(`${this.gatewaysApiUrl}/serviceGroups/${groupId}`)
      .pipe(map(response => response));
  }

  getAssociatedServices(
    groupId,
    offset = 0,
    limit = 10,
    sort = 'Name',
    dir = 'asc'
  ) {
    const query = `?limit=${limit}&offset=${offset}&sort=${sort}&dir=${dir}`;
    return this.authHttp
      .get(`${this.gatewaysApiUrl}/services/${groupId}${query}`)
      .pipe(map(response => response));
  }

  getGroupsGatewayById(groupId, offset, limit, sort, dir, searchKey): any {
    let query = `?limit=${limit}&offset=${offset}&searchKey=${searchKey}&sort=${sort}&dir=${dir}`;
    return this.authHttp
      .get(`${this.gatewaysApiUrl}/gateways/${groupId}${query}`)
      .pipe(map(response => response));
  }

  getAvailableGatewaysGroups(): any {
    return this.authHttp
      .get(this.gatewaysApiUrl + '/dropdown')
      .pipe(map(response => response));
  }

  getGatewaysGroupdetail(id): any {
    return this.authHttp
      .get(this.gatewaysApiUrl + `/${id}`)
      .pipe(map(response => response));
  }

  add_gatewayGroup(name, gateways, description, addAllGateways = false): any {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };

    return this.authHttp
      .post(
        this.gatewaysApiUrl,
        { name, gateways, description, addAllGateways },
        options
      )
      .pipe(map(response => response));
  }

  update_gatewayGroup(
    id,
    name,
    gateways,
    description,
    appendGateways = false,
    addAllGateways = false
  ): any {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    return this.authHttp
      .put(
        this.gatewaysApiUrl + `/${id}`,
        { name, gateways, description, appendGateways, addAllGateways },
        options
      )
      .pipe(map(response => response));
  }

  deleteGatewayGroup(id): any {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    return this.authHttp
      .delete(this.gatewaysApiUrl + `/${id}`, options)
      .pipe(map(response => response));
  }

  bulkDeleteGroup(
    groupIds = [],
    allSelected = false,
    excludeGroups = [],
    search = ''
  ): any {
    const query = `?search=${search}&allSelected=${allSelected}`;
    this.httpOptions['body'] = {
      groupIds: Array.from(groupIds),
      excludeGroups: Array.from(excludeGroups)
    };
    return this.authHttp
      .delete(`${this.gatewaysApiUrl}/bulk${query}`, this.httpOptions)
      .pipe(map(response => response));
  }
}
