import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { GatewayService } from '../gateway.service';
import { Gateway } from '../../models/gateway';
import { Controller } from '../../models/controller';
import { ManageService } from '../../manage-service/manage.service';
import { ControllerService } from '../../controller/controller.service';
import { Service } from '../../models/service';
import { DragulaService } from 'ng2-dragula';
import { Pagination } from '../../models/pagination';
import { PreserveSearch } from '../../models/preserve-search';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { AuthenticationService } from '../../login/authentication.service';
import { GatewayGroupsService } from '../../gateway-groups/gateway-groups.service';
import { ErrorCodeEnum } from '../../shared/emuns/error.enum';
import { tap } from 'rxjs/operators';
import { DeviceOSEnum } from '../../shared/emuns/client.enum';
import { ContextHelpEnum } from '../../shared/emuns/global.enum';
import { LicenseErrorEnum } from '../../shared/emuns/license.enum';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-gateway',
  templateUrl: './create-gateway.component.html',
  styleUrls: ['./create-gateway.component.css']
})
export class CreateGatewayComponent implements OnInit {
  public alert: GlobalMessageClass;
  public malert: GlobalMessageClass;
  gateway = new Gateway();
  public selectedProtocol: string = 'tcp';
  protocols = [
    { value: 'tcp', display: 'TCP' },
    { value: 'udp', display: 'UDP' }
  ];
  public deployment_locations = [
    { value: 0, display: 'Off-Premises' },
    { value: 1, display: 'On-Premises' }
  ];
  public communication_protocol = [
    { value: 0, display: 'TLS' },
    { value: 1, display: 'DTLS' }
  ];

  oss = [
    { value: DeviceOSEnum.UbuntuServer18, display: 'Ubuntu Server' },
    { value: DeviceOSEnum.UbuntuServer20armv8, display: 'Ubuntu Server (ARMv8)' }
    // { value: DeviceOSEnum.CentosServer, display: 'CentOS Server' }
    // { value: DeviceOSEnum.WindowsServer, display: 'Windows Server' }
  ];

  gatewayHardware = '00000000000000000000000000000000';

  services: Service[];
  private servicesList = new Array();
  private selectedService: Service;
  pagination: Pagination = new Pagination();
  private Search: PreserveSearch = new PreserveSearch();
  private assignedServices = {};
  private tempServices: Service[] = [];
  public removeServiceList: Service[] = [];
  public addServiceList: Service[] = [];
  fetchedservices = [];
  public controllers: Controller[];
  public selectedController: string;
  public pageTitle: string = 'Add Gateway';
  public gatewayGroups = [];
  //Contextual Help
  public contextualHelp: Array<any>;
  public showContext: boolean = false;
  //Contextual Help

  public ipLengthError: boolean = false;
  loading: Boolean = false;
  editing = false;
  disableGatewayGroup: Boolean = false;
  disableSaveButton = false;

  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private gatewayService: GatewayService,
    private dragulaService: DragulaService,
    private manageService: ManageService,
    private controllerService: ControllerService,
    private globalMessageService: GlobalMessageService,
    private authenticationService: AuthenticationService,
    private gatewayGroupsService: GatewayGroupsService,
    private titleService: Title
  ) {
    this.contextualHelp = ['', ''];
    this.malert = {
      closed: true,
      type: 'danger',
      message: 'Time must be valid in access restriction.'
    };
    const bag: any = this.dragulaService.find('service-bag');
    if (bag !== undefined) this.dragulaService.destroy('service-bag');
    dragulaService.drop.subscribe(value => {
      this.onDrop(value.slice(1));
    });
    dragulaService.setOptions('service-bag', {
      moves: function(el: any, container: any, handle: any): any {
        if (el.classList.contains('disableListItem')) {
          return false;
        }
        return true;
      }
    });
    // dragulaService.drop().subscribe(({ name, el, target, source, sibling }) => {
    //   this.onDrop([el, target, source, sibling]);
    // });
    // dragulaService.createGroup('service-bag', {
    //   moves: function(el: any, container: any, handle: any): any {
    //     if (el.classList.contains('disableListItem')) {
    //       return false;
    //     }
    //     return true;
    //   }
    // });
  }

  publicIPAdded() {
    if (
      this.gateway.public_ipv4_address &&
      this.gateway.public_ipv4_address.split(',').length > 5
    ) {
      this.ipLengthError = true;
    } else {
      this.ipLengthError = false;
    }
  }

  showContextHelp(name: any): void {
    if (name == ContextHelpEnum.Name) {
      this.contextualHelp = ['Name', 'Descriptive name of the gateway.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.email) {
      this.contextualHelp = [
        'email',
        'Email address of admin user responsible for gateway deployment. Gateway package will be sent to this user.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.ipv4_address) {
      this.contextualHelp = [
        'ipv4_address',
        'IP of the machine where gateway will be deployed.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.port) {
      this.contextualHelp = [
        'port',
        'Port where the gateway will be listening for requests.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.public_ipv4_address) {
      this.contextualHelp = [
        'public_ipv4_address',
        'Forwarded public IP of the gateway.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.public_ipv4_port) {
      this.contextualHelp = ['public_ipv4_port', 'Public port of the gateway.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.spa_port) {
      this.contextualHelp = [
        'spa_port',
        'UDP port where gateway will be listening for SPA requests.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.hardware_id) {
      this.contextualHelp = [
        'hardware_id',
        'Unique ID of the machine the gateway is bound to. This is visible when gateway software starts and should be updated from there.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.selectedController) {
      this.contextualHelp = [
        'selectedController',
        'Assigned controller of the gateway.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.DeviceLocation) {
      this.contextualHelp = ['DL', 'Device Location for gateway.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.buildOs) {
      this.contextualHelp = ['buildOs', 'Supported OS of the gateway.'];
      this.showContext = true;
    } else if(name == ContextHelpEnum.CommProtocol){
      this.contextualHelp = ['Cp', 'Communication protocol for gateway.'];
      this.showContext = true;
    }
  }

  closeContextualHelp(): void {
    this.showContext = false;
  }

  listClicked(service: Service) {
    if (service) {
      if (service.isClientAssigned) {
        this.malert = {
          closed: false,
          type: 'danger',
          message:
            'You cannot unassign the service that has been assigned to the clients.'
        };
      }
    }
  }

  // removeService(ev, service: Service) {
  //   if (ev.target.checked) {
  //     let index = this.removeServiceList.findIndex(
  //       removeServiceIndex => removeServiceIndex.id == service.id
  //     );
  //     if (index === -1) {
  //       this.removeServiceList.push(service);
  //     }
  //   } else {
  //     let index = this.removeServiceList.findIndex(
  //       removeServiceIndex => removeServiceIndex.id == service.id
  //     );
  //     if (index != -1) {
  //       this.removeServiceList.splice(index, 1);
  //     }
  //   }
  //   if (this.removeServiceList.length == 0) {
  //     this.removeServiceList = [];
  //   }
  // }

  addService(ev, service: Service) {
    if (ev.target.checked) {
      let index = this.addServiceList.findIndex(
        addServiceIndex => addServiceIndex.id == service.id
      );
      if (index === -1) {
        this.addServiceList.push(service);
      }
    } else {
      let index = this.addServiceList.findIndex(
        addServiceIndex => addServiceIndex.id == service.id
      );
      if (index != -1) {
        this.addServiceList.splice(index, 1);
      }
    }
    if (this.addServiceList.length == 0) {
      this.addServiceList = [];
    }
  }

  addSelectedIndex() {
    for (var i in this.addServiceList) {
      this.assignedServices[this.addServiceList[i].id] = this.addServiceList[
        i
      ].id;

      let index = this.services.findIndex(
        service => service.id == this.addServiceList[i].id
      );

      let findex = this.fetchedservices.findIndex(
        fetchedservice => fetchedservice.id == this.services[index].id
      );
      if (findex === -1) {
        this.fetchedservices.push(this.services[index]);
      }
      this.services.splice(index, 1);
    }
    this.addServiceList = [];
  }

  jsonCopy(src): any {
    return JSON.parse(JSON.stringify(src));
  }

  private onDrop(args) {
    let [el, target, source, sibling] = args;

    if (source !== target) {
      if (target.id == 'assigned-service') {
        this.assignedServices[el.id] = el.id;
        let index = this.services.findIndex(service => service.id == el.id);
        let findex = this.fetchedservices.findIndex(
          fetchedservice => fetchedservice.id == this.services[index].id
        );
        if (findex === -1) {
          this.fetchedservices.push(this.services[index]);
        }
        this.services.splice(index, 1);
      }

      if (target.id == 'available-service') {
        delete this.assignedServices[el.id];
        let index = this.fetchedservices.findIndex(
          fetchedservice => fetchedservice.id == el.id
        );
        if (this.fetchedservices[index].isClientAssigned !== true) {
          let sindex = this.services.findIndex(
            service => service.id == this.fetchedservices[index].id
          );
          if (sindex === -1) {
            this.services.push(this.fetchedservices[index]);
          }
          this.fetchedservices.splice(index, 1);
        }
      }
    }
  }

  getPage(page: number) {
    let ref = this;
    this.loading = true;
    this.pagination.currentPage = page;
    this.manageService
      .getAllServicesForGateway()
      .pipe(
        tap((res: any) => {
          this.services = res.services;
          this.pagination.totalItems = res.total;
          this.pagination.currentPage = page;

          this.controllerService
            .getAllControllers()
            .pipe(
              tap((res: any) => {
                this.controllers = res.data;
                this.selectedController = res.data[0].id;

                if (typeof this.route.params['_value']['id'] != 'undefined') {
                  this.pageTitle = 'Edit Gateway';
                  this.editing = true;
                  this.gatewayService
                    .getgatewaydetail(this.route.params['_value']['id'])
                    .subscribe(
                      gateway => {
                        this.loading = false;
                        if (!gateway[0]) {
                          this.alert = {
                            closed: false,
                            type: 'danger',
                            message: 'No gateway found with the specified ID.'
                          };
                          this.globalMessageService.pushAlert(this.alert);
                          this.router.navigate(['/gateways']);
                        } else {
                          let g = gateway[0];
                          this.gateway = g;
                          this.gateway.communication_protocol = g.security_protocol == 'TLS' ? 
                          this.communication_protocol[0].value : this.communication_protocol[1].value;
                          this.gatewayHardware = g.hardware_id;
                          this.fetchedservices = g.Services;
                          this.selectedController = g.Controllers[0].id;
                          this.selectedProtocol = g.protocol;
                          this.fetchedservices.forEach(function(
                            fetchedservice,
                            index
                          ) {
                            ref.assignedServices[fetchedservice.id] =
                              fetchedservice.id;
                            for (var i = 0; i < ref.services.length; i++) {
                              if (ref.services[i]['id'] == fetchedservice.id) {
                                ref.services.splice(i, 1);
                              }
                            }
                          });
                        }
                      },
                      err => {
                        this.loading = false;
                      }
                    );
                } else {
                  this.loading = false;
                  this.editing = false;
                  this.pageTitle = 'Add Gateway';
                  this.gateway.deployment_location = 1;
                }
                this.titleService.setTitle('Infrastructure | Gateways | ' + this.pageTitle);
              })
            )
            .subscribe({
              next: next => {},
              error: err => {
                this.loading = false;
                if (err.status === ErrorCodeEnum.Forbidden) {
                  if (err._body == LicenseErrorEnum.Invalid) {
                    this.authenticationService.showLicenseOnly(true);
                    this.router.navigate(['/license']);
                  } else {
                    this.alert = {
                      closed: false,
                      type: 'danger',
                      message: 'You are not authorized to access this page.'
                    };
                    this.globalMessageService.pushAlert(this.alert);
                  }
                }
              },
              complete: () => {}
            });
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          this.loading = false;
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'You are not authorized to access this page.'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          }
        },
        complete: () => {}
      });
  }

  ngOnInit() {
    this.disableSaveButton = false;
    let serviceSearch = this.Search.getSearch('serviceSearch');
    if (serviceSearch) {
      this.pagination.currentPage = serviceSearch.current_page;
    }
    this.getPage(this.pagination.currentPage);
    this.gateway.buildOs = this.oss[0].value;
    this.gateway.deployment_location = this.deployment_locations[0].value;
    this.gateway.communication_protocol = this.communication_protocol[0].value;
  }

  onsubmit() {
    this.loading = true;
    this.disableSaveButton = true;
    this.gateway.protocol = this.selectedProtocol;
    this.gateway.hardware_id = this.gatewayHardware;
    this.gateway.port = this.gateway.public_ipv4_port;
    // this.gateway.services = this.assignedServices;
    // this.gateway.gateway_group_id = this.selectedGatewayGroup
    this.gateway.controller = this.selectedController;

    if (this.gateway.public_ipv4_address.split(',').length > 5) {
      this.ipLengthError = true;
    } else {
      this.ipLengthError = false;
    }

    if (this.ipLengthError) {
      this.loading = false;
      this.disableSaveButton = false;
      this.alert = {
        closed: false,
        type: 'danger',
        message: 'Public IP address must not be greater than 5.'
      };
      
      this.globalMessageService.pushAlert(this.alert);
      this.globalMessageService.clearAlerts();
    } else {
      this.gatewayService.add_gateway(this.gateway).subscribe(
        response => {
          this.loading = false;
          if (response.success) {
            if (response.message == 'Subnets not available') {
              this.alert = {
                closed: false,
                type: 'success',
                message:
                  'Subnet not available. Please go to Core Settings -> Address Space Management.'
              };
            } else {
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Gateway has been added/updated successfully.'
              };
            }
            this.globalMessageService.pushAlert(this.alert);
            this.globalMessageService.clearAlerts();
            this.router.navigate(['/gateways']);
          } else {
            // this.alert = {
            //   closed: false,
            //   type: 'danger',
            //   message: response.message
            // };
            // this.globalMessageService.pushAlert(this.alert);
          }
        },
        error => {
          this.disableSaveButton = false;
          this.loading = false;
          this.globalMessageService.clearAlerts();
          // let warningAlert: GlobalMessageClass = {
          //   closed: false,
          //   type: 'danger',
          //   message:
          //     'Unable to add or update gateway because Name, Private ip should be unique',
          //   name: 'uniqueGatewayAlert'
          // };
          // this.globalMessageService.pushAlert(warningAlert);
        }
      );
    }
  }

  onCancel(): void {
    this.router.navigate(['/gateways']);
  }
}
