import { Component, OnInit, EventEmitter, Output, Input } from '@angular/core';
import * as _ from 'lodash';
import { GatewayGroupsService } from '../../../gateway-groups/gateway-groups.service';
import { GlobalMessageService } from '../../../global-messages/global-messages.service';
import { PolicyService } from '../../../policy/policy.service';
import { GatewayService } from '../../gateway.service';

@Component({
  selector: 'create-quick-gateway-group',
  templateUrl: './create-quick-gateway-group.component.html',
  styleUrls: ['./create-quick-gateway-group.component.css']
})

export class CreateQuickGatewayGroupComponent implements OnInit {

  @Output('delete') delete = new EventEmitter<any>();
  @Output('close') close = new EventEmitter<any>();
  @Output('successClose') successClose = new EventEmitter<any>();
  @Input('selectedGateways') selectedGateways = {};
  @Input('allSelected') allSelected = false;
  @Input('isFilterApplied') isFilterApplied = false;

  _: any = _;
  existingMode: boolean = false
  gatewayGroups = []
  selectedGatewayGroup = null
  groupName: string = ''
  groupDescription: string = ''
  loading: boolean = false


  constructor(
    private groupServices: GatewayGroupsService,
    private globalMessageService: GlobalMessageService
  ) {

  }

  ngOnInit() {
    this.close.subscribe((event) => {
    })
    this.groupServices
    .getAvailableGatewaysGroups()
      .subscribe(response => {
        this.gatewayGroups = response
      })
  }

  removeGateway(gateway) {
    delete this.selectedGateways[gateway.id]
  }

  changeToExisting() {
    this.existingMode = !this.existingMode
  }


  handleCreate() {
    this.loading = true
    const all = this.allSelected;
    if(!this.existingMode) {
      const gatewaysIds = Object.keys(this.selectedGateways).map(Number);

      this.groupServices.add_gatewayGroup(this.groupName, gatewaysIds, this.groupDescription, all  ).subscribe(
        response => {
          this.globalMessageService.pushAlert({
            type: 'success',
            message: 'Gateway group added successfully'
          });
          this.successClose.emit()
        },
        error => {
          this.loading = false
        }
      );
    }
    else {
      const gatewaysIds = Object.keys(this.selectedGateways)
      const selectedGroup = this.gatewayGroups.find(group => group.id === this.selectedGatewayGroup)
      this
        .groupServices
        .update_gatewayGroup(
          selectedGroup.id,
          selectedGroup.name,
          gatewaysIds,
          selectedGroup.description,
          true,
          all
        ).subscribe(
        response => {
          this.globalMessageService.pushAlert({
            type: 'success',
            message: 'Gateway group updated successfully'
          });
          this.successClose.emit()
        },
        error => {
          this.loading = false
        }
      );
    }
  }

}
