import { Input, Component, DoCheck, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import * as _ from 'lodash';
import { GlobalMessageClass } from './global-messages';
import { GlobalMessageService } from './global-messages.service';
import { trigger, style, animate, transition } from '@angular/animations';
@Component({
  selector: 'global-messages',
  templateUrl: './global-messages.component.html',
  styleUrls: ['./global-messages.component.css'],
  animations: [
    trigger('items', [
      transition('void => animate', [
        style({ opacity: 0, height: '0px' }),
        animate('0.3s linear', style({ opacity: 1, height: '*' }))
      ]),
      transition('animate => void', [
        style({ opacity: 1 }),
        animate(
          '0.3s linear',
          style({
            opacity: 0,
            height: '0px'
          })
        )
      ])
    ])
  ]
})
export class GlobalMessagesComponent implements OnInit {
  stickyAlerts: Array<GlobalMessageClass>;
  nonStickyAlerts: Array<GlobalMessageClass>;
  private shouldAnimate: boolean;

  constructor(private globalMessageService: GlobalMessageService) {}

  ngOnInit() {
    this.init();
  }

  init(): void {
    this.globalMessageService.shouldAnimate().subscribe(shouldAnimate => {
      this.shouldAnimate = shouldAnimate;
    });
    this.globalMessageService.getAlerts().subscribe(alerts => {
      this.stickyAlerts = alerts.filter(alert => alert.sticky);
      this.nonStickyAlerts = alerts.filter(alert => !alert.sticky);
    });
  }

  public closeAlert(alert: GlobalMessageClass) {
    this.globalMessageService.closeAlert(alert);
  }
}
