import { Injectable } from '@angular/core';
import { GlobalMessageClass } from './global-messages';
import { Observable, BehaviorSubject, timer } from 'rxjs';
import { AlertTimeEnum } from '../shared/emuns/global.enum';
import { ToastrService } from 'ngx-toastr';

@Injectable()
export class GlobalMessageService {
  private _alerts$ = new BehaviorSubject<GlobalMessageClass[]>([]);
  private _shouldAnimate$ = new BehaviorSubject<boolean>(true);

  public alerts$ = this._alerts$.asObservable();
  public shouldAnimate$ = this._shouldAnimate$.asObservable();

  private AUTO_CLOSE_TIME = AlertTimeEnum.AutoCloseTime;

  constructor(private toastr: ToastrService) {}

  clearAlerts(): void {
    this._alerts$.next([]);
  }

  shouldAnimate(): Observable<boolean> {
    return this.shouldAnimate$;
  }

  getAlerts(): Observable<GlobalMessageClass[]> {
    return this.alerts$;
  }

  pushAlert(alert: GlobalMessageClass) {
    this.showNewAlert(alert);
  }

  removeAlertByName(name: string) {
    const alerts = this._alerts$.value;
    const newAlerts = alerts.filter(alert => alert.name !== name);
    this._alerts$.next(newAlerts);
  }

  pushAlertIfNotPresent(alert: GlobalMessageClass) {
    console.log(alert);
    if (!this.isPresent(alert)) {
      this.pushAlert(alert);
    }
  }

  isPresent(alert: GlobalMessageClass) {
    if (alert.name) {
      return (
        this._alerts$.value.map(alert => alert.name).indexOf(alert.name) > -1
      );
    } else {
      return (
        this._alerts$.value.map(alert => alert.message).indexOf(alert.message) >
        -1
      );
    }
  }

  closeAlert(alert: GlobalMessageClass) {
    const alerts = this._alerts$.value;
    const index = alerts.indexOf(alert);
    if (index > -1) {
      const newAlerts = [...alerts.slice(0, index), ...alerts.slice(index + 1)];
      this._alerts$.next(newAlerts);
    }
  }

  getCount(): number {
    return this._alerts$.value.length;
  }

  enableAnimatedAlerts() {
    this._shouldAnimate$.next(true);
  }

  disableAnimatedAlerts() {
    this._shouldAnimate$.next(false);
  }

  showNewAlert(alert: GlobalMessageClass): void {
    if (alert.type === 'success') {
      this.toastr.success(alert.message);
    } else if (alert.type === 'info') {
      this.toastr.info(alert.message);
    } else if (alert.type === 'warning') {
      this.toastr.warning(alert.message);
    } else if (alert.type === 'error' || alert.type === 'danger') {
      this.toastr.error(alert.message);
    }
  }
}
