import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { GroupService } from '../group.service';
import { Group } from '../../models/group';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { DashboardService } from '../../dashboard/dashboard.service';
import { AuthenticationService } from '../../login/authentication.service';
import { Service } from '../../models/service';
import { ManageService } from '../../manage-service/manage.service';
import { DragulaService } from 'ng2-dragula';
import { Observable } from 'rxjs';
import { TrustedHost } from '../../models/trusted-host';
import { LicenseService } from '../../license/license.service';
import { Pagination } from '../../models/pagination';
import { ClientService } from '../../client/client.service';
import { ClientStatusEnum, ClientTypeEnum, DeviceOSEnum } from '../../shared/emuns/client.enum';
import { ContextHelpEnum } from '../../shared/emuns/global.enum';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-group',
  templateUrl: './create-group.component.html',
  styleUrls: ['./create-group.component.css']
})
export class CreateGroupComponent implements OnInit {
  public pageTitle: string = 'Add Group';
  group = new Group();
  ClientStatus = ClientStatusEnum;
  ClientType = ClientTypeEnum;
  DeviceOS = DeviceOSEnum;

  public removeServiceList: Service[] = [];
  public addServiceList: Service[] = [];
  changingMade: boolean = false;

  public assignedServices = {};
  public fetchedservices = [];
  oldservices;

  services: Service[];
  private servicesList = new Array();
  private selectedService: Service;

  public contextualHelp: Array<any>;
  public showContext: boolean = false;
  onSubmit = this.addGroup;
  isLoading = false;
  license: any;

  // Updated via child component
  private localModeHosts: TrustedHost[] = [];
  loading: Boolean = false;

  Object = Object;

  clientsLoading: boolean = false;
  allClientsPagination: Pagination = new Pagination();
  allClientsSearch: string = '';
  allClientsSort = 'id';
  allClientsSortDir = 'asc';
  adOrphand = false;
  localOrphand = false;
  clientType = ClientTypeEnum.Client;
  allClients = [];
  allClientsSelected: boolean = false;
  selectAllExcept = new Set();
  allSelectedClients = {};
  disableAddAllClients: boolean = true;

  currentClientsPagination: Pagination = new Pagination();
  currentClients = [];
  filteredCurrentClients = [];
  currentLoading: boolean = false;
  currentClientsSearch: string = '';
  allCurrentClientsSelected: boolean = false;
  disableDeleteCurrentClients: boolean = true;
  currentSelectedClients = {};
  selectCurrentExcept = new Set();
  currentClientsSort = 'id';
  currentClientsSortDir = 'desc';
  currentSelectedType;
  currentFilterOs;
  disableAddHostButton: boolean = false;
  disableSaveButton = false;

  pageSize = [10, 25, 50, 100, 200];

  public searchFiltersForm: any = {};
  public searchFilters: any = {};

  OShashMap = {
    ubuntu: 'Ubuntu Desktop',
    ubuntuserver: 'Ubuntu Server',
    ubuntu18: 'Ubuntu Desktop',
    ubuntuserver18: 'Ubuntu Server',
    ubuntuserver20armv8: 'Ubuntu Server (ARMv8)',
    centosserver: 'CentOS Server',
    windows: 'Windows',
    windowsserver: 'Windows Server',
    macintosh: 'Macintosh',
    ios: 'iOS',
    android: 'Android',
    unknown: 'Unknown'
  };

  quickFilters = [
    {
      isChecked: false,
      title: 'Allowed',
      value: 'is_active',
      status_value: 1,
      arrayValue: 1
    },
    {
      isChecked: false,
      title: 'Blocked',
      value: 'is_active',
      status_value: 0,
      arrayValue: 2
    },
    // {
    //   isChecked: false,
    //   title: 'Darkened',
    //   value: 'isDarkened',
    //   status_value: 1,
    //   arrayValue: 3
    // },
    // {
    //   isChecked: false,
    //   title: 'Not Darkened',
    //   value: 'isDarkened',
    //   status_value: 0,
    //   arrayValue: 4
    // },
    // {
    //   isChecked: false,
    //   title: 'MFA Enabled',
    //   value: 'isMfa',
    //   status_value: 1,
    //   arrayValue: 5
    // },
    // {
    //   isChecked: false,
    //   title: 'MFA Disabled',
    //   value: 'isMfa',
    //   status_value: 0,
    //   arrayValue: 6
    // },
    {
      isChecked: false,
      title: 'Offline',
      value: 'isOnline',
      status_value: 0,
      arrayValue: 7
    },
    {
      isChecked: false,
      title: 'Online',
      value: 'isOnline',
      status_value: 1,
      arrayValue: 8
    }
    // {
    //   isChecked: false,
    //   title: 'Local Orphans',
    //   value: 'localOrphand',
    //   status_value: 0,
    //   arrayValue: 9
    // },
    // {
    //   isChecked: false,
    //   title: 'AD Orphans',
    //   value: 'adOrphand',
    //   status_value: 0,
    //   arrayValue: 10
    // }
  ];
  quickFilterStatusValues = [];
  types = [
    {
      label: 'Server Side Clients',
      value: 'Server'
    },
    {
      label: 'End User Clients',
      value: 'Client'
    }
  ];
  quickFilterOsValues = [];
  quickFiltersOs = [
    // {
    //   isChecked: false,
    //   title: 'Ubuntu 16 Desktop',
    //   value: DeviceOSEnum.Ubuntu
    // },
    // {
    //   isChecked: false,
    //   title: 'Ubuntu 16 Server',
    //   value: DeviceOSEnum.UbuntuServer
    // },
    {
      isChecked: false,
      title: 'Ubuntu Desktop',
      value: DeviceOSEnum.Ubuntu18
    },
    {
      isChecked: false,
      title: 'Ubuntu Server',
      value: DeviceOSEnum.UbuntuServer18
    },
    {
      isChecked: false,
      title: 'Ubuntu Server (ARMv8)',
      value: DeviceOSEnum.UbuntuServer20armv8
    },
    {
      isChecked: false,
      title: 'CentOS Server',
      value: DeviceOSEnum.CentosServer
    },
    {
      isChecked: false,
      title: 'Windows',
      value: DeviceOSEnum.Windows
    },
    {
      isChecked: false,
      title: 'Windows Server',
      value: DeviceOSEnum.WindowsServer
    },
    {
      isChecked: false,
      title: 'Android',
      value: DeviceOSEnum.Android
    },
    {
      isChecked: false,
      title: 'iOS',
      value: DeviceOSEnum.IOS
    },
    {
      isChecked: false,
      title: 'Macintosh',
      value: DeviceOSEnum.Macintosh
    }
  ];
  quickFiltersShow = false;
  quickFiltersShowState = false;
  quickFiltersShowOs = false;
  quickFilterValues = [];
  quickFilterStatusValue = -1;
  quickFilterOsValue = '-1';

  updateHosts(hosts) {
    this.localModeHosts = hosts;
  }

  constructor(
    private groupService: GroupService,
    private licenseService: LicenseService,
    private globalMessageService: GlobalMessageService,
    private manageService: ManageService,
    private router: Router,
    private route: ActivatedRoute,
    private dragulaService: DragulaService,
    private clientService: ClientService,
    private titleService: Title
  ) {
    const bag: any = dragulaService.find('service-bag');
    if (bag !== undefined) dragulaService.destroy('service-bag');
    dragulaService.drop.subscribe(value => {
      this.onDrop(value.slice(1));
    });
    // dragulaService.drop().subscribe(({ name, el, target, source, sibling }) => {
    //   this.onDrop([el, target, source, sibling]);
    // });
  }

  ngOnInit() {
    this.disableSaveButton = false;
    this.contextualHelp = ['', ''];
    const groupId = this.route.snapshot.params.id;
    const isEditPage = !!groupId;
    this.licenseService.getLicense().subscribe(response => {
      this.license = response.license;
    });

    if (isEditPage) {
      this.pageTitle = 'Edit Group';
      this.onSubmit = this.editGroup;
    }

    this.titleService.setTitle('Client Management | Groups | ' + this.pageTitle);
    this.getPage(groupId);
    this.getAllClientsWithPagination();
  }

  // All Services methods

  getAllClientsWithPagination() {
    this.clientsLoading = true;
    this.clientService
      .getClients(
        this.allClientsPagination.currentPage,
        this.allClientsPagination.itemsPerPage,
        this.searchFilters,
        this.allClientsSort,
        this.allClientsSortDir,
        this.clientType ? this.clientType : '',
        this.adOrphand,
        this.localOrphand
      )
      .subscribe(
        response => {
          this.allClients = response.data;
          this.allClientsPagination.totalItems = response.recordsTotal;

          const self = this;
          this.allClients = response.data.map(client => {
            return {
              ...client,
              isChecked:
                self.allClientsSelected && !self.selectAllExcept.has(client.id)
                  ? true
                  : self.allSelectedClients[client.id]
                  ? true
                  : false
            };
          });
          this.clientsLoading = false;
          this.checkAlreadyAddedClients();
        },
        err => {
          this.clientsLoading = false;
        }
      );
  }

  async getPage(groupId) {
    try {
      this.loading = true;
      if (groupId) {
        this.currentLoading = true;
        const response = await this.groupService
          .getGroupDetails(groupId)
          .toPromise();
        this.group = response;
        const clientsResponse = await this.groupService
          .getCurrentGroupClients(groupId)
          .toPromise();
        this.currentClients = clientsResponse.data;
      }
    } catch (e) {
      console.log('error ', e);
    } finally {
      this.loading = false;
    }
    this.getPaginatedCurrentClients();
  }

  showContextHelp(name: any): void {
    if (name == ContextHelpEnum.Name) {
      this.contextualHelp = ['Name', 'Descriptive name of the group.'];
      this.showContext = true;
    }
  }

  closeContextualHelp(): void {
    this.showContext = false;
  }

  private onDrop(args) {
    let [el, target, source, sibling] = args;

    if (source !== target) {
      this.changingMade = true;
      if (target.id == 'assigned-service') {
        this.assignedServices[el.id] = el.id;
        let index = this.services.findIndex(service => service.id == el.id);
        let findex = this.fetchedservices.findIndex(fetchedservice =>
          index > -1 ? fetchedservice.id == this.services[index].id : false
        );
        if (findex === -1) {
          this.fetchedservices.push(this.services[index]);
        }
        this.services.splice(index, 1);
      }

      if (target.id == 'available-service') {
        delete this.assignedServices[el.id];
        let index = this.fetchedservices.findIndex(
          fetchedservice => fetchedservice.id == el.id
        );
        let sindex = this.services.findIndex(service =>
          index > -1 ? service.id == this.fetchedservices[index].id : false
        );
        if (sindex === -1) {
          this.services.push(this.fetchedservices[index]);
        }
        this.fetchedservices.splice(index, 1);
      }

      let indexForCheckList = this.removeServiceList.findIndex(
        cs => cs.id == el.id
      );
      this.removeServiceList.splice(indexForCheckList, 1);
      indexForCheckList = this.addServiceList.findIndex(cs => cs.id == el.id);
      this.addServiceList.splice(indexForCheckList, 1);
    }
  }

  removeService(ev, service: Service) {
    if (ev.target.checked) {
      let index = this.removeServiceList.findIndex(
        removeServiceIndex => removeServiceIndex.id == service.id
      );
      if (index === -1) {
        this.removeServiceList.push(service);
      }
    } else {
      let index = this.removeServiceList.findIndex(
        removeServiceIndex => removeServiceIndex.id == service.id
      );
      if (index != -1) {
        this.removeServiceList.splice(index, 1);
      }
    }

    if (this.removeServiceList.length == 0) {
      this.removeServiceList = [];
    }
  }

  addService(ev, service: Service) {
    if (ev.target.checked) {
      let index = this.addServiceList.findIndex(
        addServiceIndex => addServiceIndex.id == service.id
      );
      if (index === -1) {
        this.addServiceList.push(service);
      }
    } else {
      let index = this.addServiceList.findIndex(
        addServiceIndex => addServiceIndex.id == service.id
      );
      if (index != -1) {
        this.addServiceList.splice(index, 1);
      }
    }
    if (this.addServiceList.length == 0) {
      this.addServiceList = [];
    }
  }

  addSelectedIndex() {
    this.changingMade = true;
    for (var i in this.addServiceList) {
      this.assignedServices[this.addServiceList[i].id] = this.addServiceList[
        i
      ].id;

      let index = this.services.findIndex(
        service => service.id == this.addServiceList[i].id
      );

      let findex = this.fetchedservices.findIndex(
        fetchedservice => fetchedservice.id == this.services[index].id
      );
      if (findex === -1) {
        this.fetchedservices.push(this.services[index]);
      }
      this.services.splice(index, 1);
    }
    this.addServiceList = [];
  }

  removeSelectedIndex() {
    this.changingMade = true;
    for (var i in this.removeServiceList) {
      let obj: Service = new Service();
      obj.id = this.assignedServices[this.removeServiceList[i].id];
      delete this.assignedServices[obj.id];

      let index = this.fetchedservices.findIndex(
        fetchedservice => fetchedservice.id == this.removeServiceList[i].id
      );
      let sindex = this.services.findIndex(
        service => service.id == this.fetchedservices[index].id
      );
      if (sindex === -1) {
        this.services.push(this.fetchedservices[index]);
      }
      this.fetchedservices.splice(index, 1);
    }
    this.removeServiceList = [];
  }

  selectAll() {
    this.changingMade = true;
    for (var i in this.services) {
      this.assignedServices[this.services[i].id] =
        '' + this.services[i].id + '';
    }
    for (var j = 0; j < this.services.length; j++) {
      this.fetchedservices.push(this.services[j]);
    }
    this.services = [];
  }

  deSelectAll() {
    this.changingMade = true;
    for (var i in this.assignedServices) {
      let obj: Service = new Service();
      obj.id = this.assignedServices[this.assignedServices[i]];
      let index = this.fetchedservices.findIndex(
        fetchedService => fetchedService.id == obj.id
      );
      if (index != -1) {
        this.services.push(this.fetchedservices[index]);
      }
      delete this.assignedServices[this.assignedServices[i]];
    }
    this.fetchedservices = [];
  }

  addGroup() {
    if (this.isMixOfServerAndEndUser()) return;
    this.group.services = this.assignedServices;
    this.isLoading = true;
    this.loading = true;
    const clientIds = this.currentClients.map(client => client.id);
    this.disableSaveButton = true;
    this.groupService
      .addGroup(
        { ...this.group, name: this.group.name.trim() },
        this.localModeHosts,
        clientIds
      )
      .subscribe(
        response => {
          this.loading = false;
          this.globalMessageService.pushAlert({
            type: 'success',
            message: response
          });

          this.isLoading = false;

          this.router.navigate(['/groups']);
        },
        err => {
          this.disableSaveButton = false;
          this.loading = false;
          // this.globalMessageService.pushAlert({
          //   type: 'danger',
          //   message: JSON.parse(err._body)
          // });

          this.isLoading = false;
        }
      );
  }

  isMixOfServerAndEndUser() {
    const containsServer = this.currentClients.some(client => {
      return client.client_type === ClientTypeEnum.Server;
    });
    const containsEndUser = this.currentClients.some(client => {
      return client.client_type === ClientTypeEnum.Client;
    });
    if (containsEndUser && containsServer) {
      this.globalMessageService.pushAlert({
        type: 'danger',
        message: 'Mix of server and end user clients not allowed'
      });
      return true;
    } else return false;
  }

  editGroup() {
    // if(this.isMixOfServerAndEndUser()) return;
    this.group.services = this.assignedServices;
    this.isLoading = true;
    this.loading = true;
    const clientIds = this.currentClients.map(client => client.id);

    this.groupService
      .editGroup(
        this.group.id,
        { ...this.group, name: this.group.name.trim() },
        this.changingMade,
        clientIds
      )
      .subscribe(
        response => {
          this.loading = false;
          this.globalMessageService.pushAlert({
            type: 'success',
            message: response
          });

          this.isLoading = false;
          this.router.navigate(['/groups']);
        },
        err => {
          this.loading = false;
          // this.globalMessageService.pushAlert({
          //   type: 'danger',
          //   message: JSON.parse(err._body)
          // });

          this.isLoading = false;
        }
      );
  }

  onCancel() {
    this.router.navigate(['/groups']);
  }

  allPaginationPageChange(event) {
    this.allClientsPagination.currentPage = event;
    this.getAllClientsWithPagination();
    this.checkAlreadyAddedClients();
  }

  checkAlreadyAddedClients() {
    const currentIds = this.currentClients.map(client => client.id);
    this.allClients = this.allClients.map(service => {
      return {
        ...service,
        alreadySelected: currentIds.includes(service.id)
      };
    });
  }

  allClientsPageSizeChange(event) {
    this.allClientsPagination.itemsPerPage = event;
    this.getAllClientsWithPagination();
    this.checkAlreadyAddedClients();
  }

  handleAllClientsSearch() {
    this.selectAllExcept.clear();
    this.allClientsSelected = false;
    this.searchData();
    this.checkAlreadyAddedClients();
  }

  sortAllRecord(sort) {
    this.allClientsSort = sort;
    this.allClientsSortDir = this.allClientsSortDir == 'desc' ? 'asc' : 'desc';
    this.allClientsPagination.currentPage = 1;
    this.getAllClientsWithPagination();
    this.checkAlreadyAddedClients();
  }

  selectAllClients() {
    this.allClients = this.allClients.map(service => {
      return {
        ...service,
        isChecked: this.allClientsSelected
      };
    });
    this.selectAllExcept = new Set();
    this.allSelectedClients = {};
    this.updateSelectedCount();
  }

  updateSelectedCount() {
    let disable = Object.keys(this.allSelectedClients).length < 1;
    if (this.allClientsSelected) {
      disable = false;
    }
    this.disableAddAllClients = disable;
  }

  changeAllServiceCheckBox(client) {
    if (this.allClientsSelected) {
      // this.allClientsSelected = false

      if (this.selectAllExcept.has(client.id)) {
        this.allClients = this.allClients.map(myClient => {
          return {
            ...myClient,
            isChecked: client.id === myClient.id ? true : myClient.isChecked
          };
        });
        this.selectAllExcept.delete(client.id);
      } else {
        this.selectAllExcept.add(client.id);
        this.allClients = this.allClients.map(myClient => {
          return {
            ...myClient,
            isChecked: client.id === myClient.id ? false : myClient.isChecked
          };
        });
      }

      // this.allClients = this.allClients.map(myClient => {
      //   return {
      //     ...myClient,
      //     isChecked: client.id === myClient.id
      //   }
      // })
      // this.allSelectedClients[client.id] = client
    } else {
      if (client.isChecked) {
        this.allSelectedClients[client.id] = client;
      } else {
        delete this.allSelectedClients[client.id];
      }
    }
    this.updateSelectedCount();
  }

  addSelectedClients() {
    if (
      !this.currentClients.every(
        client => client.client_type === this.clientType
      )
    ) {
      this.globalMessageService.pushAlert({
        type: 'danger',
        message: 'Mix of server and end user clients not allowed'
      });
      return;
    }

    if (this.allClientsSelected) {
      this.clientsLoading = true;
      this.clientService
        .getClients(
          this.allClientsPagination.currentPage,
          this.allClientsPagination.itemsPerPage,
          this.searchFilters,
          this.allClientsSort,
          this.allClientsSortDir,
          this.clientType ? this.clientType : '',
          this.adOrphand,
          this.localOrphand,
          true,
          Array.from(this.selectAllExcept)
        )
        .subscribe(
          response => {
            const currentIds = this.currentClients.map(client => client.id);
            let data = response.data.filter(
              client => !currentIds.includes(client.id)
            );
            this.allClients = this.allClients.map(client => {
              return {
                ...client,
                isChecked: false
              };
            });
            this.currentClients = [...this.currentClients, ...data];
            this.allClientsSelected = false;
            this.allSelectedClients = {};
            this.updateCurrentClientsCount();
            this.clientsLoading = false;
            this.getPaginatedCurrentClients();
            this.checkAlreadyAddedClients();
          },
          err => {
            this.clientsLoading = false;
          }
        );
    } else {
      this.clientsLoading = true;
      const currentIds = this.currentClients.map(client => client.id);
      let data = Object.values(this.allSelectedClients).map((client: any) => {
        return {
          ...client,
          isChecked: false
        };
      });
      data = data.filter(client => !currentIds.includes(client.id));
      this.allClients = this.allClients.map(service => {
        return {
          ...service,
          isChecked: false
        };
      });
      this.currentClients = [...this.currentClients, ...data];
      this.allClientsSelected = false;
      this.allSelectedClients = {};
      this.updateCurrentClientsCount();
      this.clientsLoading = false;
      this.getPaginatedCurrentClients();
      this.checkAlreadyAddedClients();
    }
    this.selectAllExcept = new Set();
  }

  updateCurrentClientsCount() {
    this.currentClientsPagination.totalItems = this.currentClients.length;
  }

  getFilteredCurrentClients() {
    let clients = [...this.currentClients];

    if (this.currentFilterOs) {
      clients = clients.filter(
        client => client.device_os === this.currentFilterOs
      );
    }
    if (this.currentSelectedType) {
      clients = clients.filter(
        client => client.client_type === this.currentSelectedType
      );
    }

    return clients.filter(
      client =>
        client.Name.toLowerCase().indexOf(
          this.currentClientsSearch.toLowerCase()
        ) > -1
    );
  }

  getPaginatedCurrentClients() {
    this.currentLoading = true;
    const offset =
      this.currentClientsPagination.itemsPerPage *
      (this.currentClientsPagination.currentPage - 1);
    let services = this.getFilteredCurrentClients();

    this.currentClientsPagination.totalItems = services.length;
    this.filteredCurrentClients = services.slice(
      offset,
      offset + this.currentClientsPagination.itemsPerPage
    );
    this.disableAddHostButton = true;
    this.filteredCurrentClients.forEach(client => {
      if (client.device_os != DeviceOSEnum.Android && client.device_os != DeviceOSEnum.IOS) {
        this.disableAddHostButton = false;
      }
    });
    if (this.allCurrentClientsSelected) {
      this.filteredCurrentClients = this.filteredCurrentClients.map(client => {
        return {
          ...client,
          isChecked: !this.selectCurrentExcept.has(client.id)
        };
      });
    }
    // if(this.filteredCurrentServices.every(service => service.isChecked) && this.filteredCurrentServices.length > 0)
    // {
    //   this.allCurrentServicesSelected = true
    // }
    // else{
    //   this.allCurrentServicesSelected = false
    // }
    this.currentLoading = false;
    this.checkAlreadyAddedClients();
    this.updateSelectedCount();
    this.updateCurrentSelectedCount();
  }

  updateCurrentSelectedCount() {
    let disable = Object.keys(this.currentSelectedClients).length < 1;
    if (this.allCurrentClientsSelected) {
      disable = false;
    }
    this.disableDeleteCurrentClients = disable;
  }

  currentPaginationPageChange(event) {
    this.currentClientsPagination.currentPage = event;
    this.getPaginatedCurrentClients();
  }

  currentClientsPageSizeChange(event) {
    if (event) {
      this.currentClientsPagination.itemsPerPage = event;
      this.getPaginatedCurrentClients();
      this.checkAlreadyAddedClients();
    }
  }

  currentClientTypeChange() {
    this.currentClientsPagination.currentPage = 1;
    this.getPaginatedCurrentClients();
  }

  currentOsFilterChanged() {
    this.allCurrentClientsSelected = false;
    this.selectCurrentExcept.clear();
    this.currentClientsPagination.currentPage = 1;
    this.getPaginatedCurrentClients();
  }

  sortCurrentClients(sort) {
    this.currentClientsSort = sort;
    this.currentClientsSortDir =
      this.currentClientsSortDir == 'desc' ? 'asc' : 'desc';
    this.currentClientsPagination.currentPage = 1;
    this.currentClients = this.currentClients.sort((a, b) => {
      return this.currentClientsSortDir == 'desc'
        ? a[sort].toString().localeCompare(b[sort])
        : b[sort].toString().localeCompare(a[sort]);
    });
    this.getPaginatedCurrentClients();
  }

  selectCurrentClients() {
    this.currentSelectedClients = {};
    this.currentClients = this.currentClients.map(client => {
      return {
        ...client,
        isChecked: false
      };
    });
    this.selectCurrentExcept.clear();
    this.getPaginatedCurrentClients();
    this.updateCurrentSelectedCount();
  }

  changeCurrentClientCheckBox(client) {
    if (this.allCurrentClientsSelected) {
      if (this.selectCurrentExcept.has(client.id)) {
        this.selectCurrentExcept.delete(client.id);
      } else {
        this.selectCurrentExcept.add(client.id);
      }
    } else {
      if (client.isChecked) {
        this.currentSelectedClients[client.id] = client;
      } else {
        delete this.currentSelectedClients[client.id];
      }
      // this.allCurrentClientsSelected = false
    }
    this.updateCurrentSelectedCount();
  }

  deleteSelectedServices() {
    this.currentLoading = true;
    let temp = [];

    if (!this.allCurrentClientsSelected) {
      this.currentClients.forEach((client, i) => {
        if (client.isChecked) {
          delete this.currentSelectedClients[client.id];
        }
        if (!client.isChecked) {
          temp.push(client);
        }
      });
      this.currentClientsPagination.totalItems = temp.length;
    }

    if (this.allCurrentClientsSelected) {
      let clientsToDelete = this.getFilteredCurrentClients();
      let clientIds = clientsToDelete.map(client => client.id);
      temp = this.currentClients.filter(
        client => !clientIds.includes(client.id)
      );
    }

    setTimeout(() => {
      this.currentClients = [...temp];
      this.allCurrentClientsSelected = false;
      const offset = 0;
      this.updateCurrentSelectedCount();
      this.currentClientsPagination.currentPage = 1;
      this.getPaginatedCurrentClients;
      this.currentLoading = false;
      this.checkAlreadyAddedClients();
    }, 200);
  }

  deleteSelectedClients() {
    this.currentLoading = true;
    let temp = [];

    if (!this.allCurrentClientsSelected) {
      this.currentClients.forEach((client, i) => {
        if (client.isChecked) {
          delete this.currentSelectedClients[client.id];
        }
        if (!client.isChecked) {
          temp.push(client);
        }
      });
      this.currentClientsPagination.totalItems = temp.length;
    }

    if (this.allCurrentClientsSelected) {
      let clientsToDelete = this.getFilteredCurrentClients();
      clientsToDelete = clientsToDelete.filter(
        client => !this.selectCurrentExcept.has(client.id)
      );
      let clientIds = clientsToDelete.map(client => client.id);
      temp = this.currentClients.filter(
        client => !clientIds.includes(client.id)
      );
    }

    setTimeout(() => {
      this.currentClients = [...temp];
      this.allCurrentClientsSelected = false;
      const offset = 0;
      this.updateCurrentSelectedCount();
      this.currentClientsPagination.currentPage = 1;
      this.getPaginatedCurrentClients();
      this.currentLoading = false;
      this.checkAlreadyAddedClients();
    }, 200);
    this.selectCurrentExcept = new Set();
  }

  clientTypeChange() {
    this.allClientsPagination.currentPage = 1;
    this.selectAllExcept.clear();
    this.allSelectedClients = {};
    this.allClientsSelected = false;
    this.updateSelectedCount();
    this.getAllClientsWithPagination();
  }

  osFilterChanged() {
    this.selectAllExcept.clear();
    this.allClientsSelected = false;
    const osArray = this.quickFilterOsValues;
    delete this.searchFiltersForm.device_os;
    this.searchFiltersForm = {
      ...this.searchFiltersForm,
      device_os: osArray ? osArray.toString() : null
    };
    // if (!osArray.length) {
    //   // delete this.searchFiltersForm.device_os;
    //   this.searchFiltersForm.device_os = '-1';
    // }

    this.searchData();
  }

  applyFirstFilter() {
    this.adOrphand = false;
    this.localOrphand = false;
    const { device_os, package_status } = this.searchFiltersForm;
    this.searchFiltersForm = {
      device_os,
      package_status
    };
    if (this.quickFilterValues.indexOf(1) !== -1) {
      //Allowed
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        is_active: '1'
      };
    }
    if (this.quickFilterValues.indexOf(2) !== -1) {
      //Blockoed
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        is_active: '0'
      };
    }
    if (
      this.quickFilterValues.indexOf(1) !== -1 &&
      this.quickFilterValues.indexOf(2) !== -1
    ) {
      //Allow + Blockoed
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        is_active: '0,1'
      };
    }
    if (this.quickFilterValues.indexOf(3) !== -1) {
      //Darkened
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isDarkened: '1'
      };
    }
    if (this.quickFilterValues.indexOf(4) !== -1) {
      //Not Darkened
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isDarkened: '0'
      };
    }
    if (
      this.quickFilterValues.indexOf(3) !== -1 &&
      this.quickFilterValues.indexOf(4) !== -1
    ) {
      //dark + not dark
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isDarkened: '0,1'
      };
    }
    if (this.quickFilterValues.indexOf(5) !== -1) {
      //mfa
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isMfa: '1'
      };
    }
    if (this.quickFilterValues.indexOf(6) !== -1) {
      //Not mfa
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isMfa: '0'
      };
    }
    if (
      this.quickFilterValues.indexOf(6) !== -1 &&
      this.quickFilterValues.indexOf(5) !== -1
    ) {
      //mfa + not mfa
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isMfa: '0,1'
      };
    }
    if (this.quickFilterValues.indexOf(8) !== -1) {
      //online
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isOnline: '1'
      };
    }
    if (this.quickFilterValues.indexOf(7) !== -1) {
      //Not online
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isOnline: '0'
      };
    }
    if (
      this.quickFilterValues.indexOf(8) !== -1 &&
      this.quickFilterValues.indexOf(7) !== -1
    ) {
      //online + not online
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        isOnline: '0,1'
      };
    }

    if (this.quickFilterValues.indexOf(9) !== -1) {
      this.localOrphand = true;
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        localOrphand: true
      };
    }
    if (this.quickFilterValues.indexOf(10) !== -1) {
      this.adOrphand = true;
      this.searchFiltersForm = {
        ...this.searchFiltersForm,
        adOrphand: true
      };
    }

    this.searchData();
  }

  public searchData(): void {
    this.searchFilters = {
      ...this.searchFiltersForm,
      name: this.allClientsSearch
    };
    this.allClientsPagination.currentPage = 1;
    this.getAllClientsWithPagination();
  }

  getOSFullName(key) {
    return this.OShashMap[key];
  }

  navigate(pageTitle:string){}
}
