import { Component, OnInit, EventEmitter, Output, Input  } from '@angular/core';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { GroupService } from '../group.service';
import * as _ from 'lodash';
import { Group } from '../../models/group';
import { ActivatedRoute } from '@angular/router';
import { ClientTypeEnum } from '../../shared/emuns/client.enum';

@Component({
  selector: 'app-create-quick-group',
  templateUrl: './create-quick-group.component.html',
  styleUrls: ['./create-quick-group.component.css']
})
export class CreateQuickGroupComponent implements OnInit {

  @Output('delete') delete = new EventEmitter<any>();
  @Output('close') close = new EventEmitter<any>();
  @Output('successClose') successClose = new EventEmitter<any>();
  @Input('selectedClients') selectedClients = {};
  @Input('allSelected') allSelected = false;
  @Input('isFilterApplied') isFilterApplied = false;

  _: any = _;

  ClientType = ClientTypeEnum;
  existingMode: boolean = false
  groups = []
  selectedGroup = null
  groupName: string = ''
  groupDescription: string = ''
  loading: boolean = false
  clientType


  constructor(
    private groupServices: GroupService,
    private route: ActivatedRoute,
    private globalMessageService: GlobalMessageService
  ) { 

  }

  ngOnInit() {
    this.close.subscribe((event) => {
    })
    this.clientType = this.route.snapshot.params.type;
    this.groupServices
    .getAvailableGroups()
      .subscribe(response => {
        this.groups = response.data
      })
  }

  removeClient(client) {
    delete this.selectedClients[client.id]
  }

  changeToExisting() {
    this.existingMode = !this.existingMode
  }

  handleCreate() {
    this.loading = true
    const group = new Group()
    group.services = {}
    group.name = this.groupName;
    group.description = this.groupDescription
    const all = this.allSelected;
    if(!this.existingMode) {
      const clientIds = Object.keys(this.selectedClients)
      
      this.groupServices.addGroup(group, [], clientIds, all, this.clientType).subscribe(
        response => {
          this.globalMessageService.pushAlert({
            type: 'success',
            message: 'Group added successfully'
          });
          this.successClose.emit()
        },
        error => {
          this.loading = false
        }
      );
    } 
    else {
      const clientIds = Object.keys(this.selectedClients)
      const selectedGroup = this.groups.find(group => group.id === this.selectedGroup)
      this
        .groupServices
        .editGroup(
          selectedGroup.id,
          selectedGroup,
          false,
          clientIds,
          true,
          all,
          this.clientType
        ).subscribe(
        response => {
          this.globalMessageService.pushAlert({
            type: 'success',
            message: 'Group updated successfully'
          });
          this.successClose.emit()
        },
        error => {
          this.loading = false
        }
      );
    }
  }

}
