import { Injectable, ViewChild } from '@angular/core';
import { Router, CanActivate } from '@angular/router';
import { AuthenticationService } from '../login/authentication.service';
import { TopbarComponent } from '../topbar/topbar.component';
import { AlertServices } from '../alert-list/alerts.service';

@Injectable()
export class AuthGuard implements CanActivate {
  constructor(
    private router: Router,
    private authenticationService: AuthenticationService,
    private readonly alertService: AlertServices
  ) {}

  canActivate() {
    let currentUser = JSON.parse(localStorage.getItem('currentUser'));
    let is_first_login = JSON.parse(localStorage.getItem('is_first_login'));

    if (is_first_login) {
      this.authenticationService.fakeLogOut();
      this.router.navigate(['/change-creds']);
      return false;
    }

    const is_question_changed = JSON.parse(
      localStorage.getItem('is_question_changed')
    );
    const userData = JSON.parse(localStorage.getItem('userData'));

    if (!is_question_changed && is_first_login) {
      this.router.navigate(['/security-questions']);
      return false;
    }

    if (currentUser) {
      if (currentUser?.token) {
        // logged in so return true
        if (!this.authenticationService.isTokenExpired()) {
          this.authenticationService.updateTokenExpiry();
        } else {
          this.authenticationService.logout(true);
        }
        this.alertService.getUserAlerts();

        const expiryTime = JSON.parse(localStorage.getItem('tokenExpiry')); // Token unix time

        if (expiryTime) {
          const currentTimeSeconds = Math.floor(Date.now() / 1000); // Current unix Time 
          const timeDifference = expiryTime - currentTimeSeconds;
  
          if (timeDifference <= 0) {
            localStorage.removeItem('currentUser');
            localStorage.removeItem('tokenExpiry');
            
            this.router.navigate(['login', { sessionExpired: true }]);
            return false;
          }
        }

        return true;
      }
    }

    // not logged in so redirect to login page
    this.router.navigate(['/login']);
    return false;
  }
}
