import { Injectable } from '@angular/core';
import { ConfigService } from '../services/config.service';
import { HttpClient as AuthHttp, HttpHeaders } from '@angular/common/http';
import { AuthenticationService } from '../login/authentication.service';
import { PaginationResponse } from '../models/pagination-response';
import { ServiceProvider } from '../models/service_provider';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';

@Injectable()
export class IdpService {
  private idpApiUrl = '/idps';
  private idps: ServiceProvider[];
  private apiBaseUrl: string;
  httpOptions = {
    headers: new HttpHeaders({
      'Content-Type': 'application/json'
    })
  };

  constructor(
    private authenticationService: AuthenticationService,
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.idpApiUrl = this.apiBaseUrl + this.idpApiUrl;
  }

  /** GET from the server */
  getIdp(
    page: number = 1,
    pageSize: number = 10,
    action: string = 'active',
    sort: string = 'id',
    dir: string = 'desc',
    searchKey = ''
  ): any {
    let query = `?limit=${pageSize}&offset=${page -
      1}&action=${action}&sort=${sort}&dir=${dir}&searchKey=${searchKey}`;
    return this.authHttp
      .get(this.idpApiUrl + query)
      .pipe(map(response => response));
  }

  addIdp(idp: ServiceProvider): any {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    let queryString = '';

    if (idp.id > 0 && typeof idp.id != 'undefined') {
      return this.authHttp
        .put(this.idpApiUrl + '/' + idp.id, { idp: idp }, options)
        .pipe(map(response => response));
    } else {
      return this.authHttp
        .post(this.idpApiUrl, { idp: idp }, options)
        .pipe(map(response => response));
    }
  }

  deleteIdp(id: number, verificationPassword): any {
    var queryString = '';

    if (id > 0) {
      queryString = '/' + id;
    }
    this.httpOptions['body'] = { verificationPassword };
    return this.authHttp
      .delete(this.idpApiUrl + queryString, this.httpOptions)
      .pipe(map(this.extractData));
    //.catch(this.handleError);
  }
  deleteIdpByOtp(id: number): any {
    var queryString = '';

    if (id > 0) {
      queryString = '/otp/' + id;
    }
    return this.authHttp
      .delete(this.idpApiUrl + queryString)
      .pipe(map(this.extractData));
    //.catch(this.handleError);
  }

  getIdpDetail(id): Observable<ServiceProvider | any> {
    var queryString = '';

    if (id != '') {
      queryString = '/' + id;
    }
    return this.authHttp
      .get(this.idpApiUrl + queryString)
      .pipe(map(response => response));
  }

  private extractData(res: Response) {
    let body = res;
    return body || {};
  }

  // private handleError(error: Response | any) {
  //   // In a real world app, we might use a remote logging infrastructure
  //   let errMsg: string;
  //   if (error) {
  //     const body = error || '';
  //     const err = body.error || JSON.stringify(body);
  //     errMsg = `${error.status} - ${error.statusText || ''} ${err}`;
  //   } else {
  //     errMsg = error.message ? error.message : error.toString();
  //   }
  //   console.error(errMsg);
  //   return Observable.throw(errMsg);
  // }
}
