import { DashboardService } from '../../dashboard/dashboard.service';
import {
  Component,
  OnDestroy,
  OnInit,
  ViewContainerRef,
  ViewChildren,
  ViewChild
} from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Subscription } from 'rxjs';
import { LdapService } from '../ldap.service';
import { Ldap } from '../../models/ldap';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { AuthenticationService } from '../../login/authentication.service';
import { tap } from 'rxjs/operators';
import { BsModalService, BsModalRef } from 'ngx-bootstrap/modal';
import { ModalComponent } from '../../shared/modal/modal.component';
import * as sharedContent from '../../shared/shared-content.constant';
import { ErrorCodeEnum } from '../../shared/emuns/error.enum';
import { LicenseErrorEnum } from '../../shared/emuns/license.enum';
import { ContextHelpEnum } from '../../shared/emuns/global.enum';
import { ConfirmModalComponent } from '../../../app/shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-creat-ldapuser',
  templateUrl: './creat-ldapuser.component.html',
  styleUrls: ['./creat-ldapuser.component.css']
})
export class CreatLdapuserComponent implements OnInit {
  @ViewChild('ldapForm', { static: false }) form;
  @ViewChild('inputCa', { static: false }) inputCa;
  @ViewChild('inputCert', { static: false }) inputCert;

  public alert: GlobalMessageClass;
  ldap = new Ldap();
  public testing = false;
  public hide_loader = true;
  public user_has_error = false;
  public no_error = true;
  public user_error =
    'Either Host is down or you may have entered a wrong host in your LDAP settings.';

  private roleSubscription: Subscription;
  public pageTitle: string = 'Add Configuration';

  //Contextual Help
  public contextualHelp: Array<any>;
  public showContext: boolean = false;
  //Contextual Help
  public numberOfLdaps: number = 0;
  public showCertificatesForm: boolean = false;
  public isLdapSSL: boolean = false;
  public inputCaValue: any;
  public inputCertValue: any;

  loading: boolean = false;
  disableSaveButton = false;
  modalRef: BsModalRef;
  isDarkMode : boolean;

  selectedCondition: any = {
    label: '',
    value: ''
  };
  typeOptions = [
    { label: 'Active Directory', value: 'Active Directory' },
    { label: 'LDAP', value: 'LDAP' },
    { label: 'Google LDAP', value: 'Google LDAP' }

  ];
  
  constructor(
    vcRef: ViewContainerRef,
    private router: Router,
    private route: ActivatedRoute,
    private ldapService: LdapService,
    private globalMessageService: GlobalMessageService,
    private dashboardService: DashboardService,
    private authenticationService: AuthenticationService,
    private modalService: BsModalService,
    public dialog: MatDialog,
    private titleService: Title
  ) {
    //overlay.defaultViewContainer = vcRef;
    this.contextualHelp = ['Name', 'Test'];
  }

  showContextHelp(name: any): void {
    if (name == ContextHelpEnum.LDAP_user) {
      this.contextualHelp = [
        'LDAP_user',
        'Active Directory user info to fetch user list.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.LDAP_url) {
      this.contextualHelp = [
        'LDAP_url',
        'Active Directory Server URL. Certificates are required for LDAP(S).'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.LDAP_timeout) {
      this.contextualHelp = [
        'LDAP_timeout',
        'Active Directory timeout in miliseconds.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.LDAP_connectTimeout) {
      this.contextualHelp = [
        'LDAP_connectTimeout',
        'Active Directory connection timeout in miliseconds.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.LDAP_dc) {
      this.contextualHelp = [
        'LDAP_dc',
        'Base DN determines what entities are considered relevant. All searching and syncing functionalities of Invisily will look for entities under the DN provided. Note: Base DN must atleast include the Domain Component e.g. DC=invisily,DC=org'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.LDAP_clients_dn) {
      this.contextualHelp = [
        'LDAP_clients_dn',
        '(Directory Sync) Users that need to be synced'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.LDAP_groups_dn) {
      this.contextualHelp = [
        'LDAP_groups_dn',
        '(Directory Sync) Groups that need to be synced'
      ];
      this.showContext = true;
    }
  }

  closeContextualHelp(): void {
    this.showContext = false;
  }

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.disableSaveButton = false;
    this.loading = true;
    if (typeof this.route.params['_value']['id'] != 'undefined') {
      this.pageTitle = 'Edit Configuration';
      this.ldapService
        .getldapdetail(this.route.params['_value']['id'])
        .subscribe(
          ldap => {
            this.loading = false;
            if (!ldap) {
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'No LDAP settings found with the specified ID.'
              };
              this.globalMessageService.pushAlert(this.alert);
              this.router.navigate(['/ldaps']);
            }
            this.ldap = ldap;
            this.selectedCondition = this.typeOptions.filter(op => op.value == ldap.directory_type)[0];

            this.setLdapProtocol();
          },
          err => {
            this.loading = false;
          }
        );
    } else {
      this.dashboardService
        .countHost()
        .pipe(
          tap((res: any) => {
            this.loading = false;
            if (res.status) {
              this.numberOfLdaps = res.ldaps;
            } else {
              this.numberOfLdaps = 0;
            }
          })
        )
        .subscribe({
          next: next => {},
          error: err => {
            this.loading = false;
            if (err.status === ErrorCodeEnum.Forbidden) {
              if (err._body == LicenseErrorEnum.Invalid) {
                this.authenticationService.showLicenseOnly(true);
                this.router.navigate(['/license']);
              } else {
                this.alert = {
                  closed: false,
                  type: 'danger',
                  message: 'You are not authorized to access this page.'
                };
                this.globalMessageService.pushAlert(this.alert);
              }
            }
          },
          complete: () => {}
        });
    }
    this.titleService.setTitle('Infrastructure | LDAP Configuration | ' + this.pageTitle);
  }

  onsubmit() {
    this.loading = true;
    this.disableSaveButton = true;
    this.hide_loader = false;
    this.testing = true;
    // this.ldap.directory_type = this.selectedCondition.label;
    console.log('this.ldap :>> ', this.ldap);
    this.ldapService
      .test_settings(this.ldap)
      .pipe(
        tap(res => {
          this.loading = false;
          if (res) {
            if (res.status == 'InValidCredentials') {
              this.user_error =
                'Credentials in your LDAP settings are invalid.';
              this.user_has_error = true;
              this.no_error = false;
            } else if (res.status == 'InappropriateAuthenticationError') {
              this.user_error =
                'Inappropriate Authentication Error Check your Credentials in your LDAP settings.';
              this.user_has_error = true;
              this.no_error = false;
            } else if (res.status == 'HostNotConnected') {
              this.user_error =
                'Either Host is down or you may have entered a wrong host in your LDAP settings.';
              this.user_has_error = true;
              this.no_error = false;
            } else if (!res.status) {
              this.user_error = 'LDAP settings are invalid problem in binding';
              this.user_has_error = true;
              this.no_error = false;
            } else {
              this.no_error = true;
            }
          }
        })
      )
      .subscribe({
        next: next => console.log('Next'),
        error: err => {
          this.disableSaveButton = false;
          this.loading = false;
          this.user_error =
            'Either Host is down or you may have entered a wrong host in your LDAP settings.';
          this.user_has_error = true;
          this.hide_loader = true;
          this.testing = false;
        },
        complete: () => {
          this.hide_loader = true;
          this.testing = false;
          this.user_has_error = false;
          if (this.no_error) {
            this.ldapService.add_user(this.ldap).subscribe(response => {
              if (response.success) {
                const alert = {
                  closed: false,
                  type: 'success',
                  message: 'LDAP User has been added/edited successfully.'
                };
                this.globalMessageService.pushAlert(alert);
                this.router.navigate(['/ldaps']);
              } else {
                const alert = {
                  closed: false,
                  type: 'danger',
                  message: 'Problem adding the settings.'
                };
                this.globalMessageService.pushAlert(alert);
              }
            });
          } else {
            this.user_has_error = true;
          }
        }
      });
  }

  onCancel(): void {
    this.router.navigate(['/ldaps']);
  }

  setLdapProtocol() {
    this.isLdapSSL = /^ldaps.*$/.test(this.ldap.LDAP_url);
  }

  setCertificates() {
    if (this.inputCaValue && this.inputCertValue) {
      this.ldap.LDAP_ca = this.inputCaValue.contents;
      this.ldap.LDAP_cert = this.inputCertValue.contents;
      this.ldap.LDAP_ca_name = this.inputCaValue.name;
      this.ldap.LDAP_cert_name = this.inputCertValue.name;

      this.showCertificatesForm = false;

      this.form.controls['LDAP_url'].updateValueAndValidity();
    }
  }

  changeCertificates() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'Do you really want to change certificates?',
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        this.ldap.LDAP_ca = null;
        this.ldap.LDAP_cert = null;
        this.ldap.LDAP_ca_name = null;
        this.ldap.LDAP_cert_name = null;

        this.inputCaValue = null;
        this.inputCertValue = null;

        this.inputCa.nativeElement.value = '';
        this.inputCert.nativeElement.value = '';

        this.showCertificatesForm = true;
        this.form.controls['LDAP_url'].updateValueAndValidity();
      }
    });
  }

  cancelCertificates() {
    this.showCertificatesForm = false;
    this.form.controls['LDAP_url'].updateValueAndValidity();
  }

  handleFileUpload(evt, name) {
    if (evt.target.files && evt.target.files.length > 0) {
      const file = evt.target.files[0];
      const reader = new FileReader();
      reader.readAsText(file);
      reader.onload = () => {
        this[name] = {
          name: file.name,
          contents: reader.result
        };
      };
    }
  }

  onClick() {
    this.modalRef = this.modalService.show(ModalComponent, {
      initialState: sharedContent.passwordSuggestion,
      class: 'modal-dialog-centered'
    });
  }

  condtionChange() {
    this.ldap.directory_type = this.selectedCondition.label.value;
    console.log('this.ldap.directory_type :>> ', this.ldap.directory_type);
  }
}
