import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { LicenseService } from './license.service';
import { License } from '../models/license';
import { TrustedHost } from '../models/trusted-host';
import { PreserveSearch } from '../models/preserve-search';
import { Pagination } from '../models/pagination';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { ProgressFile } from './Progress';
import { AuthenticationService } from '../login/authentication.service';
import * as moment from 'moment';
import { interval } from 'rxjs';
import { startWith, switchMap, tap } from 'rxjs/operators';
import { of } from 'rxjs';
import { ErrorCodeEnum } from '../shared/emuns/error.enum';
import { LicenseViolationEnum } from '../shared/emuns/license.enum';
import { ContextHelpEnum, RoleEnum } from '../shared/emuns/global.enum';
import { Title } from '@angular/platform-browser';

//import { ClientService } from '../client/client.service';
@Component({
  selector: 'app-license',
  templateUrl: './license.component.html',
  styleUrls: ['./license.component.css']
})
export class LicenseComponent implements OnInit {
  ips: Array<TrustedHost> = null;
  addGlobalIp: any = '';
  addGlobalIpPort: any = '';
  addGlobalIpProtocol: any = 'tcp';
  addGlobalIpLabel: any = '';
  addGlobalIpDirection: number = 0;
  ipError: string = '';
  public alert: GlobalMessageClass;
  public pageTitle: string = 'License';
  license = null;
  public errorMessage: string;
  public progress: ProgressFile = new ProgressFile();
  //Contextual Help
  public contextualHelp: Array<any>;
  public showContext: boolean = false;
  public fileData = {};
  public sigFileData = {};
  private fileSizeLimit = 100 * 1024;
  public uploadValid: boolean = false;
  public uploading: boolean = false;
  fileInputRef = null;
  expiry_date_time: string = '';
  created_date_time: string = '';
  days_left: number = 0;
  days_left_approx: number = 0;
  is_expired: boolean;
  is_expiring_today: boolean;
  is_expiring_tomorrow: boolean;
  userrole: string;
  filesFieldsHidden: boolean = true;
  sigFileValid = false;
  processing = false;
  invalidReason = 0;
  invalid = false;
  $processSubscription;
  loading: boolean = true;

  LicenseViolation = LicenseViolationEnum;
  Role = RoleEnum;
  microSegLicense: any;
  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private licenseService: LicenseService,
    private globalMessageService: GlobalMessageService,
    private authenticationService: AuthenticationService,
    private titleService: Title /*, private clientService: ClientService*/
  ) {
    this.pageTitle = 'License';

    this.userrole = localStorage.getItem('ur');
    if (typeof this.userrole != 'undefined' && this.userrole != null) {
      this.userrole = atob(this.userrole);
    }
  }

  showContextHelp(name: any): void {
    if (name == ContextHelpEnum.license) {
      this.contextualHelp = ['license', 'Select license file to upload.'];
      this.showContext = true;
    }
  }

  renewLicenseHandler() {
    if (this.filesFieldsHidden) {
      this.filesFieldsHidden = false;
      document.getElementById('licenseRenew1').blur();
    } else {
      if (this.uploadValid && this.sigFileValid) {
        this.saveLicense();
      } else {
        alert('Please select required files for license uploading');
      }
    }
  }

  renewViolationLicenseHandler() {
    if (this.uploadValid && this.sigFileValid) {
      this.saveLicense();
    } else {
      alert('Please select required files for license uploading');
    }
  }

  handleSigFileUplaod(event) {
    if (event.target.files && event.target.files.length > 0) {
      let file = event.target.files[0];
      if (file.size / 1024 < this.fileSizeLimit) {
        let name = file.name;
        //Getting Type
        var extension = name.split('.').pop();
        if (extension == 'sign') {
          let reader = new FileReader();
          reader.readAsDataURL(file);
          reader.onload = () => {
            let data = reader.result;
            this.sigFileData = {
              data: data,
              name: name,
              type: extension
            };
            this.sigFileValid = true;
          };
        } else {
          alert(
            'Please select a valid file type, these are allowed types `sign`'
          );
          this.sigFileValid = false;
        }
      } else {
        alert('File size should be less than ' + this.fileSizeLimit + ' 50MB');
        this.sigFileValid = false;
      }
    }
  }

  handleFileUpload(evt) {
    let self = this;
    self.progress = new ProgressFile();
    self.progress.complete = false;
    self.progress.value = 0;
    if (evt.target.files && evt.target.files.length > 0) {
      let file = evt.target.files[0]; // FileList object
      if (file.size / 1024 < this.fileSizeLimit) {
        this.fileInputRef = evt.target;
        let name = file.name;
        let brokenName = name.split('-');
        var verWithExt = brokenName[brokenName.length - 1].split('.');
        verWithExt.splice(-1, 1);
        var version = verWithExt.join('.');
        let regex = /^(\d+\.)(\d+\.)(\d+)$/gm;
        let isValidVersion = regex.test(version);
        let type = file.type;
        /*if (isValidVersion == false) {
          alert('Please select a valid file name format. Format is name-version.extension.');
          self.uploadValid = false;
          return 0;
        }*/
        //this.update.buildVersion = version;
        //Getting Type
        var local_name = name.split('.');
        let ext = local_name[local_name.length - 1];
        var types = ['lic', 'text/plain'];

        if (types.includes(type) || ext === 'lic') {
          let reader = new FileReader();
          reader.readAsDataURL(file);
          reader.onload = () => {
            let data = reader.result;
            self.fileData = {
              data: data,
              name: name,
              type: type
            };
            self.uploadValid = true;
          };
          reader.onloadstart = () => {
            self.progress.show = true;
            self.progress.value = 0;
          };
          reader.onprogress = data => {
            self.progress.show = true;
            self.progress.value = (data.loaded / data.total) * 100;
          };
          reader.onloadend = () => {
            self.progress.value = 100;
            self.progress.complete = true;
            self.progress.show = false;
          };
        } else {
          alert('Only supported format is lic');
          self.uploadValid = false;
        }
      } else {
        alert('File size should be less than ' + this.fileSizeLimit + ' 50MB');
        self.uploadValid = false;
      }
    }
  }

  saveLicense() {
    if (this.fileInputRef.value != null && this.fileInputRef.value != '') {
      this.uploading = true;
      this.globalMessageService.clearAlerts();
      this.licenseService
        .addEditLicense(this.fileData, this.sigFileData)
        .subscribe(response => {
          this.fileInputRef.value = '';
          this.globalMessageService.clearAlerts();

          if (response.success) {
            this.checkLicenseProcess();
            const license = response.data;
            this.invalid = false;
            this.updateLicense(license);
            this.authenticationService.showLicenseOnly(true);
            this.filesFieldsHidden = true;
            this.alert = {
              closed: false,
              type: 'info',
              message: 'License file is being processed.'
            };
            this.globalMessageService.pushAlert(this.alert);
            //this.authenticationService.showLicenseOnly(false);
            this.router.navigate(['/license'], {
              queryParams: { refresh: new Date().getTime() }
            });
          } else {
            this.alert = {
              closed: false,
              type: 'danger',
              message:
                response.message || 'Something went wrong in saving license'
            };
            this.globalMessageService.pushAlert(this.alert);
            this.errorMessage = response.message;
          }
          this.uploading = false;
        });
    } else {
      alert('Plese select the license file');
      return false;
    }
  }

  checkLicenseProcess() {
    this.$processSubscription = interval(10000)
      .pipe(
        startWith(0),
        switchMap(() => {
          this.getLicense();
          return of('license fetching...');
        })
      )
      .subscribe(data => {});
  }

  ngOnDestroy() {
    if (this.$processSubscription) {
      this.$processSubscription.unsubscribe();
    }
  }

  closeContextualHelp(): void {
    this.showContext = false;
  }

  updateLicense(license: License) {
    if (license.violated) {
      this.invalid = true;
      this.invalidReason = license.violated;
      this.license = license;
      if (this.$processSubscription) {
        this.$processSubscription.unsubscribe();
      }
    } else if (license.number_of_days || license.number_of_days === 0) {
      this.created_date_time = moment(license.license_submission_date).format(
        'MMMM Do YYYY, h:mma'
      );

      const validUntil = moment(license.license_submission_date).add(
        license.number_of_days,
        'days'
      );

      this.expiry_date_time = validUntil.format('MMMM Do YYYY, h:mma');
      this.days_left_approx = this.days_left;
      this.is_expiring_today = validUntil.isSame(moment(), 'day');
      this.is_expiring_tomorrow = validUntil.isSame(
        moment()
          .startOf('day')
          .add(1, 'd'),
        'day'
      );
      this.is_expired = this.days_left < 0;
      this.processing = false;
      this.license = license;
      if (this.$processSubscription) {
        this.$processSubscription.unsubscribe();
      }
    } else {
      this.created_date_time = moment(license.license_submission_date).format(
        'MMMM Do YYYY, h:mma'
      );
      this.expiry_date_time = moment().format('MMMM Do YYYY, h:mma');
      this.is_expired = false;
      this.processing = true;
      this.license = license;
      if (!this.$processSubscription) {
        this.checkLicenseProcess();
      }
    }
  }

  ngOnInit() {
    this.pageTitle = 'License';
    this.titleService.setTitle('Settings | License Management');
    this.globalMessageService.clearAlerts();
    // No need to show expiry notification as the whole page is about license
    this.globalMessageService.removeAlertByName('LICENSE_NOTIFICATION');
    
    this.microSegLicense = localStorage.getItem('MicroSegLicense');
    if (this.microSegLicense == 1 || this.microSegLicense == 0) {
      this.getLicense();
    }else{
      this.loading = false;
    }
  }

  getLicense() {
    this.licenseService
      .getLicense()
      .pipe(
        tap(response => {
          this.loading = false;
          this.days_left = parseFloat(response.xLicenseDays);
          this.globalMessageService.clearAlerts();
          if (!response.valid) {
            this.authenticationService.showLicenseOnly(true);
          } else {
            this.authenticationService.showLicenseOnly(false);
          }
          const license = response.license;
          this.updateLicense(license);
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          this.loading = false;
          if (err.status === ErrorCodeEnum.Forbidden) {
            this.alert = {
              closed: false,
              type: 'danger',
              message: 'You are not authorized to access this page.'
            };
            this.globalMessageService.pushAlert(this.alert);
          } else if (err.status === ErrorCodeEnum.NotFound) {
            this.authenticationService.showLicenseOnly(true);
          }
        },
        complete: () => {}
      });
  }

  addIpAddress() {
    let length = 0;
    this.ipError = '';
    let regex = /^([0-9]{1,3})[.]([0-9]{1,3})[.]([0-9]{1,3})[.]([0-9]{1,3})$/;
    let hostname = /^(([a-zA-Z]|[a-zA-Z-_][a-zA-Z0-9\-_]*[a-zA-Z0-9])\.)*([A-Za-z]|[A-Za-z][A-Za-z0-9\-]*[A-Za-z0-9])$/;
    var result1 = regex.test(this.addGlobalIp);
    var result2 = hostname.test(this.addGlobalIp);
    if (this.ips.length < 1000) {
      if (
        this.addGlobalIp.length &&
        this.addGlobalIp != null &&
        this.addGlobalIpPort.length &&
        this.addGlobalIpPort != null &&
        this.addGlobalIpProtocol.length &&
        this.addGlobalIpProtocol != null
      ) {
        if (this.ips) {
          if (this.ips.length) {
            length = this.ips.length;
          } else {
            length = 0;
          }
        }
        if (this.addGlobalIp.length >= 250) {
          this.ipError = 'Host/IP must be less than 250 characters in length';
        } else if (result1 == false && result2 == false) {
          this.ipError = 'Invalid format of Host/IP';
        } else if (
          parseInt(this.addGlobalIpPort) < 0 ||
          parseInt(this.addGlobalIpPort) > 65535
        ) {
          this.ipError = 'Port must be between 0 to 65535 both inclusive.';
        } else {
          let alreadyExist = false;
          this.ips.forEach(ip => {
            if (
              ip.address === this.addGlobalIp &&
              ip.direction === this.addGlobalIpDirection &&
              ip.port === this.addGlobalIpPort &&
              ip.protocol === this.addGlobalIpProtocol
            ) {
              alreadyExist = true;
            }
          });
          if (alreadyExist) {
            this.ipError = 'This IP already exist.';
          } else {
            this.ips[length] = new TrustedHost();
            this.ips[length].address = this.addGlobalIp;
            this.ips[length].port = this.addGlobalIpPort;
            this.ips[length].protocol = this.addGlobalIpProtocol;
            this.ips[length].label = this.addGlobalIpLabel;
            this.ips[length].direction = this.addGlobalIpDirection;
            this.addGlobalIp = '';
            this.addGlobalIpPort = '';
            this.addGlobalIpProtocol = 'tcp';
            this.addGlobalIpLabel = '';
          }
          /*this.clientService.check_rule(0, this.addGlobalIp,this.addGlobalIpDirection,this.addGlobalIpPort,this.addGlobalIpProtocol).subscribe(response => {
            if (response.status) {
              this.ipError = response.message
            }else{
              this.ips[length] = new GlobalIps();
              this.ips[length].address = this.addGlobalIp;
              this.ips[length].port = this.addGlobalIpPort;
              this.ips[length].protocol = this.addGlobalIpProtocol;
              this.ips[length].label = this.addGlobalIpLabel;
              this.ips[length].direction = this.addGlobalIpDirection;
              this.addGlobalIp = '';
              this.addGlobalIpPort = "";
              this.addGlobalIpProtocol = "tcp";
              this.addGlobalIpLabel = "";
            }
          });*/
        }
      } else {
        this.ipError = 'You must provide a valid Host, port and protocol.';
      }
    } else {
      this.ipError = 'You can only add upto 1000 Host/IP.';
    }
  }

  onDeleteIP(ip: TrustedHost) {
    this.ipError = '';
    var index = this.ips.indexOf(ip);
    this.ips.splice(index, 1);
  }

  cancelHandler() {
    this.filesFieldsHidden = true;
  }

  /*saveUpdate(){
    this.setting.isSettingsSet = true;
    this.settingsService.addEditSetting(this.setting, this.ips).subscribe(response => {
      if(response.success){
        this.alert = {closed: false,type: "success",message: "Settings has been added/updated successfully."};
        this.globalMessageService.pushAlert(this.alert);
        this.router.navigate(['/settings']);
      } else {
        this.alert = {closed: false,type: "error",message: "Failed to add/update Settings."};
        this.globalMessageService.pushAlert(this.alert);
        this.errorMessage = response.message;
      }
    });
  }*/

  onCancel(): void {
    this.router.navigate(['/dashboard']);
  }
}
