import { Injectable } from '@angular/core';
import { ConfigService } from '../services/config.service';
import { HttpClient as AuthHttp, HttpHeaders } from '@angular/common/http';
import { AuthenticationService } from '../login/authentication.service';
import { BehaviorSubject, Observable } from 'rxjs';
import { Setting } from '../models/setting';
import { TrustedHost } from '../models/trusted-host';
import { LoginResponse } from '../models/login_response';
import { map } from 'rxjs/operators';

@Injectable()
export class LicenseService {
  private apiBaseUrl: string;
  private settingApiUrl = '/settings';
  private settingGlobalIpApiUrl = '/settings/getGlobalIps/';
  private licenseApiUrl = '/license';

  private microSegLicenseSubject = new BehaviorSubject<boolean>(this.loadInitialValue());
  microSegLicense$ = this.microSegLicenseSubject.asObservable();

  constructor(
    private authenticationService: AuthenticationService,
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.settingApiUrl = this.apiBaseUrl + this.settingApiUrl;
    this.settingGlobalIpApiUrl = this.apiBaseUrl + this.settingGlobalIpApiUrl;
    this.licenseApiUrl = this.apiBaseUrl + this.licenseApiUrl;
  }

  private loadInitialValue(): boolean {
    const storedValue = localStorage.getItem('MicroSegLicense');
    return storedValue ? JSON.parse(storedValue) : false;
  }

  updateMicroSegLicense(value: boolean): void {
    this.microSegLicenseSubject.next(value);
  }

  getLicense(): Observable<any> {
    return this.authHttp
      .get(this.licenseApiUrl)
      .pipe(map(response => response));
  }

  getGlobalIps(): Observable<any> {
    return this.authHttp
      .get(this.settingGlobalIpApiUrl)
      .pipe(map(response => response));
  }

  addEditSetting(
    setting: Setting,
    ips: Array<TrustedHost>
  ): Observable<LoginResponse | any> {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    let queryString = '';
    if (setting.id != 0 && typeof setting.id != 'undefined') {
      queryString = '/' + setting.id;
      return this.authHttp
        .put(
          this.settingApiUrl + queryString,
          { setting: setting, gips: ips },
          options
        )
        .pipe(map(this.extractData));
      //.catch(this.handleError);
    } else {
      return this.authHttp
        .post(this.settingApiUrl, { setting: setting, gips: ips }, options)
        .pipe(map(this.extractData));
      //.catch(this.handleError);
    }
  }

  addEditLicense(file, sigFileData): Observable<any> {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    let queryString = '';
    return this.authHttp
      .post(this.licenseApiUrl, { attachment: file, sigFileData }, options)
      .pipe(map(this.extractData));
    //.catch(this.handleError);
    /*if (update.id != 0 && typeof update.id != "undefined") {
      queryString = "/" + update.id;
      return this.authHttp.put(this.updatesApiUrl + queryString, { update: update, attachment: file }, options)
        .map(this.extractData)
        .catch(this.handleError);
    } else {
      return this.authHttp.post(this.updatesApiUrl, { update: update, attachment: file }, options)
        .map(this.extractData)
        .catch(this.handleError);
    }*/
  }

  private extractData(res: Response) {
    let body = res;
    return body || {};
  }

  // private handleError(error: Response | any) {
  //   // In a real world app, we might use a remote logging infrastructure
  //   let errMsg: string;
  //   if (error) {
  //     const body = error || '';
  //     const err = body.error || JSON.stringify(body);
  //     errMsg = `${error.status} - ${error.statusText || ''} ${err}`;
  //   } else {
  //     errMsg = error.message ? error.message : error.toString();
  //   }
  //   console.error(errMsg);
  //   return Observable.throw(errMsg);
  // }
}
