import { Component, OnInit } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { HttpClient } from '@angular/common/http';
import { ConfigService } from '../services/config.service';
import { AuthenticationService } from '../login/authentication.service';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import {} from '../services/config.service';
import { map } from 'rxjs/operators';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-local-user-forgot-password',
  templateUrl: './local-user-forgot-password.component.html',
  styleUrls: ['./local-user-forgot-password.component.css']
})
export class LocalUserForgotPasswordComponent implements OnInit {
  linkToken: String;
  userName;
  userPassword;
  confirmPasswordField;
  private apiBaseUrl: string;
  private localUserUrl = '/localUsers/';
  loading = false;
  hasError = true;
  errorBody: String;
  passwordUpdated = false;

  constructor(
    private configService: ConfigService,
    private activatedRoute: ActivatedRoute,
    private authService: AuthenticationService,
    private globalMessageService: GlobalMessageService,
    private http: HttpClient,
    private titleService: Title
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.localUserUrl = this.apiBaseUrl + this.localUserUrl;
    this.authService.fakeLogOut();
  }

  ngOnInit() {
    this.titleService.setTitle('Reset Password');
    this.activatedRoute.params.subscribe(param => {
      this.linkToken = param.token;
      this.checkTokenValidity();
    });
  }

  checkTokenValidity() {
    this.http
      .get(`${this.localUserUrl}checkValidToken/${this.linkToken}`)
      .pipe(map(response => response))
      .subscribe(
        data => {
          this.hasError = false;
          this.getUserFromToken();
          this.globalMessageService.clearAlerts();
        },
        error => {
          this.hasError = true;
          this.errorBody = error._body;
        }
      );
  }

  getUserFromToken() {
    this.http
      .get(`${this.localUserUrl}token/${this.linkToken}`)
      .pipe(map(response => response))
      .subscribe((data: any) => {
        this.userName = data.userName;
        this.globalMessageService.clearAlerts();
      });
  }

  submitAction() {
    this.http
      .post(`${this.localUserUrl}updatePassword/${this.linkToken}`, {
        password: this.userPassword
      })
      .pipe(map(response => response))
      .subscribe(
        data => {
          this.passwordUpdated = true;
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'success',
            message: `Password changed successfully.`,
            sticky: false
          });
        },
        error => {
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'danger',
            message: 'Error in reseting password',
            sticky: false
          });
        }
      );
  }
}
