import { Component, OnInit, ViewChild } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { User } from '../models/user';
import { UserService } from '../users/user.service';
import { AuthenticationService } from '../login/authentication.service';
import { ForgotService } from '../forgot-password/forgot.service';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import * as moment from 'moment';
import { ConfigService } from '../services/config.service';
import { Title } from '@angular/platform-browser';
import { SettingsService } from '../settings/settings.service';
import { catchError, map } from 'rxjs';
import { ThemeService } from '../theme.service';
import { LicenseService } from '../license/license.service';
import { RolesAndPermissionService } from '../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-login',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.css']
})
export class LoginComponent implements OnInit {
  user = new User();
  showLoginScreen = true;
  loading = false;
  error = '';
  onRest = true;
  showQRCode = false;
  showSecurityQuestions = false;
  qrImageData = '';
  otpGiven = '';
  otpError = false;
  loginResponse;
  public token: string;
  userName: string;
  version = '';
  showPassword: boolean = false;
  securityQuestion1: string;
  securityQuestion2: string;
  securityQuestion3: string;
  accountLocked: boolean = false;
  captchaSuccessful: boolean = false;
  showRecaptcha: boolean = true;
  recaptchaSiteKey: string = '';
  isFirstLogin = false;
  is_question_changed = false;
  isDarkMode: any;
  logo: any;
  s2sLicenseValue = false;

  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private authenticationService: AuthenticationService,
    private globalMessageService: GlobalMessageService,
    private configServices: ConfigService,
    private forgotService: ForgotService,
    private titleService: Title,
    private settingsService: SettingsService,
    private themesService: ThemeService,
    private licenseService: LicenseService,
    private rolesPermissionService: RolesAndPermissionService
  ) {
    localStorage.removeItem('ur');
    if (typeof this.route.params['_value']['sessionExpired'] != 'undefined') {
      let sessionExpired = this.route.params['_value']['sessionExpired'];

      if (sessionExpired) {
        this.error = 'Your session has expired. Please login again.';
      }
    }
    if (!localStorage.getItem('currentUser')) {
      this.authenticationService.showNavBar(false);
    }
    if (localStorage.getItem('currentUser')) {
      this.router.navigate(['/']);
    }

    this.version = this.configServices.getVersion();
    this.recaptchaSiteKey = this.configServices.getSiteKey();
  }

  showErrorIfPresent() {
    this.onRest = true;
  }

  typingHideError() {
    this.onRest = false;
  }

  ngOnInit() {
    // reset login status
    //this.authenticationService.logout();
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.logo = this.isDarkMode
      ? '/assets/images/logo-dark.png'
      : '/assets/images/invisily-logo.png';
    this.isDarkMode = this.isDarkMode ? 'dark-theme' : '';
    this.titleService.setTitle('Sign In');
  }

  resolved(captchaResponse: string) {
    this.authenticationService
      .passRecaptcha(this.user.email)
      .subscribe(data => {
        this.captchaSuccessful = true;
      });
  }

  login() {
    this.loading = true;
    this.otpError = false;
    this.otpGiven = '';
    this.authenticationService
      .login(this.user.email, this.user.password)
      .subscribe(result => {
        this.globalMessageService.clearAlerts();

        if (result.status === true) {
          if (result.admin_mfa) {
            localStorage.setItem(
              'adminMfaValue',
              JSON.stringify(result.admin_mfa)
            );
            if (!result.user.is_mfa_assigned) {
              //call create token api
              this.authenticationService
                .createLinToken(this.user.email)
                .subscribe(
                  (data: any) => {
                    this.globalMessageService.pushAlert({
                      closed: true,
                      type: 'success',
                      message: data.message
                    });
                    // this.globalMessageService.clearAlerts();
                    this.showQRCode = !result.user.is_mfa_assigned;
                    this.qrImageData = data.image_data;
                  },
                  error => {
                    this.error =
                      'Error in generating OTP token.Maybe your OTP server is down.';
                    this.loading = false;
                    this.globalMessageService.clearAlerts();
                  }
                );
            } else {
              this.globalMessageService.clearAlerts();
              this.showQRCode = true;
              this.qrImageData = '';
            }
          } else {
            this.loginResponse = result.loginResponse;
            this.userName = result.user.firstName;
            if (result.user.is_first_login) {
              localStorage.setItem(
                'is_first_login',
                result.user.is_first_login
              );
              this.isFirstLogin = result.user.is_first_login;
            }
            if (!result.user.is_question_changed) {
              localStorage.setItem(
                'is_question_changed',
                result.user.is_question_changed
              );
              this.is_question_changed = result.user.is_question_changed;
            }
            if (result.user.is_question_changed) {
              this.is_question_changed = result.user.is_question_changed;
            }
            this.setLocalItems();
            this.setLicense();
            this.setRolesData();
            this.setEmailConfigs();
            this.settingsService
              .getTimeSettings()
              .pipe(map((data: any) => data))
              .subscribe({
                next: data => {
                  localStorage.setItem(
                    'userTimeData',
                    JSON.stringify(data?.timesetting)
                  );
                  localStorage.setItem(
                    'darkMode',
                    JSON.stringify(data?.isDark)
                  );
                  this.themesService.toggleDarkMode.bind(this.themesService)(
                    data.isDark
                  );
                  this.settingsService.userSettingsLoaded.emit();
                },
                error: err => catchError(err)
              });
          }
        } else {
          // login failed
          if (
            result.message.includes('locked') &&
            result.question1 &&
            result.question2 &&
            result.question3
          ) {
            this.accountLocked = true;
            this.showSecurityQuestions = true;
            this.securityQuestion1 = result.question1;
            this.securityQuestion2 = result.question2;
            this.securityQuestion3 = result.question3;
            this.showRecaptcha =
              result.recaptchaConfigured === true ||
              result.recaptchaConfigured == 'true';
            if (!this.showRecaptcha) {
              this.captchaSuccessful = true;
            }
          }
          this.error = result.message;
          this.loading = false;
          this.globalMessageService.clearAlerts();
        }
      });
  }

  setEmailConfigs() {
    this.settingsService.getSettings('').subscribe(settings => {
      const isOauthService = settings?.data[0]?.oauth_config;
      localStorage.setItem('isOauthService', JSON.stringify(isOauthService));
    });
  }

  validateOtpHandler() {
    if (!this.accountLocked) {
      this.authenticationService
        .validateLinToken(this.user.email, this.user.password, this.otpGiven)
        .subscribe(
          (data: any) => {
            this.globalMessageService.clearAlerts();

            if (data.success) {
              this.otpError = false;
              this.loginResponse = data;
              this.userName = data.user.firstName;
              this.setLocalItems();
              this.setLicense();
              this.setRolesData();
              this.settingsService
                .getTimeSettings()
                .pipe(map((data: any) => data))
                .subscribe({
                  next: data => {
                    localStorage.setItem(
                      'userTimeData',
                      JSON.stringify(data?.timesetting)
                    );
                    localStorage.setItem(
                      'darkMode',
                      JSON.stringify(data?.isDark)
                    );
                  },
                  error: err => catchError(err)
                });
            } else {
              this.otpError = true;
              this.otpGiven = '';
            }
          },
          error => {
            this.globalMessageService.clearAlerts();
            this.otpError = true;
            this.otpGiven = '';
          }
        );
    } else {
      this.authenticationService
        .validateOTPToken(this.user.email, this.otpGiven)
        .subscribe(
          (res: any) => {
            if (res.status) {
              if (res.message == 'Valid OTP') {
                this.accountLocked = false;
              }
              this.forgotService
                .sendMail(this.user.email)
                .subscribe((result: any) => {
                  this.error = '';
                  if (result.success) {
                    const alert = {
                      type: 'success',
                      message: `An email with further instructions for account recovery has been sent to ${this.user.email}.`,
                      closed: true
                    };
                    this.globalMessageService.clearAlerts();
                    this.globalMessageService.pushAlert(alert);
                  } else {
                    this.globalMessageService.clearAlerts();
                    this.error = result.message;
                  }
                });
              this.loading = false;
              this.showQRCode = false;
              this.showSecurityQuestions = false;
              this.otpError = false;
              this.otpGiven = '';
            } else {
              this.globalMessageService.pushAlert({
                closed: true,
                type: 'warning',
                message: 'Invalid OTP.'
              });
            }
          },
          error => {
            console.log(error);
          }
        );
    }
  }

  closeHandler() {
    this.loading = false;
    this.showQRCode = false;
    this.showSecurityQuestions = false;
    this.otpError = false;
    this.otpGiven = '';
  }

  setLocalItems() {
    let res = this.loginResponse;
    let token = res.token;
    let user = res.user;
    let unixTime = res.unixTime;
    this.token = token;
    let expiryTime = res.expiryTime.session_expiry;
    let purge_allowed = res.expiryTime.purge_allowed;
    let date_format = res.expiryTime.date_format;
    let gapiKey = res.expiryTime.gapiKey;
    let freshLogin = res.expiryTime.isFirstLogin;
    let portalTimeZone = res.user?.portalTimeZone || '';
    let customUserTimeZone = res.expiryTime.customUserTimeZone || '';

    if (!user.email_verified) {
      this.globalMessageService.pushAlert({
        closed: true,
        type: 'warning',
        message: 'Your account Email address is not verified.'
      });
    }

    let tokenExpiry: moment.Moment = moment().add(expiryTime, 'minutes');

    localStorage.setItem('expiryTime', '' + expiryTime + '');
    localStorage.setItem('purgeAllowed', '' + purge_allowed + '');
    localStorage.setItem('dateFormat', '' + date_format + '');
    localStorage.setItem('gapiKey', '' + gapiKey + '');

    // if (freshLogin == true) {
    localStorage.setItem('freshLogin', '' + btoa(freshLogin) + '');
    // }

    //Setting the user id in localStorage.
    localStorage.setItem('usd', '' + btoa(user.usd) + '');

    localStorage.setItem('ur', '' + btoa(user.role) + '');

    localStorage.setItem('tokenExpiry', JSON.stringify(unixTime));

    this.authenticationService.authStateChange.next(true);
    this.authenticationService.autologout(tokenExpiry);
    // tokenExpiry
    // store username and jwt token in local storage to keep user logged in between page refreshes

    if (
      (!this.isFirstLogin && this.is_question_changed) ||
      (!this.isFirstLogin && !this.is_question_changed)
    ) {
      localStorage.setItem(
        'currentUser',
        JSON.stringify({
          userFirstName: user.firstName,
          userName: user.email,
          token: token,
          tokenExpiry: tokenExpiry,
          isDarkMode: 'is_dark'
        })
      );
    }

    localStorage.setItem(
      'userData',
      JSON.stringify({
        token: token
      })
    );

    // this.authenticationService.autologout((new Date(res.tokenExpiry).getTime())-new Date().getTime());

    localStorage.setItem('token', '' + token + '');
    localStorage.setItem('portalTimeZone', portalTimeZone);
    this.authenticationService.showNavBar(true);
    this.authenticationService.showFirstName(this.userName);

    sessionStorage.removeItem('licenseNotificationShown');

    // window.location.reload();
    this.showLoginScreen = false;
    this.router.navigate(['/']);
  }

  resendOtp() {
    this.authenticationService.resendOtp(this.user.email).subscribe(
      (res: any) => {
        this.globalMessageService.pushAlert({
          closed: true,
          type: 'success',
          message: res.message
        });
      },
      error => {}
    );
  }

  validateAnswers() {
    this.authenticationService
      .validateAnswers(
        this.user.email,
        this.user.answer_1,
        this.user.answer_2,
        this.user.answer_3
      )
      .subscribe((res: any) => {
        if (res.status) {
          if (res.mfa_enabled) {
            this.showSecurityQuestions = false;
            this.showQRCode = true;
          } else {
            this.accountLocked = false;
            this.forgotService.sendMail(this.user.email).subscribe(
              (result: any) => {
                this.error = '';
                if (result.success) {
                  const alert = {
                    type: 'success',
                    message: `An email with further instructions for account recovery has been sent to ${this.user.email}.`,
                    closed: true
                  };
                  this.globalMessageService.clearAlerts();
                  this.globalMessageService.pushAlert(alert);
                } else {
                  this.globalMessageService.clearAlerts();
                  this.error = result.message;
                }
              },
              error => {
                console.log(error);
              }
            );
            this.loading = false;
            this.showQRCode = false;
            this.showSecurityQuestions = false;
            this.otpError = false;
            this.otpGiven = '';
          }
        } else {
          this.globalMessageService.pushAlert({
            closed: true,
            type: 'warning',
            message: 'Invalid Answers.'
          });
        }
      });
  }

  togglePassword() {
    this.showPassword = !this.showPassword;
  }

  setRolesData() {
    this.rolesPermissionService.getAssignedUserTemplates().subscribe({
      next: next => {
        this.rolesPermissionService.setAssignedUserTemplates(next);
        localStorage.setItem('Roles', JSON.stringify(next));
      }
    });
  }

  setLicense() {
    this.licenseService.getLicense().subscribe({
      next: next => {
        this.s2sLicenseValue = next.license.s2s_microseg_feature;
        localStorage.setItem(
          'MicroSegLicense',
          JSON.stringify(this.s2sLicenseValue)
        );
        this.licenseService.updateMicroSegLicense(this.s2sLicenseValue);
      },
      error: err => {
        const alert = {
          closed: false,
          type: 'danger',
          message: 'You are not authorized to access this page.'
        };
        this.globalMessageService.pushAlert(alert);
      },
      complete: () => {}
    });
  }
}
