import { Component, OnDestroy, OnInit, ViewContainerRef } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Subscription } from 'rxjs';
import { ManageService } from '../manage.service';
import { Service } from '../../models/service';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { GatewayGroupsService } from '../../gateway-groups/gateway-groups.service';
import { forkJoin } from 'rxjs';
import { BsModalService, BsModalRef } from 'ngx-bootstrap/modal';
import { ContextHelpEnum } from '../../shared/emuns/global.enum';
import { Title } from '@angular/platform-browser';

//^\d+(,\d+)*$
//([1-9][0-9]{0,3}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])
@Component({
  selector: 'app-create-manage',
  templateUrl: './create-manage.component.html',
  styleUrls: ['./create-manage.component.css']
})
export class CreateManageComponent implements OnInit {
  public alert: GlobalMessageClass;
  service = new Service();
  selectedProtocol: string = 'tcp';
  protocols = [
    { value: 'tcp', display: 'TCP' },
    { value: 'udp', display: 'UDP' }
  ];
  private roleSubscription: Subscription;
  public pageTitle: string = 'Add Service';
  public ports: Array<any>;
  public mports: Array<any>;
  public proto: Array<any>;
  public showAddPort: boolean = false;
  public showAddEndpoint: boolean = false;
  public singlePort: any = '';
  public singleEndpoint: any = '';
  public singleMPort: any = '';
  public singleProtocol: any = 'tcp';
  public isEdit = false;
  //Contextual Help
  public contextualHelp: Array<any>;
  public showContext: boolean = false;
  public submitting: boolean = false;
  loading: Boolean = false;
  public gatewayGroups = [];
  public selectedGatewayGroup = null;
  disableGatewayGroup = false;
  proxyErrorMessage: string;
  mappedUrlRegex = {
    proxyEnabled: /^(?:([a-z]{1,9}):\/\/)?(?:[a-zA-Z0-9.-]+?\.(?:[a-zA-Z]{1,}(:([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?(\/.*)?$)|(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(:([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?(\/.*)?$)/,
    proxyDisabled: /^(?:([a-z]{1,9}):\/\/)?(?:[a-zA-Z0-9.-]+?\.(?:[a-zA-Z]{1,}(:([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?(\/.*)?$)|(^(N|n)one$)|(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(:([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?(\/.*)?$)/
  };
  startPort: string = '';
  endPort: string = '';
  startMPort: string = '';
  endMPort: string = '';
  portRangeEnabled: boolean = false;
  disableSaveButton = false;
  public availableDevices: Array<any>;
  public deviceNames: Array<string>;
  public savedDeviceName: string;
  //Contextual Help
  constructor(
    vcRef: ViewContainerRef,
    private router: Router,
    private route: ActivatedRoute,
    private manageService: ManageService,
    private globalMessageService: GlobalMessageService,
    private gatewayGroupServices: GatewayGroupsService,
    private titleService: Title
  ) {
    //overlay.defaultViewContainer = vcRef;
    this.contextualHelp = ['', ''];
  }

  showContextHelp(name: any): void {
    if (name == ContextHelpEnum.name) {
      this.contextualHelp = ['name', 'Descriptive service name.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.ipv4_address) {
      this.contextualHelp = [
        'ipv4_address',
        'Valid IP address or host name (domain name).'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.type) {
      this.contextualHelp = [
        'type',
        'Protocol type of the service such as HTTP, HTTPS, SSH etc.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.mappedurl) {
      this.contextualHelp = [
        'mappedurl',
        'URL/Host name assigned to the service'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.port) {
      this.contextualHelp = ['port', 'Actual port the service is bound to.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.protocol) {
      this.contextualHelp = ['protocol', 'Service protocol (TCP, UDP, etc).'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.mappedport) {
      this.contextualHelp = ['mappedport', 'Port assigned to the service.'];
      this.showContext = true;
    }
  }

  closeContextualHelp(): void {
    this.showContext = false;
  }

  handleProxyClick() {
    if (
      this.service.port &&
      this.service.port.split(',').length > 1 &&
      !this.service.proxy_enabled
    ) {
      this.proxyErrorMessage = 'Cannot enable proxy for more than 1 ports';
    } else {
      this.service.proxy_enabled = !this.service.proxy_enabled;
      this.proxyErrorMessage = '';
    }
  }

  handlePortRangeClick() {
    this.singlePort = '';
    this.singleMPort = '';
    this.startPort = '';
    this.startMPort = '';
    this.endPort = '';
    this.endMPort = '';
    this.portRangeEnabled = !this.portRangeEnabled;
  }

  evaluateEndMappedPort() {
    this.endMPort = '';
    if (this.startPort == '' || this.endPort == '' || this.startMPort == '') {
      return;
    }
    let portDifference = parseInt(this.endPort) - parseInt(this.startPort);
    if (portDifference < 0) {
      return;
    }
    this.endMPort = (parseInt(this.startMPort) + portDifference).toString();
  }

  ngOnInit() {
    this.disableSaveButton = false;
    this.service.endpoints = [];

    const devices$ = this.manageService.getAllAvailableDevices();
    devices$.subscribe(response => {
      this.availableDevices = response;
      this.deviceNames = this.availableDevices.map(
        device => device.device_name
      );
    });

    if (typeof this.route.params['_value']['id'] != 'undefined') {
      this.loading = true;
      this.pageTitle = 'Edit Service';

      const service$ = this.manageService.getmanagedetail(
        this.route.params['_value']['id']
      );
      forkJoin([devices$, service$]).subscribe(([devices, service]) => {
        this.loading = false;
        if (!service[0]) {
          this.alert = {
            closed: false,
            type: 'danger',
            message: 'No service found with the specified ID.'
          };
          this.globalMessageService.pushAlert(this.alert);
          this.router.navigate(['/services']);
        } else {
          this.service = service[0];
          // if (this.service.isGateWayAssigned) {
          //   this.alert = {
          //     closed: false,
          //     type: 'danger',
          //     message:
          //       'This service has been assigned to gateway you cannot edit it.'
          //   };
          //   this.globalMessageService.pushAlert(this.alert);
          //   this.router.navigate(['/services']);
          // }
          let device = this.availableDevices.find(
            device => device.id == this.service.device_id
          );
          this.savedDeviceName = device ? device.device_name : null;

          this.ports = this.service.port.split(',');
          this.mports = this.service.dummy_port.split(',');
          this.proto = this.service.protocol.split(',');
        }
      });
      this.isEdit = true;
    } else {
      this.service.mappedurl = 'None';
    }
    this.titleService.setTitle('Infrastructure | Services | ' + this.pageTitle);
    this.singleProtocol = this.protocols[0].value;
    this.gatewayGroupServices.getAvailableGatewaysGroups().subscribe(result => {
      this.gatewayGroups = result;
      this.gatewayGroups.map(group => {
        if (group.all_gateways && group.all_services) {
          this.service.gateway_group_id = this.gatewayGroups[0].id;
          this.disableGatewayGroup = true;
        }
      });
    });
  }

  checkIfDuplicateExists(portArray, protocolArray, dummyPortArray) {
    let flag = false;
    portArray.forEach((port, portIndex) => {
      dummyPortArray.forEach((dummyPort, dummyPortIndex) => {
        if (port == dummyPort) {
          if (
            protocolArray[portIndex] == protocolArray[dummyPortIndex] &&
            portIndex != dummyPortIndex
          ) {
            flag = true;
          }
        }
      });
    });
    return flag;
  }

  onsubmit() {
    this.submitting = true;
    this.loading = true;
    this.disableSaveButton = true;
    let portsCount = 0;
    let device = this.availableDevices.find(
      device => device.device_name === this.savedDeviceName
    );
    this.service.device_id = device ? device.id : null;

    if (this.isEdit == true) {
      portsCount = this.service.port.split(',').length;
    } else {
      if (this.service.port != null) {
        portsCount = this.service.port.split(',').length;
      }
    }
    if (portsCount >= 1 && portsCount <= 64 && this.service.port != '') {
      if (
        !this.checkIfDuplicateExists(
          this.service.port.split(','),
          this.service.protocol.split(','),
          this.service.dummy_port.split(',')
        )
      ) {
        this.service.Name = this.service.Name.trim();
        this.manageService.add_user(this.service).subscribe(
          response => {
            this.loading = false;
            if (response.success) {
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Service has been added/updated successfully.'
              };
              this.globalMessageService.pushAlert(this.alert);
              this.router.navigate(['/services']);
            } else {
              // this.alert = {
              //   closed: false,
              //   type: 'danger',
              //   message: response.message
              // };
              // this.globalMessageService.pushAlert(this.alert);
            }
          },
          err => {
            this.disableSaveButton = false;
            this.loading = false;
            this.submitting = false;
            // this.alert = {
            //   closed: false,
            //   type: 'danger',
            //   message: err.error.message
            // };
            // this.globalMessageService.pushAlert(this.alert);
          },
          () => {
            this.submitting = false;
          }
        );
      } else {
        this.loading = false;
        this.disableSaveButton = false;
        this.submitting = false;
        this.alert = {
          closed: false,
          type: 'danger',
          message: 'Service ports and mapped ports must be unique.'
        };
        this.globalMessageService.pushAlert(this.alert);
      }
    } else {
      this.loading = false;
      this.disableSaveButton = false;
      this.submitting = false;
      this.alert = {
        closed: false,
        type: 'danger',
        message:
          'Service can have 64 ports or port ranges.'
      };
      this.globalMessageService.pushAlert(this.alert);
    }
  }

  onCancel(): void {
    this.router.navigate(['/services']);
  }

  portOverlapExists(
    portArr: Array<string>,
    protocolArr: Array<string>,
    val: string,
    protocol: string
  ): boolean {
    let flag = false;
    if (val.includes(':') == false) {
      portArr.forEach((port, index) => {
        if (port.includes(':') == false) {
          if (port == val && protocolArr[index] == protocol) {
            flag = true;
          }
        } else {
          let startingPort = parseInt(port.split(':')[0]);
          let endingPort = parseInt(port.split(':')[1]);
          for (
            let currPort = startingPort;
            currPort <= endingPort;
            currPort++
          ) {
            if (parseInt(val) == currPort && protocolArr[index] == protocol) {
              flag = true;
              break;
            }
          }
        }
      });
    } else {
      let startingVal = parseInt(val.split(':')[0]);
      let endingVal = parseInt(val.split(':')[1]);
      for (let currVal = startingVal; currVal <= endingVal; currVal++) {
        portArr.forEach((port, index) => {
          if (port.includes(':') == false && protocolArr[index] == protocol) {
            if (parseInt(port) == currVal) {
              flag = true;
            }
          } else {
            let startingPort = parseInt(port.split(':')[0]);
            let endingPort = parseInt(port.split(':')[1]);
            for (
              let currPort = startingPort;
              currPort <= endingPort;
              currPort++
            ) {
              if (currVal == currPort && protocolArr[index] == protocol) {
                flag = true;
                break;
              }
            }
          }
        });
      }
    }
    return flag;
  }

  public addPortAction(): void {
    if (
      this.service.port &&
      this.service.port.split(',').length >= 1 &&
      this.service.proxy_enabled
    ) {
      this.proxyErrorMessage =
        'Cannot add more than 1 port when proxy is enabled';
    } else {
      this.showAddPort = true;
      this.proxyErrorMessage = '';
    }
  }

  public addEndpointAction(): void {
    this.showAddEndpoint = true;
  }

  public addPortCompleteAction(): void {
    if (
      this.service.port &&
      this.service.port.split(',').length >= 1 &&
      this.service.proxy_enabled
    ) {
      this.proxyErrorMessage =
        'Cannot add more than 1 port when proxy is enabled';
      return;
    } else {
      this.proxyErrorMessage = '';
    }
    if (this.portRangeEnabled) {
      if (this.startPort == '' || this.endPort == '' || this.startMPort == '') {
        this.proxyErrorMessage = 'One or more port values missing.';
        return;
      } else {
        this.proxyErrorMessage = '';
      }
      let portDifference = parseInt(this.endPort) - parseInt(this.startPort);
      if (portDifference < 0 || portDifference == 0) {
        this.proxyErrorMessage = 'Start port cannot be equal to or greater than End port.';
        return;
      }
      this.endMPort = (parseInt(this.startMPort) + portDifference).toString();
      this.singlePort = this.startPort + ':' + this.endPort;
      this.singleMPort = this.startMPort + ':' + this.endMPort;
    }
    if (this.singlePort != '' && this.singleMPort != '') {
      if (this.isEdit == false) {
        if (this.service.port != null && this.service.port != '') {
          if (
            this.portOverlapExists(
              this.service.port.split(','),
              this.service.protocol.split(','),
              this.singlePort,
              this.singleProtocol
            ) == false
          ) {
            this.service.port += ',' + this.singlePort;
          } else {
            this.proxyErrorMessage =
              'Duplicate or overlapping port values detected.';
            return;
          }
        } else {
          this.service.port = '' + this.singlePort;
        }

        if (this.service.dummy_port != null && this.service.dummy_port != '') {
          if (
            this.portOverlapExists(
              this.service.dummy_port.split(','),
              this.service.protocol.split(','),
              this.singleMPort,
              this.singleProtocol
            ) == false
          ) {
            this.service.dummy_port += ',' + this.singleMPort;
          } else {
            this.proxyErrorMessage =
              'Duplicate or overlapping port values detected.';
            return;
          }
        } else {
          this.service.dummy_port = '' + this.singleMPort;
        }

        if (this.service.protocol != null && this.service.protocol != '') {
          this.service.protocol += ',' + this.singleProtocol;
        } else {
          this.service.protocol = '' + this.singleProtocol;
        }
      } else {
        if (
          this.service.port.split(',').length >= 1 &&
          this.service.port != ''
        ) {
          if (
            this.portOverlapExists(
              this.service.port.split(','),
              this.service.protocol.split(','),
              this.singlePort,
              this.singleProtocol
            ) == false
          ) {
            this.service.port += ',' + this.singlePort;
          } else {
            this.proxyErrorMessage =
              'Duplicate or overlapping port values detected.';
            return;
          }
        } else {
          this.service.port = '' + this.singlePort;
        }

        if (
          this.service.dummy_port.split(',').length >= 1 &&
          this.service.dummy_port != ''
        ) {
          if (
            this.portOverlapExists(
              this.service.dummy_port.split(','),
              this.service.protocol.split(','),
              this.singleMPort,
              this.singleProtocol
            ) == false
          ) {
            this.service.dummy_port += ',' + this.singleMPort;
          } else {
            this.proxyErrorMessage =
              'Duplicate or overlapping port values detected.';
            return;
          }
        } else {
          this.service.dummy_port = '' + this.singleMPort;
        }

        if (
          this.service.protocol.split(',').length >= 1 &&
          this.service.protocol != ''
        ) {
          this.service.protocol += ',' + this.singleProtocol;
        } else {
          this.service.protocol = '' + this.singleProtocol;
        }
      }
      var lastWorkingChild =
        document.getElementById('ports-table').childElementCount - 1;

      document.getElementById(
        'ports-table'
      ).innerHTML += document.getElementById('ports-table').children[
        lastWorkingChild
      ].innerHTML;

      var lastCreatedChild = document.getElementById('ports-table')
        .lastElementChild;

      lastCreatedChild.classList.remove('hide');

      for (var i = 0; i < lastWorkingChild + 2; ++i) {
        document
          .getElementById('ports-table')
          .children[i].addEventListener('click', this.removeIndex.bind(this));
      }

      lastCreatedChild.children[0].innerHTML = this.singlePort;
      lastCreatedChild.children[1].innerHTML = this.singleProtocol;
      lastCreatedChild.children[2].innerHTML = this.singleMPort;
      this.showAddPort = false;
      this.singlePort = '';
      this.singleMPort = '';
      this.startPort = '';
      this.startMPort = '';
      this.endPort = '';
      this.endMPort = '';
    } else {
      this.alert = {
        closed: false,
        type: 'danger',
        message: 'Kindly fill all the values.'
      };
      this.globalMessageService.pushAlert(this.alert);
    }
  }

  public removeIndex(event: any): void {
    this.proxyErrorMessage = '';
    if (event.target.parentElement.parentElement.parentElement.rowIndex) {
      let index =
        event.target.parentElement.parentElement.parentElement.rowIndex - 2;
      event.target.parentElement.parentElement.parentElement.remove();
      let portArray = this.service.port.split(',');
      let dummyPortArray = this.service.dummy_port.split(',');
      let protocolArray = this.service.protocol.split(',');
      if (index > -1) {
        portArray.splice(index, 1);
        dummyPortArray.splice(index, 1);
        protocolArray.splice(index, 1);
      }
      this.service.port = portArray.join();
      this.service.dummy_port = dummyPortArray.join();
      this.service.protocol = protocolArray.join();
    }
  }

  public addEndpointCompleteAction(): void {
    if (this.singleEndpoint) {
        this.service.endpoints.push(this.singleEndpoint);
        this.showAddEndpoint = false;
        this.singleEndpoint = '';
    } else {
        this.alert = {
            closed: false,
            type: 'danger',
            message: 'Kindly fill all the values.'
        };
        this.globalMessageService.pushAlert(this.alert);
    }
  }

  public removeEndpointIndex(index: number): void {
    if (index >= 0 && index < this.service.endpoints.length) {
        this.service.endpoints.splice(index, 1);
    } else {
        console.log("Invalid index provided.");
    }
  }

  public checkLength(str: any, str1: any) {
    if (str.value && str1.value) {
      return str.value.split(',').length !== str1.value.split(',').length;
    }
  }

  navigate(pageTitle:string){}
}
