import { Component, OnInit, EventEmitter, Output, Input } from '@angular/core';
import * as _ from 'lodash';
import { GlobalMessageService } from '../../../global-messages/global-messages.service';
import { PolicyService } from '../../../policy/policy.service';
import { ManageService } from '../../manage.service';

@Component({
  selector: 'create-quick-service-group',
  templateUrl: './create-quick-service-group.component.html',
  styleUrls: ['./create-quick-service-group.component.css']
})
export class CreateQuickServiceGroupComponent implements OnInit {
  @Output('delete') delete = new EventEmitter<any>();
  @Output('close') close = new EventEmitter<any>();
  @Output('successClose') successClose = new EventEmitter<any>();
  @Input('selectedServices') selectedServices = {};
  @Input('allSelected') allSelected = false;
  @Input('isFilterApplied') isFilterApplied = false;

  _: any = _;
  existingMode: boolean = false;
  serviceGroups = [];
  selectedServiceGroup = null;
  groupName: string = '';
  groupDescription: string = '';
  loading: boolean = false;

  constructor(
    private groupServices: PolicyService,
    private globalMessageService: GlobalMessageService
  ) {}

  ngOnInit() {
    this.close.subscribe(event => {});
    this.groupServices
      .getAvailableServiceGroups()
      .subscribe((response: any) => {
        this.serviceGroups = response.data;
      });
  }

  removeService(service) {
    delete this.selectedServices[service.id];
  }

  changeToExisting() {
    this.existingMode = !this.existingMode;
  }

  handleCreate() {
    this.loading = true;
    const all = this.allSelected;
    if (!this.existingMode) {
      const serviceIds = Object.keys(this.selectedServices);
      this.groupServices
        .addNewGroup(this.groupName, this.groupDescription, serviceIds, all)
        .subscribe(
          (response: any) => {
            this.globalMessageService.pushAlert({
              type: 'success',
              message: response.message
            });
            this.successClose.emit();
          },
          err => {
            this.loading = false;
          }
        );
    } else {
      const serviceIds = Object.keys(this.selectedServices);
      const selectedGroup = this.serviceGroups.find(
        group => group.id === this.selectedServiceGroup
      );
      this.groupServices
        .editGroup(
          this.selectedServiceGroup,
          selectedGroup.name,
          selectedGroup.description,
          serviceIds,
          true,
          all
        )
        .subscribe(
          response => {
            this.globalMessageService.pushAlert({
              type: 'success',
              message: 'Service group updated'
            });
            this.successClose.emit();
          },
          err => {
            this.loading = false;
          }
        );
    }
  }
}
