import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { ManageService } from '../manage.service';
import { Service } from '../../models/service';
import { Pagination } from '../../models/pagination';
import { PreserveSearch } from '../../models/preserve-search';
import * as _ from 'lodash';
import { Observable, of } from 'rxjs';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { AuthenticationService } from '../../login/authentication.service';
import { Subscription, timer } from 'rxjs';
import { Subject } from 'rxjs';
import { GatewayGroupsService } from '../../gateway-groups/gateway-groups.service';
import {
  debounceTime,
  distinctUntilChanged,
  switchMap,
  first,
  tap
} from 'rxjs/operators';
import { ServiceConnectionOptionsEnum } from '../../shared/emuns/service.enum';
import { ErrorCodeEnum } from '../../shared/emuns/error.enum';
import { LicenseErrorEnum } from '../../shared/emuns/license.enum';
import { ConfirmModalComponent } from '../../../app/shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { AccessPolicyService } from '../../create-access-policy/access-policy.service';
import { RolesAndPermissionService } from '../../roles-and-permission/roles-and-permission.service';
import { DashboardService } from '../../dashboard/dashboard.service';

@Component({
  selector: 'app-manage-list',
  templateUrl: './manage-list.component.html',
  styleUrls: ['./manage-list.component.css']
})
export class ManageListComponent implements OnInit {
  showHelpText: boolean = false;
  public alert: GlobalMessageClass;
  services: any = {};
  private servicesList = new Array();
  private selectedService: Service;
  pagination: Pagination = new Pagination();
  private Search: PreserveSearch = new PreserveSearch();
  public serviceDeleteError = false;
  public serviceDeleteErrorMessage = '';
  private timerSubscription: Subscription;
  showPasswordConfimation = false;
  showPasswordConfimationBulkDelete = false;
  itemsPerPage = 10;
  ConnectionOptions = ServiceConnectionOptionsEnum;

  sort: string;
  sortDir: string;
  access_status: string;

  is_persistent = false;
  selectedConnection;

  $searchService = new Subject();
  searchKeyword: string = '';
  isSearching = false;
  isNavigatingToSeach: boolean = false;

  loading: Boolean = true;

  pageSize = [10, 25, 50, 100, 200];

  selectAll: boolean = false;

  selectAllExcept = new Set();
  selectedServices: any = new Set();

  gatewayGroups = [];
  selectedGatewayGroups = [];
  disableAllSelect: boolean = false;
  showQuickGroup: boolean = false;

  normalizedSelectedServices = {};

  isDarkMode: boolean;
  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;
  microSegLicense: any;

  contentPortalStatus: Number;
  onboardingPotalStatus: Number;
  adminsCount: number;
  servicesCount: number;
  policiesCount: number;
  alertLogsCount: number;
  gateways: number = 1;
  gatewaysActive: number;
  controllers: number = 1;
  controllersActive: number;
  countFetched: boolean = false;
  clients: number = 0;
  ldaps: number = 1;
  clientsActive: number;
  userrole: string = '';
  secondaryControllersCount: number;
  onlineSecondaryControllers: number;
  mfaServerActive: number;
  primaryControllerStatus: Boolean;
  isSettingsSet: boolean = true;

  constructor(
    private router: Router,
    private manageService: ManageService,
    private route: ActivatedRoute,
    private accessService: AccessPolicyService,
    private globalMessageService: GlobalMessageService,
    private authenticationService: AuthenticationService,
    private gatewayGroupsServices: GatewayGroupsService,
    public dialog: MatDialog,
    private titleService: Title,
    private rolesPermissionService: RolesAndPermissionService,
    private dashboardService: DashboardService
  ) {}

  searchingHandler() {
    this.$searchService
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((searchWord: string) => {
          this.loading = true;
          this.searchKeyword = searchWord;
          this.isSearching = true;
          this.pagination.currentPage = 1;
          this.selectAll = false;
          this.selectAllExcept = new Set();
          return of(
            this.getPage(
              this.pagination.currentPage,
              this.sort,
              this.sortDir,
              this.searchKeyword
            )
          );
        })
      )
      .subscribe(data => {
        this.loading = false;
      });
  }

  itemsPerPageChange() {
    this.pagination.itemsPerPage =
      this.itemsPerPage == -1 ? 99999999999 : this.itemsPerPage;
    this.getPage(
      this.pagination.currentPage,
      this.sort,
      this.sortDir,
      this.searchKeyword
    );
  }

  paginationPageChange(page) {
    this.getPage(page, this.sort, this.sortDir, this.searchKeyword);
  }

  getPage(
    page: number,
    sort,
    dir,
    searchKeyword,
    persistent = this.is_persistent,
    selectedConnection = this.selectedConnection
  ) {
    this.loading = true;
    let prevPage = this.pagination.currentPage;
    this.pagination.currentPage = page;
    this.manageService
      .getServices(
        this.pagination.currentPage,
        this.pagination.itemsPerPage,
        'all',
        sort,
        dir,
        searchKeyword,
        persistent,
        selectedConnection,
        this.selectedGatewayGroups
      )
      .pipe(
        tap((res: any) => {
          this.loading = false;
          this.isSearching = false;

          res.data = res.data.map(client => {
            return { ...client, isChecked: false };
          });

          this.services = res;

          setTimeout(() => {
            this.changeCheckStatusOfAllClients(this.selectAll);
          }, 10);

          this.pagination.totalItems = res.recordsTotal;
          this.pagination.currentPage = page;
          if (prevPage != this.pagination.currentPage) {
            let scrollToTop = window.setInterval(() => {
              let pos = window.pageYOffset;
              if (pos > 0) {
                window.scrollTo(0, pos - 20); // how far to scroll on each step
              } else {
                window.clearInterval(scrollToTop);
              }
            }, 16);
          }
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          this.loading = false;
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'You are not authorized to access this page.'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          }
        },
        complete: () => {
          // this.selectAll = false
          this.disableAllSelect =
            this.services.data.findIndex(
              service => !service.isGateWayAssigned
            ) < 0;
        }
      });
  }

  clearfun() {
    this.searchKeyword = '';
    this.getPage(
      this.pagination.currentPage,
      this.sort,
      this.sortDir,
      this.searchKeyword
    );
  }

  ngOnDestroy(): void {
    if (!this.isNavigatingToSeach) {
      this.Search.saveSearch('serviceSearch', null);
    }
  }

  public ngOnInit(): void {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.microSegLicense = localStorage.getItem('MicroSegLicense');
    this.getCRUDdata(22);
    this.searchingHandler();
    let serviceSearch = this.Search.getSearch('serviceSearch');
    if (serviceSearch) {
      this.pagination = serviceSearch.pagination;
      this.searchKeyword = serviceSearch.searchWord;
      this.is_persistent = serviceSearch.is_persistent;
    }
    this.getGatewayGroups();

    if (this.microSegLicense == 1 || this.microSegLicense == 0) {
      this.refreshData();
    }
    this.titleService.setTitle('Infrastructure | Services');
    this.countOnline();
  }

  getCRUDdata(privId) {
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false;
      this.readPermission = res?.read_permission === 1 ? true : false;
      this.updatePermission = res?.update_permission === 1 ? true : false;
      this.deletePermission = res?.delete_permission === 1 ? true : false;
    });
  }

  public refreshData(number = null): void {
    if (this.timerSubscription) {
      this.timerSubscription.unsubscribe();
    }
    if (number != null) {
      this.pagination.currentPage = number;
    }
    this.getPage(
      this.pagination.currentPage,
      this.sort,
      this.sortDir,
      this.searchKeyword
    );
    this.subscribeToData();
  }

  private subscribeToData(): void {
    this.timerSubscription = timer(10000)
      .pipe(first())
      .subscribe(() => this.refreshData());
  }

  getGatewayGroups() {
    this.loading = true;
    this.gatewayGroupsServices.getAvailableGatewaysGroups().subscribe(
      result => {
        let gatewayParams: any = this.route.snapshot.queryParamMap.getAll(
          'gatewayGroup'
        );
        let parameters = [];
        if (gatewayParams && gatewayParams.length > 0) {
          gatewayParams.forEach(param => {
            if (param.startsWith('[') && param.endsWith(']')) {
              parameters = JSON.parse(param);
            } else {
              parameters.push(parseInt(param));
            }
          });
        }
        this.gatewayGroups = result;
        this.selectedGatewayGroups = parameters.filter(selected => {
          return (
            this.gatewayGroups.findIndex(group => group.id === selected) > -1
          );
        });
      },
      null,
      () => {
        this.getPage(
          this.pagination.currentPage,
          this.sort,
          this.sortDir,
          this.searchKeyword
        );
      }
    );
  }

  handleGatewayChange() {
    this.pagination.currentPage = 1;
    this.getPage(
      this.pagination.currentPage,
      this.sort,
      this.sortDir,
      this.searchKeyword
    );
  }

  countOnline() {
    this.dashboardService
      .countHost()
      .pipe(
        tap((res: any) => {
          this.countFetched = true;
          if (res.status == true) {
            this.clients = res.clients;
            this.clientsActive = res.clientsActive;
            this.gateways = res.gateways;
            this.gatewaysActive = res.gatewaysActive;
            this.secondaryControllersCount = res.secondaryControllersCount;
            this.mfaServerActive = res.mfaServerStatus;
            this.controllersActive = res.controllersActive;
            this.ldaps = res.ldaps;
            this.isSettingsSet = res.isSettingsSet;
            this.contentPortalStatus = res.contentPortalStatus;
            this.onboardingPotalStatus = res.onboardingPotalStatus;
            this.adminsCount = res.adminsCount;
            this.servicesCount = res.servicesCount;
            this.policiesCount = res.policiesCount;
            this.alertLogsCount = res.alertLogsCount;
            this.onlineSecondaryControllers = res.onlineSecondaryControllers;
            this.primaryControllerStatus = res.primaryControllerStatus;
          } else {
            this.clients = 0;
            this.clientsActive = 0;
            this.gateways = 0;
            this.gatewaysActive = 0;
            this.controllers = 0;
            this.controllersActive = 0;
            this.ldaps = 0;
            this.mfaServerActive = 0;
            this.isSettingsSet = false;
            this.contentPortalStatus = 0;
            this.onboardingPotalStatus = 0;
          }
        })
      )
      .subscribe({
        error: err => {
          console.log(err && (err.message || err));
        },
        complete: () => {}
      });
  }

  addController() {
    if (this.ldaps !== 0) {
      this.router.navigate(['/create-controller']);
    }
  }

  addGateway() {
    if (this.controllers !== 0) {
      this.router.navigate(['/create-gateway']);
    }
  }

  addLdap() {
    if (!this.ldaps) {
      this.router.navigate(['/create-ldapuser']);
    }
    if (!this.isSettingsSet) {
      this.router.navigate(['/settings']);
    }
  }

  private onSelect(service: Service): void {
    let serviceSearch = {
      pagination: this.pagination,
      searchWord: this.searchKeyword,
      is_persistent: this.is_persistent
    };
    this.isNavigatingToSeach = true;
    this.Search.saveSearch('serviceSearch', serviceSearch);

    this.selectedService = service;
    this.router.navigate(['/services', this.selectedService.id]);
  }

  private onDelete(service: Service): void {
    this.selectedService = service;
    this.showPasswordConfimation = true;
  }

  private onAssignAll(service: Service): void {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message:
          'You are going to assign this service to all clients, Are you sure?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.selectedService = service;

        this.manageService
          .assignServiceToAllClient(service.id)
          .pipe(
            tap((response: any) => {
              if (response.success) {
                this.alert = {
                  closed: false,
                  type: 'success',
                  message: 'Service has been assigned to all clients.'
                };
                this.globalMessageService.pushAlert(this.alert);
              }
            })
          )
          .subscribe({
            next: next => {},
            error: err => {
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'Failed to assign service to all clients.'
              };
              this.globalMessageService.pushAlert(this.alert);
            },
            complete: () => {}
          });
      }
    });
  }

  private onUnAssignAll(service: Service): void {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message:
          'You are going to un-assign this service from all the clients, Are you sure?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.selectedService = service;

        this.manageService
          .unassignServiceToAllClient(service.id)
          .pipe(
            tap((response: any) => {
              if (response.success) {
                this.alert = {
                  closed: false,
                  type: 'success',
                  message:
                    'Service has been un-assigned from all clients and groups.'
                };
                this.globalMessageService.pushAlert(this.alert);
              }
            })
          )
          .subscribe({
            next: next => {},
            error: err => {
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'Failed to un-assign service from all clients.'
              };
              this.globalMessageService.pushAlert(this.alert);
            },
            complete: () => {}
          });
      }
    });
  }

  hideAlert() {
    this.serviceDeleteError = false;
    this.serviceDeleteErrorMessage = '';
  }

  add_new_user(): void {
    this.router.navigate(['/create-manage']);
  }

  cancelVerificationPassword(event) {
    this.showPasswordConfimation = false;
    this.showPasswordConfimationBulkDelete = false;
  }

  submitVerificationPassword(event) {
    this.manageService
      .delete_service(this.selectedService.id, event)
      .pipe(
        tap((response: any) => {
          if (response.success) {
            this.alert = {
              closed: false,
              type: 'success',
              message: 'Service has been deleted successfully.'
            };
            this.globalMessageService.pushAlert(this.alert);

            this.showPasswordConfimation = false;
            this.serviceDeleteError = !response.success;
            this.serviceDeleteErrorMessage = '';
            this.getPage(
              this.pagination.currentPage,
              this.sort,
              this.sortDir,
              this.searchKeyword
            );
          }
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          let error = err;
          this.serviceDeleteError = !error.success;
          this.serviceDeleteErrorMessage = 'Error in deleting service.';
        },
        complete: () => {}
      });
  }

  changePersistenceFlag(status) {
    this.is_persistent = status;
    this.pagination.currentPage = 1;
    this.selectAll = false;
    this.selectAllExcept = new Set();
    this.getPage(
      this.pagination.currentPage,
      this.sort,
      this.sortDir,
      this.searchKeyword
    );
  }

  changeGatewayLessFilter() {
    this.pagination.currentPage = 1;
    this.selectAll = false;
    this.selectAllExcept = new Set();
    this.getPage(
      this.pagination.currentPage,
      this.sort,
      this.sortDir,
      this.searchKeyword
    );
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getPage(
      this.pagination.currentPage,
      this.sort,
      this.sortDir,
      this.searchKeyword
    );
  }

  sortRecord(term) {
    this.sort = term;
    this.sortDir = this.sortDir == 'desc' ? 'asc' : 'desc';
    
    this.getPage(
      this.pagination.currentPage,
      this.sort,
      this.sortDir,
      this.searchKeyword
    );
  }

  selectAllAction() {
    this.normalizedSelectedServices = {};
    this.selectAllExcept.clear();

    if (!this.selectAll) {
      this.selectAllExcept.clear();
      this.selectedServices.clear();
    } else {
      this.selectedServices.clear();
    }
    this.changeCheckStatusOfAllClients(this.selectAll);
  }

  private changeCheckStatusOfAllClients(check) {
    this.services.data = this.services.data.map(service => {
      return {
        ...service,
        isChecked:
          (!this.selectAllExcept.has(service.id) && this.selectAll) ||
          (this.selectedServices.has(service.id) && !this.selectAll)
      };
    });
  }

  changeCheckBox(client) {
    const { isChecked, id } = client;
    if (this.selectAll) {
      if (isChecked) {
        this.selectAllExcept.delete(id);
      } else {
        this.selectAllExcept.add(id);
      }
    } else {
      if (!isChecked) {
        delete this.normalizedSelectedServices[id];
        this.selectedServices.delete(id);
      } else {
        this.normalizedSelectedServices[id] = client;
        this.selectedServices.add(id);
      }
    }
  }

  deleteselectedServices() {
    this.showPasswordConfimationBulkDelete = true;
  }

  submitVerificationPasswordBulkDelete(password) {
    const selectedIds = this.selectAll
      ? this.selectAllExcept
      : this.selectedServices;
      const serviceIds =  Array.from(selectedIds)
      this.accessService.verifyPolicies('', '', serviceIds, [], [], [], this.selectAll).subscribe(res => {
        if(res){
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'danger',
            message: 'Remove services from Entitlement(s) first',
            sticky: false
          });
        }else{
          this.manageService
          .deleteBulkServices(
            this.selectAll,
            selectedIds,
            this.searchKeyword,
            this.is_persistent,
            password
          )
          .subscribe({
            next: res => {
              this.selectAll = false;
              this.showPasswordConfimationBulkDelete = false;
              this.selectAllExcept.clear();
              this.selectedServices.clear();
              this.getPage(
                this.pagination.currentPage,
                this.sort,
                this.sortDir,
                this.searchKeyword
              );
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Service(s) deleted successfully.'
              };
              this.globalMessageService.pushAlert(this.alert);
            },
            error: err => {
              // this.showPasswordConfimationBulkDelete = false;
              // this.alert = {
              //   closed: false,
              //   type: 'danger',
              //   message: 'Something went wrong in deleting service(s).'
              // };
              // this.globalMessageService.pushAlert(this.alert);
            }
          });
      }
    });
  }

  async handleAddToGroup() {
    if (
      this.selectAll &&
      (this.selectAllExcept.size < 1 ||
        this.searchKeyword ||
        this.is_persistent)
    ) {
      this.normalizedSelectedServices = {};
    }
    if (
      this.selectAll &&
      (this.selectAllExcept.size > 0 ||
        this.searchKeyword ||
        this.is_persistent)
    ) {
      const response = await this.manageService
        .getQuickGateways(
          this.searchKeyword,
          this.is_persistent,
          Array.from(this.selectAllExcept)
        )
        .toPromise();
      const services = response.data;
      if (services) {
        let normalize = {};
        services.forEach(service => {
          normalize[service.id] = service;
        });
        this.normalizedSelectedServices = normalize;
      }
    }
    this.showQuickGroup = true;
  }

  closeQuickGroup() {
    this.showQuickGroup = false;
  }

  successCloseQuickGroup() {
    this.showQuickGroup = false;
    this.selectAll = false;
    this.selectedServices = new Set();
    this.normalizedSelectedServices = {};
    this.changeCheckStatusOfAllClients(this.selectAll);
  }

  isFilterApplied() {
    return this.selectAllExcept.size > 0 || this.searchKeyword;
  }

  onViewService(serviceId) {
    this.router.navigate(['/view-service', serviceId]);
  }
}
