import { Component, OnInit, EventEmitter, Output, Input } from '@angular/core';
import * as _ from 'lodash';
import { GlobalMessageService } from '../../../global-messages/global-messages.service';
import { ActivatedRoute, Router } from '@angular/router';
import { ManageService } from '../../manage.service';
import { forkJoin } from 'rxjs';
import { GlobalMessageClass } from '../../../global-messages/global-messages';
import { Service } from '../../../models/service';
import { ContextHelpEnum } from '../../../shared/emuns/global.enum';
import { Pagination } from '../../../models/pagination';
import * as moment from 'moment';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { map } from 'rxjs/operators';
import { ConfigService } from '../../../services/config.service';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'view-detail-service',
  templateUrl: './view-detail-service.component.html',
  styleUrls: ['./view-detail-service.component.css']
})
export class ViewDetailsServiceComponent implements OnInit {
  public alert: GlobalMessageClass;
  public hideForNow: boolean = false

  viewMode = true
  loading = true
  public ports: Array<any>;
  public mports: Array<any>;
  public proto: Array<any>;

  public availableDevices: Array<any>;
  public deviceNames: Array<string>;
  public services: any
  public detailDataForService: any
  entitlementPagination: Pagination = new Pagination();
  routingPolicyPagination: Pagination = new Pagination();
  serviceGroupPagination: Pagination = new Pagination();
  public savedDeviceName: string;

  public contextualHelp: Array<any>;
  public showContext: boolean = false;

  isView: boolean = false;
  service = new Service();
  selectedProtocol: string = 'tcp';

  summaryOfAllServices: any = [];
  summaryOfAllServicesPagination: Pagination = new Pagination();
  summaryOfAllServicesLoading: boolean;

  private apiBaseUrl: string;
  private reportingApiUrl: string = '/reports';

  sortDir = true;


  pagination: Pagination = new Pagination();

  protocols = [
    { value: 'tcp', display: 'TCP' },
    { value: 'udp', display: 'UDP' }
  ];

  mappedUrlRegex = {
    proxyEnabled: /^(?:([a-z]{1,9}):\/\/)?(?:[a-zA-Z0-9.-]+?\.(?:[a-zA-Z]{1,}(:([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?(\/.*)?$)|(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(:([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?(\/.*)?$)/,
    proxyDisabled: /^(?:([a-z]{1,9}):\/\/)?(?:[a-zA-Z0-9.-]+?\.(?:[a-zA-Z]{1,}(:([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?(\/.*)?$)|(^(N|n)one$)|(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(:([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?(\/.*)?$)/
  };




  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private manageService: ManageService,
    private globalMessageService: GlobalMessageService,
    private authHttp: AuthHttp,
    private configService: ConfigService,
    private titleService: Title
  ) {
    this.contextualHelp = ['', ''];
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.reportingApiUrl = this.apiBaseUrl + this.reportingApiUrl;
    this.titleService.setTitle('Infrastructure | Services | Service View');
  }

  ngOnInit() {
    this.isView = false;
    if (typeof this.route.params['_value']['id'] != 'undefined') {
      this.loading = true;

      this.manageService.getSingleServiceDetails(this.route.params['_value']['id'])
        .subscribe((data: any) => {
          this.detailDataForService = data
          this.entitlementPagination.totalItems = this.detailDataForService?.userEntitlements?.length
          this.routingPolicyPagination.totalItems = this.detailDataForService?.routingPolicy?.length
          this.serviceGroupPagination.totalItems = this.detailDataForService?.serviceGroups?.length
        })

      const service$ = this.manageService.getmanagedetail(
        this.route.params['_value']['id']
      )
      forkJoin([service$]).subscribe(([service]) => {
        this.loading = false;
        if (!service[0]) {
          this.alert = {
            closed: false,
            type: 'danger',
            message: 'No service found with the specified ID.'
          };
          this.globalMessageService.pushAlert(this.alert);
          this.router.navigate(['/services']);
        } else {
          this.service = service[0];
          this.ports = this.service.port.split(',');
          this.mports = this.service.dummy_port.split(',');
          this.proto = this.service.protocol.split(',');
        }
      });

      this.getServicesSummary(this.route.params['_value']['id']
      )
        .subscribe({
          next: summary => {
            this.summaryOfAllServicesPagination.totalItems = summary.recordsTotal;
            this.summaryOfAllServices = summary;
            this.summaryOfAllServicesLoading = false;
          },
          error: err => {
            this.summaryOfAllServicesLoading = false;
          }
        })
    } else {
      this.service.mappedurl = 'None';
    }
    this.isView = true;
  }

  cancelHandler() {
    this.router.navigate(['/services']);
  }

  sortEntitlementRecord(sort) {
    if (sort === 'ent_id'){
      this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => b.policy_id - a.policy_id);
      !this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => a.policy_id - b.policy_id);
    }else if (sort === 'ent_name' || sort === 'ent_client'){
      this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => a.name.localeCompare(b.name));
      !this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => b.name.localeCompare(a.name));
    }else{
      this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => b.isRestricted
      - a.isRestricted
      );
      !this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => a.isRestricted
      - b.isRestricted
      );
    }
    this.sortDir = !this.sortDir;
  }

  sortEntRecord(sort){
    if (sort === 'user_id'){
      this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => b.localid - a.localid);
      !this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => a.localid - b.localid);
    }else if (sort === 'user_name'){
      this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => a.localName.localeCompare(b.localName));
      !this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => b.localName.localeCompare(a.localName));
    }else if (sort === 'user_ip'){
      this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => a.identity_provider.localeCompare(b.identity_provider));
      !this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => b.identity_provider.localeCompare(a.identity_provider));
    }else{
      this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => a.localusername.localeCompare(b.localusername));
      !this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => b.localusername.localeCompare(a.localusername));
    }
    this.sortDir = !this.sortDir;
  }

  sortDeviceRecord(sort) {
    if (sort === 'device_name'){
      this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => a.device_name.localeCompare(b.device_name));
      !this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => b.device_name.localeCompare(a.device_name));
    }else if (sort === 'device_type'){
      this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => a.device_type.localeCompare(b.device_type));
      !this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => b.device_type.localeCompare(a.device_type));
    }else if (sort === 'device_id'){
      this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => a.device_id.localeCompare(b.device_id));
      !this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => b.device_id.localeCompare(a.device_id));
    }else if (sort === 'device_os'){
      this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => a.os.localeCompare(b.os));
      !this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => b.os.localeCompare(a.os));
    }
    else {
      this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => a.inventory_sync_state.localeCompare(b.inventory_sync_state));
      !this.sortDir && this.detailDataForService.userEntitlements?.sort((a,b) => b.inventory_sync_state.localeCompare(a.inventory_sync_state));
    }
    this.sortDir = !this.sortDir;
  }

  sortServiceRecord(sort) {
    if (sort === 'service_name'){
      this.sortDir && this.summaryOfAllServices?.data?.sort((a,b) => a.service_name.localeCompare(b.service_name));
      !this.sortDir && this.summaryOfAllServices?.data?.sort((a,b) => b.service_name.localeCompare(a.service_name));
    }else if (sort === 'total_users'){
      this.sortDir && this.summaryOfAllServices?.data?.sort((a,b) => a.total_users - b.total_users);
      !this.sortDir && this.summaryOfAllServices?.data?.sort((a,b) => b.total_users - a.total_users);
    }else if (sort === 'bytes_sent'){
      this.sortDir && this.summaryOfAllServices?.data?.sort((a,b) => a.bytes_sent - b.bytes_sent);
      !this.sortDir && this.summaryOfAllServices?.data?.sort((a,b) => b.bytes_sent - a.bytes_sent);
    }else {
      this.sortDir && this.summaryOfAllServices?.data?.sort((a,b) =>  a.bytes_received - b.bytes_received);
      !this.sortDir && this.summaryOfAllServices?.data?.sort((a,b) =>  b.bytes_received - a.bytes_received);
    }
    this.sortDir = !this.sortDir;
  }

  sortRoutingRecord(sort) {
    if (sort === 'id'){
      this.sortDir && this.detailDataForService.routingPolicy?.sort((a,b) => a.RoutingPolicyGroupId - b.RoutingPolicyGroupId);
      !this.sortDir && this.detailDataForService.routingPolicy?.sort((a,b) => b.RoutingPolicyGroupId - a.RoutingPolicyGroupId);
    }else if (sort === 'name'){
      this.sortDir && this.detailDataForService.routingPolicy?.sort((a,b) => a.RoutingPolicyGroupName.localeCompare(b.RoutingPolicyGroupName));
      !this.sortDir && this.detailDataForService.routingPolicy?.sort((a,b) => b.RoutingPolicyGroupName.localeCompare(a.RoutingPolicyGroupName));
    }else if (sort === 'gateway_id'){
      this.sortDir && this.detailDataForService.routingPolicy?.sort((a,b) => a.gatewayId - b.gatewayId);
      !this.sortDir && this.detailDataForService.routingPolicy?.sort((a,b) => b.gatewayId - a.gatewayId);
    }else if (sort === 'sg_name'){
      this.sortDir && this.detailDataForService?.serviceGroups?.sort((a,b) => a.name.localeCompare(b.name));
      !this.sortDir && this.detailDataForService?.serviceGroups?.sort((a,b) => a.name.localeCompare(b.name));
    }else if (sort === 'sg_desc'){
      this.sortDir && this.detailDataForService?.serviceGroups?.sort((a,b) => a.description.localeCompare(b.description));
      !this.sortDir && this.detailDataForService?.serviceGroups.sort((a,b) => a.description.localeCompare(b.description));
    }else if (sort === 'sg_presistent'){
      this.sortDir && this.detailDataForService?.serviceGroups?.sort((a,b) => a.is_persistent - b.is_persistent);
      !this.sortDir && this.detailDataForService?.serviceGroups?.sort((a,b) => b.is_persistent - a.is_persistent);
    }else {
      this.sortDir && this.detailDataForService.routingPolicy?.sort((a,b) =>  a.gatewayName.localeCompare(b.gatewayName));
      !this.sortDir && this.detailDataForService.routingPolicy?.sort((a,b) =>  b.gatewayName.localeCompare(a.gatewayName));
    }
    this.sortDir = !this.sortDir;
  }

  showContextHelp(name: any): void {
    if (name == ContextHelpEnum.name) {
      this.contextualHelp = ['name', 'Descriptive service name.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.ipv4_address) {
      this.contextualHelp = [
        'ipv4_address',
        'Valid IP address or host name (domain name).'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.type) {
      this.contextualHelp = [
        'type',
        'Protocol type of the service such as HTTP, HTTPS, SSH etc.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.mappedurl) {
      this.contextualHelp = [
        'mappedurl',
        'URL/Host name assigned to the service'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.port) {
      this.contextualHelp = ['port', 'Actual port the service is bound to.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.protocol) {
      this.contextualHelp = ['protocol', 'Service protocol (TCP, UDP, etc).'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.mappedport) {
      this.contextualHelp = ['mappedport', 'Port assigned to the service.'];
      this.showContext = true;
    }
  }

  closeContextualHelp(): void {
    this.showContext = false;
  }

  getServicesSummary(
    service: string,
  ): any {
    const currentPage = 0;
    const itemsPerPage = 10;

    let _startDate = '';
    let _endDate = '';
    let _services = '';
    let _clients = '';
    let dir = 'asc';
    let sort = ''

    // if (services && services.length) {
    //   _services = JSON.stringify(services.map(v => v.service_id));
    // }

    return this.authHttp
      .get(
        this.reportingApiUrl +
        '/services/summary?offset=' +
        currentPage +
        '&limit=' +
        itemsPerPage +
        '&serviceId=' +
        service +
        '&clientId=' +
        _clients +
        '&startDate=' +
        _startDate +
        '&endDate=' +
        _endDate +
        '&dir=' +
        dir +
        '&sort=' +
        sort
      )
      .pipe(map((response: Response) => response));
  }

  navigate(){}
}
