import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Pagination } from '../models/pagination';
import { MicrosegmentService } from '../microsegmentation/microsegmentation.service';
import { PreserveSearch } from '../models/preserve-search';
import { Subject } from 'rxjs';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-microsegmentation',
  templateUrl: './microsegmentation.component.html',
  styleUrls: ['./microsegmentation.component.css']
})
export class MicrosegmentationComponent implements OnInit {
  pagination = new Pagination();
  $searchMicrosegment = new Subject();
  microsegments: any = {};
  searchWord;
  loading: Boolean = true;
  loadingPort: Boolean = true;
  workloadId;
  workloadName;
  devices = [];
  destinationDevices = [];
  ports;
  protocols;
  selectedDevice;
  selectedDestination;
  clientsPlaceHolder = 'Search Destination Device';
  allClientsSelected = false;
  selectedClientTypes = [];
  selectedClientCount = 0;
  title;
  destDevices;
  destinations;
  microsegmentId;
  workloadDevices;
  sort: string;
  sortDir: string;
  pageSize = [10, 25, 50, 100, 200];

  preserveSearch = new PreserveSearch();
  navigationToEdit: boolean = false;
  clients = [];
  clientOffset = 0;
  selectedClientIds = [];
  currentClientSearch = '';
  public alert: GlobalMessageClass;

  isDarkMode: boolean;
  constructor(
    private router: Router,
    private microsegmentationService: MicrosegmentService,
    private globalMessageService: GlobalMessageService,
    private route: ActivatedRoute,
    public dialog: MatDialog,
    private titleService: Title
  ) {}

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    if (this.route.snapshot.params.id) {
      this.title = 'Add';
      this.workloadId = +this.route.snapshot.params.id;
      this.getData(this.workloadId);
      this.titleService.setTitle('S2S Microsegmenation | Current Microsegments');
    }
    const _saveSearch = this.preserveSearch.getSearch('microsegmentSaveSearch');
    if (!_saveSearch) {
      this.getMicrosegments(this.workloadId);
    } else {
      this.searchWord = _saveSearch.searchKey;
      this.pagination = _saveSearch.pagination;
      this.getMicrosegments(this.workloadId);
    }
    this.subscribeToSearch();
    this.getClients();
  }

  getData(workloadId) {
    this.microsegmentationService
      .getWorkload(workloadId)
      .subscribe((data: any) => {
        this.loading = false;
        this.loadingPort = false;
        this.selectedDevice = [];
        this.devices = [];
        this.workloadName = data.workLoad.name;
        this.workloadId = data.workLoad.id;
        this.destinationDevices = data.devices;
        if (data.devices.length > 0) {
          this.devices = [...data.workLoad.DeviceInventories];
        }
      });
  }

  clearFields() {
    // this.workloadName = '';
    // this.selectedDevice = '';
    this.clients = [];
    this.selectedClientIds = [];
    this.selectedClientCount = 0;
    this.allClientsSelected = false;
    this.selectedDestination = '';
    this.ports = '';
    this.protocols = '';
    this.selectedClientTypes = [];
  }

  getClients() {
    this.clients = this.devices.filter(data => {
      return data.id != this.selectedDevice;
    });
  }

  deviceChange() {
    this.selectedClientIds = [];
    this.selectedDestination = '';
    this.ports = '';
    this.protocols = '';
    this.selectedClientTypes = [];
    if (this.workloadId) {
      this.microsegmentationService
        .getMicrosegment(this.workloadId, this.selectedDevice)
        .subscribe((res: any) => {
          if (res.id) {
            this.microsegmentId = res.id;
          } else {
            this.microsegmentId = '';
          }
        });
    }
  }

  destinationChange() {
    this.ports = '';
    this.protocols = '';
    this.microsegmentationService
      .getMicrosegment(this.workloadId, this.selectedDevice)
      .subscribe((res: any) => {
        if (res.id) {
          this.microsegmentId = res.id;
          this.microsegmentationService
            .getMicrosegmentDestinations(
              this.microsegmentId,
              this.selectedDestination
            )
            .subscribe((res: any) => {
              if (res) {
                this.ports = res.ports;
                this.protocols = res.protocols;
              }
            });
        }
      });
  }

  selectAllClients() {
    this.destinations = this.clients;
    this.selectedClientIds = [];
    this.selectedClientCount = 0;
    this.allClientsSelected = !this.allClientsSelected;
    this.clientsPlaceHolder = !this.allClientsSelected
      ? 'Search Destinations'
      : 'All destinatiins are selected';
  }

  selectAllClientsClick(selectall) {
    if (selectall) {
      if (this.selectedClientCount > 0) {
        const dialogRef = this.dialog.open(ConfirmModalComponent, {
          panelClass: this.isDarkMode ? 'dark-theme' : '',
          data: {
            title: 'Confirmation',
            message: `Are you sure you want to select all Destinations?`,
          },
        });
    
        dialogRef.afterClosed().subscribe((result) => {
          if (result){
            this.selectAllClients();
          }
        });
      } else {
        this.selectAllClients();
      }
    } else {
      this.selectAllClients();
    }
  }

  clientChange() {
    if (this.selectedClientIds.length >= this.selectedClientCount) {
      this.selectedClientCount = this.selectedClientCount + 1;
      const lastId = this.selectedClientIds[this.selectedClientIds.length - 1];
      const client = this.clients.find(item => item.id === lastId);
      this.selectedClientTypes = [...this.selectedClientTypes, client];
    } else {
      this.selectedClientCount = this.selectedClientIds.length;
      this.selectedClientTypes = this.selectedClientTypes.filter(client => {
        return this.selectedClientIds.indexOf(client.id) !== -1;
      });
    }
    this.destinations = this.selectedClientTypes;
    this.selectedDestination = null;
    this.ports = [];
    this.protocols = [];
  }

  private showAlert(type: string, message: string, sticky: boolean) {
    const alert = {
      closed: false,
      type: type,
      message: message,
      sticky: sticky
    };
    this.globalMessageService.pushAlert(alert);
  }

  portSubmitHandler() {
    this.loadingPort = true;
    let portsArr = this.ports.split(',').filter(n => n);
    let protocolsArr = this.protocols.split(',').filter(n => n);
    let uniquePort = Array.from(new Set(portsArr));
    const destinationPort = {
      ports: this.ports,
      protocols: this.protocols.toLowerCase(),
      microsegmentId: this.microsegmentId,
      destination: this.selectedDestination
    };
    if (uniquePort.length !== portsArr.length) {
      this.loadingPort = false;
      this.showAlert('danger', 'Duplicate Ports values not allowed', false);
    } else if (portsArr.length > 16) {
      this.loadingPort = false;
      this.showAlert('danger', 'Only 16 comma seperated values allowed', false);
    } else if (protocolsArr.length !== portsArr.length) {
      this.loadingPort = false;
      this.showAlert(
        'danger',
        `${portsArr.length} comma seperated values restricted in protocols`,
        false
      );
    } else {
      this.microsegmentationService
        .createDestinationPort(destinationPort)
        .subscribe(
          (data: any) => {
            this.loadingPort = false;
            this.ports = '';
            this.protocols = '';
            this.selectedDestination = '';
            if (data.status) {
              this.showAlert('success', data.message, false);
            } else {
              this.showAlert('danger', data.message, false);
            }
            this.getMicrosegments(this.workloadId);
          },
          error => {
            this.loadingPort = false;
          }
        );
    }
  }

  subscribeToSearch() {
    this.$searchMicrosegment
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((search: { searchWord: string; typing: boolean }) => {
          this.loading = true;
          if (search.typing) {
            this.pagination.currentPage = 1;
          }
          return this.microsegmentationService.searchMicrosegments(
            this.workloadId,
            search.searchWord,
            this.pagination.itemsPerPage,
            this.pagination.currentPage,
            this.sort,
            this.sortDir
          );
        })
      )
      .subscribe((microsegments: any) => {
        this.microsegments = microsegments;
        this.pagination.totalItems = microsegments.recordsTotal;
        this.loading = false;
      });
  }

  submitHandler() {
    this.loading = true;
    let portsArr = [];
    let protocolsArr = [];
    let uniquePort = [];
    if (!this.selectedDestination) {
      this.loading = false;
      this.showAlert('danger', 'Please select destination device', false);
    } else {
      portsArr = this.ports.split(',').filter(n => n);
      protocolsArr = this.protocols.split(',').filter(n => n);
      uniquePort = Array.from(new Set(portsArr));
    }
    if (uniquePort.length !== portsArr.length) {
      this.loadingPort = false;
      this.showAlert('danger', 'Duplicate Ports values not allowed', false);
    }
    if (this.ports == '') {
      this.loading = false;
      this.showAlert('danger', 'Please select port', false);
    }
    if (this.protocols == '') {
      this.loading = false;
      this.showAlert('danger', 'Please select protocols', false);
    } else if (portsArr.length > 16) {
      this.loading = false;
      this.showAlert('danger', 'Only 16 comma seperated values allowed', false);
    } else if (protocolsArr.length !== portsArr.length) {
      this.loading = false;
      this.showAlert(
        'danger',
        `${portsArr.length} comma seperated values restricted in protocols`,
        false
      );
    } else {
      const microsegmentObj = {
        workloadId: this.workloadId,
        selectedClientIds: this.selectedClientIds,
        deviceId: this.selectedDevice,
        ports: this.ports,
        protocols: this.protocols == '' ? '' : this.protocols.toLowerCase(),
        destination: this.selectedDestination
      };
      this.microsegmentationService
        .createMicrosegment(microsegmentObj)
        .subscribe(
          (res: any) => {
            this.clearFields();
            this.selectedDevice = null;
            this.destDevices = null;
            this.getData(this.workloadId);
            this.loading = false;
            this.showAlert('success', res.message, false);
            this.getMicrosegments(this.workloadId);
          },
          error => {
            this.loading = false;
          }
        );
    }
  }

  fetchMoreClients() {
    this.clientOffset += 1;
    this.getClients();
  }

  clientFocus() {
    this.currentClientSearch = '';
    this.clients = [];
    this.getClients();
  }

  private getMicrosegments(workloadId) {
    this.loading = true;
    this.microsegmentationService
      .getAllMicrosegments(workloadId, this.pagination, this.searchWord)
      .subscribe((microsegments: any) => {
        this.loading = false;
        const { data, name, recordsTotal } = microsegments;
        this.microsegments = microsegments || {};
        this.workloadDevices = data.map(microsegment => {
          return microsegment.DeviceInventories.map(device => {
            return device.device_name;
          });
        });
        this.pagination.totalItems = recordsTotal || 0;
      });
  }

  microsegmentsList() {
    this.router.navigate(['/microsegments']);
  }

  deleteHandler(id) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Are you sure you want to delete?`,
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        this.clearFields();
        this.microsegmentationService.deleteMicrosegment(id).subscribe(res => {
          this.alert = {
            closed: false,
            type: 'success',
            message: 'Microsegment has been deleted successfully.'
          };
          this.globalMessageService.pushAlert(this.alert);
          this.getMicrosegments(this.workloadId);
        });
      }
    });
  }

  editMicrosegment(workloadId, deviceId, microsegmentId) {
    // const search = {
    //   pagination: this.pagination,
    //   searchKeyword: this.searchWord
    // }
    // this.preserveSearch.saveSearch('microsegmentSaveSearch', search);
    // this.navigationToEdit = true;
    // this.title = 'Edit';
    this.router.navigate([
      '/microsegments',
      workloadId,
      deviceId,
      microsegmentId
    ]);
    // this.clearFields();
    // this.microsegmentationService.getWorkloadMicrosegments(microsegmentId)
    //   .subscribe(
    //     res => {
    //       if(res){
    //         var arr = [];
    //         arr.push(res.workloadMicrosegment);
    //         this.destinationDevices = arr;
    //         this.selectedDevice = res.workloadMicrosegment.id;
    //         this.devices = res.microsegmentDevices;
    //         this.destDevices = res.microsegmentDevices.map((device) => device.device_name || "") || [];
    //         this.selectedClientIds = res.microsegmentDevices.map((device) => device.id || "") || [];
    //         this.destinations = res.microsegmentDevices;
    //       }
    //     }
    //   )
    // this.selectedDevice = deviceId;
  }
  addMicrosegment(workloadId) {
    const search = {
      pagination: this.pagination,
      searchKeyword: this.searchWord
    };
    this.preserveSearch.saveSearch('microsegmentSaveSearch', search);
    this.navigationToEdit = true;
    this.title = 'Add';
    this.router.navigate(['/microsegments', workloadId]);
    this.clearFields();
    this.getData(workloadId);
  }

  ngOnDestroy(): void {
    if (!this.navigationToEdit) {
      this.preserveSearch.saveSearch('microsegmentSaveSearch', '');
    }
  }

  pageChange(event) {
    this.pagination.currentPage = event;
    this.getMicrosegments(this.workloadId);
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getMicrosegments(this.workloadId);
  }

  cancelHandler() {
    this.router.navigate(['/workloads']);
  }
}
