import { Component } from '@angular/core';
import { OnboardingReportsService } from './reporting.service';
import { Pagination } from '../models/pagination';
import { Title } from '@angular/platform-browser';
import { Subject, debounceTime, distinctUntilChanged, switchMap } from 'rxjs';

@Component({
  selector: 'app-onboarding-reports',
  templateUrl: './onboarding-reports.component.html',
  styleUrls: ['./onboarding-reports.component.css']
})
export class OnboardingReportsComponent {
  today: Date;
  ReportsOfAllOnboardedClients: any = [];
  ReportsOfAllNonOnboardedClients: any = [];

  onboardedDir: string = 'asc'
  nonOnboardedDir: string = 'asc'
  onboardedSort: string;
  nonOnboardedSort: string;
  onboardedType: number = 1;
  nonOnboardedType: number = 0;

  onboardedPagination: Pagination = new Pagination();
  nonOnboardedPagination: Pagination = new Pagination();
  onboardedLoading: boolean = true;
  nonOnboardedLoading: boolean = true;
  onboardingPageSize = [10, 25, 50, 100, 200];
  nonOnboardingPageSize = [10, 25, 50, 100, 200];

  $searchOnboarded = new Subject();
  $searchNonOnboarded = new Subject();
  
  searchOnboarded: string;
  searchNonOnboarded: string;

  constructor(
    private onboardingReportsService: OnboardingReportsService,
    private titleService: Title
  ) {
    this.today = new Date();
  }

  ngOnInit() {
    this.titleService.setTitle('Reports | Onboarding Reports');
    this.onboardedPagination.id = 'onboarded_clients_reports';
    this.nonOnboardedPagination.id = 'nonOnboarded_clients_reports';
    this.getOnboardedClientReports(
      this.onboardedPagination
    );
    this.getNonOnboardedClientReports(
      this.nonOnboardedPagination
    );
    this.subscibeToOnboardedSearch();
    this.subscibeToNonOnboardedSearch();
  }

  ngAfterViewInit() {}

  ngOnDestroy() {}

  getOnboardedClientReports(pagination: Pagination) {
    this.onboardedLoading = true;
    this.onboardingReportsService.getOnboardingClientReports(
      pagination,
      this.onboardedDir,
      this.onboardedSort,
      this.onboardedType,
      this.searchOnboarded
    ).subscribe({
      next: summary => {
        this.onboardedPagination.totalItems =
          summary.recordsTotal;
        this.ReportsOfAllOnboardedClients = summary;
        this.onboardedLoading = false;
      },
      error: err => {
        this.onboardedLoading = false;
      }
    });
  }

  getNonOnboardedClientReports(pagination: Pagination) {
    this.nonOnboardedLoading = true;
    this.onboardingReportsService.getOnboardingClientReports(
      pagination,
      this.nonOnboardedDir,
      this.nonOnboardedSort,
      this.nonOnboardedType,
      this.searchNonOnboarded
    ).subscribe({
      next: summary => {
        this.nonOnboardedPagination.totalItems =
          summary.recordsTotal;
        this.ReportsOfAllNonOnboardedClients = summary;
        this.nonOnboardedLoading = false;
      },
      error: err => {
        this.nonOnboardedLoading = false;
      }
    });
  }

  pageChangeSummaryOfAllOnboardedClients(page: any) {
    this.onboardedPagination.currentPage = page;
    this.getOnboardedClientReports(this.onboardedPagination);
  }
  
  sortOnboardedRecord(term: string) {
    this.onboardedDir = this.onboardedDir == 'asc' ? 'desc' : 'asc';
    this.onboardedSort = term;
    this.getOnboardedClientReports(this.onboardedPagination);
  }
  
  pageChangeSummaryOfAllNonOnboardedClients(page: any) {
    this.nonOnboardedPagination.currentPage = page;
    this.getNonOnboardedClientReports(this.nonOnboardedPagination);
  }
  
  sortNonOnboardedRecord(term: string) {
    this.nonOnboardedDir = this.nonOnboardedDir == 'asc' ? 'desc' : 'asc';
    this.nonOnboardedSort = term;
    this.getNonOnboardedClientReports(this.nonOnboardedPagination);
  }

  subscibeToOnboardedSearch() {
    this.$searchOnboarded
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((data: any) => {
          this.onboardedLoading = true;
          this.searchOnboarded = data;
          this.onboardedPagination.currentPage = 1;
          return this.onboardingReportsService.
          getOnboardingClientReports(
            this.onboardedPagination,
            this.onboardedDir,
            this.onboardedSort,
            this.onboardedType,
            this.searchOnboarded
          );
        })
      ).subscribe({
        next: (summary: any) => {
          this.onboardedPagination.totalItems =
          summary.recordsTotal;
        this.ReportsOfAllOnboardedClients = summary;
        this.onboardedLoading = false;
        },
        error: err => {
          this.onboardedLoading = false;
        }
    });
  }

  subscibeToNonOnboardedSearch() {
    this.$searchNonOnboarded
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((data: any) => {
          this.nonOnboardedLoading = true;
          this.searchNonOnboarded = data;
          this.nonOnboardedPagination.currentPage = 1;
          return this.onboardingReportsService.
          getOnboardingClientReports(
            this.nonOnboardedPagination,
            this.nonOnboardedDir,
            this.nonOnboardedSort,
            this.nonOnboardedType,
            this.searchNonOnboarded
          );
        })
      ).subscribe({
        next: (summary: any) => {
          this.nonOnboardedPagination.totalItems =
          summary.recordsTotal;
          this.ReportsOfAllNonOnboardedClients = summary;
          this.nonOnboardedLoading = false;
        },
        error: err => {
          this.nonOnboardedLoading = false;
        }
    });
  }

  clearFunOnboarded() {
    this.searchOnboarded = '';
    this.getOnboardedClientReports(this.onboardedPagination);
  }
  
  clearFunNonOnboarded() {
    this.searchNonOnboarded = '';
    this.getNonOnboardedClientReports(this.nonOnboardedPagination);
  }
}
