import { EventEmitter, Injectable } from '@angular/core';
import { ForgotService } from '../forgot-password/forgot.service';
import { AuthenticationService } from '../login/authentication.service';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { SettingsService } from '../settings/settings.service';
import { BehaviorSubject } from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class OtpAuthenticationService {
  loading: boolean = true;
  showQRCode = false;
  showSecurityQuestions = false;
  qrImageData = '';

  otpError = null;
  error = '';
  adminMfa = false;
  failedAttemptCount = 3;
  private otpStatusCheck: any = new BehaviorSubject<any>(false);
  otpStatusCheck$ = this.otpStatusCheck.asObservable();

  private otpStatusCheckEdit: any = new BehaviorSubject<any>(false);
  otpStatusCheckEdit$ = this.otpStatusCheckEdit.asObservable();

  private otpStatusCheckDel: any = new BehaviorSubject<any>(false);
  otpStatusCheckDel$ = this.otpStatusCheckDel.asObservable();

  otpErrorStatus = new EventEmitter();
  otpFailedAttempts = new EventEmitter();

  constructor(
    private authenticationService: AuthenticationService,
    private forgotService: ForgotService,
    private globalMessageService: GlobalMessageService,
    private settingsService: SettingsService
  ) {}

  getOtpStatusValue() {
    return this.otpStatusCheck.value;
  }

  setOtpStatusValue(value: any) {
    this.otpStatusCheck.next(value);
  }
  getOtpStatusValueEdit() {
    return this.otpStatusCheckEdit.value;
  }

  setOtpStatusValueEdit(value: any) {
    this.otpStatusCheckEdit.next(value);
  }
  getOtpStatusValueDel() {
    return this.otpStatusCheckDel.value;
  }

  setOtpStatusValueDel(value: any) {
    this.otpStatusCheckDel.next(value);
  }

  verifyOtp(email: string, otp: string, option: string) {
    console.log("OPT",option);
    
    if (option == 'delete') {
      this.authenticationService.validateLinTokenByModal(email, otp).subscribe(
        (data: any) => {
          this.globalMessageService.clearAlerts();

          if (data.is_otp_verified) {
            this.otpError = true;
            this.setOtpStatusValueDel(true);
            if (this.failedAttemptCount < 3) {
              this.failedAttemptCount = 3;
            }
          } else {
            this.otpError = false;
            this.failedAttemptCount -= 1;
            if (this.failedAttemptCount == 0) {
              this.failedAttemptCount = 3;
            }
            this.setOtpStatusValueDel(false);
            this.otpErrorStatus.emit(false);
            this.otpFailedAttempts.emit(this.failedAttemptCount);
          }
        },
        error => {
          this.globalMessageService.clearAlerts();
          this.otpError = false;
        }
      );
    } else if (option == 'edit') {
      this.authenticationService.validateLinTokenByModal(email, otp).subscribe(
        (data: any) => {
          this.globalMessageService.clearAlerts();

          if (data.is_otp_verified) {
            this.otpError = true;
            this.setOtpStatusValueEdit(true);
            if (this.failedAttemptCount < 3) {
              this.failedAttemptCount = 3;
            }
          } else {
            this.otpError = false;
            this.failedAttemptCount -= 1;
            if (this.failedAttemptCount == 0) {
              this.failedAttemptCount = 3;
            }
            this.setOtpStatusValueEdit(false);
            this.otpErrorStatus.emit(false);
            this.otpFailedAttempts.emit(this.failedAttemptCount);
          }
        },
        error => {
          this.globalMessageService.clearAlerts();
          this.otpError = false;
        }
      );
    }else{
      this.authenticationService.validateLinTokenByModal(email, otp).subscribe(
        (data: any) => {
          this.globalMessageService.clearAlerts();

          if (data.is_otp_verified) {
            this.otpError = true;
            this.setOtpStatusValue(true);
            if (this.failedAttemptCount < 3) {
              this.failedAttemptCount = 3;
            }
          } else {
            this.otpError = false;
            this.failedAttemptCount -= 1;
            if (this.failedAttemptCount == 0) {
              this.failedAttemptCount = 3;
            }
            this.setOtpStatusValue(false);
            this.otpErrorStatus.emit(false);
            this.otpFailedAttempts.emit(this.failedAttemptCount);
          }
        },
        error => {
          this.globalMessageService.clearAlerts();
          this.otpError = false;
        }
      );
    }
  }

  getMfa() {
    this.settingsService.getSettings('').subscribe((res: any) => {
      this.adminMfa = res.data[0].admin_mfa;
    });

    return this.adminMfa;
  }
}
