import {
  ChangeDetectorRef,
  Component,
  ElementRef,
  EventEmitter,
  HostListener,
  Input,
  Output,
  ViewChild
} from '@angular/core';
import { OtpAuthenticationService } from './otp-authentication.service';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { GlobalMessageClass } from '../global-messages/global-messages';

@Component({
  selector: 'app-otp-verification',
  templateUrl: './otp-verification.component.html',
  styleUrls: ['./otp-verification.component.css']
})
export class OtpVerificationComponent {
  digit1: string = '';
  digit2: string = '';
  digit3: string = '';
  digit4: string = '';
  digit5: string = '';
  digit6: string = '';

  @ViewChild('digit1Input') digit1Input: ElementRef;
  @ViewChild('digit2Input') digit2Input: ElementRef;
  @ViewChild('digit3Input') digit3Input: ElementRef;
  @ViewChild('digit4Input') digit4Input: ElementRef;
  @ViewChild('digit5Input') digit5Input: ElementRef;
  @ViewChild('digit6Input') digit6Input: ElementRef;

  otpPassword: string = '';
  isDarkMode: any;
  finalValue: string = '';
  isOtpVerified: boolean = true;
  failedAttemptsCount: number;
  @Output() cancelEvenEmitter = new EventEmitter();
  @Output() submitEvenEmitter = new EventEmitter();
  @Output() mfaEmitter = new EventEmitter();
  @Output() closePopup = new EventEmitter<boolean>();
  public alert: GlobalMessageClass;
  @Input() otpOptionValue: string;
  constructor(
    private OtpAuthenticationService: OtpAuthenticationService,
    private globalMessageService: GlobalMessageService,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.OtpAuthenticationService.otpErrorStatus.subscribe((res: any) => {
      this.isOtpVerified = res;
    });

    this.OtpAuthenticationService.otpFailedAttempts.subscribe((res: any) => {
      this.failedAttemptsCount = res;
    });
  }

  ngAfterViewInit(): void {}

  onBackspace(index: number, event: KeyboardEvent): void {
    console.log('event :>> ', event);
    const isBackspace = event.key === 'Backspace';

    if (event.keyCode === 32) {
      event.preventDefault();
    } else {
      if (isBackspace) {
        if (index > 0 && this[`digit${index}`] !== null) {
          const previousInput = this[`digit${index - 1}Input`];
          if (previousInput) {
            previousInput.nativeElement.focus();
            event.preventDefault();
          }
        }
      } else {
        if (this[`digit${index}`] !== null && index < 6) {
          const nextInput = this[`digit${index + 1}Input`];
          if (nextInput) {
            nextInput.nativeElement.focus();
          }
        }
      }
    }
  }

  getFinalValue(): string {
    this.finalValue = `${this.digit1 || ''}${this.digit2 || ''}${this.digit3 ||
      ''}${this.digit4 || ''}${this.digit5 || ''}${this.digit6 || ''}`;
    return this.finalValue;
  }

  cancelHandler() {
    this.cancelEvenEmitter.emit('cancel');
  }

  VerifyOtp() {
    let userObj: any = JSON.parse(localStorage.getItem('currentUser'));

    if (this.finalValue) {
      this.OtpAuthenticationService.verifyOtp(
        userObj.userName,
        this.finalValue,
        this.otpOptionValue
      );
    }

    if (this.failedAttemptsCount == 1) {
      this.alert = {
        closed: false,
        type: 'danger',
        message: 'Verification Failed.'
      };
      this.globalMessageService.pushAlert(this.alert);
      this.closePopup.emit(false);
    }
    this.submitEvenEmitter.emit(this.finalValue);
  }

  onPaste(event: ClipboardEvent, index: number): void {
    event.preventDefault();
    const clipboardData = event.clipboardData || (window as any).clipboardData;
    console.log('Clipboard', clipboardData);

    const pastedText = clipboardData.getData('text');
    console.log('Pasted', pastedText);

    // Ensure that the pasted text contains only digits
    const digits = pastedText.match(/\d/g);
    console.log('Digits', digits);

    if (digits) {
      // Assign the digits to the corresponding input fields
      this.digit1 = digits[0] ? String(parseInt(digits[0], 10)) : '';
      this.digit2 = digits[1] ? String(parseInt(digits[1], 10)) : '';
      this.digit3 = digits[2] ? String(parseInt(digits[2], 10)) : '';
      this.digit4 = digits[3] ? String(parseInt(digits[3], 10)) : '';
      this.digit5 = digits[4] ? String(parseInt(digits[4], 10)) : '';
      this.digit6 = digits[5] ? String(parseInt(digits[5], 10)) : '';
    }
  }
}
