import { Component, OnInit } from '@angular/core';
import {PolicyReportsService} from './reporting.service';
import { Pagination } from '../models/pagination';
import * as moment from 'moment';
import { APIDataTypeEnum } from '../shared/emuns/global.enum';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-policy-reports',
  templateUrl: './policy-reports.component.html',
  styleUrls: ['./policy-reports.component.css']
})
export class PolicyReportsComponent implements OnInit {
  today: Date;
  ReportsOfAllUnusedPolicy:any = [];
  ReportsOfAllWithoutRoutingPolicy:any = [];
  ReportsOfAllUnassignedServicesPolicy:any = [];
  servicesDaterangepicker: Date[] = [new Date(moment().subtract(1, 'month').toDate()), new Date()];
  serviceSortDir: string = 'asc'
  serviceSort: string;
  withoutroutingpolicySortDir: string = 'asc'
  withoutroutingpolicySort: string;
  UnassignedServicespolicySortDir: string = 'asc'
  UnassignedServicespolicySort: string;

  summaryOfAllServicesPagination: Pagination = new Pagination();
  summaryOfAllWithoutRoutingPolicyPagination: Pagination = new Pagination();
  summaryOfAllUnassignedServicesPolicyPagination: Pagination = new Pagination();
  reportsOfAllUnusedPolicyLoading:boolean = true;
  reportsOfAllWithoutRoutingPolicyLoading:boolean = true;
  reportsOfAllUnassignedServicesPolicyLoading:boolean = true;
  pageSizeSummaryOfServices = [10,25,50,100,200];
  pageSizeSummaryOfWithoutRoutingPolicy = [10,25,50,100,200];
  pageSizeSummaryOfUnassignedServicesPolicy = [10,25,50,100,200];

  constructor (
    private PolicyReportsService:PolicyReportsService,
    private titleService: Title
  ) {
    this.today = new Date();
  }

  ngOnInit() {
    this.titleService.setTitle('Reports | Policy Reports');
    this.summaryOfAllServicesPagination.id='policy_unused_reports';
    this.summaryOfAllWithoutRoutingPolicyPagination.id='policy_withoutroutingpolicy_reports';
    this.summaryOfAllUnassignedServicesPolicyPagination.id='policy_unassignedservices_reports';
    this.getReportsOfAllUnusedPolicy(this.summaryOfAllServicesPagination);
    this.getReportsOfAllWithoutRoutingPolicy(this.summaryOfAllWithoutRoutingPolicyPagination);
    this.getReportsOfAllUnassignedServicesPolicy(this.summaryOfAllUnassignedServicesPolicyPagination);
  }
  
  ngAfterViewInit() {
  }

  ngOnDestroy() {
  }

  getReportsOfAllUnusedPolicy(pagination: Pagination){
    this.reportsOfAllUnusedPolicyLoading = true;
    this.PolicyReportsService.
    getUnusedPolicyReports(
      pagination,
      this.servicesDaterangepicker[0],
      this.servicesDaterangepicker[1],
      this.serviceSortDir,
      this.serviceSort
    )
    .subscribe({
      next: summary => {
        this.summaryOfAllServicesPagination.totalItems = summary.recordsTotal;
        this.ReportsOfAllUnusedPolicy = summary;
        this.reportsOfAllUnusedPolicyLoading = false;
      },
      error: err=>{
        this.reportsOfAllUnusedPolicyLoading = false;
      }
    })
  }


  getReportsOfAllWithoutRoutingPolicy(pagination: Pagination){
    this.reportsOfAllWithoutRoutingPolicyLoading = true;
    this.PolicyReportsService.
    getWithoutRoutingPolicyReports(
      pagination,
      this.withoutroutingpolicySortDir,
      this.withoutroutingpolicySort
    )
    .subscribe({
      next: summary => {
        this.summaryOfAllWithoutRoutingPolicyPagination.totalItems = summary.recordsTotal;
        this.ReportsOfAllWithoutRoutingPolicy = summary;
        this.reportsOfAllWithoutRoutingPolicyLoading = false;
      },
      error: err=>{
        this.reportsOfAllWithoutRoutingPolicyLoading = false;
      }
    })
  }


  getReportsOfAllUnassignedServicesPolicy(pagination: Pagination){
    this.reportsOfAllUnassignedServicesPolicyLoading = true;
    this.PolicyReportsService.
    getUnassignedServicesPolicyReports(
      pagination,
      this.UnassignedServicespolicySortDir,
      this.UnassignedServicespolicySort
    )
    .subscribe({
      next: summary => {
        this.summaryOfAllUnassignedServicesPolicyPagination.totalItems = summary.recordsTotal;
        this.ReportsOfAllUnassignedServicesPolicy = summary;
        this.reportsOfAllUnassignedServicesPolicyLoading = false;
      },
      error: err=>{
        this.reportsOfAllUnassignedServicesPolicyLoading = false;
      }
    })
  }

  clearDate (type) {
    if(type === APIDataTypeEnum.Service) {
      this.servicesDaterangepicker = [];
      this.serviceDateChange();
    }
  }

  serviceDateChange(){
    if(!this.servicesDaterangepicker)
    this.servicesDaterangepicker = [];
    this.summaryOfAllServicesPagination.currentPage = 1;
    this.getReportsOfAllUnusedPolicy(this.summaryOfAllServicesPagination);
  }

  pageChangeSummaryOfAllServices(page:any) {
    this.summaryOfAllServicesPagination.currentPage = page;
    this.getReportsOfAllUnusedPolicy(this.summaryOfAllServicesPagination);
  }


  sortServiceRecord(term: string){
    this.serviceSortDir = this.serviceSortDir == 'asc' ? 'desc' : 'asc';
    this.serviceSort = term;
    this.getReportsOfAllUnusedPolicy(this.summaryOfAllServicesPagination)
  }



  pageChangeSummaryOfAllWithoutRoutingPolicy(page:any) {
    this.summaryOfAllWithoutRoutingPolicyPagination.currentPage = page;
    this.getReportsOfAllWithoutRoutingPolicy(this.summaryOfAllWithoutRoutingPolicyPagination);
  }

  pageChangeSummaryOfAllUnassignedServicesPolicy(page:any) {
    this.summaryOfAllUnassignedServicesPolicyPagination.currentPage = page;
    this.getReportsOfAllUnassignedServicesPolicy(this.summaryOfAllUnassignedServicesPolicyPagination);
  }


  sortWithoutRoutingPolicyRecord(term: string){
    this.withoutroutingpolicySortDir = this.withoutroutingpolicySortDir == 'asc' ? 'desc' : 'asc';
    this.withoutroutingpolicySort = term;
    this.getReportsOfAllWithoutRoutingPolicy(this.summaryOfAllWithoutRoutingPolicyPagination)
  }

  exportCSV(value){
    this.PolicyReportsService.exportCSV(value)
  }

}




