import { Component } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { User } from '../models/user';
import { UserService } from '../users/user.service';
import { RecoverService } from '../recover-password/recover.service';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { BsModalService, BsModalRef } from 'ngx-bootstrap/modal';
import { ModalComponent } from '../shared/modal/modal.component';
import * as sharedContent from '../shared/shared-content.constant';
import { Title } from '@angular/platform-browser';
import { AuthenticationService } from '../login/authentication.service';

@Component({
  selector: 'app-recover-password',
  templateUrl: './recover.component.html',
  styleUrls: ['./recover.component.css']
})
export class RecoverComponent {
  password1 = '';
  password2 = '';
  loading = false;
  error = '';
  success = '';
  token = '';
  queryParams: any;
  completed = false;
  modalRef: BsModalRef;
  showText: boolean = false;
  themeColor = false;
  constructor(
    private router: Router,
    private recoverService: RecoverService,
    private route: ActivatedRoute,
    private globalMesageservice: GlobalMessageService,
    private modalService: BsModalService,
    private titleService: Title,
    private readonly authenticationService: AuthenticationService
  ) {
    this.themeColor = JSON.parse(localStorage.getItem('darkMode'));
    this.titleService.setTitle('Reset Password');
    if (typeof this.route.params['_value']['token'] != 'undefined') {
      this.token = this.route.params['_value']['token'];
      this.globalMesageservice.clearAlerts();
    }
  }

  resetInstructions() {
    this.showText = false;
  }

  changePassword() {
    this.loading = true;
    this.showText = false;
    console.log('Token ', this.token);
    if (!this.token) {
      this.authenticationService
        .changePassword(this.password2)
        .pipe()
        .subscribe({
          next: (res: any) => {
            this.loading = false;
            if (!res?.success) {
              this.globalMesageservice.showNewAlert({
                type: 'danger',
                message: res?.message
              });
            } else {
              this.globalMesageservice.showNewAlert({
                type: 'success',
                message: res?.message
              });
              localStorage.setItem('is_first_login', 'false');
              localStorage.setItem('is_first_login', 'false');
              this.router.navigate(['/security-questions']);

              console.log('res ', res);
            }
          },
          error: err => (this.loading = false)
        });
    } else {
      this.recoverService
        .CP(this.password1, this.password2, this.token)
        .subscribe(result => {
          if (result === true) {
            this.success = 'Your password has been changed successfully!';
            this.completed = true;
            this.loading = false;
            this.error = '';
          } else {
            // login failed
            this.success = '';
            this.error = 'Invalid Request Please try again.';
            this.loading = false;
          }
          this.globalMesageservice.clearAlerts();
        });
    }
  }

  onClick() {
    this.showText = !this.showText;
  }
}
