import { Injectable } from '@angular/core';
import { Router } from '@angular/router';
import { ConfigService } from '../services/config.service';
import { Observable } from 'rxjs';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { map } from 'rxjs/operators';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { catchError } from 'rxjs/operators';
import { throwError } from 'rxjs';

@Injectable()
export class RecoverService {
  public alert: GlobalMessageClass;
  private authenticationUrl = '/authentication';
  public allow: string;
  private apiBaseUrl: string;
  httpOptions = {
    headers: new HttpHeaders({ 'Content-Type': 'application/json' })
  };

  constructor(
    private httpClient: HttpClient,
    private router: Router,
    private configService: ConfigService,
    private globalMessageService: GlobalMessageService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.authenticationUrl = this.apiBaseUrl + this.authenticationUrl;
    this.checkLogin();
  }

  checkLogin(): void {
    let currentUser = JSON.parse(localStorage.getItem('currentUser'));
    if (currentUser) {
      let token = currentUser.token;
      if (token) {
        this.alert = {
          closed: false,
          type: 'success',
          message:
            'User is already logged in. Logout from the current user to perform this action.'
        };
        this.globalMessageService.pushAlert(this.alert);
        this.router.navigate(['/']);
      }
    }
  }

  CP(password1: string, password2: string, token: string): Observable<boolean> {
    return this.httpClient
      .post(
        this.authenticationUrl + '/confirmPassword',
        JSON.stringify({ password: password1, token: token }),
        this.httpOptions
      )
      .pipe(
        map((response: any) => {
          // Email Sent Successfully
          let allow = response && response.success;
          if (allow) {
            this.allow = allow;
            // return true to indicate successful Email Sent
            return true;
          } else {
            // return false to indicate failed login
            return false;
          }
        })
      );
  }

  validate(token: string): Observable<any> {
    const httpOptions = {
      headers: new HttpHeaders({ 'Content-Type': 'application/json' })
    };

    return this.httpClient.post(
      this.authenticationUrl + '/confirmToken',
      JSON.stringify({ token: token }),
      httpOptions
    );
  }
}
