import { Injectable } from '@angular/core';
import { CanActivate, Router, ActivatedRouteSnapshot, RouterStateSnapshot } from '@angular/router';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { RolesAndPermissionService } from './roles-and-permission/roles-and-permission.service';
import { RolesData } from './shared/roles/roles';

@Injectable()
export class RoleGuard implements CanActivate {
  constructor(
    private router: Router,
    private rolesAndPermissionService: RolesAndPermissionService
  ) {}

  canActivate(route: ActivatedRouteSnapshot, state: RouterStateSnapshot): Observable<boolean> | Promise<boolean> | boolean {
    const requiredRole = route.data.roleUrl;

    return this.rolesAndPermissionService.getAssignedUserTemplates().pipe(
      map((data) => {
        const permissions = data as any[];

        const routeObject = RolesData.find(privilege => privilege.name === requiredRole);
        let privilegeExist = this.doesPrivilegeExist(permissions, routeObject);
        
        if (requiredRole === 'Dashboard') {
          privilegeExist = this.doesPrivilegeExist(permissions, { privilege_id: 1 }) || this.doesPrivilegeExist(permissions, { privilege_id: 62 });
        }
  
        if (!privilegeExist) {
          const firstMatch = RolesData.find(roleData => {
            return permissions.some(
              permission =>
                permission.privilege_id === roleData.privilege_id &&
                permission.is_crud_allowed === 1
            );
          });

          if (firstMatch) {
            const indexInRolesData = RolesData.indexOf(firstMatch);
            const route = RolesData[indexInRolesData].route_name;
            this.router.navigate(['/' + route]);

            return false;
          } else {
            return true;
          }
        }

        return true;
      })
    );
  }

  doesPrivilegeExist(permissions, privilegeToCheck) {
    return permissions.some(
      permission => permission.privilege_id === privilegeToCheck.privilege_id
    );
  }

  returnFirstPrivilege(userRoles) {
    return RolesData.find(
      privilege =>
        privilege.privilege_id === userRoles[0]?.privilege_id &&
        userRoles[0]?.is_crud_allowed
    );
  }
}
