import { Component, OnInit } from '@angular/core';
import { RoutingPolicyService } from './../routing-policy.service';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { Router, ActivatedRoute } from '@angular/router';
import { Subject } from 'rxjs';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { APIDataTypeEnum } from '../../shared/emuns/global.enum';
import { DeviceOSEnum } from '../../shared/emuns/client.enum';
import { ConfirmModalComponent } from '../../../../src/app/shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-routing-policy',
  templateUrl: './create-routing-policy.component.html',
  styleUrls: ['./create-routing-policy.component.css']
})
export class CreateRoutingPolicyComponent implements OnInit {
  policyName;
  DataType = APIDataTypeEnum;
  
  gateway = {
    name: '',
    buildOs: ''
  };

  windowsOsVersion = [
    { value: '6.1.7601', display: 'Windows 7' },
    { value: '6.2.9200', display: 'Windows 8' },
    { value: '6.3.9600', display: 'Windows 8.1' },
    { value: '10.0.18363', display: 'Windows 10' },
    { value: '10.0.22000', display: 'Windows 11' }
  ];

  macOsVersions = [
    { value: '10.15.4', display: 'macOS Catalina' },
    { value: '10.14.6', display: 'macOS Mojave' },
    { value: '10.13.6', display: 'macOS High Sierra' }
  ];

  ubuntu18OsVersions = [
    { value: '18.04.4', display: '' },
    { value: '18.04.3', display: '' },
    { value: '18.04.2', display: '' },
    { value: '18.04.1', display: '' },
    { value: '18.04', display: '' }
  ];

  ubuntu16OsVersions = [
    { value: '16.04.6', display: '' },
    { value: '16.04.5', display: '' },
    { value: '16.04.4', display: '' },
    { value: '16.04.3', display: '' },
    { value: '16.04.2', display: '' },
    { value: '16.04.1', display: '' },
    { value: '16.04', display: '' }
  ];

  centosserv7OsVersions = [
    { value: '7.7', display: '' },
    { value: '7.6', display: '' },
    { value: '7.5', display: '' },
    { value: '7.4', display: '' },
    { value: '7.3', display: '' },
    { value: '7.2', display: '' },
    { value: '7.1', display: '' },
    { value: '7.0', display: '' }
  ];

  centosserv8OsVersions = [
    { value: '8.3', display: '' },
    { value: '8.2', display: '' },
    { value: '8.1', display: '' },
    { value: '8.0', display: '' }
  ];

  clientAllOs = [
    { value: DeviceOSEnum.Ubuntu, display: 'Ubuntu Desktop' },
    { value: DeviceOSEnum.UbuntuServer, display: 'Ubuntu Server' },
    { value: DeviceOSEnum.Ubuntu18, display: 'Ubuntu 18 Desktop' },
    { value: DeviceOSEnum.UbuntuServer18, display: 'Ubuntu 18 Server' },
    { value: DeviceOSEnum.Windows, display: 'Windows' },
    // { value: DeviceOSEnum.WindowsServer, display: 'Windows Server' },
    { value: DeviceOSEnum.Macintosh, display: 'macOS' },
    { value: DeviceOSEnum.CentosServer, display: 'CentOS Server' }
  ];

  deviceOs = [];

  gatewayOsVersion;
  title;
  isEditing = false;
  editingId = '';

  gateways = [];
  services = [];
  selectedGatewayIds = [];
  selectedServiceIds = [];
  selectedGatewayTypes = [];
  selectedServiceTypes = [];
  selectedGatewayCount = 0;
  selectedServiceCount = 0;
  gatewayOffset = 0;
  serviceOffset = 0;
  currentGatewaySearch = '';
  currentServiceSearch = '';
  $gatewaySearch = new Subject();
  $serviceSearch = new Subject();
  isGatewaySearching = false;
  isServiceSearching = false;
  currentEditingId;
  gatewaysPlaceHolder = 'Search Gateway or Gateway Group';
  servicesPlaceHolder = 'Search Service or Service Group';
  allGatewaysSelected = false;
  allServicesSelected = false;
  loading: Boolean = false;
  disableSaveButton = false;

  _gateway = [];
  gatewayGroup = [];
  service = [];
  serviceGroup = [];
  isDarkMode : boolean;
  
  constructor(
    private routingService: RoutingPolicyService,
    public router: Router,
    private globalMessageService: GlobalMessageService,
    private activatedRoute: ActivatedRoute,
    public dialog: MatDialog,
    private titleService: Title
  ) {}

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.disableSaveButton = false;
    this.activatedRoute.params.subscribe(data => {
      if (!data.id) {
        //create
        this.title = 'Add';
        this.getAllDropDownsData();
      } else {
        //edit
        this.title = 'Edit';
        this.isEditing = true;
        this.editingId = data.id;
        this.getDetails();
      }
      this.titleService.setTitle('Policy Management | Routing Policy | ' + this.title + ' Routing Policy ');
    });

    this.getAllOs();
    this.subscribeGatewaySearch();
    this.subscribeServiceSearch();
  }

  private getAllOs() {
    this.deviceOs = this.clientAllOs;
  }

  getDetails() {
    this.loading = true;
    this.routingService.getSinglePolicy(this.editingId).subscribe(
      (data: any) => {
        this.loading = false;
        const { os, name } = data;
        this.gateway = {
          name: name,
          buildOs: os
        };
        this.policyName = name;
        this.deviceOsSelected(false);
        this.parseAllEditData(data);
      },
      err => {
        this.loading = false;
      }
    );
  }

  deviceOsSelected(fromTemplate = true): void {
    const { buildOs } = this.gateway;
    if (buildOs === DeviceOSEnum.Windows) {
      this.gatewayOsVersion = this.windowsOsVersion;
    } else if (buildOs === DeviceOSEnum.Macintosh) {
      this.gatewayOsVersion = this.macOsVersions;
    } else if (buildOs === DeviceOSEnum.CentosServer) {
      this.gatewayOsVersion = this.centosserv8OsVersions;
    } else {
      if (buildOs === DeviceOSEnum.Ubuntu || buildOs === DeviceOSEnum.UbuntuServer) {
        this.gatewayOsVersion = this.ubuntu16OsVersions;
      } else {
        this.gatewayOsVersion = this.ubuntu18OsVersions;
      }
    }
    if (fromTemplate) {
      this.gatewayOsVersion = 'Any';
    }
  }

  checkProxyMixing() {
    const serviceIds = this.selectedServiceIds || [];
    const gatewayIds = this.selectedGatewayIds || [];
    const anyProxyService = this.services.some(service => {
      return serviceIds.includes(service.id) && service.proxy == 1;
    });
    if (anyProxyService) {
      const anyNonProxyGateway = this.gateways.some(gateway => {
        return gatewayIds.includes(gateway.id) && gateway.proxy == 0;
      });
      if (anyNonProxyGateway) {
        const alert = {
          closed: false,
          type: 'danger',
          message: 'Proxied services cannot be assigned to non-proxied gateways'
        };
        this.globalMessageService.pushAlert(alert);
        this.loading = false;
        return true;
      }
    }

    return false;
  }

  submitHandler() {
    this.loading = true;
    this.disableSaveButton = true;
    if (this.checkProxyMixing()) return true;

    this.parseAllSelectedGateways();
    const policy = {
      name: this.policyName,
      gateway: this._gateway,
      gatewayGroup: this.gatewayGroup,
      service: this.service,
      serviceGroup: this.serviceGroup,
      all_gateways: this.allGatewaysSelected,
      all_services: this.allServicesSelected
    };
    if (!this.isEditing) {
      this.routingService.createService(policy).subscribe(
        data => {
          this.loading = false;
          this.showAlert('success', 'Routing Policy Created', false);
          this.router.navigate(['/routing-policy']);
        },
        error => {
          this.disableSaveButton = false;
          this.loading = false;
          // this.showAlert('danger', JSON.parse(error._body).message, false);
        }
      );
    } else {
      this.routingService.updateService(policy, this.editingId).subscribe(
        data => {
          this.loading = false;
          this.showAlert('success', 'Routing Policy Updated', false);
          this.router.navigate(['/routing-policy']);
        },
        error => {
          this.disableSaveButton = false;
          this.loading = false;
          // this.showAlert('danger', JSON.parse(error._body).message, false);
        }
      );
    }
  }

  private showAlert(type: string, message: string, sticky: boolean) {
    const alert = {
      closed: false,
      type: type,
      message: message,
      sticky: sticky
    };
    this.globalMessageService.pushAlert(alert);
  }

  gatewayChange() {
    if (this.selectedGatewayIds.length >= this.selectedGatewayCount) {
      this.selectedGatewayCount = this.selectedGatewayCount + 1;
      const lastId = this.selectedGatewayIds[
        this.selectedGatewayIds.length - 1
      ];
      const gateway = this.gateways.find(item => item.id === lastId);
      this.selectedGatewayTypes = [...this.selectedGatewayTypes, gateway];
    } else {
      this.selectedGatewayCount = this.selectedGatewayIds.length;
      this.selectedGatewayTypes = this.selectedGatewayTypes.filter(gateway => {
        return this.selectedGatewayIds.indexOf(gateway.id) !== -1;
      });
    }
  }

  serviceChange() {
    if (this.selectedServiceIds.length >= this.selectedServiceCount) {
      this.selectedServiceCount = this.selectedServiceCount + 1;
      const lastId = this.selectedServiceIds[
        this.selectedServiceIds.length - 1
      ];
      const service = this.services.find(item => item.id === lastId);
      this.selectedServiceTypes = [...this.selectedServiceTypes, service];
    } else {
      this.selectedServiceCount = this.selectedServiceIds.length;
      this.selectedServiceTypes = this.selectedServiceTypes.filter(service => {
        return this.selectedServiceIds.indexOf(service.id) !== -1;
      });
    }
  }

  gatewayFocus() {
    this.gatewayOffset = 0;
    if (this.currentGatewaySearch) {
      this.currentGatewaySearch = '';
      this.$gatewaySearch.next('');
    } else {
      this.currentGatewaySearch = '';
    }
  }

  serviceFocus() {
    this.serviceOffset = 0;
    if (this.currentServiceSearch) {
      this.currentServiceSearch = '';
      this.$serviceSearch.next('');
    } else {
      this.currentServiceSearch = '';
    }
  }

  fetchMoreServices() {
    this.serviceOffset += 1;
    this.routingService
      .getAllDropDownData(
        this.gatewayOffset,
        this.serviceOffset,
        this.currentGatewaySearch,
        this.currentServiceSearch,
        this.currentEditingId
      )
      .subscribe((data: any) => {
        const serviceData = data.filter(
          d =>
            (d.type === APIDataTypeEnum.Service || d.type === APIDataTypeEnum.ServiceGroup) &&
            !this.selectedServiceIds.includes(d.id)
        );
        serviceData.forEach(s => (this.services = [...this.services, s]));
      });
  }
  fetchMoreGateways() {
    this.gatewayOffset += 1;
    this.routingService
      .getAllDropDownData(
        this.gatewayOffset,
        this.serviceOffset,
        this.currentGatewaySearch,
        this.currentServiceSearch,
        this.currentEditingId
      )
      .subscribe((data: any) => {
        const gatewayData = data.filter(
          d =>
            (d.type === APIDataTypeEnum.Gateway || d.type === APIDataTypeEnum.GatewayGroup) &&
            !this.selectedGatewayIds.includes(d.id)
        );
        gatewayData.forEach(c => (this.gateways = [...this.gateways, c]));
      });
  }

  selectAllGateways() {
    this.selectedGatewayIds = null;
    this.selectedGatewayCount = 0;
    this.allGatewaysSelected = !this.allGatewaysSelected;
    this.gatewaysPlaceHolder = !this.allGatewaysSelected
      ? 'Search Gateway or Gateway Group'
      : 'All Gateways are selected';
  }

  selectAllServices() {
    this.selectedServiceIds = null;
    this.selectedServiceCount = 0;
    this.allServicesSelected = !this.allServicesSelected;
    this.servicesPlaceHolder = !this.allServicesSelected
      ? 'Search Service or Service Group'
      : 'All services are selected';
  }

  selectAllServicesClick(selectall) {
    if (selectall) {
      if (this.selectedServiceCount > 0) {
        const dialogRef = this.dialog.open(ConfirmModalComponent, {
          panelClass: this.isDarkMode ? 'dark-theme' : '',
          data: {
            title: 'Confirmation',
            message: 'Are you sure you want to select all Services?',
          },
        });
    
        dialogRef.afterClosed().subscribe((result) => {
          if (result){
            this.selectAllServices();
          }
        });
      } else {
        this.selectAllServices();
      }
    } else {
      this.selectAllServices();
    }
  }

  selectAllGatewaysClick(selectall) {
    if (selectall) {
      if (this.selectedGatewayCount > 0) {
        const dialogRef = this.dialog.open(ConfirmModalComponent, {
          panelClass: this.isDarkMode ? 'dark-theme' : '',
          data: {
            title: 'Confirmation',
            message: 'Are you sure you want to select all Gateways?',
          },
        });
    
        dialogRef.afterClosed().subscribe((result) => {
          if (result){
            this.selectAllGateways();
          }
        });
      } else {
        this.selectAllGateways();
      }
    } else {
      this.selectAllGateways();
    }
  }

  private subscribeGatewaySearch() {
    this.$gatewaySearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.isGatewaySearching = true;
          this.currentGatewaySearch = term;
          this.gatewayOffset = 0;
          return this.routingService.getAllDropDownData(
            this.gatewayOffset,
            this.serviceOffset,
            this.currentGatewaySearch,
            this.currentServiceSearch,
            this.currentEditingId
          );
        })
      )
      .subscribe((data: any) => {
        this.isGatewaySearching = false;
        this.gateways = data.filter(
          d => d.type === APIDataTypeEnum.Gateway || d.type === APIDataTypeEnum.GatewayGroup
        );
      });
  }

  private subscribeServiceSearch() {
    this.$serviceSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.serviceOffset = 0;
          this.isServiceSearching = true;
          this.currentServiceSearch = term;
          return this.routingService.getAllDropDownData(
            this.gatewayOffset,
            this.serviceOffset,
            this.currentGatewaySearch,
            this.currentServiceSearch,
            this.currentEditingId
          );
        })
      )
      .subscribe((data: any) => {
        this.isServiceSearching = false;
        this.services = data.filter(
          d => d.type === APIDataTypeEnum.Service || d.type === APIDataTypeEnum.ServiceGroup
        );
      });
  }

  private parseAllEditData(data) {
    if (data.all_gateways) {
      this.selectAllGateways();
    }
    if (data.all_services) {
      this.selectAllServices();
    }
    this.selectedGatewayIds = [
      ...data.Gateways.map(_gateway => _gateway.id),
      ...data.GatewayGroups.map(g => g.id)
    ];
    this.selectedGatewayTypes = [
      ...data.Gateways.map(({ id, Name, device_os, proxy_enabled }) => {
        return {
          id,
          name: Name,
          type: 'gateway',
          device_os,
          proxy: proxy_enabled
        };
      }),
      ...data.GatewayGroups.map(({ id, name }) => {
        return {
          id,
          name,
          type: 'gatewayGroup'
        };
      })
    ];

    this.selectedServiceIds = [
      ...data.Services.map(_service => _service.id),
      ...data.ServiceGroups.map(g => g.id)
    ];
    this.selectedServiceTypes = [
      ...data.Services.map(({ id, Name, proxy_enabled }) => {
        return {
          id,
          name: Name,
          type: 'service',
          proxy: proxy_enabled
        };
      }),
      ...data.ServiceGroups.map(({ id, name }) => {
        return {
          id,
          name,
          type: 'serviceGroup'
        };
      })
    ];
    this.selectedGatewayCount = this.selectedGatewayIds.length;
    this.selectedServiceCount = this.selectedServiceIds.length;

    this.gateways = [...this.gateways, ...this.selectedGatewayTypes];
    this.services = [...this.services, ...this.selectedServiceTypes];

    this.getAllDropDownsData();
  }

  private parseAllSelectedGateways() {
    if (!this.allGatewaysSelected) {
      this._gateway = this.selectedGatewayTypes
        .filter(gateway => {
          return gateway.type === APIDataTypeEnum.Gateway;
        })
        .map(c => c.id);
      this.gatewayGroup = this.selectedGatewayTypes
        .filter(gateway => {
          return gateway.type === APIDataTypeEnum.GatewayGroup;
        })
        .map(cg => cg.id);
    }
    if (!this.allServicesSelected) {
      this.service = this.selectedServiceTypes
        .filter(s => s.type === APIDataTypeEnum.Service)
        .map(s => s.id);
      this.serviceGroup = this.selectedServiceTypes
        .filter(s => s.type === APIDataTypeEnum.ServiceGroup)
        .map(s => s.id);
    }
  }

  getAllDropDownsData() {
    this.routingService
      .getAllDropDownData(
        this.gatewayOffset,
        this.serviceOffset,
        this.currentGatewaySearch,
        this.currentServiceSearch,
        this.currentEditingId
      )
      .subscribe((data: any) => {
        const gatewayData = data.filter(
          d =>
            (d.type === APIDataTypeEnum.Gateway || d.type === APIDataTypeEnum.GatewayGroup) &&
            !this.selectedGatewayIds.includes(d.id)
        );
        gatewayData.forEach(c => (this.gateways = [...this.gateways, c]));
        const serviceData = data.filter(
          d =>
            (d.type === APIDataTypeEnum.Service || d.type === APIDataTypeEnum.ServiceGroup) &&
            !this.selectedServiceIds.includes(d.id)
        );
        serviceData.forEach(s => (this.services = [...this.services, s]));
      });
  }

  clearService(service) {
    this.selectedServiceIds = this.selectedServiceIds.filter(
      serviceId => serviceId !== service.id
    );
    this.selectedServiceCount = this.selectedServiceIds.length;
    this.selectedServiceTypes = this.selectedServiceTypes.filter(_service => {
      return this.selectedServiceIds.indexOf(_service.id) !== -1;
    });
  }

  clearGateway(gateway) {
    this.selectedGatewayIds = this.selectedGatewayIds.filter(
      gatewayId => gatewayId !== gateway.id
    );
    this.selectedGatewayCount = this.selectedGatewayIds.length;
    this.selectedGatewayTypes = this.selectedGatewayTypes.filter(_gateway => {
      return this.selectedGatewayIds.indexOf(_gateway.id) !== -1;
    });
  }
}
