import { Injectable } from '@angular/core';
import { ConfigService } from '../services/config.service';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { Pagination } from '../models/pagination';
import { map } from 'rxjs/operators';

@Injectable()
export class RoutingPolicyService {
  private policyApiUrl: string;
  constructor(
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    const baseUrl = configService.getConfiguration().apiBaseUrl;
    this.policyApiUrl = baseUrl + '/policies/routing';
  }

  getAllPolicies(
    pagination: Pagination,
    name: String = '',
    selectedGatewayIds = []
  ) {
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;

    let _name: String = '';

    if (name.trim()) {
      _name = name.trim();
    }
    let _gateways = selectedGatewayIds.join(',');
    return this.authHttp
      .get(
        this.policyApiUrl +
          `?offset=${currentPage}&limit=${itemsPerPage}&searchValue=${_name}&clients=${_gateways}`
      )
      .pipe(
        map(res => res),
        map((res: any) => {
          return {
            ...res,
            data: (res.data || []).map(v => ({
              ...v,
              gatewayName: v.all_gateways
                ? 'All Gateways'
                : v.Gateways[0] && v.Gateways[0].Name
                ? v.Gateways[0].Name +
                  `${
                    v.Gateways.length + (v.GatewayGroups || []).length > 1
                      ? '...'
                      : ''
                  }`
                : v.GatewayGroups[0] && v.GatewayGroups[0].name
                ? v.GatewayGroups[0].name +
                  `${v.GatewayGroups.length > 1 ? '...' : ''}`
                : 'Unknown',
              serviceName: v.all_services
                ? 'All Services'
                : v.Services[0] && v.Services[0].Name
                ? v.Services[0].Name +
                  `${
                    v.Services.length + (v.ServiceGroups || []).length > 1
                      ? '...'
                      : ''
                  }`
                : v.ServiceGroups[0] && v.ServiceGroups[0].name
                ? v.ServiceGroups[0].name +
                  `${v.ServiceGroups.length > 1 ? '...' : ''}`
                : 'Unknown'
            }))
          };
        })
      );
  }

  deletePolicy(id) {
    return this.authHttp
      .delete(this.policyApiUrl + `/${id}`)
      .pipe(map(res => res));
  }

  enablePolicy(id, value) {
    return this.authHttp
      .patch(this.policyApiUrl + `/${id}/enable/${value}`, {})
      .pipe(map(res => res));
  }

  getSinglePolicy(id) {
    return this.authHttp
      .get(`${this.policyApiUrl}/${id}`)
      .pipe(map(res => res));
  }

  createService(policyOption) {
    return this.authHttp
      .post(`${this.policyApiUrl}/create`, policyOption)
      .pipe(map(response => response));
  }

  updateService(policyOption, id) {
    return this.authHttp.put(`${this.policyApiUrl}/${id}`, policyOption);
  }

  getGateways(gatewayOffset, searchTerm) {
    return this.authHttp
      .get(
        `${this.policyApiUrl}/gateways?offset=${gatewayOffset}&name=${searchTerm}`
      )
      .pipe(map(res => res));
  }

  getAllDropDownData(
    gatewayOffset,
    serviceOffset,
    gatewayName,
    serviceName,
    policyId
  ) {
    let policyQuery = `&policyId=`;
    if (policyId) {
      policyQuery += policyId;
    }
    return this.authHttp
      .get(
        `${this.policyApiUrl}/filters?gatewayOffset=${gatewayOffset}&serviceOffset=${serviceOffset}&gatewayName=${gatewayName}&serviceName=${serviceName}${policyQuery}`
      )
      .pipe(map(res => res));
  }
}
