import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp, HttpHeaders } from '@angular/common/http';
import { map } from 'rxjs/operators';
import { Pagination } from '../models/pagination';
import { ConfigService } from '../services/config.service';

@Injectable()
export class SaasBridgeService {
  private bridgeUrl: string;
  httpOptions = {
    headers: new HttpHeaders({
      'Content-Type': 'application/json'
    })
  };
  constructor(
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    const baseUrl = configService.getConfiguration().apiBaseUrl;
    this.bridgeUrl = baseUrl + '/saasBridge';
  }

  getBridges(pagination: Pagination, sort, dir, search) {
    const queryString = `?limit=${
      pagination.itemsPerPage
    }&offset=${pagination.currentPage -
      1}&sort=${sort}&dir=${dir}&search=${search}`;
    return this.authHttp
      .get(`${this.bridgeUrl}/all${queryString}`)
      .pipe(map(response => response));
  }

  getBridge(id: Number) {
    return this.authHttp
      .get(`${this.bridgeUrl}/${id}`)
      .pipe(map(response => response));
  }

  addBridge(bridge) {
    return this.authHttp
      .post(`${this.bridgeUrl}`, bridge)
      .pipe(map(response => response));
  }

  updateBridge(bridge, bridgeId) {
    return this.authHttp
      .put(`${this.bridgeUrl}/${bridgeId}`, bridge)
      .pipe(map(response => response));
  }

  deleteBridge(id, verificationPassword) {
    this.httpOptions['body'] = { verificationPassword };
    return this.authHttp
      .delete(this.bridgeUrl + `/${id}`, this.httpOptions)
      .pipe(map(res => res));
  }

  getClients(search: string = '', offset: number = 0, limit: number = 10) {
    return this.authHttp
      .get(
        `${this.bridgeUrl}/clients?search=${search}&offset=${offset}&limit=${limit}`
      )
      .pipe(map(response => response));
  }

  getClientService(cleintId: Number) {
    return this.authHttp
      .get(`${this.bridgeUrl}/services/${cleintId}`)
      .pipe(map(response => response));
  }
}
