import { Component, OnInit } from '@angular/core';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { UserService } from '../users/user.service';
import { Router } from '@angular/router';
import { Popup } from '../Popup';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { User } from '../models/user';

@Component({
  selector: 'app-security-questions',
  templateUrl: './security-questions.component.html',
  styleUrls: ['./security-questions.component.css']
})
export class SecrityQuestionsComponent implements OnInit {
  
  questions: any = [{ value: 1, display: '' }];
  user = new User();
  isDarkMode: any;
  logo: any;

  constructor(
    private globalMessageService: GlobalMessageService,
    private userService: UserService,
    private router: Router,
    public dialog: MatDialog,
    private titleService: Title
  ) {}

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('modeDark'));
    this.logo = this.isDarkMode
      ? '/assets/images/logo-dark.png'
      : '/assets/images/invisily-logo.png';
    this.isDarkMode = this.isDarkMode ? 'dark-theme' : '';
    this.titleService.setTitle('Security Question');
    this.userService.getSecurityQuestions().subscribe(questions => {
      this.questions = questions.map(question => {
        return {
          value: question.id,
          display: question.question
        };
      });
    });
  }

  handleSave() {
    let user = localStorage.getItem('usd')
    if (typeof user != 'undefined' || user != null) {
      user = atob(user);
    }
    this.userService.updateSecurityQuestions(user, this.user).subscribe((res: any) => {
      if(res.state == 0){
        this.user.answer_1 = this.user.answer_2 = this.user.answer_3 = ''
        this.showAlert('danger', 'Please enter different answers', false)
      }else {
        this.showAlert('success', 'Security questions updated succesfully', false)
        localStorage.setItem('is_question_changed', 'true');
        this.router.navigate(['/'])
      }
    })
  }

  questionChange(e) {}

  private showAlert(type: string, message: string, sticky: boolean) {
    const alert = {
      closed: false,
      type: type,
      message: message,
      sticky: sticky
    };
    this.globalMessageService.pushAlert(alert);
  }
}
