import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { ConfigService } from '../services/config.service';
import * as moment from 'moment';
import { AuthShortlivedCookieService } from '../auth-shortlived-cookie.service';
import { Pagination } from '../models/pagination';
import { map } from 'rxjs/operators';

interface date {
  year: String;
  month: String;
  day: string;
}
@Injectable()
export class ServicesReportsService {
  private apiBaseUrl: string;
  private reportingApiUrl: string = '/reports';

  constructor(
    private authHttp: AuthHttp,
    private congigService: ConfigService,
    private authShortlivedCookie: AuthShortlivedCookieService
  ) {
    this.apiBaseUrl = this.congigService.getConfiguration().apiBaseUrl;
    this.reportingApiUrl = this.apiBaseUrl + this.reportingApiUrl;
  }

  getServicesReports(
    pagination: Pagination,
    startDate: Date,
    endDate: Date,
    dir: string,
    sort: string
  ): any {
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;
    let _startDate = '';
    let _endDate = '';

    if (startDate) {
      _startDate = moment(startDate).format('YYYY-MM-DD');
    }
    if (endDate) {
      _endDate = moment(endDate).format('YYYY-MM-DD');
    }

    return this.authHttp
      .get(
        this.reportingApiUrl +
          '/services/reports?offset=' +
          currentPage +
          '&limit=' +
          itemsPerPage +
          '&startDate=' +
          _startDate +
          '&endDate=' +
          _endDate +
          '&dir=' +
          dir +
          '&sort=' +
          sort
      )
      .pipe(map((response: Response) => response));
  }

  exportCSV() {
    this.authShortlivedCookie.setFor(5);
    window.location.href = `${this.reportingApiUrl}/services/export-csv`;
  }

  getServiceToUserReports(
    pagination: Pagination, 
    searchTerm='', 
    services=null, 
    serviceType='',
    dateFilter=[]
    ): any{
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;
    let _services='';
    let _from:any='';
    let _to:any='';
    if (services && services.length) {
      _services = JSON.stringify(services.map(v => v.service_id));
    }
    if(dateFilter&&dateFilter.length){

      _from=Math.floor(new Date(dateFilter[0]).getTime() / 1000);
      _to=Math.floor(new Date(dateFilter[1]).getTime() / 1000);

    }
    return this.authHttp
    .get(
      this.reportingApiUrl +
        '/services-to-user/reports?offset=' +
        currentPage +
        '&limit=' +
        itemsPerPage+
        '&searchTerm='+
        searchTerm+
        '&serviceId='+
        _services+
        '&serviceType='+
        serviceType+
        '&from='+
        _from+
        '&to='+
        _to
    )
    .pipe(map((response: Response) => response));
  }
}
