import { Component, OnInit } from '@angular/core';
import {ServicesReportsService} from './reporting.service';
import { Pagination } from '../models/pagination';
import * as moment from 'moment';
import { APIDataTypeEnum } from '../shared/emuns/global.enum';
import { Title } from '@angular/platform-browser';
import { ReportingService } from '../reportings/reporting.service';
import { Subject, debounceTime, distinctUntilChanged, switchMap } from 'rxjs';

@Component({
  selector: 'app-services-reports',
  templateUrl: './services-reports.component.html',
  styleUrls: ['./services-reports.component.css']
})
export class ServicesReportsComponent implements OnInit {
  today: Date;
  ReportsOfAllServices:any = [];
  servicesToUsers:any = [];
  topFiveServices:any = [];
  services:any = [];
  selectedServices:any = [];
  serviceDetails: any = [];
  servicesDaterangepicker: Date[] = [new Date(moment().subtract(1, 'month').toDate()), new Date()];
  serviceToUserDaterangepicker: Date[] = [new Date(moment().subtract(1, 'month').toDate()), new Date()];
  serviceType: any = [{Name:'HTTPS',service_type:'HTTPS'},{Name:'HTTP',service_type:'HTTP'}]
  serviceSortDir: string = 'asc'
  serviceSort: string;
  searchService: string = '';
  selectedServiceType: string = '';
  toggleRuleViewStyle: string = 'none';
  

  summaryOfAllServicesPagination: Pagination = new Pagination();
  servicesToUsersPagination: Pagination = new Pagination();
  topFiveServicesPagination: Pagination = new Pagination();
  serviceDetailsPagination: Pagination = new Pagination();
  reportsOfAllServicesLoading:Boolean = true;
  servicesToUserLoading:Boolean = true;

  $searchService = new Subject();

  pageSizeSummaryOfServices = [10,25,50,100,200];
  



  constructor (
    private ServicesReportsService:ServicesReportsService,
    private titleService: Title,
    private reportingService:ReportingService,
  ) {
    this.today = new Date();
  }

  ngOnInit() {
    this.titleService.setTitle('Reports | Services Reports');
    this.summaryOfAllServicesPagination.id='services_reports';
    this.servicesToUsersPagination.id = 'service_to_user';
    this.getReportsOfAllServices(this.summaryOfAllServicesPagination);
    this.getServicesToUserReport(this.servicesToUsersPagination);
    this.subscribeToSearchServices();
    this.getAllServices()
  }
  
  ngAfterViewInit() {
  }

  ngOnDestroy() {
  }

  subscribeToSearchServices(){
    this.$searchService
    .pipe(
      debounceTime(500),
      distinctUntilChanged(),
      switchMap((data: any) => {
        this.servicesToUserLoading = true;
        this.searchService = data;
        this.servicesToUsersPagination.currentPage = 1;
        return this.ServicesReportsService.
        getServiceToUserReports(
          this.servicesToUsersPagination,
          this.searchService,
          this.selectedServices,
          this.selectedServiceType,
          this.serviceToUserDaterangepicker
        );
      })
    )
    .subscribe({
      next: (users: any) => {
        this.servicesToUserLoading = false;
        const { recordsTotal } = users.data;
        this.servicesToUsers = users.data;
        this.servicesToUsersPagination.totalItems = recordsTotal;
      },
      error: err => {
        this.servicesToUserLoading = false;
      }
    });
  }

  filterServices(){
    this.servicesToUserLoading = true;
    this.ServicesReportsService.
    getServiceToUserReports(
      this.servicesToUsersPagination,
      this.searchService,
      this.selectedServices,
      this.selectedServiceType,
      this.serviceToUserDaterangepicker
    )
    .subscribe({
      next: summary => {
        this.servicesToUsersPagination.totalItems = summary.data.recordsTotal;
        this.topFiveServicesPagination = summary.topFive.recordsTotal;
        this.topFiveServices = summary.topFive;
        this.servicesToUsers = summary.data;
        this.servicesToUserLoading = false;
      },
      error: err=>{
        this.servicesToUserLoading = false;
      }
    })
  }

  getReportsOfAllServices(pagination: Pagination){
    this.reportsOfAllServicesLoading = true;
    this.ServicesReportsService.
    getServicesReports(
      pagination,
      this.servicesDaterangepicker[0],
      this.servicesDaterangepicker[1],
      this.serviceSortDir,
      this.serviceSort
    )
    .subscribe({
      next: summary => {
        this.summaryOfAllServicesPagination.totalItems = summary.recordsTotal;
        this.ReportsOfAllServices = summary;
        this.reportsOfAllServicesLoading = false;
      },
      error: err=>{
        this.reportsOfAllServicesLoading = false;
      }
    })
  }

  getServicesToUserReport(pagination: Pagination){
    this.servicesToUserLoading = true;
    this.ServicesReportsService.
    getServiceToUserReports(
      pagination,
      this.searchService,
      this.selectedServices,
      this.selectedServiceType,
      this.serviceToUserDaterangepicker
    )
    .subscribe({
      next: summary => {
        this.servicesToUsersPagination.totalItems = summary.data.recordsTotal;
        this.topFiveServicesPagination = summary.topFive.recordsTotal;
        this.topFiveServices = summary.topFive;
        this.servicesToUsers = summary.data;
        this.servicesToUserLoading = false;
      },
      error: err=>{
        this.servicesToUserLoading = false;
      }
    })
  }


  clearDate (type) {
    if(type === APIDataTypeEnum.Service) {
      this.servicesDaterangepicker = [];
      this.serviceDateChange();
    }
  }
  clearServiceToUserDate(){
    this.serviceToUserDaterangepicker = [];
    this.filterServices()
  }
  clearSearch(){
    this.searchService=''
  }

  onDateChange(type) {
    if(type === APIDataTypeEnum.Service) {
      if(!this.servicesDaterangepicker)
      this.servicesDaterangepicker = [];
      this.summaryOfAllServicesPagination.currentPage = 1;
      this.getReportsOfAllServices(this.summaryOfAllServicesPagination);
    }
  }

  serviceDateChange(){
    if(!this.servicesDaterangepicker)
    this.servicesDaterangepicker = [];
    this.summaryOfAllServicesPagination.currentPage = 1;
    this.getReportsOfAllServices(this.summaryOfAllServicesPagination);
  }

  pageChangeSummaryOfAllServices(page:any) {
    this.summaryOfAllServicesPagination.currentPage = page;
    this.getReportsOfAllServices(this.summaryOfAllServicesPagination);
  }
  pageChangeServicesToUser(page:any){
    this.servicesToUsersPagination.currentPage = page;
    this.getServicesToUserReport(this.servicesToUsersPagination);
  }

  sortServiceRecord(term: string){
    this.serviceSortDir = this.serviceSortDir == 'asc' ? 'desc' : 'asc';
    this.serviceSort = term;
    this.getReportsOfAllServices(this.summaryOfAllServicesPagination)
  }

  onChangeService(value){
      this.servicesToUsersPagination.currentPage = 1;
      this.selectedServices = value;
      this.filterServices()
  }
  onChangeServiceType(value){
    this.servicesToUsersPagination.currentPage = 1;
      this.selectedServiceType = value?value.service_type:'';
      this.filterServices()
  }
  serviceToUserDateChange(){
    this.servicesToUsersPagination.currentPage = 1;
    this.filterServices()
  }

  getAllServices(){
    this.reportingService.getAllServices().subscribe(services=>{
      this.services = services;
    });
  }
  exportCSV(){
    this.ServicesReportsService.exportCSV()
  }
  ReformatTime(TIME): string {
    const date = new Date(TIME);
    return `${date.getDate()}/${date.getMonth() +
      1}/${date.getFullYear()} ${date.getHours()}:${date
      .getMinutes()
      .toString()
      .padStart(2, '0')}`;
  }
  getServiceDetails(serviceid) {
    this.servicesToUserLoading = true;
    this.ServicesReportsService.
    getServiceToUserReports(
      this.serviceDetailsPagination,
      this.searchService,
      [{service_id:serviceid}],
      this.selectedServiceType,
      this.serviceToUserDaterangepicker
    )
    .subscribe({
      next: summary => {
        this.toggleRuleViewStyle = 'block';
        this.serviceDetailsPagination.totalItems = summary.data.recordsTotal;
        this.serviceDetails = summary.data;
        this.servicesToUserLoading = false;
      },
      error: err=>{
        this.servicesToUserLoading = false;
      }
    })

  }
  pageChangeServiceDetails(service_id,page){
    this.servicesToUserLoading = true;
    this.serviceDetailsPagination.currentPage=page
    this.ServicesReportsService.
    getServiceToUserReports(
      this.serviceDetailsPagination,
      this.searchService,
      [{service_id}],
      this.selectedServiceType,
      this.serviceToUserDaterangepicker
    )
    .subscribe({
      next: summary => {
        this.serviceDetailsPagination.totalItems = summary.data.recordsTotal;
        this.serviceDetails = summary.data;
        this.servicesToUserLoading = false;
      },
      error: err=>{
        this.servicesToUserLoading = false;
      }
    })
  }
  handleCancelModal(){
    this.toggleRuleViewStyle = 'none';
  }
  getClientName(client){
    let name=client
    name= name.split('@')[0].replace('{','').replace('}','');
    return name;
  }
}




