import { Component, OnInit, ViewChild } from '@angular/core';
import { Router, ActivatedRoute, NavigationEnd } from '@angular/router';
import { SettingsService } from './settings.service';
import { Setting } from '../models/setting';
import { TrustedHost } from '../models/trusted-host';
import { PreserveSearch } from '../models/preserve-search';
import { Pagination } from '../models/pagination';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { ProgressFile } from './Progress';
import { AuthenticationService } from '../login/authentication.service';
import { Observable, Subscription } from 'rxjs';
import { NgModel, FormControl } from '@angular/forms';
import { catchError, map, tap } from 'rxjs/operators';
import { BsModalService, BsModalRef } from 'ngx-bootstrap/modal';
import { ModalComponent } from '../shared/modal/modal.component';
import * as sharedContent from '../shared/shared-content.constant';
import { LicenseErrorEnum } from '../shared/emuns/license.enum';
import { ContextHelpEnum } from '../shared/emuns/global.enum';
import { Title } from '@angular/platform-browser';
import { ThemeService } from '../theme.service';
import { OtpAuthenticationService } from '../otp-verification/otp-authentication.service';
import { RolesAndPermissionService } from '../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-settings',
  templateUrl: './settings.component.html',
  styleUrls: ['./settings.component.css']
})
export class SettingsComponent implements OnInit {
  public alert: GlobalMessageClass;
  maxPackageExpiry = 10080;
  public pageTitle: string = 'Settings';
  setting = new Setting();
  public errorMessage: string;
  public progress: ProgressFile = new ProgressFile();
  //Contextual Help
  public contextualHelp: Array<any>;
  public showContext: boolean = false;
  public fileData = {};
  private fileSizeLimit = 100 * 1024;
  public uploadValid: boolean = false;
  public uploading: boolean = false;
  license: any = {};
  excludedSubnets: Array<any> = [];
  portalSetting = true;
  containsZero = false;
  pagination: Pagination = new Pagination();
  userOffset = 0;

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  @ViewChild('dirSyncTime', { static: false }) dirSyncTime: FormControl;
  //Contextual Help
  public smtpServices = [
    { value: '126', display: '126' },
    { value: '163', display: '163' },
    { value: 'AOL', display: 'AOL' },
    { value: 'DebugMail', display: 'DebugMail' },
    { value: 'DynectEmail', display: 'DynectEmail' },
    { value: 'FastMail', display: 'FastMail' },
    { value: 'GandiMail', display: 'GandiMail' },
    { value: 'Gmail', display: 'Gmail' },
    { value: 'Godaddy', display: 'Godaddy' },
    { value: 'GodaddyAsia', display: 'GodaddyAsia' },
    { value: 'GodaddyEurope', display: 'GodaddyEurope' },
    { value: 'hot.ee', display: 'hot.ee' },
    { value: 'Hotmail', display: 'Hotmail' },
    { value: 'iCloud', display: 'iCloud' },
    { value: 'local', display: 'Local Server' },
    { value: 'mail.ee', display: 'mail.ee' },
    { value: 'Mail.ru', display: 'Mail.ru' },
    { value: 'Maildev', display: 'Maildev' },
    { value: 'Mailgun', display: 'Mailgun' },
    { value: 'Mailjet', display: 'Mailjet' },
    { value: 'Mailosaur', display: 'Mailosaur' },
    { value: 'Mandrill', display: 'Mandrill' },
    { value: 'Naver', display: 'Naver' },
    { value: 'OAuth', display: 'OAuth 2.0' },
    { value: 'OpenMailBox', display: 'OpenMailBox' },
    { value: 'Outlook365', display: 'Outlook365' },
    { value: 'Postmark', display: 'Postmark' },
    { value: 'QQ', display: 'QQ' },
    { value: 'QQex', display: 'QQex' },
    { value: 'SendCloud', display: 'SendCloud' },
    { value: 'SendGrid', display: 'SendGrid' },
    { value: 'SendinBluez', display: 'SendinBluez' },
    { value: 'SendPulse', display: 'SendPulse' },
    { value: 'SES', display: 'SES' },
    { value: 'SES-US-EAST-1', display: 'SES-US-EAST-1' },
    { value: 'SES-US-WEST-2', display: 'SES-US-WEST-2' },
    { value: 'SES-EU-WEST-1', display: 'SES-EU-WEST-1' },
    { value: 'Sparkpost', display: 'Sparkpost' },
    { value: 'Yahoo', display: 'Yahoo' },
    { value: 'Yandex]Zoho', display: 'Yandex]Zoho' },
    { value: 'qiye.aliyun', display: 'qiye.aliyun' }
  ];

  public localSmtpSelected: boolean = false;

  public showSettingForm: boolean = true;

  ldap;

  public dateformats = [
    { value: 'dd/MM/yyyy HH:mm', display: 'dd/MM/yyyy HH:mm' },
    { value: 'MM/dd/yyyy HH:mm', display: 'MM/dd/yyyy HH:mm' },
    { value: 'yyyy/dd/MM HH:mm', display: 'yyyy/dd/MM HH:mm' },
    { value: 'yyyy/MM/dd HH:mm', display: 'yyyy/MM/dd HH:mm' }
  ];

  protocols = [
    { value: 'tls', display: 'TLS' },
    { value: 'tcp', display: 'TCP' },
    { value: 'udp', display: 'UDP' }
  ];
  deviceProtocols = this.protocols.filter(p => p.value !== 'udp');
  addressClasses = [
    {
      value: 'Class A',
      display: 'Class A (10.0.0.0 to 10.255.255.255)',
      ipCount: 1,
      start_ip: '10.0.0.0',
      end_ip: '10.255.255.255'
    },
    {
      value: 'Class B',
      display: 'Class B (172.16.0.0 to 172.31.255.255)',
      ipCount: 1,
      start_ip: '172.16.0.0',
      end_ip: '172.31.255.255'
    }
  ];

  selectedAddressClass = this.addressClasses[0];

  addressClassText: string = this.selectedAddressClass.value;

  numberOfUniqueGateways: number = 1;

  numberOfSupportedGateways: number = 1;

  totalIPsAvailable: number = 1;

  remainingSubnets: number;

  receivedGatewayCount: number;

  subnetIPs: Array<any> = [
    { ip: '0.0.0.0', excluded: false },
    { ip: '1.0.0.0', excluded: true }
  ];

  loading: Boolean = true;
  enableSiem: Boolean = false;
  disableAddHostButton: boolean = false;
  incorrectMonitoringInterval: boolean = false;
  gatewaySessionInterval: boolean = false;
  modalRef: BsModalRef;
  allTimeZonesData: any = {};
  timeZoneSelected: string = '';
  allTimeZoneLoading: boolean = true;
  currentTime: Date = new Date();
  portalTime: string = '';
  systemTime: string = '';
  timeZoneOptions = [
    { id: 1, displayName: 'Portal Time' },
    { id: 2, displayName: 'System Time' },
    { id: 3, displayName: 'Custom Time' }
  ];

  selectedTimeZoneOption = 0;

  showOtpDialog: boolean = false;
  otptoken: string = '';
  private otpStatusSubscription: Subscription;

  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private settingsService: SettingsService,
    private globalMessageService: GlobalMessageService,
    private authenticationService: AuthenticationService /*, private clientService: ClientService*/,
    private modalService: BsModalService,
    private titleService: Title,
    private OtpAuthenticationService: OtpAuthenticationService,
    private themesService : ThemeService,
    private rolesPermissionService: RolesAndPermissionService
  ) {
    this.contextualHelp = ['', ''];
    this.pageTitle = 'Settings';
  }

  
  onTimeZoneOptionChange() {}

  updateExcludeState(subnetIP) {
    this.subnetIPs = this.subnetIPs?.map(subnet => {
      if (subnetIP.ip == subnet.ip) {
        if (subnet.excluded) {
          subnet.excluded = false;
          this.remainingSubnets++;
        } else {
          subnet.excluded = true;
          this.remainingSubnets--;
        }
      }
      return subnet;
    });
  }

  changeSMTPServiceSelected(): void {
    if(this.setting.smtp_service == 'OAuth') {
      this.setting.oauth_config = true;
    }else {
      this.setting.oauth_config = false;
    }
    if (this.setting.smtp_service == 'local') {
      this.localSmtpSelected = true;
    } else {
      this.localSmtpSelected = false;
    }
  }

  changeGlobalDarkening() {
    if (this.setting.global_darkening) {
      this.setting.global_darkening = false;
    } else {
      this.setting.global_darkening = true;
    }
  }

  evaluateSubnetIPList() {
    this.selectedAddressClass = this.addressClasses.filter(
      addrClass => addrClass.value == this.addressClassText
    )[0];
    this.numberOfSupportedGateways = Math.pow(
      2,
      Math.ceil(Math.log(this.numberOfUniqueGateways) / Math.log(2))
    );
    this.remainingSubnets =
      this.numberOfSupportedGateways - this.receivedGatewayCount;
    let cutoffPoint =
      this.selectedAddressClass.ipCount / this.numberOfSupportedGateways;
    let currentIP = this.selectedAddressClass.start_ip;
    for (
      let iterCount = 0;
      iterCount < this.numberOfSupportedGateways;
      iterCount++
    ) {
      if (iterCount == 0) this.subnetIPs = [{ ip: currentIP, excluded: false }];
      else
        this.subnetIPs = [
          ...this.subnetIPs,
          { ip: currentIP, excluded: false }
        ];
      currentIP = this.nextIP(currentIP, cutoffPoint);
    }
  }

  nextIP(currentIP, cutoffPoint) {
    let arr = currentIP.split('.');
    let octet1 = parseInt(arr[0]);
    let octet2 = parseInt(arr[1]);
    let octet3 = parseInt(arr[2]);
    let octet4 = parseInt(arr[3]);
    for (let count = 0; count < cutoffPoint; count++) {
      if (octet4 < 255) octet4++;
      else {
        octet4 = 0;
        if (octet3 < 255) octet3++;
        else {
          octet3 = 0;
          if (octet2 < 255) octet2++;
          else {
            octet2 = 0;
            if (octet1 < 255) octet1++;
          }
        }
      }
    }
    return `${octet1}.${octet2}.${octet3}.${octet4}`;
  }

  showContextHelp(name: any): void {
    if (name == ContextHelpEnum.smtp_user) {
      this.contextualHelp = ['smtp_user', 'Enter SMTP username here.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.smtp_password) {
      this.contextualHelp = ['smtp_password', 'Enter SMTP password here.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.smtp_from) {
      this.contextualHelp = [
        'smtp_from',
        'Enter SMTP email address which will be visible to the recipient.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.package_link_expiry) {
      this.contextualHelp = [
        'package_link_expiry',
        'Enter time in minutes for expiry of sent package.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.session_expiry) {
      this.contextualHelp = [
        'session_expiry',
        'Enter time in minutes for expiry of admin user session. This setting will be effective after you logout and login again.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.mfa_server) {
      this.contextualHelp = ['mfa_server', 'Enter Ip address or hostname.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.autoPurgeDays) {
      this.contextualHelp = [
        'autoPurgeDays',
        'Enter time in days after which the client logs will be purged automatically.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.DateFormat) {
      this.contextualHelp = [
        'df',
        'Select a date format you wish to apply to portal.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.gapiKey) {
      this.contextualHelp = ['gapiKey', 'Enter Google Map API key in here.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.recaptcha_site_key) {
      this.contextualHelp = [
        'recaptcha_site_key',
        'Enter reCAPTCHA site key in here.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.password_expiry_days) {
      this.contextualHelp = [
        'password_expiry_days',
        'Enter password expiry time in days.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.smtp_service) {
      this.contextualHelp = [
        'smtp_service',
        'Enter SMTP service provider. Use Local server for manual configuration"'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.smtp_host) {
      this.contextualHelp = ['smtp_host', 'Enter SMTP host here."'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.smtp_port) {
      this.contextualHelp = ['smtp_port', 'Enter SMTP port here."'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.build) {
      this.contextualHelp = ['build', 'Select license file to upload.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.dirsync_time) {
      this.contextualHelp = ['dirsync_time', 'LDAP sync time in minutes'];
      this.showContext = true;
    }else if (name == ContextHelpEnum.oauth_cid) {
      this.contextualHelp = ['oauth_cid', 'Enter OAuth client ID here'];
      this.showContext = true;
    }else if (name == ContextHelpEnum.oauth_cs) {
      this.contextualHelp = ['oauth_cs', 'Enter OAuth client secret here'];
      this.showContext = true;
    }else if (name == ContextHelpEnum.oauth_tid) {
      this.contextualHelp = ['oauth_tid', 'Enter OAuth tenant id here'];
      this.showContext = true;
    }else if (name == ContextHelpEnum.oauth_rt) {
      this.contextualHelp = ['oauth_tid', 'Enter OAuth refresh token here'];
      this.showContext = true;
    }
  }

  handleFileUpload(evt) {
    let self = this;
    self.progress.complete = false;
    self.progress.value = 0;
    if (evt.target.files && evt.target.files.length > 0) {
      let file = evt.target.files[0]; // FileList object
      if (file.size / 1024 < this.fileSizeLimit) {
        let name = file.name;
        let brokenName = name.split('-');
        var verWithExt = brokenName[brokenName.length - 1].split('.');
        verWithExt.splice(-1, 1);
        var version = verWithExt.join('.');
        let regex = /^(\d+\.)(\d+\.)(\d+)$/gm;
        let isValidVerselectedHostIdsion = regex.test(version);
        let type = file.type;
        /*if (isValidVersion == false) {
          alert('Please select a valid file name format. Format is name-version.extension.');
          self.uploadValid = false;
          return 0;
        }*/
        //this.update.buildVersion = version;
        //Getting Type
        var local_name = name.split('.');
        let ext = local_name[local_name.length - 1];
        var types = ['txt', 'text/plain'];

        if (types.includes(type) || ext === 'txt') {
          if (ext === 'pkg') {
            type = ext;
          }
          let reader = new FileReader();
          reader.readAsDataURL(file);
          reader.onload = () => {
            let data = reader.result;
            self.fileData = {
              data: data,
              name: name,
              type: type
            };
            self.uploadValid = true;
          };
          reader.onloadstart = () => {
            self.progress.show = true;
            self.progress.value = 0;
          };
          reader.onprogress = data => {
            self.progress.show = true;
            self.progress.value = (data.loaded / data.total) * 100;
          };
          reader.onloadend = () => {
            self.progress.value = 100;
            self.progress.complete = true;
          };
        } else {
          alert(
            'Please select a valid file type, these are allowed types `txt`'
          );
          self.uploadValid = false;
        }
      } else {
        alert('File size should be less than ' + this.fileSizeLimit + ' 50MB');
        self.uploadValid = false;
      }
    }
  }

  navigate(title:string){
    console.log('title :>> ', title);
    if(title === 'Portal Settings'){
      console.log("IFFFF");
      this.router.navigate(['settings/portalSetting']);
    }else if(title === 'Client Settings'){
      this.router.navigate(['settings/clientSetting']);
    }else{
      this.router.navigate(['settings/coreSetting']);
    }
  }
  /*saveLicense(){
    this.uploading = true;
    this.settingsService.addEditLicense(this.fileData).subscribe(response => {
      if(response.success){
        this.alert = {closed: false,type: "success",message: "License has been added/updated successfully."};
        this.globalMessageService.pushAlert(this.alert);
        this.router.navigate(['/settings']);
      } else {
        this.alert = {closed: false,type: "error",message: "Failed to add/update License."};
        this.globalMessageService.pushAlert(this.alert);
        this.errorMessage = response.message;
      }
      this.uploading = false;
    });
  }*/

  changeUpdateSettings() {
    if (this.setting.is_settings_changed) {
      this.setting.is_settings_changed = false;
    } else {
      this.setting.is_settings_changed = true;
    }
  }

  changeCoreUpdateAllowed() {
    this.setting.core_auto_update = !this.setting.core_auto_update;
  }

  // numberOfDays(event: any): void {
  //   if (this.setting.auto_purge_days < 1) {
  //     this.setting.auto_purge_days = 1;
  //   }
  // }

  changePurgeAllowed(): void {
    if (this.setting.purge_allowed) {
      this.setting.purge_allowed = false;
    } else {
      this.setting.purge_allowed = true;
    }
  }

  changeUpdateAllowed(): void {
    if (this.setting.auto_update_state) {
      this.setting.auto_update_state = false;
    } else {
      this.setting.auto_update_state = true;
    }
  }

  changeDeviceProfiler(): void {
    this.setting.is_device_profiler_enabled = !this.setting.is_device_profiler_enabled;
  }

  changeAdminMfa() {
    this.setting.admin_mfa = !this.setting.admin_mfa;
  }

  changeMfaAllowed(): void {
    if (this.setting.mfa_server_state) {
      this.setting.mfa_server_state = false;
      this.setting.otp_state = false;
      this.setting.biometrics_state = false;
    } else {
      this.setting.mfa_server_state = true;
    }
  }

  changeOtpAllowed(): void {
    if (this.setting.otp_state) {
      this.setting.otp_state = false;
    } else {
      this.setting.otp_state = true;
    }
  }

  changeBioMetricsAllowed(): void {
    if (this.setting.biometrics_state) {
      this.setting.biometrics_state = false;
    } else {
      this.setting.biometrics_state = true;
    }
  }

  closeContextualHelp(): void {
    this.showContext = false;
  }

  getCRUDdata(privId){
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false
      this.readPermission = res?.read_permission === 1 ? true : false
      this.updatePermission = res?.update_permission === 1 ? true : false
      this.deletePermission = res?.delete_permission === 1 ? true : false
    });
  }

  ngOnInit() {
    this.getSettingsRole();
    let settingUrl = '';
    this.route.params.subscribe(data => {
      if (data && data.whichSetting === 'clientSetting') {
        this.portalSetting = false;
        this.pageTitle = 'Client Settings';
        settingUrl = '/clientSetting';
      } else {
        if (data && data.whichSetting === 'swConfiguration') {
          this.portalSetting = false;
          this.pageTitle = 'SW Configuration';
          settingUrl = '';
        } else {
          this.portalSetting = true;
          this.pageTitle =
            data.whichSetting !== 'coreSetting'
              ? 'Portal Settings'
              : 'Core Settings';
          settingUrl = '';
        }
      }
      if (this.pageTitle == 'Settings') {
        this.titleService.setTitle(this.pageTitle);
      } else {
        this.titleService.setTitle('Settings | ' + this.pageTitle);
      }
      this.getAllTimeZones();
      this.getSetting(settingUrl);
      this.settingsService.getAddressSpaceSettings().subscribe(
        response => {
          this.receivedGatewayCount = parseInt(response.gatewayCount);
          this.addressClasses = response.addressClasses?.map(addrClass => {
            return {
              value: addrClass.class_name,
              ipCount: parseInt(addrClass.ip_count),
              start_ip: addrClass.start_ip,
              end_ip: addrClass.end_ip,
              display: `${addrClass.class_name} (${addrClass.start_ip} to ${addrClass.end_ip})`
            };
          });
          if (response.activeConfig) {
            this.selectedAddressClass = response.addressClasses
              .filter(
                addrClass => addrClass.id == response.activeConfig.active_class
              )
              ?.map(addrClass => {
                return {
                  value: addrClass.class_name,
                  ipCount: parseInt(addrClass.ip_count),
                  start_ip: addrClass.start_ip,
                  end_ip: addrClass.end_ip,
                  display: `${addrClass.class_name} (${addrClass.start_ip} to ${addrClass.end_ip})`
                };
              })[0];
            this.numberOfUniqueGateways =
              response.activeConfig.required_gateways;
            this.numberOfSupportedGateways =
              response.activeConfig.supported_gateways;
            this.remainingSubnets =
              this.numberOfSupportedGateways - this.receivedGatewayCount;
          } else {
            this.selectedAddressClass = this.addressClasses[0];
          }
          this.addressClassText = this.selectedAddressClass.value;
          if (response.subnetIPList.length > 0) {
            this.subnetIPs = response.subnetIPList?.map(ip => {
              if (ip.excluded) this.remainingSubnets--;
              return {
                ip: ip.subnet_ip,
                excluded: ip.excluded
              };
            });
          }
        },
        error => {
          this.globalMessageService.pushAlert({
            type: 'danger',
            closed: false,
            message: 'Could not fetch Address Space details'
          });
        }
      );
    });
    this.validateMonitoringInterval();
    this.validateGatewaySessionInterval();
  }

  preventScroll(event: Event) {
    event.preventDefault();
  }
  ngOnDestroy() {
    if (this.otpStatusSubscription) {
      this.otpStatusSubscription.unsubscribe();
    }
  }
  
  getSettingsRole(){
    this.router.events.subscribe(event => {
      if (event instanceof NavigationEnd) {
        const currentUrl = this.router.url;
        if(currentUrl == '/settings/portalSetting'){
          this.getCRUDdata(56);
        }else if(currentUrl == '/settings/clientSetting'){
          this.getCRUDdata(57);
        }else if(currentUrl == '/settings/coreSetting'){
          this.getCRUDdata(58);
        }
      }
    });
  }

  private getSetting(settingUrl) {
    this.settingsService
      .getSettings(settingUrl)
      .pipe(
        tap(setting => {
          this.loading = false;
          this.selectedTimeZoneOption = setting?.timesetting?.time_type
            ? setting?.timesetting?.time_type
            : 1;

          this.selectedTimeZoneOption == 3
            ? (this.timeZoneSelected = setting?.timesetting?.timezone)
            : '';

          localStorage.setItem(
            'userTimeData',
            JSON.stringify({
              time_type: this.selectedTimeZoneOption,
              timezone: this.timeZoneSelected
            })
          );

          this.setting.timezone = setting?.timesetting?.timezone;
          this.setting.user_time_type = setting?.timesetting?.time_type;

          if (setting.status) {
            if (setting.data.length === 0) {
              this.setting = new Setting();
              this.alert = {
                closed: false,
                type: 'warning',
                message: 'No settings were set kindly update settings and save.'
              };
              this.globalMessageService.pushAlert(this.alert);
            } else {
              if (settingUrl) {
                this.setting = setting.data;
                this.license = setting.license || {};
                if (!this.setting.mfa_server_state) {
                  this.setting.otp_state = false;
                  this.setting.biometrics_state = false;
                }
              } else {
                this.setting = setting.data[0];
                if (this.setting.smtp_service == 'local') {
                  this.localSmtpSelected = true;
                }
                this.ldap = setting.ldap;
                this.license = setting.license || {};
                if (!this.license.mfa_feature) {
                  this.setting.mfa_server_state = false;
                  this.setting.otp_state = false;
                  this.setting.biometrics_state = false;
                }
                if (!this.setting.isSettingsSet) {
                  this.alert = {
                    closed: false,
                    type: 'warning',
                    message:
                      'No settings were set kindly update settings and save.'
                  };
                  this.globalMessageService.pushAlert(this.alert);
                }
              }
            }
          } else {
            this.alert = {
              closed: false,
              type: 'info',
              message: 'No settings were found add settings and save.'
            };
            this.globalMessageService.pushAlert(this.alert);
          }

          this.enableSiem = this.setting.siem_host !== '0.0.0.0';
          this.setting.siem_port =
            this.setting.siem_host !== '0.0.0.0' ? this.setting.siem_port : 0;
          this.setting.profiling_server_port =
            this.setting.profiling_server_host !== '0.0.0.0'
              ? this.setting.profiling_server_port
              : 0;
          this.setting.profiling_enabled =
            this.setting.profiling_server_host !== '0.0.0.0';
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          this.loading = false;
          if (err.status === 403) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'You are not authorized to access this page.'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          }
        },
        complete: () => {}
      });
  }

  getAllTimeZones() {
    this.allTimeZoneLoading = true;
    this.settingsService
      .getAllTimeZones()
      .pipe(
        map((data: any) => data.timeFormats),
        map((timeFormats: any) =>
          timeFormats?.map((format: any) => format.uniqueIdentifier)
        )
      )
      .subscribe({
        next: data => {
          this.allTimeZonesData = data;
          this.allTimeZoneLoading = false;
        },
        error: error => catchError(error)
      });
  }

  changeAutoSyncHandler() {
    const { dirsync_time } = this.setting;
    if (dirsync_time && dirsync_time[0] && dirsync_time[0] === '0') {
      if (!this.dirSyncTime.invalid) {
        this.containsZero = true;
      } else {
        this.containsZero = false;
      }
    } else {
      this.containsZero = false;
    }
  }

  validateMonitoringInterval() {
    this.incorrectMonitoringInterval = false;
    if (this.setting.process_reporting_interval) {
      if (
        Number(this.setting.process_reporting_interval) > Number(172800) ||
        Number(this.setting.process_reporting_interval) < 1
      )
        this.incorrectMonitoringInterval = true;
    }
  }

  validateGatewaySessionInterval(){
    this.gatewaySessionInterval = false;
    if (this.setting.idle_session_timeout) {
      if (
        Number(this.setting.idle_session_timeout) > Number(43200) ||
        Number(this.setting.idle_session_timeout) < 1
      )
        this.gatewaySessionInterval = true;
    }
  }

  saveUpdate() {
    let mfa = localStorage.getItem('adminMfaValue');
    localStorage.setItem('isOauthService', JSON.stringify(this.setting.oauth_config));

    if (mfa === 'true') {
      this.showOtpDialog = true;
      console.log('INSAVE', mfa);

      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheck$.subscribe(
        (res: any) => {
          console.log('OTPSTATUS', res);

          if (res == true) {
            
            if (this.otpStatusSubscription) {
              this.otpStatusSubscription.unsubscribe();
            }

            this.setting.timezone = this.timeZoneSelected;
            this.setting.user_time_type = this.selectedTimeZoneOption;

            localStorage.setItem(
              'userTimeData',
              JSON.stringify({
                time_type: this.selectedTimeZoneOption,
                timezone: this.timeZoneSelected
              })
            );

            this.setting.isSettingsSet = true;
            this.loading = true;
            this.setting.ldap = this.ldap;
            this.settingsService.addEditSetting(this.setting).subscribe(
              response => {
                this.loading = false;
                if (response.success) {
                  localStorage.setItem(
                    'adminMfaValue',
                    JSON.stringify(this.setting.admin_mfa)
                  );
                  this.alert = {
                    closed: false,
                    type: 'success',
                    message: 'Settings have been added/updated successfully.'
                  };
                  this.globalMessageService.pushAlert(this.alert);
                  window.scrollTo(0, 0);

                  // this.router.navigate(['/settings']);

                  this.setting.smtp_password = '';

                  localStorage.setItem(
                    'dateFormat',
                    '' + this.setting.date_format
                  );

                  this.settingsService.userSettingsLoaded.emit();
                } else {
                  this.alert = {
                    closed: false,
                    type: 'danger',
                    message: 'Failed to add/update Settings.'
                  };
                  this.globalMessageService.pushAlert(this.alert);
                  this.errorMessage = response.message;
                }
              },
              error => {
                this.loading = false;
              }
            );

            this.OtpAuthenticationService.setOtpStatusValue(false);
            this.showOtpDialog = false;
          }
        }
      );
    } else {
      this.setting.timezone = this.timeZoneSelected;
      this.setting.user_time_type = this.selectedTimeZoneOption;

      localStorage.setItem(
        'userTimeData',
        JSON.stringify({
          time_type: this.selectedTimeZoneOption,
          timezone: this.timeZoneSelected
        })
      );

      this.setting.isSettingsSet = true;
      this.loading = true;
      this.setting.ldap = this.ldap;
      this.settingsService.addEditSetting(this.setting).subscribe(
        response => {
          this.loading = false;
          if (response.success) {
            localStorage.setItem(
              'adminMfaValue',
              JSON.stringify(this.setting.admin_mfa)
            );
            this.alert = {
              closed: false,
              type: 'success',
              message: 'Settings have been added/updated successfully.'
            };
            this.globalMessageService.pushAlert(this.alert);
            window.scrollTo(0, 0);

            // this.router.navigate(['/settings']);

            this.setting.smtp_password = '';

            localStorage.setItem('dateFormat', '' + this.setting.date_format);

            this.settingsService.userSettingsLoaded.emit();
          } else {
            this.alert = {
              closed: false,
              type: 'danger',
              message: 'Failed to add/update Settings.'
            };
            this.globalMessageService.pushAlert(this.alert);
            this.errorMessage = response.message;
          }
        },
        error => {
          this.loading = false;
        }
      );
    }
  }

  saveClientSetting() {
    let mfa = localStorage.getItem('adminMfaValue');
    if (mfa === 'true') {
      this.showOtpDialog = true;

      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheck$.subscribe(
        (res: any) => {
          if (res == true) {
            if (this.otpStatusSubscription) {
              this.otpStatusSubscription.unsubscribe();
            }
            if (this.incorrectMonitoringInterval) {
              this.alert = {
                closed: false,
                type: 'danger',
                message:
                  'Process Reporting Interval should be less than or equal to 2880.'
              };
              this.globalMessageService.pushAlert(this.alert);
              return;
            }
            this.loading = true;
            this.settingsService.updateClientSetting(this.setting).subscribe(
              (response: any) => {
                this.loading = false;
                if (response.success) {
                  this.alert = {
                    closed: false,
                    type: 'success',
                    message: 'Settings has been added/updated successfully.'
                  };
                  this.globalMessageService.pushAlert(this.alert);
                  window.scrollTo(0, 0);
                } else {
                  this.alert = {
                    closed: false,
                    type: 'danger',
                    message: 'Failed to add/update Settings.'
                  };
                  this.globalMessageService.pushAlert(this.alert);
                  this.errorMessage = response.message;
                }
              },
              error => {
                this.loading = false;
              }
            );
            this.OtpAuthenticationService.setOtpStatusValue(false);
            this.showOtpDialog = false;
          }
        }
      );
      this.OtpAuthenticationService.setOtpStatusValue(false);
    } else {
      if (this.incorrectMonitoringInterval) {
        this.alert = {
          closed: false,
          type: 'danger',
          message:
            'Process Reporting Interval should be less than or equal to 2880.'
        };
        this.globalMessageService.pushAlert(this.alert);
        return;
      }
      this.loading = true;
      this.settingsService.updateClientSetting(this.setting).subscribe(
        (response: any) => {
          this.loading = false;
          if (response.success) {
            this.alert = {
              closed: false,
              type: 'success',
              message: 'Settings has been added/updated successfully.'
            };
            this.globalMessageService.pushAlert(this.alert);
            window.scrollTo(0, 0);
          } else {
            this.alert = {
              closed: false,
              type: 'danger',
              message: 'Failed to add/update Settings.'
            };
            this.globalMessageService.pushAlert(this.alert);
            this.errorMessage = response.message;
          }
        },
        error => {
          this.loading = false;
        }
      );
    }
  }

  saveCoreSetting() {
    let mfa = localStorage.getItem('adminMfaValue');
    if (mfa === 'true') {
      this.showOtpDialog = true;

      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheck$.subscribe(
        (res: any) => {
          if (res == true) {
            if (this.otpStatusSubscription) {
              this.otpStatusSubscription.unsubscribe();
            }
            if (this.remainingSubnets < 0) {
              this.alert = {
                closed: false,
                type: 'danger',
                message:
                  'Subnets selected are less than available unique gateways.'
              };
              this.globalMessageService.pushAlert(this.alert);
              return;
            }

            this.setting.profiling_server_port =
              this.setting.profiling_server_host !== '0.0.0.0'
                ? this.setting.profiling_server_port
                : 0;
            this.setting.profiling_enabled =
              this.setting.profiling_server_host !== '0.0.0.0';
            const {
              siem_host,
              siem_port,
              siem_protocol,
              max_clients_per_controller,
              health_status,
              data_reporting_interval,
              load_stats,
              data_monitoring_interval,
              core_auto_update,
              in_activity_timeout,
              mtls_frequency,
              mtls_blocking_timeout,
              health_interval,
              load_stats_interval,
              device_validation_freq,
              profiling_server_host,
              profiling_server_port,
              profiling_server_protocol,
              profiling_enabled
            } = this.setting;

            const coreSettings = {
              siem_host,
              siem_port,
              siem_protocol,
              profiling_server_host,
              profiling_server_port,
              profiling_server_protocol,
              profiling_enabled,
              max_clients_per_controller,
              health_status,
              data_reporting_interval,
              load_stats,
              data_monitoring_interval,
              core_auto_update,
              in_activity_timeout: +in_activity_timeout,
              mtls_frequency: +mtls_frequency,
              mtls_blocking_timeout: +mtls_blocking_timeout,
              health_interval: +health_interval,
              load_stats_interval: +load_stats_interval,
              device_validation_freq: device_validation_freq
            };

            this.setting.siem_port =
              this.setting.siem_host !== '0.0.0.0' ? this.setting.siem_port : 0;
            this.enableSiem = this.setting.siem_host !== '0.0.0.0';

            this.loading = true;
            const addressSpacePayload = {
              addressClass: this.selectedAddressClass,
              uniqueGateways: this.numberOfUniqueGateways,
              supportedGateways: this.numberOfSupportedGateways,
              exceptionList: this.subnetIPs
            };
            this.settingsService
              .updateAddressSpaceSettings(addressSpacePayload)
              .subscribe(
                response => {
                  this.loading = false;
                },
                error => {
                  this.loading = false;
                }
              );

            this.loading = true;
            this.settingsService.updateCoreSetting(coreSettings).subscribe(
              (response: any) => {
                this.loading = false;
                if (response.success) {
                  this.alert = {
                    closed: false,
                    type: 'success',
                    message: 'Settings has been added/updated successfully.'
                  };
                  this.globalMessageService.pushAlert(this.alert);
                  window.scrollTo(0, 0);
                } else {
                  this.alert = {
                    closed: false,
                    type: 'danger',
                    message: 'Failed to add/update Settings.'
                  };
                  this.globalMessageService.pushAlert(this.alert);
                  this.errorMessage = response.message;
                }
              },
              error => {
                this.loading = false;
              }
            );
            this.OtpAuthenticationService.setOtpStatusValue(false);
            this.showOtpDialog = false;
          }
        }
      );
    } else {
      if (this.remainingSubnets < 0) {
        this.alert = {
          closed: false,
          type: 'danger',
          message: 'Subnets selected are less than available unique gateways.'
        };
        this.globalMessageService.pushAlert(this.alert);
        return;
      }

      this.setting.profiling_server_port =
        this.setting.profiling_server_host !== '0.0.0.0'
          ? this.setting.profiling_server_port
          : 0;
      this.setting.profiling_enabled =
        this.setting.profiling_server_host !== '0.0.0.0';
      const {
        siem_host,
        siem_port,
        siem_protocol,
        max_clients_per_controller,
        health_status,
        data_reporting_interval,
        load_stats,
        data_monitoring_interval,
        core_auto_update,
        in_activity_timeout,
        mtls_frequency,
        mtls_blocking_timeout,
        health_interval,
        load_stats_interval,
        device_validation_freq,
        profiling_server_host,
        profiling_server_port,
        profiling_server_protocol,
        profiling_enabled
      } = this.setting;

      const coreSettings = {
        siem_host,
        siem_port,
        siem_protocol,
        profiling_server_host,
        profiling_server_port,
        profiling_server_protocol,
        profiling_enabled,
        max_clients_per_controller,
        health_status,
        data_reporting_interval,
        load_stats,
        data_monitoring_interval,
        core_auto_update,
        in_activity_timeout: +in_activity_timeout,
        mtls_frequency: +mtls_frequency,
        mtls_blocking_timeout: +mtls_blocking_timeout,
        health_interval: +health_interval,
        load_stats_interval: +load_stats_interval,
        device_validation_freq: device_validation_freq
      };

      this.setting.siem_port =
        this.setting.siem_host !== '0.0.0.0' ? this.setting.siem_port : 0;
      this.enableSiem = this.setting.siem_host !== '0.0.0.0';

      this.loading = true;
      const addressSpacePayload = {
        addressClass: this.selectedAddressClass,
        uniqueGateways: this.numberOfUniqueGateways,
        supportedGateways: this.numberOfSupportedGateways,
        exceptionList: this.subnetIPs
      };
      this.settingsService
        .updateAddressSpaceSettings(addressSpacePayload)
        .subscribe(
          response => {
            this.loading = false;
          },
          error => {
            this.loading = false;
          }
        );

      this.loading = true;
      this.settingsService.updateCoreSetting(coreSettings).subscribe(
        (response: any) => {
          this.loading = false;
          if (response.success) {
            this.alert = {
              closed: false,
              type: 'success',
              message: 'Settings has been added/updated successfully.'
            };
            this.globalMessageService.pushAlert(this.alert);
            window.scrollTo(0, 0);
          } else {
            this.alert = {
              closed: false,
              type: 'danger',
              message: 'Failed to add/update Settings.'
            };
            this.globalMessageService.pushAlert(this.alert);
            this.errorMessage = response.message;
          }
        },
        error => {
          this.loading = false;
        }
      );
    }
  }

  onCancel(): void {
    this.router.navigate(['/dashboard']);
  }

  popoeverContent(item) {
    return `${item}`;
  }

  onClick() {
    this.modalRef = this.modalService.show(ModalComponent, {
      initialState: sharedContent.passwordSuggestion,
      class: 'modal-dialog-centered'
    });
  }

  logOutClients() {
    this.loading = true;
    this.settingsService.logOutClients().subscribe({
      next: res => {
        this.loading = false;
        this.alert = {
          closed: false,
          type: 'success',
          message: 'Logout process initiated.'
        };
        this.globalMessageService.pushAlert(this.alert);
      },
      error: err => {
        this.loading = false;
      }
    });
  }

  toggleSiemSettings() {
    if (!this.enableSiem) {
      this.setting.siem_host = '0.0.0.0';
      this.setting.siem_port = 0;
    }

    if (!this.setting.profiling_enabled) {
      this.setting.profiling_server_host = '0.0.0.0';
      this.setting.profiling_server_port = 0;
    }
  }

  cancelVerificationPassword(event: any) {
    this.showOtpDialog = false;
    this.OtpAuthenticationService.setOtpStatusValue(false);
  }

  submitVerificationPassword(event: any) {
    this.otptoken = event;
    // this.otptoken = event
  }

  getClosePopup(event: any) {
    this.showOtpDialog = false;
  }
}
