import {
    Component,
    ElementRef,
    EventEmitter,
    Input,
    OnInit,
    Output,
    ViewChild
} from '@angular/core';

@Component({
    selector: 'verification-modal',
    template: `
    <div class="modal fade" [ngClass]="{'in':show,'d-block': show}" role="dialog">
        <div class="modal-dialog">
            <div class="modal-content p-5">
                <div class="row">
                    <div
                        class="custom-alert col-sm-12 col-xs-12"
                        [class.custom-alert-hide]="!show"
                        >
                        <h1 class="text-center">{{ heading }}</h1>
                        <p>{{ body }}</p>
                        <p *ngIf="showAuthError" class="alert alert-danger">
                            Password you have entered is not valid.
                        </p>
                        <input
                            type="password"
                            class="form-control"
                            id="authpassword"
                            required
                            [(ngModel)]="authPassword"
                            (focus)="hideAuthError()"
                            (keydown)="submitMyAlert($event)"
                            [autofocus]="show && focus()"
                            #authpassword
                        />
                        <button
                            id="userListAction"
                            (click)="authAction.emit(authPassword)"
                            class="btn btn-default btn-primary pull-right mt-2 mr-0 ml-2"
                            [disabled]="reSubmit || authPassword.length == 0"
                        >
                            {{ action }}
                        </button>
                        <button
                            id="userListActionCancel"
                            (click)="cancelPopup.emit()"
                            class="btn btn-default cancel_button pull-right mt-2 mr-0"
                        >
                            Cancel
                        </button>
                    </div>
                </div>
            </div>
        </div>
    </div>
  `
})

export class VerificationModalComponent implements OnInit {

    @Input() show: boolean;
    @Input() heading: string;
    @Input() body: string;
    @Input() action: string;
    @Input() showAuthError: boolean;
    @Input() authPassword: string;
    @Input() reSubmit: boolean;

    @Output() authAction: EventEmitter<string> = new EventEmitter<string>();
    @Output() cancelPopup: EventEmitter<void> = new EventEmitter<void>();

    ngOnInit(): void { }

    @ViewChild('authpassword', { static: false }) _authpassword: ElementRef;

    hideAuthError() {
        this.showAuthError = false;
    }

    submitMyAlert(event) {
        if (event.keyCode == 13) {
            if (!this.reSubmit && this.authPassword.length != 0) {
                this.authAction.emit(this._authpassword.nativeElement.value);
            }
        }
    }

    focus() {
        this._authpassword.nativeElement.focus();
    }
}
