import { Component, OnInit } from '@angular/core';
import { Router } from '@angular/router';
import { Pagination } from '../models/pagination';
import { SoftwareProcessBindingService } from './software-process-binding.service';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { Observable, Subject } from 'rxjs';
import { PreserveSearch } from '../models/preserve-search';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { DeviceOSEnum } from '../shared/emuns/client.enum';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { RolesAndPermissionService } from '../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-software-process-binding',
  templateUrl: './software-process-binding.component.html',
  styleUrls: ['./software-process-binding.component.css']
})
export class SoftwareProcessBindingComponent implements OnInit {
  showHelpText: boolean = false;
  pagination = new Pagination();
  softwares: any = {};
  $searchSoftwares = new Subject();
  softwareTobeDeleted: any;
  isNameSort: boolean = false;
  searchWord;
  preserveSearch = new PreserveSearch();
  navigationToEditSoftware: boolean = false;

  loading: Boolean = true;

  pageSize = [10, 25, 50, 100, 200];

  sort: string;
  sortDir: string;

  selectAll = false;

  selectAllExcept: any = new Set();
  selectedSoftwares: any = new Set();

  uploadCsvFile = false;
  fileData = {};
  isFileValid = false;
  isUploading = false;
  showFailedUpload: boolean = false;
  failedSoftwares = [];
  softwaresOs = [];

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  // softwaresOs = [
  //   'macOS',
  //   'Microsoft Windows 10 Pro',
  //   'Ubuntu',
  //    'Ubuntu 18.04',
  //    'Ubuntu 18.04.6 LTS',
  //    'Ubuntu 20.04'
  // ]
  selectedOs;
  isDarkMode: boolean;

  selectedIdsForDeleteCurrent: any;
  deleteAllTotal: number = 0;
  deleteCurrentTotal: number = 0;
  constructor(
    private router: Router,
    private softwareProcessBindingService: SoftwareProcessBindingService,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog,
    private titleService: Title,
    private rolesPermissionService: RolesAndPermissionService
  ) {}

  sortByNameHandler() {
    this.isNameSort = !this.isNameSort;
    if (this.searchWord) {
      this.$searchSoftwares.next({
        searchWord: this.searchWord,
        typing: false
      });
    } else {
      this.getSoftwares();
    }
  }

  getOSName(osName) {
    switch (osName) {
      case DeviceOSEnum.Ubuntu18:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer18:
        return 'Ubuntu Server';
      case DeviceOSEnum.UbuntuServer20armv8:
        return 'Ubuntu Server (ARMv8)';
      case DeviceOSEnum.Windows:
        return 'Windows';
      case DeviceOSEnum.WindowsServer:
        return 'Windows Server';
      case DeviceOSEnum.Macintosh:
        return 'Macintosh';
      case DeviceOSEnum.MacOS:
        return 'macOS';
      case DeviceOSEnum.Linux:
        return 'Linux';
      case DeviceOSEnum.IOS:
        return 'iOS';
      case DeviceOSEnum.Android:
        return 'Android';
      case DeviceOSEnum.CentosServer:
        return 'CentOS Server';
      default:
        return osName;
    }
  }

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.titleService.setTitle(
      'Device Health Check | Software Process Binding'
    );
    this.getCRUDdata(51);
    const _saveSearch = this.preserveSearch.getSearch('softwareSearch');
    if (!_saveSearch) {
      this.getSoftwares();
    } else {
      this.searchWord = _saveSearch.searchKey;
      this.pagination = _saveSearch.pagination;
      this.selectedOs = _saveSearch.selectedOs;
      this.getSoftwares();
    }
    this.subscribeToSearch();
  }

  ngOnDestroy() {
    if (!this.navigationToEditSoftware) {
      this.preserveSearch.saveSearch('softwareSearch', '');
    }
  }

  getCRUDdata(privId) {
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false;
      this.readPermission = res?.read_permission === 1 ? true : false;
      this.updatePermission = res?.update_permission === 1 ? true : false;
      this.deletePermission = res?.delete_permission === 1 ? true : false;
    });
  }

  selectAllAction() {
    this.selectAllExcept.clear();
    this.selectedSoftwares.clear();
    this.changeCheckStatusOfAllSoftwares(this.selectAll);

    this.selectedIdsForDeleteCurrent = [];
    this.deleteAllTotal = this.softwares.recordsTotal;
    this.deleteCurrentTotal = this.softwares.data.length;

    if (!this.selectAll) {
      this.selectedIdsForDeleteCurrent = [];
      this.deleteAllTotal = 0;
      this.deleteCurrentTotal = 0;
    } else {
      this.softwares.data.forEach(software => {
        this.selectedIdsForDeleteCurrent.push(software.id);
      });
    }

    
  }

  private changeCheckStatusOfAllSoftwares(check) {
    this.softwares.data = this.softwares.data.map(_software => {
      return {
        ..._software,
        isChecked:
          (!this.selectAllExcept.has(_software.id) && this.selectAll) ||
          (this.selectedSoftwares.has(_software.id) && !this.selectAll)
      };
    });
  }

  subscribeToSearch() {
    this.$searchSoftwares
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((search: { searchWord: string; typing: boolean }) => {
          this.loading = true;
          this.clearSelection();
          if (search.typing) {
            this.pagination.currentPage = 1;
          }
          return this.softwareProcessBindingService.searchSoftwares(
            search.searchWord,
            this.pagination.itemsPerPage,
            this.pagination.currentPage,
            this.sort,
            this.sortDir
          );
        })
      )
      .subscribe((softwares: any) => {
        this.softwares = softwares;
        this.pagination.totalItems = softwares.recordsTotal;
        this.loading = false;
      });
  }

  createSoftware() {
    this.router.navigate(['/create-software-process-binding']);
  }

  getSoftwares() {
    this.loading = true;
    this.softwareProcessBindingService
      .getSoftwares(this.pagination, this.sort, this.sortDir, this.searchWord)
      .subscribe(
        (response: any) => {
          this.loading = false;
          response.data = response.data.map(_software => {
            return { ..._software, isChecked: false };
          });

          this.softwares = response;

          setTimeout(() => {
            this.changeCheckStatusOfAllSoftwares(this.selectAll);
          }, 10);

          this.pagination.totalItems = response.recordsTotal;
        },
        error => {
          this.loading = false;
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'danger',
            message: 'Error in getting softwares',
            sticky: false
          });
        }
      );
  }

  changeCheckBox(software) {
    const { isChecked, id } = software;
    if (this.selectAll) {
      if (isChecked) {
        this.selectAllExcept.delete(id);
        this.deleteAllTotal = this.deleteAllTotal + 1;
        this.deleteCurrentTotal = this.deleteCurrentTotal + 1;
        this.selectedIdsForDeleteCurrent.push(id);
      } else {
        this.selectAllExcept.add(id);
        this.deleteAllTotal = this.deleteAllTotal - 1;
        this.deleteCurrentTotal = this.deleteCurrentTotal - 1;
        this.selectedIdsForDeleteCurrent = this.selectedIdsForDeleteCurrent.filter(
          selectedId => selectedId !== id
        );
      }
    } else {
      if (!isChecked) {
        this.selectedSoftwares.delete(id);
        this.deleteAllTotal = this.deleteAllTotal - 1;
        this.deleteCurrentTotal = this.deleteCurrentTotal - 1;
        this.selectedIdsForDeleteCurrent = this.selectedIdsForDeleteCurrent.filter(
          selectedId => selectedId !== id
        );
      } else {
        this.selectedSoftwares.add(id);
        this.deleteAllTotal = this.deleteAllTotal + 1;
        this.deleteCurrentTotal = this.deleteCurrentTotal + 1;
        this.selectedIdsForDeleteCurrent.push(id);
      }
    }
  }

  editSoftware(software) {
    this.preserveCurrentSearch();
    this.router.navigate(['/software-process-binding', software.id]);
  }

  preserveCurrentSearch() {
    const save = {
      pagination: this.pagination,
      searchKey: this.searchWord,
      selectedOs: this.selectedOs
    };
    this.preserveSearch.saveSearch('softwareSearch', save);
    this.navigationToEditSoftware = true;
  }

  // viewDevice(software) {
  //   this.preserveCurrentSearch()
  //   this.router.navigate(['/view-device', software.id]);
  // }

  deleteSoftware(software) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'This will delete software. Proceed?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.softwareTobeDeleted = software;
        this.softwareProcessBindingService
          .deleteSoftware(this.softwareTobeDeleted.id)
          .subscribe(
            response => {
              this.getSoftwares();
              this.globalMessageService.pushAlert({
                closed: false,
                type: 'success',
                message: 'Software has been deleted successfully.',
                sticky: false
              });
            },
            error => {
              this.loading = false;
            }
          );
      }
    });
  }

  pageChange(page) {
    this.pagination.currentPage = page;
    if (this.searchWord) {
      this.$searchSoftwares.next({
        searchWord: this.searchWord,
        typing: false
      });
    } else {
      this.getSoftwares();
    }
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getSoftwares();
  }

  clearSelection() {
    this.selectAllExcept.clear();
    this.selectAll = false;
    this.selectedIdsForDeleteCurrent = [];
  }

  sortRecord(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
    this.getSoftwares();
  }

  clearfun() {
    this.searchWord = '';
    this.getSoftwares();
  }

  bulkActions() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'This will delete all softwares. Proceed?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        let selectedIds = this.selectAll
          ? this.selectAllExcept
          : this.selectedSoftwares;
        this.softwareProcessBindingService
          .bulkActions(this.selectAll, selectedIds, this.searchWord)
          .subscribe({
            next: res => {
              this.getSoftwares();
              this.globalMessageService.pushAlert({
                closed: false,
                type: 'success',
                message:
                  'Software Processes Binding has been updated successfully.',
                sticky: false
              });
              this.deleteAllTotal = 0;
              this.deleteCurrentTotal = 0;
              this.clearSelection();
            },
            error: err => {
              this.loading = false;
            }
          });
      }
    });
  }

  deleteFromCurrentPage() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'This will delete softwares from current page. Proceed?'
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        let selectedIds = this.selectAll
          ? this.selectAllExcept
          : this.selectedSoftwares;
        if (!(selectedIds instanceof Set)) {
          return;
        }

        if (!this.selectAll) {
          this.selectedIdsForDeleteCurrent = [];
          this.selectedIdsForDeleteCurrent.push(...selectedIds);
        } else {
          let temp = this.selectedIdsForDeleteCurrent.filter(selectedId => {
            return ![...selectedIds].some(id => id === selectedId);
          });

          this.selectedIdsForDeleteCurrent = temp;
        }

        this.softwareProcessBindingService
          .deleteCurrentSoftwareProcessBinding(this.selectedIdsForDeleteCurrent)
          .subscribe({
            next: res => {
              this.getSoftwares();
              this.globalMessageService.pushAlert({
                closed: false,
                type: 'success',
                message:
                  'Software Processes Binding has been updated successfully.',
                sticky: false
              });
              this.selectedIdsForDeleteCurrent = [];
              this.deleteAllTotal = 0;
              this.deleteCurrentTotal = 0;
              this.clearSelection();
            },
            error: err => {
              this.loading = false;
            }
          });
      }
    });
  }
  osFilterChange() {
    this.clearSelection();
    this.getSoftwares();
  }
}
