import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { ConfigService } from '../services/config.service';
import { AuthShortlivedCookieService } from '../auth-shortlived-cookie.service';
import { map } from 'rxjs/operators';

@Injectable()
export class SoftwareProcessBindingService {
  private apiBaseUrl: string;
  private softwaresUrl = '/softwares';

  constructor(
    private authHttp: AuthHttp,
    private configService: ConfigService,
    private authShortlivedCookie: AuthShortlivedCookieService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.softwaresUrl = this.apiBaseUrl + this.softwaresUrl;
  }

  getSoftwares(
    pagination,
    sort: string = 'id',
    dir: string = 'desc',
    searchKey = '',
    os = ''
  ) {
    let query =
      `?limit=${pagination.itemsPerPage}` +
      `&offset=${pagination.currentPage - 1}` +
      `&sort=${sort}` +
      `&dir=${dir}` +
      `&searchTerm=${searchKey}` +
      `&os=${os}`;

    return this.authHttp
      .get(`${this.softwaresUrl}/software-process-binding${query}`)
      .pipe(map(response => response));
  }

  searchSoftwares(
    searchQuery: string,
    pageSize: number = 10,
    page: number = 1,
    sort: string = 'id',
    dir: string = 'desc'
  ) {
    const encodedUrlComponent = encodeURIComponent(searchQuery);
    let query =
      `?limit=${pageSize}` +
      `&offset=${page - 1}` +
      `&sort=${sort}` +
      `&dir=${dir}` +
      `&searchTerm=${encodedUrlComponent}`;

    return this.authHttp
      .get(`${this.softwaresUrl}/software-process-binding/${query}`)
      .pipe(map(response => response));
  }

  deleteSoftware(id) {
    return this.authHttp
      .delete(`${this.softwaresUrl}/software-process-binding/${id}`)
      .pipe(map(response => response));
  }

  getSoftware(id) {
    return this.authHttp
      .get(`${this.softwaresUrl}/software-process-binding/${id}`)
      .pipe(map(response => response));
  }

  getDistinctOS() {
    return this.authHttp
      .get(`${this.softwaresUrl}/oslist`)
      .pipe(map(response => response));
  }

  addSoftware(software, editing) {
    if (!editing) {
      return this.authHttp
        .post(`${this.softwaresUrl}/create-software-process-binding`, {
          software
        })
        .pipe(map(response => response));
    } else {
      return this.authHttp
        .put(`${this.softwaresUrl}/software-process-binding/${editing}`, {
          software
        })
        .pipe(map(response => response));
    }
  }

  bulkActions(selectAll = false, selectedIds, searchWord) {
    return this.authHttp.put(
      `${this.softwaresUrl}/bulk-actions-software-process-binding`,
      {
        selectAll,
        selectedIds: Array.from(selectedIds),
        searchWord
      }
    );
  }

  deleteCurrentSoftwareProcessBinding(selectedIds) {
    return this.authHttp.put(`${this.softwaresUrl}/bulk-actions-delete-current-process-binding`, {
      selectedIds: Array.from(selectedIds)
    });
  }
}
