import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { ConfigService } from '../services/config.service';
import { AuthShortlivedCookieService } from '../auth-shortlived-cookie.service';
import { map } from 'rxjs/operators';

@Injectable()
export class SoftwareService {
  private apiBaseUrl: string;
  private softwaresUrl = '/softwares';

  constructor(
    private authHttp: AuthHttp,
    private configService: ConfigService,
    private authShortlivedCookie: AuthShortlivedCookieService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.softwaresUrl = this.apiBaseUrl + this.softwaresUrl;
  }

  getSoftwares(
    pagination,
    sort: string = 'id',
    dir: string = 'desc',
    searchKey = '',
    os = ''
  ) {
    let query =
      `?limit=${pagination.itemsPerPage}` +
      `&offset=${pagination.currentPage - 1}` +
      `&sort=${sort}` +
      `&dir=${dir}` +
      `&searchTerm=${searchKey}` +
      `&os=${os}`;

    return this.authHttp
      .get(`${this.softwaresUrl}${query}`)
      .pipe(map(response => response));
  }

  searchSoftwares(
    searchQuery: string,
    selectedOs: string,
    pageSize: number = 10,
    page: number = 1,
    sort: string = 'id',
    dir: string = 'desc'
  ) {
    const encodedUrlComponent = encodeURIComponent(searchQuery);
    let query =
      `?limit=${pageSize}` +
      `&offset=${page - 1}` +
      `&sort=${sort}` +
      `&dir=${dir}` +
      `&searchTerm=${encodedUrlComponent}` +
      `&os=${encodeURIComponent(selectedOs)}`;

    return this.authHttp
      .get(`${this.softwaresUrl}${query}`)
      .pipe(map(response => response));
  }

  deleteSoftware(id) {
    return this.authHttp
      .delete(`${this.softwaresUrl}/${id}`)
      .pipe(map(response => response));
  }

  getSoftware(id) {
    return this.authHttp
      .get(`${this.softwaresUrl}/${id}`)
      .pipe(map(response => response));
  }

  getDistinctOS() {
    return this.authHttp
      .get(`${this.softwaresUrl}/oslist`)
      .pipe(map(response => response));
  }

  importSoftwareList() {
    return this.authHttp
      .get(`${this.softwaresUrl}/import`)
      .pipe(map(response => response));
  }

  removeObseleteVersions() {
    return this.authHttp
      .get(`${this.softwaresUrl}/remove-obselete-versions`)
      .pipe(map(response => response));
  }

  addSoftware(software, editing) {
    if (!editing) {
      return this.authHttp
        .post(`${this.softwaresUrl}`, { software })
        .pipe(map(response => response));
    } else {
      return this.authHttp
        .put(`${this.softwaresUrl}/${editing}`, { software })
        .pipe(map(response => response));
    }
  }

  // uploadCsvFile(fileData) {
  //   return this.authHttp
  //     .post(`${this.softwaresUrl}/csv/upload`, { fileData: fileData })
  //     .map(response => response);
  // }

  // downloadCsv() {
  //   this.authShortlivedCookie.setFor(5);
  //   window.location.href = `${this.softwaresUrl}/csv/download`;
  // }

  bulkActions(selectAll = false, selectedIds, searchWord, os) {
    return this.authHttp.put(`${this.softwaresUrl}/bulk-actions`, {
      selectAll,
      selectedIds: Array.from(selectedIds),
      searchWord,
      os
    });
  }
  
  deleteCurrentSoftwareMasteList(selectedIds) {
    return this.authHttp.put(`${this.softwaresUrl}/bulk-actions-delete-current`, {
      selectedIds: Array.from(selectedIds)
    });
  }
}
