import { EventEmitter, Injectable, Renderer2, RendererFactory2 } from '@angular/core';
import { BehaviorSubject } from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class ThemeService {
  private _isDarkMode: any = new BehaviorSubject<any>(this.loadInitialValue());
  isDarkMode$ = this._isDarkMode.asObservable();
  darkModeToggled = new EventEmitter<boolean>();

  private loadInitialValue(): boolean {
    const storedValue = localStorage.getItem('darkMode');
    try {
      return storedValue ? JSON.parse(storedValue) : false;
    } catch (error) {
      console.error('Error parsing stored value:', error);
      return false;
    }
  }

  toggleDarkMode(value: any) {
    this._isDarkMode.next(value);
    this.saveToLocalStorage();
    this.darkModeToggled.emit(value);
  }

  getThemeClass() {
    return this._isDarkMode.value;
  }

  private saveToLocalStorage() {
    localStorage.setItem('darkMode', JSON.stringify(this._isDarkMode.value));
  }
}
