import {
  Component,
  AfterViewInit,
  EventEmitter,
  Output,
  NgZone
} from '@angular/core';
import { NavigationEnd, Router } from '@angular/router';
import { BreakpointObserver, Breakpoints } from '@angular/cdk/layout';
import { AuthenticationService } from '../login/authentication.service';
import { AlertServices } from '../alert-list/alerts.service';
import { Observable, Subscription, interval, timer } from 'rxjs';
import { first, tap } from 'rxjs/operators';
import { RoleEnum } from '../shared/emuns/global.enum';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { io, Socket } from 'socket.io-client';
import { ConfigService } from '../services/config.service';
import { ThemeService } from '../theme.service';
import { RolesAndPermissionService } from '../roles-and-permission/roles-and-permission.service';
import { CategoryService } from './nav.service';
import { SettingsService } from '../settings/settings.service';
import { UserService } from '../users/user.service';
@Component({
  selector: 'top-bar',
  templateUrl: './topbar.component.html',
  styleUrls: ['./topbar.component.css']
})
export class TopbarComponent {
  public alert: GlobalMessageClass;
  title = 'SDP';
  showNavBar: boolean = false;
  public userName: string = '';
  public darkMode: boolean = false;
  public userrole: string = '';
  public littleShort: boolean = false;
  public mobileView: boolean = false;
  public hideNavbar: boolean = false;
  public alertCount: number = 0;
  public totalAlertCount: number = 0;
  public alerts: any = [];
  public alertLastId: number = null;
  showLicenseOnly: boolean = false;
  private screenSubscription: Subscription;
  @Output() screenStatus = new EventEmitter<{
    hide: boolean;
    mini: boolean;
    mobileView: boolean;
  }>();

  // Subscription of the observer of the screen size
  mediaQuery$: Subscription;

  // The active media query (xs | sm | md | lg | xl)
  activeMediaQuery: string;

  assignedUserTemplates: any;
  tempId: number;
  adminUser: boolean;

  Role = RoleEnum;
  socket: Socket;
  notifications = [];
  newNotficationCount: number;
  baseUrl: any;

  searchCategories: string[] = [];

  categoryRoutes = {
    Dashboard: '/dashboard',
    'All Reports': '/reports',
    'Devices Reports': '/devices_reports',
    'Services Reports': '/services_reports',
    'Policy Reports': '/policy_reports',
    'Users Reports': '/users_reports',
    'Onboarding Reports': '/onboarding_reports',
    'Infrastructure Reports': '/infrastructure_reports',
    Alerts: '/alerts',
    Entitlements: '/access-policy',
    'User Entitlements': '/user-agent-access-policy',
    'Device Entitlements': '/device-access-policy',
    'Client Access Logs': '/clients-access-logs',
    'Security Policy': '/device-policy',
    'OS Policy': '/os-policy',
    'Software Policy': '/software-policy',
    'Process Policy': '/software-process-policy',
    'Data Volume Policy': '/data-volume-policy',
    'Routing Policy': '/routing-policy',
    Services: '/services',
    'Service Groups': '/serviceGroups',
    Controllers: '/controllers',
    Gateways: '/gateways',
    'Gateways Groups': '/gatewayGroups',
    'LDAP Configuration': '/ldaps',
    'IDP Management': '/idp',
    'Server Bridges': '/server-bridge',
    'Device Bridge (x86)': '/client-bridge',
    'Device Bridge (ARM)': '/iot-bridge',
    'RDI Bridges': '/rdi-bridge',
    'RBI Bridges': '/rbi-bridge',
    'SaaS Bridges': '/saas-bridge',
    'Admin Users': '/admin',
    'Roles And Permissions': '/view_roles_and_permissions',
    Users: '/users',
    'User Groups': '/customUserGroups',
    Devices: '/devices',
    'Workload Devices': '/workload-devices',
    Microsegments: '/micro-segments',
    Workloads: '/workloads',
    'End User Clients': '/clients/Client',
    'Server Side Client': '/clients/Server',
    Groups: '/groups',
    Organizations: '/organizations',
    'Software Master List': '/softwares',
    'Unauthorized Software': '/forbidden_softwares',
    'Process Binding': '/software_process_binding',
    Client: '/clients-access-logs',
    Admin: '/admin-access-logs',
    'Portal Settings': '/settings/portalSetting',
    'Client Settings': '/settings/clientSetting',
    'Core Settings': '/settings/coreSetting',
    'License Management': '/license',
    'System Backup': '/backups',
    'Software Update': '/updates'
  };
  selectedSearchCategory: any;

  constructor(
    private router: Router,
    private authenticationManager: AuthenticationService,
    private alertService: AlertServices,
    private breakpointObserver: BreakpointObserver,
    private globalMessageService: GlobalMessageService,
    private configService: ConfigService,
    private themesService: ThemeService,
    private rolesPermissionService: RolesAndPermissionService,
    private categoryService: CategoryService,
    private zone: NgZone,
    private settingsService: SettingsService,
    private readonly userService: UserService
  ) {
    const url = configService.getConfiguration().apiBaseUrl;
    this.baseUrl = url.split('api')[0];

    this.userrole = localStorage.getItem('ur');
    if (typeof this.userrole != 'undefined' && this.userrole != null) {
      this.userrole = atob(this.userrole);
    }
    var currentUser = JSON.parse(localStorage.getItem('currentUser'));
    if (typeof currentUser != 'undefined' && currentUser != null) {
      this.userName = currentUser.userFirstName;
    }

    this.authenticationManager.showLicenseOnlyEmitter.subscribe(mode => {
      this.showLicenseOnly = mode;
    });

    this.authenticationManager.showNavBarEmitter
      .pipe(
        tap(mode => {
          var currentUser = JSON.parse(localStorage.getItem('currentUser'));
          if (typeof currentUser != 'undefined' && currentUser != null) {
            if (mode !== null || currentUser.token) {
              this.showNavBar = true;
            }
          }
          this.findLittleShort();
        })
      )
      .subscribe({
        next: () => {
          var currentUser = JSON.parse(localStorage.getItem('currentUser'));
          if (typeof currentUser == 'undefined' || currentUser == null) {
            this.showNavBar = false;
          }
        },
        error: err => {
          console.log(`Oops... ${err}`);
        },
        complete: () => {}
      });
    this.authenticationManager.showLogOutEmitter.subscribe(mode => {
      //var token = localStorage.getItem('id_token');
      if (mode) {
        this.showNavBar = false;
      }
    });

    this.authenticationManager.showFirstNameEmitter.subscribe(mode => {
      let current_user = JSON.parse(localStorage.getItem('currentUser'));
      const roles = JSON.parse(localStorage.getItem('Roles'));
      if (mode !== null || current_user) {
        this.userName = mode || current_user.userFirstName;
        if (Array.isArray(roles)) {
          this.tempId = roles[0]?.temp_id;
          this.adminUser = roles[0]?.temp_id === 1 || roles[0]?.temp_id === 2;
          const privilegeNames = roles.map(role => role.privilege_name);
          this.searchCategories = privilegeNames.filter(privilegeName =>
            this.categoryRoutes.hasOwnProperty(privilegeName)
          );
          this.themesService.isDarkMode$.subscribe((res: any) => {
            console.log('Here We Goo 2',res);
            this.darkMode = res;
          });
          this.socketConnection();
        } else {
          timer(1000).subscribe(() => {
            const retryRoles = JSON.parse(localStorage.getItem('Roles'));
            if (Array.isArray(retryRoles)) {
              this.tempId = retryRoles[0]?.temp_id;
              this.adminUser =
                retryRoles[0]?.temp_id === 1 || retryRoles[0]?.temp_id === 2;
              const privilegeNames = retryRoles.map(
                role => role.privilege_name
              );
              this.searchCategories = privilegeNames.filter(privilegeName =>
                this.categoryRoutes.hasOwnProperty(privilegeName)
              );
              this.themesService.isDarkMode$.subscribe((res: any) => {
                console.log('Here We Goo 2',res);
                this.darkMode = res;
              });

              this.socketConnection();
            }
          });
        }
      }
    });

    this.findLittleShort();
  }

  public findLittleShort() {
    let isLittleShort = JSON.parse(localStorage.getItem('littleShort'));
    if (isLittleShort) {
      this.littleShort = true;
    } else {
      if (!isLittleShort) {
        this.littleShort = false;
      }
    }
  }
  ngOnInit() {
    this.alertService.getNotificationPurge().subscribe((res: boolean) => {
      if (res === true) {
        this.alertCount = 0;
        this.totalAlertCount = 0;
      }
    });

    // this.refreshData();
    this.screenStatus.emit({
      hide: this.mobileView,
      mini: this.littleShort,
      mobileView: this.mobileView
    });
  }

  socketConnection() {
    if (this.socket && this.socket.connected) {
      console.log('Socket Already Connected');
    } else {
      this.socket = io(this.baseUrl);
      this.socket.on('connect', () => {
        console.log('New Socket Connection');
      });
    }

    if (this.tempId) {
      this.socket.emit('client_connected', { value: this.tempId });

      this.socket.on('msg_to_client', message => {
        this.notifications.push(message);
        if (message) this.getAlerts();
      });

      this.socket.on('permissions', (message, id) => {
        if (this.tempId === id) {
          this.notifications.push(message);
          if (message) this.getAlerts();
        }
      });
    }
  }

  getAlerts() {
    this.alerts = this.notifications[0];
    let flattenedData = this.notifications.reduce(
      (acc, curr) => acc.concat(curr),
      []
    );

    flattenedData.forEach(notification => {
      if (!this.alerts.some(alert => alert.id === notification.id)) {
        this.alerts.push(notification);
      }
    });

    let totalCount = this.alerts.length;
    this.alertCount = totalCount > 9 ? '9+' : totalCount;
    this.totalAlertCount = totalCount;
  }

  ngAfterViewInit(): void {
    this.screenSubscription = this.breakpointObserver
      .observe([Breakpoints.XSmall, Breakpoints.Small])
      .subscribe(result => {
        if (result.matches) {
          this.mobileView = true;
          this.hideNavbar = true;
        } else {
          this.mobileView = false;
          this.hideNavbar = false;
        }
        this.screenStatus.emit({
          hide: this.hideNavbar,
          mini: this.littleShort,
          mobileView: this.mobileView
        });
      });
  }

  openMobileViewNavigation() {
    this.hideNavbar = !this.hideNavbar;
    this.screenStatus.emit({
      hide: this.hideNavbar,
      mini: this.littleShort,
      mobileView: this.mobileView
    });
  }

  navbarStatusChange(mini: boolean) {
    this.littleShort = mini;
    this.screenStatus.emit({
      hide: this.hideNavbar,
      mini: this.littleShort,
      mobileView: this.mobileView
    });
  }

  public onAlertClick(): void {
    setTimeout(() => {
      this.alertCount = 0;
    }, 1000);
    setTimeout(() => {
      this.totalAlertCount = 0
    }, 5000);
    this.notifications = [];
  }

  onSelect(): void {
    let currentAdminUserId = localStorage.usd;
    if (
      typeof currentAdminUserId != 'undefined' ||
      currentAdminUserId != null
    ) {
      currentAdminUserId = atob(currentAdminUserId);
    }
    if (currentAdminUserId !== null && currentAdminUserId !== undefined) {
      this.router.navigate(['/admin', currentAdminUserId]);
    } else {
      this.alert = {
        closed: false,
        type: 'danger',
        message: 'Incorrect value given'
      };
      this.globalMessageService.pushAlert(this.alert);
    }
  }

  onSearchCategoryChange(selectedItem: string) {
    const roles = JSON.parse(localStorage.getItem('Roles'));

    this.categoryService.setSelectedCategory(selectedItem);
    if (this.categoryRoutes[selectedItem]) {
      this.router.navigate([this.categoryRoutes[selectedItem]]);
    }
  }

  darkModeToggle() {
    console.log('Dark Mode WAs this', this.darkMode);
    this.darkMode = !this.darkMode;
    console.log('this.darkMode', this.darkMode);
    this.zone.run(() => {
      this.themesService.toggleDarkMode(this.darkMode);
    });
    this.settingsService.userSettingsLoaded.emit();
    this.userService
      .updateUserTheme(this.darkMode)
      .pipe()
      .subscribe({
        next: res => {
          console.log(res);
        },
        error: err => {
          console.log(err);
        },
        complete: () => {}
      });
  }
}
