import { Injectable } from '@angular/core';
import { Observable } from 'rxjs';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { ConfigService } from '../services/config.service';
import { TrustedHost } from '../models/trusted-host';
import { map } from 'rxjs/operators';

@Injectable()
export class TrustedHostsService {
  private apiBaseUrl: string;
  private settingTrustedHostApiUrl = '/trusted-hosts';

  constructor(
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.settingTrustedHostApiUrl =
      this.apiBaseUrl + this.settingTrustedHostApiUrl;
  }

  getPage(
    page: number = 1,
    pageSize: number = 10,
    parentId: number,
    parentType: string,
    sort: string = 'createdAt',
    dir: string = 'desc'
  ): Observable<any> {
    let query = `?limit=${pageSize}&offset=${page -
      1}&parentId=${parentId}&parentType=${parentType}&sort=${sort}&dir=${dir}`;
    return this.authHttp
      .get(this.settingTrustedHostApiUrl + query)
      .pipe(map((response: Response) => response));
  }

  addHost(
    host: TrustedHost,
    parentId: number,
    parentType: string
  ): Observable<any> {
    const parameters = `/${parentType}/${parentId}`;

    return this.authHttp
      .post(this.settingTrustedHostApiUrl + parameters, host)
      .pipe(map((response: Response) => response));
  }

  editHost(
    id: number | string,
    host: TrustedHost,
    parentId: number,
    parentType: string
  ): Observable<any> {
    const parameters = `/${parentType}/${parentId}/${id}`;

    return this.authHttp
      .put(this.settingTrustedHostApiUrl + parameters, host)
      .pipe(map((response: Response) => response));
  }

  deleteHost(
    id: number | string,
    parentId: number,
    parentType: string,
    host: string, 
    port: string, 
    protocol: string
  ): Observable<any> {
    const parameters = `/${parentType}/${parentId}/${id}/${host}/${port}/${protocol}`;

    return this.authHttp
      .delete(this.settingTrustedHostApiUrl + parameters)
      .pipe(map((response: Response) => response));
  }

  deleteTrustedHost(address, port, protocol, parentId){
    const parameters = `/${address}/${port}/${protocol}/${parentId}`;

    return this.authHttp
      .delete(this.settingTrustedHostApiUrl + parameters)
      .pipe(map(response => response));
  }

  searchHost(
    searchTerm: string,
    parentType,
    parentId,
    pageSize: number = 10,
    page: number = 1,
    sort: string = 'createdAt',
    dir: string = 'desc'
  ): Observable<any> {
    const base64Encoded = encodeURIComponent(searchTerm);
    let query = `?limit=${pageSize}&offset=${page -
      1}&parentId=${parentId}&parentType=${parentType}&sort=${sort}&dir=${dir}&searchTerm=${base64Encoded}`;
    return this.authHttp
      .get(`${this.settingTrustedHostApiUrl + query}`)
      .pipe(map(response => response));
  }
}
