import { Injectable } from '@angular/core';
import { ConfigService } from '../services/config.service';
import { HttpClient as AuthHttp, HttpHeaders } from '@angular/common/http';
import { AuthenticationService } from '../login/authentication.service';
import { Observable } from 'rxjs';
import { Update } from '../models/update';
import { LoginResponse } from '../models/login_response';
import { AuthShortlivedCookieService } from '../auth-shortlived-cookie.service';
import { map } from 'rxjs/operators';

@Injectable()
export class UpdateService {
  private apiBaseUrl: string;
  private updatesApiUrl = '/updates';
  httpOptions = {
    headers: new HttpHeaders({ 'Content-Type': 'application/json' })
  };
  constructor(
    private authenticationService: AuthenticationService,
    private authHttp: AuthHttp,
    private configService: ConfigService,
    private authShortlivedCookie: AuthShortlivedCookieService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.updatesApiUrl = this.apiBaseUrl + this.updatesApiUrl;
  }

  /** GET users from the server */

  getUpdates(
    page: number = 1,
    pageSize: number = 10,
    sort: string,
    dir: string,
    searchTerm = ''
  ): any {
    let query = `?limit=${pageSize}&offset=${page -
      1}&sort=${sort}&dir=${dir}&searchTerm=${searchTerm}`;
    return this.authHttp
      .get(this.updatesApiUrl + query)
      .pipe(map(response => response));
  }

  addEditUpdate(
    update: Update,
    file,
    sigFileData
  ): Observable<LoginResponse | any> {
    let queryString = '';
    if (update.id != 0 && typeof update.id != 'undefined') {
      queryString = '/' + update.id;
      return this.authHttp
        .put(
          this.updatesApiUrl + queryString,
          { update: update, attachment: file, sigFileData },
          this.httpOptions
        )
        .pipe(map(this.extractData));
      // .catch(this.handleError);
    } else {
      return this.authHttp
        .post(
          this.updatesApiUrl,
          { update: update, attachment: file, sigFileData },
          this.httpOptions
        )
        .pipe(map(this.extractData));
      //.catch(this.handleError);
    }
  }

  getUpdateDetail(id): Observable<Update | any> {
    var queryString = '';

    if (id != '') {
      queryString = '/' + id;
    }
    return this.authHttp
      .get(this.updatesApiUrl + queryString)
      .pipe(map((response: Response) => response));
  }

  downloadPackage(update) {
    const { buildComponent, buildOs } = update;

    this.authShortlivedCookie.setFor(5);

    window.location.href =
      this.updatesApiUrl + `/download/${buildComponent}/${buildOs}`;
  }

  deleteUpdate(id: number): any {
    var queryString = '';

    if (id > 0) {
      queryString = '/' + id;
    }
    return this.authHttp
      .delete(this.updatesApiUrl + queryString)
      .pipe(map(this.extractData));
    // .catch(this.handleError);
  }

  private extractData(res: Response) {
    let body = res;
    return body || {};
  }

  // private handleError(error: Response | any) {
  //   // In a real world app, we might use a remote logging infrastructure
  //   let errMsg: string;
  //   if (error) {
  //     const body = error || '';
  //     const err = body.error || JSON.stringify(body);
  //     errMsg = `${error.status} - ${error.statusText || ''} ${err}`;
  //   } else {
  //     errMsg = error.message ? error.message : error.toString();
  //   }
  //   console.error(errMsg);
  //   return Observable.throw(errMsg);
  // }
}
