import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp, HttpParams } from '@angular/common/http';
import { ConfigService } from '../services/config.service';
import * as moment from 'moment';
import { AuthShortlivedCookieService } from '../auth-shortlived-cookie.service';
import { Pagination } from '../models/pagination';
import { map } from 'rxjs/operators';
import { Observable } from 'rxjs';

interface date {
  year: String;
  month: String;
  day: string;
}
@Injectable()
export class UsersReportsService {
  private apiBaseUrl: string;
  private reportingApiUrl: string = '/reports';
  private securityQuestionsApiUrl = '/securityQuestions';

  constructor(
    private authHttp: AuthHttp,
    private congigService: ConfigService,
    private authShortlivedCookie: AuthShortlivedCookieService
  ) {
    this.apiBaseUrl = this.congigService.getConfiguration().apiBaseUrl;
    this.reportingApiUrl = this.apiBaseUrl + this.reportingApiUrl;
    this.securityQuestionsApiUrl =
      this.apiBaseUrl + this.securityQuestionsApiUrl;
  }

  getUserReports(pagination: Pagination, dir: string, sort: string, searchTerm = ''): any {
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;

    return this.authHttp
      .get(
        this.reportingApiUrl +
          '/users/reports?offset=' +
          currentPage +
          '&limit=' +
          itemsPerPage +
          '&dir=' +
          dir +
          '&sort=' +
          sort +
          '&searchTerm=' +
          searchTerm
      )
      .pipe(map((response: Response) => response));
  }

  getSessionReport(userId: number, pagination: Pagination, dir: string, sort: string, searchTerm = ''): any {
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;

    const params = new HttpParams()
      .set('offset', currentPage.toString())
      .set('limit', itemsPerPage.toString())
      .set('dir', dir)
      .set('sort', sort)
      .set('userId', userId.toString())
      .set('searchTerm', searchTerm.toString());
  
    return this.authHttp.get(
      this.reportingApiUrl + '/users/SessionReports',
      { params }
    ).pipe(map((response: Response) => response));
  }

  getSessionFailedReports(userId: number, pagination: Pagination, dir: string, sort: string, searchTerm = ''): any {
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;

    const params = new HttpParams()
      .set('offset', currentPage.toString())
      .set('limit', itemsPerPage.toString())
      .set('dir', dir)
      .set('sort', sort)
      .set('userId', userId.toString())
      .set('searchTerm', searchTerm.toString());

      return this.authHttp.get(
        this.reportingApiUrl + '/users-Session-authentication-failed/reports',
        { params }
      ).pipe(map((response: Response) => response));

  }

  getUserAuthenticationFailedReports(
    pagination: Pagination,
    dir: string,
    sort: string,
    searchTerm = ''
  ): any {
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;

    return this.authHttp
      .get(
        this.reportingApiUrl +
          '/users-authentication-failed/reports?offset=' +
          currentPage +
          '&limit=' +
          itemsPerPage +
          '&dir=' +
          dir +
          '&sort=' +
          sort +
          '&searchTerm=' +
          searchTerm
      )
      .pipe(map((response: Response) => response));
  }

  getSecurityQuestions(): Observable<any> {
    return this.authHttp
      .get(this.securityQuestionsApiUrl)
      .pipe(map((response: Response) => response));
  }

  // exportCSV(){
  //   this.authShortlivedCookie.setFor(5);
  //   window.location.href = `${this.reportingApiUrl}/devices/export-csv`;
  // }
  getServiceToUsersReport(pagination: Pagination, searchTerm='', clients=null, serviceType='',dateFilter=[]):any{
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;
    let _clients='';
    let _from:any='';
    let _to:any='';
    if (clients && clients.length) {
      _clients = JSON.stringify(clients.map(v => v.agent_id));
    }
    if(dateFilter&&dateFilter.length){

      _from=Math.floor(new Date(dateFilter[0]).getTime() / 1000);
      _to=Math.floor(new Date(dateFilter[1]).getTime() / 1000);

    }
    return this.authHttp
    .get(
      this.reportingApiUrl +
        '/services-accessed-by-users/reports?offset=' +
        currentPage +
        '&limit=' +
        itemsPerPage+
        '&searchTerm='+
        searchTerm+
        '&clientId=' +
        _clients+
        '&serviceType='+
        serviceType+
        '&from='+
        _from+
        '&to='+
        _to
    )
    .pipe(map((response: Response) => response));
  }
}
