import {
  Component,
  OnDestroy,
  OnInit,
  ViewContainerRef,
  AfterViewInit,
  ViewChild,
  ElementRef,
  ChangeDetectorRef,
  NgZone
} from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Subscription } from 'rxjs';
// import { Modal } from 'angular2-modal/plugins/bootstrap';
import { BsModalService, BsModalRef } from 'ngx-bootstrap/modal';
import { UserService } from '../user.service';
import { User } from '../../models/user';
import { Role } from '../../models/role';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { Popup } from '../../Popup';
import { AuthenticationService } from '../../login/authentication.service';
import { ModalComponent } from '../../shared/modal/modal.component';
import * as sharedContent from '../../shared/shared-content.constant';
import { KeyboardKeyCodeEnum } from '../../shared/emuns/global.enum';
import { ContextHelpEnum } from '../../shared/emuns/global.enum';
import { Title } from '@angular/platform-browser';
import { ThemeService } from '../../theme.service';
import { SettingsService } from '../../settings/settings.service';
import { RolesAndPermissionService } from '../../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-create-user',
  templateUrl: './create-user.component.html',
  styleUrls: ['./create-user.component.css']
})
export class CreateUserComponent implements OnInit, AfterViewInit {
  getAuth: Popup = new Popup();
  public alert: GlobalMessageClass;
  user = new User();
  roles: Role[];
  current_user: any;
  private roleSubscription: Subscription;
  private selecteRole: any = '';
  private selectedTemplate: any = '';
  public pageTitle: string = 'Add Admin';
  confirmPassword = '';
  currentUserName: string = '';
  authPassword: string = '';
  failedPassword: number = 0;
  reSubmit: boolean = false;
  showAuthError: boolean = false;
  userID: string = '';
  isEdit: boolean = false;
  loading: Boolean = false;
  OAuthEmailValid = true;
  viewMode: boolean = false;
  questions: any = [{ value: 1, display: '' }];
  templates: any = [];
  templatesData: any = [];

  //Contextual Help
  public contextualHelp: Array<any>;
  public showContext: boolean = false;
  //Contextual Help
  public dateFormat: string = 'dd/MM/yyyy HH:mm';
  public userrole: string = '';

  @ViewChild('authpassword', { static: false }) _authpassword: ElementRef;
  constructor(
    private vcRef: ViewContainerRef,
    private modalService: BsModalService,
    private router: Router,
    private route: ActivatedRoute,
    private userService: UserService,
    private authenticationService: AuthenticationService,
    private globalMessageService: GlobalMessageService,
    private rolesPermissionService: RolesAndPermissionService,
    private cdref: ChangeDetectorRef,
    private titleService: Title,
    private themesService : ThemeService,
    private zone: NgZone,
    private settingsService: SettingsService,
  ) {
    this.userrole = localStorage.getItem('ur');
    if (typeof this.userrole != 'undefined' || this.userrole != null) {
      this.userrole = atob(this.userrole);
    }
    //overlay.defaultViewContainer = vcRef;
    this.contextualHelp = ['Name', 'Test'];
    var localDateFormat = localStorage.getItem('dateFormat');
    if (typeof localDateFormat != 'undefined' || localDateFormat != null) {
      this.dateFormat = localDateFormat;
    }
  }

  showContextHelp(name: any): void {
    if (name == ContextHelpEnum.role) {
      this.contextualHelp = ['role', 'Select desired role of user.'];
      this.showContext = true;
    }
  }

  closeContextualHelp(): void {
    this.showContext = false;
  }

  showPurgePopup() {
    this.getAuth.heading = 'Verification required';
    this.getAuth.body = 'Please enter your current password for verification.';
    this.getAuth.action = 'Confirm';
    this.getAuth.show = true;
  }

  cancelPopup() {
    this.getAuth.show = false;
  }

  ngAfterViewInit() {
    this.selecteRole = this.roles[1].id;
    this.cdref.detectChanges();
    // if (!this.isEdit) {
    //   var select = document.getElementById('role_select');
    //   select.getElementsByTagName('option')[0].selected = true;
    // }
  }

  ngOnInit() {
    this.roles = this.route.snapshot.data['roles'];
    this.rolesPermissionService.getAllTemplates().subscribe((res: any) => {
      this.templates = res.map(template => template.name);
      this.templatesData = res;
    });
    // Discard or Disable 'Public-Api' role for temporary purpose
    this.populateCurrentUserName();
    this.userService.getSecurityQuestions().subscribe(questions => {
      this.questions = questions.map(question => {
        return {
          value: question.id,
          display: question.question
        };
      });
    });
    if (typeof this.route.params['_value']['id'] != 'undefined') {
      this.loading = true;
      this.isEdit = true;
      this.pageTitle = 'Edit Admin';
      this.userService
        .getuserdetail(this.route.params['_value']['id'])
        .subscribe(
          user => {
            this.loading = false;

            if (!user) {
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'No user found with the specified ID.'
              };
              this.globalMessageService.pushAlert(this.alert);
              this.router.navigate(['/users']);
            } else {
              this.user = user;
              if (this.user.id != parseInt(this.userID)) {
                this.alert = {
                  closed: false,
                  type: 'danger',
                  message: 'You cannot edit other users.'
                };
                this.globalMessageService.pushAlert(this.alert);
                this.router.navigate(['/users']);
              }
              this.user.password = '';
              this.user.answer_1 = '';
              this.user.answer_2 = '';
              this.user.answer_3 = '';
  
              if (this.user?.Roles.length > 0) {
                this.selecteRole = user?.Roles[0].id;
              }
            }
          },
          err => {
            this.loading = false;
          }
        );
    }
    this.titleService.setTitle('IAM | Admin User | ' + this.pageTitle);
  }

  submitMyAlert(event) {
    if (event.keyCode == KeyboardKeyCodeEnum.Enter) {
      if (!this.reSubmit && this.authPassword.length != 0) {
        this.authAction();
      }
    }
  }

  emailWithoutDomainHandler(event) {
    const isOauthService = JSON.parse(localStorage.getItem('isOauthService'))
    if(isOauthService){
      if (event.includes('+')) {
        this.OAuthEmailValid = false;
      } else {
        this.OAuthEmailValid = true;
      }
    }
  }

  private populateCurrentUserName() {
    this.current_user = JSON.parse(localStorage.getItem('currentUser'));
    if (this.current_user) {
      this.currentUserName = this.current_user.userName;
      this.userID = localStorage.getItem('usd');
      if (typeof this.userID != 'undefined' || this.userID != null) {
        this.userID = atob(this.userID);
      }
    } else {
      this.currentUserName = '';
    }
  }

  onClick() {
    this.modalService.show(ModalComponent, {
      initialState: sharedContent.passwordSuggestion,
      class: 'modal-dialog-centered'
    });
  }

  authAction() {
    this.reSubmit = true;
    this.submitForm(this.authPassword, this.currentUserName);
  }

  hideAuthError() {
    this.showAuthError = false;
  }

  darkModeToggle(){
  
    this.user.is_dark = !this.user.is_dark;
    // this.setting.isDark = !this.setting.
  }

  getUsertemplate() {
    this.user.tempId = this.templatesData.filter(
      template => this.selectedTemplate === template?.name
    )[0]?.temp_id;
  }

  submitForm(password, username = null) {
    this.loading = true;
    this.user.roles = 2;
    this.getUsertemplate();
    if (this.isEdit) {
      this.current_user = JSON.parse(localStorage.getItem('currentUser'));
      this.current_user.userName = this.user.email;
      localStorage.setItem('currentUser', JSON.stringify(this.current_user));
    }
    this.userService.add_user(this.user, password).subscribe(
      (response: any) => {
        this.loading = false;
        this.authPassword = '';
        if (response.success) {
          this.reSubmit = false;
          
          this.alert = {
            closed: false,
            type: 'success',
            message: 'User added/updated successfully.'
          };
          if (this.isEdit) {
            this.authenticationService.updateToken(response.newToken);
          }
          this.globalMessageService.pushAlert(this.alert);
          // localStorage.setItem('darkMode' , JSON.stringify(this.user.is_dark))
          // this.zone.run(() => {
          //   this.themesService.toggleDarkMode.bind(this.themesService)(this.user.is_dark);
          // });
          this.settingsService.userSettingsLoaded.emit();
          this.router.navigate(['/admin']);
        } else {
          this.reSubmit = false;
          this.getAuth.show = false;
          this.alert = {
            closed: false,
            type: 'danger',
            message: response.message
          };
          this.globalMessageService.pushAlert(this.alert);
        }
      },
      err => {
        this.loading = false;
        // const alert = {
        //   closed: false,
        //   type: 'danger',
        //   message: 'Invalid password provided'
        // };
        // this.globalMessageService.pushAlert(alert);
        this.reSubmit = false;
        this.authPassword = '';
      }
    );
  }

  onsubmit() {
    this.showPurgePopup();
  }

  checkEmail() {
    this.userService.checkEmail(this.user.email).subscribe(response => {
      if (response.success) {
      } else {
      }
    });
  }

  onCancel(): void {
    this.router.navigate(['/admin']);
  }

  focus() {
    this._authpassword.nativeElement.focus();
  }

  questionChange(e) {}
}
