import {
  Component,
  OnInit,
  ElementRef,
  ViewChild,
  AfterViewInit
} from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { UserService } from '../user.service';
import { User } from '../../models/user';
import { Pagination } from '../../models/pagination';
import { PreserveSearch } from '../../models/preserve-search';
import * as _ from 'lodash';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { Popup } from '../../Popup';
import {
  BehaviorSubject,
  Observable,
  Subject,
  debounceTime,
  distinctUntilChanged,
  from,
  map,
  startWith,
  switchMap
} from 'rxjs';
import { IdentityProvierEnum, ModeEnum } from '../../shared/emuns/user.enum';
import { RoleEnum } from '../../shared/emuns/global.enum';
import { Title } from '@angular/platform-browser';
import { LocalUserService } from '../../create-local-user/local-user.service';
import { MatDialog } from '@angular/material/dialog';
import { ConfirmModalComponent } from '../../shared/confirm-modal.component';
import { FormControl, NgForm } from '@angular/forms';
import { LocalUser } from '../../models/LocalUser';

@Component({
  selector: 'app-user-edit-list',
  templateUrl: './user-groups-edit.component.html',
  styleUrls: ['./user-groups-edit.component.css']
})
export class UserGroupsEditComponent implements OnInit, AfterViewInit {
  showHelpText: boolean = false;
  getAuth: Popup = new Popup();
  getLock: Popup = new Popup();
  getUnLock: Popup = new Popup();
  getResetMfa: Popup = new Popup();
  public alert: GlobalMessageClass;
  users: any = {};
  private usersList = new Array();
  private selectedUser: User;
  pagination: Pagination = new Pagination();
  groupPagination: Pagination = new Pagination();
  userPagination: Pagination = new Pagination();
  localUserPagination: Pagination = new Pagination();
  private Search: PreserveSearch = new PreserveSearch();
  currentUserName: string = '';
  userID: string = '';
  authPassword: string = '';
  userTobeDeleted: LocalUser;
  userTobeRemoved;
  failedPassword: number = 0;
  reSubmit: boolean = false;
  showAuthError: boolean = false;
  userToDelete: User;
  userToUnLock: User;
  userToLock: User;
  userToResetMfa: User;
  public dateFormat: string = 'dd/MM/yyyy HH:mm';
  public userrole: string = '';
  admin_mfa: boolean = false;
  isNameSort: boolean = false;
  userGroups;
  isGroupLoading: boolean = true;

  keysSelectedUserId: string;
  keysLoading: boolean = false;

  pageSize = [10, 25, 50, 100, 200];
  groupSize = [10, 25, 50, 100, 200];
  listView: boolean = true;
  selectedGroups: any = new Set();

  editingId: number;

  editGroupName: string;
  editGroupDesc: string;
  groupId: number;

  allLocalUsers: any = {};

  loading: boolean = false;
  showKeys: boolean = false;

  public_key: string = '';
  secret_key: string = '';

  groupName: string;
  groupDescription: string;

  isLdapSyncing: boolean = false;

  userData;
  searchWord;

  preserveSearch = new PreserveSearch();

  selectedIdentityProvider: string;
  selectedOrganization: string;
  selectedDepartment: string;

  Role = RoleEnum;

  sort: string;
  sortDir: string;
  $searchPolicy = new Subject();
  @ViewChild('authpassword', { static: false }) _authpassword: ElementRef;
  @ViewChild('lockpassword', { static: false }) _lockpassword: ElementRef;
  @ViewChild('unlockpassword', { static: false }) _unlockpassword: ElementRef;
  @ViewChild('resetMfa', { static: false }) _resetMfa: ElementRef;

  selectAll = false;
  localUsers: any = {};
  updatedLocalUsers: any = {};
  selectAllExcept: any = new Set();
  selectedUsers: any = new Set();

  // JQL SEARCH VARS
  private defaultSelection: string = Action.Field;
  private currentEvent: string;
  private response: ApiResponse[] = [];

  $searchUsers = new Subject();
  searchKeyword: string = '';
  navigationToEditUser: boolean = false;
  rawQuery: string = '';

  addToGroupButton: boolean = false;

  @ViewChild('input') inputText!: NgForm;
  @ViewChild('event') searchInputEvent: ElementRef;
  myControl = new FormControl();
  hiddenCtrl = new FormControl();
  lastValue: string;
  currentText: string;
  isExpression: boolean = false;
  $searchUsersJQL = new BehaviorSubject<any>({});

  isAnOptionSelected: boolean = false;
  isSearchExecuted: boolean = false;
  isAlreadyClear: boolean = true;

  adUserSelectedCount: number = 0;
  excludedUserIds: number[] = [];
  excludedUsers: number[] = [];
  id: number;

  isAllUserSelected = true;

  IdentityProvier = IdentityProvierEnum;

  private fieldList: SuggestionDetails = {
    Name: Action.Field,
    Value: [],
    Valid: ['string']
  };
  private operatorList: SuggestionDetails = {
    Name: Action.Operator,
    Value: ['=', '!=', '>', '<', '<=', '>=', '(', ')'],
    Valid: ['string']
  };
  private valueList: SuggestionDetails = {
    Name: Action.Value,
    Value: [],
    Valid: ['string']
  };
  private expressionList: SuggestionDetails = {
    Name: Action.Expression,
    Value: ['And', 'Or'],
    Valid: ['string']
  };

  private operator: string[] = this.operatorList.Value;
  private value: string[] = this.valueList.Value;
  private expression: string[] = this.expressionList.Value;

  private get field(): string[] {
    return this.fieldList.Value;
  }

  filteredOptions: Observable<string[]>;
  private searchList: SelectedOption[] = [];

  private get selectionList(): SelectionDict[] {
    return [
      { Name: Action.Field, Value: this.field, NextSelection: Action.Operator },
      {
        Name: Action.Operator,
        Value: this.operator,
        NextSelection: Action.Value
      },
      {
        Name: Action.Value,
        Value: this.value,
        NextSelection: Action.Expression
      },
      {
        Name: Action.Expression,
        Value: this.expression,
        NextSelection: Action.Field
      }
    ];
  }

  departmentsData: string[] = [];
  organizationsData: string[] = [];
  userNamesData: string[] = [];
  namesData: string[] = [];
  emailsData: string[] = [];
  isDarkMode: boolean;
  constructor(
    private router: Router,
    private userService: UserService,
    private route: ActivatedRoute,
    private globalMessageService: GlobalMessageService,
    private titleService: Title,
    private localUserService: LocalUserService,
    public dialog: MatDialog
  ) {
    this.userrole = localStorage.getItem('ur');
    if (typeof this.userrole != 'undefined' || this.userrole != null) {
      this.userrole = atob(this.userrole);
    }
    var localDateFormat = localStorage.getItem('dateFormat');
    if (typeof localDateFormat != 'undefined' || localDateFormat != null) {
      this.dateFormat = localDateFormat;
    }

    const userGroupsView = localStorage.getItem('userGroupsView');

    if (userGroupsView) {
      this.listView = userGroupsView == 'list';
    }
  }

  nameSorting() {
    this.isNameSort = !this.isNameSort;
  }

  clearfun() {
    this.searchWord = '';
    this.subscribeToJqlSearch();
  }

  changeCheckBox(user) {
    const { isChecked, id, identity_provider } = user;

    if (identity_provider == IdentityProvierEnum.AD) {
      if (isChecked) {
        this.adUserSelectedCount += 1;
      } else {
        this.adUserSelectedCount -= 1;
      }
    }

    if (this.selectAll) {
      if (isChecked) {
        this.selectAllExcept.delete(id);
      } else {
        this.selectAllExcept.add(id);
      }
    } else {
      if (!isChecked) {
        this.selectedUsers.delete(id);
      } else {
        this.selectedUsers.add(id);
      }
    }
  }

  ngOnInit(): void {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.titleService.setTitle('IAM | Users Groups');

    this.route.params.subscribe(params => {
      this.id = params['id'];
      if (this.id) {
        this.initializeSearch();
        this.getEditData(this.id);
      }
    });

    this.subscribeToJqlSearch();
  }

  getEditData(id) {
    this.localUserService
      .getCustomUserGroupData(id)
      .subscribe((response: any) => {
        this.editGroupName = response.group?.group_name;
        this.editGroupDesc = response.group?.group_description;
        this.groupId = parseInt(id);
        this.isAllUserSelected = response.all_selected;
        this.userData = response.data;
        this.excludedUserIds = this.userData.map(user => user.id);
        this.excludedUsers = this.excludedUserIds;
        this.userPagination.totalItems = this.userData.length;
      });
  }

  pageSizeChange(size) {
    if (size == 'All') {
      this.loading = true;
      this.localUserService
        .searchUsersJQL(
          this.rawQuery,
          0,
          this.localUserPagination.currentPage,
          this.sort
        )
        .subscribe(res => {
          this.loading = false;
          this.localUsers = res;
          this.excludedUserIds = [];
          // this.excludedUsers = []
          this.selectedUsers.clear();
        });
    } else {
      this.localUserPagination.itemsPerPage = +size;
      this.localUserPagination.totalItems = 0;
      this.localUserPagination.currentPage = 1;
      this.localUserService
        .searchUsersJQL(
          this.rawQuery,
          this.localUserPagination.itemsPerPage,
          this.localUserPagination.currentPage,
          this.sort,
          '',
          this.groupId
        )
        .subscribe((users: any) => {
          this.localUsers = users;
          this.excludedUserIds = [];
          // this.excludedUsers = []
          this.selectedUsers.clear();
          this.isLdapSyncing =
            users.isLdapSyncing == null ? false : users.isLdapSyncing;
          this.localUserPagination.totalItems = users.recordsTotal;
          this.loading = false;
        });
    }
  }

  pageUserGroupSizeChange() {
    this.loading = true;
    this.localUserService
      .getUserGroupsPagination(
        this.localUserPagination,
        this.sort,
        this.sortDir
      )
      .subscribe(
        (response: any) => {
          this.userGroups = response;
          this.groupPagination.totalItems = response.length;
          this.loading = false;
          this.changeCheckStatusOfAllGroups();
        },
        err => {
          this.loading = false;
        }
      );
  }

  handleDirectionChange(sort) {
    this.sort = sort;
    this.sortDir = this.sortDir == 'desc' ? 'asc' : 'desc';
    this.pageUserGroupSizeChange();
  }

  handleEdit(id) {
    this.router.navigate(['/customUserGroups', id]);
  }

  changeView(view) {
    localStorage.setItem('userGroupsView', view);
    this.listView = view == 'list';
  }

  handleUpdateGroup() {
    let data = {
      rawQuery: this.currentText,
      groupName: this.editGroupName,
      groupDesc: this.editGroupDesc,
      sqlQuery: '',
      selectedIds: [...this.selectedUsers],
      allUser: this.selectAll
    };
    this.localUserService
      .updateUserGroup(data, this.groupId)
      .subscribe(response => {
        this.excludedUsers = [this.excludedUsers, ...this.selectedUsers];
        this.localUserService
          .getCustomUserGroupData(this.groupId)
          .subscribe((response: any) => {
            this.userData = response.data;
            this.userPagination.totalItems = this.userData.length;
            this.selectedUsers.clear();
            data.selectedIds = [];
          });
        this.globalMessageService.pushAlert({
          closed: false,
          type: 'success',
          message: 'User(s) updated to group successfully',
          sticky: false
        });
        this.getEditData(this.id);
      });
    this.clearForm('');
    this.subscribeToJqlSearch();
  }

  sortRecord(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
  }

  viewLocalUser(user) {
    const save = {
      pagination: this.localUserPagination,
      searchKey: this.searchWord,
      selectedIdentityProvider: this.selectedIdentityProvider,
      selectedOrganization: this.selectedOrganization,
      selectedDepartment: this.selectedDepartment
    };
    this.preserveSearch.saveSearch('localUserSearch', save);
    this.router.navigate(['/view-user', user.id]);
  }

  editLocalUser(user) {
    this.preserveCurrentSearch();
    this.router.navigate(['/user', user.id]);
  }

  deleteLocalUser(user) {
    this.userTobeDeleted = user;
    this.localUserService
      .deleteUser(this.userTobeDeleted.id, this.userTobeDeleted.userName, event)
      .subscribe(
        response => {
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'success',
            message: 'User has been deleted successfully.',
            sticky: false
          });
        },
        error => {
          this.loading = false;
        }
      );
  }

  removeLocalUserFromGroup(user) {
    this.userTobeRemoved = user;
    this.localUserService
      .deleteUserFromCustomGoup(this.userTobeRemoved.user_id, this.userTobeRemoved.group_id)
      .subscribe(
        response => {
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'success',
            message: 'User has been removed successfully.',
            sticky: false
          });
          this.getEditData(this.id);
        },
        error => {
          this.loading = false;
        }
      );
      this.subscribeToJqlSearch();
  }

  onSubmit() {
    let data = {
      editGroupName: this.editGroupName,
      editGroupDesc: this.editGroupDesc
    };

    this.localUserService
      .updateCustomGroupData(data, this.groupId)
      .subscribe(response => {
        this.globalMessageService.pushAlert({
          closed: false,
          type: 'success',
          message: 'User group updated successfully',
          sticky: false
        });
        this.router.navigate(['/customUserGroups']);
      });
  }

  preserveCurrentSearch() {
    const save = {
      pagination: this.localUserPagination,
      searchKey: this.searchWord,
      selectedIdentityProvider: this.selectedIdentityProvider,
      selectedOrganization: this.selectedOrganization,
      selectedDepartment: this.selectedDepartment
    };
    this.preserveSearch.saveSearch('localUserSearch', save);
  }

  isNotGWAdmin(user) {
    if (user.userName) {
      return user.userName.indexOf('gwadmin') == -1;
    }
    return true;
  }

  private changeCheckStatusOfAllClients(check) {
    this.localUsers.data = this.localUsers?.data?.map(_user => {
      return {
        ..._user,
        isChecked:
          this.isNotGWAdmin(_user) &&
          ((!this.selectAllExcept.has(_user.id) && this.selectAll) ||
            (this.selectedUsers.has(_user.id) && !this.selectAll))
      };
    });
  }

  pageChange(page) {
    this.localUserPagination.currentPage = page;

    if (this.searchWord) {
      this.subscribeToJqlSearch();
    } else {
      this.localUserService
        .searchUsersJQL(
          '',
          this.localUserPagination.itemsPerPage,
          this.localUserPagination.currentPage,
          this.sort,
          '',
          this.groupId
        )
        .subscribe((users: any) => {
          this.localUsers = users;
          this.excludedUserIds = [];
          // this.excludedUsers = []
          this.selectedUsers.clear();
          this.isLdapSyncing =
            users.isLdapSyncing == null ? false : users.isLdapSyncing;
          this.localUserPagination.totalItems = users.recordsTotal;
          this.loading = false;
        });
    }
  }

  selectAllAction() {
    if (!this.selectAll) {
      this.selectAllExcept.clear();
      this.selectedUsers.clear();
    } else {
      this.selectedUsers.clear();
    }
    this.changeCheckStatusOfAllClients(this.selectAll);
    this.adUserSelectedCount = 0;
    if (this.selectAll) {
      this.localUsers.data.map(_user => {
        if (
          _user.identity_provider == IdentityProvierEnum.AD &&
          _user.isChecked
        ) {
          this.adUserSelectedCount += 1;
        }
      });
    }
  }

  private changeCheckStatusOfAllGroups() {
    this.userGroups = this.userGroups.map(group => {
      return {
        ...group,
        isChecked:
          (!this.selectAllExcept.has(group.id) && this.selectAll) ||
          (this.selectedGroups.has(group.id) && !this.selectAll)
      };
    });
  }

  changeGroupCheckBox(group) {
    const { isChecked, id } = group;
    if (this.selectAll) {
      if (isChecked) {
        this.selectAllExcept.delete(id);
      } else {
        this.selectAllExcept.add(id);
      }
    } else {
      if (!isChecked) {
        this.selectedGroups.delete(id);
      } else {
        this.selectedGroups.add(id);
      }
    }
  }

  handleDelete(id) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Are you sure want to delete?`
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.localUserService.deleteCustomGroupData(id).subscribe(
          response => {
            this.globalMessageService.pushAlert({
              closed: false,
              type: 'success',
              message: 'User group deleted successfully',
              sticky: false
            });
          },
          error => {
            this.globalMessageService.pushAlert({
              closed: false,
              type: 'danger',
              message:
                'Please remove this user group from entitlement(s) first',
              sticky: false
            });
          }
        );
      }
    });
  }

  goBack() {
    this.router.navigate(['/customUserGroups']);
  }

  gotToUserPage() {
    this.router.navigate(['/users']);
  }

  ngAfterViewInit(): void {
    this.searchInputEvent.nativeElement.addEventListener('keydown', event => {
      if (event.key === 'Backspace' && this.inputText.value === '') {
        this.clearForm('');
      }
    });

    this.searchInputEvent.nativeElement.addEventListener('keyup', event => {
      if (event.ctrlKey && event.key.toLowerCase() === 'x') {
        this.clearForm('');
      }
      if (event.key === 'Backspace' && this.inputText.value === '') {
        this.clearForm('');
      }
    });
  }

  initializeSearch() {
    // Declarative Search

    this.localUserService.getCustomGroupData().subscribe((response: any) => {
      this.departmentsData.push(response['departments']);
      this.organizationsData.push(response['organization']);
      this.userNamesData.push(response['userNames']);
      this.emailsData.push(response['emails']);
      this.namesData.push(response['names']);

      this.getSearchObject();
      this.filteredOptions = this.inputText?.valueChanges.pipe(
        startWith(''),
        map(value => this._filter(value))
      );
    });
  }

  clearForm(data) {
    this.searchList = [];
    this.displayFn('');
    if (this.inputText) {
      this.inputText.reset('');
    }
    this.isAlreadyClear = true;
    this.isSearchExecuted = false;
    this.selectedUsers.clear();
  }

  getSearchObject(stop = false): void {
    let response: Observable<any>;
    response = from([
      {
        Result: {
          DisplayName: 'Identity Provider',
          SearchType: 'Field',
          AutoCompleteValues: ['Local User', 'Active Directory']
        }
      },
      {
        Result: {
          DisplayName: 'Name',
          SearchType: 'Field',
          AutoCompleteValues: this.namesData[0]
        }
      },
      {
        Result: {
          DisplayName: 'Email',
          SearchType: 'Field',
          AutoCompleteValues: this.emailsData[0]
        }
      },
      {
        Result: {
          DisplayName: 'Username',
          SearchType: 'Field',
          AutoCompleteValues: this.userNamesData[0]
        }
      },
      {
        Result: {
          DisplayName: 'Organization',
          SearchType: 'Field',
          AutoCompleteValues: this.organizationsData[0]
        }
      },
      {
        Result: {
          DisplayName: 'Department',
          SearchType: 'Field',
          AutoCompleteValues: this.departmentsData[0]
        }
      },
      {
        Result: {
          DisplayName: 'OrderBy',
          SearchType: 'Field',
          AutoCompleteValues: ['Ascending', 'Descending']
        }
      }
    ]);

    response.subscribe(val => {
      this.response.push(val.Result);
      this.fieldList.Value = this.response
        .filter(r => r.SearchType == Action.Field)
        ?.map<string>(r => r.DisplayName.toString());
      this.myControl.setValue(''); // trigger the autocomplete to populate new values
    });
  }

  // autocomplete material ui events
  _filter(value: string): string[] {
    if (value) {
      this.isSearchExecuted = true;
      this.isAlreadyClear = false;
    }
    this.currentText = this.searchList?.map(s => s.Value).join(' ');
    this.getSearchText(value);
    if (!/Ascending|Descending/i.test(this.currentText)) {
      this.lastValue = '';
    }

    if (this.isExpression) {
      this.$searchUsersJQL.next({ searchWord: this.currentText, typing: true });
    } else {
      this.selectedUsers.clear();
    }

    if (!(this.lastValue === 'Ascending' || this.lastValue === 'Descending')) {
      let optionListToBePopulated: string[] = this.getOptionList();
      var searchText = this.getSearchText(value);
      return optionListToBePopulated.filter(
        option =>
          option.toLowerCase().indexOf(searchText.toLowerCase().trim()) != -1
      );
    }
  }

  displayFn(value: string): string {
    if (!!value) {
      if (this.lastValue === 'Ascending' || this.lastValue === 'Descending') {
        this.getSearchObject(true);
      } else {
        this.searchList.push(
          new SelectedOption(
            value,
            this.currentEvent,
            this.getNextEvent(this.currentEvent)
          )
        );
      }
      this.lastValue = value;
    }
    return this.searchList.length > 0
      ? this.searchList?.map(s => s.Value).join(' ')
      : '';
  }

  // private functions
  // ------------- Get Autocomplete List START --------------------
  private getOptionList(): string[] {
    if (
      this.searchList == null ||
      this.searchList == undefined ||
      this.searchList.length === 0
    ) {
      this.currentEvent = this.defaultSelection;
      return this.field;
    }

    let lastElement: SelectedOption = (
      this.searchList.slice(-1).pop()
    );
    let currentList = this.selectionList.find(
      s => s.Name.toLowerCase() === lastElement.Next.toLowerCase()
    );
    this.currentEvent = currentList ? currentList.Name : this.defaultSelection;
    return currentList ? this.getValues(currentList) : this.field;
  }

  private getValues(currentList: SelectionDict): string[] {
    if (currentList.NextSelection == Action.Expression) {
      this.addToGroupButton = false;
    }
    if (this.currentEvent.toLowerCase() != 'value') return currentList.Value;
    var selectedField = this.getlastField();
    var selectedValue = selectedField ? selectedField.Value : '';
    var filteredResponse = this.response.find(
      r => r.DisplayName === selectedValue
    );
    return filteredResponse ? filteredResponse.AutoCompleteValues : [];
  }
  // ------------- Get Autocomplete List END --------------------

  // --------------- START : Get the search text based on which the autocomplete will populate --------
  private getSearchText(value: string): string {
    var oldText = this.searchList?.map(s => s.Value).join(' ');
    this.handleBackspace(value);
    return value.trim().replace(oldText, '');
  }

  private handleBackspace(searchValue: string): void {
    var oldText = this.searchList?.map(s => s.Value).join(' ');
    var previousListName =
      this.searchList.length != 0
        ? this.searchList[this.searchList.length - 1].PopulatedFrom
        : '';
    var prevList = this.selectionList.find(
      s => s.Name.toLowerCase() === previousListName.toLowerCase()
    );
    var prevListValue = prevList ? prevList.Value : [];

    if (previousListName == Action.Value) {
      var lastField = this.getlastField();
      var lastFieldValue = lastField ? lastField.Value : '';
      var filteredResponse = this.response.find(
        r => r.DisplayName === lastFieldValue
      );
      prevListValue = filteredResponse
        ? filteredResponse.AutoCompleteValues
        : [];
    }

    if (
      (prevListValue ? prevListValue.indexOf(searchValue) === -1 : false) &&
      oldText.trim().length > searchValue.trim().length
    )
      this.searchList.pop();
  }

  // --------------- END : Get the search text based on which the autocomplete will populate --------

  private getNextEvent(currentEvent: string): string {
    if (!(this.lastValue === 'Ascending' || this.lastValue === 'Descending')) {
      var currentList = this.selectionList.find(
        s => s.Name.toLowerCase() === currentEvent.toLowerCase()
      );
      if (
        (currentList ? currentList.NextSelection : this.defaultSelection) ==
        Action.Expression
      ) {
        this.isExpression = true;
        this.addToGroupButton = true;
      } else {
        this.isExpression = false;
        this.addToGroupButton = false;
      }
      return currentList ? currentList.NextSelection : this.defaultSelection;
    }
  }

  private getlastField(): SelectedOption | undefined {
    if (this.searchList.length === 0) return undefined;
    let i: number = this.searchList.length - 1;
    for (i; i >= 0; i--) {
      if (this.searchList[i].PopulatedFrom == Action.Field)
        return this.searchList[i];
    }
    return undefined;
  }

  subscribeToJqlSearch() {
    this.$searchUsersJQL
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((search: { searchWord: string; typing: boolean }) => {
          this.clearSelection();
          this.loading = true;

          if (search.typing && !this.rawQuery) {
            this.localUserPagination.currentPage = 1;
          }

          this.rawQuery = search.searchWord;
          return this.localUserService.searchUsersJQL(
            search.searchWord,
            this.localUserPagination.itemsPerPage,
            this.localUserPagination.currentPage,
            this.sort,
            '',
            this.groupId
          );
        })
      )
      .subscribe((users: any) => {
        this.localUsers = users;
        this.excludedUserIds = [];
        // this.excludedUsers = []
        this.selectedUsers.clear();
        this.isLdapSyncing =
          users.isLdapSyncing == null ? false : users.isLdapSyncing;
        this.pagination.totalItems = users.recordsTotal;
        this.localUserPagination.totalItems = users.recordsTotal;
        this.loading = false;

        if (this.isAllUserSelected) {
          this.localUsers = [];
          this.localUserPagination.totalItems = 0;
        }
      });
  }

  clearSelection() {
    this.selectAll = false;
    this.selectAllExcept.clear();
  }
}

class SelectedOption {
  public Value: string;
  public PopulatedFrom: string;
  public Next: string;

  constructor(value: string, populatedFrom: string, next: string) {
    this.Value = value;
    this.PopulatedFrom = populatedFrom;
    this.Next = next;
  }
}

class SuggestionDetails {
  public Name: string;
  public Valid: string[];
  public Value: string[];
}

class SelectionDict {
  public Name: string;
  public Value: string[];
  public NextSelection: string;
}

// Server response
class ApiResponse {
  public DisplayName: string;
  public SearchType: string;
  public AutoCompleteValues: string[];
}

enum Action {
  Field = 'Field',
  Operator = 'Operator',
  Value = 'Value',
  Expression = 'Expression'
}
