import { Component, OnInit, ElementRef, ViewChild } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { UserService } from '../user.service';
import { User } from '../../models/user';
import { Pagination } from '../../models/pagination';
import { PreserveSearch } from '../../models/preserve-search';
import * as _ from 'lodash';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { Popup } from '../../Popup';
import { Subject } from 'rxjs';
import {
  debounceTime,
  distinctUntilChanged,
  switchMap,
  tap
} from 'rxjs/operators';
import { ModeEnum } from '../../shared/emuns/user.enum'
import { KeyboardKeyCodeEnum, RoleEnum } from '../../shared/emuns/global.enum'
import { Title } from '@angular/platform-browser';
import { ThemeService } from '../../theme.service';
import { RolesAndPermissionService } from '../../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-user-list',
  templateUrl: './user-list.component.html',
  styleUrls: ['./user-list.component.css']
})
export class UserListComponent implements OnInit {
  showHelpText: boolean = false;
  getAuth: Popup = new Popup();
  getLock: Popup = new Popup();
  getUnLock: Popup = new Popup();
  getResetMfa: Popup = new Popup();
  public alert: GlobalMessageClass;
  users: any = {};
  private usersList = new Array();
  private selectedUser: User;
  pagination: Pagination = new Pagination();
  private Search: PreserveSearch = new PreserveSearch();
  currentUserName: string = '';
  userID: string = '';
  authPassword: string = '';
  failedPassword: number = 0;
  reSubmit: boolean = false;
  showAuthError: boolean = false;
  userToDelete: User;
  userToUnLock: User;
  userToLock: User;
  userToResetMfa: User;
  public dateFormat: string = 'dd/MM/yyyy HH:mm';
  public userrole: string = '';
  admin_mfa: boolean = false;
  isNameSort: boolean = false;

  keysSelectedUserId: string;
  keysLoading: boolean = false;

  loading: Boolean = true;
  showKeys: Boolean = false;

  public_key: String = '';
  secret_key: String = '';

  pageSize = [10, 25, 50, 100, 200];

  Role = RoleEnum;

  sort: string;
  sortDir: string;

  @ViewChild('authpassword', { static: false }) _authpassword: ElementRef;
  @ViewChild('lockpassword', { static: false }) _lockpassword: ElementRef;
  @ViewChild('unlockpassword', { static: false }) _unlockpassword: ElementRef;
  @ViewChild('resetMfa', { static: false }) _resetMfa: ElementRef;

  $searchUsers = new Subject();
  searchKeyword;
  navigationToEditUser: boolean = false;
  isDarkMode : any;

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  constructor(
    private router: Router,
    private userService: UserService,
    private route: ActivatedRoute,
    private globalMessageService: GlobalMessageService,
    private titleService: Title,
    private themesService : ThemeService,
    private rolesPermissionService: RolesAndPermissionService
  ) {
    this.userrole = localStorage.getItem('ur');
    if (typeof this.userrole != 'undefined' || this.userrole != null) {
      this.userrole = atob(this.userrole);
    }
    var localDateFormat = localStorage.getItem('dateFormat');
    if (typeof localDateFormat != 'undefined' || localDateFormat != null) {
      this.dateFormat = localDateFormat;
    }
  }

  nameSorting() {
    this.isNameSort = !this.isNameSort;
    this.getPage(this.pagination.currentPage);
  }

  subscibeToSearch() {
    this.$searchUsers
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap(data => {
          this.loading = true;
          this.searchKeyword = data;
          this.pagination.currentPage = 1;
          return this.userService.getUsers(
            this.pagination.currentPage,
            this.pagination.itemsPerPage,
            this.sort,
            this.sortDir,
            this.searchKeyword
          );
        })
      )
      .subscribe({
        next: (users: any) => {
          this.loading = false;
          const { recordsTotal } = users;
          this.users = users;
          this.pagination.totalItems = recordsTotal;
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  getCRUDdata(privId){
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false
      this.readPermission = res?.read_permission === 1 ? true : false
      this.updatePermission = res?.update_permission === 1 ? true : false
      this.deletePermission = res?.delete_permission === 1 ? true : false
    });
  }

  getPage(page: number) {
    this.loading = true;
    let prevPage = this.pagination.currentPage;
    this.pagination.currentPage = page;
    this.userService
      .getUsers(
        this.pagination.currentPage,
        this.pagination.itemsPerPage,
        this.sort,
        this.sortDir,
        this.searchKeyword
      )
      .pipe(
        tap((res: any) => {
          this.loading = false;
          this.admin_mfa = res.admin_mfa;
          this.users = res;
          this.pagination.totalItems = res.recordsTotal;
          this.pagination.currentPage = page;
          if (prevPage != this.pagination.currentPage) {
            let scrollToTop = window.setInterval(() => {
              let pos = window.pageYOffset;
              if (pos > 0) {
                window.scrollTo(0, pos - 20); // how far to scroll on each step
              } else {
                window.clearInterval(scrollToTop);
              }
            }, 16);
          }
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          this.loading = false;
        }
      });
  }

  clearfun() {
    this.searchKeyword = '';
    this.getPage(this.pagination.currentPage);
  }

  resetMfaHandler(user: User) {
    this.userToResetMfa = user;
    this.showPurgePopup(4);
  }

  resetMfa_() {
    this.resetMfaUser(this.authPassword, this.currentUserName);
  }

  resetMfaUser(password, username = null) {
    this.selectedUser = this.userToResetMfa;
    this.userService.resetMfa(this.selectedUser.id, password).subscribe(
      response => {
        this.getResetMfa.show = false;
        this.alert = {
          closed: false,
          type: 'success',
          message: 'MFA reset successfully.'
        };
        this.globalMessageService.pushAlert(this.alert);
      },
      err => {
        // const alert = {
        //   closed: false,
        //   type: 'danger',
        //   message: 'Invalid password provided'
        // };
        // this.globalMessageService.pushAlert(alert);
      }
    );
  }

  public showPurgePopup(mode: number) {
    if (mode === ModeEnum.Auth) {
      this.authPassword = '';
      this.showAuthError = false;
      this.getAuth.heading = 'Verification required';
      this.getAuth.body =
        'Please enter your current password for verification.';
      this.getAuth.action = 'Confirm';
      this.getAuth.show = true;
    } else if (mode === ModeEnum.Lock) {
      this.authPassword = '';
      this.showAuthError = false;
      this.getLock.heading = 'Verification required';
      this.getLock.body =
        'Please enter your current password for verification.';
      this.getLock.action = 'Confirm';
      this.getLock.show = true;
    } else if (mode === ModeEnum.Unlock) {
      this.authPassword = '';
      this.showAuthError = false;
      this.getUnLock.heading = 'Verification required';
      this.getUnLock.body =
        'Please enter your current password for verification.';
      this.getUnLock.action = 'Confirm';
      this.getUnLock.show = true;
    } else if (mode === ModeEnum.ResetMfa) {
      this.authPassword = '';
      this.showAuthError = false;
      this.getResetMfa.heading = 'Verification required';
      this.getResetMfa.body =
        'Please enter your current password for verification.';
      this.getResetMfa.action = 'Confirm';
      this.getResetMfa.show = true;
    }
  }

  hideAuthError() {
    this.showAuthError = false;
  }

  cancelPopup(mode: number) {
    if (mode === ModeEnum.Auth) {
      this.getAuth.show = false;
    } else if (mode === ModeEnum.Lock) {
      this.getLock.show = false;
    } else if (mode === ModeEnum.Unlock) {
      this.getUnLock.show = false;
    } else if (mode === ModeEnum.ResetMfa) {
      this.getResetMfa.show = false;
    }
  }

  authAction() {
    this.deleteUser(this.authPassword, this.currentUserName);
  }

  lockAction() {
    this.lockUser(this.authPassword, this.currentUserName);
  }

  unLockAction() {
    this.unlockUser(this.authPassword, this.currentUserName);
  }

  deleteUser(password, username = null) {
    this.selectedUser = this.userToDelete;
    this.userService.delete_user(this.selectedUser.id, password).subscribe(
      response => {
        if (response.success) {
          this.getAuth.show = false;
          this.alert = {
            closed: false,
            type: 'success',
            message: 'Admin User has been deleted successfully.'
          };
          this.globalMessageService.pushAlert(this.alert);
          this.getPage(this.pagination.currentPage);
        }
      },
      err => {
        // const alert = {
        //   closed: false,
        //   type: 'danger',
        //   message: 'Invalid password provided'
        // };
        // this.globalMessageService.pushAlert(alert);
      }
    );
  }

  lockUser(password, username = null) {
    this.selectedUser = this.userToLock;
    this.userService.lock_user(this.selectedUser.id, password).subscribe(
      response => {
        if (response.success) {
          this.getLock.show = false;
          this.alert = {
            closed: false,
            type: 'success',
            message: 'User has been locked successfully.'
          };
          this.globalMessageService.pushAlert(this.alert);
          this.getPage(this.pagination.currentPage);
        }
      },
      err => {
        //   const alert = {
        //     closed: false,
        //     type: 'danger',
        //     message: 'Invalid password provided'
        //   };
        //   this.globalMessageService.pushAlert(alert);
      }
    );
  }

  onSendEmailVerificationAgain(user: User, event: any): void {
    event.target.className += ' disabled';
    this.userService.send_verification_email(user.id).subscribe(response => {
      if (response.success) {
        this.alert = {
          closed: false,
          type: 'success',
          message: 'User verification link has been dispatched successfully.'
        };
        this.globalMessageService.pushAlert(this.alert);
      }
    });
  }

  unlockUser(password, username = null) {
    this.selectedUser = this.userToUnLock;
    this.userService.un_lock_user(this.selectedUser.id, password).subscribe(
      response => {
        if (response.success) {
          this.getUnLock.show = false;
          this.alert = {
            closed: false,
            type: 'success',
            message: 'User has been unlocked successfully.'
          };
          this.globalMessageService.pushAlert(this.alert);
          this.getPage(this.pagination.currentPage);
        }
      },
      err => {
        // const alert = {
        //   closed: false,
        //   type: 'danger',
        //   message: 'Invalid password provided'
        // };
        // this.globalMessageService.pushAlert(alert);
      }
    );
  }

  submitMyAlert(event, mode) {
    if (mode === ModeEnum.Auth) {
      if (event.keyCode == KeyboardKeyCodeEnum.Enter) {
        if (!this.reSubmit && this.authPassword.length != 0) {
          this.authAction();
        }
      }
    } else if (mode === ModeEnum.Lock) {
      if (event.keyCode == KeyboardKeyCodeEnum.Enter) {
        if (!this.reSubmit && this.authPassword.length != 0) {
          this.lockAction();
        }
      }
    } else if (mode === ModeEnum.Unlock) {
      if (event.keyCode == KeyboardKeyCodeEnum.Enter) {
        if (!this.reSubmit && this.authPassword.length != 0) {
          this.unLockAction();
        }
      }
    } else if (mode === ModeEnum.ResetMfa) {
      if (event.keyCode == KeyboardKeyCodeEnum.Enter) {
        if (!this.reSubmit && this.authPassword.length != 0) {
          this.resetMfa_();
        }
      }
    }
  }

  public ngOnInit(): void {
    this.titleService.setTitle('IAM | Admin Users');
    this.getCRUDdata(35);
    let userSearch = this.Search.getSearch('userSearch');
    if (userSearch) {
      this.pagination = userSearch.pagination;
      this.searchKeyword = userSearch.searchKeyword;
    }
    this.populateCurrentUserName();
    this.getPage(this.pagination.currentPage);
    this.subscibeToSearch();
  }

  private populateCurrentUserName() {
    let current_user = JSON.parse(localStorage.getItem('currentUser'));
    if (current_user) {
      this.currentUserName = current_user.userName;
      this.userID = localStorage.getItem('usd');
      if (typeof this.userID != 'undefined' || this.userID != null) {
        this.userID = atob(this.userID);
      }
    } else {
      this.currentUserName = '';
    }
  }

  private onSelect(user: User): void {
    let userSearch = {
      pagination: this.pagination,
      searchKeyword: this.searchKeyword
    };
    this.Search.saveSearch('userSearch', userSearch);
    this.selectedUser = user;
    this.navigationToEditUser = true;
    this.router.navigate(['/admin', this.selectedUser.id]);
  }

  ngOnDestroy() {
    if (!this.navigationToEditUser) {
      this.Search.saveSearch('userSearch', '');
    }
  }

  private onDelete(user: User): void {
    this.userToDelete = user;
    this.showPurgePopup(1);
  }

  private onLock(user: User): void {
    this.userToLock = user;
    this.showPurgePopup(2);
  }

  private onUnLock(user: User): void {
    this.userToUnLock = user;
    this.showPurgePopup(3);
  }

  add_new_user(): void {
    this.router.navigate(['/create-admin']);
  }

  focus() {
    this._authpassword.nativeElement.focus();
    this._lockpassword.nativeElement.focus();
    this._unlockpassword.nativeElement.focus();
    this._resetMfa.nativeElement.focus();
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getPage(this.pagination.currentPage);
  }

  sortRecord(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
    this.getPage(this.pagination.currentPage);
  }

  showKeysHandler(user) {
    this.showKeys = true;
    this.public_key = user.public_key;
    this.secret_key = user.secret_key;
    this.keysSelectedUserId = user.id;
  }

  hideKeysHanlder() {
    this.showKeys = false;
  }

  isPublicUser(user) {
    return (
      user && user.template && user.template === 'API User'
    );
  }

  handleRegenerateKeys() {
    const userIndex = this.users.data.findIndex(
      myUser => myUser.id === this.keysSelectedUserId
    );
    if (userIndex > -1) {
      this.keysLoading = true;
      const user = this.users.data[userIndex];
      this.userService.regenerateKeys(this.keysSelectedUserId).subscribe(
        response => {
          user.secret_key = response.secretKey;
          user.public_key = response.publicKey;
          this.users.data[userIndex] = user;
          this.public_key = response.publicKey;
          this.secret_key = response.secretKey;
        },
        err => {},
        () => {
          this.keysLoading = false;
        }
      );
    }
  }
}
