import { Injectable } from '@angular/core';
import { ConfigService } from './services/config.service';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { Pagination } from './models/pagination';
import { map } from 'rxjs/operators';

@Injectable()
export class ValidDomainsService {
  validDomainIp;
  constructor(config: ConfigService, private authHttp: AuthHttp) {
    this.validDomainIp = config.getConfiguration().apiBaseUrl + '/validDomains';
  }

  getValidDomains(
    pagination: Pagination,
    sort: string = 'id',
    dir: string = 'desc',
    searchKey = ''
  ) {
    let query = `?limit=${
      pagination.itemsPerPage
    }&offset=${pagination.currentPage -
      1}&sort=${sort}&dir=${dir}&searchKey=${searchKey}`;
    return this.authHttp
      .get(this.validDomainIp + query)
      .pipe(map(response => response));
  }

  addNewDomain(domain) {
    return this.authHttp
      .post(this.validDomainIp, domain)
      .pipe(map(response => response));
  }

  editDomain(id, editDomain) {
    return this.authHttp
      .put(`${this.validDomainIp}/${id}`, editDomain)
      .pipe(map(response => response));
  }
  deleteDomain(id) {
    return this.authHttp.delete(`${this.validDomainIp}/${id}`);
  }
}
