import { Component, OnInit, ViewChild } from '@angular/core';
import { NgModel } from '@angular/forms';
import { ValidDomainsService } from '../valid-domains.service';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { Pagination } from '../models/pagination';
import { Subject, Subscription } from 'rxjs';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { ConfirmModalComponent } from '../shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { OtpAuthenticationService } from '../otp-verification/otp-authentication.service';
import { RolesAndPermissionService } from '../roles-and-permission/roles-and-permission.service';
@Component({
  selector: 'app-valid-domains',
  templateUrl: './valid-domains.component.html',
  styleUrls: ['./valid-domains.component.css']
})
export class ValidDomainsComponent implements OnInit {
  addDomain: string;
  @ViewChild('editUserDomain', { static: false }) editUserDomain: NgModel;
  @ViewChild('domainName', { static: false }) domainName: NgModel;
  editSaveButtonDisabled = false;
  addSaveButtonDisabled = true;
  pagination: Pagination = new Pagination();
  domains: any = {};
  editDomain;
  isAdding = false;
  isEditing = false;
  loading: Boolean = true;
  $searchDomains = new Subject();
  searchKeyword;

  pageSize = [10, 25, 50, 100, 200];
  sort: string;
  sortDir: string;

  public alert: GlobalMessageClass;
  isDarkMode: boolean;
  showOtpDialog: boolean = false;
  otptoken: string = '';
  option: string;
  private otpStatusSubscription : Subscription;

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  constructor(
    private valiDomainService: ValidDomainsService,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog,
    private OtpAuthenticationService: OtpAuthenticationService,
    private rolesPermissionService: RolesAndPermissionService
  ) {}

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.getDomains();
    this.getCRUDdata(56);
    this.subscibeToSearch();
  }

  ngOnDestroy() {
    if (this.otpStatusSubscription) {
      this.otpStatusSubscription.unsubscribe();
    }
  }

  subscibeToSearch() {
    this.$searchDomains
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap(data => {
          this.loading = true;
          this.searchKeyword = data;
          this.pagination.currentPage = 1;
          return this.valiDomainService.getValidDomains(
            this.pagination,
            this.sort,
            this.sortDir,
            this.searchKeyword
          );
        })
      )
      .subscribe({
        next: (domains: any) => {
          this.loading = false;
          const { recordsTotal } = domains;
          this.domains = domains;
          this.pagination.totalItems = recordsTotal;
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  getCRUDdata(privId){
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false
      this.readPermission = res?.read_permission === 1 ? true : false
      this.updatePermission = res?.update_permission === 1 ? true : false
      this.deletePermission = res?.delete_permission === 1 ? true : false
    });
  }

  getDomains() {
    this.loading = true;
    this.valiDomainService
      .getValidDomains(
        this.pagination,
        this.sort,
        this.sortDir,
        this.searchKeyword
      )
      .subscribe({
        next: (data: any) => {
          this.domains = data;
          this.pagination.totalItems = data.recordsTotal;
          this.loading = false;
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  clearfun() {
    this.searchKeyword = '';
    this.getDomains();
  }

  addDomainhandler(event) {
    event.preventDefault();
    this.isAdding = true;
  }

  editSingleRow(domain) {
    this.editSaveButtonDisabled = false;
    this.editDomain = { ...domain };
  }

  stop(event) {}

  cancelEditDomain() {
    this.editDomain = null;
    this.editSaveButtonDisabled = false;
  }

  editDomainNameHandler() {
    this.editSaveButtonDisabled = this.editUserDomain.invalid;
  }
  addDomainNameHandler() {
    this.addSaveButtonDisabled = this.domainName.invalid;
  }

  addCancelhandler() {
    this.isAdding = false;
    this.addSaveButtonDisabled = true;
    this.addDomain = null;
  }

  saveAddDomain() {
    const addedDomain = {
      domain: this.addDomain
    };
    this.valiDomainService.addNewDomain(addedDomain).subscribe(
      data => {
        this.getDomains();
        // const { id, domain } = data;
        // const newDomain = { id, domain };
        // this.domains = [newDomain, ...this.domains];
        this.addCancelhandler();
        this.showAlert('success', 'Domain Added Successfully.', false);
      },
      error => {
        // this.showAlert('danger', error._body, false);
      }
    );
  }

  editDomainHandler(domain) {
    let mfa = localStorage.getItem('adminMfaValue');
    if (mfa === 'true') {
      this.showOtpDialog = true;
      this.option = 'edit';
      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckEdit$.subscribe(
        (res: any) => {
          if (res == true) {
            const editDomain = {
              domain: this.editDomain.domain
            };
            this.valiDomainService.editDomain(domain.id, editDomain).subscribe(
              data => {
                this.domains.data = this.domains.data.map(singleDomain => {
                  if (singleDomain.id === domain.id) {
                    return {
                      ...singleDomain,
                      domain: editDomain.domain
                    };
                  }
                  return singleDomain;
                });
                this.cancelEditDomain();
                this.showAlert(
                  'success',
                  'Domain Updated Successfully.',
                  false
                );
              },
              error => {
                // this.showAlert('danger', error._body, false);
              }
            );
            this.showOtpDialog = false;
            this.OtpAuthenticationService.setOtpStatusValueEdit(false);
          }
        }
      );
    } else {
      const editDomain = {
        domain: this.editDomain.domain
      };
      this.valiDomainService.editDomain(domain.id, editDomain).subscribe(
        data => {
          this.domains.data = this.domains.data.map(singleDomain => {
            if (singleDomain.id === domain.id) {
              return {
                ...singleDomain,
                domain: editDomain.domain
              };
            }
            return singleDomain;
          });
          this.cancelEditDomain();
          this.showAlert('success', 'Domain Updated Successfully.', false);
        },
        error => {
          // this.showAlert('danger', error._body, false);
        }
      );
    }
  }

  deleteHandler(domain) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Are you sure you want to delete this Domain?`
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        let mfa = localStorage.getItem('adminMfaValue');
        if (mfa === 'true') {
          this.showOtpDialog = true;
          this.option = 'delete';
          this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckDel$.subscribe(
            (res: any) => {
              if (res == true) {
                this.valiDomainService.deleteDomain(domain.id).subscribe(
                  data => {
                    this.alert = {
                      closed: false,
                      type: 'success',
                      message: 'Domain has been deleted successfully.'
                    };
                    this.globalMessageService.pushAlert(this.alert);

                    this.domains.data = this.domains.data.filter(
                      singleDomain => singleDomain.id !== domain.id
                    );
                  },
                  error => {
                    // this.showAlert(
                    //   'error',
                    //   'Something went wrong in deleting domain',
                    //   false
                    // );
                  }
                );
                this.showOtpDialog = false;
                this.OtpAuthenticationService.setOtpStatusValueDel(false);
              }
            }
          );
        } else {
          this.valiDomainService.deleteDomain(domain.id).subscribe(
            data => {
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Domain has been deleted successfully.'
              };
              this.globalMessageService.pushAlert(this.alert);

              this.domains.data = this.domains.data.filter(
                singleDomain => singleDomain.id !== domain.id
              );
            },
            error => {
              // this.showAlert(
              //   'error',
              //   'Something went wrong in deleting domain',
              //   false
              // );
            }
          );
        }
      }
    });
  }

  private showAlert(type: string, message: string, sticky: boolean) {
    const alert = {
      closed: false,
      type: type,
      message: message,
      sticky: sticky
    };
    this.globalMessageService.pushAlert(alert);
  }

  chagePage(page: number) {
    this.pagination.currentPage = page;
    this.getDomains();
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getDomains();
  }

  sortRecord(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
    this.getDomains();
  }

  cancelVerificationPassword(event: any) {
    this.showOtpDialog = false;
    this.OtpAuthenticationService.setOtpStatusValueEdit(false);
    this.OtpAuthenticationService.setOtpStatusValueDel(false);
  }

  submitVerificationPassword(event: any) {
    this.otptoken = event;
    // this.otptoken = event
  }

  getClosePopup(event: any) {
    this.showOtpDialog = false;
  }
}
