import {
  ValidatorFn,
  AbstractControl,
  NG_VALIDATORS,
  Validator
} from '@angular/forms';
import { Directive, Input } from '@angular/core';

const LDAP_URL_REGEX = /^(ldaps?:\/\/)(((([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|([a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(\/.*)?))(:([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?$/i;
const LDAPS_URL_REGEX = /^(ldaps:\/\/)(((([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))|([a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5}(\/.*)?))(:([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5]))?$/i;

export function ldapUrl(ldap: any): ValidatorFn {
  return (control: AbstractControl): { [key: string]: any } => {
    if (!LDAP_URL_REGEX.test(control.value)) {
      return {
        ldapUrl: 'pattern'
      };
    }

    if (
      LDAPS_URL_REGEX.test(control.value) &&
      !ldap.LDAP_ca &&
      !ldap.LDAP_cert
    ) {
      return {
        ldapUrl: 'certificatesRequired'
      };
    }

    return null;
  };
}

@Directive({
  selector: '[ldapUrl]',
  providers: [
    {
      provide: NG_VALIDATORS,
      useExisting: LdapUrlValidator,
      multi: true
    }
  ]
})
export class LdapUrlValidator implements Validator {
  @Input('ldapUrl') ldap;

  validate(control: AbstractControl): { [key: string]: any } {
    return ldapUrl(this.ldap)(control);
  }
}
