import { Component } from '@angular/core';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { ClientAccessStatusEnum, DeviceOSEnum, DeviceTypeEnum } from '../../shared/emuns/client.enum';
import { Pagination } from '../../models/pagination';
import { DeviceService } from '../../devices-list/device.service';
import { LocalUserService } from '../../create-local-user/local-user.service';
import { OrganizationService } from '../../organization-list-view/organization.service';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { ActivatedRoute, Router } from '@angular/router';
import { OsPolicyService } from '../../create-os-policy/os-policy.service';
import { LicenseService } from '../../license/license.service';
import { Title } from '@angular/platform-browser';
import { Device } from '../../models/device';


@Component({
  selector: 'app-create-workload-devices',
  templateUrl: './create-workload-devices.component.html',
  styleUrls: ['./create-workload-devices.component.css']
})
export class CreateWorkloadDevicesComponent {
  clientIds = [];
  public alert: GlobalMessageClass;
  public dateFormat: string = 'dd/MM/yyyy HH:mm';
  DeviceType = DeviceTypeEnum;
  device = new Device();
  displayedOrg = null;
  displayedDept = null;
  deviceIDErrorMessage = '';
  createMode: Boolean = true;
  editing = false;
  loading: Boolean = false;
  showHasAgent: Boolean = true;
  showDarkeningState: Boolean = true;
  showIoTDeviceType: Boolean = true;
  showHypervisorVersion: Boolean = true;
  showUserId: Boolean = false;
  showOs: Boolean = true;
  showOsVersion: Boolean = true;
  organizations = [];
  departments = [{ deptId: null, deptName: null }];
  organizationId = 0;
  deviceTypes = [
    DeviceTypeEnum.Desktop,
    DeviceTypeEnum.IoTBridgeClient,
    DeviceTypeEnum.Laptop,
    DeviceTypeEnum.VirtualDesktop,
    DeviceTypeEnum.PhysicalServer,
    DeviceTypeEnum.Smartphone,
    DeviceTypeEnum.Tablet,
    DeviceTypeEnum.ThinBridgeClient,
    DeviceTypeEnum.ServerBridgeClient,
    DeviceTypeEnum.VirtualServer,
    DeviceTypeEnum.ARMPlatform
  ];
  iotSelected: boolean = false;
  mobileSelected: boolean = false;
  regExVal = {
    uuid:
      '^[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}$',
    mac: '^([0-9A-Fa-f]{2}[:]){5}([0-9A-Fa-f]{2})$',
    mobile: '^[a-zA-Z0-9]{7,}$',
    generic: '.*'
  };
  deviceIDRegEx;
  deviceUser;
  oldDeviceUser;

  deviceOS = [];
  deviceOSVersions = [];
  globalDarkening = false;
  inWorkload = false;

  userLoading = false;
  users = [];
  selectedUser;
  viewMode = false;
  license: any;
  disableSaveButton = false;
  clients: any = {};

  clientAllOs = [
    { value: DeviceOSEnum.Ubuntu18, display: 'Ubuntu Desktop' },
    { value: DeviceOSEnum.UbuntuServer18, display: 'Ubuntu Server' },
    {
      value: DeviceOSEnum.UbuntuServer20armv8,
      display: 'Ubuntu Server (ARMv8)'
    },
    { value: DeviceOSEnum.Windows6_1, display: 'Windows 7' },
    { value: DeviceOSEnum.Windows6_2, display: 'Windows 8' },
    { value: DeviceOSEnum.Windows6_3, display: 'Windows 8.1' },
    { value: DeviceOSEnum.Windows10, display: 'Windows 10' },
    { value: DeviceOSEnum.Windows11, display: 'Windows 11' },
    { value: DeviceOSEnum.WindowsServer6_1, display: 'Windows Server 2008 R2' },
    { value: DeviceOSEnum.WindowsServer6_2, display: 'Windows Server 2012' },
    { value: DeviceOSEnum.WindowsServer6_3, display: 'Windows Server 2012 R2' },
    { value: DeviceOSEnum.WindowsServer10, display: 'Windows Server 2016+' },
    { value: DeviceOSEnum.CentosServer7, display: 'CentOS 7 Server' },
    { value: DeviceOSEnum.CentosServer8, display: 'CentOS 8 Server' },
    { value: DeviceOSEnum.Macintosh10_14, display: 'macOS Mojave' },
    { value: DeviceOSEnum.Macintosh10_15, display: 'macOS Catalina' },
    { value: DeviceOSEnum.Macintosh11, display: 'macOS Big Sur' },
    { value: DeviceOSEnum.Macintosh12, display: 'macOS Monterey' },
    { value: DeviceOSEnum.Macintosh13, display: 'macOS Ventura' },
    { value: DeviceOSEnum.IOS, display: 'iOS' },
    { value: DeviceOSEnum.Android, display: 'Android' },
    { value: DeviceOSEnum.Windows, display: 'Windows' },
    { value: DeviceOSEnum.WindowsServer, display: 'Windows Server' },
    { value: DeviceOSEnum.Macintosh, display: 'macOS' },
    { value: DeviceOSEnum.Ubuntu, display: 'Ubuntu' },
    { value: DeviceOSEnum.CentosServer, display: 'CentOS' },
    { value: DeviceOSEnum.UbuntuServer, display: 'Ubuntu Server' }
  ];

  formLoading: boolean = true;
  constructor(
    private deviceService: DeviceService,
    private userService: LocalUserService,
    private organizationService: OrganizationService,
    private router: Router,
    private globalMessageService: GlobalMessageService,
    private route: ActivatedRoute,
    private osService: OsPolicyService,
    private licenseService: LicenseService,
    private titleService: Title
  ) {}

  async ngOnInit() {
  
    this.disableSaveButton = false;
    this.loading = true;
    const darkeningResponse = (await this.deviceService
      .getGlobalDarkeningState()
      .toPromise()) as any;
    this.globalDarkening = darkeningResponse.global_darkening;
    this.viewMode = this.router.url.includes('/view-workload-device');
    if (this.viewMode) {
      this.titleService.setTitle('S2S Management | Devices | View Device');
    } else {
      this.titleService.setTitle('S2S Management | Devices | Edit Device');
    }
    this.deviceIDRegEx = this.regExVal.generic;
    this.device.device_type = DeviceTypeEnum.Desktop;
    const id = this.route.snapshot.params.id;
    this.licenseService.getLicense().subscribe(response => {
      this.license = response.license;
    });
    this.organizationService.getOrganizationNames().subscribe((orgs: any) => {
      this.organizations = orgs;
    });
    if (id) {
      const response = (await this.deviceService
        .getDevice(id)
        .toPromise()) as any;
      this.device = response.deviceData;
      this.globalDarkening = response.global_darkening;
      this.inWorkload = response.in_workload;

      if (this.device.user_id) {
        const deviceUserDS = (await this.userService
          .getSingleLocalUser(this.device.user_id)
          .toPromise()) as any;
        if (deviceUserDS && deviceUserDS.userData) {
          this.deviceUser = deviceUserDS.userData.name;
          this.oldDeviceUser = deviceUserDS.userData;
        }
      }
      this.formLoading = false;
      this.loading = false;
      this.editing = true;
      this.createMode = false;
      this.organizationService
        .getDepartment(this.device.department)
        .subscribe((dept: any) => {
          this.organizationService
            .getOrganization(dept.organization_id)
            .subscribe(async (org: any) => {
              this.organizationService
                .getDepartments(org.id)
                .subscribe((departments: any) => {
                  this.departments = [];
                  departments.data.forEach(dept => {
                    this.departments.push({
                      deptId: dept.id,
                      deptName: dept.name
                    });
                  });
                });
              this.displayedOrg = org.name;
              // await this.organizationChange(org.name);
              this.displayedDept = dept.id;
              this.getLocalUsers();
            });
        });
    } else {
      this.loading = false;
      this.formLoading = false;
      this.device.risk_score = 1;
    }
    this.deviceTypeChange(true);
    this.handleOSChange();
    this.getLocalUsers();
    if (this.viewMode) {
      await this.getClients(id);
      if (this.clientIds.length > 0) {
      } else {
        this.formLoading = false;
      }
    }
  }

  getClients(deviceId) {
    return new Promise((resolve, reject) => {
      this.deviceService.getClients(deviceId).subscribe(
        (response: any) => {
          this.clients = response;
          this.clientIds = this.clients.map(client => client.agent_id);
          resolve(null);
        },
        error => {
          reject(error);
        }
      );
    });
  }


  getLocalUsers() {
    if (this.device.department) {
      this.userService
        .getAvailableLocalUsers(
          this.device.department.deptId && this.device.department.deptId
            ? this.device.department.deptId
            : this.device.department
        )
        .subscribe((response: any) => {
          this.users = response.data;
        });
    }
  }

  handleDeviceUserChange(user) {
    if (this.editing) {
      if (this.oldDeviceUser) {
        const myUser = this.oldDeviceUser;
        this.userService
          .removeDeviceFromUser(myUser.id, this.device.id)
          .subscribe(response => {
            if (user) {
              this.userService
                .addDeviceToUser(user.id, this.device.device_name)
                .subscribe(response => {
                  this.globalMessageService.pushAlert({
                    closed: false,
                    type: 'success',
                    message: 'Device user updated'
                  });
                  this.oldDeviceUser = user;
                });
            } else {
              this.oldDeviceUser = null;
              this.globalMessageService.pushAlert({
                closed: false,
                type: 'success',
                message: 'Device removed from user successfully'
              });
            }
          });
      } else {
        if (user) {
          this.userService
            .addDeviceToUser(user.id, this.device.device_name)
            .subscribe(response => {
              this.globalMessageService.pushAlert({
                closed: false,
                type: 'success',
                message: 'Device user updated'
              });
              this.oldDeviceUser = user;
            });
        }
      }
    }
  }

  saveDevice() {
    if (!this.deviceIDValidation()) {
      this.disableSaveButton = false; // Reset disableSaveButton flag
      return;
    }
    this.disableSaveButton = true;
    this.loading = true;
    //    if(this.device.device_type == DeviceTypeEnum.Smartphone) {
    this.device.device_id = this.device.device_id.toUpperCase();
    //    }
    let deviceToSave = { ...this.device };
    if (this.deviceUser) {
      deviceToSave = { ...this.device, user_id: this.deviceUser.id };
    }
    this.deviceService.addDevice(deviceToSave, this.editing).subscribe(
      response => {
        this.loading = false;
        this.router.navigate(['/devices']);
        this.globalMessageService.pushAlert({
          closed: false,
          type: 'success',
          message: 'Device created/updated successfully',
          sticky: false
        });
        this.disableSaveButton = false;
      },
      error => {
        this.disableSaveButton = false;
        this.loading = false;
        this.globalMessageService.clearAlerts();
      }
    );
  }

  cancelHandler() {
    this.router.navigate(['/workload-devices']);
  }

  deviceTypeChange(startUp) {
    if (!startUp) {
      this.device.os = null;
      this.device.os_version = null;
    }
    const deviceType = this.device.device_type;
    this.showHasAgent = [
      DeviceTypeEnum.Laptop.toString(),
      DeviceTypeEnum.VirtualDesktop,
      DeviceTypeEnum.Desktop,
      DeviceTypeEnum.ServerBridgeClient
    ].includes(deviceType);
    this.showDarkeningState = [
      DeviceTypeEnum.Laptop.toString(),
      DeviceTypeEnum.VirtualDesktop,
      DeviceTypeEnum.Desktop,
      DeviceTypeEnum.ServerBridgeClient,
      DeviceTypeEnum.PhysicalServer,
      DeviceTypeEnum.VirtualServer
    ].includes(deviceType);
    this.showIoTDeviceType = [
      DeviceTypeEnum.IoTBridgeClient.toString()
    ].includes(deviceType);
    this.showHypervisorVersion = [
      DeviceTypeEnum.ServerBridgeClient.toString(),
      DeviceTypeEnum.VirtualServer
    ].includes(deviceType);
    this.showUserId = [
      DeviceTypeEnum.Desktop.toString(),
      DeviceTypeEnum.Laptop,
      DeviceTypeEnum.VirtualDesktop,
      DeviceTypeEnum.PhysicalServer,
      DeviceTypeEnum.Smartphone,
      DeviceTypeEnum.Tablet,
      DeviceTypeEnum.VirtualServer,
      DeviceTypeEnum.ARMPlatform
    ].includes(deviceType);
    this.showOs = [
      DeviceTypeEnum.Laptop.toString(),
      DeviceTypeEnum.VirtualDesktop,
      DeviceTypeEnum.Desktop,
      DeviceTypeEnum.Smartphone,
      DeviceTypeEnum.Tablet,
      DeviceTypeEnum.PhysicalServer,
      DeviceTypeEnum.VirtualServer,
      DeviceTypeEnum.IoTBridgeClient
    ].includes(deviceType);
    this.iotSelected = [
      DeviceTypeEnum.IoTBridgeClient.toString(),
      DeviceTypeEnum.ServerBridgeClient,
      DeviceTypeEnum.ThinBridgeClient
    ].includes(deviceType);
    this.mobileSelected = deviceType == DeviceTypeEnum.Smartphone;
    this.showOsVersion = this.showOs;
    this.changeOSListing();
    if (
      deviceType == DeviceTypeEnum.PhysicalServer ||
      deviceType == DeviceTypeEnum.ARMPlatform
    )
      this.deviceIDRegEx = this.regExVal.generic;
    else if (deviceType == DeviceTypeEnum.Smartphone)
      this.deviceIDRegEx = this.regExVal.mobile;
    else
      this.deviceIDRegEx = this.iotSelected
        ? this.regExVal.mac
        : this.regExVal.uuid;
  }

  deviceIDValidation() {
    let regExVal = this.regExVal.uuid;
    if (this.device.device_type && this.device.device_id) {
      if (
        this.device.device_type == DeviceTypeEnum.PhysicalServer ||
        this.device.device_type == DeviceTypeEnum.ARMPlatform
      ) {
        return true;
      }
      this.deviceIDErrorMessage = 'Device ID must be a valid UUID.';
      if (
        this.device.device_type == DeviceTypeEnum.IoTBridgeClient.toString() ||
        this.device.device_type == DeviceTypeEnum.ThinBridgeClient.toString() ||
        this.device.device_type == DeviceTypeEnum.ServerBridgeClient.toString()
      ) {
        regExVal = this.regExVal.mac;
        this.deviceIDErrorMessage = 'Device must be a valid MAC Address.';
        this.device.mac_address = this.device.device_id;
      }
      if (this.device.device_type == DeviceTypeEnum.Smartphone) {
        regExVal = this.regExVal.mobile;
        this.deviceIDErrorMessage =
          'Device must be at least a 7-character alphanumeric string.';
      }
      this.globalMessageService.clearAlerts();
      const regex = new RegExp(regExVal);
      if (!regex.test(this.device.device_id.toString())) {
        this.globalMessageService.pushAlert({
          closed: false,
          type: 'danger',
          message: this.deviceIDErrorMessage,
          sticky: false
        });
        return false;
      }
    }
    return true;
  }

  async organizationChange(orgName) {
    if (!orgName) {
      this.selectedUser = null;
      this.users = [];
      return;
    }

    this.departments = [];
    const org = (await this.organizationService
      .getOrganizationByName(orgName)
      .toPromise()) as any;
    const departments = (await this.organizationService
      .getDepartments(org.id)
      .toPromise()) as any;
    this.organizationId = org.id;
    for (let j = 0; j < departments.data.length; j++) {
      this.departments.push({
        deptId: departments.data[j].id,
        deptName: departments.data[j].name
      });
    }
    this.departments = [...this.departments];
    this.displayedDept = null;
  }

  departmentChange(deptId) {
    if (!deptId) {
      this.users = [];
      this.selectedUser = null;
      return;
    }

    this.device.department = deptId;
    this.getLocalUsers();
  }

  changeOSListing() {
    this.deviceOS = [];
    if (
      [
        DeviceTypeEnum.Desktop.toString(),
        DeviceTypeEnum.Laptop,
        DeviceTypeEnum.VirtualDesktop,
        DeviceTypeEnum.ServerBridgeClient
      ].indexOf(this.device.device_type) != -1
    ) {
      this.deviceOS = [
        { value: DeviceOSEnum.Macintosh, display: 'Macintosh' },
        { value: DeviceOSEnum.Ubuntu18, display: 'Ubuntu Desktop' },
        { value: DeviceOSEnum.Windows, display: 'Windows' }
      ];
    } else if (
      [DeviceTypeEnum.IoTBridgeClient.toString()].indexOf(
        this.device.device_type
      ) != -1
    ) {
      this.deviceOS = [
        {
          value: DeviceOSEnum.UbuntuServer20armv8,
          display: 'Ubuntu Server (ARMv8)'
        }
      ];
    } else if (
      [DeviceTypeEnum.PhysicalServer.toString()].indexOf(
        this.device.device_type
      ) != -1
    ) {
      this.deviceOS = [
        { value: DeviceOSEnum.WindowsServer, display: 'Windows Server' },
        { value: DeviceOSEnum.UbuntuServer18, display: 'Ubuntu Server' },
        {
          value: DeviceOSEnum.UbuntuServer20armv8,
          display: 'Ubuntu Server (ARMv8)'
        },
        { value: DeviceOSEnum.CentosServer, display: 'CentOS Server' }
      ];
    } else if (
      [DeviceTypeEnum.VirtualServer.toString()].indexOf(
        this.device.device_type
      ) != -1
    ) {
      this.deviceOS = [
        { value: DeviceOSEnum.UbuntuServer18, display: 'Ubuntu Server' },
        { value: DeviceOSEnum.WindowsServer, display: 'Windows Server' },
        { value: DeviceOSEnum.CentosServer, display: 'CentOS Server' }
      ];
    } else if (
      [DeviceTypeEnum.Smartphone, DeviceTypeEnum.Tablet.toString()].indexOf(
        this.device.device_type
      ) != -1
    ) {
      this.deviceOS = [
        { value: DeviceOSEnum.IOS, display: 'iOS' },
        { value: DeviceOSEnum.Android, display: 'Android' }
      ];
    }
    this.deviceOS = [...this.deviceOS];
  }
  handleOSChange() {
    this.deviceOSVersions = this.osService.getAllOsVersions(this.device.os);
    if (!this.editing) this.device.os_version = this.device.os ? 'any' : null;
  }
  goBack() {
    this.router.navigate(['/workload-devices']);
  }
  navigate() {}
}
