import {
  Component,
  OnInit,
  ViewChild,
  NgModule,
  ElementRef,
  HostListener
} from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Subject } from 'rxjs';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { WorkloadService } from '../workload.service';
import * as moment from 'moment';
import 'anychart';
import { MicrosegmentService } from '../../microsegmentation/microsegmentation.service';
import { ConfirmModalComponent } from '../../../app/shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { TrustedHostsService } from '../../trusted-hosts/trusted-hosts.service';
import { ClientService } from '../../client/client.service';
import { Clipboard } from '@angular/cdk/clipboard';
import { RolesAndPermissionService } from '../../roles-and-permission/roles-and-permission.service';
import { Pagination } from '../../models/pagination';
declare var anychart: any;
import { Coordinates, EdgeDisplayData, NodeDisplayData } from 'sigma/types';
import Sigma from 'sigma';
import Graph from 'graphology';

interface State {
  hoveredNode?: string;
  searchQuery: string;
  selectedNode?: string;
  suggestions?: Set<string>;
  hoveredNeighbors?: Set<string>;
}

interface ExtendedNodeDisplayData extends NodeDisplayData {
  shape: {
    type: string;
    url: string;
    ratio?: number;
    x?: number;
    y?: number;
    isParial: string;
  };
}

@Component({
  selector: 'app-microsegments',
  templateUrl: './microsegments.component.html',
  styleUrls: ['./microsegments.component.css']
})
export class MicrosegmentsComponent implements OnInit {
  deviceOffset: number = 0;
  deviceName: string;
  devicesList = [];
  selectedDeviceIds = [];
  workloadName: string = '';
  loading: boolean = false;
  isEditing: boolean = false;
  editingId: string;
  $deviceSearch = new Subject();
  isDeviceSearching = false;
  displayStyle = 'none';
  toggleDisplayStyle = 'none';
  toggleRuleStyle = 'none';
  toggleRuleViewStyle = 'none';
  check = false;
  graphBtn = false;
  mode: string = '';
  showHelpText: boolean = false;
  durationSelected: string = '60';
  thresholdSelected: string = '100';
  workloadId;
  percentage: number = 0;
  myTimer;
  createdOnce = false;
  myChart = null;
  // private timerSubscription: Subscription;
  showConfirm: boolean = false;
  allConnectedNodes: any;
  graphValues: any;
  clientId: number;

  existingRuleData: Array<Object> = [{}];
  protocol: Array<String> = ['TCP', 'UDP'];
  sourceDevice: Array<String> = [];
  sourceDeviceData: Array<String> = [];
  destDevice: Array<String> = [];
  sourceIp: Array<String> = [];
  workloadData: Array<any> = [];

  openFields: boolean = false;
  showAddRules: boolean = false;
  portCheck: boolean = true;
  ruleData: any;
  selPort: number;
  deviceId: number;
  exitingRuleLength: number = 0;

  selProtocol: string;
  selSourceIp: string;
  selDestDevice: string;
  selSourceDevice: string;
  selDestinationIp: string;
  isDragEnabled: boolean = true;
  isDarkMode: boolean;

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;
  pageSize = [10, 25, 50, 100, 200];
  listView: Boolean = true;

  pagination = new Pagination();
  currentPage = 1;
  size = 10;
  totalPages = 1;
  selectedPageSize: number;
  isTagEnable: boolean = false;
  workloadEdges = [];
  edge = {
    microsegTag: '',
    workloadTag: '',
    srcWorkloadTag: ''
  };

  //SIGMA>JS
  @ViewChild('container1') container: ElementRef | null = null;
  @ViewChild('searchInput') searchInput: HTMLInputElement | null = null;
  searchSuggestions: any;

  sigma?: Sigma | undefined;
  graph: any = new Graph({ multi: true });
  state: State = { searchQuery: '' };
  mappedObject: any;

  searchKeyword = '';
  $workloadSearch = new Subject();
  discoveryDataLocalStorage: any;
  constructor(
    public router: Router,
    private workloadService: WorkloadService,
    private microsegmentsService: MicrosegmentService,
    private globalMessageService: GlobalMessageService,
    private activatedRoute: ActivatedRoute,
    public dialog: MatDialog,
    private trustedHostsService: TrustedHostsService,
    private titleService: Title,
    private clientService: ClientService,
    private elementRef: ElementRef,
    private clipboard: Clipboard,
    private rolesPermissionService: RolesAndPermissionService
  ) {}

  // @ViewChild('workload1', { static: false }) container;

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.titleService.setTitle(
      'S2S Microsegmentations | Microsegments'
    );
    this.getCRUDdata(65);
    this.activatedRoute.params.subscribe(data => {
      // if (data.id) {
      //   this.isEditing = true;
      //   this.editingId = data.id;
      //   // this.getWorkDiscovery(data.id);
      //   this.workloadService
      //     .getWorkloadById(data.id)
      //     .subscribe((response: any) => {
      //       this.workloadName = response.name;
      //       this.selectedDeviceIds =
      //         response.DeviceInventories.map(device => device.id || '') || [];
      //     });
      // }
      this.getDevices();
      this.subscribeDeviceSearch();
      this.loadRuleData();
      this.workloadService
        .getAllWorkLoadDevices()
        .subscribe((response: any) => {
          this.ruleData = response.devicesData;
          this.sourceDeviceData = response.devicesData.map(item => {
            const { device_name, ...rest } = item;
            return device_name;
          });
          this.sourceDevice = [...new Set(this.sourceDeviceData)];
        });

        //see here

      this.workloadService
        .getAllS2sDiscovery()
        .subscribe(response => {
          this.workloadData = this.graphValues.nodes.map(item => {
            const { id, name, device, ...rest } = item;
            return {
              device_name: name,
              ip_address: id,
              device: device
            };
          });
          this.sourceDevice = [...new Set(this.sourceDeviceData)];
        });
    });
    this.loadPaginatedS2sDiscovery();
    this.subscribeWorkloadSearch();
  }

  subscribeWorkloadSearch() {
    this.$workloadSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          return this.workloadService.getAllPaginatedS2sDiscovery(
            this.currentPage,
            this.size,
            this.searchKeyword
          );
        })
      )
      .subscribe(
        (response: any) => {
          this.pagination.totalItems = response?.pagination?.totalItems;
          this.workloadEdges = response?.edges;
          console.log('this.workloadEdges :>> ', this.workloadEdges);
        },
        null,
        () => {
          this.loading = false;
        }
      );
  }

  ngOnDestroy() {
    // this.mode = 'idle';
    // this.workloadService
    //   .saveAllWorkDiscovery(
    //     this.mode,
    //     this.durationSelected,
    //     this.thresholdSelected
    //   )
    //   .subscribe(_response => {});

    if (this.sigma) {
      this.sigma.kill();
    }
  }

  // private subscribeToData(): void {
  //   this.timerSubscription = Observable.timer(60000)
  //     .first()
  //     .subscribe(() => this.getWorkDiscovery(this.editingId));
  // }

  zoomIn() {
    // this.myChart && this.myChart.zoomIn();
    if (this.sigma) {
      // Get the current camera ratio
      const currentRatio = this.sigma.getCamera().ratio;

      // Calculate the new ratio for zooming in (you can adjust the factor as needed)
      const newRatio = currentRatio / 1.2;

      // Zoom to the new ratio
      this.sigma.getCamera().animate({ ratio: newRatio }, { duration: 200 });
    }
  }

  zoomOut() {
    // this.myChart && this.myChart.zoomOut();
    if (this.sigma) {
      // Get the current camera ratio
      const currentRatio = this.sigma.getCamera().ratio;

      // Calculate the new ratio for zooming out (you can adjust the factor as needed)
      const newRatio = currentRatio * 1.2;

      // Zoom to the new ratio
      this.sigma.getCamera().animate({ ratio: newRatio }, { duration: 200 });
    }
  }

  getCRUDdata(privId) {
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false;
      this.readPermission = res?.read_permission === 1 ? true : false;
      this.updatePermission = res?.update_permission === 1 ? true : false;
      this.deletePermission = res?.delete_permission === 1 ? true : false;
    });
  }

  changeView(view) {
    localStorage.setItem('serviceGroupView', view);
    this.listView = view == 'list' ? true : false;

    if (this.listView == false) {
      if (this.sigma) {
        this.graph.clear();
        this.sigma.kill();
        this.sigma = null;
        this.getWorkDiscovery();
        // this.workloadService
        //   .getWorkloadById(this.editingId)
        //   .subscribe((response: any) => {
        //     this.workloadName = response.name;
        //     this.selectedDeviceIds =
        //       response.DeviceInventories.map(device => device.id || '') || [];
        //   });
      } else {
        this.activatedRoute.params.subscribe(data => {
          this.isEditing = true;
          this.editingId = data.id;
          this.graph.clear();
          this.getWorkDiscovery();
          // this.workloadService
          //   .getWorkloadById(this.editingId)
          //   .subscribe((response: any) => {
          //     this.workloadName = response.name;
          //     this.selectedDeviceIds =
          //       response.DeviceInventories.map(device => device.id || '') ||
          //       [];
          //   });
        });
      }
    }
  }

  loadPaginatedS2sDiscovery() {
    this.workloadService
      .getAllPaginatedS2sDiscovery(
        this.currentPage,
        this.size,
        this.searchKeyword
      )
      .subscribe((response: any) => {
        this.workloadEdges = response?.edges;
        console.log('this.workloadEdges :>> ', this.workloadEdges);
        this.pagination.totalItems = response?.pagination?.totalItems;
        this.totalPages = response?.pagination?.totalPages || 1;
      });
  }
  pageChange(page: number) {
    if (page >= 1 && page <= this.totalPages) {
      this.currentPage = page;
      this.pagination.currentPage = this.currentPage;
      this.loadPaginatedS2sDiscovery();
    }
  }
  pageSizeChange(size) {
    this.pagination.itemsPerPage = this.selectedPageSize;
    this.pagination.currentPage = 1;
    this.currentPage = 1;
    this.size = this.selectedPageSize;
    this.loadPaginatedS2sDiscovery();
  }

  saveMicroSegTag(edge) {
    const tagName = edge.microsegTag;
    const workloadTag = edge.workloadTag;
    const srcWorkloadTag = edge.srcWorkloadTag;
    this.workloadService
      .saveAllMicroSegmentTag(edge, tagName, workloadTag, srcWorkloadTag)
      .subscribe((res: any) => {
        if (res) {
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'success',
            message: 'Workload tag(s) updated successfully',
            sticky: false
          });

          const updatedEdgeIndex = this.mappedObject.edges.findIndex(
            e => e.id === edge.id
          );
          if (updatedEdgeIndex >= 0) {
            this.mappedObject.edges[updatedEdgeIndex].microsegTag = tagName;

            // Trigger the re-render function
            this.state.searchQuery = '';
            this.setSearchQuery(this.state.searchQuery);

            if (this.sigma) {
              this.graph.clear();
              this.sigma.kill();
              this.sigma = null;
              this.getWorkDiscovery();
            }
          }
          this.loadPaginatedS2sDiscovery();
        }
      });
  }

  getUpdatedData() {
    this.loadRuleData();
    this.workloadService
      .getAllWorkLoadDevices()
      .subscribe((response: any) => {
        this.ruleData = response.devicesData;
        this.sourceDevice = response.devicesData.map(item => {
          const { device_name, ...rest } = item;
          return device_name;
        });
        this.sourceDevice = [...new Set(this.sourceDevice)];
      });
    this.workloadService.getAllS2sDiscovery().subscribe(response => {
      this.workloadData = this.graphValues.nodes.map(item => {
        const { id, name, device, ...rest } = item;
        return {
          device_name: name,
          ip_address: id,
          device: device
        };
      });
      this.sourceDevice = [...new Set(this.sourceDevice)];
    });
  }
  getWorkDiscovery() {
    this.searchInput = document.getElementById(
      'search-input'
    ) as HTMLInputElement;
    this.searchSuggestions = document.getElementById(
      'suggestions'
    ) as HTMLDataListElement;

    console.log('IN GET WORK DISCOVERY');
    this.workloadService
      .getAllS2sDiscovery()
      .subscribe((response: any) => {
        console.log('graph :>> ', response);
        // SIGMA
        const sourceNodePositions: Record<string, Coordinates> = {};

        this.mappedObject = {
          nodes: response.nodes.map((node: any) => {
            return {
              key: node.id,
              fill: { src: node.src },
              device: node.device,
              attributes: {
                label: node.name,
                x: Math.random() * 1700,
                y: Math.random() * 1000,
                size: 8,
                nodeMicroSegTag: []
              }
            };
          }),
          edges: response.edges.map((edge: any) => {
            return {
              key: edge.checkKey,
              source: edge.from,
              target: edge.to,
              name: edge.name,
              id: edge.id,
              destinationName: edge.destinationName,
              protocol: edge.protocol,
              destinationPort: edge.destinationPort,
              client: edge.client,
              normal: {
                stroke: {
                  color: edge.normal.stroke.color,
                  thickness: edge.normal.stroke.thickness,
                  dash: edge.normal.stroke.dash,
                  lineJoin: edge.normal.stroke.lineJoin
                }
              },
              hovered: {
                stroke: {
                  color: edge.hovered.stroke.color,
                  thickness: edge.hovered.stroke.thickness,
                  dash: edge.hovered.stroke.dash,
                  lineJoin: edge.hovered.stroke.lineJoin
                }
              },
              selected: {
                stroke: {
                  color: edge.selected.stroke.color,
                  thickness: edge.selected.stroke.thickness,
                  dash: edge.selected.stroke.dash,
                  lineJoin: edge.selected.stroke.lineJoin
                }
              },
              isDisabled: edge.isDisabled,
              type: edge.type,
              microsegTag: edge.microsegTag,
              attributes: {
                color: '#32a852',
                allowed_comm: edge.allowed_comm,
                size: '2',
                isPartial: edge.normal.stroke.dash
              }
            };
          })
        };

        this.mappedObject.nodes.forEach((node: any) => {
          // Check if node key is not in nodePositions to avoid overriding existing positions
          if (!sourceNodePositions[node.key]) {
            sourceNodePositions[node.key] = {
              x: node.attributes.x,
              y: node.attributes.y
            };
          }
        });

        const nodeMicroSegTagMap: Record<string, string | any> = {};

        this.mappedObject.edges.forEach((edge: any) => {
          const { source, target, microsegTag } = edge;

          // Check if microsegTag is defined before adding to the map
          if (microsegTag !== undefined && microsegTag !== null) {
            // For source
            if (!nodeMicroSegTagMap[source]) {
              nodeMicroSegTagMap[source] = [];
            }
            // Add unique tags to the array
            if (!nodeMicroSegTagMap[source]?.includes(microsegTag)) {
              nodeMicroSegTagMap[source]?.push(microsegTag);
            }

            // For target
            if (!nodeMicroSegTagMap[target]) {
              nodeMicroSegTagMap[target] = [];
            }
            // Add unique tags to the array
            if (!nodeMicroSegTagMap[target]?.includes(microsegTag)) {
              nodeMicroSegTagMap[target]?.push(microsegTag);
            }
          }
        });

        // Add nodeMicroSegTag property to each node in mappedObject.nodes
        this.mappedObject.nodes.forEach((node: any) => {
          const nodeID = node.key;
          node.attributes.nodeMicroSegTag = nodeMicroSegTagMap[nodeID] || [];
        });

        this.mappedObject.edges.forEach((edge: any) => {
          // Extract source and target from the edges
          const source = edge.source;
          const target = edge.target;

          // Check if source and target are valid
          if (source && target) {
            // Adjust the target position near the source
            const sourcePosition = sourceNodePositions[source];
            const targetNode = this.mappedObject.nodes.find(
              (node: any) => node.key === target
            );

            if (sourcePosition && targetNode) {
              targetNode.x = sourcePosition.x + (Math.random() - 0.5) * 200;
              targetNode.y = sourcePosition.y + (Math.random() - 0.5) * 200;
            }
          }
        });

        this.graph.import(this.mappedObject);
        if (this.container)
          this.sigma = new Sigma(this.graph, this.container?.nativeElement, {
            allowInvalidContainer: true
          });

        if (this.sigma) {
          // this.sigma.settings({ datalist: this.graph.nodes().map(node => this.graph.getNodeAttribute(node, 'label')) });
          if (this.isDarkMode) {
            this.sigma.setSetting('defaultNodeColor', '#aeaeae');
            this.sigma.setSetting('labelColor', { color: '#aeaeae' });
          } else {
            this.sigma.setSetting('defaultNodeColor', '#000');
            this.sigma.setSetting('labelColor', { color: '#000' });
          }
          // this.sigma.setSetting('defaultEdgeColor', '#dad');
          this.sigma.on('enterNode', ({ node }) => this.setHoveredNode(node));
          this.sigma.on('leaveNode', () => this.setHoveredNode(undefined));
          this.sigma.setSetting('nodeReducer', this.nodeReducer.bind(this));
          this.sigma.setSetting('edgeReducer', this.edgeReducer.bind(this));
          // this.sigma.setSetting('edgeRenderer', this.renderEdgeLabel.bind(this));
          this.sigma.setSetting('renderEdgeLabels', true);
          this.sigma.on('clickNode', edge => this.handleEdgeClick(edge));
        }

        console.log('this.sigma :>> ', this.sigma);
      });
  }

  handleEdgeClick(edge: any) {
    this.graphBtn = true;

    this.allConnectedNodes = this.mappedObject.edges.filter(
      (graphEdge: any) => {
        return graphEdge.source === edge.node || graphEdge.target === edge.node;
      }
    );

    if (this.allConnectedNodes) {
      this.toggleDisplayStyle = 'block';
    } else {
      this.toggleDisplayStyle = 'none';
    }
  }

  updateStatus(item) {
    console.log('item :>> ', item);
    const microseg = item;
    item = {
      checkKey: item.key,
      from: item.source,
      to: item.target,
      name: item.name,
      id: item.id,
      destinationName: item.destinationName,
      protocol: item.protocol,
      destinationPort: item.destinationPort,
      client: item.client,
      normal: {
        stroke: {
          color: item.normal.stroke.color,
          thickness: item.normal.stroke.thickness,
          dash: item.normal.stroke.dash,
          lineJoin: item.normal.stroke.lineJoin
        }
      },
      hovered: {
        stroke: {
          color: item.hovered.stroke.color,
          thickness: item.hovered.stroke.thickness,
          dash: item.hovered.stroke.dash,
          lineJoin: item.hovered.stroke.lineJoin
        }
      },
      selected: {
        stroke: {
          color: item.selected.stroke.color,
          thickness: item.selected.stroke.thickness,
          dash: item.selected.stroke.dash,
          lineJoin: item.selected.stroke.lineJoin
        }
      },
      isDisabled: item.isDisabled,
      type: item.type,
      allowed_comm: item.attributes.allowed_comm
    };

    let protocol = item.protocol.toLowerCase();

    console.log('item z:>> ', item);
    const host = {
      address: item.from,
      port: item.destinationPort,
      protocol: protocol,
      label: item.from,
      direction: 1,
      isPortInRange: false
    };
    item.type !== 'manual'
      ? this.clientService
          .getClientId(item.client)
          .subscribe((response: any) => {
            this.clientId = response.client;
            console.log('this.clientId :>> ', this.clientId);
            this.workloadService
              .toggleS2sDiscovery(item.id, item.type)
              .subscribe(response => {
                console.log('response :>> ', response);
                if (this.sigma) {
                  this.graph.clear();
                  this.sigma.kill();
                  this.sigma = null;
                  this.getWorkDiscovery();
                }
                this.getWorkDiscovery();
                if (
                  item.isDisabled &&
                  item.type !== 'manual' &&
                  response.status
                ) {
                  console.log('IFFF');
                  this.trustedHostsService
                    .addHost(host, this.clientId, 'Client')
                    .subscribe(response => {
                      response;
                    });
                } else if (
                  item.isDisabled &&
                  item.type !== 'manual' &&
                  !response.status
                ) {
                  console.log('ELSE');

                  this.trustedHostsService
                    .deleteTrustedHost(
                      item.from,
                      item.destinationPort,
                      item.protocol,
                      this.clientId
                    )
                    .subscribe(response => {
                      response;
                    });
                }
              });
          })
      : this.workloadService
          .toggleS2sDiscovery(item.id, item.type)
          .subscribe(response => {
            if (this.sigma) {
              this.graph.clear();
              this.sigma.kill();
              this.sigma = null;
              this.getWorkDiscovery();
            } else {
              this.getWorkDiscovery();
            }
          });
  }

  updateStatusTabular(item) {
    console.log('item :>> ', item);

    let protocol = item.protocol.toLowerCase();

    const host = {
      address: item.from,
      port: item.destinationPort,
      protocol: protocol,
      label: item.name,
      direction: 1,
      isPortInRange: false
    };
    item.type !== 'manual'
      ? this.clientService
          .getClientId(item.client)
          .subscribe((response: any) => {
            this.clientId = response.client;
            console.log('this.clientId :>> ', this.clientId);
            this.workloadService
              .toggleS2sDiscovery(item.id, item.type)
              .subscribe(response => {
                item.allowed_comm = response.status
                this.globalMessageService.pushAlert({
                  closed: true,
                  type: 'success',
                  message: 'Rule updated successfully'
                });
                console.log('response :>> ', response);
                this.getWorkDiscovery();
                if (
                  item.isDisabled &&
                  item.type !== 'manual' &&
                  response.status
                ) {
                  console.log('IFFF');
                  this.trustedHostsService
                    .addHost(host, this.clientId, 'Client')
                    .subscribe(response => {
                      response;
                    });
                } else if (
                  item.isDisabled &&
                  item.type !== 'manual' &&
                  !response.status
                ) {
                  console.log('ELSE');

                  this.trustedHostsService
                    .deleteTrustedHost(
                      item.from,
                      item.destinationPort,
                      item.protocol,
                      this.clientId
                    )
                    .subscribe(response => {
                      response;
                    });
                }
              });
          })
      : this.workloadService
          .toggleS2sDiscovery(item.id, item.type)
          .subscribe(response => {
            item.allowed_comm = response.status
            this.globalMessageService.pushAlert({
              closed: true,
              type: 'success',
              message: 'Rule updated successfully'
            });
            this.getWorkDiscovery();
          });
  }
  openPopup() {
    this.displayStyle = 'block';
  }

  refreshData() {
    this.getWorkDiscovery();
  }

  closePopup() {
    this.displayStyle = 'none';
    // this.check = false;
  }

  closeTogglePopup() {
    this.toggleDisplayStyle = 'none';
    // this.check = false;
  }

  showModal() {
    this.toggleRuleViewStyle = 'block';
    this.toggleRuleStyle = 'none';
  }

  showViewModal() {
    this.toggleRuleStyle = 'block';
    this.toggleRuleViewStyle = 'none';
  }

  checkFields() {
    return (
      this.selSourceDevice &&
      this.selSourceIp &&
      this.selDestDevice &&
      this.selDestinationIp &&
      this.selProtocol &&
      this.selPort
    );
  }

  handleSave() {
    this.checkFields();
    const verifyPort = this.checkPort();
    if (!verifyPort) return;
    const deviceId =
      this.ruleData.filter(
        item => item.device_name === this.selSourceDevice
      )[0] ||
      this.workloadData.filter(
        item => item.device_name === this.selSourceDevice
      )[0];
    this.workloadService
      .allS2sDiscoveryManualData({
        selSourceDevice: this.selSourceDevice,
        selDestDevice: this.selDestDevice,
        selSourceIp: this.selSourceIp,
        selDestinationIp: this.selDestinationIp,
        selPort: this.selPort,
        selProtocol: this.selProtocol,
        deviceId: deviceId.id || deviceId.device
      })
      .subscribe(
        response => {
          this.toggleRuleStyle = 'none';
          this.globalMessageService.pushAlert({
            closed: true,
            type: 'success',
            message: 'Rule added successfully'
          });

          if (!this.listView) {
            if (this.sigma) {
              this.graph.clear();
              this.sigma.kill();
              this.sigma = null;
              this.getWorkDiscovery();
              this.emptyFields();
            } else {
              this.getWorkDiscovery();
              this.emptyFields();
            }
          } else {
            this.emptyFields();
            this.loadRuleData();
            // this.refreshData();
            this.loadPaginatedS2sDiscovery();
          }
        },
        error => {
          this.globalMessageService.pushAlert({
            closed: true,
            type: 'danger',
            message: 'Failed to add rule'
          });
          this.emptyFields();
        }
      );
  }

  handleCancel() {
    this.toggleRuleStyle = 'none';
    this.emptyFields();
  }

  handleViewCancel() {
    this.toggleRuleViewStyle = 'none';
  }

  emptyFields() {
    this.selSourceDevice = null;
    this.selSourceIp = null;
    this.selDestDevice = null;
    this.selDestinationIp = null;
    this.selPort = null;
    this.selProtocol = null;
  }

  loadRuleData() {
    this.workloadService
      .getAllManualRuleData()
      .subscribe((response: any) => {
        this.existingRuleData = response.manualRuleData;
        this.exitingRuleLength = response.manualRuleData.length;
      });
  }

  showBtnModal() {
    this.showAddRules = true;
  }

  disableBtn() {
    this.showAddRules = false;
  }

  checkPort() {
    if (
      !Number.isInteger(this.selPort) ||
      this.selPort < 1 ||
      this.selPort > 65535
    ) {
      this.portCheck = false;
      this.selPort = null;
      return false;
    }
    this.portCheck = true;
    return true;
  }

  handleAdd() {
    this.openFields = !this.openFields;
  }

  handleRemoveManualRule(item) {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      disableClose: true, 
      data: {
        title: 'Confirmation',
        message: `Are you sure you want to remove rule?`
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        if(item){
          this.workloadService
          .deleteFromAllManualRules(item)
          .subscribe(response => {
            this.globalMessageService.pushAlert({
              closed: true,
              type: 'success',
              message: 'Rule removed successfully'
            });

            if (!this.listView) {
              if (this.sigma) {
                this.graph.clear();
                this.sigma.kill();
                this.sigma = null;
                this.getWorkDiscovery();
                this.loadPaginatedS2sDiscovery();
                this.loadRuleData();
              } else {
                this.getWorkDiscovery();
                this.loadPaginatedS2sDiscovery();
                this.loadRuleData();
              }
            } else {
              this.emptyFields();
              this.loadRuleData();
              this.loadPaginatedS2sDiscovery();
            }
          });
          this.handleCancel();
        }
      }
    })
  }

  handleRemoveRule(item) {
    if (item?.type === 'manual') {
      (item.source_device = item?.name),
        (item.source_ip = item?.from),
        (item.destination_device = item?.destinationName),
        (item.destination_ip = item?.to),
        (item.destination_port = item?.destinationPort),
        (item.protocol = item?.protocol),
        (item.workloadId = this.editingId);
        (item.workloadTag = item?.workloadTag);
    }
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      disableClose: true, 
      data: {
        title: 'Confirmation',
        message: `Are you sure you want to remove rule?`
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        if(item?.type === 'manual'){
          this.workloadService
          .deleteFromAllManualRules(item)
          .subscribe(response => {
            this.globalMessageService.pushAlert({
              closed: true,
              type: 'success',
              message: 'Rule removed successfully'
            });
            // this.loadPaginatedS2sDiscovery();
            // this.loadRuleData();
            // this.refreshData();

            if (!this.listView) {
              if (this.sigma) {
                this.graph.clear();
                this.sigma.kill();
                this.sigma = null;
                this.getWorkDiscovery();
                this.loadPaginatedS2sDiscovery();
                this.loadRuleData();
              } else {
                this.getWorkDiscovery();
                this.loadPaginatedS2sDiscovery();
                this.loadRuleData();
              }
            } else {
              this.emptyFields();
              this.loadRuleData();
              this.loadPaginatedS2sDiscovery();
            }
          });
          this.handleCancel();
        }else if(item?.type === 'auto'){
          this.workloadService
          .deleteFromAllAutoRules(item)
          .subscribe(response => {
            this.globalMessageService.pushAlert({
              closed: true,
              type: 'success',
              message: 'Rule removed successfully'
            });
            // this.loadPaginatedS2sDiscovery();
            // this.loadRuleData();
            // this.refreshData();

            if (!this.listView) {
              if (this.sigma) {
                this.graph.clear();
                this.sigma.kill();
                this.sigma = null;
                this.getWorkDiscovery();
                this.loadPaginatedS2sDiscovery();
                this.loadRuleData();
              } else {
                this.getWorkDiscovery();
                this.loadPaginatedS2sDiscovery();
                this.loadRuleData();
              }
            } else {
              this.emptyFields();
              this.loadRuleData();
              this.loadPaginatedS2sDiscovery();
            }
          });
          this.handleCancel();
        }
      }
    });
  }

  onSelectSource() {
    if (this.selDestDevice) {
      this.selDestDevice = null;
      this.selDestinationIp = null;
    }
    this.destDevice = this.sourceDevice.filter(
      item => item !== this.selSourceDevice
    );
    this.destDevice = [...new Set(this.destDevice)];
    let sourceDevice =
      this.ruleData.filter(
        item => item.device_name === this.selSourceDevice
      )[0] ||
      this.workloadData.filter(
        item => item.device_name === this.selSourceDevice
      )[0];
    this.selSourceIp = sourceDevice.ip_address;
  }

  onSelectDestination() {
    const destinationDevice =
      this.ruleData.filter(
        item => item.device_name === this.selDestDevice
      )[0] ||
      this.workloadData.filter(
        item => item.device_name === this.selDestDevice
      )[0];
    this.selDestinationIp = destinationDevice.ip_address;
  }

  saveWorkDiscovery() {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      disableClose: true, 
      data: {
        title: 'Confirmation',
        message: `Delete existing discovery data? This may delete workloads.`,
        isMicrosegment: true
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result === 1) {
        this.workloadService
        .flushDicovery()
        .subscribe(response => {
          this.globalMessageService.pushAlert({
            closed: true,
            type:'success',
            message: 'Previous discovery data deleted successfully'
          });
          this.getWorkDiscovery();
          this.loadPaginatedS2sDiscovery();
        });

        this.mode = 'discovery';
        this.workloadService
          .saveAllWorkDiscovery(
            this.mode,
            this.durationSelected,
            this.thresholdSelected
          )
          .subscribe(_response => {
            this.check = true;
            this.displayStyle = 'none';
            let total_discovery_time_ms = Number(this.durationSelected) * 60 * 1000;
            let one_percent_duration_ms = total_discovery_time_ms / 100;
            let start = moment();
            this.myTimer = setInterval(() => {
              let timePassed = moment().diff(start);
              let percentage = (timePassed / total_discovery_time_ms) * 100;
              percentage = Number(Math.round(percentage));
              this.percentage = Math.min(percentage, 100);
              if (percentage == 100) {
                this.mode = 'idle';
                this.workloadService
                  .saveAllWorkDiscovery(
                    this.mode,
                    this.durationSelected,
                    this.thresholdSelected
                  )
                  .subscribe(_response => {
                    if (!this.listView) {
                      if (this.sigma) {
                        this.graph.clear();
                        this.sigma.kill();
                        this.sigma = null;
                      }
                      this.getWorkDiscovery();
                      clearInterval(this.myTimer);
                    } else {
                      this.getWorkDiscovery();
                      clearInterval(this.myTimer);
                    }
                  });
                this.stopWorkDiscovery();
              }
            }, one_percent_duration_ms);
          });
      }else if(result !== 1) {
        this.mode = 'discovery';
        this.workloadService
          .saveAllWorkDiscovery(
            this.mode,
            this.durationSelected,
            this.thresholdSelected
          )
          .subscribe(_response => {
            this.check = true;
            this.displayStyle = 'none';
            let total_discovery_time_ms = Number(this.durationSelected) * 60 * 1000;
            let one_percent_duration_ms = total_discovery_time_ms / 100;
            let start = moment();
            this.myTimer = setInterval(() => {
              let timePassed = moment().diff(start);
              let percentage = (timePassed / total_discovery_time_ms) * 100;
              percentage = Number(Math.round(percentage));
              this.percentage = Math.min(percentage, 100);
              if (percentage == 100) {
                this.mode = 'idle';
                this.workloadService
                  .saveAllWorkDiscovery(
                    this.mode,
                    this.durationSelected,
                    this.thresholdSelected
                  )
                  .subscribe(_response => {
                    if (!this.listView) {
                      if (this.sigma) {
                        this.graph.clear();
                        this.sigma.kill();
                        this.sigma = null;
                      }
                      this.getWorkDiscovery();
                      clearInterval(this.myTimer);
                    } else {
                      this.getWorkDiscovery();
                      clearInterval(this.myTimer);
                    }
                  });
                this.stopWorkDiscovery();
              }
            }, one_percent_duration_ms);
          });
      }
    })
  }

  stopWorkDiscovery() {
    this.check = false;
    this.mode = 'idle';
    this.workloadService
      .saveAllWorkDiscovery(
        this.mode,
        this.durationSelected,
        this.thresholdSelected
      )
      .subscribe(_response => {
        clearInterval(this.myTimer);
      });
    if (!this.listView) {
      if (this.sigma) {
        this.graph.clear();
        this.sigma.kill();
        this.sigma = null;
      }
      this.getWorkDiscovery();
      clearInterval(this.myTimer);
    } else {
      this.getWorkDiscovery();
      clearInterval(this.myTimer);
    }
    this.percentage = 0;
  }

  startWorkDiscovery() {
    this.microsegmentsService
      .discoverAllMicrosegments()
      .subscribe(response => {
        this.globalMessageService.pushAlert({
          type: 'success',
          message: 'Workload has been published successfully.'
        });
      });
  }

  cancelWorkDiscovery() {
    this.router.navigate([`/workloads`]);
    // alert('cancel')
  }

  getDevices() {
    this.workloadService
      .getDevicesForWorkload(
        this.deviceOffset,
        this.deviceName,
        this.editingId || null
      )
      .subscribe(response => {
        this.devicesList = [...this.devicesList, ...(response.data || [])];
      });
  }

  subscribeDeviceSearch() {
    this.$deviceSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.isDeviceSearching = true;
          this.deviceOffset = 0;
          this.deviceName = term;
          return this.workloadService.getDevicesForWorkload(
            this.deviceOffset,
            this.deviceName,
            this.editingId || null
          );
        })
      )
      .subscribe((response: any) => {
        this.isDeviceSearching = false;
        this.devicesList = response.data;
      });
  }

  fetchMoreDevices() {
    this.deviceOffset += 1;
    this.getDevices();
  }

  submitHandler() {
    this.loading = true;
    if (this.isEditing) {
      this.workloadService
        .editWorkload(this.editingId, this.workloadName, this.selectedDeviceIds)
        .subscribe(
          _response => {
            this.globalMessageService.pushAlert({
              closed: true,
              type: 'success',
              message: 'Workload saved successfully'
            });
            this.router.navigate(['/workloads']);
          },
          () => {
            this.loading = false;
          },
          () => {
            this.loading = false;
          }
        );
    } else {
      this.workloadService
        .createWorkload(this.workloadName, this.selectedDeviceIds)
        .subscribe(
          _response => {
            this.globalMessageService.pushAlert({
              closed: true,
              type: 'success',
              message: 'Workload created successfully'
            });
            this.router.navigate(['/workloads']);
          },
          () => {
            this.loading = false;
          },
          () => {
            this.loading = false;
          }
        );
    }
  }

  deviceFocus() {
    this.deviceOffset = 0;
    this.deviceName = '';
    this.$deviceSearch.next('');
  }

  onTableMouseDown(event: MouseEvent) {
    if (event.target instanceof HTMLTableCellElement) {
      this.isDragEnabled = false;
      event.stopPropagation();

      // Add a listener to handle blur events
      document.addEventListener('blur', this.onModalBlur);
    }
  }

  copyToClipboard(value: string) {
    let copiedContent = this.clipboard.copy(value);
    if (copiedContent) {
      this.globalMessageService.pushAlert({
        closed: false,
        type: 'success',
        message: 'Successfully copied.'
      });
    } else {
      this.globalMessageService.pushAlert({
        closed: false,
        type: 'danger',
        message: 'Copy operation failed'
      });
    }
  }

  onTableMouseUp() {
    this.isDragEnabled = true;
  }

  onModalBlur = () => {
    // This function will be called when the modal loses focus
    this.isDragEnabled = true;
    // Remove the blur event listener
    document.removeEventListener('blur', this.onModalBlur);
  };

  setSearchQuery(event: any): void {
    let query = typeof event === 'string' ? event : event?.target?.value || '';

    this.state.searchQuery = query;
    if (query) {
      const lcQuery = query;
      const suggestions = this.graph
        .nodes()
        .map((n: any) => ({
          id: n,
          label: (this.graph.getNodeAttribute(n, 'label') as string) || '',
          nodeMicroSegTag:
            this.graph.getNodeAttribute(n, 'nodeMicroSegTag') || ''
        }))
        .filter((data: any) => {
          return (
            data.label.includes(lcQuery) ||
            (data.nodeMicroSegTag && data.nodeMicroSegTag.includes(lcQuery))
          );
        })
        .map((id: any) => id);
      // If we have a single perfect match, then we consider the user has selected a node
      if (suggestions.length === 1 && suggestions[0].label === lcQuery) {
        this.state.selectedNode = suggestions[0].id;
        this.state.suggestions = undefined;

        // Move the camera to center it on the selected node (you may need to adjust this logic)
        const nodePosition = this.state.selectedNode
          ? (this.sigma?.getNodeDisplayData(
              this.state.selectedNode
            ) as Coordinates)
          : { x: 0, y: 0 }; // Provide default coordinates or handle the case where selectedNode is undefined

        this.sigma?.getCamera().animate(
          { ...nodePosition, ratio: 0.05 },
          {
            duration: 600
          }
        );
      } else {
        // Display the suggestions list
        this.state.selectedNode = undefined;
        this.state.suggestions = new Set<string>(
          suggestions.map((item: any) => item.id)
        );
      }
    } else {
      // If the query is empty, reset the selectedNode / suggestions state
      this.state.selectedNode = undefined;
      this.state.suggestions = undefined;
      this.state.searchQuery = '';
    }

    this.sigma?.refresh();
  }

  setHoveredNode(node?: string): void {
    // Add logic for updating state based on the hovered node...
    // ...
    if (node) {
      this.state.hoveredNode = node;
      this.state.hoveredNeighbors = new Set(this.graph.neighbors(node));
    } else {
      this.state.hoveredNode = undefined;
      this.state.hoveredNeighbors = undefined;
    }
    // Refresh rendering:
    this.sigma?.refresh();
  }

  nodeReducer(node: any, data: any): Partial<ExtendedNodeDisplayData> {
    const displayData = data as NodeDisplayData;

    const res: Partial<ExtendedNodeDisplayData> = { ...displayData };

    if (
      this.state.hoveredNeighbors &&
      !this.state.hoveredNeighbors.has(node.toString()) &&
      this.state.hoveredNode !== node
    ) {
      res.label = '';
      res.hidden = true;
    }

    if (this.state.selectedNode === node) {
      res.highlighted = true;
    } else if (
      this.state.suggestions &&
      !this.state.suggestions.has(node.toString())
    ) {
      res.label = '';
      res.hidden = true;
    }
    return res;
  }

  edgeReducer(edge: any, data: any): Partial<EdgeDisplayData> {
    const displayData = data as EdgeDisplayData;

    const res: Partial<EdgeDisplayData> = { ...displayData };
    let isAllowed = data.allowed_comm;

    if (data.isPartial === '10 5' && isAllowed == true) {
      res.color = '#FFA500';
    } else if (data.isPartial === '10 5' && isAllowed == false) {
      res.color = '#FFA500';
    } else if (data.isPartial === '0' && isAllowed == true) {
      res.color = '#32a852';
    } else if (data.isPartial === '0' && isAllowed == false) {
      res.color = '#ff0000';
    }

    if (
      this.state.hoveredNode &&
      !this.graph.hasExtremity(edge, this.state.hoveredNode)
    ) {
      res.hidden = true;
    }

    if (
      this.state.suggestions &&
      (!this.state.suggestions.has(this.graph.source(edge)) ||
        !this.state.suggestions.has(this.graph.target(edge)))
    ) {
      res.hidden = true;
    }

    res.label = this.graph.getEdgeAttribute(edge, 'label') || '';
    // res.drawLabels = true;
    return res;
  }
}

